| package |
package := Package name: 'GlorpCore'.
package paxVersion: 1;
	basicComment: 'This package contains some of the fundamental Glorp objects, defining session and its caching. Oddly, for defining such a basic object, it comes very late in the loading sequence, precisely because those objects use lots of other Glorp framework code.

This is part of the GLORP system, an open-source library (LGPL(S)) for persisting Smalltalk objects in relational databases. For more information see the #notice property, the accompanying files and/or the web sites http://www.glorp.org or http://www.sourceforge.net/projects/glorp 
'.

package basicPackageVersion: 'image'.


package classNames
	add: #Cache;
	add: #CacheManager;
	add: #CachePolicy;
	add: #CompoundSelectCommand;
	add: #DuplicatePrimaryKeyException;
	add: #FixedSizeQueue;
	add: #GlorpCorePackage;
	add: #GlorpSession;
	add: #PersistentObject;
	add: #TimedExpiryCachePolicy;
	add: #TimedProxyReaper;
	add: #TimedStrongCachePolicy;
	add: #WeakVWCachePolicy;
	yourself.

package methodNames
	add: #CompoundQuery -> #sqlWith:;
	add: #DatabaseAccessor -> #createConstraint:;
	add: #DatabaseAccessor -> #denyReads;
	add: #DatabaseAccessor -> #denyWrites;
	add: #DatabaseAccessor -> #insertOnly;
	add: #DatabaseAccessor -> #permitNothing;
	add: #DatabaseAccessor -> #permitsInserts;
	add: #DatabaseAccessor -> #permitsUpdates;
	add: #DatabaseField -> #matchesField:;
	add: #DatabaseTable -> #hasForeignKeyConstraintMatching:;
	add: #DatabaseTable -> #hasForeignKeyConstraintTheSameAs:;
	add: #DatabaseTable -> #hasUniqueConstraintMatching:;
	add: #DB2Sequence -> #backupReserveCommand:in:;
	add: #DB2Sequence -> #tableSelectCommandFor:in:reserving:;
	add: #FirebirdSequence -> #tableSelectCommandFor:in:reserving:;
	add: #GlorpExpression -> #sqlString;
	add: #OracleSequence -> #backupReserveCommand:in:;
	add: #OracleSequence -> #tableSelectCommandFor:in:reserving:;
	add: #PGSequence -> #backupReserveCommand:in:;
	add: #PGSequence -> #tableSelectCommandFor:in:reserving:;
	add: #SimpleQuery -> #addBoundExpressionsIn:to:;
	add: #SimpleQuery -> #sqlWith:;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: 'GlorpDatabase';
	add: 'GlorpExpressions';
	add: 'GlorpMisc';
	add: 'GlorpQueries';
	add: 'GlorpUnitOfWork';
	yourself).

package!

"Class Definitions"!

Object subclass: #Cache
	instanceVariableNames: 'items policy mainCache extraReferences'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #CacheManager
	instanceVariableNames: 'subCaches session'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #CachePolicy
	instanceVariableNames: 'expiryAction numberOfElements'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #FixedSizeQueue
	instanceVariableNames: 'maximumSize items'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #GlorpCorePackage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #GlorpSession
	instanceVariableNames: 'system currentUnitOfWork cache accessor applicationData reaper errorAction modificationTracker'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #PersistentObject
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #TimedProxyReaper
	instanceVariableNames: 'weakValueDictionary keysSortedByExpiryTime indexOffset mutex reaperThread delay maximumWork nextId timeout done startTime'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
CachePolicy subclass: #TimedExpiryCachePolicy
	instanceVariableNames: 'timeout'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
CachePolicy subclass: #TimedStrongCachePolicy
	instanceVariableNames: 'timeout'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
CachePolicy subclass: #WeakVWCachePolicy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
QuerySelectCommand subclass: #CompoundSelectCommand
	instanceVariableNames: 'commands'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
GlorpError subclass: #DuplicatePrimaryKeyException
	instanceVariableNames: 'existingObject newObject'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!CompoundQuery methodsFor!

sqlWith: aDictionary

	self prepare.
	session platform queryWithUnsupportedOperationsEliminatedFrom: self do: [:newQuery | 
		^newQuery
			prepare;
			sqlWith: aDictionary].
	^CompoundSelectCommand
		forQuery: self
		parameters: aDictionary
		useBinding: session useBinding
		session: session.! !
!CompoundQuery categoriesFor: #sqlWith:!executing!public! !

!DatabaseAccessor methodsFor!

createConstraint: aConstraint
	^self addConstraint: aConstraint!

denyReads
	deniedCommands := SelectCommand glorpAllSubclasses.!

denyWrites
	deniedCommands := RowBasedCommand glorpAllSubclasses.!

insertOnly
	self denyCommands: (Array with: UpdateCommand with: DeleteCommand).!

permitNothing
	deniedCommands := DatabaseCommand glorpAllSubclasses asArray.!

permitsInserts
	deniedCommands isNil ifTrue: [^true].
	^(deniedCommands includes: InsertCommand) not.!

permitsUpdates
	deniedCommands isNil ifTrue: [^true].
	^(deniedCommands includes: UpdateCommand) not.! !
!DatabaseAccessor categoriesFor: #createConstraint:!obsolete!private! !
!DatabaseAccessor categoriesFor: #denyReads!permissions!private! !
!DatabaseAccessor categoriesFor: #denyWrites!permissions!private! !
!DatabaseAccessor categoriesFor: #insertOnly!permissions!private! !
!DatabaseAccessor categoriesFor: #permitNothing!permissions!private! !
!DatabaseAccessor categoriesFor: #permitsInserts!permissions!private! !
!DatabaseAccessor categoriesFor: #permitsUpdates!permissions!private! !

!DatabaseField methodsFor!

matchesField: aField
	table name = aField table name ifFalse: [^false].
	name = aField name ifFalse: [^false].
	isNullable = aField isNullable ifFalse: [^false].
	isPrimaryKey = aField isPrimaryKey ifFalse: [^false].
	isUnique = aField isUnique ifFalse: [^false].
	^true! !
!DatabaseField categoriesFor: #matchesField:!private!testing! !

!DatabaseTable methodsFor!

hasForeignKeyConstraintMatching: aConstraint
	^self foreignKeyConstraints
		anySatisfy: [:each | (each matchesForeignKeyConstraint: aConstraint)].!

hasForeignKeyConstraintTheSameAs: aConstraint

	^self foreignKeyConstraints anySatisfy: [:each | (each isTheSameAs: aConstraint)].!

hasUniqueConstraintMatching: aConstraint
	
	^self uniqueConstraints anySatisfy: [:each | each matchesUniqueConstraint: aConstraint]! !
!DatabaseTable categoriesFor: #hasForeignKeyConstraintMatching:!private!testing! !
!DatabaseTable categoriesFor: #hasForeignKeyConstraintTheSameAs:!private!testing! !
!DatabaseTable categoriesFor: #hasUniqueConstraintMatching:!public!testing! !

!DB2Sequence methodsFor!

backupReserveCommand: anInteger in: aSession
	"syscat.tables is a public view of all the tables.
	Don't bind the number in the fetch first nn clause."

	| command stream |
	stream := WriteStream on: (String new: 200).
	stream nextPutAll: 'select next value for '.
	stream nextPutAll: self qualifiedName.
	stream nextPutAll: ' from SYSCAT.TABLES'.
	stream nextPutAll: ' fetch first '.
	stream nextPutAll: anInteger printString.
	stream nextPutAll: ' rows only'.
	command := SQLStringSelectCommand new.
	command setSQLString: stream contents.
	command parameters: (Array with: anInteger).
	command session: aSession.
	^command.!

tableSelectCommandFor: aTable in: aSession reserving: anInteger
	"Don't bind the number in the fetch first nn clause."
	| stream |
	(tableSelectCommand notNil and: [tableSelectCommand parameters includes: anInteger]) ifFalse:
			[stream := WriteStream on: (String new: 200).
			stream nextPutAll: 'select next value for '.
			stream nextPutAll: self qualifiedName.
			stream nextPutAll: ' from '.
			stream nextPutAll: aTable name.
			stream nextPutAll: ' fetch first '.
			stream nextPutAll: anInteger printString.
			stream nextPutAll: ' rows only'.
			tableSelectCommand := SQLStringSelectCommand new.
			tableSelectCommand parameters: (Array with: anInteger).
			tableSelectCommand setSQLString: stream contents.
			tableSelectCommand blockFactor: anInteger.
			tableSelectCommand session: aSession].
	^tableSelectCommand.! !
!DB2Sequence categoriesFor: #backupReserveCommand:in:!private! !
!DB2Sequence categoriesFor: #tableSelectCommandFor:in:reserving:!public!SQL! !

!FirebirdSequence methodsFor!

tableSelectCommandFor: aTable in: aSession reserving: anInteger
	"Since tableSelectCommand was never nilled, a second pundle version reusing this sequence used to get an SQL string embedding the first's integer value.  If it was a version of a different pundle, its integer probably differed;  when a call to reserve e.g. 1233 returns a maximum got by reusing the prior version's call to reserve e.g. 85, the result is 1148 duplicate keys.  If #insertOnly was set,  replication failed on an illegal command.  If we reached one of the fix-up states where it was not, then our cache assured us we were updating and we happily overwrote e.g. the blob for a class definition with that for a method definition (as I experienced :-/).  As sequences are set in the descriptor system and reused, this persisted until the session was discarded or its system was revised.  We now check the integer."

	| stream useBinding |
	"Binding into this doesn't seem to work"
	useBinding := false.
	(tableSelectCommand notNil and: [tableSelectCommand parameters includes: anInteger]) ifFalse:
		[stream := WriteStream on: (String new: 200).
		stream nextPutAll: 'select gen_id('.
		stream nextPutAll: self qualifiedName.
		stream nextPut: $,.
		stream nextPutAll: (useBinding ifTrue: ['?'] ifFalse: [anInteger printString]).
		stream nextPutAll: ') from rdb$database'.
		tableSelectCommand := SQLStringSelectCommand new.
		tableSelectCommand parameters: (Array with: anInteger).
		tableSelectCommand setSQLString: stream contents.
		tableSelectCommand session: aSession].
	^tableSelectCommand! !
!FirebirdSequence categoriesFor: #tableSelectCommandFor:in:reserving:!public!SQL! !

!GlorpExpression methodsFor!

sqlString
	| command |
	command := QuerySelectCommand new
		useBinding: false;
		session: self ultimateBaseExpression descriptor session.
	self printSQLOn: command withParameters: #().
	^command contents.! !
!GlorpExpression categoriesFor: #sqlString!As yet unclassified!private! !

!OracleSequence methodsFor!

backupReserveCommand: anInteger in: aSession
	| command stream |
	stream := WriteStream on: (String new: 200).
	stream nextPutAll: 'select '.
	stream nextPutAll: self qualifiedName.
	stream nextPutAll: '.nextval from SYS.ALL_OBJECTS '.
	stream nextPutAll: ' WHERE rownum <= '.
	stream
		nextPutAll:
			(aSession useBinding ifTrue: ['?'] ifFalse: [anInteger printString]).
	command := SQLStringSelectCommand new.
	command setSQLString: stream contents.
	command parameters: (Array with: anInteger).
	command session: aSession.
	^command.!

tableSelectCommandFor: aTable in: aSession reserving: anInteger
	| stream |
	(tableSelectCommand notNil and: [tableSelectCommand parameters includes: anInteger]) ifFalse:
			[stream := WriteStream on: (String new: 200).
			stream nextPutAll: 'select '.
			stream nextPutAll: self qualifiedName.
			stream nextPutAll: '.nextval from '.
			stream nextPutAll: aTable name.
			stream nextPutAll: ' WHERE rownum <= '.
			stream nextPutAll: (aSession useBinding ifTrue: ['?'] ifFalse: [anInteger printString]).
			tableSelectCommand := SQLStringSelectCommand new.
			tableSelectCommand parameters: (Array with: anInteger).
			tableSelectCommand setSQLString: stream contents.
			tableSelectCommand blockFactor: anInteger.
			tableSelectCommand session: aSession].
	^tableSelectCommand.! !
!OracleSequence categoriesFor: #backupReserveCommand:in:!private! !
!OracleSequence categoriesFor: #tableSelectCommandFor:in:reserving:!private!SQL! !

!PGSequence methodsFor!

backupReserveCommand: anInteger in: aSession
	| stream command |
	stream := WriteStream on: (String new: 200).
	stream nextPutAll: 'select '.
	GlorpHelper
		print: [:i | 'nextval(''' , self qualifiedName , ''')']
		on: stream
		for: (1 to: anInteger)
		separatedBy: ','.
	command := SQLStringSelectCommand new.
	command setSQLString: stream contents.
	command parameters: (Array with: anInteger).
	command session: aSession.
	^command.!

tableSelectCommandFor: aTable in: aSession reserving: anInteger
	| stream |
	(tableSelectCommand notNil and: [tableSelectCommand parameters includes: anInteger]) ifFalse:
			[stream := WriteStream on: (String new: 200).
			stream nextPutAll: 'select nextval('.
			self qualifiedName printOn: stream.
			stream nextPutAll: ') from ' , aTable name.
			stream nextPutAll: ' limit '.
			stream nextPutAll: (aSession useBinding ifTrue: ['?'] ifFalse: [anInteger printString]).
			tableSelectCommand := SQLStringSelectCommand new.
			tableSelectCommand parameters: (Array with: anInteger).
			tableSelectCommand setSQLString: stream contents.
			tableSelectCommand session: aSession].
	^tableSelectCommand.! !
!PGSequence categoriesFor: #backupReserveCommand:in:!private! !
!PGSequence categoriesFor: #tableSelectCommandFor:in:reserving:!public!SQL! !

!SimpleQuery methodsFor!

addBoundExpressionsIn: aCommand to: aCollection
	self joins , self fields, (Array with: self whereClause)
		do:
			[:eachBigExpression | 
			eachBigExpression isGlorpExpression
				ifTrue: [eachBigExpression do:
					[:eachIndividualExpressionNode | 
					eachIndividualExpressionNode
						addBoundExpressionsIn: aCommand
						to: aCollection]]].!

sqlWith: aDictionary
	| result |
	self prepare.
	customCommand isNil ifFalse: [
		result := customCommand copy.
		result query: self; parameters: aDictionary; useBinding: session useBinding; session: session.
		^result].
	^QuerySelectCommand
		forQuery: self
		parameters: aDictionary
		useBinding: session useBinding
		session: session.! !
!SimpleQuery categoriesFor: #addBoundExpressionsIn:to:!accessing!private! !
!SimpleQuery categoriesFor: #sqlWith:!As yet unclassified!private! !

"End of package definition"!

"Source Globals"!

"Classes"!

Cache guid: (GUID fromString: '{CC721ED6-5438-4551-8352-DAF8113DD2E8}')!
Cache comment: 'This is the per-class cache of instances read from the database.

Instance Variables:
	items	<Dictionary from: Object to: Object>	The cached items, keyed by their primary key values
	policy	<CachePolicy>	The settings for this cache.


'!
!Cache categoriesForClass!Unclassified! !
!Cache methodsFor!

at: key ifAbsent: aBlock 
	| item value |
	item := self basicAt: key ifAbsent: [^aBlock value].
	value := policy contentsOf: item.
	(self hasItemExpired: item) 
		ifTrue: 
			[policy takeExpiryActionForKey: key withValue: value in: self. 
			(items includesKey: key) ifFalse: [^aBlock value]].
	^value.!

at: key ifAbsentPut: aBlock 
	| item |
	item := self at: key ifAbsent: [nil].
	^item isNil 
		ifTrue: [ | newItem|
			newItem := policy cacheEntryFor: aBlock value.
			self markEntryAsCurrent: newItem.
			items at: key put: newItem]
		ifFalse: [self markEntryAsCurrent: item. item].!

basicAt: anObject ifAbsent: aBlock

	^items at: anObject ifAbsent: aBlock.!

cachePolicy
	^policy.!

cachePolicy: aCachePolicy
	policy := aCachePolicy.
	self initializeCache.!

do: aBlock
	items do: aBlock.!

expireAll
	items do: [:each | policy expire: each].!

expiredInstanceFor: key
	"Return the expired instance. Used for refreshing so that we don't recursively try and refresh when we get the instance to be refreshed"
	| item value |
	item := self basicAt: key ifAbsent: [self error: 'No expired instance found'].
	value := policy contentsOf: item.
	(self hasItemExpired: item) ifFalse: [self error: 'No expired instance found'].
	^value.!

expireKey: key ifAbsent: aBlock
	| item |
	item := self basicAt: key ifAbsent: [^aBlock value].
	policy expire: item.!

hasExpired: key
	| item |
	item := self basicAt: key ifAbsent: [^false].
	^self hasItemExpired: item.!

hasItemExpired: anItem
	^(policy hasExpired: anItem) and: [(mainCache session isRegistered: (policy contentsOf: anItem)) not].!

includesKey: key
	"Return true if we include the object. Don't listen to any expiry policy"
	self basicAt: key ifAbsent: [^false].
	^true.!

includesKey: key as: anObject
	"Return true if we include the object, and it matches the given object. If we include a different object with the same key, raise an exception. Don't listen to any expiry policy"
	| item value |
	item := self basicAt: key ifAbsent: [^false].
	value := policy contentsOf: item.
	value == anObject ifFalse: [
		(DuplicatePrimaryKeyException new: anObject existing: value) signal].
	^true.!

includesKey: key withClass: aClass
	"Return true if we include the object, and it matches our class. Don't listen to any expiry policy"
	| item value |
	item := self basicAt: key ifAbsent: [^false].
	value := policy contentsOf: item.
	^value isKindOf: aClass.!

initialize!

initializeCache

	items := policy newItemsIn: self.
	extraReferences := policy collectionForExtraReferences.!

mainCache
	^mainCache!

mainCache: aCacheManager
	mainCache := aCacheManager!

markAsCurrentAtKey: key
	| item |
	item := self basicAt: key ifAbsent: [^false].
	^policy markEntryAsCurrent: item in: self.!

markEntryAsCurrent: anItem
	"The policy has told us to mark an item as current. This is only really useful for weak policies, which tell us to keep an additional pointer to the object in a (presumably) fixed-size collection"
	extraReferences isNil ifFalse: [extraReferences add: anItem].!

mournKeyOf: anEphemeron
	policy takeExpiryActionForKey: anEphemeron key withValue: anEphemeron value in: self.!

numberOfElements
	^items size.!

release
	policy release: self.
	extraReferences := nil.!

removeKey: key ifAbsent: aBlock

	^items removeKey: key ifAbsent: aBlock.!

session
	^mainCache session.!

willRemoveItems
	^policy willRemoveItems.! !
!Cache categoriesFor: #at:ifAbsent:!lookup!private! !
!Cache categoriesFor: #at:ifAbsentPut:!lookup!private! !
!Cache categoriesFor: #basicAt:ifAbsent:!private! !
!Cache categoriesFor: #cachePolicy!accessing!private! !
!Cache categoriesFor: #cachePolicy:!accessing!private! !
!Cache categoriesFor: #do:!private! !
!Cache categoriesFor: #expireAll!lookup!private! !
!Cache categoriesFor: #expiredInstanceFor:!private! !
!Cache categoriesFor: #expireKey:ifAbsent:!lookup!private! !
!Cache categoriesFor: #hasExpired:!lookup!private! !
!Cache categoriesFor: #hasItemExpired:!lookup!private! !
!Cache categoriesFor: #includesKey:!lookup!private! !
!Cache categoriesFor: #includesKey:as:!lookup!private! !
!Cache categoriesFor: #includesKey:withClass:!lookup!private! !
!Cache categoriesFor: #initialize!initialize!private! !
!Cache categoriesFor: #initializeCache!initialize!private! !
!Cache categoriesFor: #mainCache!accessing!private! !
!Cache categoriesFor: #mainCache:!accessing!private! !
!Cache categoriesFor: #markAsCurrentAtKey:!private! !
!Cache categoriesFor: #markEntryAsCurrent:!private! !
!Cache categoriesFor: #mournKeyOf:!finalization!private! !
!Cache categoriesFor: #numberOfElements!accessing!private! !
!Cache categoriesFor: #release!initialize!private! !
!Cache categoriesFor: #removeKey:ifAbsent:!lookup!private! !
!Cache categoriesFor: #session!accessing!private! !
!Cache categoriesFor: #willRemoveItems!accessing!private! !

!Cache class methodsFor!

new

	^super new initialize.!

newFor: aClass in: aCacheManager 
	| newCache descriptor |
	descriptor := aCacheManager session isNil ifFalse: [aCacheManager system descriptorFor: aClass].
	newCache := Cache new.
	newCache mainCache: aCacheManager.
	descriptor isNil 
		ifTrue: [newCache cachePolicy: CachePolicy default]
		ifFalse: [newCache cachePolicy: descriptor cachePolicy].
	^newCache.! !
!Cache class categoriesFor: #new!instance creation!private! !
!Cache class categoriesFor: #newFor:in:!instance creation!private! !

CacheManager guid: (GUID fromString: '{F8719A00-9246-4452-9691-62B2A4E128A3}')!
CacheManager comment: 'This is the entire cache for a session, consisting of multiple sub-caches, one per class.

Instance Variables:
	session	<Session>	The containing session.
	subCaches	<Dictionary from: Class to: Cache>	The per-class caches.


'!
!CacheManager categoriesForClass!Unclassified! !
!CacheManager methodsFor!

at: aKey insert: anObject

	| subCache |
	subCache := self cacheForClass: anObject class.
	subCache at: aKey ifAbsentPut: [anObject].!

cacheFor: anObject
	"Get the cache for a particular object. Since this could conceivably be passed a proxy, check for that. The cache for an uninstantiated proxy is kind of ambiguous, treat it as nil.  This could also be a class"

	| nonMetaClass |
	nonMetaClass := anObject isBehavior ifTrue: [anObject] ifFalse: [anObject class].
	^nonMetaClass == Proxy
		ifTrue: [anObject isInstantiated ifTrue: [self cacheFor: anObject getValue] ifFalse: [nil]]
		ifFalse: [self cacheForClass: nonMetaClass].!

cacheForClass: aClass
	^subCaches at: aClass
		ifAbsentPut: [self makeCacheFor: aClass]!

containsObject: anObject key: aKey

	| cache |
	cache := self cacheForClass: anObject class.
	^cache includesKey: aKey as: anObject.!

containsObjectForClass: aClass key: aKey

	| cache |
	cache := self cacheForClass: aClass.
	^cache includesKey: aKey.!

expireAll
	subCaches do: [:each | each expireAll].!

expireClass: aClass key: aKey

	^self expireClass: aClass key: aKey ifAbsent: [self error: 'Object not in cache'].!

expireClass: aClass key: aKey ifAbsent: failBlock 
	| cache |
	cache := self cacheForClass: aClass.
	(cache includesKey: aKey withClass: aClass) ifFalse: [^failBlock value].
	cache expireKey: aKey ifAbsent: [ failBlock value ].!

expiredInstanceOf: aClass key: aKey

	^(self cacheForClass: aClass) expiredInstanceFor: aKey.!

hasExpired: anObject
	| key cache |
	key := (session descriptorFor: anObject) primaryKeyFor: anObject.
	cache := self cacheFor: anObject.
	cache isNil ifTrue: [^false].  "We have an uninstantiated proxy."
	^cache hasExpired: key.!

hasExpired: aClass key: key
	| cache |
	cache := self cacheFor: aClass.
	^cache hasExpired: key.!

hasObjectExpiredOfClass: aClass withKey: key
	| cache |
	cache := self cacheForClass: aClass.
	^cache hasExpired: key.!

initialize

	subCaches := IdentityDictionary new: 100.!

lookupClass: aClass key: aKey

	^self lookupClass: aClass key: aKey ifAbsent: [self error: 'cache miss'].!

lookupClass: aClass key: aKey ifAbsent: failBlock

	| object |
	object := (self cacheForClass: aClass) at: aKey ifAbsent: failBlock.
	^(object isKindOf: aClass)
		ifTrue: [ object ]
		ifFalse: [ failBlock value ].!

lookupClass: aClass key: aKey ifAbsentPut: failBlock

	^(self cacheForClass: aClass) at: aKey ifAbsentPut: failBlock.!

makeCacheFor: aClass 
	| rootClass cache |
	rootClass := session isNil ifTrue: [aClass] ifFalse: [session lookupRootClassFor: aClass].
	cache := subCaches at: rootClass ifAbsentPut: [ Cache newFor: rootClass in: self ].
	subCaches at: aClass put: cache.
	^cache!

markAsCurrentOfClass: aClass key: key
	| cache |
	aClass == Proxy ifTrue: [^self].
	cache := self cacheForClass: aClass.
	cache markAsCurrentAtKey: key.!

numberOfElements
	^subCaches inject: 0 into: [:sum :each |
		sum + each numberOfElements].!

release
	subCaches do: [:each | each release].!

removeClass: aClass key: aKey

	^self removeClass: aClass key: aKey ifAbsent: [self error: 'Object not in cache'].!

removeClass: aClass key: aKey ifAbsent: failBlock 
	| cache |
	cache := self cacheForClass: aClass.
	(cache includesKey: aKey withClass: aClass) ifFalse: [^failBlock value].
	cache removeKey: aKey ifAbsent: [ failBlock value ].!

session

	^session.!

session: aSession 
	session := aSession.!

system

	^self session system.! !
!CacheManager categoriesFor: #at:insert:!adding!private! !
!CacheManager categoriesFor: #cacheFor:!private!private/caching! !
!CacheManager categoriesFor: #cacheForClass:!initialize/release!private! !
!CacheManager categoriesFor: #containsObject:key:!private!querying! !
!CacheManager categoriesFor: #containsObjectForClass:key:!private!querying! !
!CacheManager categoriesFor: #expireAll!private!querying! !
!CacheManager categoriesFor: #expireClass:key:!private!querying! !
!CacheManager categoriesFor: #expireClass:key:ifAbsent:!private!querying! !
!CacheManager categoriesFor: #expiredInstanceOf:key:!private!private/caching! !
!CacheManager categoriesFor: #hasExpired:!private!querying! !
!CacheManager categoriesFor: #hasExpired:key:!private!querying! !
!CacheManager categoriesFor: #hasObjectExpiredOfClass:withKey:!private!querying! !
!CacheManager categoriesFor: #initialize!initialize/release!private! !
!CacheManager categoriesFor: #lookupClass:key:!private!querying! !
!CacheManager categoriesFor: #lookupClass:key:ifAbsent:!private!querying! !
!CacheManager categoriesFor: #lookupClass:key:ifAbsentPut:!private!querying! !
!CacheManager categoriesFor: #makeCacheFor:!initialize/release!private! !
!CacheManager categoriesFor: #markAsCurrentOfClass:key:!private!querying! !
!CacheManager categoriesFor: #numberOfElements!accessing!private! !
!CacheManager categoriesFor: #release!initialize/release!private! !
!CacheManager categoriesFor: #removeClass:key:!private!querying! !
!CacheManager categoriesFor: #removeClass:key:ifAbsent:!private!querying! !
!CacheManager categoriesFor: #session!accessing!private! !
!CacheManager categoriesFor: #session:!accessing!private! !
!CacheManager categoriesFor: #system!accessing!private! !

!CacheManager class methodsFor!

forSession: aSession 
	^self new session: aSession.!

new

	^super new initialize.! !
!CacheManager class categoriesFor: #forSession:!instance creation!private! !
!CacheManager class categoriesFor: #new!instance creation!private! !

CachePolicy guid: (GUID fromString: '{4534237C-8F06-4867-85ED-F4C47E549A8C}')!
CachePolicy comment: 'A CachePolicy implements the different possible policies we might use for caching. The superclass implements the trivial policy of keeping all objects forever.

The policy also controls what we store in the cache. In general, it''s assumed to be a cache entry of some sort, and the policy is responsible for wrapping and unwrapping objects going to and from the cache. The default policy is that the objects themselves are the cache entry (saving one object per cached object in overhead).

Instance Variables
	size	<Number>	The minimum cache size we want to use.
	expiryAction <Symbol> What to do when an object has expired. Currently hard-coded as one of #remove, #notify, #refresh, #notifyAndRemove.
'!
!CachePolicy categoriesForClass!Unclassified! !
!CachePolicy methodsFor!

cacheEntryFor: anObject

	^anObject.!

collectionForExtraReferences
	^nil.!

contentsOf: aCacheEntry

	^aCacheEntry.!

dictionaryClass
	^Dictionary.!

expire: anItem
	"Force anItem to be expired. This may be ignored if the policy doesn't respect this, and is ignored by default"!

expiryAction
	^expiryAction!

expiryAction: aSymbol
	"See class comment for possible values"
	expiryAction := aSymbol!

hasExpired: aCacheEntry

	^false.!

initialize

	numberOfElements := 100.
	expiryAction := #remove.!

markEntryAsCurrent: aCacheEntry in: aCache

	^self.!

newItemsIn: aCache
	^self dictionaryClass new: 20.!

notifyOfExpiry: anObject in: aCache 
	anObject glorpNoticeOfExpiryIn: aCache session.!

numberOfElements
	^numberOfElements!

numberOfElements: anInteger
	numberOfElements := anInteger!

release: aCache
	(expiryAction == #notify or: [expiryAction == #notifyAndRemove])
		ifTrue: [
			aCache do: [:each |
				(self contentsOf: each)  glorpNoticeOfExpiryIn: aCache session]].!

takeExpiryActionForKey: key withValue: anObject in: aCache
	expiryAction == #refresh
		ifTrue: [aCache session refresh: anObject].
	(#(#notify #notifyAndRemove) includes: expiryAction) ifTrue: [
		self notifyOfExpiry: anObject in: aCache].
	(#(#remove #notifyAndRemove) includes: expiryAction) ifTrue: [
		aCache removeKey: key ifAbsent: []].!

willRemoveItems
	^(expiryAction == #notifyAndRemove or: [expiryAction == #remove])! !
!CachePolicy categoriesFor: #cacheEntryFor:!private!wrap/unwrap! !
!CachePolicy categoriesFor: #collectionForExtraReferences!initialize!private! !
!CachePolicy categoriesFor: #contentsOf:!private!wrap/unwrap! !
!CachePolicy categoriesFor: #dictionaryClass!accessing!private! !
!CachePolicy categoriesFor: #expire:!expiry!private! !
!CachePolicy categoriesFor: #expiryAction!accessing!private! !
!CachePolicy categoriesFor: #expiryAction:!accessing!private! !
!CachePolicy categoriesFor: #hasExpired:!private!wrap/unwrap! !
!CachePolicy categoriesFor: #initialize!initialize!private! !
!CachePolicy categoriesFor: #markEntryAsCurrent:in:!private!wrap/unwrap! !
!CachePolicy categoriesFor: #newItemsIn:!initialize!private! !
!CachePolicy categoriesFor: #notifyOfExpiry:in:!expiry!private! !
!CachePolicy categoriesFor: #numberOfElements!accessing!private! !
!CachePolicy categoriesFor: #numberOfElements:!accessing!private! !
!CachePolicy categoriesFor: #release:!expiry!private! !
!CachePolicy categoriesFor: #takeExpiryActionForKey:withValue:in:!expiry!private! !
!CachePolicy categoriesFor: #willRemoveItems!expiry!private! !

!CachePolicy class methodsFor!

default
	"Use of the WeakVWCache is tied in with Ephemera which we do not expect to get working in VW3."

	Dialect isVWWithNameSpaces ifTrue: [^WeakVWCachePolicy new].
	^self new!

new

	^super new initialize.! !
!CachePolicy class categoriesFor: #default!instance creation!private! !
!CachePolicy class categoriesFor: #new!instance creation!private! !

FixedSizeQueue guid: (GUID fromString: '{4B34235C-B3AD-4ABD-8195-4343BB43E021}')!
FixedSizeQueue comment: 'This is a fixed size queue of objects. It''s intended for keeping around a fixed number of references to objects in a weak dictionary. As such its API is rather limited (one method), and it''s write-only.

Instance Variables:
	items	<OrderedCollection>	The items in the queue
	maximumSize	<Integer>	How many items we''re allowed'!
!FixedSizeQueue categoriesForClass!Unclassified! !
!FixedSizeQueue methodsFor!

add: anObject
	items add: anObject.
	items size > maximumSize ifTrue: [items removeFirst].!

maximumSize
	^maximumSize!

maximumSize: anInteger
	maximumSize := anInteger.
	items := OrderedCollection new: maximumSize + 1.!

printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: '('.
	aStream nextPutAll: items size printString.
	aStream nextPut: $/.
	aStream nextPutAll: maximumSize printString.
	aStream nextPutAll: ')'.! !
!FixedSizeQueue categoriesFor: #add:!api!private! !
!FixedSizeQueue categoriesFor: #maximumSize!accessing!private! !
!FixedSizeQueue categoriesFor: #maximumSize:!accessing!private! !
!FixedSizeQueue categoriesFor: #printOn:!printing!private! !

!FixedSizeQueue class methodsFor!

maximumSize: anInteger
	^self basicNew maximumSize: anInteger.!

new
	self error: 'must supply a size'.!

new: anInteger
	^self maximumSize: anInteger.! !
!FixedSizeQueue class categoriesFor: #maximumSize:!instance creation!private! !
!FixedSizeQueue class categoriesFor: #new!instance creation!private! !
!FixedSizeQueue class categoriesFor: #new:!instance creation!private! !

GlorpCorePackage guid: (GUID fromString: '{F47E40F4-099C-4F98-B7C2-C2CD6D85B489}')!
GlorpCorePackage comment: 'This class represents package information for the GlorpCore package, for systems that don''t have things like package properties to store things like copyright and license information.

LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you''ll need to release the
source of any modifications.

If you require any further clarification, we''d be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'!
!GlorpCorePackage categoriesForClass!Unclassified! !
!GlorpCorePackage class methodsFor!

LICENSE
"LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you'll need to release the
source of any modifications.

If you require any further clarification, we'd be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"! !
!GlorpCorePackage class categoriesFor: #LICENSE!instance creation!public! !

GlorpSession guid: (GUID fromString: '{B1974966-942E-4116-A6D1-47623DF02C0C}')!
GlorpSession comment: 'This is the main class for communicating between the user and Glorp. It acts as a broker for reading queries, and as the main point for registering objects for modification.

Instance Variables:

	system	<DescriptorSystem>	The definition of our tables, classes, and the relationships between them.
	currentUnitOfWork	<UnitOfWork | nil>	If we have an active unit of work, this will hold it. All modifications are tracked through the unit of work.
	cache	<CacheManager>	Our cache of objects that we''ve already read.
	accessor	<DatabaseAccessor>	Our connection to the database.
	application	<?>	application-specific data
	reaper <TimedProxyReaper> If we are using TimedProxy for some of our relationships, this holds the reaper which will expire them on timeout.
	errorAction <Block> What block should we evaluate when there''s an error.
	modificationTracker <GlorpVWWriteBarrierTracker> VisualWorks-specific. This holds the GlorpVWWriteBarrierTracker in use for this session. This uses immutability to keep track of objects which are changed.
'!
!GlorpSession categoriesForClass!Unclassified! !
!GlorpSession methodsFor!

accessor
	^accessor!

accessor: aDatabaseAccessor 
	accessor := aDatabaseAccessor.
	system isNil ifFalse: [system platform: (accessor platform)].!

addToReaper: aProxy

	aProxy class == TimedProxy ifFalse: [^self].
	self reaper add: aProxy.!

applicationData
	^applicationData!

applicationData: anObject	
	applicationData := anObject!

beginTransaction
	accessor beginTransaction.!

beginUnitOfWork

	self hasUnitOfWork ifTrue: [self error: 'Cannot nest units of work yet'].
	currentUnitOfWork := UnitOfWork new.
	currentUnitOfWork session: self.!

cacheAt: aKey  forClass: aClass ifNone: failureBlock

	^cache lookupClass: aClass key: aKey ifAbsent: failureBlock.!

cacheAt: keyObject put: valueObject 
	^cache at: keyObject insert: valueObject!

cacheContainsObject: anObject
	| descriptor key |
	(descriptor := self descriptorFor: anObject) isNil ifTrue: [^false].
	(key := descriptor primaryKeyFor: anObject) isNil ifTrue: [^false].
	^[self cacheContainsObject: anObject key: key] on: DuplicatePrimaryKeyException do: [:ex | false]!

cacheContainsObject: anObject key: aKey 
	"Just test containment, don't return the result or trigger anything due to expiration"
	aKey isNil ifTrue: [^false].
	^cache
		containsObject: anObject
		key: aKey.!

cacheContainsObjectForClass: aClass key: aKey 
	"Just test containment, don't return the result or trigger anything due to expiration"
	aKey isNil ifTrue: [^false].
	^cache
		containsObjectForClass: aClass
		key: aKey.!

cacheContainsObjectForRow: aDatabaseRow
	^self 
		cacheContainsObjectForClass: aDatabaseRow owner class
		key: aDatabaseRow primaryKey!

cacheFor: anObject
	^cache cacheFor: anObject.!

cacheForClass: aClass
	^cache cacheForClass: aClass.!

cacheLookupForClass: aClass key: aKey 
	^self 
		cacheAt: aKey
		forClass: aClass
		ifNone: [nil]!

cacheLookupObjectForRow: aDatabaseRow
	^self 
		cacheLookupForClass: aDatabaseRow owner class
		key: aDatabaseRow primaryKey!

cacheRemoveObject: anObject
	| key |
	key := (self descriptorFor: anObject) primaryKeyFor: anObject.
	cache removeClass: anObject class key: key ifAbsent: [].!

commandForHomogeneousRows: aCollectionOfDatabaseRows
	"Create the command for this set of rows. These are assumed to be homogeneous, in that they are all for the same table, and are all the same type of operation (i.e. all insert or all update)"
	| representativeRow |
	representativeRow := aCollectionOfDatabaseRows first.
	representativeRow forDeletion ifTrue: [self error: 'invalid way of handling deletes (this should never happen)'].
	^(self shouldInsert: representativeRow)
		ifTrue: [InsertCommand forRows: aCollectionOfDatabaseRows useBinding: self useBinding session: self]
		ifFalse: [UpdateCommand forRows: aCollectionOfDatabaseRows useBinding: self useBinding session: self].!

commandForRow: aDatabaseRow 
	
	(aDatabaseRow forDeletion) ifTrue: [^DeleteCommand forRow: aDatabaseRow useBinding: self useBinding session: self].
	^(self shouldInsert: aDatabaseRow)
		ifTrue: [^InsertCommand forRow: aDatabaseRow useBinding: self useBinding session: self]
		ifFalse: [^UpdateCommand forRow: aDatabaseRow useBinding: self useBinding session: self].!

commitTransaction
	accessor commitTransaction!

commitUnitOfWork
	[self isInTransaction
		ifTrue: [currentUnitOfWork commit]
		ifFalse: [self inTransactionDo: [currentUnitOfWork commit]]] 
	ifCurtailed: [self rollbackUnitOfWork].
	currentUnitOfWork := nil.!

commitUnitOfWorkAndContinue
	"Commit the current unit of work, but then keep going with the same set of registered objects, with their state updated to reflect current values."
	| registeredObjects |	
	currentUnitOfWork isNil ifTrue: [^self error: 'Not in unit of work'].
	registeredObjects := currentUnitOfWork registeredObjects.
	currentUnitOfWork deletedObjects do: [:each | registeredObjects remove: each].
	self commitUnitOfWork.
	self beginUnitOfWork.
	self registerAll: registeredObjects.!

copy

	^self shallowCopy postCopy.!

count: aClass
	^self execute: (SimpleQuery count: aClass).!

count: aClass where: anExpression
	^self execute: (SimpleQuery count: aClass where: anExpression).!

createDeleteRowsFor: anObject in: rowMap
	"Create records for rows that require deletion"

	(self descriptorFor: anObject) createDeleteRowsFor: anObject in: rowMap.!

createRowsFor: anObject in: rowMap
	| descriptor |
	(descriptor := self descriptorFor: anObject class) isNil
		ifFalse: [descriptor createRowsFor: anObject in: rowMap].!

createTables
	"Create the tables for our descriptor system. This will fail if the tables already exist. Use recreateTables for that."

	self createTables: system allTables.!

createTables: aCollection
	"Create the tables for our descriptor system. This will fail if the tables already exist. Use recreateTables for that."

	| sequences |
	system platform areSequencesExplicitlyCreated
		ifTrue:
			[sequences := self system allSequencesFor: aCollection.
			sequences do: [:each | accessor createSequence: each ifError: [:ex | self halt]]].
	aCollection do: [:each | accessor createTable: each ifError: [:ex | self halt]].
	aCollection do: [:each | accessor createTableIndexes: each ifError: [:ex | self halt]].
	self platform supportsConstraints ifTrue: [aCollection do: [:each | accessor createTableFKConstraints: each ifError: [:ex | self halt]]].!

currentLogin
	^accessor currentLogin!

currentServerUTCTimestampOrNil
	"This is kind of ugly, but it is Smalltalk Dialect neutral"

	| sqlString result |
	sqlString := [self platform sqlTextCurrentServerUTCTimestamp]
		on: Dialect error
		do:  [:exception | exception return: nil].
	sqlString == nil ifTrue: [^nil].
	result := [self accessor executeSQLString: sqlString]
		on: Dialect error
		do:  [:exception | exception return: nil].
	^result == nil
		ifTrue: [nil]
		ifFalse: 
			[(result first isEmpty or: [(result first first isKindOf: Dialect timestampClass) not])
				ifTrue: [nil]
				ifFalse: [result first first]]!

delete: anObject
	| realObject |
	"Get the real object, instantiating if necessary"
	realObject := anObject glorpRealObject.
	self hasUnitOfWork
		ifTrue: [currentUnitOfWork delete: realObject]
		ifFalse:
			[self inUnitOfWorkDo: [
				currentUnitOfWork delete: realObject]].!

deleteAll: aCollection

 	aCollection do: [:each | self delete: each].!

descriptorFor: aClassOrInstance

	^system descriptorFor: aClassOrInstance.!

doDDLOperation: aBlock
	"Execute DDL SQL - something which creates/drops/modifies tables or other schema information. The main reason for this method is that some databases require a transaction, while others don't require it, and still others don't allow it. Try to do the right thing."
	
	^self accessor doDDLOperation: aBlock.!

dontUseModificationTracker
	modificationTracker release.
	modificationTracker := nil.!

dropTables
	"Drop the tables for our descriptor system, and their associated sequences."

	self dropTables: system allTables.!

dropTables: aCollection
	"Drop the tables for our descriptor system, and their associated sequences."

	| sequences |
	accessor dropTables: ((TableSorter for: aCollection) sort reverse).
	system platform areSequencesExplicitlyCreated ifTrue: [
		sequences := system allSequencesFor: aCollection.
		accessor dropSequences: sequences].!

errorAction
	^errorAction!

errorAction: aThreeArgumentBlockExceptionSessionCommand
	errorAction := aThreeArgumentBlockExceptionSessionCommand.!

execute: aQuery
	^aQuery executeWithParameters: #() in: self.!

expiredInstanceOf: aClass key: key

	^cache expiredInstanceOf: aClass key: key.!

filterDeletionFrom: anObject

	self hasUnitOfWork ifFalse: [^anObject].
	currentUnitOfWork shouldFilterDeletions ifFalse: [^anObject].
	(currentUnitOfWork willDelete: anObject)
		ifTrue: [^nil].
	^anObject.!

forceSaveOf: anObject 
	"This is the same as registering an object, but it tells us to disregard whether/how the object has changed and to save all of its fields. This is useful if we need to register an object after changes have occurred, and can be used to implement something more like an object by object save facility. Still somewhat experimental."
	| realObject |
	realObject := self realObjectFor: anObject ifNone: [^self].
	
	self inUnitOfWorkDo: [currentUnitOfWork registerAsNew: realObject].!

groupWrite: aCollectionOfDatabaseRows
	| command |
	aCollectionOfDatabaseRows
		do: [:each | each preWriteAssignSequencesUsing: self].
	command := self commandForHomogeneousRows: aCollectionOfDatabaseRows.
	accessor executeCommand: command.
	^command succeeded
		ifTrue: [true]
		ifFalse:
			[((GlorpWriteFailure new)
				command: command;
				object: (aCollectionOfDatabaseRows collect: [:each | each owner]))
				signal].!

hasDescriptorFor: aClass

	^system hasDescriptorFor: aClass.!

hasExpired: anObject 
	^cache hasExpired: anObject.!

hasExpired: aClass key: key
	^cache hasExpired: aClass key: key.!

hasObjectExpiredOfClass: aClass withKey: key
	^cache hasObjectExpiredOfClass: aClass withKey: key.!

hasUnitOfWork

	^currentUnitOfWork notNil.!

initialize
	
	self initializeCache.!

initializeCache

	cache release.
	cache := CacheManager forSession: self.!

initializeReaper

	self releaseReaper.
	reaper := TimedProxyReaper new.!

inTransactionDo: aBlock
	"Execute aBlock in a database transaction. If one is already in progress, just continue. If not, start a transaction."
	^self accessor inTransactionDo: aBlock.!

inUnitOfWorkDo: aBlock
	| result myUnitOfWork |
	[
	myUnitOfWork := self hasUnitOfWork not.
	myUnitOfWork ifTrue: [self beginUnitOfWork].
	result := aBlock argumentCount = 1 ifTrue: [aBlock value: self] ifFalse: [aBlock value].
	myUnitOfWork ifTrue: [
		self commitUnitOfWork]]
		ifCurtailed:
			[myUnitOfWork ifTrue: [self rollbackUnitOfWork]].
	^result.!

isInTransaction
	^accessor isInTransaction.!

isLoggedIn
	^accessor isLoggedIn.!

isNew: anObject 
	"When registering, do we need to add this object to the collection of new objects? New objects are treated specially when computing what needs to be written, since we don't have their previous state"

	| key descriptor |
	(currentUnitOfWork notNil and: [currentUnitOfWork isRegistered: anObject]) ifTrue: [^false].
	descriptor := self descriptorFor: anObject.
	descriptor isNil ifTrue: [^false].
	"For embedded values we assume that they are not new. This appears to work. I can't really justify it."
	self needsWork: 'cross your fingers'.
	descriptor mapsPrimaryKeys ifFalse: [^false].

	key := descriptor primaryKeyFor: anObject.
	key isNil ifTrue: [^true].
	"If the cache contains the object, but the existing entry is due to be deleted, then count this entry as a new one being added with the same primary key (ick) as the old one"
	^[(self cacheContainsObject: anObject key: key) not]
		on: DuplicatePrimaryKeyException
		do: [:ex |
			(currentUnitOfWork notNil and: [currentUnitOfWork willDelete: ex existingObject]) 
				ifTrue: [
					self cacheRemoveObject: ex existingObject.
					ex return: true]
				ifFalse: [ex pass]].!

isRegistered: anObject

	currentUnitOfWork isNil ifTrue: [^false].
	^currentUnitOfWork isRegistered: anObject.!

isUninstantiatedProxy: anObject
	^anObject isGlorpProxy and: [anObject isInstantiated not].!

login
	self isLoggedIn ifTrue: [^nil].
	^self loginIfError: [:ex | ex pass]!

loginIfError: aBlock
	| result |
	result := self accessor loginIfError: aBlock.
	system platform characterEncoding: accessor encoding.
	^result.!

logout
	^self accessor logout.!

lookupRootClassFor: aClass
	| descriptor |
	descriptor := self system descriptorFor: aClass.
	^descriptor notNil
			ifTrue: [ descriptor typeMappingRootDescriptor describedClass ]
			ifFalse: [ aClass ]!

markAsCurrentOfClass: aClass key: key

	cache markAsCurrentOfClass: aClass key: key.!

migrateFromSystem: from toSystem: to
	| fromTables toTables toDelete toAdd toModify newVersion |
	fromTables := from allTables copy.
	toTables := to allTables copy.
	toDelete := fromTables reject: [:each | 
		toTables anySatisfy: [:otherTable | otherTable sqlString sameAs: each sqlString]].
	toAdd := toTables reject: [:each | 
		fromTables anySatisfy: [:otherTable | otherTable sqlString sameAs: each sqlString]].
	toModify := fromTables reject: [:each | (toDelete includes: each) or: [toAdd includes: each]].
	self doDDLOperation: [
		self createTables: toAdd.
		self dropTables: toDelete.
		toModify do: [:each | 
			newVersion := to tableNamed: each sqlString.
			self updateTableFrom: each to: newVersion]].!

migrateFromVersion: fromVersionNumber toVersion: toVersionNumber 
	self halt.
	"Get the class name of our system, strip off version number, replace with to and from, create systems for them."!

modificationTracker
	"If we killed off the tracker due to a session reset, then bring it back again."
	modificationTracker == #needed ifTrue: [self useModificationTracker].
	^modificationTracker!

modificationTracker: aGlorpModificationTrackingPolicy
	modificationTracker := aGlorpModificationTrackingPolicy!

modify: anObject in: aBlock
	"Evaluate aBlock inside a unit of work. Start a database transaction at the beginning and commit it at the end. If we don't terminate normally, roll everything back. Register anObject before starting. Explicitly circumvent the registration of  the block result at the end, because that can cause problems if we're doing things like delete and re-insert and end up returning e.g. the deleted object"
	| result |
	self transact: [
		self register: anObject.
		result := aBlock value.
		nil].
	^result.!

platform
	^self system platform.!

postCopy

	super postCopy.
	self initializeCache.
	currentUnitOfWork := nil.
	accessor := accessor copy.!

privateGetCache

	^cache.!

privateGetCurrentUnitOfWork

	^currentUnitOfWork.!

privateRegisterAsNew: anObject 
	"Register the object as one which is not already in cache. Private!! Normally just use the register: operation."
	currentUnitOfWork isNil ifTrue: [^self].
	currentUnitOfWork registerAsNew: anObject.!

privateRegisterAsOld: anObject 
	"Register the object as something we already read from the database, skipping the isNew: test. Private!! Normally you would just use register:"
	| realObject |
	currentUnitOfWork isNil ifTrue: [^self].
	realObject := self realObjectFor: anObject ifNone: [^self].

	currentUnitOfWork register: realObject!

read: aClass
	^self execute: (SimpleQuery returningManyOf: aClass).!

read: aClass limit: anInteger
	^self execute: (SimpleQuery returningManyOf: aClass limit: anInteger).!

read: aClass orderBy: aCriteria
	| query |
	query := SimpleQuery returningManyOf: aClass.
	query orderBy: aCriteria.
	^self execute: query.!

read: aClass where: aBlock
	^self execute: (SimpleQuery returningManyOf: aClass where: aBlock).!

read: aClass where: aBlock limit: anInteger
	^self
		execute:
			(SimpleQuery
				returningManyOf: aClass
				where: aBlock
				limit: anInteger).!

readManyOf: aClass
	^self execute: (SimpleQuery returningManyOf: aClass).!

readManyOf: aClass limit: anInteger
	^self execute: (SimpleQuery returningManyOf: aClass limit: anInteger).!

readManyOf: aClass orderBy: aCriteria
	| query |
	query := SimpleQuery returningManyOf: aClass.
	query orderBy: aCriteria.
	^self execute: query.!

readManyOf: aClass where: aBlock
	^self execute: (SimpleQuery returningManyOf: aClass where: aBlock).!

readManyOf: aClass where: aBlock limit: anInteger
	^self
		execute:
			(SimpleQuery
				returningManyOf: aClass
				where: aBlock
				limit: anInteger).!

readOneOf: aClass
	^self execute: (SimpleQuery returningOneOf: aClass).!

readOneOf: aClass where: aBlock
	^self execute: (Query returningOneOf: aClass where: aBlock).!

realObjectFor: anObject 
	"If this is a proxy, return the contents (if available). Otherwise, return nil"
	^self realObjectFor: anObject ifNone: [nil].!

realObjectFor: anObject ifNone: aBlock
	"If this is a proxy, return the contents (if available). Otherwise, evalute the block"
	^anObject isGlorpProxy 
		ifTrue: [anObject isInstantiated ifTrue: [anObject getValue] ifFalse: [aBlock value]]
		ifFalse: [anObject]!

reaper

	reaper isNil ifTrue: [self initializeReaper].
	^reaper.!

recreateTables
	"Drop and re-create the tables for our descriptor system, and their associated sequences."

	self recreateTables: system allTables.!

recreateTables: aCollection
	"Drop the tables for our descriptor system, and their associated sequences."

	| sequences |
	accessor dropTables: (TableSorter for: aCollection) sort reverse.
	system platform areSequencesExplicitlyCreated
		ifTrue:
			[
			sequences := self system allSequencesFor: aCollection.
			accessor dropSequences: sequences.
			sequences
				do: [:each | accessor createSequence: each ifError: [:ex | self halt]]].
	aCollection do: [:each | accessor createTable: each ifError: [:ex | self halt]].
	aCollection do: [:each | accessor createTableIndexes: each ifError: [:ex | self halt]].
	self platform supportsConstraints ifTrue: [aCollection do: [:each | accessor createTableFKConstraints: each ifError: [:ex | self halt]]].!

recreateTables: aCollection ifError: aBlock
	"Drop the tables for our descriptor system, and their associated sequences."

	(TableSorter for: aCollection) sort reverse
		do: [ :table | accessor dropTable: table ifError: [ :ex | ex "so we can insert a breakpoint"] ].

	system platform areSequencesExplicitlyCreated
		ifTrue: [(self system allSequencesFor: aCollection)
					do: [:each | 
						accessor dropSequence: each ifAbsent: [:ex | ex "so we can insert a breakpoint" ].
						accessor createSequence: each ifError: [:ex | ^aBlock value: ex ]]].

	aCollection do: [:each | accessor createTable: each ifError: [:ex | ^aBlock value: ex]].
	aCollection do: [:each | accessor createTableIndexes: each ifError: [:ex | ^aBlock value: ex ]].
	self platform supportsConstraints ifTrue: [aCollection do: [:each | accessor createTableFKConstraints: each ifError: [:ex | ^aBlock value: ex]]].!

recreateTablesIfError: aBlock
	"Drop and re-create the tables for our descriptor system, and their associated sequences."

	self recreateTables: system allTables ifError: aBlock .!

refresh: anObject
	| exp query realObject descriptor |
	realObject := self realObjectFor: anObject ifNone: [^self].
	descriptor := self descriptorFor: realObject.
	descriptor isNil
		ifTrue: [self error: 'Cannot refresh an object with no descriptor'].
	exp := descriptor
		primaryKeyExpressionFor: realObject
		basedOn: BaseExpression new
		relation: #=.
	query := Query returningOneOf: realObject class where: exp.
	query shouldRefresh: true.
	^self execute: query.!

register: anObject 
	"Register the object with our current unit of work. We need to do this *before* making changes to the objects."
	| realObject |
	currentUnitOfWork isNil ifTrue: [^self].
	realObject := self realObjectFor: anObject ifNone: [^self].

	(self isNew: realObject) 
		ifTrue: [currentUnitOfWork registerAsNew: realObject]
		ifFalse: [currentUnitOfWork register: realObject]!

registerAll: aCollection
	aCollection do: [:ea | self register: ea].!

registerForDelete: anObject 
	"Register the object with our current unit of work, with intent to delete it."
	| realObject |
	currentUnitOfWork isNil ifTrue: [^self].
	realObject := self realObjectFor: anObject ifNone: [^self].

	currentUnitOfWork registerForDelete: realObject.!

releaseReaper

	reaper release.!

requireUnitOfWork
	"Start a unit of work if there isn't one already."
	self hasUnitOfWork ifFalse: [self beginUnitOfWork].!

reset
	self initializeCache.
	self releaseReaper.
	accessor reset.
	currentUnitOfWork := nil.
	modificationTracker release.
	modificationTracker isNil ifFalse: [modificationTracker := #needed].!

resetPermissions
	accessor isNil ifFalse: [accessor resetPermissions].!

reusePreparedStatements
	^accessor reusePreparedStatements!

reusePreparedStatements: aBoolean
	accessor reusePreparedStatements: aBoolean.!

rollbackTransaction
	accessor execute: [ accessor rollbackTransaction].!

rollbackUnitOfWork

	currentUnitOfWork isNil ifTrue: [^self].
	currentUnitOfWork abort.
	currentUnitOfWork := nil.!

rollbackUnitOfWorkFor: anObject
	currentUnitOfWork abort: anObject!

save: anObject 
	"Make sure anObject will be written to the database by starting a unit of work if we aren't already in one, and registering the object. This is in most ways the same as just registering an object, but it tells us to disregard whether/how the object has changed and to save all of its fields. This is useful if we need to register an object *after* changes have occurred, and can be used to implement something more like an object by object save facility. Still somewhat experimental."
	| realObject |
	realObject := self realObjectFor: anObject ifNone: [^self].
	
	self hasUnitOfWork
		ifTrue:	[currentUnitOfWork registerAsNew: realObject]
		ifFalse:
			[self inUnitOfWorkDo: [
				currentUnitOfWork registerAsNew: realObject]].!

saveAndContinue
	"A shorter synonym"
	^self commitUnitOfWorkAndContinue.!

schema

	^ self accessor schema!

sendPostFetchEventTo: anObject
	^anObject glorpPostFetch: self.!

sendPostFetchValidateEventTo: anObject
	^anObject glorpPostFetchValidate: self.!

sendPostWriteEventTo: anObject
	anObject glorpPostWrite: self.!

sendPreWriteEventTo: anObject
	anObject glorpPreWrite: self.!

shouldInsert: aDatabaseRow 
	(currentUnitOfWork notNil and: [currentUnitOfWork forcesUpdateFor: aDatabaseRow]) ifTrue: [^false].
	^(self cacheContainsObjectForRow: aDatabaseRow) not.!

shouldUseModificationTracker: aBoolean
	aBoolean
		ifTrue: [modificationTracker isNil ifTrue: [self useModificationTracker]]
		ifFalse: [modificationTracker := nil].!

system
	^system!

system: aSystem 
	aSystem session: self.
	accessor isNil ifFalse: [aSystem platform: (accessor currentLogin database)].
	system := aSystem.
	system validate.!

tablesInCommitOrder

	^system tablesInCommitOrder.!

transact: aBlock
	"Evaluate aBlock inside a unit of work. Start a database transaction at the beginning and commit it at the end. If we don' terminate normally, roll everything back.  This might more consistently be  called inUnitOfWorkWithTransactionDo:, but that's too verbose"
	| alreadyInTransaction alreadyInUnitOfWork result |
	[[
	alreadyInUnitOfWork := false.
	alreadyInTransaction := self isInTransaction.
	alreadyInTransaction ifFalse: [self beginTransaction].
	alreadyInUnitOfWork := self hasUnitOfWork.
	alreadyInUnitOfWork ifFalse: [self beginUnitOfWork].
	result := aBlock argumentCount = 1 ifTrue: [aBlock value: self] ifFalse: [aBlock value].
	self register: result.
	alreadyInUnitOfWork ifFalse: [self commitUnitOfWork].
	alreadyInTransaction ifFalse: [self commitTransaction]]
		ifCurtailed:
			[alreadyInUnitOfWork ifFalse: [self rollbackUnitOfWork]]]
			ifCurtailed: [alreadyInTransaction ifFalse: [self rollbackTransaction]].
	^result.!

undelete: anObject
	self undelete: anObject ifNotDeleted: [self error: 'Trying to undelete something that was not deleted'].!

undelete: anObject ifNotDeleted: aBlock
	| realObject |
	"Get the real object, instantiating if necessary"
	realObject := anObject glorpRealObject.
	self hasUnitOfWork
		ifTrue: [currentUnitOfWork undelete: realObject ifNotDeleted: aBlock]
		ifFalse:
			[self error: 'Can only undelete while the unit of work is in progress'].!

updateTableFrom: aTable to: newTable
	"Update the given table based on the metadata."
	
	| newFields deletedFields modifiedFields newFKConstraints deletedFKConstraints newUniqueConstraints deletedUniqueConstraints |
	newFKConstraints := newTable foreignKeyConstraints reject: 
		[:each | aTable hasForeignKeyConstraintMatching: each].
	deletedFKConstraints := aTable foreignKeyConstraints reject: 
		[:each | newTable hasForeignKeyConstraintMatching: each].
	newUniqueConstraints := newTable uniqueConstraints reject: 
		[:each | aTable hasUniqueConstraintMatching: each].
	deletedUniqueConstraints := aTable uniqueConstraints reject: 
		[:each | newTable hasUniqueConstraintMatching: each].
	"So far, we just handle adding and deleting fields, not changing them."
	newFields := newTable fields reject: 
		[:each | aTable hasFieldNamed: each name].
	deletedFields := aTable fields reject: 
		[:each | newTable hasFieldNamed: each name].
	modifiedFields := newTable fields reject: 
		[:each | newFields includes: each].
	modifiedFields := modifiedFields reject: 
		[:each | (aTable fieldNamed: each name) isTheSameAs: each].	
	"Cheap version, for modified fields, drop and re-add them."
	newFields addAll: modifiedFields.
	deletedFields addAll: modifiedFields.
	deletedFKConstraints do: [:each | accessor dropConstraint: each].
	deletedUniqueConstraints do: [:each | accessor dropConstraint: each].
	accessor dropFields: deletedFields.
	accessor addFields: newFields.
	newFKConstraints do: [:each | accessor addConstraint: each].
	newUniqueConstraints do: [:each | accessor addConstraint: each]!

useBinding
	self accessor isNil ifTrue: [^false].
	^self accessor useBinding.!

useModificationTracker
	Dialect isVisualWorks ifFalse: [^self].
	modificationTracker := (Smalltalk
		at: #'Glorp.GlorpVWWriteBarrierTracker'
		ifAbsent: [nil]) ifNotNil: [:class | class new].
	modificationTracker session: self.!

virtualCollectionOf: aClass
	^GlorpVirtualCollection on: aClass in: self.!

writeHomogeneousRows: aCollectionOfDatabaseRows
	"Write out a collection of rows belonging to a single table"

	| representativeRow succeeded maximumSize start |

	aCollectionOfDatabaseRows isEmpty ifTrue: [^true].
	representativeRow := aCollectionOfDatabaseRows first.
	representativeRow shouldBeWritten ifFalse: [^true].
	succeeded := true.
	maximumSize := self system maximumSizeToGroupWriteFor: aCollectionOfDatabaseRows.
	start := 1.
	[start > aCollectionOfDatabaseRows size] whileFalse: [
		| end |
		end := (start + maximumSize - 1) min: aCollectionOfDatabaseRows size.
		succeeded := succeeded
			& (self writeHomogeneousRows: aCollectionOfDatabaseRows from: start to: end).
		start := end + 1].
	^succeeded.!

writeHomogeneousRows: aCollectionOfDatabaseRows from: startIndex to: endIndex
	"Write out the specified range from a collection of rows belonging to a single table"

	| succeeded |
	succeeded := true.
	^(self system 	
			canUseGroupedWritesFor: (aCollectionOfDatabaseRows at: startIndex) table)
		ifTrue: [self groupWrite: (aCollectionOfDatabaseRows copyFrom: startIndex to: endIndex)]
		ifFalse:
			[startIndex to: endIndex do: [:i |
				 |eachRow rowSucceeded |
				eachRow := aCollectionOfDatabaseRows at: i.
				rowSucceeded := self writeRow: eachRow.
				succeeded := succeeded & rowSucceeded].
			succeeded].!

writeRow: aDatabaseRow
	| command success |
	aDatabaseRow shouldBeWritten ifFalse: [^true].
	aDatabaseRow preWriteAssignSequencesUsing: self.
	command := self commandForRow: aDatabaseRow.
	accessor executeCommand: command.
	success := true.
	command succeeded
		ifFalse:
			[success := (GlorpWriteFailure new)
				command: command;
				object: (Array with: aDatabaseRow owner);
				signal].
	^success.! !
!GlorpSession categoriesFor: #accessor!accessing!private! !
!GlorpSession categoriesFor: #accessor:!accessing!private! !
!GlorpSession categoriesFor: #addToReaper:!accessing!private! !
!GlorpSession categoriesFor: #applicationData!accessing!private! !
!GlorpSession categoriesFor: #applicationData:!accessing!private! !
!GlorpSession categoriesFor: #beginTransaction!api/transactions!private! !
!GlorpSession categoriesFor: #beginUnitOfWork!api/transactions!private! !
!GlorpSession categoriesFor: #cacheAt:forClass:ifNone:!caching!private! !
!GlorpSession categoriesFor: #cacheAt:put:!caching!private! !
!GlorpSession categoriesFor: #cacheContainsObject:!caching!private! !
!GlorpSession categoriesFor: #cacheContainsObject:key:!caching!private! !
!GlorpSession categoriesFor: #cacheContainsObjectForClass:key:!caching!private! !
!GlorpSession categoriesFor: #cacheContainsObjectForRow:!caching!private! !
!GlorpSession categoriesFor: #cacheFor:!accessing!private! !
!GlorpSession categoriesFor: #cacheForClass:!accessing!private! !
!GlorpSession categoriesFor: #cacheLookupForClass:key:!caching!private! !
!GlorpSession categoriesFor: #cacheLookupObjectForRow:!caching!private! !
!GlorpSession categoriesFor: #cacheRemoveObject:!caching!private! !
!GlorpSession categoriesFor: #commandForHomogeneousRows:!As yet unclassified!private! !
!GlorpSession categoriesFor: #commandForRow:!internal/writing!public! !
!GlorpSession categoriesFor: #commitTransaction!api/transactions!private! !
!GlorpSession categoriesFor: #commitUnitOfWork!api/transactions!private! !
!GlorpSession categoriesFor: #commitUnitOfWorkAndContinue!api/transactions!private! !
!GlorpSession categoriesFor: #copy!copying!private! !
!GlorpSession categoriesFor: #count:!api/queries!private! !
!GlorpSession categoriesFor: #count:where:!api/queries!private! !
!GlorpSession categoriesFor: #createDeleteRowsFor:in:!internal/writing!private! !
!GlorpSession categoriesFor: #createRowsFor:in:!internal/writing!private! !
!GlorpSession categoriesFor: #createTables!private!table creation! !
!GlorpSession categoriesFor: #createTables:!public!table creation! !
!GlorpSession categoriesFor: #currentLogin!accessing!private! !
!GlorpSession categoriesFor: #currentServerUTCTimestampOrNil!accessing!public! !
!GlorpSession categoriesFor: #delete:!api/queries!private! !
!GlorpSession categoriesFor: #deleteAll:!api/queries!private! !
!GlorpSession categoriesFor: #descriptorFor:!api!private! !
!GlorpSession categoriesFor: #doDDLOperation:!api/transactions!private! !
!GlorpSession categoriesFor: #dontUseModificationTracker!initialize!private! !
!GlorpSession categoriesFor: #dropTables!private!table creation! !
!GlorpSession categoriesFor: #dropTables:!private!table creation! !
!GlorpSession categoriesFor: #errorAction!accessing!private! !
!GlorpSession categoriesFor: #errorAction:!accessing!private! !
!GlorpSession categoriesFor: #execute:!api/queries!private! !
!GlorpSession categoriesFor: #expiredInstanceOf:key:!private! !
!GlorpSession categoriesFor: #filterDeletionFrom:!private!read/write! !
!GlorpSession categoriesFor: #forceSaveOf:!api!private! !
!GlorpSession categoriesFor: #groupWrite:!private!read/write! !
!GlorpSession categoriesFor: #hasDescriptorFor:!api!private! !
!GlorpSession categoriesFor: #hasExpired:!api/queries!private! !
!GlorpSession categoriesFor: #hasExpired:key:!caching!private! !
!GlorpSession categoriesFor: #hasObjectExpiredOfClass:withKey:!caching!private! !
!GlorpSession categoriesFor: #hasUnitOfWork!api/transactions!private! !
!GlorpSession categoriesFor: #initialize!initialize!private! !
!GlorpSession categoriesFor: #initializeCache!initialize!private! !
!GlorpSession categoriesFor: #initializeReaper!initialize!private! !
!GlorpSession categoriesFor: #inTransactionDo:!api/transactions!private! !
!GlorpSession categoriesFor: #inUnitOfWorkDo:!api/transactions!private! !
!GlorpSession categoriesFor: #isInTransaction!api/transactions!private! !
!GlorpSession categoriesFor: #isLoggedIn!login!private! !
!GlorpSession categoriesFor: #isNew:!private!testing! !
!GlorpSession categoriesFor: #isRegistered:!caching!private! !
!GlorpSession categoriesFor: #isUninstantiatedProxy:!private!testing! !
!GlorpSession categoriesFor: #login!login!public! !
!GlorpSession categoriesFor: #loginIfError:!login!private! !
!GlorpSession categoriesFor: #logout!login!private! !
!GlorpSession categoriesFor: #lookupRootClassFor:!caching!private! !
!GlorpSession categoriesFor: #markAsCurrentOfClass:key:!private! !
!GlorpSession categoriesFor: #migrateFromSystem:toSystem:!private!table creation! !
!GlorpSession categoriesFor: #migrateFromVersion:toVersion:!private!table creation! !
!GlorpSession categoriesFor: #modificationTracker!accessing!private! !
!GlorpSession categoriesFor: #modificationTracker:!accessing!private! !
!GlorpSession categoriesFor: #modify:in:!api/transactions!private! !
!GlorpSession categoriesFor: #platform!accessing!private! !
!GlorpSession categoriesFor: #postCopy!copying!private! !
!GlorpSession categoriesFor: #privateGetCache!private! !
!GlorpSession categoriesFor: #privateGetCurrentUnitOfWork!private! !
!GlorpSession categoriesFor: #privateRegisterAsNew:!private! !
!GlorpSession categoriesFor: #privateRegisterAsOld:!private! !
!GlorpSession categoriesFor: #read:!api/queries!private! !
!GlorpSession categoriesFor: #read:limit:!api/queries!private! !
!GlorpSession categoriesFor: #read:orderBy:!api/queries!private! !
!GlorpSession categoriesFor: #read:where:!api/queries!private! !
!GlorpSession categoriesFor: #read:where:limit:!api/queries!private! !
!GlorpSession categoriesFor: #readManyOf:!api/queries!private! !
!GlorpSession categoriesFor: #readManyOf:limit:!api/queries!private! !
!GlorpSession categoriesFor: #readManyOf:orderBy:!api/queries!private! !
!GlorpSession categoriesFor: #readManyOf:where:!api/queries!private! !
!GlorpSession categoriesFor: #readManyOf:where:limit:!api/queries!private! !
!GlorpSession categoriesFor: #readOneOf:!api/queries!private! !
!GlorpSession categoriesFor: #readOneOf:where:!api/queries!private! !
!GlorpSession categoriesFor: #realObjectFor:!private! !
!GlorpSession categoriesFor: #realObjectFor:ifNone:!private! !
!GlorpSession categoriesFor: #reaper!accessing!private! !
!GlorpSession categoriesFor: #recreateTables!private!table creation! !
!GlorpSession categoriesFor: #recreateTables:!public!table creation! !
!GlorpSession categoriesFor: #recreateTables:ifError:!public!table creation! !
!GlorpSession categoriesFor: #recreateTablesIfError:!private!table creation! !
!GlorpSession categoriesFor: #refresh:!api/queries!private! !
!GlorpSession categoriesFor: #register:!api!private! !
!GlorpSession categoriesFor: #registerAll:!api!private! !
!GlorpSession categoriesFor: #registerForDelete:!api!private! !
!GlorpSession categoriesFor: #releaseReaper!initialize!private! !
!GlorpSession categoriesFor: #requireUnitOfWork!api/transactions!private! !
!GlorpSession categoriesFor: #reset!initialize!private! !
!GlorpSession categoriesFor: #resetPermissions!initialize!private! !
!GlorpSession categoriesFor: #reusePreparedStatements!private!testing! !
!GlorpSession categoriesFor: #reusePreparedStatements:!accessing!private! !
!GlorpSession categoriesFor: #rollbackTransaction!api/transactions!private! !
!GlorpSession categoriesFor: #rollbackUnitOfWork!api/transactions!private! !
!GlorpSession categoriesFor: #rollbackUnitOfWorkFor:!api/transactions!private! !
!GlorpSession categoriesFor: #save:!api/transactions!private! !
!GlorpSession categoriesFor: #saveAndContinue!api/transactions!private! !
!GlorpSession categoriesFor: #schema!accessing!private! !
!GlorpSession categoriesFor: #sendPostFetchEventTo:!events!private! !
!GlorpSession categoriesFor: #sendPostFetchValidateEventTo:!events!private! !
!GlorpSession categoriesFor: #sendPostWriteEventTo:!events!private! !
!GlorpSession categoriesFor: #sendPreWriteEventTo:!events!private! !
!GlorpSession categoriesFor: #shouldInsert:!internal/writing!private! !
!GlorpSession categoriesFor: #shouldUseModificationTracker:!initialize!private! !
!GlorpSession categoriesFor: #system!accessing!private! !
!GlorpSession categoriesFor: #system:!api!private! !
!GlorpSession categoriesFor: #tablesInCommitOrder!internal/writing!private! !
!GlorpSession categoriesFor: #transact:!api/transactions!private! !
!GlorpSession categoriesFor: #undelete:!api/queries!private! !
!GlorpSession categoriesFor: #undelete:ifNotDeleted:!api/queries!private! !
!GlorpSession categoriesFor: #updateTableFrom:to:!public!table creation! !
!GlorpSession categoriesFor: #useBinding!private!testing! !
!GlorpSession categoriesFor: #useModificationTracker!initialize!private! !
!GlorpSession categoriesFor: #virtualCollectionOf:!private!virtual collections! !
!GlorpSession categoriesFor: #writeHomogeneousRows:!private!read/write! !
!GlorpSession categoriesFor: #writeHomogeneousRows:from:to:!private!read/write! !
!GlorpSession categoriesFor: #writeRow:!private!read/write! !

!GlorpSession class methodsFor!

forSystem: aSystem 
	^self new system: aSystem!

new

	^super new initialize.! !
!GlorpSession class categoriesFor: #forSystem:!instance creation!private! !
!GlorpSession class categoriesFor: #new!instance creation!private! !

PersistentObject guid: (GUID fromString: '{349BFEAB-4329-48A0-BC9F-F3F325346DDC}')!
PersistentObject comment: 'This is an abstract superclass for Glorp persistent objects. Things don''t have to subclass from this in order to be persistent, but if they do, they will inherit some methods which may be useful.'!
!PersistentObject categoriesForClass!Unclassified! !
!PersistentObject methodsFor!

= anObject
	"Implement equality to compare primary keys if they are set. Make sure that if we have proxies, we check through them."

	| descriptor myKey |
	self class = anObject yourSelf class ifFalse: [^false].
	descriptor := self descriptor.	

	"If the primary keys aren't equal, then we know for sure we are unequal. Otherwise we have to check further."
	myKey := descriptor primaryKeyCollectionFor: self.
	myKey = (descriptor primaryKeyCollectionFor: anObject)
		ifFalse: [^false].

	"If the primary key contains nils, then fall back to the default equality comparison. If they don't, and are equal, then the objects are equal."
	^(myKey includes: nil) 
		ifTrue: [super = anObject yourSelf]
		ifFalse: [true].!

bePersistent
	"Queue our object to be saved to the database upon commit."

	self session
		requireUnitOfWork;
		register: self!

classModel
	^self descriptor classModel!

commitUnitOfWork
	"Commit the entire unit of work; this may include objects other than myself."

	self session hasUnitOfWork ifFalse: [^self].
	self session commitUnitOfWork!

delete
	"Queue our object to be removed from the database on commit."

	self session delete: self!

descriptor
	^self session descriptorFor: self class!

find
	"This is a query by example facility. We take whichever fields are filled in the object and read back any objects in the database that match."
	
	| instances |
	instances := GlorpVirtualCollection on: self class in: self session.
	self descriptor mappings do: [:eachMapping | 
		instances := self findMapping: eachMapping in: instances].
	^instances.!

findMapping: aMapping in: aVirtualCollection
	
	aMapping hasAttribute not ifTrue: [^aVirtualCollection].
	aMapping attribute name = #session ifTrue: [^aVirtualCollection].
	^(aMapping attribute getValueFrom: self)
		ifNil: [aVirtualCollection]
		ifNotNil: [:value | aVirtualCollection select: [:each | self matchAttribute: (each get: aMapping attribute name) to: value]].!

hash
	"Implement hash to compare primary keys if they are set. Make sure that if we have proxies, we check through them."

	| myKey |

	"If the primary key contains nils, then fall back to the default hash. If they don't, hash the collection."
	myKey := self descriptor primaryKeyCollectionFor: self.
	^(myKey includes: nil) 
		ifTrue: [super hash]
		ifFalse: [myKey hash]!

matchAttribute: anExpression to: value
	"We match expressions differently depending on what type they are."
	
	^(value isString and: [value includes: $%]) 
		ifTrue: [anExpression like: value] 
		ifFalse: [anExpression = value].!

refresh
	"Refresh our object from the database and rollback any changes registered in the unit of work."

	self rollback.
	self session refresh: self!

rollback
	"Rollback our entries in the queue so that we are not involved in the commit."

	self session hasUnitOfWork ifFalse: [^self].
	self session rollbackUnitOfWorkFor: self!

rollbackUnitOfWork
	"Rollback the entire unit of work; this may include objects other than myself."

	self session hasUnitOfWork ifFalse: [^self].
	self session rollbackUnitOfWork!

session
	session isNil ifTrue: [session := self class getSession].
	^session.!

session: aGlorpSession
	session := aGlorpSession! !
!PersistentObject categoriesFor: #=!comparing!private! !
!PersistentObject categoriesFor: #bePersistent!actions!private! !
!PersistentObject categoriesFor: #classModel!accessing!private! !
!PersistentObject categoriesFor: #commitUnitOfWork!actions!private! !
!PersistentObject categoriesFor: #delete!actions!private! !
!PersistentObject categoriesFor: #descriptor!accessing!private! !
!PersistentObject categoriesFor: #find!finding objects!private! !
!PersistentObject categoriesFor: #findMapping:in:!private! !
!PersistentObject categoriesFor: #hash!comparing!private! !
!PersistentObject categoriesFor: #matchAttribute:to:!private! !
!PersistentObject categoriesFor: #refresh!actions!private! !
!PersistentObject categoriesFor: #rollback!actions!private! !
!PersistentObject categoriesFor: #rollbackUnitOfWork!actions!private! !
!PersistentObject categoriesFor: #session!accessing!private! !
!PersistentObject categoriesFor: #session:!accessing!private! !

!PersistentObject class methodsFor!

classModel
	^self descriptor classModel!

count
	"Count the number of objects in the database"
	"Person count"

	^ self getSession count: self!

descriptor
	^self getSession descriptorFor: self!

find: primaryKey
	"Query the database for an object whose primary key = the given value."
	"Person find: 1"

	| session |
	session := self getSession.
	^session execute: (self query: primaryKey).!

findAll
	"Query the database for all objects of this class."
	"Person findAll"
	^self getSession read: self.!

findWhere: aBlock
	"Query the database for all objects of this class satisfying aBlock. The block has to be a query block, meaning you can't execute arbitrary code, but rather it will be analyzed and used to generate SQL."
	"Person findWhere: [:each | (each name = 'John Doe') & (each address houseNum = 1000)]"

	^self getSession read: self where: aBlock.!

getSession
	"Return the session to be used for the current query. In general, we don't know how to find our session. Callers can catch the CannotFindSession exception and resume it, or subclasses can implement this method to find the session using mechanisms like process environment, a global, or whatever else they have available."
	^CannotFindSession raise.!

new
	^super new initialize.!

query: primaryKey
	"Return a query object that will query the database for an object whose primary key = the given value. If you want to just return the object, use find: instead. The advantage of this is that you can use this to get a basic query, but then modify it before executing it."
	"Person query: 1"

	| pkAttribute session |
	session := self getSession.
	pkAttribute := (session system descriptorFor: self) primaryKeyMappings first attribute name.
	^Query readOneOf: self where: [:each | (each perform: pkAttribute) = primaryKey].!

queryAll
	"Return a query object that will query the database for all objects of this class. If you want to just return the object, use find: instead. The advantage of this is that you can use this to get a basic query, but then modify it before executing it."
	"Person queryAll"
	^Query read: self.!

queryWhere: aBlock
	"Return a Query object that will query the database for all objects of this class satisfying aBlock. The block has to be a query block, meaning you can't execute arbitrary code, but rather it will be analyzed and used to generate SQL. If you want to just return the objects, use findWhere: instead. The advantage of this is that you can use this to get a basic query, but then modify it before executing it, e.g. using AND: or OR:."
	"Person queryWhere: [:each | (each name = 'John Doe') & (each address houseNum = 1000)]"

	^Query read: self where: aBlock.!

table
	^self descriptor ifNotNil: [:descriptor | descriptor table]! !
!PersistentObject class categoriesFor: #classModel!accessing!private! !
!PersistentObject class categoriesFor: #count!finding objects!private! !
!PersistentObject class categoriesFor: #descriptor!accessing!private! !
!PersistentObject class categoriesFor: #find:!finding objects!private! !
!PersistentObject class categoriesFor: #findAll!finding objects!private! !
!PersistentObject class categoriesFor: #findWhere:!finding objects!private! !
!PersistentObject class categoriesFor: #getSession!private!session! !
!PersistentObject class categoriesFor: #new!instance creation!private! !
!PersistentObject class categoriesFor: #query:!finding objects!private! !
!PersistentObject class categoriesFor: #queryAll!finding objects!private! !
!PersistentObject class categoriesFor: #queryWhere:!finding objects!private! !
!PersistentObject class categoriesFor: #table!accessing!private! !

TimedProxyReaper guid: (GUID fromString: '{78167F0D-0207-4715-AB34-DC895C4BAED6}')!
TimedProxyReaper comment: 'Overview:

A TimedProxyReaper collects TimedProxies and, using a background process, uninstantiates when they are due to expire. You can control the number of seconds between each invocation of the background process, with the message #delay:, and you can control the amount of work the background process will do each time it runs with the message #maximumWork:.

API Methods:
'!
!TimedProxyReaper categoriesForClass!Unclassified! !
!TimedProxyReaper methodsFor!

add: aTimedProxy
	[mutex wait.
	self noWaitAdd: aTimedProxy] ensure: [mutex signal].
	self restartIfNecessary.!

at: id ifAbsent: errorBlock
	| result |
	[mutex wait.
	result := weakValueDictionary at: id ifAbsent: [^errorBlock value]]
		ensure: [mutex signal].
	^result.!

delay
	"Answer the delay, in seconds, between runs of the reaper process."

	^delay!

delay: seconds
	"Set delay between runs of the reaper process in seconds."

	delay := seconds!

finish
"	Transcript cr; show: 'Done', (Time millisecondClockValue // 1000) printString."
	done := true.!

forkReaperThread

	| block |
	block := [
		[done] whileFalse: [
			(Delay forSeconds: self delay) wait.
			self reapExpiredObjects]].
	reaperThread := block forkAt: Processor userBackgroundPriority!

includes: aTimedProxy

	self at: aTimedProxy glorpId ifAbsent: [^false].
	^true!

initialize
	"Note: The sort used by keysSortedByExpiryTime _must_ be stable."
	done := false.
	self delay: 5.	"seconds"
	self maximumWork: 100.	"milliseconds"
	nextId := 1.
	weakValueDictionary := Dialect weakValueDictionaryClass new.
	keysSortedByExpiryTime := SortedCollection
		sortBlock: [:associationA :associationB | associationA value <= associationB value	"timeToDie"].

	indexOffset := 0.
	mutex := Semaphore forMutualExclusion.
	self forkReaperThread!

maximumWork
	"Answer the maximum amount of time the reaper thread will run for before yielding."

	^maximumWork!

maximumWork: milliseconds
	"Set the maximum amount of time the reaper thread will run for before yielding."

	maximumWork := milliseconds!

mournKeyOf: anEphemeron
	"Don't really need to do anything."

	anEphemeron value uninstantiate.!

nextSequenceNumber

	| id |
	id := nextId.
	nextId := nextId + 1.
	^id!

noWaitAdd: aTimedProxy

	| association id |
	aTimedProxy glorpId isNil ifTrue: [aTimedProxy glorpId: self nextSequenceNumber].
	id := aTimedProxy glorpId.	
	association := Association key: id value: aTimedProxy glorpTimeToDie.
	weakValueDictionary at: id put: aTimedProxy.
	aTimedProxy glorpAssociation: association.
	keysSortedByExpiryTime add: association.
	aTimedProxy glorpReaper: self.!

privateReapObjects
	"Private - Go through all of the objects (sorted by time) and remove those which have expired from the receiver's weakValueDictionary.
	Answer the number of objects reaped."

	| numberReaped timesUp |
	startTime := Time millisecondClockValue.
	numberReaped := 0.
	timesUp := false.
	[(Delay forMilliseconds: 100) wait.
	timesUp := true] forkAt: Processor activePriority + 1.
	keysSortedByExpiryTime
		do:
			[:association | 
			done | timesUp ifTrue: [^numberReaped].
			(self shouldReap: association)
				ifTrue:
					[| timedProxy |
					numberReaped := numberReaped + 1.
					association key isNil
						ifFalse:
							["Been touched in the meantime. Just skip over it, but don't reap"
							timedProxy := weakValueDictionary removeKey: association key ifAbsent: [nil].
							timedProxy class == TimedProxy ifTrue: [timedProxy uninstantiate]]]
				ifFalse: [^numberReaped]].
	^numberReaped!

reapExpiredObjects
	| numberReaped |
	[mutex wait.
	keysSortedByExpiryTime isEmpty ifTrue: [self finish].
	done ifTrue: [^self].
	numberReaped := self privateReapObjects.	"If the cache is empty, shut down"
	numberReaped timesRepeat: [keysSortedByExpiryTime removeFirst].
	indexOffset := indexOffset + numberReaped] ensure: [mutex signal].!

release

	done := true.
	maximumWork := 0.
	reaperThread notNil ifTrue: [reaperThread terminate].
	reaperThread := nil.!

remove: aTimedProxy ifAbsent: errorBlock

	^self removeKey: aTimedProxy glorpId ifAbsent: errorBlock.!

removeKey: key ifAbsent: errorBlock
	"Note: The key is NOT removed from keysSortedByExpiryTime. The reaper
	will eventually reap the given key, and ignore it since it is already gone.
	Removing objects from the middle of keysSortedByExpiryTime will screw
	up the indexing of objects that are registered later. Don't do it."

	| result |
	[mutex wait.
	result := weakValueDictionary removeKey: key ifAbsent: [^errorBlock value]]
		ensure: [mutex signal].
	^result.!

restartIfNecessary
	[mutex wait.
	done
		ifTrue:
			["Transcript cr; show: 'Restarting', (Time millisecondClockValue // 1000) printString."
			done := false.
			self forkReaperThread]] ensure: [mutex signal].!

shouldReap: anAssociation
	"Check that either we've passed the start time, or that there's been a wraparound in the millisecond value and we're now larger than it by some ridiculous margin, which we'll call 24 hours."

	| expireTime milliseconds shouldReap |
	anAssociation key isNil ifTrue: [^true].  "It's been touched. Skip over but don't reap, handled elsewhere"
	expireTime := anAssociation value.
	milliseconds := "24 * 60 * 60 * 1000" 86400000.
	shouldReap := expireTime <= startTime or: [expireTime - milliseconds > startTime].
	^shouldReap.!

touch: aTimedProxy
	[| association |
	mutex wait.
	association := aTimedProxy glorpAssociation.
	association isNil ifTrue: [^self].
	association key: nil.	"This indicates that the key should not be reaped at this time."
	self noWaitAdd: aTimedProxy] ensure: [mutex signal].! !
!TimedProxyReaper categoriesFor: #add:!caching!private! !
!TimedProxyReaper categoriesFor: #at:ifAbsent:!caching!private! !
!TimedProxyReaper categoriesFor: #delay!accessing!private! !
!TimedProxyReaper categoriesFor: #delay:!accessing!private! !
!TimedProxyReaper categoriesFor: #finish!private!reaping! !
!TimedProxyReaper categoriesFor: #forkReaperThread!private!reaping! !
!TimedProxyReaper categoriesFor: #includes:!caching!private! !
!TimedProxyReaper categoriesFor: #initialize!initializing!public! !
!TimedProxyReaper categoriesFor: #maximumWork!accessing!private! !
!TimedProxyReaper categoriesFor: #maximumWork:!accessing!private! !
!TimedProxyReaper categoriesFor: #mournKeyOf:!private!reaping! !
!TimedProxyReaper categoriesFor: #nextSequenceNumber!accessing!private! !
!TimedProxyReaper categoriesFor: #noWaitAdd:!caching!private! !
!TimedProxyReaper categoriesFor: #privateReapObjects!private!reaping! !
!TimedProxyReaper categoriesFor: #reapExpiredObjects!private!reaping! !
!TimedProxyReaper categoriesFor: #release!initializing!private! !
!TimedProxyReaper categoriesFor: #remove:ifAbsent:!caching!private! !
!TimedProxyReaper categoriesFor: #removeKey:ifAbsent:!caching!private! !
!TimedProxyReaper categoriesFor: #restartIfNecessary!private!reaping! !
!TimedProxyReaper categoriesFor: #shouldReap:!private!reaping! !
!TimedProxyReaper categoriesFor: #touch:!caching!private! !

!TimedProxyReaper class methodsFor!

delay: seconds
	^self new delay: seconds!

delay: seconds maximumWork: milliseconds
	^self new delay: seconds; maximumWork: milliseconds!

new
	^super new initialize! !
!TimedProxyReaper class categoriesFor: #delay:!As yet unclassified!private! !
!TimedProxyReaper class categoriesFor: #delay:maximumWork:!As yet unclassified!private! !
!TimedProxyReaper class categoriesFor: #new!As yet unclassified!private! !

TimedExpiryCachePolicy guid: (GUID fromString: '{A2693947-3C5A-4C66-B860-D0970B15D2FD}')!
TimedExpiryCachePolicy comment: 'This implements a cache that notes that an object is stale after some amount of time since it has been read.

Instance Variables:
	timeout	<Integer>	The time in seconds until we note an object as needing refreshing.


'!
!TimedExpiryCachePolicy categoriesForClass!Unclassified! !
!TimedExpiryCachePolicy methodsFor!

cacheEntryFor: anObject

	^Array with: self totalSeconds with: anObject.!

contentsOf: aCacheEntry

	^aCacheEntry at: 2.!

expire: aCacheEntry

	^aCacheEntry at: 1 put: 0.!

hasExpired: aCacheEntry

	^(self totalSeconds - (aCacheEntry at: 1)) >= timeout.!

initialize

	super initialize.
	timeout := 300.!

markEntryAsCurrent: aCacheEntry in: aCache

	aCacheEntry at: 1 put: self totalSeconds.!

timeout
	^timeout!

timeout: seconds
	timeout := seconds!

timeoutSeconds
	^timeout!

timeoutSeconds: anInteger
	timeout := anInteger.!

totalSeconds

	^Dialect totalSeconds.! !
!TimedExpiryCachePolicy categoriesFor: #cacheEntryFor:!private!wrap/unwrap! !
!TimedExpiryCachePolicy categoriesFor: #contentsOf:!private!wrap/unwrap! !
!TimedExpiryCachePolicy categoriesFor: #expire:!private!wrap/unwrap! !
!TimedExpiryCachePolicy categoriesFor: #hasExpired:!private!wrap/unwrap! !
!TimedExpiryCachePolicy categoriesFor: #initialize!initialize!private! !
!TimedExpiryCachePolicy categoriesFor: #markEntryAsCurrent:in:!private!wrap/unwrap! !
!TimedExpiryCachePolicy categoriesFor: #timeout!accessing!private! !
!TimedExpiryCachePolicy categoriesFor: #timeout:!accessing!private! !
!TimedExpiryCachePolicy categoriesFor: #timeoutSeconds!accessing!private! !
!TimedExpiryCachePolicy categoriesFor: #timeoutSeconds:!accessing!private! !
!TimedExpiryCachePolicy categoriesFor: #totalSeconds!private!utility! !

TimedStrongCachePolicy guid: (GUID fromString: '{CF719B50-A873-49B8-A9D6-BD76052F86B0}')!
TimedStrongCachePolicy comment: 'This implements a cache that notes that an object is stale after some amount of time since it has been read.

Instance Variables:
	timeout	<Integer>	The time in seconds until we note an object as needing refreshing.


'!
!TimedStrongCachePolicy categoriesForClass!Unclassified! !
!TimedStrongCachePolicy methodsFor!

cacheEntryFor: anObject

	^Array with: self totalSeconds with: anObject.!

contentsOf: aCacheEntry

	^aCacheEntry at: 2.!

expire: aCacheEntry

	^aCacheEntry at: 1 put: 0.!

hasExpired: aCacheEntry

	^(self totalSeconds - (aCacheEntry at: 1)) >= timeout.!

initialize

	super initialize.
	timeout := 300.!

markEntryAsCurrent: aCacheEntry in: aCache

	aCacheEntry at: 1 put: self totalSeconds.!

timeout
	^timeout!

timeout: seconds
	timeout := seconds!

timeoutSeconds
	^timeout!

timeoutSeconds: anInteger
	timeout := anInteger.!

totalSeconds

	^Dialect totalSeconds.! !
!TimedStrongCachePolicy categoriesFor: #cacheEntryFor:!private!wrap/unwrap! !
!TimedStrongCachePolicy categoriesFor: #contentsOf:!private!wrap/unwrap! !
!TimedStrongCachePolicy categoriesFor: #expire:!private!wrap/unwrap! !
!TimedStrongCachePolicy categoriesFor: #hasExpired:!private!wrap/unwrap! !
!TimedStrongCachePolicy categoriesFor: #initialize!initialize!private! !
!TimedStrongCachePolicy categoriesFor: #markEntryAsCurrent:in:!private!wrap/unwrap! !
!TimedStrongCachePolicy categoriesFor: #timeout!accessing!private! !
!TimedStrongCachePolicy categoriesFor: #timeout:!accessing!private! !
!TimedStrongCachePolicy categoriesFor: #timeoutSeconds!accessing!private! !
!TimedStrongCachePolicy categoriesFor: #timeoutSeconds:!accessing!private! !
!TimedStrongCachePolicy categoriesFor: #totalSeconds!private!utility! !

WeakVWCachePolicy guid: (GUID fromString: '{B8BECFC1-8268-4641-AEE0-7926DC96506E}')!
WeakVWCachePolicy comment: 'This is a cache policy that uses VisualWorks 7.x weak references (ephemerons) to store references to objects, letting them vanish if not referenced. It uses the numberOfElements inst var as an indicator of how many objects to keep hard references to, preventing objects from disappearing too quickly.

Instance Variables:'!
!WeakVWCachePolicy categoriesForClass!Unclassified! !
!WeakVWCachePolicy methodsFor!

collectionForExtraReferences

	^FixedSizeQueue maximumSize: self numberOfReferencesToKeepAround.!

dictionaryClass
	^Dialect smalltalkAt: #EphemeralValueDictionary ifAbsent: [Dictionary].!

markEntryAsCurrent: item in: aCache
	aCache markEntryAsCurrent: item.!

newItemsIn: aCache
	| items |
	items := super newItemsIn: aCache.
	items manager: aCache.
	^items.!

numberOfReferencesToKeepAround
	^numberOfElements.! !
!WeakVWCachePolicy categoriesFor: #collectionForExtraReferences!expiry!private! !
!WeakVWCachePolicy categoriesFor: #dictionaryClass!accessing!private! !
!WeakVWCachePolicy categoriesFor: #markEntryAsCurrent:in:!expiry!private! !
!WeakVWCachePolicy categoriesFor: #newItemsIn:!initialize-release!private! !
!WeakVWCachePolicy categoriesFor: #numberOfReferencesToKeepAround!accessing!private! !

CompoundSelectCommand guid: (GUID fromString: '{0DAD4F8D-83F3-486F-808B-5983D9B465F8}')!
CompoundSelectCommand comment: 'This is a select command that uses a number of sub-commands to print multiple queries, and combine them into one for execution. Used in conjunction with compound queries, e.g. using UNION ALL. The query knows what the combination operation is.

Instance Variables:
	commands	<(Collection of: DatabaseCommand)>	The commands we are to print.'!
!CompoundSelectCommand categoriesForClass!Unclassified! !
!CompoundSelectCommand methodsFor!

arrayBoundFields
	" *** This method was defined by Glorp.DatabaseCommand as a subclass responsibility.
	Replace its body with a proper implementation. *** "

	self error: 'Subclass responsibility stub not reimplemented'!

bindings
	^commands inject: #() into: [:sum :each | sum, each bindings].!

bindingTypes
	^self commands inject: #() into: [:sum :each | sum, each bindingTypes].!

commands
	commands isNil ifTrue: [commands := query queries collect: [:each | each sqlWith: parameters]].
	^commands.!

parameters: aDictionary
	parameters := aDictionary.
	commands := nil.!

printSQL
	self nextPutAll: 'SELECT '.
	self requiresDistinct ifTrue: [self nextPutAll: 'DISTINCT '].
	self nextPutAll: '* FROM '.
	self platform supportsAliasInCompoundQuery ifTrue: [self nextPut: $(].
	GlorpHelper 
		do: [:each | 
			self platform parenthesizeCommandsInCompoundQuery ifTrue: [self nextPut: $(].
			self nextPutAll: each sqlString.
			self platform parenthesizeCommandsInCompoundQuery ifTrue: [self nextPut: $)]]
		for: self commands
		separatedBy: [query printOperationOn: self].
	self platform supportsAliasInCompoundQuery ifTrue: [self nextPut: $)].
	query printCompoundSelectAliasesOn: self.
	query printOrderingOn: self.
	query printGroupByOn: self.!

requiresDistinct
	^self query requiresDistinct.! !
!CompoundSelectCommand categoriesFor: #arrayBoundFields!accessing!private! !
!CompoundSelectCommand categoriesFor: #bindings!accessing!private! !
!CompoundSelectCommand categoriesFor: #bindingTypes!accessing!private! !
!CompoundSelectCommand categoriesFor: #commands!accessing!private! !
!CompoundSelectCommand categoriesFor: #parameters:!accessing!private! !
!CompoundSelectCommand categoriesFor: #printSQL!accessing!public! !
!CompoundSelectCommand categoriesFor: #requiresDistinct!accessing!private! !

DuplicatePrimaryKeyException guid: (GUID fromString: '{DF94D55A-1C81-4BFF-B9C1-F444F9514902}')!
DuplicatePrimaryKeyException comment: 'This is an exception we raise when we detect two different objects with the same primary key in our cache.
'!
!DuplicatePrimaryKeyException categoriesForClass!Unclassified! !
!DuplicatePrimaryKeyException methodsFor!

defaultMessageText
	^'Duplicate objects with same primary key'.!

existingObject
	^existingObject!

existingObject: anObject
	existingObject := anObject!

newObject
	^newObject!

newObject: anObject
	newObject := anObject! !
!DuplicatePrimaryKeyException categoriesFor: #defaultMessageText!accessing!private! !
!DuplicatePrimaryKeyException categoriesFor: #existingObject!accessing!private! !
!DuplicatePrimaryKeyException categoriesFor: #existingObject:!accessing!private! !
!DuplicatePrimaryKeyException categoriesFor: #newObject!accessing!private! !
!DuplicatePrimaryKeyException categoriesFor: #newObject:!accessing!private! !

!DuplicatePrimaryKeyException class methodsFor!

new: newObject existing: existingObject
	^self new
		newObject: newObject;
		existingObject: existingObject;
		yourself.! !
!DuplicatePrimaryKeyException class categoriesFor: #new:existing:!instance creation!private! !

"Binary Globals"!

