| package |
package := Package name: 'GlorpDolphinPort'.
package paxVersion: 1;
	basicComment: 'This is part of the GLORP system, an open-source library (LGPL(S)) for persisting Smalltalk objects in relational databases. For more information see the accompanying files and/or the web sites http://www.glorp.org or http://www.sourceforge.net/projects/glorp
-------------------------------------------------
LGPL(S)

The code is Copyright (C) 2000-2003  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you''ll need to release the
source of any modifications.

If you require any further clarification, we''d be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA'.

package basicPackageVersion: 'image'.


package classNames
	add: #EqualOnContentReadStream;
	yourself.

package methodNames
	add: #BlockClosure -> #asGlorpExpression;
	add: #BlockClosure -> #asGlorpExpressionForDescriptor:;
	add: #BlockClosure -> #asGlorpExpressionOn:;
	add: #BlockClosure -> #asGlorpExpressionOn:in:;
	add: #BlockClosure -> #asGlorpExpressionOn:withUltimateBase:;
	add: #BlockClosure -> #glorpIsBlock;
	add: #ByteArray -> #asHexString;
	add: #Date -> #asTimestamp;
	add: #Date -> #months;
	add: #DBAbstractRow -> #asArray;
	add: #DBAbstractRow -> #do:;
	add: #DBAbstractRow -> #first;
	add: #DBAbstractRow -> #last;
	add: #DBAbstractRow -> #size;
	add: #DBConnection -> #isInTransaction;
	add: #Number -> #asNumber;
	add: #Number -> #glorpPrintSQLOn:;
	add: #Object -> #glorpEndOfIndexedInstVars;
	add: #Object -> #glorpIsBlock;
	add: #Object -> #glorpStartOfIndexedInstVars;
	add: #ProtoObject -> #_isProxy;
	add: #ProtoObject -> #basicAt:;
	add: #ProtoObject -> #basicAt:put:;
	add: #ProtoObject -> #class;
	add: #ProtoObject -> #errorAt:;
	add: #ProtoObject -> #instVarAt:put:;
	add: #ProtoObject -> #isImmediate;
	add: #ProtoObject -> #isImmutable;
	add: #ProtoObject -> #performMethod:;
	add: #ProtoObject -> #performMethod:arguments:;
	add: #ProtoObject -> #performMethod:with:;
	add: #ProtoObject -> #whileMutableDo:;
	add: #RBSmallDictionary -> #keys;
	add: #RBSmallDictionary -> #values;
	add: #SequenceableCollection -> #atIndex:;
	add: #SequenceableCollection -> #isSameSequenceAs:;
	add: #Time -> #millisecond:;
	add: #TimeStamp -> #addMilliseconds:;
	add: #TimeStamp -> #addSeconds:;
	add: #TimeStamp -> #asDate;
	add: #TimeStamp -> #asTime;
	add: #TimeStamp -> #dayOfMonth;
	add: #TimeStamp -> #days;
	add: #TimeStamp -> #hour;
	add: #TimeStamp -> #hours;
	add: #TimeStamp -> #millisecond:;
	add: #TimeStamp -> #milliseconds;
	add: #TimeStamp -> #minute;
	add: #TimeStamp -> #minutes;
	add: #TimeStamp -> #monthIndex;
	add: #TimeStamp -> #months;
	add: #TimeStamp -> #second;
	add: #TimeStamp -> #seconds;
	add: #TimeStamp -> #subtractSeconds:;
	add: #TimeStamp -> #year;
	add: #TimeStamp -> #years;
	add: 'Object class' -> #glorpAllSubclasses;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Database\Database Connection Base';
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Refactoring Browser\Parser\RBParser';
	yourself).

package!

"Class Definitions"!

ReadStream subclass: #EqualOnContentReadStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!BlockClosure methodsFor!

asGlorpExpression

	^self asGlorpExpressionOn: (Smalltalk at: #BaseExpression) new.!

asGlorpExpressionForDescriptor: aDescriptor

	| base |
	base := (Smalltalk at: #BaseExpression) new.
	base descriptor: aDescriptor.
	^self asGlorpExpressionOn: base.!

asGlorpExpressionOn: anExpression
	^(self value: (Smalltalk at: #MessageArchiver) new)
		asGlorpExpressionOn: anExpression!

asGlorpExpressionOn: anExpression in: aQuery
	^(self
		value: ((Smalltalk at: #MessageArchiver) new privateGlorpQuery: aQuery))
			asGlorpExpressionOn: anExpression!

asGlorpExpressionOn: anExpression withUltimateBase: aBaseExpression
	"Build the expression, knowing that some parts of it may refer to an outer context based on aBaseExpression. This is primarily useful for subselect blocks"
	| archiver tree |
	archiver := (Smalltalk at: #MessageArchiver) new.
	tree := self value: archiver.
	^tree
		asGlorpExpressionOn: anExpression
		basedOn: archiver
		withUltimateBase: aBaseExpression!

glorpIsBlock
	^true.! !
!BlockClosure categoriesFor: #asGlorpExpression!glorp!private! !
!BlockClosure categoriesFor: #asGlorpExpressionForDescriptor:!glorp!private! !
!BlockClosure categoriesFor: #asGlorpExpressionOn:!glorp!private! !
!BlockClosure categoriesFor: #asGlorpExpressionOn:in:!glorp!private! !
!BlockClosure categoriesFor: #asGlorpExpressionOn:withUltimateBase:!As yet unclassified!private! !
!BlockClosure categoriesFor: #glorpIsBlock!glorp!private! !

!ByteArray methodsFor!

asHexString
	| stream |
	stream := WriteStream on: String new.
	self printHexOn: stream.
	^stream contents! !
!ByteArray categoriesFor: #asHexString!converting!public! !

!Date methodsFor!

asTimestamp
	^TimeStamp date: self!

months
	^self monthIndex.! !
!Date categoriesFor: #asTimestamp!As yet unclassified!private! !
!Date categoriesFor: #months!printing!private! !

!DBAbstractRow methodsFor!

asArray
	^Array withAll: contents!

do: aBlock
	contents do: aBlock!

first
	^self atIndex: 1!

last
	^contents last!

size
	^contents size! !
!DBAbstractRow categoriesFor: #asArray!As yet unclassified!private! !
!DBAbstractRow categoriesFor: #do:!As yet unclassified!private! !
!DBAbstractRow categoriesFor: #first!As yet unclassified!private! !
!DBAbstractRow categoriesFor: #last!As yet unclassified!private! !
!DBAbstractRow categoriesFor: #size!As yet unclassified!private! !

!DBConnection methodsFor!

isInTransaction
	^transaction notNil! !
!DBConnection categoriesFor: #isInTransaction!As yet unclassified!private! !

!Number methodsFor!

asNumber
	^self!

glorpPrintSQLOn: aCommand

	"VisualWorks has this unpleasant habit of appending characters to anything float-like that's not actually an instance of Float, which happens way down in the guts of the printing, so it's hard to avoid. This seems to be the only reasonable way to work around it without resorting to inefficient and non-portable print policies"
	| basic real |
	basic := self printString.
	real := basic last isDigit ifTrue: [basic] ifFalse: [basic copyFrom: 1 to: basic size - 1].
	aCommand nextPutAll: real.! !
!Number categoriesFor: #asNumber!As yet unclassified!private! !
!Number categoriesFor: #glorpPrintSQLOn:!printing!private! !

!Object methodsFor!

glorpEndOfIndexedInstVars
	"Return the index at which indexed instance variables start. The same as basicSize for most dialects, but funny for ObjectStudio"

	^self basicSize.!

glorpIsBlock
	^false.!

glorpStartOfIndexedInstVars
	"Return the index at which indexed instance variables start. 1 for most dialects, but funny for ObjectStudio"
	^1.! !
!Object categoriesFor: #glorpEndOfIndexedInstVars!glorp!private! !
!Object categoriesFor: #glorpIsBlock!glorp!private! !
!Object categoriesFor: #glorpStartOfIndexedInstVars!glorp!private! !

!Object class methodsFor!

glorpAllSubclasses
	"We need an isolation layer here because ObjectStudio's allSubclasses returns symbols"
	^self allSubclasses.! !
!Object class categoriesFor: #glorpAllSubclasses!class initialization!private! !

!ProtoObject methodsFor!

_isProxy
	^true.!

basicAt: aSmallInteger
	<primitive: 49>
	^self errorAt: aSmallInteger!

basicAt: aSmallInteger put: anObject
	<primitive: 50>
	^self errorAt: aSmallInteger!

class
	"Answer a <classDescription> which is the class of the receiver.
	The primitive should not fail."

	<primitive: 111>
	^self primitiveFailed!

errorAt: aSmallInteger
	aSmallInteger error: 'Could not read value at index'!

instVarAt: anInteger put: anObject
	"Store a value into a fixed variable in the receiver.  The numbering of
	the variables corresponds to the named instance variables.  Fail if
	the index is not an Integer or is not the index of a fixed variable.
	Answer with the value stored as the result.  (Using this message
	violates the principle that each object has sovereign control over the
	storing of values into its instance variables.)."

	<primitive: 74>
	"Access beyond fixed fields"
	^self basicAt: anInteger - self class instSize put: anObject!

isImmediate
	^false.!

isImmutable
	<primitive: 177>
	^self primitiveFailed!

performMethod: method
	"Evaluate the first argument, a CompiledMethod, with the receiver as
	receiver."

	^self performMethod: method arguments: #()!

performMethod: method arguments: args
	"Evaluate the first argument, a CompiledMethod, with the receiver as
	receiver.  The other argument is the list of arguments of the method.
	The number of arguments expected by the method must match the size of the
	Array."

	^method value: self withArguments: args!

performMethod: method with: arg1
	"Evaluate the first argument, a CompiledMethod, with the receiver as
	receiver.  The other argument is the argument of the method. The method
	must be expecting one argument."

	^self performMethod: method arguments: (Array with: arg1)!

whileMutableDo: aBlock
	aBlock value.! !
!ProtoObject categoriesFor: #_isProxy!doesNotUnderstand!private! !
!ProtoObject categoriesFor: #basicAt:!accessing!private! !
!ProtoObject categoriesFor: #basicAt:put:!accessing!private! !
!ProtoObject categoriesFor: #class!accessing!public! !
!ProtoObject categoriesFor: #errorAt:!exceptions!public! !
!ProtoObject categoriesFor: #instVarAt:put:!public! !
!ProtoObject categoriesFor: #isImmediate!public!testing! !
!ProtoObject categoriesFor: #isImmutable!public!testing! !
!ProtoObject categoriesFor: #performMethod:!accessing!private! !
!ProtoObject categoriesFor: #performMethod:arguments:!accessing!private! !
!ProtoObject categoriesFor: #performMethod:with:!accessing!private! !
!ProtoObject categoriesFor: #whileMutableDo:!accessing!private! !

!RBSmallDictionary methodsFor!

keys
	^keys copyFrom: 1 to: size!

values
	^values copyFrom: 1 to: size! !
!RBSmallDictionary categoriesFor: #keys!As yet unclassified!private! !
!RBSmallDictionary categoriesFor: #values!As yet unclassified!private! !

!SequenceableCollection methodsFor!

atIndex: anIndex
	"This method offers a way of providing polymorphism between me and classes that want to imitate me but cannot override #at: safely.  It is useful in cross-dialect applications such as Glorp.  Use at:, not this method, unless this polymorphism is wanted."

	^self at: anIndex!

isSameSequenceAs: anotherObject
	self size = anotherObject size ifFalse: [^false].
	1 to: self size do: [:i | (self at: i) = (anotherObject at: i) ifFalse: [^false]].
	^true! !
!SequenceableCollection categoriesFor: #atIndex:!accessing!private! !
!SequenceableCollection categoriesFor: #isSameSequenceAs:!As yet unclassified!private! !

!Time methodsFor!

millisecond: anInteger
	self setMilliseconds: (milliseconds truncateTo: 1000) + anInteger! !
!Time categoriesFor: #millisecond:!As yet unclassified!private! !

!TimeStamp methodsFor!

addMilliseconds: anInteger
	^(self addSeconds: anInteger // 1000)
		millisecond: anInteger \\ 1000;
		yourself!

addSeconds: anInteger
	^self class fromSeconds: self asSeconds + anInteger!

asDate
	^date!

asTime
	^time!

dayOfMonth
	^date dayOfMonth!

days
	^date dayOfMonth!

hour
	^time hours!

hours
	^time hours!

millisecond: anInteger
	time millisecond: anInteger!

milliseconds
	^time milliseconds!

minute
	^time minutes!

minutes
	^time minutes!

monthIndex
	^date monthIndex!

months
	^date months!

second
	^time seconds!

seconds
	^time seconds!

subtractSeconds: anInteger
	^self class fromSeconds: self asSeconds - anInteger!

year
	^date year!

years
	^date year! !
!TimeStamp categoriesFor: #addMilliseconds:!As yet unclassified!private! !
!TimeStamp categoriesFor: #addSeconds:!As yet unclassified!private! !
!TimeStamp categoriesFor: #asDate!As yet unclassified!private! !
!TimeStamp categoriesFor: #asTime!As yet unclassified!private! !
!TimeStamp categoriesFor: #dayOfMonth!As yet unclassified!private! !
!TimeStamp categoriesFor: #days!accessing!private! !
!TimeStamp categoriesFor: #hour!As yet unclassified!private! !
!TimeStamp categoriesFor: #hours!accessing!private! !
!TimeStamp categoriesFor: #millisecond:!As yet unclassified!private! !
!TimeStamp categoriesFor: #milliseconds!accessing!private! !
!TimeStamp categoriesFor: #minute!As yet unclassified!private! !
!TimeStamp categoriesFor: #minutes!accessing!private! !
!TimeStamp categoriesFor: #monthIndex!accessing!private! !
!TimeStamp categoriesFor: #months!accessing!private! !
!TimeStamp categoriesFor: #second!As yet unclassified!private! !
!TimeStamp categoriesFor: #seconds!accessing!private! !
!TimeStamp categoriesFor: #subtractSeconds:!As yet unclassified!private! !
!TimeStamp categoriesFor: #year!As yet unclassified!private! !
!TimeStamp categoriesFor: #years!accessing!private! !

"End of package definition"!

"Source Globals"!

"Classes"!

EqualOnContentReadStream guid: (GUID fromString: '{5C791EBE-6238-4D95-9D0A-BD56C9747284}')!
EqualOnContentReadStream comment: 'Two EqualAdapterReadStreams are equal if their collections are equal.  This is the sole point of the class:  to avoid obstructing equality checks when using streams to adapt objects.'!
!EqualOnContentReadStream categoriesForClass!Unclassified! !
!EqualOnContentReadStream methodsFor!

= anObject
	^self class = anObject class and:
		[self collection = anObject collection]!

collection
	"Our hierarchy has this (with heavy warning against use) in some dialects, not in others.  We don't need the warning, given the limited use of this class, and must have it, so provide it in case it's not there in a superclass."

	^collection!

hash
	^collection hash! !
!EqualOnContentReadStream categoriesFor: #=!comparing!private! !
!EqualOnContentReadStream categoriesFor: #collection!private! !
!EqualOnContentReadStream categoriesFor: #hash!comparing!private! !

"Binary Globals"!

