| package |
package := Package name: 'GlorpMisc'.
package paxVersion: 1;
	basicComment: 'This package defines miscellaneous Glorp classes that don''t fit nicely into one of the other packages. This includes the Dialect class for portability, numerous exception classes, and the catchall GlorpHelper.

This is part of the GLORP system, an open-source library (LGPL(S)) for persisting Smalltalk objects in relational databases. For more information see the #notice property, the accompanying files and/or the web sites http://www.glorp.org or http://www.sourceforge.net/projects/glorp 
'.

package basicPackageVersion: 'image'.


package classNames
	add: #CannotAutomaticallyDetermineJoin;
	add: #CannotFindSession;
	add: #Dialect;
	add: #GlorpDatabaseReadError;
	add: #GlorpDatabaseWriteError;
	add: #GlorpError;
	add: #GlorpHelper;
	add: #GlorpIllegalCommand;
	add: #GlorpInvalidExpressionError;
	add: #GlorpInvalidTypeError;
	add: #GlorpMiscPackage;
	add: #GlorpTransactionFailure;
	add: #GlorpWriteFailure;
	add: #InvalidSessionSetup;
	add: #MultipleMappingsForInstanceVariable;
	add: #NoTableFound;
	add: #ValidationError;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	yourself).

package!

"Class Definitions"!

Object subclass: #Dialect
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: 'dialectName timestampClass lookedUpNames isNamespaceAware lastSystemTime lastServerTime'!
Object subclass: #GlorpHelper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #GlorpMiscPackage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Error subclass: #GlorpError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
GlorpError subclass: #CannotAutomaticallyDetermineJoin
	instanceVariableNames: 'constraints tables'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
GlorpError subclass: #CannotFindSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
GlorpError subclass: #GlorpDatabaseReadError
	instanceVariableNames: 'command databaseError accessor'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
GlorpError subclass: #GlorpDatabaseWriteError
	instanceVariableNames: 'command databaseError accessor'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
GlorpError subclass: #GlorpIllegalCommand
	instanceVariableNames: 'command'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
GlorpError subclass: #GlorpInvalidExpressionError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
GlorpError subclass: #GlorpInvalidTypeError
	instanceVariableNames: 'object expectedType'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
GlorpError subclass: #GlorpTransactionFailure
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
GlorpError subclass: #GlorpWriteFailure
	instanceVariableNames: 'command object'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
GlorpError subclass: #ValidationError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ValidationError subclass: #InvalidSessionSetup
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ValidationError subclass: #MultipleMappingsForInstanceVariable
	instanceVariableNames: 'attributeNames readableMappings'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ValidationError subclass: #NoTableFound
	instanceVariableNames: 'expectedTables'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

"End of package definition"!

"Source Globals"!

"Classes"!

Dialect guid: (GUID fromString: '{3CF78D2B-6223-412A-BE63-FA29F55D670A}')!
Dialect comment: 'This is an encapsulation layer for dialect differences between Smalltalk. It has a variety of class messages that do different things depending which dialect we are running on.'!
!Dialect categoriesForClass!Unclassified! !
!Dialect class methodsFor!

contentsOfFileNamed: aString
	"Given a text file name, read its contents"

	| stream contents fileClass |
	fileClass := Dialect isVisualAge
		ifTrue: [self smalltalkAt: #CfsReadFileStream]
		ifFalse: [Dialect isDolphin ifTrue: [self smalltalkAt: #File] ifFalse: [self halt]].
	stream := fileClass read: aString.
	[contents := stream contents] ensure: [stream close].
	^contents!

convert: aString toByteArrayWithEncoding: encodingName
	"TODO: should use the encoding"
	^aString asByteArray!

convert: aByteArray toStringWithEncoding: encodingName
	self isVisualWorks ifTrue: [^aByteArray asStringEncoding: encodingName].
	self error: 'Not yet implemented'.!

needsCollectionMementoFor: aCollection
	^true.!

selectorsFor: aClass do: aBlock
	aClass selectors do: aBlock! !
!Dialect class categoriesFor: #contentsOfFileNamed:!files!public! !
!Dialect class categoriesFor: #convert:toByteArrayWithEncoding:!public!strings! !
!Dialect class categoriesFor: #convert:toStringWithEncoding:!private!strings! !
!Dialect class categoriesFor: #needsCollectionMementoFor:!collections!private! !
!Dialect class categoriesFor: #selectorsFor:do:!public!reflection! !

GlorpHelper guid: (GUID fromString: '{6C65FF92-6B51-4A7E-AE67-F7E130B04992}')!
GlorpHelper comment: 'This is a helper class for methods that I''d like to be able to add as extensions, but don''t want to for fear of conflicting with dialects that have them, or people who implement them for their own projects, and any other miscellaneous extensions. One extension is that this class is in the Smalltalk namespace, so is universally accessible, and it provides a way for class extensions that don''t have visibility of the Glorp namespace to access the Dialect class.'!
!GlorpHelper categoriesForClass!Unclassified! !
!GlorpHelper class methodsFor!

dialect
	"Provide a way to refer to the dialect class from an extension or class that doesn't include the Glorp namespace, while still avoiding explicit namespace imports (which confuse other dialects when the 'pool dictionaries' don't exist)."

	| glorpNamespace |
	glorpNamespace := Smalltalk at: #Glorp ifAbsent: [nil].
	^(glorpNamespace isNil ifTrue: [Smalltalk] ifFalse: [glorpNamespace]) at: #Dialect ifAbsent: [Dialect]!

do: aBlock for: aCollection separatedBy: separatorBlock

	| array |
	array := aCollection asArray.
	1 to: array size do: [:i |
		| each |
		each := array at: i.
		aBlock value: each.
		i = array size ifFalse: [
			separatorBlock value]].!

glorpConstantExpressionClass
	^self dialect glorpConstantExpressionClass!

print: printBlock on: stream for: aCollection separatedBy: separatorString

	| array |
	array := aCollection asArray.
	1 to: array size do: [:index |
		stream nextPutAll: (printBlock value: (array at: index)).
		index == array size ifFalse: [
			stream nextPutAll: separatorString]].!

separate: aCollection by: aOneArgumentBlock

	^aCollection inject: Dictionary new into: [:dict :each |
		| val |
		val := aOneArgumentBlock value: each.
		(dict at: val ifAbsentPut: [OrderedCollection new]) add: each].! !
!GlorpHelper class categoriesFor: #dialect!classes!public! !
!GlorpHelper class categoriesFor: #do:for:separatedBy:!helpers!private! !
!GlorpHelper class categoriesFor: #glorpConstantExpressionClass!classes!private! !
!GlorpHelper class categoriesFor: #print:on:for:separatedBy:!helpers!private! !
!GlorpHelper class categoriesFor: #separate:by:!helpers!private! !

GlorpMiscPackage guid: (GUID fromString: '{030A6046-DA5D-4CF1-86F8-D2298BB3BC76}')!
GlorpMiscPackage comment: 'This class represents package information for the GlorpMisc package, for systems that don''t have things like package properties to store things like copyright and license information.

LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you''ll need to release the
source of any modifications.

If you require any further clarification, we''d be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'!
!GlorpMiscPackage categoriesForClass!Unclassified! !
!GlorpMiscPackage class methodsFor!

LICENSE
"LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you'll need to release the
source of any modifications.

If you require any further clarification, we'd be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"! !
!GlorpMiscPackage class categoriesFor: #LICENSE!instance creation!public! !

GlorpError guid: (GUID fromString: '{95321D7A-6910-49C0-802D-29C564CB2BED}')!
GlorpError comment: 'This is an abstract superclass for GLORP exceptions.
'!
!GlorpError categoriesForClass!Unclassified! !
!GlorpError methodsFor!

ancestorOf: anExceptionThingy
	"VA Compatibility"
	^self handles: anExceptionThingy!

databaseError
"rh 8/1/2004 21:26 for #getMessageText"

	^self class name!

getMessageText
	^messageText ifNil: ['']!

getVisualAgeMessageText
	
	| tempTag |
	tempTag := self basicTag.
	^tempTag isNil ifTrue: [self printString] ifFalse: [tempTag errorText].!

getVisualWorksMessageText
	"In VisualWorks, we often get arrays of ExternalDatabaseExceptions, whose parameter is an array of ExternalDatabaseError objects. Or we might just get an exception, or we might get nil or something else entirely, in which case we fall back to a default error message."
	| dbError |
	self databaseError isNil ifTrue: [^messageText := self defaultMessageText].
	dbError := self databaseError.
	dbError glorpIsCollection ifTrue: [dbError := dbError first].
	messageText := [dbError dbmsErrorString] on: Error do: [:ex | 
		ex return: ([dbError messageText] on: Error do: [:anotherEx |
			anotherEx return: self defaultMessageText])].
	^messageText.!

isResumable
	^self class mayResume.!

messageText
	(messageText isNil or: [messageText isEmpty]) ifTrue: [
		messageText := self getMessageText].
	^messageText.!

session
	"We don't know what our session is, return nil by default."
	^nil.!

signal
	"VisualWorks has non-ANSI semantics for this method. Re-implement to do the standard thing"
	^Dialect isVisualWorks ifTrue: [self raise] ifFalse: [super signal].!

signal: aString
	"VisualWorks has non-ANSI semantics for this method. Re-implement to do the standard thing"

	^Dialect isVisualWorks
		ifTrue:
			[self messageText: aString.
			self raise]
		ifFalse: [super signal: aString].! !
!GlorpError categoriesFor: #ancestorOf:!private!testing! !
!GlorpError categoriesFor: #databaseError!accessing!private! !
!GlorpError categoriesFor: #getMessageText!As yet unclassified!private! !
!GlorpError categoriesFor: #getVisualAgeMessageText!accessing!private! !
!GlorpError categoriesFor: #getVisualWorksMessageText!accessing!private! !
!GlorpError categoriesFor: #isResumable!private!testing! !
!GlorpError categoriesFor: #messageText!accessing!private! !
!GlorpError categoriesFor: #session!accessing!public! !
!GlorpError categoriesFor: #signal!ANSI!private! !
!GlorpError categoriesFor: #signal:!ANSI!private! !

!GlorpError class methodsFor!

ancestorOf: anExceptionThingy
	"VA Compatibility"
	^self exceptionalEvent handles: anExceptionThingy!

description
	"For ObjectStudio"
	^super description.!

mayResume
	"This is a VisualWorks convention. Reimplement this here so that it will also be found in other dialects that don't have it in the superclass."
	^false!

signal
	^self new signal.!

signal: aString
	^self new signal: aString.! !
!GlorpError class categoriesFor: #ancestorOf:!exception compatibility!private! !
!GlorpError class categoriesFor: #description!exception compatibility!private! !
!GlorpError class categoriesFor: #mayResume!private!testing! !
!GlorpError class categoriesFor: #signal!ANSI!private! !
!GlorpError class categoriesFor: #signal:!ANSI!private! !

CannotAutomaticallyDetermineJoin guid: (GUID fromString: '{A0244CFC-D717-42A3-81C3-9DBA56BAD159}')!
CannotAutomaticallyDetermineJoin comment: 'When we create descriptors, Glorp will try to determine the join to use between two tables based on foreign key constraints, if it hasn''t been specified explicitly. If we can''t find any foreign key constraints, or if we find too many, and can''t decide automatically which one to use, we raise this exception.

This is an exception raised when creating 
Instance Variables:
	constraints	<Array of: ForeignKeyConstraint>	The list of constraints we found.
	tables	<Collection of: DatabaseTable>	The two tables we were trying to find a join between.

'!
!CannotAutomaticallyDetermineJoin categoriesForClass!Unclassified! !
!CannotAutomaticallyDetermineJoin methodsFor!

constraints
	constraints isNil ifTrue: [constraints := #()].
	^constraints!

constraints: aCollection
	constraints := aCollection!

defaultMessageText
	^'Cannot calculate the join automatically, not enough information.'.!

tables
	^tables!

tables: databaseTables
	tables := databaseTables! !
!CannotAutomaticallyDetermineJoin categoriesFor: #constraints!accessing!private! !
!CannotAutomaticallyDetermineJoin categoriesFor: #constraints:!accessing!private! !
!CannotAutomaticallyDetermineJoin categoriesFor: #defaultMessageText!accessing!private! !
!CannotAutomaticallyDetermineJoin categoriesFor: #tables!accessing!private! !
!CannotAutomaticallyDetermineJoin categoriesFor: #tables:!accessing!private! !

CannotFindSession guid: (GUID fromString: '{DA2E1BE6-830A-46D0-B945-999DF3767FC6}')!
CannotFindSession comment: 'This is an error that we raise when trying to determine what session to use for an operation, and we don''t have enough information to figure it out.
'!
!CannotFindSession categoriesForClass!Unclassified! !
!CannotFindSession methodsFor!

defaultMessageText
	^'Cannot find the right session to use - send the #session: message first.'.! !
!CannotFindSession categoriesFor: #defaultMessageText!printing!private! !

!CannotFindSession class methodsFor!

mayResume
	^true.! !
!CannotFindSession class categoriesFor: #mayResume!private!testing! !

GlorpDatabaseReadError guid: (GUID fromString: '{6C457A57-5B7A-46A4-8CAB-41CC068281A7}')!
GlorpDatabaseReadError comment: 'This is the error we raise when we encounter an exception inside Glorp''s database read code. This is uniform across dialects, so it can be reliably caught. We wrap the underlying exception.

Instance Variables:
	accessor	<DatabaseAccessor>	The accessor we were executing against.
	command	<DatabaseCommand>	The command we were executing
	databaseError	<Exception>	The exception that the system raised.

'!
!GlorpDatabaseReadError categoriesForClass!Unclassified! !
!GlorpDatabaseReadError methodsFor!

accessor
	^accessor!

accessor: aDatabaseAccessor
	accessor := aDatabaseAccessor!

command
	^command!

command: aDatabaseCommand
	command := aDatabaseCommand!

databaseError
	^databaseError!

databaseError: aDatabaseErrorObject
	databaseError := aDatabaseErrorObject!

session
	^command session.! !
!GlorpDatabaseReadError categoriesFor: #accessor!accessing!private! !
!GlorpDatabaseReadError categoriesFor: #accessor:!accessing!private! !
!GlorpDatabaseReadError categoriesFor: #command!accessing!private! !
!GlorpDatabaseReadError categoriesFor: #command:!accessing!private! !
!GlorpDatabaseReadError categoriesFor: #databaseError!accessing!private! !
!GlorpDatabaseReadError categoriesFor: #databaseError:!accessing!private! !
!GlorpDatabaseReadError categoriesFor: #session!accessing!public! !

!GlorpDatabaseReadError class methodsFor!

mayResume
	^true.! !
!GlorpDatabaseReadError class categoriesFor: #mayResume!initialize-release!private! !

GlorpDatabaseWriteError guid: (GUID fromString: '{A866B60E-762A-4A18-A470-DD36A626CF9C}')!
GlorpDatabaseWriteError comment: 'This is the error we raise when we encounter an exception inside Glorp''s database write code. This is uniform across dialects, so it can be reliably caught. We wrap the underlying exception.

Instance Variables:
	accessor	<DatabaseAccessor>	The accessor we were executing against.
	command	<DatabaseCommand>	The command we were executing
	databaseError	<Exception>	The exception that the system raised.

'!
!GlorpDatabaseWriteError categoriesForClass!Unclassified! !
!GlorpDatabaseWriteError methodsFor!

accessor
	^accessor!

accessor: anObject
	accessor := anObject!

command
	^command!

command: aDatabaseCommand
	command := aDatabaseCommand!

databaseError
	^databaseError!

databaseError: aDatabaseErrorObject
	databaseError := aDatabaseErrorObject!

initialize
	"Initialize a newly created instance. This method must answer the receiver."

	super initialize.
	" *** Edit the following to properly initialize instance variables ***"
	command := nil.
	databaseError := nil.
	" *** And replace this comment with additional initialization code *** "
	^self!

session
	^command session.! !
!GlorpDatabaseWriteError categoriesFor: #accessor!accessing!private! !
!GlorpDatabaseWriteError categoriesFor: #accessor:!accessing!private! !
!GlorpDatabaseWriteError categoriesFor: #command!accessing!private! !
!GlorpDatabaseWriteError categoriesFor: #command:!accessing!private! !
!GlorpDatabaseWriteError categoriesFor: #databaseError!accessing!private! !
!GlorpDatabaseWriteError categoriesFor: #databaseError:!accessing!private! !
!GlorpDatabaseWriteError categoriesFor: #initialize!initialize-release!private! !
!GlorpDatabaseWriteError categoriesFor: #session!accessing!public! !

GlorpIllegalCommand guid: (GUID fromString: '{FD193589-C384-480C-8A8C-3ED435901655}')!
GlorpIllegalCommand comment: 'This is the exception we raise if we try to execute a command for which permission has been denied at the Glorp level. e.g. we try to insert while denyWrites is active.

Instance Variables:
	command	<DatabaseCommand>	The command we tried to execute.

'!
!GlorpIllegalCommand categoriesForClass!Unclassified! !
!GlorpIllegalCommand methodsFor!

command
	^command!

command: aDatabaseCommand
	command := aDatabaseCommand!

defaultMessageText
	^'Illegal Command'!

session
	^command session.! !
!GlorpIllegalCommand categoriesFor: #command!accessing!private! !
!GlorpIllegalCommand categoriesFor: #command:!accessing!private! !
!GlorpIllegalCommand categoriesFor: #defaultMessageText!accessing!private! !
!GlorpIllegalCommand categoriesFor: #session!accessing!public! !

GlorpInvalidExpressionError guid: (GUID fromString: '{9AEB7908-4D53-4323-8427-A803FA56D317}')!
GlorpInvalidExpressionError comment: 'This represents an invalid expression condition in a query.'!
!GlorpInvalidExpressionError categoriesForClass!Unclassified! !
GlorpInvalidTypeError guid: (GUID fromString: '{44F8B337-C8B7-4B55-A4A2-6304ADD56731}')!
GlorpInvalidTypeError comment: 'When mapping, if we encounter an object in a relationship that doesn''t match the type we''re expecting from the ClassModel/AttributeModel then we raise this exception. 

Instance Variables:
	expectedType	<GlorpClassModel>		The classModel for the type we were expecting.
	object	<Object>	The object that contained the invalid reference.

'!
!GlorpInvalidTypeError categoriesForClass!Unclassified! !
!GlorpInvalidTypeError methodsFor!

defaultMessageText
	^'Object being written does not match the type of the attribute: ''', self object printString,  ''' ~= ''' , self expectedType printString, ''''.!

expectedType
	^expectedType!

expectedType: aGlorpClassModel
	expectedType := aGlorpClassModel!

object
	^object!

object: anObject
	object := anObject! !
!GlorpInvalidTypeError categoriesFor: #defaultMessageText!accessing!private! !
!GlorpInvalidTypeError categoriesFor: #expectedType!accessing!private! !
!GlorpInvalidTypeError categoriesFor: #expectedType:!accessing!private! !
!GlorpInvalidTypeError categoriesFor: #object!accessing!private! !
!GlorpInvalidTypeError categoriesFor: #object:!accessing!private! !

GlorpTransactionFailure guid: (GUID fromString: '{4588ECEF-3812-4C83-9915-B92AE2A3FA37}')!
GlorpTransactionFailure comment: 'This is the exception we raise if we encounter a database error trying to commit a transaction.
'!
!GlorpTransactionFailure categoriesForClass!Unclassified! !
!GlorpTransactionFailure methodsFor!

defaultMessageText
	^'Optimistic Lock Conflict'.! !
!GlorpTransactionFailure categoriesFor: #defaultMessageText!accessing!private! !

GlorpWriteFailure guid: (GUID fromString: '{2C6889D7-7306-4C82-89AD-541915FBE9FB}')!
GlorpWriteFailure comment: 'This is the error we raise if we encounter an error trying to write some rows. This typically indicates an optimistic lock failure, although it might also happen if we violated a constraint.

Instance Variables:
	command	<DatabaseCommand>	The command we were trying to execute.
	object	<Object>	The object or objects we were trying to write.

'!
!GlorpWriteFailure categoriesForClass!Unclassified! !
!GlorpWriteFailure methodsFor!

command
	^command!

command: anObject
	command := anObject!

defaultMessageText
	^'Database write failed'.!

object
	^object!

object: anObject
	object := anObject!

session
	^self command session.! !
!GlorpWriteFailure categoriesFor: #command!accessing!private! !
!GlorpWriteFailure categoriesFor: #command:!accessing!private! !
!GlorpWriteFailure categoriesFor: #defaultMessageText!accessing!private! !
!GlorpWriteFailure categoriesFor: #object!accessing!private! !
!GlorpWriteFailure categoriesFor: #object:!accessing!private! !
!GlorpWriteFailure categoriesFor: #session!accessing!public! !

!GlorpWriteFailure class methodsFor!

mayResume
	^true.! !
!GlorpWriteFailure class categoriesFor: #mayResume!private!testing! !

ValidationError guid: (GUID fromString: '{19D50D6F-20E5-4665-9845-A632230BBCB4}')!
ValidationError comment: 'ValidationError is a superclass for all errors that are raise while validating a descriptor. The exception may resume.

'!
!ValidationError categoriesForClass!Unclassified! !
!ValidationError methodsFor!

description
	^self messageText!

model
	^self tag!

model: aClass
	self parameter: aClass! !
!ValidationError categoriesFor: #description!printing!private! !
!ValidationError categoriesFor: #model!accessing!private! !
!ValidationError categoriesFor: #model:!accessing!private! !

!ValidationError class methodsFor!

mayResume

	^true! !
!ValidationError class categoriesFor: #mayResume!private!testing! !

InvalidSessionSetup guid: (GUID fromString: '{FC1CD619-D3F8-40D1-B4C5-44AA3187B8F6}')!
InvalidSessionSetup comment: 'InvalidSessionSetup will be raised the descriptor system we are validating is different from the current session descriptor.

'!
!InvalidSessionSetup categoriesForClass!Unclassified! !
MultipleMappingsForInstanceVariable guid: (GUID fromString: '{F7567FA4-DCB8-4730-B697-F6011D43102C}')!
MultipleMappingsForInstanceVariable comment: 'This error raises when validating a descriptor if there is more than one mapping to the same instance variable.

Instance Variables:
	attributeNames	<Collection of String>	the list of attributes referencing the same instance variable
	readableMappings	<Collection of Mapping>	the list of mappings referencing the same instance variable

'!
!MultipleMappingsForInstanceVariable categoriesForClass!Unclassified! !
!MultipleMappingsForInstanceVariable methodsFor!

attributeNames
	^attributeNames!

attributeNames: anObject
	attributeNames := anObject!

messageText
	^'Multiple readable mappings for one instance variable:<1s>' expandMacrosWith: self readableMappings  printString!

readableMappings
	^readableMappings!

readableMappings: anObject
	readableMappings := anObject! !
!MultipleMappingsForInstanceVariable categoriesFor: #attributeNames!accessing!private! !
!MultipleMappingsForInstanceVariable categoriesFor: #attributeNames:!accessing!private! !
!MultipleMappingsForInstanceVariable categoriesFor: #messageText!accessing!private! !
!MultipleMappingsForInstanceVariable categoriesFor: #readableMappings!accessing!private! !
!MultipleMappingsForInstanceVariable categoriesFor: #readableMappings:!accessing!private! !

NoTableFound guid: (GUID fromString: '{44C40E65-F625-41D9-BFC3-CC755F0DE2CB}')!
NoTableFound comment: 'NoTableFound is raise if there is possible mismatch between an active record and database table names

Instance Variables:
	expectedTables	<OrderedCollection>	 expected table names to map the model

'!
!NoTableFound categoriesForClass!Unclassified! !
!NoTableFound methodsFor!

expectedTables
	^expectedTables!

expectedTables: aCollection
	expectedTables := aCollection!

messageText
	self model isString ifTrue: [^'No table <1s> found' expandMacrosWith: self model].
	^'No table found for <1s>, expected to see one of  <2s>' expandMacrosWith: self model name with: self expectedTables printString!

printOn: aStream
	aStream nextPutAll: self messageText! !
!NoTableFound categoriesFor: #expectedTables!accessing!private! !
!NoTableFound categoriesFor: #expectedTables:!accessing!private! !
!NoTableFound categoriesFor: #messageText!accessing!private! !
!NoTableFound categoriesFor: #printOn:!printing!private! !

"Binary Globals"!

