| package |
package := Package name: 'GlorpQueries'.
package paxVersion: 1;
	basicComment: 'This package defines Glorp Queries and various classes that they use, including ElementBuilders, CursoredStreams, VirtualCollections, MessageArchiver (for parsing blocks) and Proxy. 

This is part of the GLORP system, an open-source library (LGPL(S)) for persisting Smalltalk objects in relational databases. For more information see the #notice property, the accompanying files and/or the web sites http://www.glorp.org or http://www.sourceforge.net/projects/glorp 
'.

package basicPackageVersion: 'image'.


package classNames
	add: #AbstractProxy;
	add: #AbstractReadQuery;
	add: #AdHocProxy;
	add: #CompositeQuery;
	add: #CompoundQuery;
	add: #DataElementBuilder;
	add: #DictionaryBuilder;
	add: #ElementBuilder;
	add: #FilteringQuery;
	add: #GlorpCursoredStream;
	add: #GlorpDictionaryDescription;
	add: #GlorpQueries;
	add: #GlorpVirtualCollection;
	add: #JoinPrinter;
	add: #MessageArchiver;
	add: #MessageArchiverForGetMapping;
	add: #ObjectBuilder;
	add: #Proxy;
	add: #Query;
	add: #ReadQuery;
	add: #SelectingElementBuilder;
	add: #SimpleQuery;
	add: #SQLite3PragmaQuery;
	add: #SQLite3Proxy;
	add: #TimedProxy;
	add: #Tracing;
	yourself.

package methodNames
	add: #Join -> #addSource:target:;
	add: #ObjectExpression -> #aggregate:as:where:;
	add: #ObjectExpression -> #existenceCheck:as:;
	add: #ProtoObject -> #asGlorpExpression;
	add: #ProtoObject -> #asGlorpExpressionOn:;
	add: #ProtoObject -> #asGlorpExpressionOn:basedOn:withUltimateBase:;
	add: #ProtoObject -> #isGlorpExpression;
	add: #ProtoObject -> #isGlorpProxy;
	add: #SubSelectExpression -> #initializeQuery;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: 'GlorpExpressions';
	add: 'GlorpMisc';
	yourself).

package!

"Class Definitions"!

Object subclass: #ElementBuilder
	instanceVariableNames: 'instance requiresPopulating key expression cursor fieldTranslations isExpired row query'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #GlorpDictionaryDescription
	instanceVariableNames: 'keyExpression valueExpression'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #GlorpQueries
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #JoinPrinter
	instanceVariableNames: 'joinsToProcess availableTables query'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #Query
	instanceVariableNames: 'session prepared expectedRows collectionType ordering maximumLobSizeToRetrieveDirectly readsOneObject groupBy'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #Tracing
	instanceVariableNames: 'base allTracings retrievalExpressions alsoFetchExpressions query'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Collection subclass: #GlorpVirtualCollection
	instanceVariableNames: 'query session realObjects parameters'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ElementBuilder subclass: #DataElementBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ElementBuilder subclass: #DictionaryBuilder
	instanceVariableNames: 'keyBuilder valueBuilder'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ElementBuilder subclass: #ObjectBuilder
	instanceVariableNames: 'queryCache'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ElementBuilder subclass: #SelectingElementBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Query subclass: #AbstractReadQuery
	instanceVariableNames: 'resultClass whereClause base limit offset proxyType shouldRefresh tracing'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Query subclass: #CompositeQuery
	instanceVariableNames: 'queries'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Query subclass: #CompoundQuery
	instanceVariableNames: 'queries operation primaryQuery specialBuilders requiresDistinct'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Query subclass: #FilteringQuery
	instanceVariableNames: 'collection filter collectBlock'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
AbstractReadQuery subclass: #ReadQuery
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
AbstractReadQuery subclass: #SimpleQuery
	instanceVariableNames: 'builders fields joins tablePrefix customCommand tablesToPrint'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SimpleQuery subclass: #SQLite3PragmaQuery
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Stream subclass: #GlorpCursoredStream
	instanceVariableNames: 'command cursor mightHaveDuplicateRows builders buildersThatReturnResults databaseAccessor peekBuffer rawPeekBuffer statement'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ProtoObject subclass: #AbstractProxy
	instanceVariableNames: 'value isInstantiated'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ProtoObject subclass: #MessageArchiver
	instanceVariableNames: 'mySelector myArguments myReceiver query'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
AbstractProxy subclass: #AdHocProxy
	instanceVariableNames: 'readBlock'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
AbstractProxy subclass: #Proxy
	instanceVariableNames: 'session query parameters action'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Proxy subclass: #SQLite3Proxy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Proxy subclass: #TimedProxy
	instanceVariableNames: 'id reaper secondsToLive timeToDie association'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
MessageArchiver subclass: #MessageArchiverForGetMapping
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!Join methodsFor!

addSource: aField target: anotherField 
	| value targetValue |
	value := (self isConstant: aField) 
				ifTrue: [ConstantExpression for: aField]
				ifFalse: [aField].
	sources add: value.
	targetValue := anotherField class == SimpleQuery
		ifTrue: [anotherField asGlorpExpressionOn: base] 
		ifFalse: [anotherField].
	(self isConstant: targetValue) ifTrue: [
		self error: 'You are attempting to set a constant value as the target of a relationship. I suspect you want to set it on the source instead. Either that or you''re trying to use a field name where a field object is required.'].
	targets add: targetValue! !
!Join categoriesFor: #addSource:target:!api!private! !

!ObjectExpression methodsFor!

aggregate: anExpression as: aggregateFunctionName where: whereExpression
	"Make a subselect of the expression that is our argument, returning an aggregated value. e.g.
session read: GlorpCustomer where: 	[:each |  
	(each aggregate: eachCust accounts as: #countStar where: [:acct | acct price > 1000]]) = 1].
	This is the most general form of aggregation convenience method, and everything else is implemented in terms of it"

	| q expression next objectExpression retrievalExpression basicExpression |
	"We may be passed an expression corresponding to either a field, or to a full object, depending on the aggregate function. In either case, build backwards from the first full object"
	objectExpression := anExpression mapping isRelationship 
		ifTrue: [anExpression] ifFalse: [anExpression base].
	q := Query readManyOf: objectExpression referenceClass.
	expression := q baseExpression.
	next := objectExpression.
	[next base isNil] whileFalse: [
		expression := next mapping addPhantomMappingExpressionOn: expression.
		next := next base].
	"We have to force construct the equality expression, because we don't want 'self' rebuilt onto the other base"
	expression := RelationExpression new outerJoin: false; relation: #=; leftChild: expression; rightChild: self.
	q whereClause: expression.
	"We need to rebuild the thing we're using to retrieve in the right context. Also, if we're called directly, the symbol may be wrapped as a constantExpression. Unwrap it"
	basicExpression := anExpression rebuildOn: q baseExpression startingFrom: objectExpression.
	retrievalExpression := (basicExpression get: aggregateFunctionName asGlorpExpression value).
	q retrieve: retrievalExpression.
	whereExpression == nil ifFalse: [
		q AND: (whereExpression asGlorpExpressionOn: basicExpression)].
	^q asGlorpExpressionOn: self ultimateBaseExpression.!

existenceCheck: anExpression as: functionName
	"Make a subselect of the expression that is our argument, returning it wrapped in either an exists or a not exists, e.g. 
	session read: GlorpCustomer where: 	[:each |  
		each existenceCheck: eachCust accounts as: #exists:].
	This is the most general form of existence checking convenience method, others are implemented in terms of it. It's very similar to aggregate:as:where:, but simpler."

	| q expression next objectExpression |
	"We may be passed an expression corresponding to either a field, or to a full object, depending on the aggregate function. In either case, build backwards from the first full object"
	objectExpression := anExpression mapping isRelationship 
		ifTrue: [anExpression] ifFalse: [anExpression base].
	q := Query readManyOf: objectExpression mapping referenceClass.
	expression := q baseExpression.
	next := objectExpression.
	[next base isNil] whileFalse: [
		expression := next mapping addPhantomMappingExpressionOn: expression.
		next := next base].
	"We have to force construct the equality expression, because we don't want 'self' rebuilt onto the other base"
	expression := RelationExpression new outerJoin: false; relation: #=; leftChild: expression; rightChild: self.
	q whereClause: expression.
	"Just read back a literal constant, we don't actually care about the data"
	q retrieve: ([:each | 'x' ] asGlorpExpression getFunction: #distinct withArguments: #()).
	^self get: functionName withArguments: (Array with: (q asGlorpExpressionOn: self ultimateBaseExpression)).! !
!ObjectExpression categoriesFor: #aggregate:as:where:!api-aggregates!private! !
!ObjectExpression categoriesFor: #existenceCheck:as:!api-aggregates!private! !

!ProtoObject methodsFor!

asGlorpExpression
	^GlorpHelper glorpConstantExpressionClass for: self.!

asGlorpExpressionOn: anExpression

	^self asGlorpExpression.!

asGlorpExpressionOn: anExpression basedOn: aMessageArchiver withUltimateBase: aBaseExpression

	^self asGlorpExpression.!

isGlorpExpression
	^false.!

isGlorpProxy
	^false.! !
!ProtoObject categoriesFor: #asGlorpExpression!doesNotUnderstand!private! !
!ProtoObject categoriesFor: #asGlorpExpressionOn:!doesNotUnderstand!private! !
!ProtoObject categoriesFor: #asGlorpExpressionOn:basedOn:withUltimateBase:!doesNotUnderstand!private! !
!ProtoObject categoriesFor: #isGlorpExpression!private!testing! !
!ProtoObject categoriesFor: #isGlorpProxy!doesNotUnderstand!private! !

!SubSelectExpression methodsFor!

initializeQuery
	query := Query
		returningManyOf: localBase descriptor describedClass
		where: rightChild.
	query tablePrefix: ('s', localBase ultimateBaseExpression availableSubSelectNumber printString, 't').
"	query baseExpression: localBase."
	query session: localBase descriptor session.
	localBase descriptor tables do: [:eachTable |
		eachTable primaryKeyFields do: [:each | query retrieve: [:base | base getField: each]]].! !
!SubSelectExpression categoriesFor: #initializeQuery!accessing!private! !

"End of package definition"!

"Source Globals"!

"Classes"!

ElementBuilder guid: (GUID fromString: '{F3C3726E-A9CD-4BDF-ABD9-08804C6F7909}')!
ElementBuilder comment: 'This is the abstract superclass of builders. These assemble information, either primitive data or objects, from a database result set.

Subclasses must implement the following messages:
	building objects
		buildObjectFrom:
		findInstanceForRow:useProxy:
	selecting fields
		fieldsFromMyPerspective

Instance Variables:
	expression	<MappingExpression>	The expression we''re mapping. e.g. if the query is reading people, this might be the expression corresponding to "each address", meaning that we build the address object related to the main Person instance by the given relationship. 
	fieldTranslations	<Array of: Integer>	 The translation of the field positions from where they are in the descriptor to where they are in the row we''re reading. This is done so we can read the rows efficiently, by index, rather than doing lots of dictionary lookups by name. If we''re doing a simple read, the translations will probably be a no-op, but if we read multiple objects, some of the fields will be in different positions than they are in our table definition.
	instance	<Object>	The thing we''re constructing.
	isExpired	<Boolean>	If our instance is in cache, we use that instead. However, if the instance has expired, then we do something different (most likely force a refresh) than if it''s still alive.
	key	<Object>	The key for this row. This is lazily computed, and "self" is used a special marker to indicate that it hasn''t been computed yet.
	query	<AbstractReadQuery>	the query that we''re building results for.
	requiresPopulating	<Boolean>	Do we need to populate the object. Will be false if the object was found in cache and hasn''t expired.
	row	<Array>	The database results. May actually be a result set row of some sort rather than an array, depending on the dialect, but should always respond to indexing protocol.

'!
!ElementBuilder categoriesForClass!Unclassified! !
!ElementBuilder methodsFor!

buildObjectFrom: anArray 
	self subclassResponsibility.!

buildsObjects
	^false.!

canCauseDuplicateRows
	^false.!

copy
	^self shallowCopy postCopy.!

cursor
	^cursor!

cursor: aGlorpCursoredStream
	cursor := aGlorpCursoredStream!

didValidationFail: validateResult
	^validateResult == false.!

expression
	^expression!

expression: anExpression
	expression := anExpression.!

fieldsForSelectStatement

	^self subclassResponsibility.!

fieldsFromMyPerspective
	self subclassResponsibility.!

fieldTranslations
	^fieldTranslations!

fieldTranslations: aDictionary 
	fieldTranslations := aDictionary.!

findInstanceForRow: aRow proxyType: proxyType
	self subclassResponsibility.!

hasFieldTranslations
	^self fieldTranslations notNil!

initialize!

instance
	^instance!

instanceValue
	^instance!

knitResultIn: aSimpleQuery
	"Connect up our built object with any other builders that use the same thing"

	^self.!

postCopy
	instance := nil.
	row := nil.
	requiresPopulating := false.
	isExpired := false.!

query
	^query!

query: aQuery 
	query := aQuery!

reconcileWithBuilders: oldCollection andCopy: newCollection!

registerObjectInUnitOfWork
	"If there is a current unit of work, then we must register in it, after population because that way the state is already in place. The nil checks are mostly for safety during unit tests, as those conditions should never occur in real use"
	query isNil ifTrue: [^self].
	query session isNil ifTrue: [^self].
	query session register: instance.!

requiresDistinct

	^expression requiresDistinct.!

requiresPopulating
	^true!

returnValueIn: aCursoredStream
	^self instance!

row
	^row!

row: anArray
	"Since nil is a possible key value, use self as a special marker to indicate we haven't found the key yet"
	row == anArray ifFalse: [
		key := self].
	row := anArray.!

sendPostFetchValidateIn: aSession OnFailureDo: aBlock
	| validateResult |
	validateResult := aSession sendPostFetchValidateEventTo: self instance.
	(self didValidationFail: validateResult) ifTrue: aBlock.!

session
	^expression descriptor session.!

setup
	"Do any initialization required once we've got our parameters"
	^self.!

subBuilders
	^#().!

system
	^self session system.!

translateFieldPosition: aDatabaseField 
	fieldTranslations isNil ifTrue: [^aDatabaseField position].
	^fieldTranslations at: aDatabaseField.!

withAllSubBuilders
	^Array with: self.! !
!ElementBuilder categoriesFor: #buildObjectFrom:!building objects!private! !
!ElementBuilder categoriesFor: #buildsObjects!private!testing! !
!ElementBuilder categoriesFor: #canCauseDuplicateRows!accessing!private! !
!ElementBuilder categoriesFor: #copy!copying!private! !
!ElementBuilder categoriesFor: #cursor!accessing!private! !
!ElementBuilder categoriesFor: #cursor:!accessing!private! !
!ElementBuilder categoriesFor: #didValidationFail:!building objects!private! !
!ElementBuilder categoriesFor: #expression!accessing!private! !
!ElementBuilder categoriesFor: #expression:!accessing!private! !
!ElementBuilder categoriesFor: #fieldsForSelectStatement!private!selecting fields! !
!ElementBuilder categoriesFor: #fieldsFromMyPerspective!private!selecting fields! !
!ElementBuilder categoriesFor: #fieldTranslations!accessing!private! !
!ElementBuilder categoriesFor: #fieldTranslations:!accessing!private! !
!ElementBuilder categoriesFor: #findInstanceForRow:proxyType:!building objects!private! !
!ElementBuilder categoriesFor: #hasFieldTranslations!executing!private! !
!ElementBuilder categoriesFor: #initialize!initializing!private! !
!ElementBuilder categoriesFor: #instance!accessing!private! !
!ElementBuilder categoriesFor: #instanceValue!accessing!private! !
!ElementBuilder categoriesFor: #knitResultIn:!building objects!private! !
!ElementBuilder categoriesFor: #postCopy!copying!private! !
!ElementBuilder categoriesFor: #query!accessing!private! !
!ElementBuilder categoriesFor: #query:!accessing!private! !
!ElementBuilder categoriesFor: #reconcileWithBuilders:andCopy:!copying!private! !
!ElementBuilder categoriesFor: #registerObjectInUnitOfWork!building objects!private! !
!ElementBuilder categoriesFor: #requiresDistinct!accessing!private! !
!ElementBuilder categoriesFor: #requiresPopulating!accessing!private! !
!ElementBuilder categoriesFor: #returnValueIn:!accessing!private! !
!ElementBuilder categoriesFor: #row!accessing!private! !
!ElementBuilder categoriesFor: #row:!accessing!private! !
!ElementBuilder categoriesFor: #sendPostFetchValidateIn:OnFailureDo:!building objects!private! !
!ElementBuilder categoriesFor: #session!accessing!private! !
!ElementBuilder categoriesFor: #setup!initializing!private! !
!ElementBuilder categoriesFor: #subBuilders!accessing!private! !
!ElementBuilder categoriesFor: #system!accessing!private! !
!ElementBuilder categoriesFor: #translateFieldPosition:!private!translating fields! !
!ElementBuilder categoriesFor: #withAllSubBuilders!accessing!private! !

!ElementBuilder class methodsFor!

basicClassFor: anExpression in: aQuery

	^(anExpression hasDescriptor and: [anExpression mapsSimpleObject not])
		ifTrue: [ObjectBuilder] 
		ifFalse: [DataElementBuilder].!

basicFor: anExpression in: aQuery

	^(self basicClassFor: anExpression in: aQuery) new
		expression: anExpression;
		query: aQuery;
		setup.!

classFor: anExpression in: aQuery
	
	((aQuery returnsDictionary and: [anExpression == aQuery baseExpression])
		or: [anExpression returnsDictionary])
			ifTrue: [^DictionaryBuilder].
	^self basicClassFor: anExpression in: aQuery.!

for: anExpression in: aQuery

	^(self classFor: anExpression in: aQuery) new
		expression: anExpression;
		query: aQuery;
		setup.!

new

	^super new initialize.! !
!ElementBuilder class categoriesFor: #basicClassFor:in:!private! !
!ElementBuilder class categoriesFor: #basicFor:in:!instance creation!private! !
!ElementBuilder class categoriesFor: #classFor:in:!private! !
!ElementBuilder class categoriesFor: #for:in:!instance creation!private! !
!ElementBuilder class categoriesFor: #new!instance creation!private! !

GlorpDictionaryDescription guid: (GUID fromString: '{79234069-7A2B-46CC-A439-581FEF29CEA8}')!
GlorpDictionaryDescription comment: 'When querying for a dictionary, it''s not enough to tell the query to return its collection in a dictionary. We need to be able to describe both the key and the value. So we would pass an instance of this as the collectionType: to a query or mapping.

Instance Variables:
	keyExpression	<GlorpExpression | Block>	The description of the key
	valueExpression	<GlorpExpression | Block>	The description of the value

'!
!GlorpDictionaryDescription categoriesForClass!Unclassified! !
!GlorpDictionaryDescription methodsFor!

copyEmpty: anInteger
	"Masquerade as a instance of a collection :-)"
	^Dictionary new: anInteger.!

keyExpression
	^keyExpression!

keyExpression: anExpression
	keyExpression := anExpression!

new
	"Masquerade as a class"
	^Dictionary new.!

new: anInteger
	"Masquerade as a class"
	^Dictionary new: anInteger.!

valueExpression
	^valueExpression!

valueExpression: anExpression
	valueExpression := anExpression! !
!GlorpDictionaryDescription categoriesFor: #copyEmpty:!instance creation!private! !
!GlorpDictionaryDescription categoriesFor: #keyExpression!accessing!private! !
!GlorpDictionaryDescription categoriesFor: #keyExpression:!accessing!private! !
!GlorpDictionaryDescription categoriesFor: #new!instance creation!private! !
!GlorpDictionaryDescription categoriesFor: #new:!instance creation!private! !
!GlorpDictionaryDescription categoriesFor: #valueExpression!accessing!private! !
!GlorpDictionaryDescription categoriesFor: #valueExpression:!accessing!private! !

GlorpQueries guid: (GUID fromString: '{006E51F4-52EF-4784-B8A2-FB748E30EFF1}')!
GlorpQueries comment: 'This class represents package information for the GlorpQueries package, for systems that don''t have things like package properties to store things like copyright and license information.

LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you''ll need to release the
source of any modifications.

If you require any further clarification, we''d be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'!
!GlorpQueries categoriesForClass!Unclassified! !
JoinPrinter guid: (GUID fromString: '{97EC3192-9750-410F-A2D6-35C2428F9A7F}')!
JoinPrinter comment: 'This is a helper class used for printing collections of joins in the <expletive deleted> ANSI join syntax. This syntax wants us to embed all of the joins into the initial list of tables, so we have to come up with an order for printing that makes sure we get all the joins in properly. This is more complicated than you might think.

Instance Variables:
	availableTables	<(Collection of: DatabaseTable)>	Which are the tables that we plan to print.
	joinsToProcess	<(Collection of: Join)>	Which joins do we have to put in between those tables somehow or other.
	query	<Query>	What is the query in which we are printing all of this.

'!
!JoinPrinter categoriesForClass!Unclassified! !
!JoinPrinter methodsFor!

allTables
	^query tablesToPrint.!

nextJoin
	^joinsToProcess detect: [:eachJoinExpression |
		eachJoinExpression tablesForANSIJoin anySatisfy: [:eachTable |
			availableTables includes: eachTable]].!

printJoinsOn: aCommand
	"This is pretty hideous. Figure out an order in which to print these stupid joins. As a bonus, they may be circular, so we may have to collapse joins together. And if we collapse, we can't print as we go because we don't know how many parentheses to add, so store them up and print them all at the end"
	| allJoins |
	joinsToProcess := query joins copy.
	availableTables := Set with: self rootTable.
	allJoins := OrderedCollection new.
	[joinsToProcess isEmpty] whileFalse: [
		| next nextTable allRelatedJoins fullJoin|
		next := self nextJoin.
		joinsToProcess remove: next.
		nextTable := next tableToJoinFrom: availableTables.
		allRelatedJoins := joinsToProcess select: [:each | (each tableToJoinFrom: availableTables) = nextTable].
		fullJoin := allRelatedJoins inject: next into: [:sum :each | sum AND: each].
		joinsToProcess removeAll: allRelatedJoins.
		allJoins add: (fullJoin->nextTable).
		availableTables addAll: (next tablesForANSIJoin)].
	allJoins size timesRepeat: [aCommand platform parenthesizeCommandsInCompoundQuery ifTrue: [aCommand nextPut: $(]].
	aCommand nextPutAll: self rootTable sqlTableName.
	allJoins do: [:each | 
		each key printForANSIJoinTo: each value on: aCommand.
		aCommand platform parenthesizeCommandsInCompoundQuery ifTrue: [aCommand nextPut: $)]].
	self printLeftoverTablesOn: aCommand.!

printLeftoverTablesOn: aCommand
	"Now there might be leftover tables whose joins were implied directly by the where clause"
	| leftOverTables |
	leftOverTables := self allTables asSet copy.
	availableTables do: [:each | leftOverTables remove: each ifAbsent: []].
	leftOverTables isEmpty ifFalse: [aCommand nextPutAll: ', '].
	GlorpHelper
		print: [:each | each sqlTableName]
		on: aCommand
		for: leftOverTables
		separatedBy: ', '.!

query: aQuery
	query := aQuery.	
	self setUp.!

rootTable
	"Pick a table to start with"
	^self allTables first.!

setUp
	availableTables := Set new.! !
!JoinPrinter categoriesFor: #allTables!accessing!private! !
!JoinPrinter categoriesFor: #nextJoin!printing!private! !
!JoinPrinter categoriesFor: #printJoinsOn:!printing!public! !
!JoinPrinter categoriesFor: #printLeftoverTablesOn:!printing!private! !
!JoinPrinter categoriesFor: #query:!accessing!private! !
!JoinPrinter categoriesFor: #rootTable!accessing!private! !
!JoinPrinter categoriesFor: #setUp!initializing!private! !

!JoinPrinter class methodsFor!

for: aQuery
	^self new query: aQuery.! !
!JoinPrinter class categoriesFor: #for:!instance creation!private! !

Query guid: (GUID fromString: '{224A687A-6752-4C03-B00F-36017E6EF311}')!
Query comment: 'This is an abstract superclass for Glorp queries.

Subclasses must implement the following messages:
	accessing
		readsOneObject
	executing
		executeWithParameters:in:

Instance Variables:
	collectionType	<Class>	What sort of collection should this query return its results in.
	expectedRows	<Integer>	Roughly how many rows should we expect to get back. Doesn''t have to be right, but can be used to set buffer sizes appropriately in the database.
	ordering	<(SequenceableCollection of: (GlorpExpression | Block | Symbol))>	If we order the results, what are the ordering criteria. Either an expression, but one whose root ends up evaluating to a field, or a block or symbol that can be converted to same.
	prepared	<Boolean>	Has this query already been prepared (at the Glorp level), this says nothing about prepared statements for the database, which will be separate objects.
	session	<GlorpSession>	What session should this query execute in. Can be nil until we start to prepare the query.
'!
!Query categoriesForClass!Unclassified! !
!Query methodsFor!

collectionType
	"Note that queries default the collection type to array, while mappings default to OrderedCollection. I think it makes sense"
	collectionType isNil ifTrue: [collectionType := Array].
	^collectionType.!

collectionType: aClass
	collectionType := aClass.!

copy
	^self shallowCopy postCopy.!

copyAttributesTo: newQuery
	"We're creating a new query similar to ourselves, ut not a direct copy. Copy over the relevant attributes. Note that we don't copy over the ordering, because that's less universally appropriate. Caller beware."

	newQuery expectedRows: self expectedRows.
	newQuery collectionType: self collectionType.
	newQuery readsOneObject: self readsOneObject.
	newQuery maximumLobSizeToRetrieveDirectly: self maximumLobSizeToRetrieveDirectly.!

executeIn: aSession

	^self executeWithParameters:#() in: aSession.!

executeWithParameters: parameterArray in: aSession 
	self subclassResponsibility.!

expectedRows
	"How many rows do we think it's likely this query will bring back. Used for tweaking things like block factor"
	^expectedRows isNil
		ifTrue: [expectedRows := self readsOneObject ifTrue: [1] ifFalse: [100]]
		ifFalse: [expectedRows].!

expectedRows: anInteger
	"How many rows do we think it's likely this query will bring back. Used for tweaking things like block factor"
	expectedRows := anInteger!

expressionBlockFor: anOrderingCriteria
	"Allow us to use symbols interchangeably with simple blocks for ordering, so 
	#firstName is equivalent to [:each | each firstName]. Also, allow chains of symbols, so #(owner firstName)"

	anOrderingCriteria isGlorpExpression ifTrue: [^anOrderingCriteria].
	anOrderingCriteria glorpIsBlock ifTrue: [^anOrderingCriteria].

	anOrderingCriteria isSymbol 
		ifTrue: [^[:each | each perform: anOrderingCriteria]].
	
	"otherwise, we assume it's a collection of symbols, the only other valid case"
	anOrderingCriteria do: [:each | each isSymbol ifFalse: [self error: 'invalid ordering criteria']].
	^[:each | anOrderingCriteria inject: each into: [:sum :eachExpression |
		sum perform: eachExpression]].!

groupBy
	^groupBy.!

groupBy: aBlock
	groupBy := aBlock.!

initialize

	prepared := false.
	maximumLobSizeToRetrieveDirectly := 32768.!

maximumLobSizeToRetrieveDirectly
	^maximumLobSizeToRetrieveDirectly!

maximumLobSizeToRetrieveDirectly: anInteger
	maximumLobSizeToRetrieveDirectly := anInteger.!

orderBy: aBlock
	ordering isNil 
		ifTrue: [ordering := Array with: aBlock]
		ifFalse: [ordering := ordering , (Array with: aBlock)].!

ordering
	^ordering.!

platform
	^session system platform.!

postCopy

	prepared := false.!

printCompoundSelectAliasesOn: aStream
	^self.!

printGroupByOn: aStream
	groupBy isNil ifTrue: [^self].
	aStream nextPutAll: ' GROUP BY '.
	groupBy printSQLOn: aStream withParameters: nil.!

printOn: aStream
	"I hate the printing with namespaces in vw"
      | title |
        title := self class name.
        aStream nextPutAll:
                ((title at: 1) isVowel ifTrue: ['an '] ifFalse: ['a ']).
        aStream nextPutAll: title!

printOrderingOn: aStream
	"Before printing, send each expression >>rewriteFunctionArguments.
	It has no effect for most expressions and most functions, but is
	needed with some platforms (see Substring)."

	ordering isNil ifTrue: [^self].
	aStream nextPutAll: ' ORDER BY '.
	GlorpHelper
		do: [:each |
			each
				rewriteFunctionArguments;
				printSQLOn: aStream withParameters: nil]
		for: ordering
		separatedBy: [aStream nextPutAll: ', ']!

readsOneObject
	readsOneObject isNil ifTrue: [^false].
	^readsOneObject.!

readsOneObject: aBoolean 
	readsOneObject := aBoolean!

resultCollectionFor: aCursoredStream
	| results finalAnswer |
	collectionType isNil ifTrue: [^aCursoredStream upToEnd].
	collectionType == aCursoredStream class ifTrue: [^aCursoredStream].
	results := (aCursoredStream newCollectionOfSize: self expectedRows for: collectionType) writeStream.
	results nextPutAll: aCursoredStream upToEnd.
	finalAnswer := results contents.
	session register: finalAnswer.
	^finalAnswer.!

retrieveDictionaryWithKey: keyExpression value: valueExpression

	self collectionType: (GlorpDictionaryDescription new keyExpression: keyExpression; valueExpression: valueExpression).!

returnsDictionary

	^collectionType class == GlorpDictionaryDescription.!

rewriteExcept
	"Rewrite any EXCEPT operation we have into a NOT EXISTS tests. Only a CompoundQuery would have such an operation"!

rewriteIntersect
	"Rewrite any INTERSECT operation we have into an EXISTS tests. Only a CompoundQuery would have such an operation"!

session
	^session!

session: aSession 
	session := aSession!

setOrdering: aCollection
	ordering := aCollection.!

shortCircuitEmptyReturn: parameters
	^false.! !
!Query categoriesFor: #collectionType!accessing!private! !
!Query categoriesFor: #collectionType:!accessing!private! !
!Query categoriesFor: #copy!copying!private! !
!Query categoriesFor: #copyAttributesTo:!copying!private! !
!Query categoriesFor: #executeIn:!executing!private! !
!Query categoriesFor: #executeWithParameters:in:!executing!private! !
!Query categoriesFor: #expectedRows!accessing!private! !
!Query categoriesFor: #expectedRows:!accessing!private! !
!Query categoriesFor: #expressionBlockFor:!ordering!private! !
!Query categoriesFor: #groupBy!ordering!private! !
!Query categoriesFor: #groupBy:!ordering!private! !
!Query categoriesFor: #initialize!initialize!private! !
!Query categoriesFor: #maximumLobSizeToRetrieveDirectly!accessing!private! !
!Query categoriesFor: #maximumLobSizeToRetrieveDirectly:!accessing!private! !
!Query categoriesFor: #orderBy:!ordering!private! !
!Query categoriesFor: #ordering!ordering!private! !
!Query categoriesFor: #platform!accessing!private! !
!Query categoriesFor: #postCopy!copying!private! !
!Query categoriesFor: #printCompoundSelectAliasesOn:!private!sql generation! !
!Query categoriesFor: #printGroupByOn:!private!sql generation! !
!Query categoriesFor: #printOn:!printing!private! !
!Query categoriesFor: #printOrderingOn:!public!sql generation! !
!Query categoriesFor: #readsOneObject!accessing!private! !
!Query categoriesFor: #readsOneObject:!accessing!private! !
!Query categoriesFor: #resultCollectionFor:!executing!private! !
!Query categoriesFor: #retrieveDictionaryWithKey:value:!private!specifying retrievals! !
!Query categoriesFor: #returnsDictionary!private!specifying retrievals! !
!Query categoriesFor: #rewriteExcept!private!sql generation! !
!Query categoriesFor: #rewriteIntersect!private!sql generation! !
!Query categoriesFor: #session!accessing!private! !
!Query categoriesFor: #session:!accessing!private! !
!Query categoriesFor: #setOrdering:!ordering!private! !
!Query categoriesFor: #shortCircuitEmptyReturn:!private!testing! !

!Query class methodsFor!

count: aClass
	^self count: aClass where: nil.!

count: aClass where: anExpression
	| query |
	query := self returningOneOf: aClass where: anExpression.
	query retrieve: #countStar.
	^query.!

new

	^super new initialize.!

read: aClass
	"readManyOf: and returningManyOf: are synonyms. This now seems more natural to me, to be consistent with session API"
	^self returningManyOf: aClass where: nil.!

read: aClass where: anExpression
	^self returningManyOf: aClass where: anExpression.!

readManyOf: aClass
	"readManyOf: and returningManyOf: are synonyms. This now seems more natural to me, to be consistent with session API"
	^self returningManyOf: aClass where: nil.!

readManyOf: aClass where: anExpression
	^self returningManyOf: aClass where: anExpression.!

readOneOf: aClass
	^self returningOneOf: aClass.!

readOneOf: aClass where: anExpression
	^self returningOneOf: aClass where: anExpression.!

returningManyOf: aClass
	^self returningManyOf: aClass where: nil.!

returningManyOf: aClass where: anExpression
	"Backward-compatibility, since we changed the class name."
	^SimpleQuery returningManyOf: aClass where: anExpression.!

returningOneOf: aClass
	"Backward-compatibility, since we changed the class name."
	^SimpleQuery returningOneOf: aClass.!

returningOneOf: aClass where: anExpression
	"Backward-compatibility, since we changed the class name."
	^SimpleQuery returningOneOf: aClass where: anExpression.! !
!Query class categoriesFor: #count:!instance creation!private! !
!Query class categoriesFor: #count:where:!instance creation!private! !
!Query class categoriesFor: #new!instance creation!private! !
!Query class categoriesFor: #read:!instance creation!private! !
!Query class categoriesFor: #read:where:!instance creation!private! !
!Query class categoriesFor: #readManyOf:!instance creation!private! !
!Query class categoriesFor: #readManyOf:where:!instance creation!private! !
!Query class categoriesFor: #readOneOf:!instance creation!private! !
!Query class categoriesFor: #readOneOf:where:!instance creation!private! !
!Query class categoriesFor: #returningManyOf:!instance creation!private! !
!Query class categoriesFor: #returningManyOf:where:!instance creation!private! !
!Query class categoriesFor: #returningOneOf:!instance creation!private! !
!Query class categoriesFor: #returningOneOf:where:!instance creation!private! !

Tracing guid: (GUID fromString: '{82EFBD88-8178-48EE-81CE-64923D559ADE}')!
Tracing comment: 'A tracing is a collection of expressions representing the graph of other objects which
are to be read at the same time as the root object.

Instance Variables:

	base	<Expression>	The base expression representing the root object. Same as the parameter to the query block
	allTracings	<Collection of: Expression>	The expressions representing each of the associated objects. e.g. base accounts, base amount serviceCharge .
	alsoFetchExpressions	<(Collection of: GlorpExpression)>	Objects to also retrieve, but not included in the result set, just knitted together with the other related objects.
	retrievalExpressions	<(Collection of: GlorpExpression)>	Objects to also retrieve, and to include in teh result set

'!
!Tracing categoriesForClass!Unclassified! !
!Tracing methodsFor!

addExpression: anExpression
	self addExpression: anExpression andDo: [:ignore |].!

addExpression: anExpression andDo: aBlock
	| exp |
	exp := anExpression asGlorpExpressionOn: self base in: query.
	(allTracings includes: exp) ifFalse: [
		allTracings add: exp.
		aBlock value: exp].!

additionalExpressions
	| all |
	alsoFetchExpressions isEmpty ifTrue: [^retrievalExpressions].
	all := OrderedCollection new.
	all addAll: self retrievalExpressions.
	all addAll: self alsoFetchExpressions.
	^all.!

allTracings

	^allTracings.!

alsoFetch: anExpression 
	"Add the expression as something which will be explicitly retrieved and knit together with other results, but NOT included in the result list"

	self addExpression: anExpression andDo: [:exp |
		alsoFetchExpressions add: exp].!

alsoFetchExpressions
	^alsoFetchExpressions!

base
	^query isNil ifTrue: [base] ifFalse: [query baseExpression].!

base: anExpression 
	base := anExpression!

copy
	^self shallowCopy postCopy.!

initialize

	base := BaseExpression new.
	allTracings := OrderedCollection new: 2.
	retrievalExpressions := Set new: 3.
	alsoFetchExpressions := Set new: 3.!

postCopy
	allTracings := allTracings copy.
	retrievalExpressions := retrievalExpressions copy.
	alsoFetchExpressions := alsoFetchExpressions copy.!

retrievalExpressions
	^retrievalExpressions.!

retrieve: anExpression 
	"Add the expression as something which will be explicitly retrieved and knit together with other results, and included in the result list"
	self addExpression: anExpression andDo: [:exp |
		retrievalExpressions add: exp].!

setup
	"We have been put into a query. If we aren't to trace anything else, trace the base"
	retrievalExpressions isEmpty ifTrue: [
		allTracings addFirst: base.
		retrievalExpressions add: base].!

tracesThrough: aMapping

	^aMapping isStoredInSameTable.!

updateBase: aBaseExpression
	"Make sure we have the same base as the query"
	| transformed |
	transformed := IdentityDictionary new.
	base == aBaseExpression ifTrue: [^self].
	base := aBaseExpression.
	allTracings := allTracings collect: [:each | | new | new := each asGlorpExpressionOn: base.
		transformed at: each put: new.
		new].
	retrievalExpressions := retrievalExpressions collect: [:each | 
		transformed at: each].
	alsoFetchExpressions := alsoFetchExpressions collect: [:each | 
		transformed at: each].! !
!Tracing categoriesFor: #addExpression:!accessing!private! !
!Tracing categoriesFor: #addExpression:andDo:!accessing!private! !
!Tracing categoriesFor: #additionalExpressions!accessing!private! !
!Tracing categoriesFor: #allTracings!accessing!private! !
!Tracing categoriesFor: #alsoFetch:!api!private! !
!Tracing categoriesFor: #alsoFetchExpressions!accessing!private! !
!Tracing categoriesFor: #base!accessing!private! !
!Tracing categoriesFor: #base:!accessing!private! !
!Tracing categoriesFor: #copy!copying!private! !
!Tracing categoriesFor: #initialize!initialize!private! !
!Tracing categoriesFor: #postCopy!copying!private! !
!Tracing categoriesFor: #retrievalExpressions!accessing!private! !
!Tracing categoriesFor: #retrieve:!api!private! !
!Tracing categoriesFor: #setup!private!setup! !
!Tracing categoriesFor: #tracesThrough:!private!querying! !
!Tracing categoriesFor: #updateBase:!private!setup! !

!Tracing class methodsFor!

for: aQuery
	^self new base: aQuery whereClause ultimateBaseExpression.!

new

	^super new initialize.! !
!Tracing class categoriesFor: #for:!instance creation!private! !
!Tracing class categoriesFor: #new!instance creation!private! !

GlorpVirtualCollection guid: (GUID fromString: '{D4B7532A-6EA5-4DC2-B7F4-2F1425A2997C}')!
GlorpVirtualCollection comment: 'This represents a virtual collection, i.e. one that we haven''t really read into memory yet. It responds to a reasonable subset of collection protocol, and will read the elements into memory only when necessary. So, e.g. a select: operation takes a query block, and is equivalent to AND:ing that query block to the main query.

To create a virtual collection, ask the session for one. e.g. session virtualCollectionOf: AClass.

This is an initial version which will read in the objects fairly eagerly. An optimization might be to defer certain types of operations depending on whether the block can be evaluated into SQL or not. e.g.
  collect: [:each | each name]
can be turned into a retrieve: operation. But 
  collect: [:each | each printString]
cannot. We could try to check the block for operations like collect: and detect:, deferring the point at which the objects will be read in.

Handling of ordering is also a little bit funny. The blocks we like for ordering aren''t compatible with sortedCollection type blocks. It''d be nice to be more compatible.

'!
!GlorpVirtualCollection categoriesForClass!Unclassified! !
!GlorpVirtualCollection methodsFor!

add: newObject
	self readOnlyError.!

AND: aBlock
	query AND: aBlock.!

collect: aBlock
	^self realObjects collect: aBlock!

copy
	^self shallowCopy postCopy.!

copyAndGrowBy: anInteger
	"For VA compatibility"
	^super glorpShallowCopy.!

copyFrom: start to: stop
	^self realObjects copyFrom: start to: stop.!

do: aBlock
	self realObjects do: aBlock.!

first
	^self realObjects first.!

getQuery
	^query.!

isEmpty
	^self realObjects isEmpty.!

isInstantiated
	^realObjects notNil.!

last
	^self realObjects last.!

on: aClass in: aSession
	query := Query returningManyOf: aClass.
	session := aSession.!

orderBy: aBlockOrExpression
	query orderBy: aBlockOrExpression.!

parameters: aCollection
	parameters := aCollection!

postCopy
	query := query copy.
	realObjects := nil.!

printOn: aStream
	self isInstantiated
		ifTrue: [super printOn: aStream]
		ifFalse:
			[aStream nextPutAll: 'a virtual collection of '.
			query notNil ifTrue: [aStream nextPutAll: query resultClass name]].!

readOnlyError
	self error: 'Virtual collections are read-only'.!

realObjects
	realObjects isNil ifTrue: [	realObjects := query executeWithParameters: parameters in: session].
	^realObjects.!

reject: aBlock
	^self copy AND: [:each | (aBlock value: each) not].!

remove: oldObject ifAbsent: anExceptionBlock
	self readOnlyError.!

select: aBlock
	^self isInstantiated ifTrue: [self realObjects select: aBlock] ifFalse: [self copy AND: aBlock].!

setQuery: aQuery
	"Semi-private. Set the internal query"
	query := aQuery.!

setSession: aSession
	"Semi-private. Set the session"
	session := aSession.!

size
	^self realObjects size.! !
!GlorpVirtualCollection categoriesFor: #add:!adding!private! !
!GlorpVirtualCollection categoriesFor: #AND:!private! !
!GlorpVirtualCollection categoriesFor: #collect:!enumerating!private! !
!GlorpVirtualCollection categoriesFor: #copy!copying!private! !
!GlorpVirtualCollection categoriesFor: #copyAndGrowBy:!copying!private! !
!GlorpVirtualCollection categoriesFor: #copyFrom:to:!copying!private! !
!GlorpVirtualCollection categoriesFor: #do:!enumerating!private! !
!GlorpVirtualCollection categoriesFor: #first!accessing!private! !
!GlorpVirtualCollection categoriesFor: #getQuery!private! !
!GlorpVirtualCollection categoriesFor: #isEmpty!private!testing! !
!GlorpVirtualCollection categoriesFor: #isInstantiated!private!testing! !
!GlorpVirtualCollection categoriesFor: #last!accessing!private! !
!GlorpVirtualCollection categoriesFor: #on:in:!initialize-release!private! !
!GlorpVirtualCollection categoriesFor: #orderBy:!accessing!private! !
!GlorpVirtualCollection categoriesFor: #parameters:!accessing!private! !
!GlorpVirtualCollection categoriesFor: #postCopy!copying!private! !
!GlorpVirtualCollection categoriesFor: #printOn:!printing!private! !
!GlorpVirtualCollection categoriesFor: #readOnlyError!private! !
!GlorpVirtualCollection categoriesFor: #realObjects!private! !
!GlorpVirtualCollection categoriesFor: #reject:!enumerating!private! !
!GlorpVirtualCollection categoriesFor: #remove:ifAbsent:!private!removing! !
!GlorpVirtualCollection categoriesFor: #select:!enumerating!private! !
!GlorpVirtualCollection categoriesFor: #setQuery:!accessing!private! !
!GlorpVirtualCollection categoriesFor: #setSession:!accessing!private! !
!GlorpVirtualCollection categoriesFor: #size!accessing!private! !

!GlorpVirtualCollection class methodsFor!

on: aClass in: aSession
	^self new on: aClass in: aSession.! !
!GlorpVirtualCollection class categoriesFor: #on:in:!instance creation!private! !

DataElementBuilder guid: (GUID fromString: '{88034480-5145-4F97-9F98-B7F25A327879}')!
DataElementBuilder comment: 'This builds raw data items rather than persistent objects with descriptors. Used if we do something like 
  aQuery retrieve: [:each | each address streetName].
giving us back simple data objects.
This makes building them quite simple.'!
!DataElementBuilder categoriesForClass!Unclassified! !
!DataElementBuilder methodsFor!

buildObjectFrom: anArray
	self row: anArray.
	instance := self valueOf: expression.!

didValidationFail: validateResult

	"In other words, don't treat the result of the validation send to the object as a failure if the object was the boolean false, so returning self would normally be interpreted as the failure"
	validateResult == self instance ifTrue: [^false].

	^super didValidationFail: validateResult.!

fieldsForSelectStatement
	^Array with: expression.!

fieldsFromMyPerspective
	^expression mappedFields.!

findInstanceForRow: aRow proxyType: proxyType

	^self.! !
!DataElementBuilder categoriesFor: #buildObjectFrom:!building objects!private! !
!DataElementBuilder categoriesFor: #didValidationFail:!building objects!private! !
!DataElementBuilder categoriesFor: #fieldsForSelectStatement!private!selecting fields! !
!DataElementBuilder categoriesFor: #fieldsFromMyPerspective!private!selecting fields! !
!DataElementBuilder categoriesFor: #findInstanceForRow:proxyType:!building objects!private! !

DictionaryBuilder guid: (GUID fromString: '{FFBD4D6C-CC91-45AE-B992-36944792724E}')!
DictionaryBuilder comment: 'This is a subclass of builders that builds dictionaries. Basically it needs to build both the key and the value and then assemble them into a dictionary.

Instance Variables:
	keyBuilder	<ElementBuilderr>	The builder for the key
	valueBuilder	<ElementBuilder>	The builder for the value.

'!
!DictionaryBuilder categoriesForClass!Unclassified! !
!DictionaryBuilder methodsFor!

buildObjectFrom: anArray
	"We just build an association out of the sub-builders "
	| |
	self row: anArray.!

canCauseDuplicateRows
	^expression class == MappingExpression and: [expression mapping isToManyRelationship].!

fieldsForSelectStatement
	^#()!

fieldsFromMyPerspective
	^#()!

findInstanceForRow: aRow proxyType: proxyType

	instance := Association new.!

instanceValue
	^self instance value!

knitResultIn: aCursoredStream
	"Connect up our built object with any other objects that reference it. Used if we retrieve more than one thing in the same query"
	| relatedBuilder |
	instance key: keyBuilder instance.
	instance value: valueBuilder instance.

	"Code from ObjectBuilder>>knitResultIn:"
	expression canKnit ifFalse: [^self].
	relatedBuilder := aCursoredStream objectBuilderFor: expression base.
	(relatedBuilder isNil or: [relatedBuilder instance isNil]) 
		ifFalse: 
			[expression mapping knitFromBuilder: relatedBuilder toBuilder: self]!

postCopy!

reconcileWithBuilders: oldCollection andCopy: newCollection
	"Fix up references to our internal sub-builders"
	| keyIndex valueIndex |
	keyIndex := oldCollection indexOf: keyBuilder.
	keyBuilder := newCollection at: keyIndex.
	valueIndex := oldCollection indexOf: valueBuilder.
	valueBuilder := newCollection at: valueIndex.!

setup
	"Do any initialization once we've got our parameters"
	| base collectionType |
	base := query baseExpression.
	collectionType := expression == base ifTrue: [query collectionType] ifFalse: [expression mapping query collectionType].
	keyBuilder := ElementBuilder for: (collectionType keyExpression asGlorpExpressionOn: expression) in: query.
	valueBuilder := ElementBuilder basicFor: (collectionType valueExpression asGlorpExpressionOn: expression) in: query.!

subBuilders
	^Array with: keyBuilder with: valueBuilder.!

withAllSubBuilders
	^Array with: self with: keyBuilder with: valueBuilder.! !
!DictionaryBuilder categoriesFor: #buildObjectFrom:!building objects!private! !
!DictionaryBuilder categoriesFor: #canCauseDuplicateRows!accessing!private! !
!DictionaryBuilder categoriesFor: #fieldsForSelectStatement!private!selecting fields! !
!DictionaryBuilder categoriesFor: #fieldsFromMyPerspective!private!selecting fields! !
!DictionaryBuilder categoriesFor: #findInstanceForRow:proxyType:!building objects!private! !
!DictionaryBuilder categoriesFor: #instanceValue!accessing!private! !
!DictionaryBuilder categoriesFor: #knitResultIn:!building objects!private! !
!DictionaryBuilder categoriesFor: #postCopy!copying!private! !
!DictionaryBuilder categoriesFor: #reconcileWithBuilders:andCopy:!copying!private! !
!DictionaryBuilder categoriesFor: #setup!initialize-release!private! !
!DictionaryBuilder categoriesFor: #subBuilders!accessing!private! !
!DictionaryBuilder categoriesFor: #withAllSubBuilders!accessing!private! !

ObjectBuilder guid: (GUID fromString: '{7369DE70-FC65-4D60-8F67-6346F29879D3}')!
ObjectBuilder comment: 'This builds full-blown persistent objects with descriptors. This is the most common type of builder.'!
!ObjectBuilder categoriesForClass!Unclassified! !
!ObjectBuilder methodsFor!

buildObjectFrom: anArray
	self row: anArray.
	self requiresPopulating ifTrue: [self populateInstance].
	(self system shouldResetExpiryTimeForObjectOfClass: instance class readFrom: self query)
		ifTrue: [self session markAsCurrentOfClass: instance class key: self key].!

buildsObjects
	^true.!

cachedQueryFor: aMapping
	^self queryCache at: aMapping ifAbsent: [nil].!

cachedQueryFor: aMapping is: aQuery
	"Save a query which we will use when constructing filtered queries"
	self queryCache at: aMapping put: aQuery.!

canBuild
	"If we have a regular object with a nil primary key, or if we have an embedded object whose values are all nil, we can't build anything (probably due to an outer join)"
	^self descriptor mapsPrimaryKeys
		ifTrue: [self key notNil]
		ifFalse: [true].!

canCache
	| keyFields |
	keyFields := self descriptor keyFields.
	^keyFields notNil and: [keyFields notEmpty].!

canCauseDuplicateRows
	^expression class == MappingExpression and: [expression mapping isToManyRelationship].!

command
	^cursor command.!

createObjectOfClass: aClass
	"Try to create objects using new, so initialization can run. But if we can't do it, then just use basicNew."
	^[aClass new] on: Error do: [:ex | ex return: aClass basicNew].!

descriptor
	
	^expression descriptor.!

fieldsForSelectStatement
	^self fieldsFromThePerspectiveOfTheMainSelect: (self fieldsFromMyPerspective).!

fieldsFromMyPerspective
	^query returnProxies 
		ifTrue: [self descriptor table primaryKeyFields]
		ifFalse: [self descriptor fieldsForSelectStatement]!

fieldsFromThePerspectiveOfTheMainSelect: aCollection
	^expression translateFields: aCollection.!

findInstanceForRow: aRow proxyType: proxyType
	instance := nil.
	self row: aRow.
	self canBuild ifFalse: [^self].
	(self shouldProxyInstanceFor: proxyType)
		ifTrue: [^self newProxyWithNewQueryForClass: proxyType].
	self lookupCachedObject.
	self instanceExistsButIsDeleted
		ifTrue:
			[instance := nil.
			^self].
	instance isNil
		ifFalse:
			[requiresPopulating := requiresPopulating | query shouldRefresh.
			^self].
	requiresPopulating := true.
	instance := self createObjectOfClass: (expression descriptor
		describedConcreteClassFor: self row
		withBuilder: self).
	self canCache ifTrue: [self session cacheAt: self key put: instance].!

initialize

	requiresPopulating := false.
	isExpired := false.!

instance: anObject
	instance := anObject!

instanceDescriptor
	"Return the descriptor for the instance we're going to build. Note that we have to look it up by instance in order to find subclass. "
	| basicDescriptor |
	instance isNil ifTrue: [^nil].
	basicDescriptor := self system descriptorFor: instance.
	basicDescriptor isNil ifFalse: [^basicDescriptor].
	^expression descriptor.!

instanceExistsButIsDeleted
	instance isNil ifTrue: [^false].
	instance := self session filterDeletionFrom: instance.
	^instance isNil.!

key
	^self primaryKey.!

knitResultIn: aCursoredStream 
	"Connect up our built object with any other objects that reference it. Used if we retrieve more than one thing in the same query"

	| relatedBuilder |
	expression canKnit ifFalse: [^self].
	relatedBuilder := aCursoredStream objectBuilderFor: expression base.
	(relatedBuilder isNil or: [relatedBuilder instance isNil]) 
		ifFalse: 
			[expression mapping knitFromBuilder: relatedBuilder toBuilder: self]!

lookupCachedObject
	| resultClass |
	self canBuild ifFalse: [^self].
	self canCache
		ifTrue:
			[
			resultClass := expression descriptor describedClass.
			(self session hasExpired: resultClass key: self key)
				ifTrue:
					[instance := self session expiredInstanceOf: resultClass key: self key.
					requiresPopulating := true.
					isExpired := true]
				ifFalse:
					[instance := self session cacheLookupForClass: resultClass key: self key.
					requiresPopulating := instance isNil]].!

newProxyWithNewQueryForClass: aClass
	| parameters |
	parameters := IdentityDictionary new.
	self descriptor primaryTable primaryKeyFields
		do:
			[:eachField | parameters at: eachField put: (self valueOfField: eachField in: row)].
	instance := aClass new
		session: self session;
		parameters: parameters;
		query: self queryForNewProxy.
	self session addToReaper: instance.
	^self.!

populateInstance
	| tracker |
	key isNil ifTrue: [^self].
	self instanceDescriptor populateObject: instance inBuilder: self.
	tracker := self session modificationTracker.
	tracker isNil ifFalse: [tracker track: instance].!

postCopy
	super postCopy.
	queryCache := nil.!

primaryKey
	"We use self as a special guard value to indicate that the value hasn't changed"
	key == self ifFalse: [^key].
	self canCache ifFalse: [^nil].
	key := self descriptor keyFields
		collect: [:each | self valueOfField: each].
	key size = 1 ifTrue: [key := key first].
	^key.!

printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	expression printOn: aStream.
	aStream nextPut: $).!

queryCache
	queryCache isNil ifTrue: [queryCache := IdentityDictionary new: 5].
	^queryCache.!

queryForNewProxy
	"Create a proxy with a primary key query in which the parameters are the primary key fields. We can't just reuse a mapping's query, it has to be a new primary key one, because the mapping's query has just run and been told to return proxies. This kind of stinks"
	| whereExpression |
	whereExpression := Join new.
	self descriptor primaryTable primaryKeyFields do: [:eachField | 
		whereExpression addSource: eachField target: eachField].
	^Query
		returningOneOf: query resultClass 
		where: whereExpression!

requiresPopulating
	^requiresPopulating and: [self returnProxies not].!

requiresPopulating: aBoolean
	requiresPopulating := aBoolean!

returnProxies
	^query returnProxies.!

returnValueIn: aCursoredStream
	"If we're a to-many relationship, what we retrieve: is the collection containing us, not the individual items"
	| relatedBuilder |
	expression canKnit ifFalse: [^self instance].
	expression mapping isToManyRelationship ifFalse: [^self instance].
	relatedBuilder := aCursoredStream elementBuilderFor: expression base.
	(relatedBuilder isNil or: [relatedBuilder instance isNil]) ifTrue: [^self instance].
	^expression mapping getValueFrom: relatedBuilder instance.!

shouldProxyInstanceFor: proxyType
	^proxyType notNil.! !
!ObjectBuilder categoriesFor: #buildObjectFrom:!building objects!private! !
!ObjectBuilder categoriesFor: #buildsObjects!private!testing! !
!ObjectBuilder categoriesFor: #cachedQueryFor:!accessing!private! !
!ObjectBuilder categoriesFor: #cachedQueryFor:is:!accessing!private! !
!ObjectBuilder categoriesFor: #canBuild!building objects!private! !
!ObjectBuilder categoriesFor: #canCache!building objects!private! !
!ObjectBuilder categoriesFor: #canCauseDuplicateRows!accessing!private! !
!ObjectBuilder categoriesFor: #command!accessing!private! !
!ObjectBuilder categoriesFor: #createObjectOfClass:!building objects!private! !
!ObjectBuilder categoriesFor: #descriptor!accessing!private! !
!ObjectBuilder categoriesFor: #fieldsForSelectStatement!private!selecting fields! !
!ObjectBuilder categoriesFor: #fieldsFromMyPerspective!private!selecting fields! !
!ObjectBuilder categoriesFor: #fieldsFromThePerspectiveOfTheMainSelect:!private!selecting fields! !
!ObjectBuilder categoriesFor: #findInstanceForRow:proxyType:!building objects!private! !
!ObjectBuilder categoriesFor: #initialize!initializing!private! !
!ObjectBuilder categoriesFor: #instance:!accessing!private! !
!ObjectBuilder categoriesFor: #instanceDescriptor!building objects!private! !
!ObjectBuilder categoriesFor: #instanceExistsButIsDeleted!building objects!private! !
!ObjectBuilder categoriesFor: #key!accessing!private! !
!ObjectBuilder categoriesFor: #knitResultIn:!building objects!private! !
!ObjectBuilder categoriesFor: #lookupCachedObject!building objects!private! !
!ObjectBuilder categoriesFor: #newProxyWithNewQueryForClass:!building objects!private! !
!ObjectBuilder categoriesFor: #populateInstance!building objects!private! !
!ObjectBuilder categoriesFor: #postCopy!copying!private! !
!ObjectBuilder categoriesFor: #primaryKey!accessing!private! !
!ObjectBuilder categoriesFor: #printOn:!printing!private! !
!ObjectBuilder categoriesFor: #queryCache!accessing!private! !
!ObjectBuilder categoriesFor: #queryForNewProxy!building objects!private! !
!ObjectBuilder categoriesFor: #requiresPopulating!accessing!private! !
!ObjectBuilder categoriesFor: #requiresPopulating:!accessing!private! !
!ObjectBuilder categoriesFor: #returnProxies!accessing!private! !
!ObjectBuilder categoriesFor: #returnValueIn:!accessing!private! !
!ObjectBuilder categoriesFor: #shouldProxyInstanceFor:!building objects!private! !

SelectingElementBuilder guid: (GUID fromString: '{261BDDA5-4E58-495F-89A9-DE6C83A62F97}')!
SelectingElementBuilder comment: 'This is a special subclass of ElementBuilder used when we execute a query that reads multiple different things, but in any row, we expect only one of them to have a value. This is useful if we''re doing inheritance queries where each concrete subclass is in a separate table. We can join the queries together, but we expect each row to contain only one particular subclass instance.'!
!SelectingElementBuilder categoriesForClass!Unclassified! !
!SelectingElementBuilder methodsFor!

buildObjectFrom: anArray
	^self.!

fieldsForSelectStatement
	^#().!

fieldsFromMyPerspective
	^#().!

findInstanceForRow: aRow proxyType: proxyType
	^self.!

returnValueIn: aCursoredStream
	| builder |
	builder := aCursoredStream privateGetBuilders
		detect:
			[:each | 
			each instance notNil
				and: [each expression ~~ aCursoredStream command query baseExpression]]
		ifNone:
			[self error: 'Internal error in computing horizontal inheritance query.'].
	^builder instance.! !
!SelectingElementBuilder categoriesFor: #buildObjectFrom:!building objects!private! !
!SelectingElementBuilder categoriesFor: #fieldsForSelectStatement!private!selecting fields! !
!SelectingElementBuilder categoriesFor: #fieldsFromMyPerspective!private!selecting fields! !
!SelectingElementBuilder categoriesFor: #findInstanceForRow:proxyType:!building objects!private! !
!SelectingElementBuilder categoriesFor: #returnValueIn:!building objects!private! !

AbstractReadQuery guid: (GUID fromString: '{9EF6F8F3-C446-4D66-81E4-419B29718111}')!
AbstractReadQuery comment: 'An abstract superclass for Glorp queries. Most query types will descend from this, but a couple are specialized.

Subclasses must implement the following messages:
	executing
		rowsFromDatabaseWithParameters:

Instance Variables:
	base	<BaseExpression>	In Glorp expressions it''s very important that expressions we''re trying to compose together are built on the same base, which represents the object that we''re talking about in the query. This holds the base expression that we''re expecting to use in this query.
	limit	<Integer>	Some databases support limiting the number of rows retrieved. If we can, we''ll use this variable to set that.
	offset	<Integer>	Some databases support starting from a given point in the collection of retrievable rows, often used in combination with limit. So, e.g. get me the rows, sorted by name, starting at the 1012th and giving me 40 of them.
	proxyType	<Class | nil>	If this is nil, do not use proxies. Otherwise, use this class to create proxies for relationships. At the moment, likely to be either Proxy or TimedProxy.
	readsOneObject	<Boolean>	Do we return only one object, or a collection. That is, is this a readManyOf: or a readOneOf:
	resultClass	<Behavior>	What kind of objects are we looking for.
	shouldRefresh	<Boolean>	If the result is already in cache, should we refresh it according to what the database says, or just return it as is.
	tracing	<Tracing>	The tracing. This describes the graph of objects we will actually retrieve.
	whereClause	<GlorpExpression>	What is the criteria on which we will filter the results.

'!
!AbstractReadQuery categoriesForClass!Unclassified! !
!AbstractReadQuery methodsFor!

adjustWhereClause
	^self.!

alsoFetch: anExpression
	self tracing alsoFetch: (self expressionBlockFor: anExpression).!

AND: anExpression
	"Allow you to send AND: or OR: directly to a query to build up a query dynamically without needing to mess with the criteria explicitly"

	whereClause := (anExpression
		asGlorpExpressionOn: base) AND: ((whereClause asGlorpExpressionOn: base) asGeneralGlorpExpression).!

asFullQuery
	^self.!

baseExpression
	^base.!

baseExpression: aBaseExpression
	base := aBaseExpression.!

checkCacheWithParameters: aDictionary ifPresent: aBlock
	| primaryKey result filteredResult |
	readsOneObject ifFalse: [^nil].
	self shouldRefresh ifTrue: [^nil].
	"Check if we have retrieval expressions for something other than the base. If so, we're not actually reading this object, or at least not just this object"
	self tracing retrievalExpressions do: [:each | each base isNil ifFalse: [^self]].
	primaryKey := self primaryKeyFrom: aDictionary.
	"We can't figure out the primary key"
	primaryKey isNil ifTrue: [^nil].
	"The primary key is invalid (i.e. has nulls in it). So we've found the entry, and we know it's nil"
	primaryKey = #() ifTrue: [^aBlock value: nil].
	"If it's expired, make sure we do the read but still refresh"
  	(session hasExpired: resultClass key: primaryKey) ifTrue: [
		self shouldRefresh: true.
		^nil].
	result := session 
		cacheAt: primaryKey
		forClass: resultClass
		ifNone: [nil].
	filteredResult := session filterDeletionFrom: result.
	^filteredResult isNil ifTrue: [nil] ifFalse: [
		(session system shouldResetExpiryTimeForObjectOfClass: filteredResult class readFrom: self)
			ifTrue: [self session markAsCurrentOfClass: filteredResult class key: primaryKey].
		aBlock value: filteredResult].!

checkValidity
	resultClass glorpCanHaveDescriptorDirectly ifFalse: [self error: 'resultClass must be a class'].
	self descriptor isNil ifTrue: [self error: 'No descriptor for ', self resultClass name].
	(ordering notNil and: [self descriptor supportsOrdering not]) ifTrue: [self error: 'The descriptor for ', self resultClass name, ' does not support ordering in queries'].!

copyAttributesTo: newQuery
	"We're creating a new query similar to ourselves, but not a direct copy. Copy over the relevant attributes."

	super copyAttributesTo: newQuery.
	newQuery limit: self limit.
	newQuery offset: self offset.
	newQuery proxyType: self proxyType.
	newQuery shouldRefresh: self shouldRefresh.!

defaultTracing
	| defaultTracing |
	defaultTracing := Tracing new.
	defaultTracing base: base.
	^defaultTracing.!

descriptor
	^session descriptorFor: resultClass.!

except: aQuery
	^self minus: aQuery.!

executeWithParameters: parameterArray in: aSession 
	| |
	session := aSession.
	self requiresFullQuery 
		ifTrue: [^self asFullQuery executeWithParameters: parameterArray in: aSession].
	self checkValidity.
	self setUpExpressions.
	self checkCacheWithParameters: parameterArray ifPresent: [:hit | session register: hit. ^hit].
	self setupTracing.
	^self 
		readFromDatabaseWithParameters: parameterArray.!

hasLimit
	^self limit notNil.!

hasOffset
	^self offset notNil and: [self offset > 0].!

hasTracing
	"Return true if we've given this query a tracing already"
	^false.!

initialize
	
	super initialize.
	proxyType := nil.
	shouldRefresh := false.!

initResultClass: aClass whereClause: theCriteria singleObject: aBoolean 
	resultClass := aClass.
	self whereClause: ((theCriteria isNil or: [theCriteria = true or: [theCriteria = false]]) ifTrue: [EmptyExpression on: theCriteria] ifFalse: [theCriteria]).
	readsOneObject := aBoolean.
	aBoolean ifTrue: [self limit: 1].!

intersect: aQuery
	aQuery shortCircuitEmptyReturn ifTrue: [^aQuery].
	self shortCircuitEmptyReturn ifTrue: [^self].
	^CompoundQuery newIntersect addQuery: self; addQuery: aQuery.!

limit
	^limit.!

limit: anInteger
	limit := anInteger.!

limitMakesSense
	^self hasLimit not or: [self limit > 0].!

minus: aQuery
	aQuery shortCircuitEmptyReturn ifTrue: [^self].
	self shortCircuitEmptyReturn ifTrue: [^aQuery].
	^CompoundQuery newMinus addQuery: self; addQuery: aQuery.!

offset
	^offset!

offset: anObject
	offset := anObject.!

OR: anExpression
	"Allow you to send AND: or OR: directly to a query to build up a query dynamically without needing to mess with the criteria explicitly"

	whereClause := (anExpression asGlorpExpressionOn: base)
		OR: (whereClause asGlorpExpressionOn: base) asGeneralGlorpExpression.!

postCopy
	super postCopy.
	base := BaseExpression new.
	whereClause := whereClause asGlorpExpressionOn: base.
	session isNil ifFalse: [self setUpExpressions].!

prepare
	prepared ifTrue: [^self].
	self setupTracing.
	prepared := true.!

primaryKeyFrom: aDictionary
	"Construct a primary key from the given parameters."

	self prepare.
	(aDictionary
		anySatisfy:
			[:each | self session system shouldForeignKeyValueBeTreatedAsNull: each])
		ifTrue: [^#()].
	^self whereClause primaryKeyFromDictionary: aDictionary.!

privateAnd: anExpression
	"AND these two together without checking for common bases, rebuilding, etc. Everything had better be right"
	whereClause := whereClause AND: anExpression.!

proxyType
	"Answer the type of proxies the receiver should build when instantiating objects:
		nil - Do not build proxies.
		TimedProxy - Build TimedProxies.
		Proxy - Build Proxies."

	^proxyType!

proxyType: aClass
	
	proxyType := aClass.!

readFromDatabaseWithParameters: anArray
	| answerStream result |
	answerStream := self rowsFromDatabaseWithParameters: anArray.
	^readsOneObject
		ifTrue: [result := answerStream next. answerStream release. result]
		ifFalse: [self resultCollectionFor: answerStream].!

requiresFullQuery
	^false.!

resultClass
	^resultClass!

resultType
	"Return the database type that we are expecting to return from this query. This only makes sense if we're returning a single field. If we return a collection of types, just return nil here. If we return a mapped object, the #type message will cause an exception."
	self tracing retrievalExpressions size = 1 ifFalse: [^nil].
	^self tracing retrievalExpressions asArray first type.!

retrieve: anExpression
	self tracing retrieve: (self expressionBlockFor: anExpression).!

retrieveMax: anExpression
	self tracing retrieve: ((self expressionBlockFor: anExpression) asGlorpExpression get: #max).!

retrieveMin: anExpression
	self tracing retrieve: ((self expressionBlockFor: anExpression) asGlorpExpression get: #min).!

retrieveSum: anExpression
	self tracing retrieve: ((self expressionBlockFor: anExpression) asGlorpExpression get: #sum).!

returnProxies
	^proxyType notNil!

returnProxies: aBoolean
	
	aBoolean ifTrue: [proxyType := Proxy] ifFalse: [proxyType := nil].!

returnTimedProxies
	^proxyType == TimedProxy!

returnTimedProxies: aBoolean

	aBoolean ifTrue: [proxyType := TimedProxy] ifFalse: [proxyType := nil]!

rowsFromDatabaseWithParameters: anArray
	self subclassResponsibility.!

setUpBase
	whereClause isGlorpExpression ifTrue: [base := whereClause queryLevelBaseExpression] ifFalse: [base := BaseExpression new].!

setUpExpressions
	(whereClause isGlorpExpression and: [whereClause queryLevelBaseExpression notNil])
		ifTrue:
			[base := whereClause queryLevelBaseExpression.
			base hasDescriptor
				ifFalse: [base descriptor: (session descriptorFor: resultClass)]]
		ifFalse:
			[base isNil ifTrue: [base := BaseExpression new].
			base descriptor: (session descriptorFor: resultClass).
			whereClause := whereClause asGlorpExpressionOn: base in: self].
	ordering isNil
		ifFalse:
			[ordering := ordering
				collect:
					[:each | (self expressionBlockFor: each) asGlorpExpressionOn: base in: self]].
	groupBy isNil ifFalse: [groupBy := (self expressionBlockFor: groupBy) asGlorpExpressionOn: base in: self].
	tracing isNil ifFalse: [tracing updateBase: base].
	base descriptor typeResolver setUpExpressionsForQuery: self.!

setupTracing
	self setUpExpressions.
	self tracing setup.
	self tracing additionalExpressions do: [:each |
		each hasDescriptor ifTrue: [
			each descriptor trace: self tracing context: each].
			each validate].!

shortCircuitEmptyReturn
	"If we have a literal false for criteria, we never need to go to the database.
	We also don't need to go to database if limit doesnt make sense in terms of knowing beforehand that no results will be returned"

	(whereClause class == EmptyExpression and: [whereClause isFalse]) ifTrue: [^true].
	self limitMakesSense ifFalse: [^true].
	^false.!

shortCircuitEmptyReturn: parameters
	"If we have a literal false for criteria, we never need to go to the database.
	We also don't need to go to database if limit doesnt make sense in terms of knowing beforehand that no results will be returned"

	(whereClause class == EmptyExpression and: [whereClause isFalse])
		ifTrue: [^true].
	self limitMakesSense ifFalse: [^true].
	self prepare.
	(whereClause shortCircuitOnNullComparison: parameters) ifTrue: [^true].
	^false.!

shouldExpressionBeJoin: anExpression
	"Return true if the expression should be a join in our context (which means it refers only to tables we have in our from list) or if it should be ANDed into the where clause). Only relevant if we're using ANSI joins"
	^anExpression
		allSatisfy:
			[:eachNode | 
			| queryBase |
			queryBase := eachNode queryLevelBaseExpression.
			queryBase isNil or: [	queryBase = self baseExpression]].!

shouldRefresh
	^shouldRefresh!

shouldRefresh: aBoolean
	shouldRefresh := aBoolean!

tracing
	tracing isNil ifTrue: [tracing := self defaultTracing].
	^tracing.!

tracing: aTracing
	tracing := aTracing.
	tracing updateBase: base.
	tracing setup.!

unionAll: aQuery
	aQuery shortCircuitEmptyReturn ifTrue: [^self].
	self shortCircuitEmptyReturn ifTrue: [^aQuery].
	^CompoundQuery newUnionAll addQuery: self; addQuery: aQuery.!

validateWhereClause
	resultClass glorpCanHaveDescriptorDirectly ifFalse: [self error: 'Invalid result class for query'].
	whereClause isJoin ifFalse: [
		whereClause do: [:each | each validate]].
	tracing isNil ifTrue: [^self].
	tracing allTracings do: [:eachTracing |
		eachTracing do: [:eachExpressionNode | eachExpressionNode validate]].!

whereClause
	^whereClause!

whereClause: anExpression
	whereClause := anExpression.
	self setUpBase.! !
!AbstractReadQuery categoriesFor: #adjustWhereClause!preparing!private! !
!AbstractReadQuery categoriesFor: #alsoFetch:!private!specifying retrievals! !
!AbstractReadQuery categoriesFor: #AND:!boolean operations!private! !
!AbstractReadQuery categoriesFor: #asFullQuery!converting!private! !
!AbstractReadQuery categoriesFor: #baseExpression!accessing!private! !
!AbstractReadQuery categoriesFor: #baseExpression:!accessing!private! !
!AbstractReadQuery categoriesFor: #checkCacheWithParameters:ifPresent:!caching!private! !
!AbstractReadQuery categoriesFor: #checkValidity!private!validation! !
!AbstractReadQuery categoriesFor: #copyAttributesTo:!copying!private! !
!AbstractReadQuery categoriesFor: #defaultTracing!accessing!private! !
!AbstractReadQuery categoriesFor: #descriptor!accessing!private! !
!AbstractReadQuery categoriesFor: #except:!boolean operations!private! !
!AbstractReadQuery categoriesFor: #executeWithParameters:in:!executing!private! !
!AbstractReadQuery categoriesFor: #hasLimit!private!testing! !
!AbstractReadQuery categoriesFor: #hasOffset!private!testing! !
!AbstractReadQuery categoriesFor: #hasTracing!private!testing! !
!AbstractReadQuery categoriesFor: #initialize!initialize!private! !
!AbstractReadQuery categoriesFor: #initResultClass:whereClause:singleObject:!initialize!private! !
!AbstractReadQuery categoriesFor: #intersect:!boolean operations!private! !
!AbstractReadQuery categoriesFor: #limit!accessing!private! !
!AbstractReadQuery categoriesFor: #limit:!accessing!private! !
!AbstractReadQuery categoriesFor: #limitMakesSense!private!testing! !
!AbstractReadQuery categoriesFor: #minus:!boolean operations!private! !
!AbstractReadQuery categoriesFor: #offset!accessing!private! !
!AbstractReadQuery categoriesFor: #offset:!accessing!private! !
!AbstractReadQuery categoriesFor: #OR:!boolean operations!private! !
!AbstractReadQuery categoriesFor: #postCopy!copying!private! !
!AbstractReadQuery categoriesFor: #prepare!preparing!private! !
!AbstractReadQuery categoriesFor: #primaryKeyFrom:!caching!private! !
!AbstractReadQuery categoriesFor: #privateAnd:!private! !
!AbstractReadQuery categoriesFor: #proxyType!accessing!private! !
!AbstractReadQuery categoriesFor: #proxyType:!accessing!private! !
!AbstractReadQuery categoriesFor: #readFromDatabaseWithParameters:!executing!private! !
!AbstractReadQuery categoriesFor: #requiresFullQuery!private!testing! !
!AbstractReadQuery categoriesFor: #resultClass!accessing!private! !
!AbstractReadQuery categoriesFor: #resultType!accessing!private! !
!AbstractReadQuery categoriesFor: #retrieve:!private!specifying retrievals! !
!AbstractReadQuery categoriesFor: #retrieveMax:!private!specifying retrievals! !
!AbstractReadQuery categoriesFor: #retrieveMin:!private!specifying retrievals! !
!AbstractReadQuery categoriesFor: #retrieveSum:!private!specifying retrievals! !
!AbstractReadQuery categoriesFor: #returnProxies!accessing!private! !
!AbstractReadQuery categoriesFor: #returnProxies:!accessing!private! !
!AbstractReadQuery categoriesFor: #returnTimedProxies!private!testing! !
!AbstractReadQuery categoriesFor: #returnTimedProxies:!accessing!private! !
!AbstractReadQuery categoriesFor: #rowsFromDatabaseWithParameters:!executing!private! !
!AbstractReadQuery categoriesFor: #setUpBase!accessing!private! !
!AbstractReadQuery categoriesFor: #setUpExpressions!executing!private! !
!AbstractReadQuery categoriesFor: #setupTracing!private!tracing! !
!AbstractReadQuery categoriesFor: #shortCircuitEmptyReturn!private!testing! !
!AbstractReadQuery categoriesFor: #shortCircuitEmptyReturn:!private!testing! !
!AbstractReadQuery categoriesFor: #shouldExpressionBeJoin:!preparing!private! !
!AbstractReadQuery categoriesFor: #shouldRefresh!accessing!private! !
!AbstractReadQuery categoriesFor: #shouldRefresh:!accessing!private! !
!AbstractReadQuery categoriesFor: #tracing!accessing!private! !
!AbstractReadQuery categoriesFor: #tracing:!accessing!private! !
!AbstractReadQuery categoriesFor: #unionAll:!boolean operations!private! !
!AbstractReadQuery categoriesFor: #validateWhereClause!executing!private! !
!AbstractReadQuery categoriesFor: #whereClause!accessing!private! !
!AbstractReadQuery categoriesFor: #whereClause:!accessing!private! !

!AbstractReadQuery class methodsFor!

newQuery
	^SimpleQuery new.!

read: aClass limit: anInteger
	^self returningManyOf: aClass limit: anInteger.!

read: aClass where: anExpression limit: anInteger
	^self
		returningManyOf: aClass
		where: anExpression
		limit: anInteger.!

readManyOf: aClass limit: anInteger
	^self returningManyOf: aClass limit: anInteger.!

readManyOf: aClass where: anExpression limit: anInteger
	^self
		returningManyOf: aClass
		where: anExpression
		limit: anInteger.!

returningManyOf: aClass
	^self returningManyOf: aClass where: nil.!

returningManyOf: aClass limit: anInteger
	^self
		returningManyOf: aClass
		where: nil
		limit: anInteger.!

returningManyOf: aClass where: anExpression
	^self returningManyOf: aClass where: anExpression limit: nil.!

returningManyOf: aClass where: anExpression limit: anInteger
	^(self newQuery)
		limit: anInteger;
		initResultClass: aClass
			whereClause: anExpression
			singleObject: false;
		yourself.!

returningOneOf: aClass
	^self returningOneOf: aClass where: nil.!

returningOneOf: aClass where: anExpression
	^self newQuery
		initResultClass: aClass
		whereClause: anExpression
		singleObject: true.! !
!AbstractReadQuery class categoriesFor: #newQuery!instance creation!private! !
!AbstractReadQuery class categoriesFor: #read:limit:!instance creation!private! !
!AbstractReadQuery class categoriesFor: #read:where:limit:!instance creation!private! !
!AbstractReadQuery class categoriesFor: #readManyOf:limit:!instance creation!private! !
!AbstractReadQuery class categoriesFor: #readManyOf:where:limit:!instance creation!private! !
!AbstractReadQuery class categoriesFor: #returningManyOf:!instance creation!private! !
!AbstractReadQuery class categoriesFor: #returningManyOf:limit:!instance creation!private! !
!AbstractReadQuery class categoriesFor: #returningManyOf:where:!instance creation!private! !
!AbstractReadQuery class categoriesFor: #returningManyOf:where:limit:!instance creation!private! !
!AbstractReadQuery class categoriesFor: #returningOneOf:!instance creation!private! !
!AbstractReadQuery class categoriesFor: #returningOneOf:where:!instance creation!private! !

CompositeQuery guid: (GUID fromString: '{AE32AB3F-061C-40A0-A9BF-C947B981373F}')!
CompositeQuery comment: 'This is a query that contains several sub-queries, and executes them all in sequence, returning the concatenation of all the results. This is different from a CompoundQuery, which groups multiple queries together according to operations like UNION ALL, or INTERSECT. This just puts together several different queries and executes them separately, combining the results in Smalltalk. CompoundQuery is more powerful, and can take advantage of database features, but this is more flexible.'!
!CompositeQuery categoriesForClass!Unclassified! !
!CompositeQuery methodsFor!

addQuery: aQuery
	queries add: aQuery.!

baseExpression
	^queries first baseExpression.!

collectionType
	^self primaryQuery collectionType.!

copyOfBuilders 
	^self primaryQuery copyOfBuilders.!

executeWithParameters: parameterArray in: aSession
	| results |
	results := self collectionType new writeStream.
	queries do: [:each |
		results nextPutAll: (each executeWithParameters: parameterArray in: aSession)].
	^results contents.!

initialize
	super initialize.
	queries := OrderedCollection new.!

postCopy

	super postCopy.
	queries := queries collect: [:each | each copy].!

prepare
	queries do: [:each | each session: session].
	queries do: [:each | each prepare].!

primaryQuery
	^queries first.!

proxyType
	^self primaryQuery proxyType.!

proxyType: aClass	
	"We currently ignore proxy types and use the ones from our main query."
	^self.! !
!CompositeQuery categoriesFor: #addQuery:!accessing!private! !
!CompositeQuery categoriesFor: #baseExpression!accessing!private! !
!CompositeQuery categoriesFor: #collectionType!accessing!private! !
!CompositeQuery categoriesFor: #copyOfBuilders!preparing!private! !
!CompositeQuery categoriesFor: #executeWithParameters:in:!executing!private! !
!CompositeQuery categoriesFor: #initialize!initialize!private! !
!CompositeQuery categoriesFor: #postCopy!copying!private! !
!CompositeQuery categoriesFor: #prepare!preparing!private! !
!CompositeQuery categoriesFor: #primaryQuery!accessing!private! !
!CompositeQuery categoriesFor: #proxyType!accessing!private! !
!CompositeQuery categoriesFor: #proxyType:!accessing!private! !

CompoundQuery guid: (GUID fromString: '{88333FEA-0237-4CAF-AB6C-C906D3D66AFE}')!
CompoundQuery comment: 'This is a query that has more than one subquery, combined by expressions like UNION ALL or INTERSECTION. It doesn''t do all that much except print its subqueries and combine them according to the operation.

Instance Variables:
	operation	<String>	The name of the operation we are using to combine the queries.
	primaryQuery	<Query>	The first query that we contain. This determines what objects we actually return. All of the queries should match up in terms of what they return, or the database is unlikely to accept this as a valid query.
	queries	<(SequenceableCollection of: SimpleQuery)>	The queries we contain.
	specialBuilders <Array of: ElementBuilder> If we want to do something to modify our result, we can add our own set of builders here, which will be added to those of the internal queries. Motivating example is doing a horizontal inheritance query, where we can end up with one of N different objects in the row, and we just want to return that one directly.

'!
!CompoundQuery categoriesForClass!Unclassified! !
!CompoundQuery methodsFor!

addQuery: aQuery
	queries isEmpty ifTrue: [primaryQuery := aQuery].
	queries add: aQuery.!

baseExpression
	^queries first baseExpression.!

copyOfBuilders 
	^primaryQuery copyOfBuilders.!

except: aQuery
	^self minus: aQuery.!

executeWithParameters: parameterArray in: aSession
	session := aSession.
	^self 
		readFromDatabaseWithParameters: parameterArray.!

initialize
	super initialize.
	requiresDistinct := true.
	queries := OrderedCollection new: 3.!

isMinus
	^operation = 'EXCEPT'.!

isUnionAll
	^operation = #'UNION ALL'.!

joins
	^queries inject: #() into: [:sum :each | sum, each joins].!

limit
	^primaryQuery limit.!

limit: anInteger
	"We currently ignore limit sets and use the one from our contained queries."
	^self.!

mightHaveDuplicateRows
	^primaryQuery mightHaveDuplicateRows.!

minus: aQuery
	aQuery shortCircuitEmptyReturn ifTrue: [^self].
	^CompoundQuery newMinus addQuery: self; addQuery: aQuery.!

offset
	^primaryQuery offset.!

offset: anInteger
	"We currently ignore offset sets and use the one from our contained queries."
	^self.!

operation: aString
	operation := aString.!

postCopy

	super postCopy.
	queries := queries collect: [:each | each copy].
	primaryQuery := queries first.!

prepare
	| |
	queries do: [:each | each session: session].
	queries do: [:each | each prepare].
	ordering isNil ifFalse: [
		ordering := ordering collect: [:each |
			(self expressionBlockFor: each) asGlorpExpressionOn: queries first baseExpression]].
	groupBy isNil ifFalse: [groupBy := (self expressionBlockFor: groupBy) asGlorpExpressionOn: queries first baseExpression].!

primaryQuery: aQuery
	primaryQuery := aQuery.!

printCompoundSelectAliasesOn: aStream
	"We only want one table. This is a somewhat convoluted way of getting 't1' uniformly appended"
	aStream nextPutAll: ' '.
	self session platform supportsAliasInCompoundQuery ifTrue: [
		primaryQuery tablesToPrint first printSQLOn: aStream withParameters: nil].!

printOperationOn: aCommand
	aCommand space.
	aCommand nextPutAll: (self platform compoundOperationFor: operation).
	aCommand space.!

proxyType
	^primaryQuery proxyType.!

proxyType: aClass	
	"We currently ignore proxy types and use the ones from our main query."
	^self.!

queries
	^queries!

queries: aCollectionOfQueries
	queries := aCollectionOfQueries!

readFromDatabaseWithParameters: anArray
	| answerStream result |
	answerStream := self rowsFromDatabaseWithParameters: anArray.
	^self readsOneObject
		ifTrue: [result := answerStream next. answerStream release. result]
		ifFalse: [self resultCollectionFor: answerStream].!

requiresDistinct
	^requiresDistinct!

requiresDistinct: aBoolean
	requiresDistinct := aBoolean.!

resultClass
	^primaryQuery resultClass.!

rewriteExcept
	"Rewrite any EXCEPT operation we have into a NOT EXISTS test with a correlated subselect. Only a CompoundQuery would have an EXCEPT"
	
	| |
	operation == #EXCEPT ifFalse: [^self].
	queries := queries collect: [:each | each rewriteExcept].
	^self rewriteSetOperationUsing: #notExists:!

rewriteIntersect
	"Rewrite any INTERSECT operation we have into an EXISTS test for a correlated subselect. Only a CompoundQuery would have an intersect"
	
	| |
	operation == #INTERSECT ifFalse: [^self].
	queries := queries collect: [:each | each rewriteIntersect].
	^self rewriteSetOperationUsing: #exists:!

rewriteSetOperationUsing: aSelector
	"Rewrite any INTERSECT OR EXCEPT operation we have into a correlated EXISTS or NOT EXISTS test, respectively. We take the sub-query, and AND into it a constraint that its base expression is equal to the first queries base expression, and then do either an exists: or a notExists:, depending on aSelector."
	
	| newQuery |
	newQuery := queries first copy.
	2 to: queries size do: [:i | | q expression mainExpression |
		q := (queries at: i) copy.
		expression := RelationExpression new outerJoin: false; relation: #=; leftChild: newQuery baseExpression; rightChild: q baseExpression.
		mainExpression := RelationExpression new outerJoin: false; relation: #AND; leftChild: q whereClause; rightChild: expression.
		q whereClause: mainExpression.
		newQuery AND: [:each | each perform: aSelector with: q]].
	^newQuery.!

rowsFromDatabaseWithParameters: anArray
	^session accessor
		executeCommand: (self sqlWith: anArray)
		returnCursor: true.!

shortCircuitEmptyReturn
	^false.!

shortCircuitEmptyReturn: parameters
	^false.!

shouldRefresh
	^primaryQuery shouldRefresh.!

shouldRefresh: aBoolean
	"We currently ignore sets of this, and use the value from our primary query."
	^self.!

sqlString
	"Mostly for debugging"
	^(self sqlWith: #()) sqlString.!

tablesToPrint
	^self queries first tablesToPrint.!

tracing
	^primaryQuery tracing.!

unionAll: aQuery
	aQuery shortCircuitEmptyReturn ifTrue: [^self].
	^self isUnionAll
		ifTrue:
			[queries add: aQuery.
			self]
		ifFalse:
			[(CompoundQuery newUnionAll)
				addQuery: self;
				addQuery: aQuery].!

whereClause
	| allCriteria |
	allCriteria := queries inject: #() into: [:sum :each | sum, each whereClause].
	^ExpressionGroup new addAll: allCriteria.! !
!CompoundQuery categoriesFor: #addQuery:!accessing!private! !
!CompoundQuery categoriesFor: #baseExpression!accessing!private! !
!CompoundQuery categoriesFor: #copyOfBuilders!preparing!private! !
!CompoundQuery categoriesFor: #except:!boolean operations!private! !
!CompoundQuery categoriesFor: #executeWithParameters:in:!executing!private! !
!CompoundQuery categoriesFor: #initialize!initializing!private! !
!CompoundQuery categoriesFor: #isMinus!private!testing! !
!CompoundQuery categoriesFor: #isUnionAll!private!testing! !
!CompoundQuery categoriesFor: #joins!accessing!private! !
!CompoundQuery categoriesFor: #limit!accessing!private! !
!CompoundQuery categoriesFor: #limit:!accessing!private! !
!CompoundQuery categoriesFor: #mightHaveDuplicateRows!accessing!private! !
!CompoundQuery categoriesFor: #minus:!boolean operations!private! !
!CompoundQuery categoriesFor: #offset!accessing!private! !
!CompoundQuery categoriesFor: #offset:!accessing!private! !
!CompoundQuery categoriesFor: #operation:!accessing!private! !
!CompoundQuery categoriesFor: #postCopy!copying!private! !
!CompoundQuery categoriesFor: #prepare!preparing!private! !
!CompoundQuery categoriesFor: #primaryQuery:!accessing!private! !
!CompoundQuery categoriesFor: #printCompoundSelectAliasesOn:!printing!public! !
!CompoundQuery categoriesFor: #printOperationOn:!printing!private! !
!CompoundQuery categoriesFor: #proxyType!accessing!private! !
!CompoundQuery categoriesFor: #proxyType:!accessing!private! !
!CompoundQuery categoriesFor: #queries!accessing!private! !
!CompoundQuery categoriesFor: #queries:!accessing!private! !
!CompoundQuery categoriesFor: #readFromDatabaseWithParameters:!executing!private! !
!CompoundQuery categoriesFor: #requiresDistinct!accessing!private! !
!CompoundQuery categoriesFor: #requiresDistinct:!accessing!private! !
!CompoundQuery categoriesFor: #resultClass!As yet unclassified!private! !
!CompoundQuery categoriesFor: #rewriteExcept!private!sql generation! !
!CompoundQuery categoriesFor: #rewriteIntersect!private!sql generation! !
!CompoundQuery categoriesFor: #rewriteSetOperationUsing:!private!sql generation! !
!CompoundQuery categoriesFor: #rowsFromDatabaseWithParameters:!executing!private! !
!CompoundQuery categoriesFor: #shortCircuitEmptyReturn!As yet unclassified!private! !
!CompoundQuery categoriesFor: #shortCircuitEmptyReturn:!As yet unclassified!private! !
!CompoundQuery categoriesFor: #shouldRefresh!accessing!private! !
!CompoundQuery categoriesFor: #shouldRefresh:!accessing!private! !
!CompoundQuery categoriesFor: #sqlString!printing!private! !
!CompoundQuery categoriesFor: #tablesToPrint!printing!private! !
!CompoundQuery categoriesFor: #tracing!accessing!private! !
!CompoundQuery categoriesFor: #unionAll:!boolean operations!private! !
!CompoundQuery categoriesFor: #whereClause!accessing!private! !

!CompoundQuery class methodsFor!

newIntersect
	^self new operation: #INTERSECT.!

newMinus
	^self new operation: #EXCEPT.!

newUnionAll
	^self new operation: #'UNION ALL'.! !
!CompoundQuery class categoriesFor: #newIntersect!instance creation!private! !
!CompoundQuery class categoriesFor: #newMinus!instance creation!private! !
!CompoundQuery class categoriesFor: #newUnionAll!instance creation!private! !

FilteringQuery guid: (GUID fromString: '{6E8126B8-FFF5-4346-AE56-049C5BB84FD3}')!
FilteringQuery comment: 'This is a query that expects all the objects it might return to already be in memory, and in a collection from which it can pick them out by evaluating a block. This is used to implement filtered reads on a mapping. When one of the mappings fires, it reads all the relevant results, and so each instance  using that mapping can get the results out of memory without going to the database again.

Instance Variables:
	collectBlock	<Block>	A one-argument block we evaluate to actually collect up the results. That is, internally we are likely to have a more complex representation in the collection, e.g. an array containing various extra information plus the objects. This gets the objects out of the collection.
	collection	<(GlorpVirtualCollection of: Object)>	The collection on which we filter. Typically a virtual collection, at least for filtered reads, but can also handle a normal collection.
	filter	<Block>	The two-argument block we use to filter. The first argument is the entry from the collection, the second is the parameters to the query that were used.
	readsOneObject	<Boolean>	A duplicate of readsOneObject in AbstractReadyQuery. Do we return a collection, or a single result.

'!
!FilteringQuery categoriesForClass!Unclassified! !
!FilteringQuery methodsFor!

collectBlock
	^collectBlock!

collectBlock: aBlock
	collectBlock:= aBlock.!

collection
	^collection!

collection: aCollection
	aCollection class == GlorpVirtualCollection ifTrue: [collectionType := aCollection getQuery collectionType].
	collection := aCollection.!

collectionTypesAgree: aCollection
	^self collectionType new class == aCollection class!

executeWithParameters: parameterArray in: aSession
	| filtered result |
	session := aSession.
	collection class == GlorpVirtualCollection
		ifTrue:
			[collection setSession: aSession.
			collection parameters: parameterArray.
			collection := collection realObjects].
	filtered := self filterResultsForParameters: parameterArray.
	result := self transformsValues
		ifTrue: [filtered collect: [:each | self valueFor: each]]
		ifFalse: [filtered].
	result := (self collectionTypesAgree: result) ifTrue: [result] ifFalse: [self resultCollectionFor: result readStream].
	^self readsOneObject
		ifTrue: [result detect: [:each | true] ifNone: [nil]]
		ifFalse: [result].!

filter
	^filter!

filter: aBlock
	filter := aBlock!

filterResultsForParameters: parameterArray
	| passing seen |
	seen := IdentitySet new.
	passing := collection
		select:
			[:each | 
			| passes new value |
			passes := filter value: each value: parameterArray.
			value := self valueFor: each.
			passes
				ifTrue:
					[new := (seen includes: value) not.
					new ifTrue: [seen add: value]].
			passes and: [new]].
	^passing.!

resultClass
	collection class == GlorpVirtualCollection ifTrue: [^collection getQuery resultClass].
	^(collection detect: [:each | true] ifNone: [nil]) class!

shortCircuitEmptyReturn: parameters
	^false.!

transformsValues
	^collectBlock notNil.!

valueFor: anObject
	collectBlock isNil ifTrue: [^anObject].
	^collectBlock value: anObject.! !
!FilteringQuery categoriesFor: #collectBlock!accessing!private! !
!FilteringQuery categoriesFor: #collectBlock:!accessing!private! !
!FilteringQuery categoriesFor: #collection!accessing!private! !
!FilteringQuery categoriesFor: #collection:!accessing!private! !
!FilteringQuery categoriesFor: #collectionTypesAgree:!executing!private! !
!FilteringQuery categoriesFor: #executeWithParameters:in:!executing!private! !
!FilteringQuery categoriesFor: #filter!accessing!private! !
!FilteringQuery categoriesFor: #filter:!accessing!private! !
!FilteringQuery categoriesFor: #filterResultsForParameters:!executing!public! !
!FilteringQuery categoriesFor: #resultClass!accessing!private! !
!FilteringQuery categoriesFor: #shortCircuitEmptyReturn:!private!testing! !
!FilteringQuery categoriesFor: #transformsValues!executing!private! !
!FilteringQuery categoriesFor: #valueFor:!executing!private! !

!FilteringQuery class methodsFor!

on: aCollection filter: aBlock
	^self new
		collection: aCollection;
 		filter: aBlock.! !
!FilteringQuery class categoriesFor: #on:filter:!instance creation!private! !

ReadQuery guid: (GUID fromString: '{3FD6CA96-3FF8-4B0F-9FCF-2B280A2B02DC}')!
ReadQuery comment: 'This represents a general read query. By general we mean that it might require more than one trip to the database. It computes a "tracing" indicating which groups of objects can be read simultaneously, then constructs a group of corresponding SimpleQuery instances and executes them.'!
!ReadQuery categoriesForClass!Unclassified! !
!ReadQuery methodsFor!

asQueryWithNullsForOtherClasses: aClass
	"Construct a specialized form of the query that's set up to retrieve instances of aClass, but with a bunch of empty fields that get populated with nulls, so we can retrieve several different classes at the same go."

	| newQuery nullField fieldsToIgnore markerExpression fieldsMinusTheConstant pseudoMappingName newBase tablesToIgnore |
	newQuery := self asSimpleQueryFor: aClass.	"We need to rebuild the where clause to refer to one of the subclass pseudo-mappings, because that's what we're retrieving. We don't want it against the actual class we're querying for, because we need the fields to be exactly the same for each of the sub-queries."
	pseudoMappingName := self descriptor typeResolver
		pseudoAttributeNameForClass: aClass.
	newBase := newQuery baseExpression get: pseudoMappingName.
	newBase bePhantom.
	newQuery where: (newQuery whereClause rebuildOn: newBase).	
	"We don't want it to try and retrieve the parent class. Instead, we alsoFetch: each of the subclasses, and insert a special builder that will select from among the valid results. To prevent it from trying to do anything with the main class itself, remove that from the tracing, and replace it with a special constant marker that we know will generate only one field, that we can remove later."
	newQuery tracing retrievalExpressions remove: newQuery tracing base.
	newQuery tracing allTracings remove: newQuery tracing base.
	markerExpression := ConstantExpression for: 'ignore me'.
	newQuery tracing retrievalExpressions add: markerExpression.
	newQuery tracing allTracings add: markerExpression.
	newQuery prepare.
	newQuery
		privateReplaceBuilderWith:
			(SelectingElementBuilder new expression: markerExpression).	
	"For each retrieval that isn't our class, change the fields being returned to nulls, and wrap a cast around them so that SQL doesn't complain. First, we figure out which fields those are."
	fieldsToIgnore := IdentityDictionary new.
	newQuery tracing alsoFetchExpressions
		do:
			[:each | 
			each mapping referenceClass = aClass
				ifFalse:
					[each fieldAliases
						do: [:eachFieldAlias | fieldsToIgnore at: eachFieldAlias put: each]]].	
	"Now we remove the constant field we know we caused to be generated earlier, and replace the others that we don't want with nulls"
	fieldsMinusTheConstant := newQuery fields
		copyFrom: 1
		to: newQuery fields size - 1.
	newQuery
		privateSetFields:
			(fieldsMinusTheConstant
				collect:
					[:eachField | 
					(fieldsToIgnore includesKey: eachField)
						ifFalse: [eachField]
						ifTrue:
							[nullField := self nullFieldFor: eachField.
							nullField]]).	
	"Supress all tables except the one(s) we actually want."
	newQuery prepare.
	tablesToIgnore := (fieldsToIgnore keys collect: [:each | each table]) asSet asArray.
	tablesToIgnore := tablesToIgnore, (newQuery baseExpression tablesToPrint).
	newQuery tablesToPrint: (newQuery tablesToPrint reject: [:each | tablesToIgnore includes: each]) asArray.	
	^newQuery.!

asSimpleQueryFor: aClass
	| newQuery newCriteria newBase |
	"Rebuild the expression, because this means a full query is being split into multiple sub-queries, e.g. for an inheritance read. The expression may get prepared differently in each case (e.g. table aliases), so we can't share"
	newBase := BaseExpression new.
	newBase descriptor: (session system descriptorFor: aClass).
	newCriteria := whereClause rebuildOn: newBase startingFrom: newBase withOuterScopeBase: newBase.
	newQuery := SimpleQuery new
		initResultClass: aClass
		whereClause: newCriteria
		singleObject: readsOneObject.
	self copyAttributesTo: newQuery.
	newQuery session: session.
	newQuery setOrdering: ordering.
	newQuery groupBy: groupBy.
	newQuery setUpExpressions.
	newQuery tracing: tracing copy.
	^newQuery.!

executeSubQueries: subQueries withParameters: anArray
	^self shouldUseSingleQuery
		ifTrue: [self executeSubQueriesCombined: subQueries withParameters: anArray]
		ifFalse: [self executeSubQueriesIndividually: subQueries withParameters: anArray].!

executeSubQueriesCombined: subQueries withParameters: anArray
	"Execute out sub queries as a single query with a UNION ALL between the sub-queries. Then select from the results the one which has a value for that row. Used for horizontal inheritance queries."
	| combinedQuery |
	combinedQuery := CompoundQuery newUnionAll.
	self copyAttributesTo: combinedQuery.
	combinedQuery requiresDistinct: false.
	subQueries 	do: 	[:eachQuery | 
		combinedQuery addQuery: eachQuery].
	^combinedQuery executeWithParameters: anArray in: session.!

executeSubQueriesIndividually: subQueries withParameters: anArray
	| result |
	result := OrderedCollection new.
	subQueries 	do: 	[:eachQuery | 
		| individualResult |
		individualResult := eachQuery readFromDatabaseWithParameters: anArray.
		eachQuery readsOneObject
			ifTrue: [individualResult isNil ifFalse: [result add: individualResult]]
			ifFalse: [result addAll: individualResult]].
	^self readsOneObject ifTrue: [result first] ifFalse: [result].!

nullFieldFor: eachField
	self platform requiresCastsForNullsInUnions ifFalse: [^(ConstantExpression for: nil) alias: eachField table name, 'nullField', eachField position printString].
	^FunctionExpression
		for: #cast:
		withArguments: (Array with: (ConstantExpression for: eachField type))
		basedOn: (ConstantExpression for: nil).!

readFromDatabaseWithParameters: anArray 
	| subQueries |
	subQueries := self descriptor classesRequiringIndependentQueries 
		collect: [:eachClass | 
			self shouldUseSingleQuery 
				ifTrue: [self asQueryWithNullsForOtherClasses: eachClass]
				ifFalse: [self asSimpleQueryFor: eachClass]].
	^self executeSubQueries: subQueries withParameters: anArray.!

shouldUseSingleQuery
	^self descriptor typeResolver shouldUseSingleQuery.! !
!ReadQuery categoriesFor: #asQueryWithNullsForOtherClasses:!converting!private! !
!ReadQuery categoriesFor: #asSimpleQueryFor:!converting!private! !
!ReadQuery categoriesFor: #executeSubQueries:withParameters:!executing!private! !
!ReadQuery categoriesFor: #executeSubQueriesCombined:withParameters:!executing!private! !
!ReadQuery categoriesFor: #executeSubQueriesIndividually:withParameters:!executing!private! !
!ReadQuery categoriesFor: #nullFieldFor:!converting!private! !
!ReadQuery categoriesFor: #readFromDatabaseWithParameters:!executing!private! !
!ReadQuery categoriesFor: #shouldUseSingleQuery!accessing!private! !

SimpleQuery guid: (GUID fromString: '{70B19DCB-C418-4B3B-A1FB-39219E635F3A}')!
SimpleQuery comment: 'This is a query that is directly executable. A single query might be more than we can do in a single database read, so we might have to break it down into simple queries. But at the moment we just break anything down into an equivalent single query.

Instance Variables:
	builders	<OrderedCollection of: ElementBuilder)>	The builders that will assemble the object from the row that this query returns.
	fields	<OrderedCollection of: DatabaseField>	The fields being selected.
	joins	<OrderedCollection of: Join> While preparing the query, we will need to build up a list of joins implied by the relationships in the where clause, ordering, and so forth. This acts as storage for those.
	tablePrefix	<String>	If we are part of a subselect, we want to prefix all of our table aliases with some known, common prefix, so that we won''t have our own t1 which would get confused with the parent query''s t1. The normal convention is something like ''s1'' for subselect 1, so we get s1t1, and so forth.
	command <DatabaseCommand> We can tell the query to use a specific command instead of generating one appropriately. Note that if this is set, it overrides most of the other sql generation.

'!
!SimpleQuery categoriesForClass!Unclassified! !
!SimpleQuery methodsFor!

addAllOrderingsForBuilder: aBuilder
	
	| expression fieldExpressions |
	expression := aBuilder expression.
	fieldExpressions := expression descriptor primaryTable primaryKeyFields collect: [:each | expression getField: each].
	fieldExpressions do: [:each | self addSingleOrderingForBuilder: each].!

addFields: aliasedFields returningTranslationForFields: originalFields distinct: isDistinct
	"The query has computed a set of fields the way the mappings see them, which are then transformed to account for field aliasing in embedded mappings. Add those to our collection, and set up the translation which knows which fields are at which index in the resulting row. If necessary, note that those fields are selected as distinct"

	| translation |
	translation := IdentityDictionary new.
	aliasedFields
		with: originalFields
		do:
			[:aliased :original | 
			| position fieldToAdd|
			position := self findPositionofAliasedField: aliased.
			position = 0
				ifTrue:  
					[fieldToAdd := isDistinct ifTrue: [aliased asDistinctFieldOn: self baseExpression] ifFalse: [aliased].
					fields add: fieldToAdd .
					position := fields size].
			translation at: original put: position].
	^translation.!

addJoin: anExpression
	joins addAll: anExpression asIndependentJoins.!

addOrderingForToManyJoins
	"If we have a join to a to-many relation, then we need for all the joined bits to be grouped together in order in the result set"
	| problemBuilders baseBuilders |
	problemBuilders := builders select: [:each | each canCauseDuplicateRows].
	baseBuilders := problemBuilders
		collect:
			[:each | 
			builders
				detect: [:eachBuilder | eachBuilder expression == each expression base]
				ifNone: [nil]].
	baseBuilders do: [:each | 
		each isNil ifFalse: [self addAllOrderingsForBuilder: each]].!

addSingleOrderingForBuilder: aFieldExpression
	"Add the ordering to the query, as long as we don't already have an equivalent ordering there."	

	ordering isNil ifTrue: [^self orderBy: aFieldExpression].
	ordering 
		detect: [:eachExistingOrdering | 
			eachExistingOrdering = aFieldExpression 
				or: [eachExistingOrdering field = aFieldExpression field and: [eachExistingOrdering base = aFieldExpression base]]]
		ifNone: [self orderBy: aFieldExpression].!

adjustWhereClause
	"Make any adjustments to the where clause. This includes adjusting direct object equality expressions to be primary key comparisons, removing primary key comparisons where we can compare a foreign key, and condensing boolean constants out of expressions."

	whereClause := whereClause rewriteEquality. "Special because otherwise we'd be throwing away the top-level one"
	whereClause := whereClause rewriteBooleanEquality: nil.
	whereClause do: [:each | each rewriteEqualityExpressions].
	whereClause do: [:each | each condensePrimaryKeyComparisons].
	whereClause := whereClause condensePrimaryKeyComparison.
	whereClause := whereClause condenseBooleanEquality.
	whereClause do: [:each | each condenseBooleanEqualityExpressions].
	(whereClause isConstantExpression and: [whereClause value = true])
		ifTrue: [
			whereClause := EmptyExpression on: whereClause value.
			whereClause base: base.].
	whereClause do: [:each | each rewriteFunctionArguments].!

asFullQuery
	| newQuery |
	newQuery := ReadQuery new
		initResultClass: resultClass whereClause: whereClause singleObject: readsOneObject.
	self copyAttributesTo: newQuery.
	newQuery setOrdering: ordering.
	newQuery groupBy: groupBy.
	^newQuery.!

asGlorpExpression
	^self asGlorpExpressionOn: BaseExpression new.!

asGlorpExpressionOn: anExpression basedOn: aMessageArchiver withUltimateBase: aBaseExpression
	^self asGlorpExpressionOn: aBaseExpression.!

assignTableAliases
	| tableNumber allExpressions |
	whereClause isJoin ifTrue: [^self].
	tableNumber := 1.
	allExpressions := ExpressionGroup with: whereClause.
	allExpressions addAll: ordering.
	allExpressions add: groupBy.
	allExpressions addAll: joins.
	builders do: [:each | allExpressions add: each expression].
	allExpressions
		do:
			[:each | tableNumber := each assignTableAliasesStartingAt: tableNumber prefix: self tablePrefix].
	"We've aliased tables, joins may now become independent"
	joins := joins inject: OrderedCollection new into: [:sum :each | (sum addAll: each asIndependentJoins). sum].!

basicRebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression
	"Rebuild the query, which is part of a subselect expression, into the new context. The only thing that really matters here is that we get correlated subselect pieces inside the query right. The rest is independent of our main expression"
	prepared := false.
	builders := nil.
	base := SubSelectBaseExpression on: (base trueBase rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression).
	whereClause := whereClause asGlorpExpressionOn: base withUltimateBase: ultimateBaseExpression.
	joins := joins collect: [:each | each asGlorpExpressionOn: base withUltimateBase: ultimateBaseExpression].
	ordering isNil ifFalse: [ordering := ordering collect: [:each | each asGlorpExpressionOn: base withUltimateBase: ultimateBaseExpression]].
	groupBy isNil ifFalse: [groupBy := groupBy asGlorpExpressionOn: base withUltimateBase: ultimateBaseExpression].
"	fields := fields collect: [:each | each isGlorpExpression 
		ifTrue: [each asGlorpExpressionOn: base withUltimateBase: ultimateBaseExpression]
		ifFalse: [each]]."
	fields := OrderedCollection new.!

builders

	^builders.!

command: aDatabaseCommand
	"Explicitly set the database command we will use. This is normally used in order to provide a SQLStringSelectCommand, and specify the sql for the query."
	customCommand := aDatabaseCommand.!

computeFields
	builders 
		do: [:each | self computeFieldsFor: each]!

computeFieldsFor: anElementBuilder 
	| translatedFields |
	translatedFields := self 
		addFields: anElementBuilder fieldsForSelectStatement
		returningTranslationForFields: anElementBuilder fieldsFromMyPerspective
		distinct: anElementBuilder requiresDistinct.
	anElementBuilder fieldTranslations: translatedFields.!

computeTablesToPrint
	| allTables stoppingPoint |
	allTables := Set new.
	fields do: [:each | allTables addAll: each tablesToPrint].
	stoppingPoint := base.
	allTables
		addAll:
			(stoppingPoint isNil
				ifTrue: [whereClause allTablesToPrint]
				ifFalse: [whereClause allTablesToPrintStoppingAt: stoppingPoint]).
	joins
		do:
			[:eachJoin | 
			allTables
				addAll:
					(stoppingPoint isNil
						ifTrue: [eachJoin allTablesToPrint]
						ifFalse: [eachJoin allTablesToPrintStoppingAt: stoppingPoint])].
	ordering isNil
		ifFalse: [ordering do: [:each | 
			each field isNil ifFalse: [allTables add: each field table]]].
	(groupBy notNil and: [groupBy field notNil])
		ifTrue: [allTables add: groupBy field table].
	self tracing allTracings
		do: [:each | allTables addAll: each allTablesToPrint].
	^allTables asSortedCollection.!

copyOfBuilders 
	"Copy the builders, but make sure that subBuilders also get matched up"
	| newBuilders |
	newBuilders := builders collect: [:each | each copy].
	newBuilders do: [:each | each reconcileWithBuilders: builders andCopy: newBuilders].
	^newBuilders.!

elementBuilderFor: anExpression 
	^builders detect: [:each | each expression == anExpression] ifNone: [nil].!

fields
	fields isNil ifTrue: [self prepare].
	^fields!

findPositionofAliasedField: aField
	1 to: fields size do: [:i |
		| existingField |
		existingField := fields at: i.
		existingField asField isNil
			ifTrue: [
				existingField isSameAliasedFieldAs: aField]
			ifFalse: [		
				(existingField asField isSameAliasedFieldAs: aField)
			ifTrue: [^i]]].
	^0.!

fixJoins
	| pseudoJoins realJoins |
	pseudoJoins := joins select: [:each | each tablesForANSIJoin size < 2].
	pseudoJoins do: [:each | whereClause := each AND: whereClause].
	realJoins :=  joins select: [:each| each tablesForANSIJoin size >= 2].
	joins := realJoins.!

hasEmptyWhereClause
	"If we have regular where clause entries, or if we have joins that aren't going to be printed in the tables portion, then we're not empty"
	whereClause willPrintAsWhereClause ifTrue: [^false].
	self useANSIJoins ifTrue: [^true].
	^joins isEmpty.!

hasTracing
	"Return true if we've given this query a tracing already"
	^builders notNil.!

initResultClass: aClass whereClause: anExpression singleObject: aBoolean 
	super 
		initResultClass: aClass
		whereClause: anExpression
		singleObject: aBoolean.
	prepared := false.
	fields := OrderedCollection new.
	joins := OrderedCollection new.!

isPrepared
	^prepared.!

joins
	^joins.!

mightHaveDuplicateRows

	^builders anySatisfy: [:each | each canCauseDuplicateRows].!

postCopy
	super postCopy.
	fields := OrderedCollection new.
	joins := OrderedCollection new.
	builders := nil.
	tablesToPrint := nil.!

prepare
	prepared ifTrue: [^self].
	self setupTracing.
	whereClause prepareIn: self.
	self assignTableAliases.
	self fixJoins.
	self computeFields.
	prepared := true.!

printANSITablesOn: aCommand
	"Print ourselves using the JOIN... USING syntax. Note that we have to put the joins in the right order because we're not allowed to refer to tables not mentioned yet. Great syntax. Reminds me of Pascal. And so easy to deal with."

	| printer |
	printer := JoinPrinter for: self.
	printer printJoinsOn: aCommand.!

printJoinsOn: aCommand
	| noLeadIn|
	self platform supportsANSIJoins ifTrue: [^self].
	joins isEmpty ifTrue: [^self].
	noLeadIn := whereClause isEmptyExpression.
	noLeadIn ifFalse: [aCommand nextPutAll: ' AND ('].
	GlorpHelper 
			do: [:each | each printSQLOn: aCommand withParameters: aCommand parameters]
			for: joins
			separatedBy: [aCommand nextPutAll: ' AND '].
	noLeadIn ifFalse: [aCommand nextPut: $)].!

printNormalTablesOn: aCommand
	self printNormalTablesOn: aCommand excluding: #().!

printNormalTablesOn: aCommand excluding: aCollection
	aCollection do: [:each | self tablesToPrint remove: each].
	GlorpHelper
		print: [:table | table sqlTableName]
		on: aCommand
		for: self tablesToPrint
		separatedBy: ', '.
	^self tablesToPrint.!

printOffsetOn: aCommand
	self hasOffset ifFalse: [^self].
	self platform printOffset: self offset on: aCommand.!

printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' for '.
	aStream nextPutAll: (resultClass isNil ifTrue: ['nil'] ifFalse: [resultClass name]).!

printPostLimitOn: aCommand
	self hasLimit ifFalse: [^self].
	self platform printPostLimit: self limit on: aCommand.!

printPreLimitOn: aCommand
	self hasLimit ifFalse: [^self].
	self platform printPreLimit: self limit on: aCommand.!

printSelectFields: aCollection on: stream 
	| fieldsToPrint |
	(aCollection anySatisfy: [:each | each isDistinct]) 
		ifTrue: [
			fieldsToPrint := aCollection collect: [:each | each asField].
			stream nextPutAll: 'DISTINCT ']
		ifFalse: [fieldsToPrint := aCollection].

	"For SQL Server, the TOP has to come after the distinct, but before the fields"
	self printPreLimitOn: stream.
			
	GlorpHelper 
		do: [:field | field printSelectSQLOn: stream withParameters: nil. ]
		for: fieldsToPrint
		separatedBy: [stream nextPutAll: ', '].!

printSelectFieldsOn: stream 
     self printSelectFields: fields on: stream!

printTablesOn: aCommand
	aCommand
		cr;
		nextPutAll: ' FROM '.
	self useANSIJoins
		ifTrue: [self printANSITablesOn: aCommand]
		ifFalse: [self printNormalTablesOn: aCommand].!

printWhereClauseOn: aCommand
	self hasEmptyWhereClause
		ifFalse:
			[aCommand
				cr;
				nextPutAll: ' WHERE '.
			whereClause printSQLOn: aCommand withParameters: aCommand parameters].!

privateAddBuilder: anElementBuilder

	builders := builders copyWith: anElementBuilder.!

privateReplaceBuilderWith: anElementBuilder

	| oldBuilder |
	oldBuilder := self elementBuilderFor: anElementBuilder expression.
	oldBuilder isNil ifTrue: [self error: 'No builder found to replace.'].
	builders at: (builders indexOf: oldBuilder) put: anElementBuilder.!

privateSetFields: aCollection
	fields := aCollection.!

queries
	"A hack so you can write things that manipulate both this and compound queries the same way"
	^Array with: self.!

rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression
	"Rebuild the query, which is part of a subselect expression, into the new context. Don't do it if it's not necessary. But if it is necessary, also the copy the query. Otherwise two mappings using the same subselect query will mess each other up."
	self == anExpression ifTrue: [^aBaseExpression].
	self baseExpression ultimateBaseExpression == ultimateBaseExpression ifTrue: [^self].
	^self shallowCopy basicRebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression.!

requiresFullQuery
	| descriptor independent |
	descriptor := self descriptor.
	"This isn't even a valid query."
	descriptor isNil ifTrue: [^false].
	independent := descriptor classesRequiringIndependentQueries.
	^(independent size ~= 1) or: [(independent at: 1) ~= descriptor describedClass].!

rowsFromDatabaseWithParameters: anArray

	| alreadyTriedAction command |
	(self shortCircuitEmptyReturn: anArray) ifTrue: [^session accessor cursorFor: (ReadStream on: #()) command: (self sqlWith: anArray)].
	alreadyTriedAction := false.
	command := self sqlWith: anArray.
	^[session accessor
		executeCommand: command
		returnCursor: true] 
	on: GlorpDatabaseReadError 
	do: [:ex | 
		alreadyTriedAction ifTrue: [ex pass] ifFalse: [alreadyTriedAction := true].
		session errorAction isNil 
			ifTrue: [ex pass] 
			ifFalse: [
				session errorAction value: ex value: session value: self.	
				ex retry]].!

setUpExpressions
	super setUpExpressions.
	self validateWhereClause.!

setupTracing
	| |
	builders isNil ifFalse: [^self].	"Already been done"
	super setupTracing.
	builders := tracing allTracings asArray
		inject: #()
		into:
			[:sum :each | sum , (ElementBuilder for: each in: self) withAllSubBuilders].
	self addOrderingForToManyJoins.!

signature
	session useBinding ifFalse: [^''].
	^self sqlWith: Dictionary new.!

sql: aString
	"Specify the sql string to use for this query, instead of having it be generated. Note that doing this will override the normal sql generation mechanisms."

	self command: ((Smalltalk at: #SQLStringSelectCommand) new setSQLString: aString)!

sqlString
	"Mostly for debugging"
	^(self sqlWith: #()) sqlString.!

tablePrefix
	^tablePrefix!

tablePrefix: aString
	tablePrefix := aString.!

tablesToPrint
	tablesToPrint isNil ifTrue: [tablesToPrint := self computeTablesToPrint].
	^tablesToPrint.!

tablesToPrint: aCollection
	tablesToPrint := aCollection.!

traceExpressionInContextFor: anExpression
	^anExpression rebuildOn: base.!

useANSIJoins
	^self session platform supportsANSIJoins.!

where: anExpression
	whereClause := anExpression.
	self setUpBase.! !
!SimpleQuery categoriesFor: #addAllOrderingsForBuilder:!preparing!private! !
!SimpleQuery categoriesFor: #addFields:returningTranslationForFields:distinct:!fields!private! !
!SimpleQuery categoriesFor: #addJoin:!preparing!private! !
!SimpleQuery categoriesFor: #addOrderingForToManyJoins!preparing!private! !
!SimpleQuery categoriesFor: #addSingleOrderingForBuilder:!preparing!private! !
!SimpleQuery categoriesFor: #adjustWhereClause!preparing!private! !
!SimpleQuery categoriesFor: #asFullQuery!converting!private! !
!SimpleQuery categoriesFor: #asGlorpExpression!converting!private! !
!SimpleQuery categoriesFor: #asGlorpExpressionOn:basedOn:withUltimateBase:!converting!private! !
!SimpleQuery categoriesFor: #assignTableAliases!preparing!public! !
!SimpleQuery categoriesFor: #basicRebuildOn:startingFrom:withOuterScopeBase:!converting!private! !
!SimpleQuery categoriesFor: #builders!accessing!private! !
!SimpleQuery categoriesFor: #command:!accessing!private! !
!SimpleQuery categoriesFor: #computeFields!executing!private! !
!SimpleQuery categoriesFor: #computeFieldsFor:!executing!public! !
!SimpleQuery categoriesFor: #computeTablesToPrint!private!sql generation! !
!SimpleQuery categoriesFor: #copyOfBuilders!preparing!private! !
!SimpleQuery categoriesFor: #elementBuilderFor:!accessing!private! !
!SimpleQuery categoriesFor: #fields!accessing!private! !
!SimpleQuery categoriesFor: #findPositionofAliasedField:!fields!public! !
!SimpleQuery categoriesFor: #fixJoins!preparing!private! !
!SimpleQuery categoriesFor: #hasEmptyWhereClause!private!testing! !
!SimpleQuery categoriesFor: #hasTracing!private!testing! !
!SimpleQuery categoriesFor: #initResultClass:whereClause:singleObject:!initialize!private! !
!SimpleQuery categoriesFor: #isPrepared!preparing!private! !
!SimpleQuery categoriesFor: #joins!accessing!private! !
!SimpleQuery categoriesFor: #mightHaveDuplicateRows!private!testing! !
!SimpleQuery categoriesFor: #postCopy!copying!private! !
!SimpleQuery categoriesFor: #prepare!preparing!private! !
!SimpleQuery categoriesFor: #printANSITablesOn:!private!sql generation! !
!SimpleQuery categoriesFor: #printJoinsOn:!private!sql generation! !
!SimpleQuery categoriesFor: #printNormalTablesOn:!private!sql generation! !
!SimpleQuery categoriesFor: #printNormalTablesOn:excluding:!private!sql generation! !
!SimpleQuery categoriesFor: #printOffsetOn:!private!sql generation! !
!SimpleQuery categoriesFor: #printOn:!printing!private! !
!SimpleQuery categoriesFor: #printPostLimitOn:!private!sql generation! !
!SimpleQuery categoriesFor: #printPreLimitOn:!private!sql generation! !
!SimpleQuery categoriesFor: #printSelectFields:on:!private!sql generation! !
!SimpleQuery categoriesFor: #printSelectFieldsOn:!private!sql generation! !
!SimpleQuery categoriesFor: #printTablesOn:!private!sql generation! !
!SimpleQuery categoriesFor: #printWhereClauseOn:!private!sql generation! !
!SimpleQuery categoriesFor: #privateAddBuilder:!private! !
!SimpleQuery categoriesFor: #privateReplaceBuilderWith:!private! !
!SimpleQuery categoriesFor: #privateSetFields:!private! !
!SimpleQuery categoriesFor: #queries!As yet unclassified!private! !
!SimpleQuery categoriesFor: #rebuildOn:startingFrom:withOuterScopeBase:!converting!public! !
!SimpleQuery categoriesFor: #requiresFullQuery!private!testing! !
!SimpleQuery categoriesFor: #rowsFromDatabaseWithParameters:!executing!private! !
!SimpleQuery categoriesFor: #setUpExpressions!executing!private! !
!SimpleQuery categoriesFor: #setupTracing!preparing!private! !
!SimpleQuery categoriesFor: #signature!private!sql generation! !
!SimpleQuery categoriesFor: #sql:!accessing!public! !
!SimpleQuery categoriesFor: #sqlString!private!sql generation! !
!SimpleQuery categoriesFor: #tablePrefix!accessing!private! !
!SimpleQuery categoriesFor: #tablePrefix:!accessing!private! !
!SimpleQuery categoriesFor: #tablesToPrint!public!sql generation! !
!SimpleQuery categoriesFor: #tablesToPrint:!public!sql generation! !
!SimpleQuery categoriesFor: #traceExpressionInContextFor:!preparing!private! !
!SimpleQuery categoriesFor: #useANSIJoins!private!testing! !
!SimpleQuery categoriesFor: #where:!accessing!private! !

SQLite3PragmaQuery guid: (GUID fromString: '{2C0914AF-6D6A-40DB-B82F-A297969354F2}')!
SQLite3PragmaQuery comment: 'This is a query intended to execute a SQLite3 PRAGMA command. The command behaves like a stored function that takes a parameter and returns a result set. The meta information schema for SQLite is not available in the standard views, but it can be eventually deduced by executing particular pragma commands.
'!
!SQLite3PragmaQuery categoriesForClass!Unclassified! !
!SQLite3PragmaQuery methodsFor!

printWhereClauseOn: aCommand

	self hasEmptyWhereClause
		ifFalse: 
			[whereClause rightChild printSQLOn: aCommand withParameters: aCommand parameters]!

sqlWith: aDictionary
	| result |
	self prepare.
	customCommand isNil ifFalse: [
		result := customCommand copy.
		result query: self; parameters: aDictionary; useBinding: session useBinding; session: session.
		^result].
	^(Smalltalk at: #SQLite3PragmaCommand)
		forQuery: self
		parameters: aDictionary
		useBinding: session useBinding
		session: session.! !
!SQLite3PragmaQuery categoriesFor: #printWhereClauseOn:!public!sql generation! !
!SQLite3PragmaQuery categoriesFor: #sqlWith:!public!sql generation! !

!SQLite3PragmaQuery class methodsFor!

newQuery
	^SQLite3PragmaQuery new
	"^super newQuery"!

returningManyOf: aClass where: anExpression
	"Backward-compatibility, since we changed the class name."
	^self returningManyOf: aClass where: anExpression limit: nil! !
!SQLite3PragmaQuery class categoriesFor: #newQuery!instance creation!public! !
!SQLite3PragmaQuery class categoriesFor: #returningManyOf:where:!instance creation!public! !

GlorpCursoredStream guid: (GUID fromString: '{462FDC0D-B50C-4C51-8D19-1FF43F1BFCD5}')!
GlorpCursoredStream comment: 'This is a Glorp wrapper for a database cursor. It has stream-like behaviour, which is implemented by getting rows from the underlying cursor and constructing the objects as required. All queries go through one of these streams, although in the default case they will get all the results out of it (at which point the cursor will be closed) and then return them in another kind of collection. But it''s possible to use one of these as the return type, in which case you can make use of the open cursor to incrementally get results.

Instance Variables:
	builders	<(Collection of: ObjectBuilder)>	The ObjectBuilders used to construct the objects we return out of the rows.
	buildersThatReturnResults	<(SequenceableCollection of: ElementBuilder)>	
	command	<DatabaseCommand>	The command that was executed to give us this cursor.
	cursor	<>	The underlying database cursor. Class varies by dialect. We deal with the lack of polymorphism by indirecting all cursor operations through the accessor.
	databaseAccessor	<DatabaseAccessor>	The accessor that the command was executed against.
	mightHaveDuplicateRows	<Boolean>	May this stream have duplicate rows? This can happen, for example, if we do a join that causes some sub-section of the row to be duplicated. If so, we need to check for this and remove duplicates from the object stream as we go. This forces us to peek ahead.
	peekBuffer	<Object>	The peek buffer (holding an object) that we use when peeking ahead.
	rawPeekBuffer	<>	The peek buffer (holding a row) that we use when peeking ahead. We need to buffer at both levels.
	statement	<GlorpPreparedStatement>	The actual prepared statement that the command ran in order to get our results.

'!
!GlorpCursoredStream categoriesForClass!Unclassified! !
!GlorpCursoredStream methodsFor!

accessor
	^databaseAccessor.!

buildersThatReturnResults
	buildersThatReturnResults isNil ifTrue: [
		buildersThatReturnResults := OrderedCollection new.
		builders
			do: [:each | 	
				| expression shouldBeRetrieved |
				"For some bizarre reason, includes: fails on ObjectStudio for this"
				expression := each expression.
				shouldBeRetrieved := (self query tracing retrievalExpressions detect: [:foo |
					foo == expression] ifNone: [nil]) notNil.
				(shouldBeRetrieved and: [
					(buildersThatReturnResults anySatisfy: [:eachBuilder | eachBuilder expression == expression]) not])
						ifTrue: [buildersThatReturnResults add: each]].
		buildersThatReturnResults := buildersThatReturnResults asArray].
	^buildersThatReturnResults.!

buildObjectsForRow: aRow
	self query isNil ifTrue: [^aRow].
	builders
		do:
			[:each | each findInstanceForRow: aRow proxyType: self query proxyType].
	builders do: [:each | each buildObjectFrom: aRow].
	builders
		do:
			[:each | 
			each
				sendPostFetchValidateIn: self session 
				OnFailureDo: [each instance: nil]].
	builders
		do: [:each | each instance == nil ifFalse: [each knitResultIn: self]].
	builders do: [:each | self session sendPostFetchEventTo: each instance].
	^self buildersThatReturnResults size = 1
		ifTrue: [self buildersThatReturnResults first returnValueIn: self]
		ifFalse:
			[self buildersThatReturnResults collect: [:each | each returnValueIn: self]].!

collect: aBlock 
	| newStream |
	newStream := WriteStream on: (Array new: 100).
	[self atEnd] whileFalse: [newStream nextPut: (aBlock value: self next)].
	^newStream contents!

collectionTypeFor: aCollectionType
	"Make sure that we can reasonably return aCollectionType. If not, default to something we can"

	aCollectionType class == GlorpDictionaryDescription ifFalse: [^aCollectionType].
	self buildersThatReturnResults size = 1 ifTrue: [^aCollectionType].
	^Array.!

command
	^command!

command: aDatabaseCommand
	command := aDatabaseCommand.
	aDatabaseCommand isNil ifTrue: [^self].
	aDatabaseCommand query isNil ifTrue: [^self].
	databaseAccessor := aDatabaseCommand query session accessor.
	builders := self query copyOfBuilders.
	builders do: [:each | each cursor: self].!

contents
	self shouldNotImplement.!

cursor
	^cursor!

cursor: aStream
	cursor := aStream!

databaseAccessor: aDatabaseAccessor
	databaseAccessor := aDatabaseAccessor!

detect: aBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument.
	Answer the first element for which aBlock evaluates to true."

	^self detect: aBlock ifNone: [self error: 'not found']!

detect: aBlock ifNone: exceptionBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument.
	Answer the first element for which aBlock evaluates to true."

	self do: [:each | (aBlock value: each) ifTrue: [^each]].
	^exceptionBlock value!

do: aBlock 
	"Evaluate aBlock for each of the elements of the receiver."

	[self atEnd]
		whileFalse: [aBlock value: self next]!

elementBuilderFor: anExpression 
	^builders detect: [:each | each expression == anExpression] ifNone: [nil].!

flush
	^self.!

hasCompleteAnswerFor: objects
	| nextAnswer |
	self atEnd ifTrue: [^true].
	self mightHaveDuplicateRows ifFalse: [^true].
	nextAnswer := self peek.
	^objects ~= nextAnswer!

initialize!

isReadable
	^true.!

isWritable
	^false.!

mightHaveDuplicateRows
	mightHaveDuplicateRows isNil ifTrue: [mightHaveDuplicateRows := self query isNil ifTrue: [false] ifFalse: [self query mightHaveDuplicateRows]].
	^mightHaveDuplicateRows.!

newCollectionOfSize: aSize for: aCollectionType
	"Make sure that we can reasonably return aCollectionType. If not, default to something we can"
	| theType |
	theType := self collectionTypeFor: aCollectionType.
	^theType isBehavior ifTrue: [theType new: aSize] ifFalse: [theType copyEmpty: aSize].!

next
	| answer session |
	self atEnd ifTrue: [^nil].
	session := self session.

	[self atEnd ifTrue: [^nil].
	answer := self nextAnswer.
	answer == nil] 
		whileTrue.

	[self hasCompleteAnswerFor: answer] whileFalse: [self nextAnswer].
	session == nil ifFalse: [session privateRegisterAsOld: answer].
	^answer.!

next: anInteger
	| result |
	Dialect isObjectStudio ifFalse: [^super next: anInteger].
	result := OrderedCollection new: anInteger.
	anInteger timesRepeat: [result add: self next].
	^result.!

nextAnswer
	| result |
	peekBuffer isNil ifFalse: [
		result := peekBuffer.
		peekBuffer := nil.
		^result].
	^self nextAnswerFromUnderlyingCursor.!

nextAnswerFromUnderlyingCursor
	| objects |
	rawPeekBuffer isNil ifTrue: [rawPeekBuffer := cursor next].
	rawPeekBuffer isNil ifTrue: [^nil].
	objects := self buildObjectsForRow: rawPeekBuffer.
	rawPeekBuffer := nil.
	^objects.!

nextObjectFrom: aRow
	| objects |
	objects := self buildObjectsForRow: aRow.
	self mightHaveDuplicateRows ifFalse: [
		self registerInstances.
		^objects].!

nextPut: anObject
	self shouldNotImplement.!

objectBuilderFor: anExpression 
	"Return the builder for this expression, but only those that directly build compound objects. Most importantly, exclude dictionary builders"
	^builders detect: [:each | 
		each buildsObjects and: [each expression == anExpression]] ifNone: [nil].!

peek
	peekBuffer = nil ifFalse: [^peekBuffer].
	peekBuffer := self nextAnswer.
	^peekBuffer.!

privateGetBuilders
	^builders.!

query
	command isNil ifTrue: [^nil].
	^command query.!

query: x!

registerInstances
	self session isNil ifTrue: [^self].
	builders do: [:each | self session register: each instance].!

release
	super release.
	self releaseCursor.
	cursor := nil.
	rawPeekBuffer := nil.
	peekBuffer := nil.
	statement isNil
		ifFalse:
			[databaseAccessor reusePreparedStatements
				ifTrue: [statement busy: false]
				ifFalse: [statement release].
			statement := nil].
	databaseAccessor := nil.!

releaseCursor
	
	self accessor isNil
		ifFalse:
			[self accessor releaseCursor: cursor in: statement.
			self accessor completedCommand: command].!

rows
	"This will extract all the rows and replace the internal stream with a regular stream on the rows. This is useful for debugging, but in real use rather destroys the purpose of the cursored stream"
	| rows |
	rows := cursor upToEnd.
	self releaseCursor.
	cursor := rows readStream.
	^rows.!

session
	^self query isNil ifTrue: [nil] ifFalse: [self query session].!

statement
	^statement.!

statement: aGlorpPreparedStatement
	statement := aGlorpPreparedStatement.!

upToEnd
	| result |
	result := WriteStream on: (Array new: 100).
	[self atEnd] whileFalse: [
			| nextValue |
			"Stupid VA's cursors don't deal with atEnd or isEmpty nicely, so check for nil"
			nextValue := self next.
			nextValue == nil ifFalse: [	result nextPut: nextValue]].
	^result contents! !
!GlorpCursoredStream categoriesFor: #accessor!private!testing! !
!GlorpCursoredStream categoriesFor: #buildersThatReturnResults!building objects!private! !
!GlorpCursoredStream categoriesFor: #buildObjectsForRow:!building objects!private! !
!GlorpCursoredStream categoriesFor: #collect:!iterating!private! !
!GlorpCursoredStream categoriesFor: #collectionTypeFor:!building objects!private! !
!GlorpCursoredStream categoriesFor: #command!accessing!private! !
!GlorpCursoredStream categoriesFor: #command:!accessing!private! !
!GlorpCursoredStream categoriesFor: #contents!accessing!private! !
!GlorpCursoredStream categoriesFor: #cursor!accessing!private! !
!GlorpCursoredStream categoriesFor: #cursor:!accessing!private! !
!GlorpCursoredStream categoriesFor: #databaseAccessor:!accessing!private! !
!GlorpCursoredStream categoriesFor: #detect:!iterating!private! !
!GlorpCursoredStream categoriesFor: #detect:ifNone:!iterating!private! !
!GlorpCursoredStream categoriesFor: #do:!iterating!private! !
!GlorpCursoredStream categoriesFor: #elementBuilderFor:!building objects!private! !
!GlorpCursoredStream categoriesFor: #flush!accessing!private! !
!GlorpCursoredStream categoriesFor: #hasCompleteAnswerFor:!building objects!private! !
!GlorpCursoredStream categoriesFor: #initialize!initializing!private! !
!GlorpCursoredStream categoriesFor: #isReadable!private!testing! !
!GlorpCursoredStream categoriesFor: #isWritable!private!testing! !
!GlorpCursoredStream categoriesFor: #mightHaveDuplicateRows!accessing!private! !
!GlorpCursoredStream categoriesFor: #newCollectionOfSize:for:!building objects!private! !
!GlorpCursoredStream categoriesFor: #next!accessing!private! !
!GlorpCursoredStream categoriesFor: #next:!accessing!private! !
!GlorpCursoredStream categoriesFor: #nextAnswer!building objects!private! !
!GlorpCursoredStream categoriesFor: #nextAnswerFromUnderlyingCursor!building objects!private! !
!GlorpCursoredStream categoriesFor: #nextObjectFrom:!building objects!private! !
!GlorpCursoredStream categoriesFor: #nextPut:!accessing!private! !
!GlorpCursoredStream categoriesFor: #objectBuilderFor:!building objects!private! !
!GlorpCursoredStream categoriesFor: #peek!accessing!private! !
!GlorpCursoredStream categoriesFor: #privateGetBuilders!accessing!private! !
!GlorpCursoredStream categoriesFor: #query!accessing!private! !
!GlorpCursoredStream categoriesFor: #query:!accessing!private! !
!GlorpCursoredStream categoriesFor: #registerInstances!building objects!private! !
!GlorpCursoredStream categoriesFor: #release!initializing!private! !
!GlorpCursoredStream categoriesFor: #releaseCursor!initializing!private! !
!GlorpCursoredStream categoriesFor: #rows!debugging!private! !
!GlorpCursoredStream categoriesFor: #session!accessing!private! !
!GlorpCursoredStream categoriesFor: #statement!accessing!private! !
!GlorpCursoredStream categoriesFor: #statement:!accessing!private! !
!GlorpCursoredStream categoriesFor: #upToEnd!accessing!private! !

!GlorpCursoredStream class methodsFor!

on: aStream
	^self basicNew cursor: aStream.!

on: aStream accessor: aDatabaseAccessor
	^(self on: aStream)
		databaseAccessor: aDatabaseAccessor.!

on: aStream for: aCommand
	^(self on: aStream)
		command: aCommand.!

on: aStream for: aCommand accessor: aDatabaseAccessor
	^(self on: aStream)
		command: aCommand;
		databaseAccessor: aDatabaseAccessor.! !
!GlorpCursoredStream class categoriesFor: #on:!instance creation!private! !
!GlorpCursoredStream class categoriesFor: #on:accessor:!instance creation!private! !
!GlorpCursoredStream class categoriesFor: #on:for:!instance creation!private! !
!GlorpCursoredStream class categoriesFor: #on:for:accessor:!instance creation!private! !

AbstractProxy guid: (GUID fromString: '{09D0F93B-9C84-4CC4-89F9-BA3591FA29ED}')!
AbstractProxy comment: 'This is an abstract superclass for proxy objects. It inherits from ProtoObject, which is part of the platform specific layer, but implements most of the proxy functionality.

Instance Variables:
	isInstantiated	<Boolean>	Has this proxy been instantiated or not.
	value	<Object>	What we are proxying for.

'!
!AbstractProxy categoriesForClass!Unclassified! !
!AbstractProxy methodsFor!

~= anObject 

	^(self = anObject) not!

= anObject
	"In VA we inherit this as identity. Need to override"
	^anObject = self getValue.!

basicPrintOn: aStream 
	aStream nextPut: ${.
	isInstantiated 
		ifTrue: [self getValue basicPrintOn: aStream]
		ifFalse: [aStream nextPutAll: 'uninstantiated proxy'].
	aStream nextPut: $}!

basicPrintString
	| aStream |
	aStream := WriteStream on: String new.
	self basicPrintOn: aStream.
	^aStream contents!

basicSize
	^self getValue basicSize.!

doesNotUnderstand: aMessage

	^self getValue perform: aMessage selector withArguments: aMessage arguments.!

doesNotUnderstand: sel args: args 
	"This is the variant for ObjectStudio"
	^self getValue perform: sel withArguments: args.!

getValue

	self subclassesMustImplement!

glorpPostFetch: aSession!

glorpPostFetchValidate: aSession!

hash
	^self getValue hash.!

inspect
	^self getValue inspect.!

isGlorpProxy
	^true.!

isImmutable
	^self getValue isImmutable.!

isInstantiated
	isInstantiated isNil ifTrue: [isInstantiated := false].
	^isInstantiated!

isKindOf: aClass
	self isInstantiated ifTrue: [^value isKindOf: aClass].
	^false!

printOn: aStream 
	aStream nextPut: ${.
	isInstantiated 
		ifTrue: [self getValue printOn: aStream]
		ifFalse: [aStream nextPutAll: 'uninstantiated proxy'].
	aStream nextPut: $}!

printString

	| aStream |
	aStream := WriteStream on: (String new: 16).
	self printOn: aStream.
	^aStream contents!

proxyInitialize

	isInstantiated := false.!

store
	"For Object Studio compatibility"
	^self inspectorString.!

uninstantiate

	value := nil.
	isInstantiated := false.! !
!AbstractProxy categoriesFor: #~=!accessing!private! !
!AbstractProxy categoriesFor: #=!accessing!private! !
!AbstractProxy categoriesFor: #basicPrintOn:!printing!private! !
!AbstractProxy categoriesFor: #basicPrintString!printing!private! !
!AbstractProxy categoriesFor: #basicSize!overrides!private! !
!AbstractProxy categoriesFor: #doesNotUnderstand:!dnu handling!private! !
!AbstractProxy categoriesFor: #doesNotUnderstand:args:!dnu handling!private! !
!AbstractProxy categoriesFor: #getValue!api!private! !
!AbstractProxy categoriesFor: #glorpPostFetch:!notification!private! !
!AbstractProxy categoriesFor: #glorpPostFetchValidate:!notification!private! !
!AbstractProxy categoriesFor: #hash!overrides!private! !
!AbstractProxy categoriesFor: #inspect!overrides!private! !
!AbstractProxy categoriesFor: #isGlorpProxy!private!testing! !
!AbstractProxy categoriesFor: #isImmutable!overrides!private! !
!AbstractProxy categoriesFor: #isInstantiated!private!testing! !
!AbstractProxy categoriesFor: #isKindOf:!accessing!private! !
!AbstractProxy categoriesFor: #printOn:!printing!private! !
!AbstractProxy categoriesFor: #printString!printing!private! !
!AbstractProxy categoriesFor: #proxyInitialize!initialize!private! !
!AbstractProxy categoriesFor: #store!overrides!private! !
!AbstractProxy categoriesFor: #uninstantiate!accessing!private! !

!AbstractProxy class methodsFor!

new

	^super new proxyInitialize.! !
!AbstractProxy class categoriesFor: #new!instance creation!private! !

MessageArchiver guid: (GUID fromString: '{F86CB56D-006B-4237-BCAA-B502F3E4B103}')!
MessageArchiver comment: 'This is what we use to construct GlorpExpression trees out of blocks. An instance of MessageArchiver is passed into the block. It has a doesNotUnderstand: handler, and in response to almost any message, will just return a new MessageArchiver which knows it is descended from this one. The result (assuming no inlined messages or other tricky stuff happened) is a tree of MessageArchivers recording all the sent messages. We can then traverse that and construct the appropriate expression tree.

Instance Variables:
	myArguments	<(Collection of: (Object | ProtoObject))>	The arguments to the message that caused me to be constructed.
	myReceiver	<MessageArchiver | nil>	The object that received the message that caused me to be constructed. For the initial MessageArchiver that''s passed in, this will be nil. Otherwise it is a MessageArchiver.
	mySelector	<Symbol>	The selector of the message that caused me to be constructed.
	query	<AbstractReadQuery>	The query for which I am constructing the expression. This is here so that we can construct the final expression tree directly onto its BaseExpression.

'!
!MessageArchiver categoriesForClass!Unclassified! !
!MessageArchiver methodsFor!

= anObject
	"Needed because VA's abtObservableWrapper implements =. Should be portable."
	^MessageArchiver 
		receiver: self
		selector: #= 
		arguments: (Array with: anObject).!

asGlorpExpression
	^self asGlorpExpressionOn: BaseExpression new!

asGlorpExpressionOn: aBaseExpression 
	| arguments |
	myReceiver == nil ifTrue: [^query isNil ifTrue: [aBaseExpression] ifFalse: [query baseExpression]].
	arguments := myArguments collect: [:each | each asGlorpExpressionOn: aBaseExpression].
	^self sendMessageTo: (myReceiver asGlorpExpressionOn: aBaseExpression) withArguments: arguments.!

asGlorpExpressionOn: aBaseExpression basedOn: aMessageArchiver withUltimateBase: ultimateBaseExpression
	"We know we are built from aMessageArchiver, but parts of us may refer to an outer scope. Those parts should be built on ultimateBaseExpression rather than aBaseExpression. e.g. consider 
 [:each | each amount amount = ((Query readManyOf: GlorpBankTransaction
			where: [:eachTrans | eachTrans owner = each owner]) 
				retrieve: [:eachTrans | eachTrans amount amount max])]."

	| arguments |
	self == aMessageArchiver ifTrue: [^aBaseExpression].
	myReceiver == nil ifTrue: [^query isNil ifTrue: [ultimateBaseExpression] ifFalse: [query baseExpression]].
	arguments := myArguments
		collect:
			[:each | 
			each
				asGlorpExpressionOn: aBaseExpression
				basedOn: aMessageArchiver
				withUltimateBase: ultimateBaseExpression].
	^self 
		sendMessageTo: (myReceiver
			asGlorpExpressionOn: aBaseExpression
			basedOn: aMessageArchiver
			withUltimateBase: ultimateBaseExpression)
		withArguments: arguments.!

asText
	^self basicPrintString asText.!

basicDoesNotUnderstand: aMessage 
	"Invoke this to avoid infinite recursion in the case of internal errors. We want a dialect-independent way of getting a walkback window, so we'll invoke it against a different object"


	(Array with: self) doesNotUnderstand: aMessage.!

basicDoesNotUnderstand: sel args: args 
	"Invoke this to avoid infinite recursion in the case of internal errors. We want a dialect-independent way of getting a walkback window, so we'll invoke it against a different object"

	(Array with: self) doesNotUnderstand: sel args: args.!

basicPrintString

	^self printString.!

class
	^MessageArchiver!

doesNotUnderstand: aMessage 
	| sel |
	sel := aMessage selector.
	sel == #doesNotUnderstand: ifTrue: [self basicDoesNotUnderstand: aMessage].
	(sel size >= 8 and: [(sel copyFrom: 1 to: 8) = 'perform:']) 
		ifTrue: 
			[^self get: aMessage arguments first
				withArguments: (aMessage arguments copyFrom: 2 to: aMessage arguments size)].
	^self get: sel withArguments: aMessage arguments.!

doesNotUnderstand: sel args: args 
	"This is the variant for ObjectStudio"
	sel == #doesNotUnderstand: ifTrue: [self basicDoesNotUnderstand: sel args: args].
	(sel size >= 8 and: [(sel copyFrom: 1 to: 8) = 'perform:']) 
		ifTrue: 
			[^self get: args first
				withArguments: (args copyFrom: 2 to: args size)].
	^self get: sel withArguments: args.!

get: aSymbol 
	^MessageArchiver receiver: self
		selector: aSymbol
		arguments: #().!

get: aSymbol withArguments: anArray 
	| arguments |
	arguments := anArray collect: [:each | each glorpIsBlock 
		ifTrue: [(Dialect argumentCountFor: each) = 1
			ifTrue: [each value: self]
			ifFalse: [each value]]
		ifFalse: [each]].
	^MessageArchiver receiver: self
		selector: aSymbol
		arguments: arguments.!

getMapping: aMapping named: aString
	"In this case, the name may actually be a string rather than a symbol."
	^MessageArchiverForGetMapping
		receiver: self
		selector: aString
		arguments: (Array with: aMapping with: aString).!

glorpIsBlock
	^false.!

halt
	"Support this so that we can debug inside query blocks. For portability, send it to a different object so that we don't have to care how halt is implemented"

	(Array with: self) halt.!

inspect

	"Not exactly the intended semantics, but should be portable"
	(Array with: self) inspect.!

inspectorClasses
	^Array with: (Dialect smalltalkAt: 'Tools.Trippy.BasicInspector')!

inspectorSize
  ^2!

mustBeBoolean
	nil error: 'mustBeBoolean - this probably indicates you are trying to use an optimized message like and: or ifTrue: inside a Glorp expression block. Use AND: or & instead.'!

os_Equal: anObject
	^self = anObject!

printOn: aStream

	aStream nextPutAll: self printString.!

printString

	"Hard-code this for maximum dialect portability"
	^'a MessageArchiver'.!

privateGlorpQuery
	^query.!

privateGlorpQuery: aQuery
	query := aQuery.!

privateGlorpReceiver

	^myReceiver!

privateGlorpSelector

	^mySelector!

receiver: aMessageCollector selector: aSymbol arguments: aCollection

	myReceiver := aMessageCollector.
	mySelector := aSymbol.
	myArguments := aCollection.!

sendMessageTo: anExpression withArguments: arguments
	"Build our node by sending our message to anExpression"

	^anExpression get: mySelector withArguments: arguments.! !
!MessageArchiver categoriesFor: #=!doesNotUnderstand!private! !
!MessageArchiver categoriesFor: #asGlorpExpression!expression creation!private! !
!MessageArchiver categoriesFor: #asGlorpExpressionOn:!expression creation!private! !
!MessageArchiver categoriesFor: #asGlorpExpressionOn:basedOn:withUltimateBase:!expression creation!private! !
!MessageArchiver categoriesFor: #asText!debugging!private! !
!MessageArchiver categoriesFor: #basicDoesNotUnderstand:!doesNotUnderstand!private! !
!MessageArchiver categoriesFor: #basicDoesNotUnderstand:args:!doesNotUnderstand!private! !
!MessageArchiver categoriesFor: #basicPrintString!debugging!private! !
!MessageArchiver categoriesFor: #class!debugging!private! !
!MessageArchiver categoriesFor: #doesNotUnderstand:!doesNotUnderstand!private! !
!MessageArchiver categoriesFor: #doesNotUnderstand:args:!doesNotUnderstand!private! !
!MessageArchiver categoriesFor: #get:!expression protocol!private! !
!MessageArchiver categoriesFor: #get:withArguments:!expression protocol!private! !
!MessageArchiver categoriesFor: #getMapping:named:!expression protocol!private! !
!MessageArchiver categoriesFor: #glorpIsBlock!private!testing! !
!MessageArchiver categoriesFor: #halt!debugging!private! !
!MessageArchiver categoriesFor: #inspect!debugging!private! !
!MessageArchiver categoriesFor: #inspectorClasses!As yet unclassified!private! !
!MessageArchiver categoriesFor: #inspectorSize!debugging!private! !
!MessageArchiver categoriesFor: #mustBeBoolean!debugging!private! !
!MessageArchiver categoriesFor: #os_Equal:!doesNotUnderstand!private! !
!MessageArchiver categoriesFor: #printOn:!debugging!private! !
!MessageArchiver categoriesFor: #printString!debugging!private! !
!MessageArchiver categoriesFor: #privateGlorpQuery!private!private/accessing! !
!MessageArchiver categoriesFor: #privateGlorpQuery:!private!private/accessing! !
!MessageArchiver categoriesFor: #privateGlorpReceiver!private!private/accessing! !
!MessageArchiver categoriesFor: #privateGlorpSelector!private!private/accessing! !
!MessageArchiver categoriesFor: #receiver:selector:arguments:!initialize!private! !
!MessageArchiver categoriesFor: #sendMessageTo:withArguments:!expression creation!private! !

!MessageArchiver class methodsFor!

receiver: aMessageCollector selector: aSymbol arguments: aCollection

	^self new
		receiver: aMessageCollector
		selector: aSymbol
		arguments: aCollection.! !
!MessageArchiver class categoriesFor: #receiver:selector:arguments:!instance creation!private! !

AdHocProxy guid: (GUID fromString: '{E16B06C2-C7B4-4225-B544-B37EAC2E8252}')!
AdHocProxy comment: 'This represents a proxy that gets its value in a user-defined way, by evaluating a block.

Instance Variables:
	readBlock	<BlockClosure>	The zero-argument block we evaluate to get the proxied object.

'!
!AdHocProxy categoriesForClass!Unclassified! !
!AdHocProxy methodsFor!

class

	^AdHocProxy!

getValue

	self isInstantiated ifTrue: [^value].
	[value := readBlock value] ensure: [isInstantiated := true].
	^value!

glorpReadBlock: aZeroArgumentBlock
	readBlock := aZeroArgumentBlock!

printOn: aStream 
	aStream nextPut: ${.
	isInstantiated 
		ifTrue: [self getValue printOn: aStream]
		ifFalse: [aStream nextPutAll: 'uninstantiated AdHocProxy'].
	aStream nextPut: $}!

proxyInitialize

	super proxyInitialize.
	readBlock := [nil]! !
!AdHocProxy categoriesFor: #class!accessing!private! !
!AdHocProxy categoriesFor: #getValue!api!private! !
!AdHocProxy categoriesFor: #glorpReadBlock:!api!private! !
!AdHocProxy categoriesFor: #printOn:!printing!private! !
!AdHocProxy categoriesFor: #proxyInitialize!initialize!private! !

!AdHocProxy class methodsFor!

readBlock: aZeroArgumentBlock

	^self new glorpReadBlock: aZeroArgumentBlock! !
!AdHocProxy class categoriesFor: #readBlock:!instance creation!private! !

Proxy guid: (GUID fromString: '{A471C382-5F40-4D68-9318-55E3F7C43F22}')!
Proxy comment: 'This implements the Glorp database proxy behaviour, for proxies that get their result by evaluating a query using a particular set of parameters, in the context of a particular Glorp query.

Instance Variables:
	parameters	<Dictionary | SequenceableCollection>	Our parameters, keyed either by name or by index.
	query	<Query>	The query we will evaluate. Note that for a mapping, all proxies will share the same query instance.
	session	<GlorpSession>	The session in which this proxy exists, and against which the query will be executed to find our value.

'!
!Proxy categoriesForClass!Unclassified! !
!Proxy methodsFor!

class

	^Proxy
	"self isInstantiated ifTrue: [^value class].
	query isNil ifTrue: [^Proxy].
	query resultClass isNil ifTrue: [^Proxy].
	^query resultClass."!

getValue
	self isInstantiated ifTrue: [^value].
	parameters isNil ifTrue: [parameters := Dictionary new: 0].
	[value := query isNil
		ifTrue: [nil]
		ifFalse: [query executeWithParameters: parameters in: session]]
		ensure: [isInstantiated := true].
	self glorpTriggerProxyAction: value.
	^value.!

glorpOwner
	"Return the object for which this proxy was created. Note that this is not kept track of unless we have a proxy action."
	action isNil ifTrue: [^nil].
	^action key.!

glorpOwner: anObject
	"If we don't have an action, the owner is unnecessary. Ignore it. If we do have one, put it in as the key of an association with the action block following it."
	action isNil ifTrue: [^self].
	action key: anObject.!

glorpProxyAction
	"Return the block we expect to evaluate when the proxy is fired."
	action isNil ifTrue: [^nil].
	^action value.!

glorpProxyAction: aTwoArgumentBlock
	"Set the action we'll take when the proxy fires. This is a two argument block that takes the owning object and the value of the proxy as arguments."
	aTwoArgumentBlock isNil ifTrue: [^action := nil].
	action := nil -> aTwoArgumentBlock.!

glorpTriggerProxyAction: target
	"Evaluate our action with the owner and the newly retrieved target."
	| owner block |
	action isNil ifTrue: [^self].
	owner := action key.
	block := action value.
	block value: owner value: target.!

isKindOf: aClass
	self isInstantiated ifTrue: [^value isKindOf: aClass].
	query isNil ifTrue: [^false].
	query resultClass isNil ifTrue: [^false].
	query resultClass isBehavior ifFalse: [^false].
	^query resultClass includesBehavior: aClass.!

parameters
	^parameters!

parameters: aDictionary
	parameters := aDictionary.!

printOn: aStream 
	aStream nextPut: ${.
	isInstantiated 
		ifTrue: [self getValue printOn: aStream]
		ifFalse: 
			[aStream nextPutAll: 'uninstantiated '.
			query isNil
				ifTrue: [aStream nextPutAll: '???']
				ifFalse: [
					query readsOneObject ifFalse: [aStream nextPutAll: 'collection of '].
					aStream nextPutAll: query resultClass name]].
	aStream nextPut: $}!

query
	^query!

query: aQuery 
	query := aQuery!

session
	^session!

session: aSession 
	session := aSession! !
!Proxy categoriesFor: #class!accessing!private! !
!Proxy categoriesFor: #getValue!api!private! !
!Proxy categoriesFor: #glorpOwner!private!proxy actions! !
!Proxy categoriesFor: #glorpOwner:!private!proxy actions! !
!Proxy categoriesFor: #glorpProxyAction!private!proxy actions! !
!Proxy categoriesFor: #glorpProxyAction:!private!proxy actions! !
!Proxy categoriesFor: #glorpTriggerProxyAction:!private!proxy actions! !
!Proxy categoriesFor: #isKindOf:!accessing!private! !
!Proxy categoriesFor: #parameters!accessing!private! !
!Proxy categoriesFor: #parameters:!accessing!private! !
!Proxy categoriesFor: #printOn:!printing!private! !
!Proxy categoriesFor: #query!accessing!private! !
!Proxy categoriesFor: #query:!accessing!private! !
!Proxy categoriesFor: #session!accessing!private! !
!Proxy categoriesFor: #session:!accessing!private! !

!Proxy class methodsFor!

returningManyOf: aClass where: aBlock 
	^self new query: (Query returningManyOf: aClass where: aBlock)!

returningManyOf: aClass where: aBlock in: aSession
	
	^self new 
		query: (Query returningManyOf: aClass where: aBlock);
		session: aSession!

returningOneOf: aClass where: aBlock 
	^self new query: (Query returningOneOf: aClass where: aBlock)!

returningOneOf: aClass where: aBlock in: aSession
	
	^self new 
		query: (Query returningOneOf: aClass where: aBlock);
		session: aSession.! !
!Proxy class categoriesFor: #returningManyOf:where:!instance creation!private! !
!Proxy class categoriesFor: #returningManyOf:where:in:!instance creation!private! !
!Proxy class categoriesFor: #returningOneOf:where:!instance creation!private! !
!Proxy class categoriesFor: #returningOneOf:where:in:!instance creation!private! !

SQLite3Proxy guid: (GUID fromString: '{03651ECD-016F-4ED2-A8BE-405A6FA7EDAB}')!
SQLite3Proxy comment: 'SQLiteProxy is different from a normal proxy in that its >>glorpTriggerProxyAction: method passes back the result of the block, which the >>getValue method reassigns to the originally retrieved #value inst var. The idea is to have a proxy that represents a DatabaseField, when there is no such entity in any SQLite schema table, so it must be constructed from a variety of special SQLite3 function calls (called pragmas).
'!
!SQLite3Proxy categoriesForClass!Unclassified! !
!SQLite3Proxy methodsFor!

class
	^SQLite3Proxy!

getValue
	"This method further modifies the retrieved value using the action block."

	self isInstantiated ifTrue: [^value].
	parameters isNil ifTrue: [parameters := Dictionary new: 0].
	
	[value := query isNil
				ifTrue: [nil]
				ifFalse: [query executeWithParameters: parameters in: session]]
			ensure: [isInstantiated := true].
	value := self glorpTriggerProxyAction: value.
	^value!

glorpTriggerProxyAction: target
	"Evaluate our action with the owner and the newly retrieved target.
	Return the result of this evaluation, or self if there is no action.
	Since the return value has ambiguous semantics, this needs work."

	| owner block |
	action isNil ifTrue: [^self].
	owner := action key.
	block := action value.
	^block value: owner value: target.! !
!SQLite3Proxy categoriesFor: #class!accessing!public! !
!SQLite3Proxy categoriesFor: #getValue!api!public! !
!SQLite3Proxy categoriesFor: #glorpTriggerProxyAction:!proxy actions!public! !

TimedProxy guid: (GUID fromString: '{B1F9087D-3100-4D7B-9409-62DD5972F710}')!
TimedProxy comment: 'This is a proxy which only holds onto its value for a fixed amount of time, and then uninstantiates. This is useful in highly-connected object graphs, where we might like to get rid of parts of a large tree that we are no longer using, but where simple weak references won''t give enough information. For one to one relationships, we can cut the proxy relation, the object will remain in cache if it is registered or being otherwised referenced, and if we ask for it again, we won''t have to go to the database because it will be found in cache. It''s probably not such a good idea to use these kind of proxies for collections, where we will have to go to the database again if we need the value. See also TimedProxyReaper.

Instance Variables:
	association	<Association> The association in the TimedProxyReaper which holds onto our identifier plus a time after which we should expire.
	id	<Integer>	An ID, assigned to us by the reaper, so that it doesn''t hold strong references to us.
	reaper	<TimedProxyReaper>	The reaper which controls this proxy.
	secondsToLive	<ArithmeticValue>	How much time, in seconds, we should wait after we were last touched before expiring.
	timeToDie	<Integer>	The millisecond clock value at which we expire. Is this a problem if there are dialects/versions for which the millisecond clock wraps around?

'!
!TimedProxy categoriesForClass!Unclassified! !
!TimedProxy methodsFor!

class
	^TimedProxy.!

doesNotUnderstand: aMessage

	self glorpTouch.
	^super doesNotUnderstand: aMessage!

glorpAssociation
	^association!

glorpAssociation: anAssociation
	association := anAssociation!

glorpId
	^id!

glorpId: uniqueInteger
	id := uniqueInteger!

glorpReaper: aTimedProxyReaper
	reaper := aTimedProxyReaper!

glorpSecondsToLive
	"Answer the number of seconds the receiver should live after having been instantiated or sent a message."

	^secondsToLive!

glorpSecondsToLive: seconds

	secondsToLive := seconds.
	self updateExpiryTime.
	reaper notNil ifTrue: [self glorpTouch]!

glorpTimeToDie
	"Answer the time at which the receiver is scheduled to die."

	^timeToDie!

glorpTouch
	self updateExpiryTime.
	reaper isNil ifFalse: [reaper touch: self].!

proxyInitialize
	super proxyInitialize.
	secondsToLive := 30.
	self updateExpiryTime.!

uninstantiate

	super uninstantiate.
	association := nil.!

updateExpiryTime
	timeToDie := Time millisecondClockValue + (secondsToLive * 1000).! !
!TimedProxy categoriesFor: #class!accessing!private! !
!TimedProxy categoriesFor: #doesNotUnderstand:!accessing!private! !
!TimedProxy categoriesFor: #glorpAssociation!accessing!private! !
!TimedProxy categoriesFor: #glorpAssociation:!accessing!private! !
!TimedProxy categoriesFor: #glorpId!accessing!private! !
!TimedProxy categoriesFor: #glorpId:!accessing!private! !
!TimedProxy categoriesFor: #glorpReaper:!accessing!private! !
!TimedProxy categoriesFor: #glorpSecondsToLive!accessing!private! !
!TimedProxy categoriesFor: #glorpSecondsToLive:!accessing!private! !
!TimedProxy categoriesFor: #glorpTimeToDie!accessing!private! !
!TimedProxy categoriesFor: #glorpTouch!accessing!private! !
!TimedProxy categoriesFor: #proxyInitialize!accessing!private! !
!TimedProxy categoriesFor: #uninstantiate!accessing!private! !
!TimedProxy categoriesFor: #updateExpiryTime!accessing!private! !

MessageArchiverForGetMapping guid: (GUID fromString: '{611143DA-354D-43AA-A899-4F6C9B38F962}')!
MessageArchiverForGetMapping comment: 'This is a special subclass of MessageArchiver which constructs its expression using getMapping:named: rather than just get:. It''s possible it ought to be generalized to one that''s parameterized by what message it sends, but right now I only need the one...
'!
!MessageArchiverForGetMapping categoriesForClass!Unclassified! !
!MessageArchiverForGetMapping methodsFor!

sendMessageTo: anExpression withArguments: arguments
	"Send our own particular message. Note that our arguments will have been wrapped as constant expressions, and we don't want that, so unwrap them"
	^anExpression getMapping: arguments first value named: (arguments at: 2) value.! !
!MessageArchiverForGetMapping categoriesFor: #sendMessageTo:withArguments:!expression creation!private! !

"Binary Globals"!

