| package |
package := Package name: 'GlorpDatabase'.
package paxVersion: 1;
	basicComment: 'This package contains Glorp objects to represent the database, and for portability amongst different databases via the DatabasePlatform subclasses.

This is part of the GLORP system, an open-source library (LGPL(S)) for persisting Smalltalk objects in relational databases. For more information see the #notice property, the accompanying files and/or the web sites http://www.glorp.org or http://www.sourceforge.net/projects/glorp 
'.

package basicPackageVersion: 'image'.


package classNames
	add: #AccessPlatform;
	add: #AdabasDPlatform;
	add: #AdabasLikePlatform;
	add: #AdHocVersionGenerator;
	add: #DatabaseAccessor;
	add: #DatabaseCommand;
	add: #DatabaseConverter;
	add: #DatabaseField;
	add: #DatabaseIndex;
	add: #DatabasePlatform;
	add: #DatabaseRow;
	add: #DatabaseSequence;
	add: #DatabaseTable;
	add: #DatabaseTableConstraint;
	add: #DB2IdentitySequence;
	add: #DB2Platform;
	add: #DB2Sequence;
	add: #DelegatingDatabaseConverter;
	add: #DeleteCommand;
	add: #DolphinDatabaseAccessor;
	add: #FirebirdPlatform;
	add: #FirebirdSequence;
	add: #ForeignKeyConstraint;
	add: #GlorpDatabasePackage;
	add: #GlorpDatabaseValueGenerator;
	add: #GlorpNullConverter;
	add: #GlorpPreparedStatement;
	add: #InMemorySequence;
	add: #InsertCommand;
	add: #JustSelectTheMaximumSequenceValueAndAddOne;
	add: #Login;
	add: #MySQLODBCPlatform;
	add: #MySQLPlatform;
	add: #MySQLSequence;
	add: #NamedSequence;
	add: #NullSequence;
	add: #OcelotPlatform;
	add: #OracleODBCPlatform;
	add: #OraclePlatform;
	add: #OracleSequence;
	add: #PGSequence;
	add: #PluggableDatabaseConverter;
	add: #PostgreSQLPlatform;
	add: #PrimaryKeyConstraint;
	add: #QuerySelectCommand;
	add: #RowBasedCommand;
	add: #SAPDBPlatform;
	add: #SelectCommand;
	add: #SQLite3Platform;
	add: #SQLite3PragmaCommand;
	add: #SQLite3Sequence;
	add: #SQLServerPlatform;
	add: #SQLServerSequence;
	add: #SQLStringCommand;
	add: #SQLStringSelectCommand;
	add: #TableBasedSequence;
	add: #TimestampVersionGenerator;
	add: #UniqueConstraint;
	add: #UpdateCommand;
	add: #VersionGenerator;
	add: #VersionNumberVersionGenerator;
	add: #VersionType;
	yourself.

package methodNames
	add: #ElementBuilder -> #valueOf:;
	add: #ElementBuilder -> #valueOfField:;
	add: #ElementBuilder -> #valueOfField:in:;
	add: #GlorpCursoredStream -> #atEnd;
	add: #GlorpCursoredStream -> #rawPeek;
	add: #GlorpExpression -> #condensePrimaryKeyComparison;
	add: #Object -> #isDatabaseField;
	add: #ProtoObject -> #isDatabaseField;
	add: #RelationExpression -> #condensePrimaryKeyComparison;
	add: #RelationExpression -> #generalExpressionPart;
	add: #SubSelectExpression -> #condensePrimaryKeyComparison;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Database\Database Connection Base';
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\..\Object Arts\Dolphin\System\Base64\Dolphin Base64';
	add: 'GlorpDatabaseTypes';
	add: 'GlorpDolphinPort';
	add: 'GlorpExpressions';
	add: 'GlorpMisc';
	add: 'GlorpQueries';
	yourself).

package!

"Class Definitions"!

Object subclass: #DatabaseAccessor
	instanceVariableNames: 'connection currentLogin logging logger logOnly reusePreparedStatements deniedCommands mutex dependents'
	classVariableNames: 'LoggingEnabled'
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #DatabaseCommand
	instanceVariableNames: 'useBinding stream sqlString session'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #DatabaseConverter
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #DatabaseField
	instanceVariableNames: 'table name isPrimaryKey position type isNullable isUnique isLockKey defaultValue'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #DatabaseIndex
	instanceVariableNames: 'name fields'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #DatabasePlatform
	instanceVariableNames: 'types converters useBinding reservedWords functions characterEncoding'
	classVariableNames: 'UseBindingIfSupported'
	poolDictionaries: ''
	classInstanceVariableNames: 'converterRepository'!
Object subclass: #DatabaseRow
	instanceVariableNames: 'table contents shouldBeWritten owner forDeletion status oldVersion generated parent operation'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: 'missingFieldIndicator emptySlot'!
Object subclass: #DatabaseTable
	instanceVariableNames: 'name schema fields foreignKeyConstraints indexes parent lockKeyFields primaryKeyFields isImaginary replacementSubSelect uniqueConstraints'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #DatabaseTableConstraint
	instanceVariableNames: 'name table shouldCreateInDatabase fields'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #GlorpDatabasePackage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #GlorpDatabaseValueGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #GlorpPreparedStatement
	instanceVariableNames: 'signature statement busy'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #Login
	instanceVariableNames: 'database username password connectString name schema secure'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabaseAccessor subclass: #DolphinDatabaseAccessor
	instanceVariableNames: 'currentStatement'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabaseCommand subclass: #RowBasedCommand
	instanceVariableNames: 'row allRows rowCount failed'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabaseCommand subclass: #SelectCommand
	instanceVariableNames: 'parameters blockFactor'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabaseCommand subclass: #SQLStringCommand
	instanceVariableNames: 'parameters maximumBlobSizeToRetrieveDirectly'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
RowBasedCommand subclass: #DeleteCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
RowBasedCommand subclass: #InsertCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
RowBasedCommand subclass: #UpdateCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SelectCommand subclass: #QuerySelectCommand
	instanceVariableNames: 'query boundExpressions'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
QuerySelectCommand subclass: #SQLStringSelectCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SQLStringSelectCommand subclass: #SQLite3PragmaCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabaseConverter subclass: #DelegatingDatabaseConverter
	instanceVariableNames: 'host stToDbSelector dbToStSelector'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabaseConverter subclass: #GlorpNullConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabaseConverter subclass: #PluggableDatabaseConverter
	instanceVariableNames: 'stToDb dbToSt'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabasePlatform subclass: #AdabasLikePlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabasePlatform subclass: #DB2Platform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabasePlatform subclass: #MySQLPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabasePlatform subclass: #OcelotPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabasePlatform subclass: #OraclePlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabasePlatform subclass: #PostgreSQLPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabasePlatform subclass: #SQLite3Platform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabasePlatform subclass: #SQLServerPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
AdabasLikePlatform subclass: #AdabasDPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
AdabasLikePlatform subclass: #SAPDBPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
MySQLPlatform subclass: #MySQLODBCPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
OcelotPlatform subclass: #FirebirdPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
OraclePlatform subclass: #OracleODBCPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SQLServerPlatform subclass: #AccessPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabaseTableConstraint subclass: #ForeignKeyConstraint
	instanceVariableNames: 'suffixExpression sourceFields targetFields'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabaseTableConstraint subclass: #PrimaryKeyConstraint
	instanceVariableNames: 'tableName schema type'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabaseTableConstraint subclass: #UniqueConstraint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
GlorpDatabaseType subclass: #VersionType
	instanceVariableNames: 'underlyingType generator generatesOverExistingValues'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
GlorpDatabaseValueGenerator subclass: #DatabaseSequence
	instanceVariableNames: 'reservedNumbers'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
GlorpDatabaseValueGenerator subclass: #VersionGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabaseSequence subclass: #InMemorySequence
	instanceVariableNames: 'count'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabaseSequence subclass: #NamedSequence
	instanceVariableNames: 'name tableSelectCommand schema'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabaseSequence subclass: #NullSequence
	instanceVariableNames: ''
	classVariableNames: 'Singleton'
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabaseSequence subclass: #SQLite3Sequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DatabaseSequence subclass: #TableBasedSequence
	instanceVariableNames: 'sequenceTableName'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
NamedSequence subclass: #DB2IdentitySequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
NamedSequence subclass: #FirebirdSequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
NamedSequence subclass: #JustSelectTheMaximumSequenceValueAndAddOne
	instanceVariableNames: 'field table'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
NamedSequence subclass: #MySQLSequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
NamedSequence subclass: #OracleSequence
	instanceVariableNames: 'count sequenceIncrement accessProtect'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
NamedSequence subclass: #PGSequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
NamedSequence subclass: #SQLServerSequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
OracleSequence subclass: #DB2Sequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
VersionGenerator subclass: #AdHocVersionGenerator
	instanceVariableNames: 'block numberOfArguments'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
VersionGenerator subclass: #TimestampVersionGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
VersionGenerator subclass: #VersionNumberVersionGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!ElementBuilder methodsFor!

valueOf: anExpression
	^expression valueInBuilder: self.!

valueOfField: aField
	"aField is either a database field, or a constant expression containing a non-varying value that isn't derived from the row"
	(aField class == ConstantExpression) ifTrue: [^aField value].
	^self row atIndex: (self translateFieldPosition: aField).!

valueOfField: aField in: aRow
	"Since the elementBuilder now holds the row, #valueOfField: is preferred protocol, but some things (e.g. ad hoc mapping blocks) might still be using this, so left for compatibility"
	(aField class == ConstantExpression) ifTrue: [^aField value].
	^aRow atIndex: (self translateFieldPosition: aField).! !
!ElementBuilder categoriesFor: #valueOf:!private!translating fields! !
!ElementBuilder categoriesFor: #valueOfField:!private!translating fields! !
!ElementBuilder categoriesFor: #valueOfField:in:!private!translating fields! !

!GlorpCursoredStream methodsFor!

atEnd
	"Release ourselves fairly aggressively when we hit the end"
	| accessor |
	peekBuffer isNil ifFalse: [^false].
	rawPeekBuffer isNil ifFalse: [^false].
	cursor isNil ifTrue: [^true].
	"For test purposes, fabricate an accessor if we don't have one, it's not really needed for a non-db stream"
	accessor := self accessor isNil ifTrue: [DatabaseAccessor new] ifFalse: [self accessor].
	^([accessor isCursorAtEnd: cursor] on: Dialect error do: [:ex | databaseAccessor handleError: ex for: command])
		ifTrue:
			[self release.
			true]
		ifFalse: [	"We have to do this, because VA won't answer true to atEnd until we've read past end"
			self rawPeek isNil].!

rawPeek
	"Peek at the underlying rows, but don't build objects"
	rawPeekBuffer isNil ifFalse: [^rawPeekBuffer].
	rawPeekBuffer := cursor next.
	^rawPeekBuffer.! !
!GlorpCursoredStream categoriesFor: #atEnd!private!testing! !
!GlorpCursoredStream categoriesFor: #rawPeek!private! !

!GlorpExpression methodsFor!

condensePrimaryKeyComparison
	^self.! !
!GlorpExpression categoriesFor: #condensePrimaryKeyComparison!preparing!private! !

!Object methodsFor!

isDatabaseField
	^false! !
!Object categoriesFor: #isDatabaseField!As yet unclassified!private! !

!ProtoObject methodsFor!

isDatabaseField
	^false! !
!ProtoObject categoriesFor: #isDatabaseField!public! !

!RelationExpression methodsFor!

condensePrimaryKeyComparison
	"This tries to condense expressions of the form
         something relatedObject id = x
      with 
          something foreignKeyToRelatedObjectId = x, saving a join (and making the thing work at all, in the case where x = nil"

	| fieldBeingCompared baseObjectExpression join sourceField |
	(relation = #= or: [relation = #<>]) ifFalse: [^self].
	self leftChild mappedFields size = 1 ifFalse: [^self].
	fieldBeingCompared := self leftChild field.
	fieldBeingCompared isDatabaseField ifFalse: [^self].
	fieldBeingCompared isPrimaryKey ifFalse: [^self].
	baseObjectExpression := self leftChild base.
	baseObjectExpression representsDerivedObject ifFalse: [^self].	"If we have an expression for a join, don't even try."
	baseObjectExpression join isJoin ifFalse: [^self].
	join := baseObjectExpression join asGlorpExpressionOn: baseObjectExpression base.
	sourceField := join sourceForTarget: fieldBeingCompared.
	sourceField isNil ifTrue: [^self].	"OK, we've got a match, replace ourselves with the shortcut"
	^(sourceField isGlorpExpression and: [sourceField isConstantExpression])
		ifTrue: [self rightChild get: relation withArguments: (Array with: sourceField)]
		ifFalse: [(self leftChild base base getField: sourceField) get: relation withArguments: (Array with: self rightChild)]!

generalExpressionPart
	| left right |
	left := leftChild generalExpressionPart.
	right := rightChild generalExpressionPart.
	(left isNil and: [right isNil]) ifTrue: [^nil].
	left isNil ifTrue: [^right].
	right isNil ifTrue: [^left].
	^self.! !
!RelationExpression categoriesFor: #condensePrimaryKeyComparison!preparing!public! !
!RelationExpression categoriesFor: #generalExpressionPart!converting!private! !

!SubSelectExpression methodsFor!

condensePrimaryKeyComparison
	"Relies on this having been called after the children are condensed. If the child has had its reference to our base condensed away, then we aren't necessary, the comparison is at the same level and no longer needs a subselect"
	^leftChild queryLevelBaseExpression == rightChild queryLevelBaseExpression ifTrue: [rightChild] ifFalse: [self].! !
!SubSelectExpression categoriesFor: #condensePrimaryKeyComparison!preparing!private! !

"End of package definition"!

"Source Globals"!

"Classes"!

DatabaseAccessor guid: (GUID fromString: '{23D57502-47F0-4F9E-9584-FCE0DDFA0F35}')!
DatabaseAccessor comment: 'This translates between the the general, portable Glorp protocol and whatever needs to be done to access a database in a particular smalltalk dialect. Most of the real work is done in its subclasses.

Subclasses must implement the following messages:
	accessing
		connectionClassForLogin:
	executing
		basicExecuteSQLString:
		externalDatabaseErrorSignal
		rowCount
	login
		loginIfError:
		logout
		showDialog:

Instance Variables
	connection	<varies>	The underlying database connection
	currentLogin	<Login> The glorp-level login we are using
	logging	<Boolean>	(default is false) Is logging on or off
	logOnly <Boolean> (default is false) If logging is true, and logOnly is true, then the commands are ONLY logged and not executed.
	logger <TextCollector> (default is Transcript) The user can put any TextCollector API supporting class in here to collect information. It must support #show: and #cr.
	permittedCommands	<Collection of: DatabaseCommand class>	The list of commands we are permitted to use. This allows us to filter commands, indicating that a particular session should only be doing inserts right now, not updates, deletes, or queries. Or that it should only be doing writes, not reads.
	reusePreparedStatements	<Boolean>	Should we try to reuse prepared statements,that have the same SQL, or just re-prepare each time.

'!
!DatabaseAccessor categoriesForClass!Unclassified! !
!DatabaseAccessor methodsFor!

addConstraint: aConstraint
	^self
		addConstraint: aConstraint
		ifError:
			[:ex | 
			| msg |
			msg := ex messageText ifNil: [''].
			msg isEmpty ifTrue: [msg := 'No error message provided for top level exception.'].
			Transcript
				cr;
				show: ('Add constraint <1s> ERROR: (<2s>) ' expandMacrosWith: aConstraint name with: msg).
			ex return: nil]!

addConstraint: aConstraint ifError: aBlock
	| addString |
	addString := self platform createConstraintAddStatement: aConstraint.
	addString isEmpty ifTrue: [^self].
	^self
		executeInDDLOperation: [ self executeSQLStringNoResult: addString ]
		ifError: aBlock!

addFields: aCollection
	self
		addFields: aCollection
		ifError:
			[:ex | 
			| msg |
			msg := ex messageText ifNil: [''].
			msg isEmpty ifTrue: [msg := 'No error message provided for top level exception.'].
			Transcript
				cr;
				show: ('Add field ERROR: (<2s>) ' expandMacrosWith: msg).
			ex return: nil]!

addFields: aCollection ifError: aBlock

	aCollection
		do: 	[:aField | 
			self execute: [self executeSQLStringNoResult: (self platform createAddColumnStatementFor: aField)]
				ifError: aBlock ].!

allowCommand: aCommand during: aBlock
	| oldCommands |
	oldCommands := self deniedCommands.
	((self deniedCommands ifNil: [#()]) includes: aCommand)
		ifTrue: [deniedCommands := deniedCommands copyWithout: aCommand].
	aBlock ensure: [deniedCommands := oldCommands].!

alsoDenyCommand: aCommand
	self deniedCommands isNil
		ifTrue: [self denyCommand: aCommand]
		ifFalse:
			[self deniedCommands: self deniedCommands , (Array with: aCommand)].!

alterField: aField newType: aType
	^self
		alterField: aField
		newType: aType
		ifError:
			[:ex | 
			| msg |
			msg := ex messageText ifNil: [''].
			msg isEmpty ifTrue: [msg := 'No error message provided for top level exception.'].
			Transcript
				cr;
				show:
						('Alter column <1s> to set new type <2s> ERROR: (<3s>) '
								expandMacrosWith: aField name
								with: aType sqlString
								with: msg).
			ex return: nil]!

alterField: aField newType: aType ifError: aBlock

	^self doDDLOperation: [
		self 
			execute: [self executeSQLStringNoResult: 
						(self platform createAlterColumnStatement: aField newType: aType)]
			ifError: aBlock ]!

alterField: aField setDefault: aType
	self
		alterField: aField
		setDefault: aType
		ifError:
			[:ex | 
			| msg |
			msg := ex messageText ifNil: [''].
			msg isEmpty ifTrue: [msg := 'No error message provided for top level exception.'].
			Transcript
				cr;
				show:
						('Alter column  <1s> to set default <2s>  ERROR: (<2s>) '
								expandMacrosWith: aField name
								with: aType sqlString
								with: msg).
			ex return: nil]!

alterField: aField setDefault: anObject ifError: aBlock

	^self doDDLOperation: [
		self 
			execute: [self executeSQLStringNoResult: 
						(self platform createAlterColumnStatement: aField setDefault: anObject )]
			ifError: aBlock ]!

alterFieldDropDefault: aField
	self
		alterFieldDropDefault: aField
		ifError:
			[:ex | 
			| msg |
			msg := ex messageText ifNil: [''].
			msg isEmpty ifTrue: [msg := 'No error message provided for top level exception.'].
			Transcript
				cr;
				show: ('Alter column  <1s> to drop default ERROR: (<2s>) ' expandMacrosWith: aField name with: msg).
			ex return: nil]!

alterFieldDropDefault: aField ifError: aBlock

	^self doDDLOperation: [
		self 
			execute: [self executeSQLStringNoResult: 
						(self platform createAlterColumnStatementDropDefault: aField)]
			ifError: aBlock ]!

alterFieldDropNotNull: aField
	self
		alterFieldDropNotNull: aField
		ifError:
			[:ex | 
			| msg |
			msg := ex messageText ifNil: [''].
			msg isEmpty ifTrue: [msg := 'No error message provided for top level exception.'].
			Transcript
				cr;
				show: ('Alter column  <1s> to drop not null ERROR: (<2s>) ' expandMacrosWith: aField name with: msg).
			ex return: nil]!

alterFieldDropNotNull: aField ifError: aBlock

	^self doDDLOperation: [
		self 
			execute: [self executeSQLStringNoResult: 
						(self platform createAlterColumnStatementDropNotNull: aField)]
			ifError: aBlock ]!

alterFieldSetNotNull: aField
	self
		alterFieldSetNotNull: aField
		ifError:
			[:ex | 
			| msg |
			msg := ex messageText ifNil: [''].
			msg isEmpty ifTrue: [msg := 'No error message provided for top level exception.'].
			Transcript
				cr;
				show: ('Alter column  <1s> to set not null ERROR: (<2s>) ' expandMacrosWith: aField name with: msg).
			ex return: nil]!

alterFieldSetNotNull: aField ifError: aBlock

	^self doDDLOperation: [
		self 
			execute: [self executeSQLStringNoResult: 
						(self platform createAlterColumnStatementSetNotNull: aField)]
			ifError: aBlock ]!

basicExecuteSQLString: aString
	self subclassResponsibility.!

basicExecuteSQLString: aString bindingArray: aBindingArray
	self subclassResponsibility.!

basicExecuteSQLString: aString returnResult: returnResult binding: aBindingArray doing: aBlock
	^self subclassResponsibility!

basicExecuteSQLStringNoResult: aString doing: aBlock
	^self basicExecuteSQLString: aString returnResult: false binding: nil doing: aBlock.!

beginTransaction
	^self subclassResponsibility.!

beReadOnly
	self denyWrites.!

checkPermissionFor: command
	deniedCommands isNil ifTrue: [^self].
	(deniedCommands includes: command class)
		ifTrue: [(GlorpIllegalCommand new command: command) signal].!

commitTransaction
	^self subclassResponsibility.!

completedCommand: aCommand
	"Indicate that we've completed the command. Release associated resources."
	^self.!

connection
	^connection!

connection: aPlatformDatabaseConnection
	connection := aPlatformDatabaseConnection!

connectionClass
	^self connectionClassForLogin: currentLogin!

connectionClassForLogin: aLogin
	^self subclassResponsibility.!

copy

	^self shallowCopy postCopy.!

copyDataFromField: fromField toField: toField ifError: errorBlock

	self
		executeInDDLOperation: [ self executeSQLStringNoResult: (self platform sqlStatementToCopyDataFromColumn: fromField to: toField )]
		ifError: errorBlock!

copyDataFromTable: fromTable toTable: anotherTable ifError: errorBlock

	self
		executeInDDLOperation: [ self executeSQLStringNoResult: (self platform sqlStatementToCopyDataFromTable: fromTable to: anotherTable) ]
		ifError: errorBlock!

createSequence: aDatabaseSequence ifError: aBlock 

	self
		executeInDDLOperation: [ self executeSQLStringNoResult: aDatabaseSequence creationString]
		ifError: aBlock!

createTable: aGlorpDatabaseTable ifError: aBlock 
	"This method should be used to create a database table from aTable"
	aGlorpDatabaseTable isImaginary ifTrue: [^self].
	self
		executeInDDLOperation: [
			self executeSQLStringNoResult: (self platform createTableStatementStringFor: aGlorpDatabaseTable)]
		ifError: aBlock!

createTableFKConstraints: aGLORBDatabaseTable ifError: aBlock 
	"This method should be used to define foreign key constraints for a database table from aTable"

	aGLORBDatabaseTable foreignKeyConstraintsForCreation
		do: [ :each | self addConstraint: each ifError: aBlock ].!

createTableIndexes: aGlorpDatabaseTable ifError: aBlock
	"Create all indexes for this table"
	
	(self platform createTableIndexStatementStringsFor: aGlorpDatabaseTable)
		do: [:each | self execute: [self executeSQLStringNoResult: each] ifError: aBlock]!

currentLogin
	^currentLogin!

currentLogin: aLogin 
	currentLogin := aLogin!

cursorFor: aStream
	^GlorpCursoredStream on: aStream accessor: self.!

cursorFor: answerStream command: aCommand
	^aCommand isNil
		ifTrue: [self cursorFor: answerStream]
		ifFalse: [GlorpCursoredStream on: answerStream for: aCommand accessor: self].!

deniedCommands
	^deniedCommands.!

deniedCommands: aCollection
	deniedCommands := aCollection.!

denyCommand: aCommand
	(aCommand isBehavior) ifFalse: [self error: 'invalid command type'].
	self denyCommands: (Array with: aCommand).!

denyCommands: aCollection
	deniedCommands := aCollection asArray.!

doDDLOperation: aBlock
	"Execute DDL SQL - something which creates/drops/modifies tables or other schema information. The main reason for this method is that some databases require a transaction, while others don't require it, and still others don't allow it. Try to do the right thing."

	^self platform requiresTransactionForTableOperations ifTrue: [self inTransactionDo: aBlock] ifFalse: [aBlock value]!

dropConstraint: aConstraint
	^self
		dropConstraint: aConstraint
		ifError:
			[:ex | 
			| msg |
			msg := ex messageText ifNil: [''].
			msg isEmpty ifTrue: [msg := 'no error message provided for top level exception'].
			Transcript
				cr;
				show: ('Drop constraint <1s> ERROR: (<2s>' expandMacrosWith: aConstraint name with: msg).
			ex return: nil]!

dropConstraint: aConstraint ifError: aBlock
	| dropString |
	self platform supportsConstraints ifFalse: [^self].
	dropString := self platform createConstraintDropStatement: aConstraint.
	dropString isEmpty ifTrue: [^self].
	self
		executeInDDLOperation: [ self executeSQLStringNoResult: dropString ]
		ifError:aBlock!

dropFields: aCollection
	self
		dropFields: aCollection
		ifError:
			[:ex | 
			| msg |
			msg := ex messageText ifNil: [''].
			msg isEmpty ifTrue: [msg := 'No error message provided for top level exception.'].
			Transcript
				cr;
				show: ('Drop field ERROR: (<2s>) ' expandMacrosWith: msg).
			ex return: nil]!

dropFields: aCollection ifError: aBlock

	aCollection
		do: 	[:aField | 
			self execute: [self executeSQLStringNoResult: (self platform createDropColumnStatementFor: aField)]
				ifError: aBlock ].!

dropForeignKeyConstraintsForTable: aTable ifError: aBlock 
	aTable isImaginary ifTrue: [^self].
	aTable  foreignKeyConstraints do: [:each | self dropConstraint: each ifError: [ :ex | aBlock value: ex]].!

dropSequence: aDatabaseSequence ifAbsent: aBlock 
	self
		executeInDDLOperation: [ self executeSQLStringNoResult: aDatabaseSequence dropString ]
		ifError: aBlock!

dropSequences: anArray
	anArray
		do:
			[:each | 
			self
				dropSequence: each
				ifAbsent:
					[:ex | 
					Transcript show: (ex messageText ifNil: [ex printString]).
					ex return: nil]].!

dropTable: aTable ifError: aBlock 
	aTable isImaginary ifTrue: [^self].
	self dropForeignKeyConstraintsForTable: aTable ifError: aBlock.
	self dropTableDefinition: aTable ifError: aBlock.!

dropTableDefinition: aTable ifError: aBlock
	"The dropping of the primary key constraints (a singular object not a collection, despite its name) is probably not needed before dropping the table by any database platform.  If 'platform supportsConstraints' is false then it will not be done, since dropConstraint:ifError: assumes it is called on a foreign key constraint (the case in all other callers) and does nothing if the platform does not support them."

	aTable isImaginary ifTrue: [^self].
	aTable hasPrimaryKeyConstraints ifTrue: [self dropConstraint: aTable  primaryKeyConstraints ifError: [:ex | aBlock value: ex]].
	self dropTableNamed: aTable qualifiedName ifAbsent: aBlock.!

dropTableNamed: aString
	self
		dropTableNamed: aString
		ifAbsent:
			[:ex | 
			| msg |
			msg := ex messageText ifNil: [''].
			msg isEmpty ifTrue: [msg := 'No error message provided for top level exception.'].
			Transcript
				cr;
				show: ('Drop table <1s> ERROR: (<2s>) ' expandMacrosWith: aString with: msg).
			ex return: nil]!

dropTableNamed: aString ifAbsent: aBlock 
	self doDDLOperation: [
		self 
			execute: [self executeSQLStringNoResult: (self platform createDropTableStatementFor: aString )]
			ifError: aBlock].!

dropTables: anArray
	anArray do: [:eachTable | eachTable foreignKeyConstraints do: [:eachConstraint | self dropConstraint: eachConstraint]].

	anArray
		do:
			[:each | 
			self
				dropTableDefinition: each
				ifError:
					[:ex | 
					| msg |
					msg := ex messageText ifNil: [''].
					msg isEmpty ifTrue: [msg := 'no error message provided for top level exception'].
					Transcript
						cr;
						show: 'ERROR : (' , msg , ' )'.
					ex return: nil]]!

encoding
	"The connection is specific to the dialect;  some dialects will need to override this method."

	^connection encoding!

encoding: aSymbol
	"The platform is a Glorp databasePlatform, so its setter code is generic across all dialects.  The connection is specific to the dialect;  some dialects will need to override how its encoding is set.  In the hope that most dialects will find this works for them, and that coders porting to dialects in which it does not will benefit from seeing the walkback and knowing they have something to do, we attempt to set both at top level."

	self platform characterEncoding: aSymbol.
	connection encoding: aSymbol.!

errorsToCatch
	^Dialect error.!

execute: aBlock

	^self execute: aBlock ifError: [:ex | ex pass].!

execute: aBlock ifError: errorBlock

	^aBlock on: self errorsToCatch do: errorBlock.!

executeCommand: command
	^self executeCommand: command returnCursor: false.!

executeCommand: command returnCursor: aBoolean
	^mutex
		critical:
			[| result |
			self checkPermissionFor: command.
			result := [(self useBinding and: [command useBinding])
				ifTrue: [command executeBoundIn: self]
				ifFalse: [command executeUnboundIn: self]]
				on: Dialect error
				do: [:ex | self handleError: ex for: command].
			aBoolean ifTrue: [result] ifFalse: [result upToEnd]].!

executeCommandBound: command
	"By default, we don't handle binding"
	^command executeUnboundIn: self.!

executeCommandUnbound: aCommand
	| cursor |
	self logCommand: aCommand bound: false.
	self logOnly ifTrue: [^self].
	cursor := self basicExecuteSQLString: aCommand sqlString.
	cursor command: aCommand.
	^cursor!

executeInDDLOperation: aBlock ifError: errorBlock
	"Execute DDL SQL - something which creates/drops/modifies tables or other schema information. The main reason for this method is that some databases require a transaction, while others don't require it, and still others don't allow it. Try to do the right thing."

	^ self doDDLOperation: [
		self
			execute: aBlock
			ifError: errorBlock ]!

executeSQLString: aString
	| result block time |
	self log: aString.
	self logOnly ifTrue: [^self].
	block := [result := (self basicExecuteSQLString: aString) upToEnd].
	time := self logging
		ifTrue: [Time millisecondsToRun: block]
		ifFalse:
			[block value.
			0].
	self logTime: time.
	^result.!

executeSQLString: aString binding: aBindingArray
	| result block time |
	self log: aString.
	self logOnly ifTrue: [^self].
	block := [ 
		result := (self basicExecuteSQLString: aString binding: aBindingArray) upToEnd].
	time := self logging
		ifTrue: [Time millisecondsToRun: block]
		ifFalse:
			[block value.
			0].
	self logTime: time.
	^result.!

executeSQLStringNoResult: aString
	^self
		executeSQLStringNoResult: aString
		doing: []
		forCommand: nil.!

executeSQLStringNoResult: aString doing: aBlock forCommand: aCommand
	"Execute the string and discard the result. Evaluate aBlock before releasing the statement's answer stream"

	| block time |
	aCommand isNil ifTrue: [self log: aString] ifFalse: [self logCommand: aCommand bound: false].
	self logOnly ifTrue: [^self].
	block := [self basicExecuteSQLStringNoResult: aString doing: aBlock].
	time := self logging
		ifTrue: [Time millisecondsToRun: block]
		ifFalse:
			[block value.
			0].
	self logTime: time.!

externalDatabaseErrorSignal

	self subclassResponsibility.!

handleError: exception for: command
	"Signal a GlorpDatabase... error.  If it is handled to return true (any handler or defaultAction had better not return true in response to an error that logging in or logging out could reraise and on which it would again return true) and we have a reusable login, try to logout and in again and restart, otherwise exit."

	| errorClass error |
	errorClass := command isReadCommand
		ifTrue: [GlorpDatabaseReadError]
		ifFalse: [GlorpDatabaseWriteError].
	error := errorClass new.
	error command: command.
	error databaseError: (self innerExceptionFor: exception).
	error accessor: self.
	(error signal ~~ true or: [currentLogin isNil or: [currentLogin secure]])
		ifTrue: [exception return: nil]
		ifFalse:
			[self reset.
			self logout.
			self reLogin.
			exception restart].!

initialize
	deniedCommands := nil.
	mutex := Semaphore forMutualExclusion.!

innerExceptionFor: exception
	^exception tag.!

inTransactionDo: aBlock
	"Execute aBlock in a database transaction. If one is already in progress, just continue. If not, start a transaction."

	| alreadyInTransaction result |
	[ 
	alreadyInTransaction := self isInTransaction.
	alreadyInTransaction ifFalse: [self beginTransaction].
	result := aBlock argumentCount = 1 ifTrue: [aBlock value: self] ifFalse: [aBlock value].
	alreadyInTransaction ifFalse: [self commitTransaction]]
		ifCurtailed:
			[alreadyInTransaction ifFalse: [self rollbackTransaction]].
	^result.!

isCursorAtEnd: aCursor
	"Return true if aCursor is at its end. This implementation may not work for all, but it seems like a reasonable default"

	^aCursor atEnd.!

isExistingTable: tableName inSchema: schemaName

	| databasePlatform sqlStatement result |
	databasePlatform := self currentLogin platform.
	sqlStatement := databasePlatform sqlStatementToTestExistenceOfTable: tableName inSchema: schemaName.
	result := self executeSQLString: sqlStatement.
	^result notEmpty
		ifTrue: [result first first > 0]
		ifFalse: [false]!

isInTransaction
	^self subclassResponsibility.!

listTableNamesInSchemaNamed: schemaString

	| databasePlatform sqlStatement result |
	databasePlatform := self currentLogin platform.
	sqlStatement := databasePlatform sqlStatementToListTablesInSchema: schemaString.
	result := self executeSQLString: sqlStatement.
	^result isEmpty
		ifTrue: [#()]
		ifFalse: [result collect: [:ea| ea first]]!

log: aStringOrBlock
	"Log an arbitrary string"
	| string |
	self logging ifFalse: [^self].
	string := aStringOrBlock isString
		ifTrue: [aStringOrBlock]
		ifFalse: [aStringOrBlock value].
	string size > 4000 ifTrue: [string := string copyFrom: 1 to: 4000].
	self logger
		cr;
		show: string.!

logCommand: aDatabaseCommand bound: aBoolean

	"Log the execution of a command"
	| string |
	self changed: #command: with: aDatabaseCommand.
	self logging ifFalse: [^self].
	string := aDatabaseCommand sqlString.
 	aBoolean ifTrue: [string := string , '  ' , aDatabaseCommand bindings printString].
	string size > 4000 ifTrue: [string := string copyFrom: 1 to: 4000].
	self logger
		cr;
		show: string.!

logError: anErrorObject 
	self log: anErrorObject printString!

logger

	^logger ifNil: [Transcript]!

logger: aTextCollector

	logger := aTextCollector!

logging
	logging isNil ifTrue: [^self class loggingEnabled].
	^logging.!

logging: aBoolean 
	logging := aBoolean!

login
	
	| |
	self loginIfError: [:ex | 
		ex pass].
	"Just to help avoid confusion if someone thinks they're getting a login object back from this"
	^nil.!

loginIfError: aBlock

	self subclassResponsibility.!

logOnly

	logOnly ifNil: [logOnly := false].
	^logOnly and: [self logging]!

logOnly: aBoolean

	logOnly := aBoolean!

logout
	^self subclassResponsibility.!

logTime: aTimeOrBlock
	| time |
	self logOnly ifTrue: [^self].
	self changed: #time: with: aTimeOrBlock.
	self logging ifFalse: [^self].
	time := (Dialect isBlock: aTimeOrBlock)
		ifTrue: [aTimeOrBlock value]
		ifFalse: [aTimeOrBlock].
	self logger
		cr;
		show: '(' , (time / 1000.0) printString , ' s)'.!

myDependents
	^dependents.!

myDependents: aCollection
	dependents := aCollection.!

numberOfPreparedStatements
	^0!

permitEverything
	"Note that nil is treated as a flag indicating that all commands are allowed"
	deniedCommands := nil.!

permitNothingDuring: aBlock
	| oldCommands |
	oldCommands := self deniedCommands.
	self permitNothing.
	aBlock ensure: [deniedCommands := oldCommands].!

platform
	^currentLogin database.!

populateField: aDatabaseField with: aValue ifError: errorBlock

	self
		executeInDDLOperation: [
			self executeSQLStringNoResult: (self platform sqlStatementToPopulateColumn: aDatabaseField with: aValue) ]
		ifError: errorBlock!

postCopy!

renameField: aDatabaseField to: nameString ifError: errorBlock

	self
		executeInDDLOperation: [ self executeSQLStringNoResult: (self platform sqlStatementToRenameColumn: aDatabaseField to: nameString) ]
		ifError: errorBlock!

renameTable: aDatabaseTable to: newTableName inPlatform: aDatabasePlatform ifError: errorBlock

	| sqlStatement |
	sqlStatement := aDatabasePlatform sqlStatementToRenameTable: aDatabaseTable to: newTableName.	
	self
		executeInDDLOperation: [ self executeSQLStringNoResult: sqlStatement ]
		ifError: errorBlock!

reset
	"Don't reset the permissions, because the replicator wants to reset sessions without changing their characteristics"
"	self permitEverything."!

resetPermissions
	self permitEverything.!

returnRowCount: aBoolean
	^self.!

reusePreparedStatements
	^reusePreparedStatements and: [self useBinding].!

reusePreparedStatements: aBoolean
	reusePreparedStatements := aBoolean.!

rollbackTransaction
	^self subclassResponsibility.!

rowCount
	^self subclassResponsibility!

schema

	^ self currentLogin schema!

showDialog: aString

	self subclassResponsibility.!

useBinding
	^self platform useBinding.! !
!DatabaseAccessor categoriesFor: #addConstraint:!executing!public! !
!DatabaseAccessor categoriesFor: #addConstraint:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #addFields:!executing!public! !
!DatabaseAccessor categoriesFor: #addFields:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #allowCommand:during:!permissions!private! !
!DatabaseAccessor categoriesFor: #alsoDenyCommand:!permissions!private! !
!DatabaseAccessor categoriesFor: #alterField:newType:!executing!public! !
!DatabaseAccessor categoriesFor: #alterField:newType:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #alterField:setDefault:!executing!public! !
!DatabaseAccessor categoriesFor: #alterField:setDefault:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #alterFieldDropDefault:!executing!public! !
!DatabaseAccessor categoriesFor: #alterFieldDropDefault:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #alterFieldDropNotNull:!executing!public! !
!DatabaseAccessor categoriesFor: #alterFieldDropNotNull:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #alterFieldSetNotNull:!executing!public! !
!DatabaseAccessor categoriesFor: #alterFieldSetNotNull:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #basicExecuteSQLString:!executing!private! !
!DatabaseAccessor categoriesFor: #basicExecuteSQLString:bindingArray:!executing!private! !
!DatabaseAccessor categoriesFor: #basicExecuteSQLString:returnResult:binding:doing:!executing!public! !
!DatabaseAccessor categoriesFor: #basicExecuteSQLStringNoResult:doing:!executing!private! !
!DatabaseAccessor categoriesFor: #beginTransaction!private!transactions! !
!DatabaseAccessor categoriesFor: #beReadOnly!permissions!private! !
!DatabaseAccessor categoriesFor: #checkPermissionFor:!permissions!private! !
!DatabaseAccessor categoriesFor: #commitTransaction!private!transactions! !
!DatabaseAccessor categoriesFor: #completedCommand:!cursors!private! !
!DatabaseAccessor categoriesFor: #connection!accessing!private! !
!DatabaseAccessor categoriesFor: #connection:!accessing!private! !
!DatabaseAccessor categoriesFor: #connectionClass!accessing!private! !
!DatabaseAccessor categoriesFor: #connectionClassForLogin:!accessing!private! !
!DatabaseAccessor categoriesFor: #copy!copying!private! !
!DatabaseAccessor categoriesFor: #copyDataFromField:toField:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #copyDataFromTable:toTable:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #createSequence:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #createTable:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #createTableFKConstraints:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #createTableIndexes:ifError:!executing!public! !
!DatabaseAccessor categoriesFor: #currentLogin!accessing!private! !
!DatabaseAccessor categoriesFor: #currentLogin:!accessing!private! !
!DatabaseAccessor categoriesFor: #cursorFor:!cursors!private! !
!DatabaseAccessor categoriesFor: #cursorFor:command:!cursors!private! !
!DatabaseAccessor categoriesFor: #deniedCommands!accessing!private! !
!DatabaseAccessor categoriesFor: #deniedCommands:!accessing!private! !
!DatabaseAccessor categoriesFor: #denyCommand:!permissions!private! !
!DatabaseAccessor categoriesFor: #denyCommands:!permissions!private! !
!DatabaseAccessor categoriesFor: #doDDLOperation:!public!transactions! !
!DatabaseAccessor categoriesFor: #dropConstraint:!executing!public! !
!DatabaseAccessor categoriesFor: #dropConstraint:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #dropFields:!executing!public! !
!DatabaseAccessor categoriesFor: #dropFields:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #dropForeignKeyConstraintsForTable:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #dropSequence:ifAbsent:!executing!private! !
!DatabaseAccessor categoriesFor: #dropSequences:!executing!private! !
!DatabaseAccessor categoriesFor: #dropTable:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #dropTableDefinition:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #dropTableNamed:!executing!public! !
!DatabaseAccessor categoriesFor: #dropTableNamed:ifAbsent:!executing!private! !
!DatabaseAccessor categoriesFor: #dropTables:!executing!public! !
!DatabaseAccessor categoriesFor: #encoding!accessing!public! !
!DatabaseAccessor categoriesFor: #encoding:!accessing!public! !
!DatabaseAccessor categoriesFor: #errorsToCatch!executing!private! !
!DatabaseAccessor categoriesFor: #execute:!executing!private! !
!DatabaseAccessor categoriesFor: #execute:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #executeCommand:!executing!private! !
!DatabaseAccessor categoriesFor: #executeCommand:returnCursor:!executing!private! !
!DatabaseAccessor categoriesFor: #executeCommandBound:!executing!private! !
!DatabaseAccessor categoriesFor: #executeCommandUnbound:!executing!public! !
!DatabaseAccessor categoriesFor: #executeInDDLOperation:ifError:!private!transactions! !
!DatabaseAccessor categoriesFor: #executeSQLString:!executing!public! !
!DatabaseAccessor categoriesFor: #executeSQLString:binding:!executing!public! !
!DatabaseAccessor categoriesFor: #executeSQLStringNoResult:!executing!private! !
!DatabaseAccessor categoriesFor: #executeSQLStringNoResult:doing:forCommand:!executing!public! !
!DatabaseAccessor categoriesFor: #externalDatabaseErrorSignal!executing!private! !
!DatabaseAccessor categoriesFor: #handleError:for:!executing!public! !
!DatabaseAccessor categoriesFor: #initialize!initializing!private! !
!DatabaseAccessor categoriesFor: #innerExceptionFor:!executing!private! !
!DatabaseAccessor categoriesFor: #inTransactionDo:!private!transactions! !
!DatabaseAccessor categoriesFor: #isCursorAtEnd:!cursors!private! !
!DatabaseAccessor categoriesFor: #isExistingTable:inSchema:!public!testing! !
!DatabaseAccessor categoriesFor: #isInTransaction!private!transactions! !
!DatabaseAccessor categoriesFor: #listTableNamesInSchemaNamed:!public!queries metadata! !
!DatabaseAccessor categoriesFor: #log:!logging!public! !
!DatabaseAccessor categoriesFor: #logCommand:bound:!logging!public! !
!DatabaseAccessor categoriesFor: #logError:!logging!private! !
!DatabaseAccessor categoriesFor: #logger!logging!public! !
!DatabaseAccessor categoriesFor: #logger:!logging!public! !
!DatabaseAccessor categoriesFor: #logging!logging!private! !
!DatabaseAccessor categoriesFor: #logging:!logging!private! !
!DatabaseAccessor categoriesFor: #login!login!private! !
!DatabaseAccessor categoriesFor: #loginIfError:!login!private! !
!DatabaseAccessor categoriesFor: #logOnly!logging!public! !
!DatabaseAccessor categoriesFor: #logOnly:!logging!public! !
!DatabaseAccessor categoriesFor: #logout!login!private! !
!DatabaseAccessor categoriesFor: #logTime:!logging!public! !
!DatabaseAccessor categoriesFor: #myDependents!notification!private! !
!DatabaseAccessor categoriesFor: #myDependents:!notification!private! !
!DatabaseAccessor categoriesFor: #numberOfPreparedStatements!accessing!private! !
!DatabaseAccessor categoriesFor: #permitEverything!permissions!private! !
!DatabaseAccessor categoriesFor: #permitNothingDuring:!permissions!private! !
!DatabaseAccessor categoriesFor: #platform!accessing!private! !
!DatabaseAccessor categoriesFor: #populateField:with:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #postCopy!copying!private! !
!DatabaseAccessor categoriesFor: #renameField:to:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #renameTable:to:inPlatform:ifError:!executing!private! !
!DatabaseAccessor categoriesFor: #reset!initializing!private! !
!DatabaseAccessor categoriesFor: #resetPermissions!permissions!private! !
!DatabaseAccessor categoriesFor: #returnRowCount:!accessing!private! !
!DatabaseAccessor categoriesFor: #reusePreparedStatements!accessing!private! !
!DatabaseAccessor categoriesFor: #reusePreparedStatements:!accessing!private! !
!DatabaseAccessor categoriesFor: #rollbackTransaction!private!transactions! !
!DatabaseAccessor categoriesFor: #rowCount!executing!private! !
!DatabaseAccessor categoriesFor: #schema!accessing!private! !
!DatabaseAccessor categoriesFor: #showDialog:!login!private! !
!DatabaseAccessor categoriesFor: #useBinding!accessing!private! !

!DatabaseAccessor class methodsFor!

classForThisPlatform
	Dialect isDolphin ifTrue: [^Dialect smalltalkAt: #DolphinDatabaseAccessor].
	self error: 'unknown dialect'.
	^self!

forLogin: aLogin 
	^self classForThisPlatform new currentLogin: aLogin!

loggingEnabled
	LoggingEnabled isNil ifTrue: [LoggingEnabled := false].
	^LoggingEnabled!

loggingEnabled: aBoolean 
	LoggingEnabled := aBoolean!

new

	^super new initialize.! !
!DatabaseAccessor class categoriesFor: #classForThisPlatform!instance creation!private! !
!DatabaseAccessor class categoriesFor: #forLogin:!instance creation!private! !
!DatabaseAccessor class categoriesFor: #loggingEnabled!accessing!private! !
!DatabaseAccessor class categoriesFor: #loggingEnabled:!accessing!private! !
!DatabaseAccessor class categoriesFor: #new!instance creation!private! !

DatabaseCommand guid: (GUID fromString: '{4280CD82-4836-4AEC-BA8F-006E1975E8FA}')!
DatabaseCommand comment: 'A DatabaseCommand is what a query uses to actually generate its SQL and to execute it. A command wraps a stream, into which the SQL is written, but with knowledge of the platform for which it''s being written, and potentially details about how it should execute.

Subclasses must implement the following messages:
	accessing
		arrayBoundFields
		bindings
		bindingTypes
		parameterTypeSignature
	executing
		printSQL
	testing
		succeeded

Instance Variables:
	platform	<DatabasePlatform>	The database for which we''re to generate/execute SQL.
	sqlString	<String>	The resulting SQL.
	stream	<Stream>	The stream into which we generate the SQL. Once we''ve finished and got sqlString this isn''t necessary any more.
	useBinding	<Boolean>	Should we use binding, or print parameters directly into the string.

'!
!DatabaseCommand categoriesForClass!Unclassified! !
!DatabaseCommand methodsFor!

arrayBoundFields
	self subclassResponsibility!

bindings
	self subclassResponsibility.!

bindingTypes
	self subclassResponsibility.!

bindingTypesForEXDI
	^self bindingTypes collect: [:each | each exdiType].!

blockFactor
	^5  "A reasonable default if we don't know"!

canBind: aValue to: aType
	useBinding ifFalse: [^false].
	^self platform canBind: aValue to: aType underlyingType.!

contents
	^stream contents.!

cr
	stream cr.!

executeBoundIn: anAccessor
	^anAccessor executeCommandBound: self.!

executeUnboundIn: anAccessor
	^anAccessor executeCommandUnbound: self.!

hasMultipleRows
	^false.!

initialize
	useBinding := true.
	stream := WriteStream on: (String new: 100).!

isReadCommand
	^false.!

maximumLobSizeToRetrieveDirectly
	^32768.!

needsRowCount 

	^false.!

next: sizeInteger putAll: aSequenceableCollection startingAt: startInteger
	^stream next: sizeInteger putAll: aSequenceableCollection startingAt: startInteger!

nextPut: aCharacter
	^stream nextPut: aCharacter.!

nextPutAll: aString
	^stream nextPutAll: aString.!

nextPutString: aString
	^self nextPutAll: aString.!

parameterTypeSignature
	self subclassResponsibility.!

platform
	self session isNil ifTrue: [^nil].
	^self session platform.!

platform: aDatabasePlatform
	self error: 'This API is obsolete. Pass the session instead'.!

policy
	^stream policy.!

print: anObject

	anObject printOn: self!

printSQL
	self subclassResponsibility.!

query
	^nil.!

session
	^session.!

session: aGlorpSession
	session := aGlorpSession.!

shouldUseGroupWriting
	^(self hasMultipleRows and: [self platform supportsGroupWritingFor: self])!

signature
	^self sqlString, self parameterTypeSignature.!

space
	stream space.!

sqlString
	sqlString isNil ifTrue: [
		stream isNil ifTrue: [stream := WriteStream on: (String new: 100)].
		self printSQL.
		sqlString := stream contents.
		stream := nil].
	^sqlString.!

succeeded

	^self subclassResponsibility.!

system
	self session isNil ifTrue: [^nil].
	^self session system.!

tables
	"Return a list of the tables we will affect"
	^self subclassResponsibility.!

useBinding
	^useBinding!

useBinding: aBoolean
	useBinding := aBoolean! !
!DatabaseCommand categoriesFor: #arrayBoundFields!accessing!private! !
!DatabaseCommand categoriesFor: #bindings!accessing!private! !
!DatabaseCommand categoriesFor: #bindingTypes!accessing!private! !
!DatabaseCommand categoriesFor: #bindingTypesForEXDI!accessing!private! !
!DatabaseCommand categoriesFor: #blockFactor!accessing!private! !
!DatabaseCommand categoriesFor: #canBind:to:!private!testing! !
!DatabaseCommand categoriesFor: #contents!private!stream behaviour! !
!DatabaseCommand categoriesFor: #cr!private!stream behaviour! !
!DatabaseCommand categoriesFor: #executeBoundIn:!executing!private! !
!DatabaseCommand categoriesFor: #executeUnboundIn:!executing!private! !
!DatabaseCommand categoriesFor: #hasMultipleRows!private!testing! !
!DatabaseCommand categoriesFor: #initialize!initializing!private! !
!DatabaseCommand categoriesFor: #isReadCommand!private!testing! !
!DatabaseCommand categoriesFor: #maximumLobSizeToRetrieveDirectly!accessing!private! !
!DatabaseCommand categoriesFor: #needsRowCount!private!testing! !
!DatabaseCommand categoriesFor: #next:putAll:startingAt:!public!stream behavior! !
!DatabaseCommand categoriesFor: #nextPut:!private!stream behaviour! !
!DatabaseCommand categoriesFor: #nextPutAll:!private!stream behaviour! !
!DatabaseCommand categoriesFor: #nextPutString:!private!stream behaviour! !
!DatabaseCommand categoriesFor: #parameterTypeSignature!accessing!private! !
!DatabaseCommand categoriesFor: #platform!As yet unclassified!private! !
!DatabaseCommand categoriesFor: #platform:!accessing!public! !
!DatabaseCommand categoriesFor: #policy!private!stream behaviour! !
!DatabaseCommand categoriesFor: #print:!private!stream behaviour! !
!DatabaseCommand categoriesFor: #printSQL!executing!private! !
!DatabaseCommand categoriesFor: #query!accessing!private! !
!DatabaseCommand categoriesFor: #session!accessing!public! !
!DatabaseCommand categoriesFor: #session:!As yet unclassified!public! !
!DatabaseCommand categoriesFor: #shouldUseGroupWriting!public!testing! !
!DatabaseCommand categoriesFor: #signature!accessing!private! !
!DatabaseCommand categoriesFor: #space!private!stream behaviour! !
!DatabaseCommand categoriesFor: #sqlString!accessing!private! !
!DatabaseCommand categoriesFor: #succeeded!private!testing! !
!DatabaseCommand categoriesFor: #system!accessing!public! !
!DatabaseCommand categoriesFor: #tables!private!reporting! !
!DatabaseCommand categoriesFor: #useBinding!accessing!private! !
!DatabaseCommand categoriesFor: #useBinding:!accessing!private! !

!DatabaseCommand class methodsFor!

new
	^super new initialize.! !
!DatabaseCommand class categoriesFor: #new!instance creation!private! !

DatabaseConverter guid: (GUID fromString: '{D7B7D071-9051-499E-9B0D-046089DD7B28}')!
DatabaseConverter comment: 'This is a superclass for objects that convert back and forth between database representations and object representations.

Subclasses must implement the following messages:
	converting
		convert:fromDatabaseRepresentationAs:
		convert:toDatabaseRepresentationAs:

Instance Variables:
	name	<ByteSymbol>	The name for this converter. The name is used, concatenated with #Converter, to create a symbol that can be performed against the DatabasePlatform to give this converter. So, e.g. booleanToIntegerConverter.

'!
!DatabaseConverter categoriesForClass!Unclassified! !
!DatabaseConverter methodsFor!

convert: anObject fromDatabaseRepresentationAs: aDatabaseType 
	self subclassResponsibility!

convert: anObject toDatabaseRepresentationAs: aDatabaseType 
	self subclassResponsibility!

initialize
	name := #unnamed.!

name
	^name.!

name: aString
	name := aString.!

printOn: aString
	aString nextPutAll: 'DatabaseConverter(', name, ')'.! !
!DatabaseConverter categoriesFor: #convert:fromDatabaseRepresentationAs:!converting!private! !
!DatabaseConverter categoriesFor: #convert:toDatabaseRepresentationAs:!converting!private! !
!DatabaseConverter categoriesFor: #initialize!initialize!private! !
!DatabaseConverter categoriesFor: #name!accessing!private! !
!DatabaseConverter categoriesFor: #name:!accessing!private! !
!DatabaseConverter categoriesFor: #printOn:!printing!private! !

!DatabaseConverter class methodsFor!

new

	^super new initialize.! !
!DatabaseConverter class categoriesFor: #new!instance creation!private! !

DatabaseField guid: (GUID fromString: '{BB766F0A-D9EE-477F-8910-74B833DFED90}')!
DatabaseField comment: 'This represents a field in a database table. See also DatabaseTable. Glorp relies heavily on identity of fields for comparison purposes, so be careful when copying these.

Instance Variables:
	defaultValue	<Object>	What is the default value of this field, if unspecified in the creation statement.
	isLockKey	<Boolean>	Is this field part of an optimistic lock key. If so, then on update we will generate an update statement that checks that this value hasn''t changed since the object wasn''t read.
	isNullable	<Boolean>	Is this field allowed to be null?
	isPrimaryKey	<Boolean>	Is this field part of the primary key?
	isUnique	<Boolean>	Does this field have a unique constraint in the database?
	name	<String>	The name of the field.
	position	<Object | ProtoObject>	description of position
	table	<DatabaseTable>	The table that contains this field.
	type	<GlorpDatabaseType>	The type of  data that this field contains.

'!
!DatabaseField categoriesForClass!Unclassified! !
!DatabaseField methodsFor!

<= aField
	^self name <= aField name!

asConstraintReferenceString

	^table name, ' (', self name, ')'.!

asDistinctFieldOn: anExpression

	^(anExpression getField: self) asDistinctFieldOn: anExpression.!

asField
	^self.!

asGlorpExpression

	^ParameterExpression forField: self basedOn: nil.!

asGlorpExpressionOn: anExpression

	^ParameterExpression forField: self basedOn: anExpression.!

beIndexed
	self table addIndex: (DatabaseIndex forField: self).!

beLockKey

	isLockKey := true.
	self table isNil ifFalse: [self table addAsLockKeyField: self]!

beNullable: aBoolean

	self isPrimaryKey ifFalse: [ isNullable := aBoolean ]!

bePrimaryKey
	isPrimaryKey := true.
	isNullable := false.
	self table isNil ifFalse: [self table addAsPrimaryKeyField: self]!

converterForStType: aClass

	^self type converterForStType: (aClass isBehavior ifTrue: [aClass] ifFalse: [aClass class]).!

defaultValue
	^defaultValue!

defaultValue: anObject
	defaultValue := anObject!

foreignKeyConstraints
	^self table foreignKeyConstraints select: [:each | each sourceFields includes: self].!

impliedSmalltalkType
	"Return the default Smalltalk type corresponding to our database type"

	^self type impliedSmalltalkType.!

initialize
	isPrimaryKey := false.
	isNullable := true.
	isUnique := false.
	isLockKey := false.!

isDatabaseField
	^true!

isDistinct
	^false.!

isGenerated

	^type isGenerated or: [defaultValue notNil].!

isIndexed
	^self table hasIndexForField: self.!

isLockKey
	isLockKey isNil ifTrue: [isLockKey := false].
	^isLockKey!

isMappable
	"Return true if we're something that can be returned in a select statement. Always true for fields"
	^true.!

isNullable
	
	^isNullable.!

isPrimaryKey
	"Private - Answer the value of the receiver's ''isPrimaryKey'' instance variable."

	^isPrimaryKey yourSelf.!

isSameAliasedFieldAs: aField
	"Return true if we are the same as another aliased field. For non-aliased fields, identity is assured, so this is redundant. Note that we == the string. This will work because we know these must both be shallow copies of the same thing"
	self class == aField class ifFalse: [^false].
	^self table == aField table and: [self name == aField name].!

isTheSameAs: aDatabaseField
	"Validate that two fields are exactly the same. Most useful for testing purposes, rather than normal usage, which is why we don't just use the #= method."
	self name asUppercase = aDatabaseField name asUppercase ifFalse: [^false].
	self isPrimaryKey = aDatabaseField isPrimaryKey ifFalse: [^false].
	self isTheSameTypeAs: aDatabaseField.
	self isNullable = aDatabaseField isNullable ifFalse: [^false].
	self isUnique = aDatabaseField isUnique ifFalse: [^false].
	self isLockKey = aDatabaseField isLockKey ifFalse: [^false].
	self defaultValue = aDatabaseField defaultValue ifFalse: [^false].
	^true.!

isTheSameTypeAs: aDatabaseField
	| myType aType |
	self isPrimaryKey
		ifTrue:
		   [myType := self type.
		   aType := aDatabaseField type.
		   ^(myType = aType 
				or: [(myType isSerial or: [aType isSerial ]) 
					and: [myType impliedSmalltalkType = aType impliedSmalltalkType ]]) ].
		
	^self type = aDatabaseField type!

isUnique
	isUnique isNil ifTrue: [isUnique := false].
	^isUnique.!

isUnique: aBoolean
	isUnique := aBoolean.!

name
	"Private - Answer the value of the receiver's ''name'' instance variable."

	^name!

name: aString
	"Private - Set the value of the receiver's ''name'' instance variable to the argument, anObject."

	name := aString asString.!

platform
	^self type platform.!

position
	^position!

position: anObject
	position := anObject!

postInitializeIn: aDescriptorSystem
	"Any initialization that has to be delayed until we're in the table"
	type initializeForField: self in: aDescriptorSystem.!

printForConstraintNameOn: aStream maxLength: maxLength 
	| constraintName |
	constraintName := table name , '_' , name.
	constraintName size > maxLength 
		ifTrue: [constraintName := constraintName copyFrom: 1 to: maxLength].
	aStream nextPutAll: constraintName!

printNameOn: aStream withParameters: anArray 
	aStream nextPutAll: self name!

printOn: aStream
	
	aStream nextPutAll: 'Field'.
	aStream 
		nextPutAll: '(';
		nextPutAll: (table isNil ifTrue: [''] ifFalse: [table name]);
		nextPutAll: '.';
		nextPutAll: name;
		nextPutAll: ')'.!

printQualifiedSQLOn: aStream withParameters: aDictionary 
	aStream nextPutAll: self qualifiedName!

printSelectSQLOn: aStream withParameters: anArray 
	self printSQLOn: aStream withParameters: anArray.!

printSQLOn: aStream withParameters: anArray 
	aStream nextPutAll: self quotedQualifiedName.!

printUnqualifiedSQLOn: aStream withParameters: anArray 
	aStream nextPutAll: self name.!

qualifiedName

	^table isNil 
		ifTrue: [self name]
		ifFalse: [self table qualifiedName, '.', self name].!

quotedQualifiedName

	| basicName |
	basicName := self platform nameForColumn: self name.
	^table isNil 
		ifTrue: [basicName]
		ifFalse: [self table qualifiedName, '.', basicName].!

table
	^table!

table: anObject

	table := anObject!

tablesToPrint
	^Array with: table!

type

	^type!

type: aDatabaseType

	type := aDatabaseType.
	(type notNil and: [type isSerial]) ifTrue: [self bePrimaryKey].!

typeString

	^type typeString!

withTable: aTable
	"Return a copy of ourselves, with the table set to aTable. Presumably aTable is an aliased version of our table"
	^self copy table: aTable.! !
!DatabaseField categoriesFor: #<=!comparing!private! !
!DatabaseField categoriesFor: #asConstraintReferenceString!obsolete!private! !
!DatabaseField categoriesFor: #asDistinctFieldOn:!converting!private! !
!DatabaseField categoriesFor: #asField!converting!private! !
!DatabaseField categoriesFor: #asGlorpExpression!converting!private! !
!DatabaseField categoriesFor: #asGlorpExpressionOn:!converting!private! !
!DatabaseField categoriesFor: #beIndexed!configuring!private! !
!DatabaseField categoriesFor: #beLockKey!configuring!private! !
!DatabaseField categoriesFor: #beNullable:!configuring!private! !
!DatabaseField categoriesFor: #bePrimaryKey!configuring!private! !
!DatabaseField categoriesFor: #converterForStType:!converting!private! !
!DatabaseField categoriesFor: #defaultValue!accessing!private! !
!DatabaseField categoriesFor: #defaultValue:!configuring!private! !
!DatabaseField categoriesFor: #foreignKeyConstraints!accessing!private! !
!DatabaseField categoriesFor: #impliedSmalltalkType!accessing!private! !
!DatabaseField categoriesFor: #initialize!initializing!private! !
!DatabaseField categoriesFor: #isDatabaseField!As yet unclassified!private! !
!DatabaseField categoriesFor: #isDistinct!private!testing! !
!DatabaseField categoriesFor: #isGenerated!private!testing! !
!DatabaseField categoriesFor: #isIndexed!private!testing! !
!DatabaseField categoriesFor: #isLockKey!private!testing! !
!DatabaseField categoriesFor: #isMappable!private!testing! !
!DatabaseField categoriesFor: #isNullable!private!testing! !
!DatabaseField categoriesFor: #isPrimaryKey!private!testing! !
!DatabaseField categoriesFor: #isSameAliasedFieldAs:!copying!private! !
!DatabaseField categoriesFor: #isTheSameAs:!comparing!private! !
!DatabaseField categoriesFor: #isTheSameTypeAs:!comparing!private! !
!DatabaseField categoriesFor: #isUnique!private!testing! !
!DatabaseField categoriesFor: #isUnique:!configuring!private! !
!DatabaseField categoriesFor: #name!accessing!private! !
!DatabaseField categoriesFor: #name:!accessing!private! !
!DatabaseField categoriesFor: #platform!accessing!private! !
!DatabaseField categoriesFor: #position!accessing!private! !
!DatabaseField categoriesFor: #position:!accessing!private! !
!DatabaseField categoriesFor: #postInitializeIn:!initializing!private! !
!DatabaseField categoriesFor: #printForConstraintNameOn:maxLength:!obsolete!private! !
!DatabaseField categoriesFor: #printNameOn:withParameters:!obsolete!private! !
!DatabaseField categoriesFor: #printOn:!printing!private! !
!DatabaseField categoriesFor: #printQualifiedSQLOn:withParameters:!obsolete!private! !
!DatabaseField categoriesFor: #printSelectSQLOn:withParameters:!printing!private! !
!DatabaseField categoriesFor: #printSQLOn:withParameters:!printing!private! !
!DatabaseField categoriesFor: #printUnqualifiedSQLOn:withParameters:!printing!private! !
!DatabaseField categoriesFor: #qualifiedName!private!querying! !
!DatabaseField categoriesFor: #quotedQualifiedName!private!querying! !
!DatabaseField categoriesFor: #table!accessing!private! !
!DatabaseField categoriesFor: #table:!accessing!private! !
!DatabaseField categoriesFor: #tablesToPrint!accessing!private! !
!DatabaseField categoriesFor: #type!accessing!private! !
!DatabaseField categoriesFor: #type:!configuring!private! !
!DatabaseField categoriesFor: #typeString!database!private! !
!DatabaseField categoriesFor: #withTable:!copying!private! !

!DatabaseField class methodsFor!

named: aString

	^self error: 'type needed'!

named: aString type: dbType

	^super new initialize
		name: aString;
		type: dbType!

new

	^self error: 'dbType needed'! !
!DatabaseField class categoriesFor: #named:!instance creation!private! !
!DatabaseField class categoriesFor: #named:type:!instance creation!private! !
!DatabaseField class categoriesFor: #new!instance creation!private! !

DatabaseIndex guid: (GUID fromString: '{414BF203-18DD-44B1-9CCC-2A6D21C94C40}')!
DatabaseIndex comment: 'This describes an index in the database. Glorp doesn''t use it internally, other than when creating tables in the database.

Instance Variables:
	fields	<(SequenceableCollection of: DatabaseField)>	The set of fields that are to be indexed.
	name	<String>	The name of the index. We will try and generate one based on the table if one is not supplied.

'!
!DatabaseIndex categoriesForClass!Unclassified! !
!DatabaseIndex methodsFor!

addField: aField
	fields add: aField!

addFields: aCollection
	fields addAll: aCollection!

fields
	^fields.!

generateName
	"Generate a name for this index that will fit within the size limits of the database. This only takes into account the first field being used. If there's more than one that has multiple columns and the same first one, then the user will have to manually provide a name"

	| stream maxLength tableLength roomToBeVerbose minTableLength maxTableLength |
	maxLength := fields first table platform maximumLengthOfTableName.
	minTableLength := ((maxLength - 8) // 2) min: 10.
	maxTableLength := (maxLength // 2) - 1.
	roomToBeVerbose := minTableLength >= 10.
	tableLength := roomToBeVerbose ifTrue: [maxTableLength] ifFalse: [minTableLength].

	stream := WriteStream on: (String new: 100).
	stream nextPutAll: (fields first table name copyFrom: 1 to: ((fields first table name size) min: tableLength)).
	stream nextPutAll: '_'.
	stream nextPutAll: (fields first name copyFrom: 1 to: ((fields first name size) min: tableLength)).
	stream nextPutAll: (maxLength - stream position >= 4 ifTrue: ['_IDX'] ifFalse: ['X']).
	^stream contents.!

indexesField: aField
	^fields includes: aField.!

initialize
	fields := OrderedCollection new: 2.!

isTheSameAs: aDatabaseIndex
	"Validate that two indexes are exactly the same. Most useful for testing purposes, rather than normal usage, which is why we don't just use the #= method."
	self name asUppercase = aDatabaseIndex name asUppercase ifFalse: [^false].
	self fields with: aDatabaseIndex fields  do: [:a :b | (a isTheSameAs: b) ifFalse: [^false]].	
	^true.!

name
	name isNil ifTrue: [name := self generateName].
	^name.!

name: aString
	name := aString.!

printStatementOn: aStream
	aStream nextPut: $(.
	fields do: [ :f | aStream nextPutAll: f name; nextPut: $, ].
	aStream skip: -1; nextPut: $)! !
!DatabaseIndex categoriesFor: #addField:!initializing!private! !
!DatabaseIndex categoriesFor: #addFields:!initializing!private! !
!DatabaseIndex categoriesFor: #fields!accessing!private! !
!DatabaseIndex categoriesFor: #generateName!printing!private! !
!DatabaseIndex categoriesFor: #indexesField:!private!testing! !
!DatabaseIndex categoriesFor: #initialize!initializing!private! !
!DatabaseIndex categoriesFor: #isTheSameAs:!comparing!private! !
!DatabaseIndex categoriesFor: #name!accessing!public! !
!DatabaseIndex categoriesFor: #name:!accessing!private! !
!DatabaseIndex categoriesFor: #printStatementOn:!printing!private! !

!DatabaseIndex class methodsFor!

forField: aField
	^(self new initialize)
		addField: aField;
		yourself!

forFields: aCollection
	^(self new)
		addFields: aCollection;
		yourself!

new
	^super new initialize.! !
!DatabaseIndex class categoriesFor: #forField:!instance creation!private! !
!DatabaseIndex class categoriesFor: #forFields:!instance creation!private! !
!DatabaseIndex class categoriesFor: #new!instance creation!private! !

DatabasePlatform guid: (GUID fromString: '{7F4523CF-5D9C-4AD5-95A0-2CED8C02F851}')!
DatabasePlatform comment: 'This is an abstract superclass for classes that provide a compatibility layer for different databases. They encapsulate different SQL syntaxes, binding mechanism, types that are available, and general limits.

Subclasses must implement the following messages:
	accessing
		vwEXDIConnectionClass
	binding
		bindingsForGroupWritingFor:
	constants
		areSequencesExplicitlyCreated
		supportsMillisecondsInTimes
	sequences
		databaseSequenceClass
	testing
		usesArrayBindingRatherThanGrouping
	types
		int4
		sequence
		serial
		timestamp
		varbinary
		varchar

Instance Variables:
	characterEncoding	<ByteSymbol>	What character encoding should we use for the database connection.
	converters	<Dictionary from: Symbol to: DatabaseConverter>	The available type converters. So, for example, if there is a boolean to integer conversion, we expect it in this dictionary associated with the symbol #booleanToInteger. If that symbol isn''t there, we will look up the converter by performing #booleanToIntegerConverter and caching the result.
	functions	<FunctionExpression>	Which functions are available for this database. There is a set of common functions created by FunctionExpression, plus we can define our own additions or overrides that are specific to this database.
	reservedWords	<Collection of: String>	Which names are reserved words in this database. Any that we use as table or column names will need to be quoted.
	types	<Dictionary from: Symbol to: GlorpDatabaseType>	A cache of available types, mapped by name to the type instance.
	useBinding	<Boolean>	By default, should we use binding for this database.

'!
!DatabasePlatform categoriesForClass!Unclassified! !
!DatabasePlatform methodsFor!

areSequencesExplicitlyCreated

	self subclassResponsibility.!

asTimestamp: anObject for: aType
	anObject isNil ifTrue: [^nil].
	^anObject class = Time
		ifTrue: [Dialect newTimestampFromDate: Date today time: anObject]
		ifFalse: [anObject asTimestamp].!

batchWriteStatementTerminatorString
	"^<String> This statement return the string to be used to devide several statement during batch write ..."

	^';'!

bindingsForGroupWritingFor: aCommand
	"Return the bindings array for a group write. This can be in different formats, depending on the database and perhaps the mechanism in place."
	self subclassResponsibility.!

blob
	^self typeNamed: #blob ifAbsentPut: [GlorpBlobType new].!

boolean
	"By default, we represent booleans as small integers."
	^self smallint.!

booleanToBooleanConverter
	^DelegatingDatabaseConverter
		named: #booleanToBoolean
		hostedBy: self
		fromStToDb: #convertBooleanToDBBoolean:for:
		fromDbToSt: #convertDBBooleanToBoolean:for:.!

booleanToIntegerConverter
	^DelegatingDatabaseConverter
		named: #booleanToInteger
		hostedBy: self
		fromStToDb: #convertBooleanToInteger:for:
		fromDbToSt: #convertIntegerToBoolean:for:.!

booleanToStringTFConverter
	^DelegatingDatabaseConverter
		named: #booleanToString
		hostedBy: self
		fromStToDb: #convertBooleanToTF:for:
		fromDbToSt: #convertTFToBoolean:for:.!

booleanToStringYesNoConverter
	^DelegatingDatabaseConverter
		named: #booleanToString
		hostedBy: self
		fromStToDb: #convertBooleanToYesNo:for:
		fromDbToSt: #convertYesNoToBoolean:for:.!

booleanToStringYNConverter
	^DelegatingDatabaseConverter
		named: #booleanToString
		hostedBy: self
		fromStToDb: #convertBooleanToYN:for:
		fromDbToSt: #convertYNToBoolean:for:.!

byteArrayToString: aByteArray for: aType
	^aByteArray isNil ifTrue: [nil] ifFalse: [Dialect convert: aByteArray toStringWithEncoding: self characterEncoding].!

byteArrayToStringConverter
	^DelegatingDatabaseConverter
		named: #byteArrayToString
		hostedBy: self
		fromStToDb: #convertStringToByteArray:for:
		fromDbToSt: #convertByteArrayToString:for:.!

canBind: aValue to: aType
	(Dialect unbindableClassNames includes: aValue class name) ifTrue: [^false].
	^true.!

canUseInWhereClause: aValue type: aType
	(Dialect isFloatingPoint: aValue) ifTrue: [^false].
	^true.!

capitalWritingOfColumnName
	"^<Boolean> This method returns true, if the dbms wants to have column 
	names written in capital letters"

	^false!

capitalWritingOfCreatorName
	"^<Boolean> This method returns true, if the dbms wants to have column 
	names written in capital letters"

	^true!

capitalWritingOfSQLCommands
	"^<Boolean>"

	^true!

capitalWritingOfTableName
	"^<Boolean>"

	^true!

char
	^self typeNamed: #char ifAbsentPut: [GlorpCharType new].!

char: anInteger
	^self char width: anInteger.!

character
	^self char!

characterEncoding
	"We hope that this will be set appropriately for the connection, but make sure the default is fairly harmless, and will in most cases complain rather than write characters incorrectly"
	characterEncoding isNil ifTrue: [characterEncoding := #ascii].
	^characterEncoding!

characterEncoding: anEncodingName
	characterEncoding := anEncodingName.!

charactersThatNeedEscaping
	^#($' )!

clob
	^self typeNamed: #clob ifAbsentPut: [GlorpClobType new].!

columnNameSeparatorString
	"^<String> This statement return the string to be used to devide several columns ..."

	^','!

compoundOperationFor: aSymbol
	"Return the platform specific version of a compound statement symbol"
	^aSymbol.!

convertBooleanToDBBoolean: aBoolean for: aType
	(self usesNullForFalse and: [aBoolean isNil]) ifTrue: [^false].
	^aBoolean.!

convertBooleanToInteger: aBoolean for: aType
	aBoolean isNil ifTrue: [^nil].
	^aBoolean ifTrue: [1] ifFalse: [0].!

convertBooleanToTF: aBoolean for: aType
	aBoolean isNil ifTrue: [^aBoolean].
	^aBoolean ifTrue: ['T'] ifFalse: ['F'].!

convertBooleanToYesNo: aBoolean for: aType
	aBoolean isNil ifTrue: [^aBoolean].
	^aBoolean ifTrue: ['YES'] ifFalse: ['NO'].!

convertBooleanToYN: aBoolean for: aType
	aBoolean isNil ifTrue: [^aBoolean].
	^aBoolean ifTrue: ['Y'] ifFalse: ['N'].!

convertByteArrayToSQLString: aByteArray for: aType
	"This is a slightly misleading name. Really what we want is conversion to a Blob-type of field. So if it's a bytearray, we can just let the driver take care of that conversion, but if we're given a string, we should convert it."

	^aByteArray isString
		ifTrue:
			[Dialect
				convert: aByteArray
				toByteArrayWithEncoding: self characterEncoding]
		ifFalse: [aByteArray].!

convertByteArrayToString: aByteArray for: aType
	^aByteArray asString.!

convertDBBooleanToBoolean: aBoolean for: aType
	^aBoolean.!

converterNamed: aSymbol

	^self converters at: aSymbol ifAbsentPut: [
		| converter |
		converter := self perform: (aSymbol, 'Converter') asSymbol.
		converter name: aSymbol]!

converters
	converters isNil ifTrue: [converters := IdentityDictionary new].
	^converters.!

convertIntegerToBoolean: anInteger for: aType
	anInteger isNil ifTrue: [^anInteger].
	anInteger = 1 ifTrue: [^true].
	anInteger = 0 ifTrue: [^false].
	self error: 'invalid boolean conversion'.!

convertStringToByteArray: aString for: aType
	^aString asByteArray.!

convertTFToBoolean: aString for: aType
	aString isNil ifTrue: [^aString].
	aString = 'T' ifTrue: [^true].
	aString = 'F' ifTrue: [^false].
	self error: 'invalid boolean conversion'.!

convertToDouble: aNumber for: type 

	aNumber isNil ifTrue: [^nil].
	^Dialect coerceToDoublePrecisionFloat: aNumber!

convertToFloat: aNumber for: aType 
	aNumber isNil ifTrue: [^nil].
	^aNumber asFloat.!

convertToInteger: aNumber for: aType 
	^aNumber isNil ifTrue: [aNumber] ifFalse: [aNumber asNumber asInteger].!

convertToNumber: aNumber for: aType
	"Convert to an arbitrary numeric type. Make sure our precision matches. *Hate* floating point"

	| exponent result |
	aNumber isNil ifTrue: [^nil].
	aType precision isNil ifTrue: [^aNumber].
	(Dialect doesPrecisionOf: aNumber equal: aType precision) ifTrue: [^aNumber].
	exponent := Dialect
		coerceToDoublePrecisionFloat: (10 raisedToInteger: aType precision).
	result := (aNumber * exponent) rounded / exponent.
	^aNumber isInteger ifTrue: [result rounded] ifFalse: [result].!

convertYesNoToBoolean: aString for: aType
	aString isNil ifTrue: [^aString].
	aString asUppercase = 'YES' ifTrue: [^true].
	aString asUppercase = 'NO' ifTrue: [^false].
	self error: 'invalid boolean conversion'.!

convertYNToBoolean: aString for: aType
	aString isNil ifTrue: [^aString].
	aString = 'Y' ifTrue: [^true].
	aString = 'N' ifTrue: [^false].
	self error: 'invalid boolean conversion'.!

createAddColumnStatementFor: aField
	| stream |
	stream := WriteStream on: String new.
	stream 
		nextPutAll: 'ALTER TABLE';
		space.
	self printDDLTableNameFor: aField table on: stream.
	stream
		space;
		nextPutAll: ' ADD ';
		nextPutAll: self startColumnAddString.
	self  printColumn: aField on: stream.
	stream nextPutAll: self  endColumnAddString.
		
	^stream contents!

createAlterColumnStatement: aField newType: aType

	^self createAlterColumnStatement: aField newType: aType usingExpression: nil!

createAlterColumnStatement: aField newType: aType usingExpression: expression

	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
	self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' ALTER COLUMN';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll:  ' TYPE ';
		nextPutAll: aType typeString.
	expression ifNotNil: [ sqlStatementStream nextPutAll: ' USING ', expression ].

	^sqlStatementStream contents!

createAlterColumnStatement: aField setDefault: anObject
	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
	self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' ALTER COLUMN';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll:  'SET DEFAULT';
		space.
	anObject printOn: sqlStatementStream.
	
	^sqlStatementStream contents!

createAlterColumnStatementDropDefault: aField
	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
		self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' ALTER COLUMN';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll:  'DROP DEFAULT '.

	^sqlStatementStream contents!

createAlterColumnStatementDropNotNull: aField
	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
		self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' ALTER COLUMN';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll:  'DROP';
		space;
		nextPutAll:  self sqlTextForNOTNULLAttributeConstraint.

	
	^sqlStatementStream contents!

createAlterColumnStatementSetNotNull: aField
	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
		self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' ALTER COLUMN';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll:  'SET';
		space;
		nextPutAll:  self sqlTextForNOTNULLAttributeConstraint.

	
	^sqlStatementStream contents!

createConstraintAddStatement:  aConstraint
	| sqlStatementStream |
	self supportsAlterTableForConstraints ifFalse: [^''].
	aConstraint shouldCreateInDatabase ifFalse: [^''].
	self supportsConstraints
			ifTrue: [sqlStatementStream := WriteStream on: String new.
					sqlStatementStream
						nextPutAll:  'ALTER TABLE';
						space.
					self printDDLTableNameFor: aConstraint table on: sqlStatementStream.
					(aConstraint creationStringFor: self)
						do: [ :each | sqlStatementStream nextPutAll: ' ADD ', each.]
						separatedBy: [sqlStatementStream nextPut: $,].
					^sqlStatementStream contents].
	^String new!

createConstraintDropStatement: aConstraint
	| aStream |
	self supportsAlterTableForConstraints ifFalse: [^''].
	aConstraint shouldCreateInDatabase ifFalse: [^''].
	aStream := WriteStream on: (String new: 50).
	aStream nextPutAll: 'ALTER TABLE '.
	self printDDLTableNameFor: aConstraint table on: aStream.
	aStream nextPutAll: ' DROP '.
	self writeConstraintToDrop: aConstraint onStream: aStream.
	^aStream contents.!

createDropColumnStatementFor: aField
	| stream |
	stream := WriteStream on: String new.
	stream 
		nextPutAll: 'ALTER TABLE';
		space.
	self printDDLTableNameFor: aField table  on: stream.
	stream
		space;
		nextPutAll: 'DROP COLUMN';
		space;
		nextPutAll: (self nameForColumn: aField name).
		
	^stream contents!

createDropTableStatementFor: aTableNameString
	"^<String> This method returns a string which can be used to drop a database table ..."

	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll: 'DROP TABLE';
		space;
		nextPutAll: aTableNameString.

	^sqlStatementStream contents!

createForeignKeyConstraintAddStatement: aForeignKeyConstraint
	"Callback from the ForeignKeyConstraint. Returns collection of statements "
	| ws |
	aForeignKeyConstraint shouldCreateInDatabase ifFalse: [^#('')].
	ws := WriteStream on: (String new: 50).
	ws
		nextPutAll: 'CONSTRAINT ';
		nextPutAll: aForeignKeyConstraint name;
		nextPutAll: ' FOREIGN KEY ('.
	GlorpHelper print: [:each | each name] on: ws for: aForeignKeyConstraint sourceFields separatedBy: ','.
	ws 	nextPutAll: ') REFERENCES ';
		nextPutAll: aForeignKeyConstraint targetTable qualifiedName;
		nextPutAll: ' ('.
	GlorpHelper print: [:each | each name] on: ws for: aForeignKeyConstraint targetFields separatedBy: ','.
	ws nextPutAll: ')'.
	aForeignKeyConstraint suffixExpression isNil
		ifFalse:
			[ws
				space;
				nextPutAll: aForeignKeyConstraint suffixExpression].

	^OrderedCollection with: ws contents.!

createPrimaryKeyConstraintAddStatement: aPrimaryKeyConstraint
	"Callback from the PrimaryKeyConstraint. Returns collection of statements "
	| aStream statements |
	aPrimaryKeyConstraint shouldCreateInDatabase ifFalse: [^''].
	statements := OrderedCollection new.
	aStream := WriteStream on: String new.
	aStream nextPutAll: 'CONSTRAINT '.
	aStream nextPutAll: aPrimaryKeyConstraint name.
	aStream nextPutAll: ' PRIMARY KEY  ('.
	GlorpHelper 
		print: [:each | each name]
		on: aStream
		for: aPrimaryKeyConstraint primaryKeyFields
		separatedBy: ','.
	aStream nextPut: $).
	statements add: aStream contents.
	self primaryKeysAreAutomaticallyUnique ifTrue: [^statements].

	aStream reset.
	aStream nextPutAll: ' CONSTRAINT '.
	aStream nextPutAll: (self usesPrimaryKeyUniqueConstraintNames 
							ifTrue: [aPrimaryKeyConstraint table name, '_UNIQ'] 
							ifFalse: ['']).
	aStream nextPutAll: ' UNIQUE  ('.
	GlorpHelper 
		print: [:each | each name]
		on: aStream
		for: aPrimaryKeyConstraint  primaryKeyFields
		separatedBy: ','.
	aStream nextPut: $).
	statements add: aStream contents.

	^statements!

createTableFKContraintsStatementStringsFor: aGLORPDatabaseTable

	^aGLORPDatabaseTable foreignKeyConstraintsForCreation
		collect: [:each |	self createConstraintAddStatement: each ]!

createTableIndexStatementStringsFor: aGLORPDatabaseTable
	^aGLORPDatabaseTable indexes
		collect:
			[:idx | 
			| sqlStatementStream |
			sqlStatementStream := WriteStream on: String new.
			sqlStatementStream nextPutAll: 'CREATE INDEX '.
			sqlStatementStream nextPutAll: idx name.
			sqlStatementStream nextPutAll: ' ON '.
			self printDDLTableNameFor: aGLORPDatabaseTable on: sqlStatementStream.
			idx printStatementOn: sqlStatementStream.
			sqlStatementStream contents].!

createTableStatementStringFor: aGLORPDatabaseTable 
	"^<String> This method returns a string which can be used to create a database table ..."

	| sqlStatementStream tmpString |
	tmpString := 'create table'.
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll: (self capitalWritingOfSQLCommands 
			ifTrue: [tmpString asUppercase]
			ifFalse: [tmpString]);
		space.
	self printDDLTableNameFor: aGLORPDatabaseTable on: sqlStatementStream.

	"Now print the columns specification for each field in the table ..."
	self 
		printColumnsSpecificationFor: aGLORPDatabaseTable
		on: sqlStatementStream.
	self supportsConstraints ifTrue:
		[aGLORPDatabaseTable hasPrimaryKeyConstraints ifTrue: 
			[sqlStatementStream nextPutAll: ', '.
			(aGLORPDatabaseTable primaryKeyConstraints creationStringFor: self)
				do: [ :each | sqlStatementStream nextPutAll: each.]
				separatedBy: [sqlStatementStream nextPut: $,]].

		aGLORPDatabaseTable hasUniqueConstraints ifTrue:
			[sqlStatementStream nextPutAll: ', '.
			aGLORPDatabaseTable uniqueConstraints do:
				[:each | sqlStatementStream nextPutAll: (each creationString)]
				separatedBy: [sqlStatementStream nextPut: $,]]].

	sqlStatementStream
		nextPut: $).
	^sqlStatementStream contents!

createUniqueConstraintAddStatement: aUniqueConstraint
	"Callback from the UniqueConstraint. Returns collection of statements "
	
	| stream |
	aUniqueConstraint shouldCreateInDatabase ifFalse: [^#('')].
	stream := WriteStream on: (String new: 50).
	stream
		nextPutAll: 'CONSTRAINT ';
		nextPutAll: aUniqueConstraint name;
		nextPutAll: ' UNIQUE ('.
	GlorpHelper 
		print: [:each | each name] 
		on: stream 
		for: aUniqueConstraint fields 
		separatedBy: ','.
	stream nextPutAll: ')'.
	^OrderedCollection with: stream contents!

databaseIdentitySequenceClass
	"This method applies only to classes that have both types--database sequences and identity columns.
	Currently, only DB2 has both types. The sequence is prefered, and the identity type has minimal support."

	self databaseSequenceClass!

databaseSequenceClass
	self subclassResponsibility.!

date
	"Return the type we use to represent dates. By default, we use the same type as for timestamps."
	^self timestamp.!

dateConverter
	^DelegatingDatabaseConverter
		named: #date
		hostedBy: self
		fromStToDb: #toDate:for:
		fromDbToSt: #readDate:for:.	"#printDate:for:"!

datetime
	^self timestamp!

ddlTableNameFor: aTableOrString
	"
	Either a databaseTable or a string can be passed in the argument.
	"

	^ aTableOrString isString
		ifTrue: [ self capitalWritingOfTableName 
			ifTrue: [ aTableOrString asUppercase ] 
			ifFalse:[ aTableOrString ] ]
		ifFalse: [ self ddlTableNameFor: aTableOrString qualifiedName ]!

deleteViewWithTableSyntax

	^false!

double
	^self subclassResponsibility.!

endColumnAddString
	"The string we use as a suffix when adding a colum"
	^''.!

endColumnQuote
	"Return the character used to stop quoting the name of a column so that we can avoid conflicts with reserved names. ANSI says to use double quotes, so that's the default."
	^'"'.!

escapeFor: aCharacter
	aCharacter = $' ifTrue: [^''''''].
	^String with: $\ with: aCharacter.!

exdiTypeForDates
	^#Date.!

functions
	functions isNil ifTrue: [self initializeFunctions].
	^functions.!

generatorFor: aType withBlock: aBlock
	"Don't cache this, because we want to return different instances depending on aType"
	^VersionType underlyingType: aType generator: (AdHocVersionGenerator block: aBlock).!

hasSubtransaction
	"^<Boolean> This method returns true, if the used dbms is able to execute multiple sql-statements
	transferred via a command line transmitted from client to server - otherwise I return false"

	^true!

initializeFunctions
	"Subclasses should override to add database-specific functions"

	functions := FunctionExpression createBasicFunctionsFor: self.!

initializeReservedWords
	reservedWords := Set new.!

inMemorySequence
	^self typeNamed: #inMemorySequence ifAbsentPut: [
		GlorpInMemorySequenceDatabaseType representedBy: self int4].!

int
	^self integer.!

int2

	^self subclassResponsibility!

int4

	^self subclassResponsibility!

int8

	^self subclassResponsibility!

integer

	^self typeNamed: #integer ifAbsentPut: [GlorpIntegerType new].!

integerToString: anInteger for: aType
	anInteger isNil ifTrue: [^nil].
	^self padString: anInteger printString for: aType.!

integerToStringConverter
	^DelegatingDatabaseConverter
		named: #integerToString
		hostedBy: self
		fromStToDb: #integerToString:for:
		fromDbToSt: #stringToInteger:for:.!

isAccessPlatform

	^false!

isDB2Platform

	^false!

isMySQLPlatform
	^false.!

isOcelotPlatform

	^false!

isODBCPlatform

	^false!

isOraclePlatform

	^false!

isPostgreSQLPlatform

	^false!

isSQLite3Platform

	^false!

isSQLServerPlatform

	^false!

maximumLengthOfColumnName
	"^<Integer> I return the maximum length of a column name.  (I am not sent at present;  I am here so that if I and #maximumLengthOfTableName (which is sent) should ever differ on some database platform, I can be used in revised truncation logic.)"

	^18!

maximumLengthOfTableName
	"^<Integer> I return the max. length of a table name"

	^18!

maximumQueryValueSize
	"If there's a limit on how big a bound input parameter can be for a query, return it here. By default, no maximum. "
	^536870911 "VisualWorks SmallInteger maxVal. Lacking infinity, this will do as a default."!

maximumSizeToGroupWriteFor: aCollectionOfDatabaseRows
	"If we are going to group write, how many rows of this collection should we do it for at once"
	^aCollectionOfDatabaseRows size.!

maxLengthOfDatabaseName
	"^<Integer>I return the max. length of a database name"

	^8!

maxSQLBufferLength
	"^<Integer> I return the maximum length of a sql command stream"

	^8192!

measuresStringFieldWidthsInBytes
	"Return true if we measure the width of a string in bytes rather than characters. So, a 255-character field can hold 255 bytes worth of string, not 255 characters. This is important with encodings"
	^false.!

nameForColumn: aColumnString
	| upper |
	upper := self capitalWritingOfColumnName ifTrue: [aColumnString asUppercase] ifFalse: [aColumnString].
	(self reservedWords includes: aColumnString asLowercase) ifFalse: [^upper].
	^self startColumnQuote, upper, self endColumnQuote.!

nameForPrimaryKeyConstraintsWhenDropping
	"Return the name we use to describe a constraint when dropping them. This is required because MySQL wants you to use 'DROP PRIMARY KEY' rather than the ANSI 'DROP CONSTRAINT'."
	^'CONSTRAINT'.!

newDatabaseIdentitySequenceNamed: aString
	"Return an identity sequence, with the given name. This method is useful for platforms
	which provide regular sequence objects as well as identity column objects. Glorp uses
	sequences when they are available, but pre-existing DB2 tables may contain identity
	columns, so we need to be able to recognize them."

	^self databaseIdentitySequenceClass named: aString!

newDatabaseSequenceNamed: aString
	"Return a sequence of the type we use, with the given name"
	^self databaseSequenceClass named: aString.!

nullConversion: anObject for: aType
	^anObject.!

nullConverter
	^GlorpNullConverter new
		name: #null.!

number

	^self typeNamed: #number ifAbsentPut: [GlorpNumericType new].!

numberToDoubleConverter
	^DelegatingDatabaseConverter
		named: #numberToDouble
		hostedBy: self
		fromStToDb: #convertToDouble:for:
		fromDbToSt: #convertToDouble:for:.!

numberToFloatConverter
	^DelegatingDatabaseConverter
		named: #numberToFloat
		hostedBy: self
		fromStToDb: #convertToFloat:for:
		fromDbToSt: #convertToFloat:for:.!

numberToIntegerConverter
	^DelegatingDatabaseConverter
		named: #numberToInteger
		hostedBy: self
		fromStToDb: #convertToInteger:for:
		fromDbToSt: #convertToInteger:for:.!

numberToNumberConverter
	^DelegatingDatabaseConverter
		named: #numberToNumber
		hostedBy: self
		fromStToDb: #convertToNumber:for:
		fromDbToSt: #convertToNumber:for:.!

numeric
	^self number.!

nvarchar
	^self subclassResponsibility.!

paddingCharacter
	^1 asCharacter.!

padString: aString for: aType 
	| padding trimmed |
	aString isNil ifTrue: [^nil].
	aString isString ifFalse: [self error: 'Invalid data type'].
	(self usesNullForEmptyStrings and: [aString isEmpty]) ifTrue: [^nil].
	trimmed := self trimString: aString for: aType.
				
	(aType isVariableWidth or: [trimmed size = aType width]) ifTrue: [^trimmed].
	padding := String new: aType width - aString size.
	padding atAllPut: self paddingCharacter.
	^aString , padding!

parenthesizeCommandsInCompoundQuery
	^true!

postfixTableNameBeforeDeleting

	^false!

predefinedKeywords
	"
		^<OrderdCollection of: String> This method returns a list of preserved keyword, which should
		not be used in database-, table or column names or any othe names in the platform system
	"

	^OrderedCollection new!

prefixQualifierBeforeCreatingAndDeleting

	^true!

prefixQualifierSeparatorString
	"^<String> This statement return the string to be used to separate the qualifier and the table/column name"

	^'.'!

prefixTableNameBeforeDeleting

	^false!

primaryKeysAreAutomaticallyUnique
	"Return false if, in addition to specifying something as a primary key, we must separately specify it as unique"

	^false.!

printBlob: aByteArray on: aStream for: aType
	aByteArray isNil ifTrue: [^aStream nextPutAll: 'NULL'].
	aStream nextPutAll: 'X'''.
	aByteArray
		do:
			[:each | 
			each < 16 ifTrue: [aStream nextPut: $0].
			each
				printOn: aStream
				base: 16
				showRadix: false].
	aStream nextPut: $'!

printBoolean: aBoolean for: aType
	"If the platform is MS SQL Server, a boolean object in a SELECT needs to be converted to  BIT type, since T, Y, or true make SQLServer complain. "
	| stream |
	stream := WriteStream on: String new.
	aBoolean glorpPrintSQLOn: stream.
	^stream contents.!

printColumn: eachGlorpDatabaseField on: sqlStatementStream
	"Print the column specification for the CREATE TABLE statement.
	Don't show a default value for serial columns, since their semanitics are different."

	sqlStatementStream
		nextPutAll: (self nameForColumn: eachGlorpDatabaseField name);
		space;
		nextPutAll: eachGlorpDatabaseField typeString;
		space;
		nextPutAll: 
				(eachGlorpDatabaseField type isSerial
					ifTrue: ['']
					ifFalse: [self sqlTextForDEFAULT: eachGlorpDatabaseField defaultValue]);
		space;
		nextPutAll:
				(eachGlorpDatabaseField isNullable
						ifTrue: [self sqlTextForNULLAttributeConstraint]
						ifFalse: [self sqlTextForNOTNULLAttributeConstraint]);
		space;
		nextPutAll:
				(eachGlorpDatabaseField isUnique
						ifTrue: [self sqlTextForUNIQUEAttributeConstraint]
						ifFalse: [self sqlTextForNOTUNIQUEAttributeConstraint]).!

printColumnsSpecificationFor: aGlorpDatabaseTable on: sqlStatementStream
	aGlorpDatabaseTable fields isEmpty not
		ifTrue:
			[| sepFlag |
			sqlStatementStream
				space;
				nextPut: $(.
			sepFlag := false.
			aGlorpDatabaseTable fields
				do:
					[:eachGlorpDatabaseField | 
					sepFlag
						ifTrue:
							[sqlStatementStream nextPutAll: self columnNameSeparatorString].
					self printColumn: eachGlorpDatabaseField on: sqlStatementStream.
					sepFlag := true]].!

printDate: aTimestamp isoFormatOn: stream
	"Print the date as yyyy-mm-dd"

	| monthNumber dayOfMonth |
	aTimestamp isNil ifTrue: [^'NULL'].
	stream nextPutAll: aTimestamp year printString.
	stream nextPut: $-.
	monthNumber := aTimestamp monthIndex.
	stream nextPutAll: (GlorpDatabaseType padToTwoDigits: monthNumber).
	stream nextPut: $-.
	dayOfMonth := aTimestamp dayOfMonth.
	stream nextPutAll: (GlorpDatabaseType padToTwoDigits: dayOfMonth)!

printDDLTableNameFor: aGLORBDatabaseTable on: sqlStatementStream
	"This method just writes the name of a table to a stream, in a way appropriate for DDL statements"
	
	sqlStatementStream
		nextPutAll: (self capitalWritingOfTableName 
				ifTrue:[ aGLORBDatabaseTable qualifiedName asUppercase ] 
				ifFalse:[ aGLORBDatabaseTable qualifiedName ]).!

printOffset: anInteger on: aCommand
	"By default, do nothing"!

printPostLimit: anInteger on: aCommand
	"By default, do nothing"!

printPreLimit: anInteger on: aCommand
	"By default, do nothing"!

printPrimaryKeyConstraintsOn: sqlStatementStream for: aTable 
	"This method print the constraint specification on sqlStatementStream"

	aTable primaryKeyFields isEmpty ifTrue: [^self].
	(self createPrimaryKeyConstraintAddStatement: aTable primaryKeyConstraints)
		do: [ :each | sqlStatementStream nextPutAll: each].!

printSqlStatementToCopyDataFromColumn: fromColumn to: toColumn on: aStream
	"
	Example:
	UPDATE test_bank_acct
	SET close_balance = balance
	WHERE close_balance IS NULL OR close_balance <> balance
	"
	| toColumnName fromColumnName |
	toColumnName := self nameForColumn: toColumn name.
	fromColumnName := self nameForColumn: fromColumn name.
	aStream
		nextPutAll: 'UPDATE ';
		nextPutAll: (self ddlTableNameFor: fromColumn table);
		nextPutAll: ' SET ';
		nextPutAll: toColumnName;
		nextPutAll: ' = ';
		nextPutAll: fromColumnName;
		nextPutAll: ' WHERE ';
		nextPutAll: toColumnName;
		nextPutAll: ' IS NULL OR ';
		nextPutAll: toColumnName;
		nextPutAll: ' <> ';
		nextPutAll: fromColumnName!

printSqlStatementToCopyDataFromTable: fromTable to: toTable on: aStream

	aStream
		nextPutAll: 'INSERT INTO ';
		nextPutAll: (self ddlTableNameFor: toTable);
		nextPutAll: ' ( '.

	GlorpHelper 
		print: [:each | self nameForColumn: each name]
		on: aStream
		for: fromTable fields
		separatedBy: ','.


	aStream
		nextPutAll: ' ) ';
		nextPutAll: ' SELECT '.

	GlorpHelper 
		print: [:each | self nameForColumn: each name]
		on: aStream
		for: fromTable fields
		separatedBy: ','.

	aStream
		nextPutAll: ' FROM ';
		nextPutAll: (self ddlTableNameFor: fromTable)!

printSqlStatementToListTablesInSchema: schemaString on: aStream
	" Not standardized, although SQL Server, MySQL and PostgreSQL use the INFORMATION_SCHEMA "

	| lowerSchema |
	self usesInformationSchema
		ifFalse: [^self subclassResponsibility].

	lowerSchema := schemaString isNil
		ifTrue: [schemaString]
		ifFalse: [schemaString asLowercase].
	aStream
		nextPutAll: 'SELECT table_name FROM information_schema.tables ';
		nextPutAll: 'WHERE table_type = ';
		nextPut: $';
		nextPutAll: 'BASE TABLE';
		nextPut: $'.
	(lowerSchema notNil and: [lowerSchema notEmpty])
		ifTrue: [
			aStream
				nextPutAll: ' AND lower(table_schema) = ';
				nextPut: $';
				nextPutAll: lowerSchema;
				nextPut: $'.].
	aStream nextPut: $;!

printSqlStatementToPopulateColumn: aDatabaseField with: aValue on: aStream

	| converter dbValue |

	converter := aDatabaseField converterForStType: aValue class.
	dbValue := converter
		convert: aValue
		toDatabaseRepresentationAs: aDatabaseField type.

	aStream
		nextPutAll: 'UPDATE ';
		nextPutAll: (self ddlTableNameFor: aDatabaseField table);
		nextPutAll: ' SET ';
		nextPutAll: (self nameForColumn: aDatabaseField name);
		nextPutAll: ' = ';
		nextPutAll: dbValue printString;
		nextPutAll: ' WHERE (';
		nextPutAll: ( self nameForColumn: aDatabaseField name);
		nextPutAll: ' IS NULL OR ';
		nextPutAll: (self nameForColumn: aDatabaseField name);
		nextPutAll: ' <> ';
		nextPutAll: dbValue printString;
		nextPutAll: ')'!

printSqlStatementToRenameColumn: oldColumn to: newColumnName on: aStream

	aStream
		nextPutAll: 'ALTER TABLE ';
		nextPutAll: (self ddlTableNameFor: oldColumn table);
		nextPutAll: ' RENAME ';
		nextPutAll:  (self nameForColumn:  oldColumn name);
		nextPutAll: ' TO ';
		nextPutAll:  (self nameForColumn: newColumnName).!

printSqlStatementToRenameTable: aDatabaseTable to: newTableName on: aStream
	" Tje default SQL statement is
	ALTER TABLE <oldTableName> RENAME TO <newTableName>.
	Different platforms may use another form. Override in subclasses if necessary.
	"

	aStream
		nextPutAll: 'ALTER TABLE ';
		nextPutAll: (self ddlTableNameFor: aDatabaseTable);
		nextPutAll: ' RENAME TO ';
		nextPutAll: newTableName!

printSqlStatementToTestExistenceOfTable: tableName inSchema: schemaString on: aStream
	" Not standardized, although SQL Server, MySQL and PostgreSQL use the INFORMATION_SCHEMA "

	| lowerTableName lowerSchema |
	self usesInformationSchema
		ifFalse: [^self subclassResponsibility].

	lowerTableName := tableName asLowercase.
	lowerSchema := schemaString isNil
		ifTrue: [schemaString]
		ifFalse: [schemaString asLowercase].
	aStream
		nextPutAll: 'SELECT count(*) FROM information_schema.tables ';
		nextPutAll: 'WHERE lower(table_name) = ';
		nextPut: $';
		nextPutAll: lowerTableName;
		nextPut: $'.
	(lowerSchema notNil and: [lowerSchema notEmpty])
		ifTrue: [
			aStream
				nextPutAll: ' AND lower(table_schema) = ';
				nextPut: $';
				nextPutAll: lowerSchema;
				nextPut: $'.].
	aStream
		nextPutAll: ' AND table_type = ';
		nextPut: $';
		nextPutAll: 'BASE TABLE';
		nextPut: $';
		nextPut: $;!

printTime: aTime isoFormatOn: aStream 

	self
		printTime: aTime
		isoFormatOn: aStream
		milliseconds: self supportsMillisecondsInTimes.
	^self!

printTime: aTimestamp isoFormatOn: stream milliseconds: aBoolean
	"Print the time as hh:mm:ss.mmm"
	| ms |
	aTimestamp isNil ifTrue: [^nil].
	stream nextPutAll: (GlorpDatabaseType padToTwoDigits: aTimestamp hours).
	stream nextPut: $:.
	stream nextPutAll: (GlorpDatabaseType padToTwoDigits: aTimestamp minutes).
	stream nextPut: $:.
	stream nextPutAll: (GlorpDatabaseType padToTwoDigits: aTimestamp seconds).
	aBoolean ifFalse: [^self].
	ms := aTimestamp milliseconds.
	ms = 0 ifTrue: [^self].
	stream nextPut: $..
	stream nextPutAll: (GlorpDatabaseType padToThreeDigits: ms).!

printTimestamp: aTimestamp on: aStream for: aType
	aTimestamp isNil ifTrue: [aTimestamp glorpPrintSQLOn: aStream. ^self].

	aStream nextPut: $'; nextPutAll: aTimestamp printString; nextPut: $'.!

queryWithUnsupportedOperationsEliminatedFrom: aQuery do: aBlock
	"If aQuery has operations that we don't support, rewrite it to do them in terms of lower level operations. In particular, rewrite INTERSECT/EXCEPT operations into EXISTS clauses in a single query. Pass the new query to aBlock."
	^self.!

raw
	^self blob.!

readDate: anObject for: aType
	"format '2003-03-13"
	anObject isNil ifTrue: [^nil].
	anObject class == Date ifTrue: [^anObject].
	anObject isString ifTrue: [
		^self readDateFromStream: (ReadStream on: anObject) for: aType].
	^anObject asDate.!

readDateFromStream: aStream for: aType
	"Seems like we get to do this ourselves, in a lowest common denominator kind of way. Translate into GMT if we've got a timezone."
	"assumes ISO format.
	self readTimestamp: '2003-03-03 15:29:28.337-05' for: nil.
	self readTimestamp: '2003-03-03 19:29:28.337-05' for: nil

"
	|  years months days |
	years := (aStream upTo: $-) asNumber.
	months := (aStream upTo: $-) asNumber.
	days := (aStream upTo: $ ) asNumber.
	^Dialect newDateWithYears: years months: months days: days.!

readTime: anObject for: aType
	"format 15:29:28.337-05  (timezone optional)"
	anObject isNil ifTrue: [^nil].
	anObject class == Time ifTrue: [^anObject].
	anObject isString ifTrue: [
		^self readTimeFromStream: (ReadStream on: anObject) for: aType].
	^anObject asTime.!

readTimeFromStream: aStream for: aType
	"Seems like we get to do this ourselves, in a lowest common denominator kind of way. Ignore timezones right now"
	"assumes ISO format.
	self readTimestamp: '2003-03-03 15:29:28.337-05' for: nil.
	self readTimestamp: '2003-03-03 19:29:28.337-05' for: nil

"
	| hours minutes seconds milliseconds timeZoneOffset millisecondAccumulator |
	hours := (aStream upTo: $:) asNumber.
	minutes := (aStream upTo: $:) asNumber.
	seconds := (aStream next: 2) asNumber.
	(aStream peek = $.) 
		ifTrue: [
			aStream next.
			millisecondAccumulator := WriteStream on: String new.
			[aStream atEnd not and: [aStream peek isDigit]] whileTrue: [
				millisecondAccumulator nextPut: aStream next].
			milliseconds := millisecondAccumulator contents asNumber]
		ifFalse: [milliseconds := 0].
	timeZoneOffset := aStream upToEnd asNumber.
	^Dialect newTimeWithHours: hours minutes: minutes seconds: seconds milliseconds: milliseconds.
	"^Dialect addSeconds: (timeZoneOffset * -1* 60 * 60) to: aTime."!

readTimestamp: anObject for: aType
	"Seems like we get to do this ourselves, in a lowest common denominator kind of way. Translate into GMT if we've got a timezone."

	"assumes ISO format.
	self readTimestamp: '2003-03-03 15:29:28.337-05' for: nil.
	self readTimestamp: '2003-03-03 19:29:28.337-05' for: nil"

	anObject isNil ifTrue: [^nil].
	anObject class == Dialect timestampClass ifTrue: [^anObject].
	anObject isString
		ifTrue:
			[| stream |
			stream := ReadStream on: anObject.
			^self readTimestampFromStream: stream for: aType].
	^anObject asTimestamp.!

readTimestampFromStream: aStream for: aType

	| years months days hours minutes seconds millisecondAccumulator milliseconds timeZoneOffset |
	years := (aStream upTo: $-) asNumber.
	months := (aStream upTo: $-) asNumber.
	days := (aStream upTo: $ ) asNumber.
	hours := (aStream upTo: $:) asNumber.
	minutes := (aStream upTo: $:) asNumber.
	seconds := (aStream next: 2) asNumber.
	(aStream peek = $.) 
		ifTrue: [
			aStream next.
			millisecondAccumulator := WriteStream on: String new.
			[aStream atEnd not and: [aStream peek isDigit]] whileTrue: [
				millisecondAccumulator nextPut: aStream next].
			milliseconds := millisecondAccumulator contents asNumber]
		ifFalse: [milliseconds := 0].
	timeZoneOffset := aStream upToEnd asNumber.
	timeZoneOffset isNil ifTrue: [timeZoneOffset := 0].
	timeZoneOffset := timeZoneOffset * 60 * 60.
	^Dialect 
		newTimestampWithYears: years
		months: months
		days: days
		hours: hours
		minutes: minutes
		seconds: seconds
		milliseconds: milliseconds
		offset:timeZoneOffset.!

requiresCastsForNullsInUnions
	"If we have explicit nulls in a query which is unioned, do we need to cast them to the appropriate type in order to keep the database happy."
	^false.!

requiresEscapeFor: aCharacter
	^self charactersThatNeedEscaping includes: aCharacter.!

requiresTransactionForTableOperations
	"Return true if we need a transaction in order to do table creation, deletion, and so forth. Some databases require it. Others don't like it"
	^true.!

reservedWords
	reservedWords isNil ifTrue: [self initializeReservedWords].
	^reservedWords.!

reusePreparedStatements
	"The ordinary choice is to initialize the accessor to reuse statements whenever binding is being used."

	^true!

sequence

	^self subclassResponsibility!

serial
	self subclassResponsibility.!

smallint

	^self typeNamed: #smallint ifAbsentPut: [GlorpSmallIntType new].!

sqlStatementToCopyDataFromColumn: fromDatabaseField to: toDatabaseField

	| stream |

	stream := WriteStream on: String new.

	self
		printSqlStatementToCopyDataFromColumn: fromDatabaseField
		to: toDatabaseField
		on: stream.

	^ stream contents!

sqlStatementToCopyDataFromTable: fromDatabaseTable to: toDatabaseTable

	| stream |

	stream := WriteStream on: String new.

	self
		printSqlStatementToCopyDataFromTable: fromDatabaseTable
		to: toDatabaseTable
		on: stream.

	^ stream contents!

sqlStatementToListTablesInSchema: schemaString

	| stream |
	stream := WriteStream on: String new.
	self printSqlStatementToListTablesInSchema: schemaString on: stream.
	^stream contents!

sqlStatementToPopulateColumn: aDatabaseField with: aValue

	| stream |

	stream := WriteStream on: String new.

	self
		printSqlStatementToPopulateColumn: aDatabaseField
		with: aValue
		on: stream.

	^ stream contents!

sqlStatementToRenameColumn: oldColumn to: newColumnName

	| stream |

	stream := WriteStream on: String new.

	self
		printSqlStatementToRenameColumn: oldColumn
		to: newColumnName
		on: stream.

	^ stream contents!

sqlStatementToRenameTable: aDatabaseTable to: newTableName

	| stream |

	stream := WriteStream on: String new.

	self printSqlStatementToRenameTable: aDatabaseTable to: newTableName on: stream.

	^ stream contents!

sqlStatementToTestExistenceOfTable: tableName inSchema: schemaString

	| stream |
	stream := WriteStream on: String new.
	self printSqlStatementToTestExistenceOfTable: tableName inSchema: schemaString on: stream.
	^stream contents!

sqlTextCurrentServerUTCTimestamp

	self subclassResponsibility!

sqlTextForBeginTransaction
	"comment"

	^'BEGIN'!

sqlTextForDEFAULT: defaultValue
	"^<String>. Answer the column DEFAULT clause as a string with the given value.
	This is used in CREATE TABLE statements as part of each column specification."
	
	| strm |
	^defaultValue
		ifNotNil:
			[strm := String new writeStream.
			strm nextPutAll: 'DEFAULT '.
			defaultValue glorpPrintSQLOn: strm.
			strm contents]
		ifNil: ['']!

sqlTextForNOTNULLAttributeConstraint
	"^<String>"

	^'NOT NULL'!

sqlTextForNOTNULLWithDefaultAttributeConstraint
	"^<String>"

	^'NOT NULL WITH DEFAULT'!

sqlTextForNOTUNIQUEAttributeConstraint
	^''.!

sqlTextForNULLAttributeConstraint
	"^<String>"

	^'NULL'!

sqlTextForUNIQUEAttributeConstraint
	^'UNIQUE'.!

sqlWildcardForMultipleCharacters
	"^<String> This method returns the used wildcard string for multiple characters"
	
	^'%'!

sqlWildcardForSingleCharacter
	"^<String> This method returns the used wildcard string for single characters"
	
	^'_'!

startColumnAddString
	"The string we use as a prefix when adding a colum"
	^' COLUMN '.!

startColumnQuote
	"Return the character used to start quoting the name of a column so that we can avoid conflicts with reserved names. ANSI says to use double quotes, so that's the default."
	^'"'.!

stringToByteArray: aString for: aType
	| string |
	string := self unpadString: aString for: aType.
	^string isNil ifTrue: [nil] ifFalse: [Dialect convert: string toByteArrayWithEncoding: self characterEncoding].!

stringToInteger: aString for: aType
	| string |
	string := self unpadString: aString for: aType.
	^string isNil ifTrue: [nil] ifFalse: [string asNumber truncated].!

stringToStringConverter
	^DelegatingDatabaseConverter
		named: #stringToString
		hostedBy: self
		fromStToDb: #padString:for:
		fromDbToSt: #unpadString:for:.!

stringToSymbol: aString for: aType
	| string |
	string := self unpadString: aString for: aType.
	^string isNil ifTrue: [nil] ifFalse: [string asSymbol].!

supportsAliasInCompoundQuery
	"Return true if, when we have a subselect, we can (and/or must) alias the result with a table name. Access doesn't like the table name"
	^true.!

supportsAlterTableForConstraints

	^true!

supportsANSIJoins
	"Do we support the JOIN <tableName> USING <criteria> syntax. Currently hard-coded, but may also vary by database version"
	^false.!

supportsAuthentication
	^true!

supportsBinding
	"Return true if this platform supports binding parameters rather than printing them as strings into the SQL statement"
	^false.!

supportsCaseInsensitiveLike
	^false.!

supportsConstraints
	"Return true if we support integrity constraints. That is, foreign key, primary key or unique constraints."
	^true.!

supportsDecimalsOnAllNumerics
	"Return true if a general 'numeric' type will allow numbers after the decimal place"
	^true.!

supportsGroupWritingFor: aCommand
	"Return true if this platform can do array binding or an equivalent for this command"
	^false.!

supportsInformationSchema
	^true.!

supportsIntersect
	"Do we support the INTERSECT or MINUS operations."
	^true.!

supportsLimit
	"Do we support anything analogous to the postgresql LIMIT, returning only the first N rows"
	^false.!

supportsMillisecondsInTimes
	self subclassResponsibility.!

supportsMultipleOpenCursors
	"Can this database support multiple open cursors at once"
	^true.!

supportsOffset
	"Do we support anything analogous to the postgresql OFFSET, skipping the first N rows"
	^false.!

supportsRowCount
	^true.!

supportsSchemaForLogin

	^true!

supportsSchemas
	"Return true if this platform supports schemas. Access and SQLite 3 do not."

	^true!

supportsStringQueryOnBlobs
	^true.!

supportsTableOwners
	"Return true if this platform supports table owners, i.e. expects table names of the form Bern.TW_* rather than just TW_* in its SQL."
"Access, Firebird and PostGreSQL do not, Oracle does, others I know not."

	^false!

supportsVariableSizedNumerics
	"Return true if this platform can support numbers with a varying size and number of decimal places. Access, notably, doesn't seem to be able to"
	^true.!

symbolToString: aSymbol for: aType
	aSymbol isNil ifTrue: [^nil].
	^self padString: aSymbol asString for: aType.!

symbolToStringConverter
	^DelegatingDatabaseConverter
		named: #symbolToString
		hostedBy: self
		fromStToDb: #symbolToString:for:
		fromDbToSt: #stringToSymbol:for:.!

tableNameFor: aStringRepresentingATableName
	"Return the table name, as modified to be used in a database statement."
	^aStringRepresentingATableName.!

text
	^self clob.!

time
	self subclassResponsibility.!

timeConverter
	^DelegatingDatabaseConverter
		named: #time
		hostedBy: self
		fromStToDb: #toTime:for:
		fromDbToSt: #readTime:for:.	"#printTime:for:"!

timestamp
	self subclassResponsibility.!

timestampConverter
	^DelegatingDatabaseConverter
		named: #timestamp
		hostedBy: self
		fromStToDb: #toTimestamp:for:
		fromDbToSt: #readTimestamp:for:.	"#printTimestamp:for:"!

timestampToDateConverter
	^DelegatingDatabaseConverter
		named: #timestampToDate
		hostedBy: self
		fromStToDb: #toTimestamp:for:
		fromDbToSt: #readDate:for:.!

toDate: anObject for: aType
	anObject isNil ifTrue: [^nil].
	anObject class = Date ifTrue: [^anObject].
	^anObject asDate.!

toGlorpType: aGlorpType fromString: aString
	"Return a Smalltalk instance of the type indicated by aGlorpType."
	
	| aClass |
	^aString
		ifNotNil:
			[aClass := aGlorpType impliedSmalltalkType.
			aClass readFrom: aString readStream]!

toTime: anObject for: aType
	anObject isNil ifTrue: [^nil].
	anObject class = Time ifTrue: [^anObject].
	^anObject asTime.!

toTimestamp: anObject for: aType	
	anObject isNil ifTrue: [^nil].
	anObject class = Dialect timestampClass ifTrue: [^anObject].

	^anObject asTimestamp.!

trimString: aString for: aType 
	| trimmedString length excess |
	aType width isNil ifTrue: [^aString].
	self measuresStringFieldWidthsInBytes ifFalse: [
		^aString size > aType width ifTrue: [aString copyFrom: 1 to: aType width] ifFalse: [aString]].
	"If our characters are less than half the number of bytes it can hold, assume it will fit and don't try to do this"
	(aString size < (aType width // 2)) ifTrue: [^aString].
	"If we're doing defaults that don't map characters to multi-bytes, don't do this"
	(#(ascii default) includes: self characterEncoding) ifTrue: [
		^aString size > aType width ifTrue: [aString copyFrom: 1 to: aType width] ifFalse: [aString]].
	trimmedString := aString.
	[length := (Dialect convert: trimmedString toByteArrayWithEncoding: self characterEncoding) size.
	excess := length - aType width.
	excess > 0 
		"Try to be careful and trim as little as possible, while still keeping the number of steps reasonable for large excesses"
		ifTrue: [trimmedString := trimmedString copyFrom: 1 to: trimmedString size - ((excess // 2) max: 1)]
		ifFalse: [^trimmedString].
	true] whileTrue.!

typeNamed: aSymbol ifAbsentPut: aBlock

	| type |
	type := self types at: aSymbol ifAbsentPut: [
		| newType |
		newType := aBlock value.
		newType platform: self.
		newType selector: aSymbol].
	type hasParameters ifTrue: [type := type copy].
	^type.!

types
	types == nil ifTrue: [types := IdentityDictionary new].
	^types.!

typesWithValues
	"Return our types list, but makes sure it has at least basic types in it"
	self varchar.
	self int4.
	self integer.
	self int.
	self number.
	self timestamp.
	self char.
	self blob.
	self clob.
	self raw.
	self serial.
	self text.
	^self types.!

typeWithDatabaseName: aString characterWidth: aNumber
	"Return our type which matches the database name for the type. Assumes all variable width fields are strings right now."
	| baseType matchingTypes simpleTypes |
	matchingTypes := self typesWithValues values asArray select: [:each | each typeName asLowercase = aString asLowercase].
	matchingTypes isEmpty ifTrue: [
		Transcript cr; show: 'Cannot find type ', aString, ' using integer instead.'. matchingTypes := Array with: self integer].
	"We may get generated and non-generated both matching, e.g. inMemorySequence and int. Prefer the non-generated."
	simpleTypes := matchingTypes reject: [:each | each isGenerated].
	baseType := simpleTypes isEmpty ifTrue: [matchingTypes first] ifFalse: [simpleTypes first].
	^baseType hasWidth ifTrue: [baseType copy width: aNumber] ifFalse: [baseType].!

typeWithDatabaseName: aString characterWidth: aNumber extraInformation: anythingElse
	"Return our type which matches the database name for the type. Assumes all variable width fields are strings right now."
	^self typeWithDatabaseName: aString characterWidth: aNumber.!

unpadString: aString for: aType

	aString isNil ifTrue: [^nil].
	^aType isVariableWidth 
		ifTrue: [aString]
		ifFalse: [(ReadStream on: aString) upTo: 1 asCharacter].!

useBinding
	"Return true if we should use binding"

	self supportsBinding ifFalse: [^false].
	useBinding isNil ifTrue: [useBinding := self class useBindingIfSupported].
	^useBinding!

useBinding: aBoolean
	useBinding := aBoolean.!

useMicrosoftOuterJoins
	"Return true if we use the Microsoft x *= y syntax for outer joins"
	^false.!

useOracleOuterJoins
	"Return true if we use the old Oracle x = y (+) syntax for outer joins"
	^false.!

usesArrayBindingRatherThanGrouping
	"Return true if we use array binding for grouped writes rather than printing the sql multiple times. Only applies if we support grouped writes"
	^self subclassResponsibility.!

usesIdentityColumns
	^false.!

usesInformationSchema

	^false!

usesNullForEmptyStrings
	"Return true if this database is likely to use nil as an empty string value"
	^false.!

usesNullForFalse
	"Return true if this database is likely to use nil as an empty string value"
	^false.!

usesPrimaryKeyUniqueConstraintNames
	"When dropping/creating primary key constraints, do they have distinguished names."
	^true.!

validateTableName: tableNameString
	" <Boolean> I return true, if the choosen tableNameString is valid for the platform"

 	^( tableNameString size <= self maximumLengthOfTableName ) 
			and: [ (self predefinedKeywords includes: tableNameString asLowercase) not ]!

varbinary
	^self subclassResponsibility.!

varbinary: anInteger
	^self varbinary width: anInteger.!

varchar
	^self subclassResponsibility.!

varChar
	^self varchar.!

varchar: anInt

	^self varchar width: anInt.!

varChar: anInt

	^self varchar width: anInt.!

versionFieldFor: aType
	"Don't cache this, because we want to return different instances depending on aType"
	^(VersionType underlyingType: aType) generatesOverExistingValues: true; platform: self.!

vwConnectionClass
	^self isODBCPlatform 
			ifTrue: [self vwODBCConnectionClass]
			ifFalse: [self vwEXDIConnectionClass]!

vwEXDIConnectionClass
	^self subclassResponsibility!

vwODBCConnectionClass
	"Prefer Threaded over not if nothing is loaded, but use whatever is loaded if there is anything"

	'ODBCThreadedConnection' asQualifiedReference ifDefinedDo:
		[:class | ^class].
	'ODBCConnection' asQualifiedReference ifDefinedDo:
		[:class | ^class].		
	^'ODBCThreadedConnection' asQualifiedReference valueIfUndefinedLoadFrom: 'ODBCThapiEXDI'!

writeConstraintToDrop: aConstraint onStream: aStream
	aStream
		nextPutAll: 'CONSTRAINT ';
		nextPutAll: aConstraint name.! !
!DatabasePlatform categoriesFor: #areSequencesExplicitlyCreated!constants!private! !
!DatabasePlatform categoriesFor: #asTimestamp:for:!conversion-times!private! !
!DatabasePlatform categoriesFor: #batchWriteStatementTerminatorString!constants!private! !
!DatabasePlatform categoriesFor: #bindingsForGroupWritingFor:!binding!private! !
!DatabasePlatform categoriesFor: #blob!private!types! !
!DatabasePlatform categoriesFor: #boolean!private!types! !
!DatabasePlatform categoriesFor: #booleanToBooleanConverter!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #booleanToIntegerConverter!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #booleanToStringTFConverter!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #booleanToStringYesNoConverter!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #booleanToStringYNConverter!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #byteArrayToString:for:!conversion-strings!private! !
!DatabasePlatform categoriesFor: #byteArrayToStringConverter!conversion-strings!private! !
!DatabasePlatform categoriesFor: #canBind:to:!private!testing! !
!DatabasePlatform categoriesFor: #canUseInWhereClause:type:!private!testing! !
!DatabasePlatform categoriesFor: #capitalWritingOfColumnName!constants!private! !
!DatabasePlatform categoriesFor: #capitalWritingOfCreatorName!constants!private! !
!DatabasePlatform categoriesFor: #capitalWritingOfSQLCommands!constants!private! !
!DatabasePlatform categoriesFor: #capitalWritingOfTableName!constants!private! !
!DatabasePlatform categoriesFor: #char!private!types! !
!DatabasePlatform categoriesFor: #char:!private!types! !
!DatabasePlatform categoriesFor: #character!private!types! !
!DatabasePlatform categoriesFor: #characterEncoding!accessing!private! !
!DatabasePlatform categoriesFor: #characterEncoding:!accessing!private! !
!DatabasePlatform categoriesFor: #charactersThatNeedEscaping!constants!private! !
!DatabasePlatform categoriesFor: #clob!private!types! !
!DatabasePlatform categoriesFor: #columnNameSeparatorString!constants!private! !
!DatabasePlatform categoriesFor: #compoundOperationFor:!database-specific!private! !
!DatabasePlatform categoriesFor: #convertBooleanToDBBoolean:for:!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #convertBooleanToInteger:for:!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #convertBooleanToTF:for:!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #convertBooleanToYesNo:for:!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #convertBooleanToYN:for:!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #convertByteArrayToSQLString:for:!conversion-strings!private! !
!DatabasePlatform categoriesFor: #convertByteArrayToString:for:!conversion-strings!private! !
!DatabasePlatform categoriesFor: #convertDBBooleanToBoolean:for:!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #converterNamed:!private!type converters! !
!DatabasePlatform categoriesFor: #converters!private!type converters! !
!DatabasePlatform categoriesFor: #convertIntegerToBoolean:for:!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #convertStringToByteArray:for:!conversion-strings!private! !
!DatabasePlatform categoriesFor: #convertTFToBoolean:for:!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #convertToDouble:for:!conversion-numbers!private! !
!DatabasePlatform categoriesFor: #convertToFloat:for:!conversion-numbers!private! !
!DatabasePlatform categoriesFor: #convertToInteger:for:!conversion-numbers!private! !
!DatabasePlatform categoriesFor: #convertToNumber:for:!conversion-numbers!private! !
!DatabasePlatform categoriesFor: #convertYesNoToBoolean:for:!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #convertYNToBoolean:for:!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #createAddColumnStatementFor:!private!services columns! !
!DatabasePlatform categoriesFor: #createAlterColumnStatement:newType:!private!services columns! !
!DatabasePlatform categoriesFor: #createAlterColumnStatement:newType:usingExpression:!private!services columns! !
!DatabasePlatform categoriesFor: #createAlterColumnStatement:setDefault:!private!services columns! !
!DatabasePlatform categoriesFor: #createAlterColumnStatementDropDefault:!private!services columns! !
!DatabasePlatform categoriesFor: #createAlterColumnStatementDropNotNull:!private!services columns! !
!DatabasePlatform categoriesFor: #createAlterColumnStatementSetNotNull:!private!services columns! !
!DatabasePlatform categoriesFor: #createConstraintAddStatement:!public!services constraints! !
!DatabasePlatform categoriesFor: #createConstraintDropStatement:!public!services constraints! !
!DatabasePlatform categoriesFor: #createDropColumnStatementFor:!private!services columns! !
!DatabasePlatform categoriesFor: #createDropTableStatementFor:!private!services tables! !
!DatabasePlatform categoriesFor: #createForeignKeyConstraintAddStatement:!private!services constraints! !
!DatabasePlatform categoriesFor: #createPrimaryKeyConstraintAddStatement:!public!services constraints! !
!DatabasePlatform categoriesFor: #createTableFKContraintsStatementStringsFor:!obsolete!private! !
!DatabasePlatform categoriesFor: #createTableIndexStatementStringsFor:!public!services tables! !
!DatabasePlatform categoriesFor: #createTableStatementStringFor:!public!services tables! !
!DatabasePlatform categoriesFor: #createUniqueConstraintAddStatement:!public!services constraints! !
!DatabasePlatform categoriesFor: #databaseIdentitySequenceClass!public!sequences! !
!DatabasePlatform categoriesFor: #databaseSequenceClass!private!sequences! !
!DatabasePlatform categoriesFor: #date!private!types! !
!DatabasePlatform categoriesFor: #dateConverter!conversion-times!private! !
!DatabasePlatform categoriesFor: #datetime!private!types! !
!DatabasePlatform categoriesFor: #ddlTableNameFor:!private!services tables! !
!DatabasePlatform categoriesFor: #deleteViewWithTableSyntax!constants!private! !
!DatabasePlatform categoriesFor: #double!private!types! !
!DatabasePlatform categoriesFor: #endColumnAddString!constants!private! !
!DatabasePlatform categoriesFor: #endColumnQuote!private!services columns! !
!DatabasePlatform categoriesFor: #escapeFor:!conversion-strings!private! !
!DatabasePlatform categoriesFor: #exdiTypeForDates!exdi specific!private! !
!DatabasePlatform categoriesFor: #functions!functions!private! !
!DatabasePlatform categoriesFor: #generatorFor:withBlock:!private!types! !
!DatabasePlatform categoriesFor: #hasSubtransaction!constants!private! !
!DatabasePlatform categoriesFor: #initializeFunctions!functions!private! !
!DatabasePlatform categoriesFor: #initializeReservedWords!constants!private! !
!DatabasePlatform categoriesFor: #inMemorySequence!private!types! !
!DatabasePlatform categoriesFor: #int!private!types! !
!DatabasePlatform categoriesFor: #int2!private!types! !
!DatabasePlatform categoriesFor: #int4!private!types! !
!DatabasePlatform categoriesFor: #int8!private!types! !
!DatabasePlatform categoriesFor: #integer!private!types! !
!DatabasePlatform categoriesFor: #integerToString:for:!conversion-strings!private! !
!DatabasePlatform categoriesFor: #integerToStringConverter!conversion-strings!private! !
!DatabasePlatform categoriesFor: #isAccessPlatform!private!testing! !
!DatabasePlatform categoriesFor: #isDB2Platform!private!testing! !
!DatabasePlatform categoriesFor: #isMySQLPlatform!private!testing! !
!DatabasePlatform categoriesFor: #isOcelotPlatform!private!testing! !
!DatabasePlatform categoriesFor: #isODBCPlatform!private!testing! !
!DatabasePlatform categoriesFor: #isOraclePlatform!private!testing! !
!DatabasePlatform categoriesFor: #isPostgreSQLPlatform!private!testing! !
!DatabasePlatform categoriesFor: #isSQLite3Platform!public!testing! !
!DatabasePlatform categoriesFor: #isSQLServerPlatform!private!testing! !
!DatabasePlatform categoriesFor: #maximumLengthOfColumnName!constants!public! !
!DatabasePlatform categoriesFor: #maximumLengthOfTableName!constants!private! !
!DatabasePlatform categoriesFor: #maximumQueryValueSize!private!testing! !
!DatabasePlatform categoriesFor: #maximumSizeToGroupWriteFor:!binding!private! !
!DatabasePlatform categoriesFor: #maxLengthOfDatabaseName!constants!private! !
!DatabasePlatform categoriesFor: #maxSQLBufferLength!constants!private! !
!DatabasePlatform categoriesFor: #measuresStringFieldWidthsInBytes!private!testing! !
!DatabasePlatform categoriesFor: #nameForColumn:!private!services columns! !
!DatabasePlatform categoriesFor: #nameForPrimaryKeyConstraintsWhenDropping!constants!private! !
!DatabasePlatform categoriesFor: #newDatabaseIdentitySequenceNamed:!public!sequences! !
!DatabasePlatform categoriesFor: #newDatabaseSequenceNamed:!private!sequences! !
!DatabasePlatform categoriesFor: #nullConversion:for:!conversion-null!private! !
!DatabasePlatform categoriesFor: #nullConverter!conversion-null!private! !
!DatabasePlatform categoriesFor: #number!private!types! !
!DatabasePlatform categoriesFor: #numberToDoubleConverter!conversion-numbers!private! !
!DatabasePlatform categoriesFor: #numberToFloatConverter!conversion-numbers!private! !
!DatabasePlatform categoriesFor: #numberToIntegerConverter!conversion-numbers!private! !
!DatabasePlatform categoriesFor: #numberToNumberConverter!conversion-numbers!private! !
!DatabasePlatform categoriesFor: #numeric!private!types! !
!DatabasePlatform categoriesFor: #nvarchar!private!types! !
!DatabasePlatform categoriesFor: #paddingCharacter!conversion-strings!private! !
!DatabasePlatform categoriesFor: #padString:for:!conversion-strings!private! !
!DatabasePlatform categoriesFor: #parenthesizeCommandsInCompoundQuery!constants!private! !
!DatabasePlatform categoriesFor: #postfixTableNameBeforeDeleting!constants!private! !
!DatabasePlatform categoriesFor: #predefinedKeywords!general services!private! !
!DatabasePlatform categoriesFor: #prefixQualifierBeforeCreatingAndDeleting!constants!private! !
!DatabasePlatform categoriesFor: #prefixQualifierSeparatorString!constants!private! !
!DatabasePlatform categoriesFor: #prefixTableNameBeforeDeleting!constants!private! !
!DatabasePlatform categoriesFor: #primaryKeysAreAutomaticallyUnique!constants!private! !
!DatabasePlatform categoriesFor: #printBlob:on:for:!conversion-strings!public! !
!DatabasePlatform categoriesFor: #printBoolean:for:!conversion-boolean!private! !
!DatabasePlatform categoriesFor: #printColumn:on:!private!services columns! !
!DatabasePlatform categoriesFor: #printColumnsSpecificationFor:on:!private!services columns! !
!DatabasePlatform categoriesFor: #printDate:isoFormatOn:!conversion-times!public! !
!DatabasePlatform categoriesFor: #printDDLTableNameFor:on:!private!services tables! !
!DatabasePlatform categoriesFor: #printOffset:on:!database-specific!private! !
!DatabasePlatform categoriesFor: #printPostLimit:on:!database-specific!private! !
!DatabasePlatform categoriesFor: #printPreLimit:on:!database-specific!private! !
!DatabasePlatform categoriesFor: #printPrimaryKeyConstraintsOn:for:!obsolete!private! !
!DatabasePlatform categoriesFor: #printSqlStatementToCopyDataFromColumn:to:on:!private!services columns! !
!DatabasePlatform categoriesFor: #printSqlStatementToCopyDataFromTable:to:on:!private!services columns! !
!DatabasePlatform categoriesFor: #printSqlStatementToListTablesInSchema:on:!public!services tables! !
!DatabasePlatform categoriesFor: #printSqlStatementToPopulateColumn:with:on:!private!services columns! !
!DatabasePlatform categoriesFor: #printSqlStatementToRenameColumn:to:on:!private!services columns! !
!DatabasePlatform categoriesFor: #printSqlStatementToRenameTable:to:on:!private!services tables! !
!DatabasePlatform categoriesFor: #printSqlStatementToTestExistenceOfTable:inSchema:on:!public!services tables! !
!DatabasePlatform categoriesFor: #printTime:isoFormatOn:!conversion-times!private! !
!DatabasePlatform categoriesFor: #printTime:isoFormatOn:milliseconds:!conversion-times!private! !
!DatabasePlatform categoriesFor: #printTimestamp:on:for:!conversion-times!private! !
!DatabasePlatform categoriesFor: #queryWithUnsupportedOperationsEliminatedFrom:do:!database-specific!private! !
!DatabasePlatform categoriesFor: #raw!private!types! !
!DatabasePlatform categoriesFor: #readDate:for:!conversion-times!private! !
!DatabasePlatform categoriesFor: #readDateFromStream:for:!conversion-times!private! !
!DatabasePlatform categoriesFor: #readTime:for:!conversion-times!private! !
!DatabasePlatform categoriesFor: #readTimeFromStream:for:!conversion-times!private! !
!DatabasePlatform categoriesFor: #readTimestamp:for:!conversion-times!private! !
!DatabasePlatform categoriesFor: #readTimestampFromStream:for:!conversion-times!private! !
!DatabasePlatform categoriesFor: #requiresCastsForNullsInUnions!private!testing! !
!DatabasePlatform categoriesFor: #requiresEscapeFor:!constants!private! !
!DatabasePlatform categoriesFor: #requiresTransactionForTableOperations!constants!private! !
!DatabasePlatform categoriesFor: #reservedWords!constants!private! !
!DatabasePlatform categoriesFor: #reusePreparedStatements!public!testing! !
!DatabasePlatform categoriesFor: #sequence!private!types! !
!DatabasePlatform categoriesFor: #serial!private!types! !
!DatabasePlatform categoriesFor: #smallint!private!types! !
!DatabasePlatform categoriesFor: #sqlStatementToCopyDataFromColumn:to:!private!services columns! !
!DatabasePlatform categoriesFor: #sqlStatementToCopyDataFromTable:to:!private!services columns! !
!DatabasePlatform categoriesFor: #sqlStatementToListTablesInSchema:!public!services tables! !
!DatabasePlatform categoriesFor: #sqlStatementToPopulateColumn:with:!private!services columns! !
!DatabasePlatform categoriesFor: #sqlStatementToRenameColumn:to:!private!services columns! !
!DatabasePlatform categoriesFor: #sqlStatementToRenameTable:to:!private!services tables! !
!DatabasePlatform categoriesFor: #sqlStatementToTestExistenceOfTable:inSchema:!public!services tables! !
!DatabasePlatform categoriesFor: #sqlTextCurrentServerUTCTimestamp!constants!public! !
!DatabasePlatform categoriesFor: #sqlTextForBeginTransaction!constants!private! !
!DatabasePlatform categoriesFor: #sqlTextForDEFAULT:!constants!private! !
!DatabasePlatform categoriesFor: #sqlTextForNOTNULLAttributeConstraint!constants!private! !
!DatabasePlatform categoriesFor: #sqlTextForNOTNULLWithDefaultAttributeConstraint!constants!private! !
!DatabasePlatform categoriesFor: #sqlTextForNOTUNIQUEAttributeConstraint!constants!private! !
!DatabasePlatform categoriesFor: #sqlTextForNULLAttributeConstraint!constants!private! !
!DatabasePlatform categoriesFor: #sqlTextForUNIQUEAttributeConstraint!constants!private! !
!DatabasePlatform categoriesFor: #sqlWildcardForMultipleCharacters!constants!private! !
!DatabasePlatform categoriesFor: #sqlWildcardForSingleCharacter!constants!private! !
!DatabasePlatform categoriesFor: #startColumnAddString!constants!private! !
!DatabasePlatform categoriesFor: #startColumnQuote!private!services columns! !
!DatabasePlatform categoriesFor: #stringToByteArray:for:!conversion-strings!private! !
!DatabasePlatform categoriesFor: #stringToInteger:for:!conversion-strings!private! !
!DatabasePlatform categoriesFor: #stringToStringConverter!conversion-strings!private! !
!DatabasePlatform categoriesFor: #stringToSymbol:for:!conversion-strings!private! !
!DatabasePlatform categoriesFor: #supportsAliasInCompoundQuery!private!testing! !
!DatabasePlatform categoriesFor: #supportsAlterTableForConstraints!public!testing! !
!DatabasePlatform categoriesFor: #supportsANSIJoins!private!testing! !
!DatabasePlatform categoriesFor: #supportsAuthentication!constants!private! !
!DatabasePlatform categoriesFor: #supportsBinding!private!testing! !
!DatabasePlatform categoriesFor: #supportsCaseInsensitiveLike!private!testing! !
!DatabasePlatform categoriesFor: #supportsConstraints!constants!public! !
!DatabasePlatform categoriesFor: #supportsDecimalsOnAllNumerics!private!testing! !
!DatabasePlatform categoriesFor: #supportsGroupWritingFor:!private!testing! !
!DatabasePlatform categoriesFor: #supportsInformationSchema!private!testing! !
!DatabasePlatform categoriesFor: #supportsIntersect!private!testing! !
!DatabasePlatform categoriesFor: #supportsLimit!private!testing! !
!DatabasePlatform categoriesFor: #supportsMillisecondsInTimes!constants!private! !
!DatabasePlatform categoriesFor: #supportsMultipleOpenCursors!private!testing! !
!DatabasePlatform categoriesFor: #supportsOffset!private!testing! !
!DatabasePlatform categoriesFor: #supportsRowCount!private!testing! !
!DatabasePlatform categoriesFor: #supportsSchemaForLogin!public!testing! !
!DatabasePlatform categoriesFor: #supportsSchemas!public!testing! !
!DatabasePlatform categoriesFor: #supportsStringQueryOnBlobs!private!testing! !
!DatabasePlatform categoriesFor: #supportsTableOwners!private!testing! !
!DatabasePlatform categoriesFor: #supportsVariableSizedNumerics!constants!private! !
!DatabasePlatform categoriesFor: #symbolToString:for:!conversion-strings!private! !
!DatabasePlatform categoriesFor: #symbolToStringConverter!conversion-strings!private! !
!DatabasePlatform categoriesFor: #tableNameFor:!private!services tables! !
!DatabasePlatform categoriesFor: #text!private!types! !
!DatabasePlatform categoriesFor: #time!private!types! !
!DatabasePlatform categoriesFor: #timeConverter!conversion-times!private! !
!DatabasePlatform categoriesFor: #timestamp!private!types! !
!DatabasePlatform categoriesFor: #timestampConverter!conversion-times!private! !
!DatabasePlatform categoriesFor: #timestampToDateConverter!conversion-times!private! !
!DatabasePlatform categoriesFor: #toDate:for:!conversion-times!private! !
!DatabasePlatform categoriesFor: #toGlorpType:fromString:!conversion-strings!private! !
!DatabasePlatform categoriesFor: #toTime:for:!conversion-times!private! !
!DatabasePlatform categoriesFor: #toTimestamp:for:!conversion-times!private! !
!DatabasePlatform categoriesFor: #trimString:for:!conversion-strings!private! !
!DatabasePlatform categoriesFor: #typeNamed:ifAbsentPut:!private!types! !
!DatabasePlatform categoriesFor: #types!private!types! !
!DatabasePlatform categoriesFor: #typesWithValues!private!type helpers! !
!DatabasePlatform categoriesFor: #typeWithDatabaseName:characterWidth:!private!types! !
!DatabasePlatform categoriesFor: #typeWithDatabaseName:characterWidth:extraInformation:!private!types! !
!DatabasePlatform categoriesFor: #unpadString:for:!conversion-strings!private! !
!DatabasePlatform categoriesFor: #useBinding!private!testing! !
!DatabasePlatform categoriesFor: #useBinding:!private!testing! !
!DatabasePlatform categoriesFor: #useMicrosoftOuterJoins!private!testing! !
!DatabasePlatform categoriesFor: #useOracleOuterJoins!private!testing! !
!DatabasePlatform categoriesFor: #usesArrayBindingRatherThanGrouping!private!testing! !
!DatabasePlatform categoriesFor: #usesIdentityColumns!private!testing! !
!DatabasePlatform categoriesFor: #usesInformationSchema!public!testing! !
!DatabasePlatform categoriesFor: #usesNullForEmptyStrings!constants!private! !
!DatabasePlatform categoriesFor: #usesNullForFalse!constants!private! !
!DatabasePlatform categoriesFor: #usesPrimaryKeyUniqueConstraintNames!private!testing! !
!DatabasePlatform categoriesFor: #validateTableName:!private!services tables! !
!DatabasePlatform categoriesFor: #varbinary!private!types! !
!DatabasePlatform categoriesFor: #varbinary:!private!types! !
!DatabasePlatform categoriesFor: #varchar!private!types! !
!DatabasePlatform categoriesFor: #varChar!private!types! !
!DatabasePlatform categoriesFor: #varchar:!private!types! !
!DatabasePlatform categoriesFor: #varChar:!private!types! !
!DatabasePlatform categoriesFor: #versionFieldFor:!private!types! !
!DatabasePlatform categoriesFor: #vwConnectionClass!accessing!private! !
!DatabasePlatform categoriesFor: #vwEXDIConnectionClass!accessing!private! !
!DatabasePlatform categoriesFor: #vwODBCConnectionClass!accessing!public! !
!DatabasePlatform categoriesFor: #writeConstraintToDrop:onStream:!public!services constraints! !

!DatabasePlatform class methodsFor!

createLoginFromConnectionDictionary: aDict 
	self subclassResponsibility!

loginWithConnectionDictionary: aDict 
	| platformClass |
	platformClass := self glorpAllSubclasses
				detect: [:cls | cls understandsConnectionDictionary: aDict].
	^ platformClass isNil
		ifFalse: [platformClass createLoginFromConnectionDictionary: aDict] ifTrue: [nil]!

understandsConnectionDictionary: aDict

	^false!

useBindingByDefault
	^true!

useBindingIfSupported
	"If I am nil (the default), each platform that supportsBinding lazily initializes to its defaultUseBinding value.  If I have a value, I force lazy initialization of each platform that supportsBinding to my value.  Normally, a value is set only for specific testing purposes."

	UseBindingIfSupported isNil ifTrue: [^self useBindingByDefault].
	^UseBindingIfSupported!

useBindingIfSupported: aBooleanOrNil
	"If I am nil (the default), each platform that supportsBinding lazily initializes to its defaultUseBinding value.  If I have a value, I force lazy initialization of each platform that supportsBinding to my value.  Normally, a value is set only for specific testing purposes."

	UseBindingIfSupported := aBooleanOrNil.! !
!DatabasePlatform class categoriesFor: #createLoginFromConnectionDictionary:!*eoglorp!private! !
!DatabasePlatform class categoriesFor: #loginWithConnectionDictionary:!*eoglorp!private! !
!DatabasePlatform class categoriesFor: #understandsConnectionDictionary:!*eoglorp!private! !
!DatabasePlatform class categoriesFor: #useBindingByDefault!public!testing! !
!DatabasePlatform class categoriesFor: #useBindingIfSupported!private!testing! !
!DatabasePlatform class categoriesFor: #useBindingIfSupported:!private!testing! !

DatabaseRow guid: (GUID fromString: '{7F5BCB13-D6DA-4A75-9BF5-14C82FE747FE}')!
DatabaseRow comment: 'This represents the data to be written out to a row. Database rows are normally stored in a rowmap, keyed according to their table and the object that did the primary writes to them. We expect that that''s only one object, although embedded values are an exception to that.

Instance Variables:

	table	<DatabaseTable>	The table holding the data
	contents	<IdentityDictionary>	Holds the fields with their values, indirectly through FieldValueWrapper instances.
	shouldBeWritten	<Boolean>	Normally true, but can be set false to suppress writing of a particular row. Used with embedded value mappings, where we create their row, unify it with the parent row, and suppress writing of the original row.
	owner	<Object>	The primary object that wrote into this row, would also be the key into the rowmap.
	forDeletion <Boolean> Is this row to be deleted or written.
	status <Symbol> Used for topological sort to determine write order. We keep a symbol indicating the "visited" status of the row while doing the sort. This doesn''t separate the sort from the row, using each row directly as a node in the graph. This is less general, but uses less memory than other alternatives.
	parent <DatabaseRow> Also used during sorting. Keep track of our parent node.
	oldVersion <DatabaseRow> The old version of us. Used if we increment a version field to keep track of what its value was before.
	generated <Boolean> True if we have already had field with generated types generated in the preWrite phase. This is needed because we generate all sequences before doing any writes. But if we use an explicit writeRow: operation, then we want to make sure that generated values are generated. And some generated types write over what was there before (e.g. optimistic lock fields). So we need a way of knowing if generation has already been done. Note that this doesn''t apply if using post-write sequence generation (i.e. identity columns) because then it''s all after the write. This makes certain things not work as well, but that''s what you get for using identity columns.
	'!
!DatabaseRow categoriesForClass!Unclassified! !
!DatabaseRow methodsFor!

= aRow
	self class == aRow class ifFalse: [^false].
	^self equals: aRow.!

at: aField

	^self at: aField ifAbsent: [self error: 'missing field'].!

at: aField ifAbsent: absentBlock

	^(self wrapperAt: aField ifAbsent: [^absentBlock value]) contents.!

at: aField ifAbsentPut: absentBlock
	| old |
	old := contents at: aField position.
	old == self class emptySlot ifTrue: [absentBlock value] ifFalse: [^old contents]!

atFieldNamed: aString

	| field |
	field := table fieldNamed: aString.
	^self at: field!

atFieldNamed: aString ifAbsent: aBlock

	| field |
	field := table fieldNamed: aString ifAbsent: aBlock.
	^self at: field ifAbsent: aBlock.!

atFieldNamed: aString put: anObject

	| field |
	field := table fieldNamed: aString.
	^self at: field put: anObject!

contentsAt: aField ifAbsentPut: absentBlock

	| old newValue |
	old := contents at: aField position.
	^old == self class emptySlot 
		ifTrue: [
			newValue := absentBlock value.
			contents at: aField position put: newValue.
			newValue]
		ifFalse: [old].!

doesNotUnderstand: aMessage
	^self atFieldNamed: aMessage selector asString ifAbsent: [super doesNotUnderstand: aMessage].!

equalityStringForField: aDatabaseField

	| stream |
	stream := WriteStream on: (String new: 50).
	self printEqualityStringForField: aDatabaseField on: stream.
	^stream contents.!

equals: aRow 
	self forDeletion = aRow forDeletion ifFalse: [^false].
	self fieldsAndValuesDo: 
			[:eachField :eachWrapper | 
			| otherValue |
			otherValue := aRow at: eachField ifAbsent: [self class missingFieldIndicator].
			eachWrapper = otherValue ifFalse: [^false]].
	^true!

expectsToGetValueFor: aField
	"Are we constrained to a field which is presumably going to get a value, which for the moment must mean it's a serial type. This is primarily required for identity column databases, because in other cases we will pre-allocate sequence numbers, and this should never come up"
	| wrapper innerWrappers testBlock containing |
	wrapper := self wrapperAt: aField ifAbsent: [^false].
	innerWrappers := wrapper allInnerWrappers.
	testBlock := [:eachField | (eachField ~~ aField and: [eachField type isSerial]) ifTrue: [^true]].
	(Array with: wrapper), innerWrappers do: [:eachWrapper |
		containing := eachWrapper containedBy.
		containing glorpIsCollection 
			ifTrue: [containing keysDo: testBlock]
			ifFalse: [testBlock value: containing key]].
	^false.!

fields
	^self table fields select: [:each | (self includesField: each)].!

fieldsAndValidValuesDo: aBlock 
	"If iterating over fields and values, we include wrappers with no value assigned yet. This might or might not be what we want. This one just iterates over ones with actual values"
	self isEmpty ifTrue: [^self].
	table fields do: [:each |  | value |
		value := self at: each ifAbsent: [self class missingFieldIndicator].
		value == self class missingFieldIndicator ifFalse: [
			aBlock value: each value: value]].!

fieldsAndValuesDo: aBlock 
	table isNil ifTrue: [^self].
	table fields do: [:each | aBlock value: each value: (self at: each ifAbsent: [self class missingFieldIndicator])]!

fieldsDo: aBlock 
	1 to: contents size do: [:i |
		| value |
		value := contents at: i.
		(value == self class emptySlot) ifFalse: [aBlock value: (self table fields at: i)]].!

fieldsNeedingUpdate
	"Return a list of the fields that are different from the previous version (if we know what that was)"

	| result |
	oldVersion isNil ifTrue: [^self nonPrimaryKeyFields].
	result := OrderedCollection new: contents size.
	self fieldsDo: 
			[:each | 
			each isPrimaryKey 
				ifFalse: 
					[(oldVersion at: each ifAbsent: [self class missingFieldIndicator]) 
						= (self at: each) ifFalse: [result add: each]]].
	^result!

fieldValuesDo: aBlock 
	contents do: [:each | each == self class emptySlot ifFalse: [aBlock value: each]].!

forDeletion
	^forDeletion.!

forDeletion: aBoolean
	forDeletion := aBoolean.!

hash
	| basic |
	basic := self table hash.
	self fieldsAndValuesDo: 
			[:eachField :eachValue | 
			basic := basic bitXor: eachField hash.
			basic := basic bitXor: eachValue hash].
	^basic.!

hasValueFor: aField

	^(self wrapperAt: aField ifAbsent: [^false]) hasValue.!

includesField: aField
	^(contents at: aField position) ~~ self class emptySlot.!

initialize
	shouldBeWritten := true.
	forDeletion := false.
	generated := false.!

isEmpty
	| empty |
	contents isNil ifTrue: [^true].
	empty := self class emptySlot.
	contents do: [:each | each == empty ifFalse: [^false]].
	^true.!

keysAndValuesDo: aBlock
	1 to: contents size do: [:i |
		| value | 
		value := contents at: i.
		value == self class emptySlot ifFalse: [
			aBlock value: (self table fields at: i) value: value contents]].!

lockKeyFields

	^table lockKeyFields!

needsValueFromSequence: aSequence 
	self table fields do: 
			[:each | 
			((self shouldGenerateValueFor: each) and: [each type sequence == aSequence]) 
				ifTrue: [^true]].
	^false!

nonGeneratedFields
	"Return a list of our fields that are not generated"
	| result |
	result := OrderedCollection new: contents size.
	self table fields do: [:field | 
		field isGenerated not ifTrue: [result add: field]].
	^result.!

nonGeneratedFieldsWithValues
	"Return a list of our fields that a) are not generated or b) have values. That is, exclude values we expect the database to generate"
	| result |
	result := OrderedCollection new: contents size.
	self fieldsAndValidValuesDo: [:field :value | 
		(value notNil or: [field isGenerated not]) ifTrue: [result add: field]].
	^result.!

nonPrimaryKeyFields
	| result |
	result := OrderedCollection new: contents size.
	self fieldsDo: [:field | 
		field isPrimaryKey ifFalse: [result add: field]].
	^result.!

oldVersionAt: aField ifAbsent: aBlock
	oldVersion isNil ifTrue: [^aBlock value].
	^oldVersion at: aField ifAbsent: aBlock.!

owner

	^owner.!

owner: anObject

	owner := anObject.!

parent
	^parent!

parent: aDatabaseRow
	parent := aDatabaseRow.!

postWriteAssignSequencesUsing: anAccessor

	self table fields do: [:each |
		(self hasValueFor: each) ifFalse: [
			each type postWriteAssignSequenceValueFor: each in: self using: anAccessor]].!

preWriteAssignSequencesUsing: aSession
	"Skip if we've already been generated"
	generated ifTrue: [^self].
	self table fields do: 
			[:each | 
			(self shouldGenerateValueFor: each in: aSession) 
				ifTrue: 
					[each type 
						preWriteAssignSequenceValueFor: each
						in: self
						using: (aSession isNil ifTrue: [nil] ifFalse: [aSession accessor])]].
	generated := true.!

primaryKey

	| |
	self table primaryKeyFields isEmpty ifTrue: [^nil].
	^self table hasCompositePrimaryKey 
		ifTrue: [
			self table primaryKeyFields 
				collect: [:each | 	self at: each]]
		ifFalse: [self at: self table primaryKeyFields first ifAbsent: [nil]].!

printEqualityStringForField: aDatabaseField on: aCommand 
	"Get around PostgreSQL bug.  Qualified names cannot appear in SET expression."
	aCommand nextPutAll: (aDatabaseField platform nameForColumn: aDatabaseField name).
	aCommand nextPutAll: ' = '.
	self printValueOfField: aDatabaseField on: aCommand!

printEqualityTemplateForField: aDatabaseField on: aCommand
	"Get around PostgreSQL bug.  Qualified names cannot appear in SET expression."
	| bind |
	aCommand nextPutAll: (aDatabaseField platform nameForColumn: aDatabaseField name).
	aCommand nextPutAll: ' = '.
	bind := aCommand canBind: (self at: aDatabaseField ifAbsent: [nil]) to: aDatabaseField type.
	bind
		ifTrue: [aCommand nextPutAll: '?']
		ifFalse: [self printValueOfField: aDatabaseField on: aCommand].!

printEqualityTemplateForField: aDatabaseField on: aCommand withBinding: aBoolean
	"Get around PostgreSQL bug.  Qualified names cannot appear in SET expression."
	aCommand nextPutAll: (aDatabaseField platform nameForColumn: aDatabaseField name).
	aCommand nextPutAll: ' = '.
	aBoolean 
		ifTrue: [aCommand nextPutAll: '?']
		ifFalse: [self printValueOfField: aDatabaseField on: aCommand].!

printFieldNamesOn: aWriteStream 
	GlorpHelper 
		do: [:each | aWriteStream nextPutAll: each name]
		for: self table fields
		separatedBy: [aWriteStream nextPutAll: ','].!

printLockEqualityForField: aDatabaseField on: aStream
	| expression |
	expression := ((BaseExpression new getTable: aDatabaseField table) getField: aDatabaseField) get: #= withArguments: (Array with: (oldVersion at: aDatabaseField)).
	expression printSQLOn: aStream withParameters: #().!

printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: '(' , ((table isNil or: [table name isNil]) ifTrue: [''] ifFalse: [table name]),  ')'.
	aStream cr.
	self fieldsAndValuesDo: [:eachField :eachWrapper |
		aStream nextPutAll: '    '.
		eachField printOn: aStream.
		aStream nextPutAll: '->'.
		eachWrapper printOn: aStream.
		aStream cr.].!

printPrimaryKeyStringOn: aStream 
	"If there is no primary key (i.e. this is a link table) use all the values that we have"
	| fields |
	fields := table primaryKeyFields isEmpty 
		ifTrue: [self fields]
		ifFalse: [table primaryKeyFields].
	GlorpHelper
		do:  [:eachField |
			self printEqualityStringForField: eachField on: aStream]
		for: fields
		separatedBy: [aStream nextPutAll: ' AND '].!

printPrimaryKeyTemplateOn: aStream
	"If there is no primary key (i.e. this is a link table) use all the values that we have"
	| fields |
	fields := table primaryKeyFields isEmpty 
		ifTrue: [self fields]
		ifFalse: [table primaryKeyFields].
	GlorpHelper
		do:  [:eachField |
			self printEqualityTemplateForField: eachField on: aStream]
		for: fields
		separatedBy: [aStream nextPutAll: ' AND '].!

printValueOfField: aDatabaseField on: aWriteStream 
	aDatabaseField type print: (self at: aDatabaseField ifAbsent: [nil]) on: aWriteStream.!

relatedRowsIn: aRowMap
	| relatedRows |
	relatedRows := OrderedCollection new.
	self table foreignKeyConstraints
		do:
			[:eachConstraint | relatedRows addAll: (eachConstraint rowsRelatedTo: self)].
	^relatedRows.!

shouldBeWritten
	table isNil ifTrue: [^shouldBeWritten].
	^shouldBeWritten and: [table shouldBeWritten].!

shouldBeWritten: aBoolean 
	shouldBeWritten := aBoolean!

shouldGenerateValueFor: aField
	^self shouldGenerateValueFor: aField in: nil.!

shouldGenerateValueFor: aField in: aSession
	| isInsert |
	aField isGenerated ifFalse: [^false].
	aField type generatesOverExistingValues ifTrue: [^true].
	"For those that don't write over existing values, only update them if this is an insert. Otherwise we can't tell the difference between an empty column and one which we're just not updating in this unit of work. This requires the session, but the session may be nil - if so, default to true, because that usage is just for reserving sequence numbers, and it's better to err on the side of too many."
	isInsert := aSession isNil ifTrue: [true] ifFalse: [aSession shouldInsert: self].
	^isInsert and: [(self hasValueFor: aField) not].!

status
	^status!

status: aSymbol
	status := aSymbol!

table
	"Private - Answer the value of the receiver's ''table'' instance variable."

	^table!

table: aDatabaseTable
	table := aDatabaseTable.
	contents := Array new: (aDatabaseTable fields size).
	contents atAllPut: self class emptySlot.!

values
	| result |
	result := OrderedCollection new.
	self keysAndValuesDo: [:key :value |
		result add: value].
	^result!

withAllFieldsIn: aRow 
	"aRow represents our original state. Make sure that we have all the fields in aRow, using nil values for any that are missing. This is needed if, e.g. we have been removed from a 1-many relationship, so we don't get a value generated for our foreign key, but we should still write it as a nil. We have to distinguish this from the case of a value that simply hasn't changed."

	aRow isEmpty ifTrue: [^self].
	oldVersion := aRow.
	self numberOfFields = table fields size ifTrue: [^self].
	aRow fieldsAndValidValuesDo: 
			[:eachField :eachValue | 
			(self includesField: eachField) ifFalse: [self at: eachField put: nil]].
	^self!

wrapperAt: aField

	^self wrapperAt: aField ifAbsent: [self error: 'Field not found'].!

wrapperAt: aField ifAbsent: aBlock
	| thing |
	contents isNil ifTrue: [^aBlock value].
	thing := contents at: aField position.
	^thing == self class emptySlot ifTrue: [aBlock value] ifFalse: [thing]!

wrapperAt: aField put: aWrapper
	| old |
	old := contents at: aField position.
	old == aWrapper ifTrue: [^self].
	contents at: aField position put: aWrapper.
	aWrapper isNowContainedBy: self and: aField.! !
!DatabaseRow categoriesFor: #=!comparing!private! !
!DatabaseRow categoriesFor: #at:!accessing!private! !
!DatabaseRow categoriesFor: #at:ifAbsent:!accessing!private! !
!DatabaseRow categoriesFor: #at:ifAbsentPut:!accessing!public! !
!DatabaseRow categoriesFor: #atFieldNamed:!accessing!private! !
!DatabaseRow categoriesFor: #atFieldNamed:ifAbsent:!accessing!private! !
!DatabaseRow categoriesFor: #atFieldNamed:put:!accessing!private! !
!DatabaseRow categoriesFor: #contentsAt:ifAbsentPut:!accessing!private! !
!DatabaseRow categoriesFor: #doesNotUnderstand:!accessing!private! !
!DatabaseRow categoriesFor: #equalityStringForField:!printing!private! !
!DatabaseRow categoriesFor: #equals:!private!testing! !
!DatabaseRow categoriesFor: #expectsToGetValueFor:!private!querying! !
!DatabaseRow categoriesFor: #fields!accessing!private! !
!DatabaseRow categoriesFor: #fieldsAndValidValuesDo:!initializing!private! !
!DatabaseRow categoriesFor: #fieldsAndValuesDo:!initializing!private! !
!DatabaseRow categoriesFor: #fieldsDo:!enumerating!private! !
!DatabaseRow categoriesFor: #fieldsNeedingUpdate!printing!private! !
!DatabaseRow categoriesFor: #fieldValuesDo:!enumerating!private! !
!DatabaseRow categoriesFor: #forDeletion!accessing!private! !
!DatabaseRow categoriesFor: #forDeletion:!accessing!private! !
!DatabaseRow categoriesFor: #hash!private!testing! !
!DatabaseRow categoriesFor: #hasValueFor:!private!querying! !
!DatabaseRow categoriesFor: #includesField:!accessing!private! !
!DatabaseRow categoriesFor: #initialize!initializing!private! !
!DatabaseRow categoriesFor: #isEmpty!private!testing! !
!DatabaseRow categoriesFor: #keysAndValuesDo:!enumerating!private! !
!DatabaseRow categoriesFor: #lockKeyFields!accessing!private! !
!DatabaseRow categoriesFor: #needsValueFromSequence:!private!sequencing! !
!DatabaseRow categoriesFor: #nonGeneratedFields!accessing!private! !
!DatabaseRow categoriesFor: #nonGeneratedFieldsWithValues!accessing!private! !
!DatabaseRow categoriesFor: #nonPrimaryKeyFields!accessing!private! !
!DatabaseRow categoriesFor: #oldVersionAt:ifAbsent:!accessing!private! !
!DatabaseRow categoriesFor: #owner!accessing!private! !
!DatabaseRow categoriesFor: #owner:!accessing!private! !
!DatabaseRow categoriesFor: #parent!accessing!private! !
!DatabaseRow categoriesFor: #parent:!accessing!private! !
!DatabaseRow categoriesFor: #postWriteAssignSequencesUsing:!private!sequencing! !
!DatabaseRow categoriesFor: #preWriteAssignSequencesUsing:!private!sequencing! !
!DatabaseRow categoriesFor: #primaryKey!accessing!private! !
!DatabaseRow categoriesFor: #printEqualityStringForField:on:!printing!private! !
!DatabaseRow categoriesFor: #printEqualityTemplateForField:on:!printing!private! !
!DatabaseRow categoriesFor: #printEqualityTemplateForField:on:withBinding:!printing!private! !
!DatabaseRow categoriesFor: #printFieldNamesOn:!printing!private! !
!DatabaseRow categoriesFor: #printLockEqualityForField:on:!printing!private! !
!DatabaseRow categoriesFor: #printOn:!printing!private! !
!DatabaseRow categoriesFor: #printPrimaryKeyStringOn:!printing!private! !
!DatabaseRow categoriesFor: #printPrimaryKeyTemplateOn:!printing!private! !
!DatabaseRow categoriesFor: #printValueOfField:on:!printing!private! !
!DatabaseRow categoriesFor: #relatedRowsIn:!private!relationships! !
!DatabaseRow categoriesFor: #shouldBeWritten!private!querying! !
!DatabaseRow categoriesFor: #shouldBeWritten:!configuring!private! !
!DatabaseRow categoriesFor: #shouldGenerateValueFor:!private!sequencing! !
!DatabaseRow categoriesFor: #shouldGenerateValueFor:in:!private!sequencing! !
!DatabaseRow categoriesFor: #status!accessing!private! !
!DatabaseRow categoriesFor: #status:!accessing!private! !
!DatabaseRow categoriesFor: #table!accessing!private! !
!DatabaseRow categoriesFor: #table:!configuring!private! !
!DatabaseRow categoriesFor: #values!accessing!private! !
!DatabaseRow categoriesFor: #withAllFieldsIn:!configuring!private! !
!DatabaseRow categoriesFor: #wrapperAt:!accessing!private! !
!DatabaseRow categoriesFor: #wrapperAt:ifAbsent:!accessing!private! !
!DatabaseRow categoriesFor: #wrapperAt:put:!accessing!private! !

!DatabaseRow class methodsFor!

emptySlot
	emptySlot == nil ifTrue: [emptySlot := Object new].
	^emptySlot.!

missingFieldIndicator
	missingFieldIndicator == nil ifTrue: [missingFieldIndicator := Object new].
	^missingFieldIndicator.!

new

	^super new initialize.!

newForTable: aTable

	^self new table: aTable.!

newForTable: aTable withOwner: anObject

	^self new table: aTable; owner: anObject.! !
!DatabaseRow class categoriesFor: #emptySlot!private! !
!DatabaseRow class categoriesFor: #missingFieldIndicator!private! !
!DatabaseRow class categoriesFor: #new!instance creation!private! !
!DatabaseRow class categoriesFor: #newForTable:!instance creation!private! !
!DatabaseRow class categoriesFor: #newForTable:withOwner:!instance creation!private! !

DatabaseTable guid: (GUID fromString: '{710C2471-90EC-45B8-A082-3A5D88F2715B}')!
DatabaseTable comment: 'This is our internal representation of a table in the database. It holds onto enough information for us to both map to the table, and to create it in the database if required.

Instance Variables
	fields	<(Collection of: DatabaseField)>	The columns in the table.
	foreignKeyConstraints	<(Collection of: ForeignKeyConstraint)>	The foreign key constraints between this table and others.
	indexes	<(Collection of: DatabaseIndex)>	The indexes on this table.
	isImaginary	<Boolean>	We allow "imaginary" tables, which don''t really exist in the database. This is most useful in connection with embedded mappings. For example, we can have a Currency object which maps to a CURRENCY table, but there is no currency table in the database, and instead we have a number of embedded mappings that put those fields inside another table.
	lockKeyFields	<Collection of: DatabaseField>	Which fields (which should also be in our field list) are checked when doing optimistic locking
	name	<String>	The name of the table.
	parent	<DatabaseTable | nil>	For a normal table, this is nil. When we create an aliased table we remember who the original table was using this field.
	primaryKeyFields	<(SequenceableCollection of: (DatabaseField))>	Which fields (which should also be in our field list) are primary keys.
	replacementSubSelect	<CompoundQuery | SimpleQuery>	I honestly don''t remember at the moment what this does, but it seems to be used when we''re traversing relationships that are in imaginary tables in order to figure out what the actual join has to be. And maybe the join needs to be done in a subselect rather than as a simple join in certain cases.
	schema	<String>	What is the name of the schema in which this table exists. This is used when printing the qualified name of the table. 
	uniqueConstraints	<(Collection of: UniqueConstraint)>	The unique constraints for this table
'!
!DatabaseTable categoriesForClass!Unclassified! !
!DatabaseTable methodsFor!

<= aTable
	^self qualifiedName <= aTable qualifiedName.!

addAsLockKeyField: aField 

	lockKeyFields add: aField!

addAsPrimaryKeyField: aField

	(fields includes: aField) ifFalse: [self error: 'Primary key field not part of the table'].
	aField isPrimaryKey ifFalse: ["Somebody is calling this as API, since there's an example out on the net that does it that way - just do the right thing instead"
		^aField bePrimaryKey].
	(primaryKeyFields notNil and: [primaryKeyFields includes: aField]) ifTrue: [^self].
	primaryKeyFields := primaryKeyFields isNil ifTrue: [Array with: aField] ifFalse: [primaryKeyFields, (Array with: aField)].!

addField: aField

	fields add: aField.
	aField isPrimaryKey ifTrue: [
		self addAsPrimaryKeyField: aField].
	aField table: self.
	aField position: fields size.
	^aField.!

addFields: aCollection fromAccessor: aDatabaseAccessor
	| stream |
	aCollection
		do:
			[:each | 
			aDatabaseAccessor
				execute:
					[stream := WriteStream on: String new.
					stream
						nextPutAll: 'ALTER TABLE ';
						nextPutAll: self name;
						nextPutAll: ' ADD ';
						nextPutAll: self platform startColumnAddString.
					self platform printColumn: each on: stream.
					stream nextPutAll: self platform endColumnAddString.
					aDatabaseAccessor executeSQLStringNoResult: stream contents]
				ifError: [:ex | ex pass]].!

addForeignKeyFrom: sourceField to: targetField
	^self
		addForeignKeyFrom: sourceField
		to: targetField
		suffixExpression: nil.!

addForeignKeyFrom: sourceField to: targetField from: sourceField2 to: targetField2
	^self
		addForeignKeyFromAll: (Array with: sourceField with: sourceField2)
		toAll: (Array with: targetField with: targetField2)
		suffixExpression: nil.!

addForeignKeyFrom: sourceField to: targetField from: sourceField2 to: targetField2 from: sourceField3 to: targetField3
	^self
		addForeignKeyFromAll: (Array with: sourceField with: sourceField2 with: sourceField3)
		toAll: (Array with: targetField with: targetField2 with: targetField3)
		suffixExpression: nil.!

addForeignKeyFrom: sourceField to: targetField suffixExpression: suffixExpression
	| newFK |
	newFK := ForeignKeyConstraint
					sourceField: sourceField
					targetField: targetField
					suffixExpression: suffixExpression.
	newFK name: (newFK name , (foreignKeyConstraints size + 1) printString).
	foreignKeyConstraints add: newFK.
	^newFK.!

addForeignKeyFrom: sourceField toField: targetField

	^self addForeignKeyFrom: sourceField to: targetField!

addForeignKeyFrom: sourceField toTable: targetTable

	^self addForeignKeyFrom: sourceField toField: (targetTable fieldNamed: 'ID' ifAbsent: [^nil])!

addForeignKeyFromAll: sourceFields toAll: targetFields
	^self addForeignKeyFromAll: sourceFields toAll: targetFields suffixExpression: nil.!

addForeignKeyFromAll: sourceFields toAll: targetFields suffixExpression: suffixExpression
	| newFK |
	newFK := ForeignKeyConstraint
					sourceFields: sourceFields
					targetFields: targetFields
					suffixExpression: suffixExpression.
	newFK name: (newFK name , (foreignKeyConstraints size + 1) printString).
	^foreignKeyConstraints add: newFK.!

addIndex: aDatabaseIndex
	^indexes add: aDatabaseIndex.!

addIndexForField: aField
	^indexes add: (DatabaseIndex forField: aField).!

addIndexForField: aField and: anotherField
	^self addIndexForFields: (Array with: aField with: anotherField).!

addIndexForField: aField and: anotherField and: thirdField
	^self addIndexForFields: (Array with: aField with: anotherField with: thirdField).!

addIndexForFields: aFieldCollection
	^indexes add: (DatabaseIndex forFields: aFieldCollection).!

addUniqueConstraintFor: sourceFields

	| newConstraint |
	newConstraint := UniqueConstraint
		forTable: self
		fields: sourceFields.
	^uniqueConstraints add: newConstraint!

allSequences
	"Return a dictionary of all the sequences associated with this table, indexed by name."

	| result |
	result := Dictionary new.
	self fields do: [:eachField | 
 		eachField type hasSequence ifTrue:
			[result at: eachField type sequence name put: eachField type sequence]].
	^result.!

createFieldNamed: aString type: dbType

	| existingField |
	existingField := fields detect: [:each | each name = aString] ifNone: [nil].
	(existingField notNil)
	ifTrue: [self error: 'field ', aString, ' already exists'].
	^self addField: (DatabaseField named: aString type: dbType)!

creator
	^self schema!

creator: aString
	"For backward-compatibility. Use schema: instead."
	self schema: aString.!

dropFields: aCollection fromAccessor: aDatabaseAccessor
	aCollection
		do:
			[:each | 
			aDatabaseAccessor
				execute:
					[aDatabaseAccessor
						executeSQLStringNoResult:
							'ALTER TABLE ' , self name , ' DROP COLUMN ' , each name]
				ifError: [:ex | ex pass]].!

dropForeignKeyConstraintsFromAccessor: aDatabaseAccessor 
	self foreignKeyConstraints 
		do: [:each | aDatabaseAccessor dropConstraint: each]!

dropFromAccessor: aDatabaseAccessor
	aDatabaseAccessor dropTable: self  ifError: [].!

dropPrimaryKeyConstraintsFromAccessor: aDatabaseAccessor
	(self primaryKeyFields isEmpty not and: [self platform usesPrimaryKeyUniqueConstraintNames])
		ifTrue: [
			aDatabaseAccessor	execute:
				[aDatabaseAccessor
						executeSQLStringNoResult:
							'ALTER TABLE ' , self name , ' DROP '
								, self platform nameForPrimaryKeyConstraintsWhenDropping , ' '
								, self primaryKeyUniqueConstraintName]
				ifError:
					[:ex | 
					Transcript show: ex messageText.
					ex return: nil].
		aDatabaseAccessor execute: [
			aDatabaseAccessor executeSQLStringNoResult:
					'ALTER TABLE ' , self name , ' DROP '
						, self platform nameForPrimaryKeyConstraintsWhenDropping , ' '
						, self primaryKeyConstraintName]
			ifError: [:ex | 
				Transcript show: ex messageText.
				ex return: nil]].!

fieldNamed: aString

	^self fieldNamed: aString ifAbsent: [self error: 'No field ', aString, ' in ', self name].!

fieldNamed: aString ifAbsent: aBlock

	^self fieldNamed: aString ifAbsent: aBlock caseSensitive: false.!

fieldNamed: aString ifAbsent: aBlock caseSensitive: aBoolean
	| fieldName |
	fieldName := aBoolean ifTrue: [aString asString] ifFalse: [aString asUppercase].
	^fields
		detect:
			[:each | (aBoolean ifTrue: [each name] ifFalse: [each name asUppercase]) = fieldName]
		ifNone: aBlock.!

fields

	^fields!

foreignKeyConstraints
	"Private - Answer the value of the receiver's ''foreignKeyConstraints'' instance variable."

	^foreignKeyConstraints!

foreignKeyConstraintsForCreation
	"Return the foreign key constraints that should actually be created in the database. Omit those that refer to imaginary tables"
	^foreignKeyConstraints select: [:each |
		each isImaginary not and: [each shouldCreateInDatabase]].!

foreignKeyConstraintsForField: aField
	^self foreignKeyConstraints select: [:each | each sourceFields includes: aField].!

hasCompositePrimaryKey

	^primaryKeyFields size > 1.!

hasConstraints
	^self hasForeignKeyConstraints or: [self hasPrimaryKeyConstraints or: [self hasUniqueConstraints]]!

hasConstraintsTo: aTable
	self foreignKeyConstraints
		do: [:each | (each refersToTable: aTable) ifTrue: [^true]].
	^false.!

hasConstraintsToAnyOf: aCollectionOfTables
	aCollectionOfTables do: [:each |
		(self hasConstraintsTo: each) ifTrue: [^true]].
	^false.!

hasFieldNamed: aString

	^(self fieldNamed: aString ifAbsent: [nil] caseSensitive: false) notNil.!

hasForeignKeyConstraints
	^foreignKeyConstraints isEmpty not!

hasForeignKeyConstraintsForField: aField
	^self foreignKeyConstraints anySatisfy: [:each | each sourceFields includes: aField].!

hasForeignKeyConstraintsForTargetField: aField
	^self foreignKeyConstraints anySatisfy: [:each | each targetFields includes: aField].!

hasIndexForField: aField
	^(indexes detect: [:each | each indexesField: aField] ifNone: [nil]) notNil.!

hasIndexNamed: aString

	^indexes anySatisfy: [:each | each name = aString]!

hasPrimaryKeyConstraints
	^self primaryKeyFields isEmpty not.!

hasUniqueConstraints
	^uniqueConstraints isEmpty not!

imaginaryReplacementName
	| stream |
	stream := WriteStream on: (String new: 100).
	stream nextPut: $(.
	stream nextPutAll: (replacementSubSelect sqlWith: #()) sqlString.
	stream nextPutAll: ') '.
	^stream contents.!

indexes
	^indexes.!

initialize

	schema := ''.
	fields := OrderedCollection new.
	foreignKeyConstraints := OrderedCollection new: 4.
	uniqueConstraints := OrderedCollection new.
	lockKeyFields := OrderedCollection new.
	indexes := OrderedCollection new.
	isImaginary := false.!

isAliased
	^parent notNil.!

isEmpty
	^fields isEmpty.!

isImaginary
	^isImaginary.!

isImaginary: aBoolean
	isImaginary := aBoolean.!

isTheSameAs: aDatabaseTable
	"Validate that two tables are exactly the same. Most useful for testing purposes, rather than normal usage, which is why we don't just use the #= method."
	self name asUppercase = aDatabaseTable name asUppercase ifFalse: [^false].
	"For the test purposes, we won't worry about schema right now."
"	self schema = aDatabaseTable schema ifFalse: [^false]."
	self fields size = aDatabaseTable fields size ifFalse: [^false].
	self fields asSortedCollection with: aDatabaseTable fields asSortedCollection do: [:a :b | (a isTheSameAs: b) ifFalse: [^false]].
	self foreignKeyConstraints size = aDatabaseTable foreignKeyConstraints size ifFalse: [^false].
	self foreignKeyConstraints with: aDatabaseTable foreignKeyConstraints do: [:a :b | (a isTheSameAs: b) ifFalse: [^false]].	
	self indexes with: aDatabaseTable indexes do: [:a :b | (a isTheSameAs: b) ifFalse: [^false]].
	self lockKeyFields with: aDatabaseTable lockKeyFields do: [:a :b | (a isTheSameAs: b) ifFalse: [^false]].	
	self primaryKeyFields size = aDatabaseTable primaryKeyFields size ifFalse: [^false].
	self primaryKeyFields asSortedCollection with: aDatabaseTable primaryKeyFields asSortedCollection do: [:a :b | (a isTheSameAs: b) ifFalse: [^false]].
	self isImaginary = aDatabaseTable isImaginary ifFalse: [^false].
	^true.
	"Note, we ignore parent and replacementSubselect"!

lockKeyFields

	^lockKeyFields!

name

	^name!

name: anObject

	name := anObject!

newFieldNamed: aString

	^self error: 'use #createFieldNamed:type:'!

nonPrimaryKeyFields

	^self fields reject: [:each | each isPrimaryKey].!

parent

	^parent.!

parent: aDatabaseTable

	parent := aDatabaseTable.!

platform
	fields do: [:each | ^each type platform].
	^nil.!

postInitializeIn: aDescriptorSystem
	"Any initialization that happens after all the fields have been added"
	1 to: fields size do: [:i |
		| each | 
		"If fields were read from the database, set their position now"
		each := fields at: i.
		each table: self.
		each position: i.
		each postInitializeIn: aDescriptorSystem].!

primaryKeyConstraintName

	^self name, '_PK'.!

primaryKeyConstraints
	"Answer the value of the receiver's ''PrimaryKeyConstraints'' instance variable."
	^self hasPrimaryKeyConstraints
		ifTrue: [ PrimaryKeyConstraint forTable: self ]
		ifFalse: [ nil ]!

primaryKeyFields

	primaryKeyFields isNil ifTrue: [primaryKeyFields := fields select: [:each | each isPrimaryKey yourSelf "might be a proxy"]].
	^primaryKeyFields.!

primaryKeyUniqueConstraintName
	^self platform usesPrimaryKeyUniqueConstraintNames ifTrue: [self name, '_UNIQ'] ifFalse: [''].!

printAddPrimaryKeyConstraintsOn: aStream for: aDatabaseAccessor 
	self primaryKeyFields isEmpty ifTrue: [^self].
	aStream nextPutAll: 'ALTER TABLE ' , self name.
	aStream nextPutAll: ' ADD CONSTRAINT '.
	aStream nextPutAll: self primaryKeyConstraintName.
	aStream nextPutAll: ' PRIMARY KEY  ('.
	GlorpHelper 
		print: [:each | each name]
		on: aStream
		for: self primaryKeyFields
		separatedBy: ','.
	aStream nextPut: $).

	aStream
		nextPutAll: ',';
		cr.

	aStream nextPutAll: ' ADD CONSTRAINT '.
	aStream nextPutAll: self primaryKeyUniqueConstraintName.
	aStream nextPutAll: ' UNIQUE  ('.
	GlorpHelper 
		print: [:each | each name]
		on: aStream
		for: self primaryKeyFields
		separatedBy: ','.
	aStream nextPut: $)!

printDelimiterOn: aStream
	
	aStream
		nextPut: $,;
		cr!

printOn: aStream

	super printOn: aStream.
	aStream 
		nextPutAll: '(';
		nextPutAll: ((schema isNil or: [schema isEmpty]) ifTrue: [''] ifFalse: [schema, '.']);
		nextPutAll: (name isNil ifTrue: [''] ifFalse: [name]);
		nextPutAll: ')'.!

printPrimaryKeyConstraintsOn: aStream for: aDatabaseAccessor 
	self primaryKeyFields isEmpty ifTrue: [^self].
	aStream nextPutAll: 'CONSTRAINT '.
	aStream nextPutAll: self primaryKeyConstraintName.
	aStream nextPutAll: ' PRIMARY KEY  ('.
	GlorpHelper 
		print: [:each | each name]
		on: aStream
		for: self primaryKeyFields
		separatedBy: ','.
	aStream nextPut: $).

	aStream
		nextPutAll: ',';
		cr.

	aStream nextPutAll: 'CONSTRAINT '.
	aStream nextPutAll: self primaryKeyUniqueConstraintName.
	aStream nextPutAll: ' UNIQUE  ('.
	GlorpHelper 
		print: [:each | each name]
		on: aStream
		for: self primaryKeyFields
		separatedBy: ','.
	aStream nextPut: $)!

printSQLOn: aWriteStream withParameters: aDictionary
	aWriteStream nextPutAll: self qualifiedName!

qualifiedName

	^(schema isNil or: [schema isEmpty]) ifTrue: [name] ifFalse: [schema, '.', name].!

removeFieldNamed: aString
	^self removeFieldNamed: aString ifAbsent: [self error: 'No such field'].!

removeFieldNamed: aString ifAbsent: aBlock
	
	fields remove: (self fieldNamed: aString ifAbsent: aBlock).
	1 to: fields size do: [:i | (fields at: i) position: i].!

replacementSubSelect: aQuery
	replacementSubSelect := aQuery.!

schema
	^schema!

schema: aString

	schema := aString.
	parent isNil ifTrue: [
		self sequences do: [:each | each schema: aString]].!

sequences
	"Return a collection (which is likely either empty or of size 1) of all the sequences we use."
	| sequences |
	sequences := OrderedCollection new: 2.
	self primaryKeyFields do: [:each | each type hasSequence ifTrue: [sequences add: each type sequence]].
	^sequences.!

shouldBeWritten
	^isImaginary not.!

sqlString

	^name.!

sqlTableName
	"Our name, as appropriate for the list of tables in a SQL statement. Take into account aliasing"

	^parent isNil
		ifTrue:
			[self isImaginary
				ifTrue: [self imaginaryReplacementName]
				ifFalse: [self qualifiedName]]
		ifFalse: [parent sqlTableName , ' ' , self qualifiedName].!

uniqueConstraints
	"Private - Answer the value of the receiver's ''uniqueConstraints'' instance variable."

	^uniqueConstraints!

validateIn: aDescriptorSystem
	"Do any tests we can to make sure that this definition makes sense in this context"! !
!DatabaseTable categoriesFor: #<=!comparing!private! !
!DatabaseTable categoriesFor: #addAsLockKeyField:!private!private/fields! !
!DatabaseTable categoriesFor: #addAsPrimaryKeyField:!private!private/fields! !
!DatabaseTable categoriesFor: #addField:!fields!private! !
!DatabaseTable categoriesFor: #addFields:fromAccessor:!obsolete!private! !
!DatabaseTable categoriesFor: #addForeignKeyFrom:to:!fields!private! !
!DatabaseTable categoriesFor: #addForeignKeyFrom:to:from:to:!fields!private! !
!DatabaseTable categoriesFor: #addForeignKeyFrom:to:from:to:from:to:!fields!private! !
!DatabaseTable categoriesFor: #addForeignKeyFrom:to:suffixExpression:!fields!private! !
!DatabaseTable categoriesFor: #addForeignKeyFrom:toField:!fields!private! !
!DatabaseTable categoriesFor: #addForeignKeyFrom:toTable:!fields!private! !
!DatabaseTable categoriesFor: #addForeignKeyFromAll:toAll:!fields!private! !
!DatabaseTable categoriesFor: #addForeignKeyFromAll:toAll:suffixExpression:!fields!private! !
!DatabaseTable categoriesFor: #addIndex:!indexes!private! !
!DatabaseTable categoriesFor: #addIndexForField:!indexes!private! !
!DatabaseTable categoriesFor: #addIndexForField:and:!indexes!private! !
!DatabaseTable categoriesFor: #addIndexForField:and:and:!indexes!private! !
!DatabaseTable categoriesFor: #addIndexForFields:!indexes!private! !
!DatabaseTable categoriesFor: #addUniqueConstraintFor:!fields!public! !
!DatabaseTable categoriesFor: #allSequences!accessing!private! !
!DatabaseTable categoriesFor: #createFieldNamed:type:!fields!private! !
!DatabaseTable categoriesFor: #creator!accessing!private! !
!DatabaseTable categoriesFor: #creator:!accessing!private! !
!DatabaseTable categoriesFor: #dropFields:fromAccessor:!obsolete!private! !
!DatabaseTable categoriesFor: #dropForeignKeyConstraintsFromAccessor:!obsolete!private! !
!DatabaseTable categoriesFor: #dropFromAccessor:!obsolete!private! !
!DatabaseTable categoriesFor: #dropPrimaryKeyConstraintsFromAccessor:!obsolete!private! !
!DatabaseTable categoriesFor: #fieldNamed:!fields!private! !
!DatabaseTable categoriesFor: #fieldNamed:ifAbsent:!fields!private! !
!DatabaseTable categoriesFor: #fieldNamed:ifAbsent:caseSensitive:!fields!private! !
!DatabaseTable categoriesFor: #fields!accessing!private! !
!DatabaseTable categoriesFor: #foreignKeyConstraints!accessing!private! !
!DatabaseTable categoriesFor: #foreignKeyConstraintsForCreation!accessing!private! !
!DatabaseTable categoriesFor: #foreignKeyConstraintsForField:!constraints!private! !
!DatabaseTable categoriesFor: #hasCompositePrimaryKey!private!testing! !
!DatabaseTable categoriesFor: #hasConstraints!public!testing! !
!DatabaseTable categoriesFor: #hasConstraintsTo:!private!testing! !
!DatabaseTable categoriesFor: #hasConstraintsToAnyOf:!private!testing! !
!DatabaseTable categoriesFor: #hasFieldNamed:!private!testing! !
!DatabaseTable categoriesFor: #hasForeignKeyConstraints!private!testing! !
!DatabaseTable categoriesFor: #hasForeignKeyConstraintsForField:!constraints!private! !
!DatabaseTable categoriesFor: #hasForeignKeyConstraintsForTargetField:!constraints!private! !
!DatabaseTable categoriesFor: #hasIndexForField:!indexes!private! !
!DatabaseTable categoriesFor: #hasIndexNamed:!private!testing! !
!DatabaseTable categoriesFor: #hasPrimaryKeyConstraints!private!testing! !
!DatabaseTable categoriesFor: #hasUniqueConstraints!public!testing! !
!DatabaseTable categoriesFor: #imaginaryReplacementName!printing!private! !
!DatabaseTable categoriesFor: #indexes!indexes!private! !
!DatabaseTable categoriesFor: #initialize!initialize!public! !
!DatabaseTable categoriesFor: #isAliased!accessing!private! !
!DatabaseTable categoriesFor: #isEmpty!private!testing! !
!DatabaseTable categoriesFor: #isImaginary!accessing!private! !
!DatabaseTable categoriesFor: #isImaginary:!accessing!private! !
!DatabaseTable categoriesFor: #isTheSameAs:!comparing!public! !
!DatabaseTable categoriesFor: #lockKeyFields!accessing!private! !
!DatabaseTable categoriesFor: #name!accessing!private! !
!DatabaseTable categoriesFor: #name:!accessing!private! !
!DatabaseTable categoriesFor: #newFieldNamed:!fields!private! !
!DatabaseTable categoriesFor: #nonPrimaryKeyFields!accessing!private! !
!DatabaseTable categoriesFor: #parent!accessing!private! !
!DatabaseTable categoriesFor: #parent:!accessing!private! !
!DatabaseTable categoriesFor: #platform!accessing!private! !
!DatabaseTable categoriesFor: #postInitializeIn:!initialize!private! !
!DatabaseTable categoriesFor: #primaryKeyConstraintName!obsolete!private! !
!DatabaseTable categoriesFor: #primaryKeyConstraints!accessing!private! !
!DatabaseTable categoriesFor: #primaryKeyFields!accessing!private! !
!DatabaseTable categoriesFor: #primaryKeyUniqueConstraintName!obsolete!private! !
!DatabaseTable categoriesFor: #printAddPrimaryKeyConstraintsOn:for:!obsolete!private! !
!DatabaseTable categoriesFor: #printDelimiterOn:!obsolete!private! !
!DatabaseTable categoriesFor: #printOn:!printing!private! !
!DatabaseTable categoriesFor: #printPrimaryKeyConstraintsOn:for:!obsolete!private! !
!DatabaseTable categoriesFor: #printSQLOn:withParameters:!printing!private! !
!DatabaseTable categoriesFor: #qualifiedName!accessing!private! !
!DatabaseTable categoriesFor: #removeFieldNamed:!fields!private! !
!DatabaseTable categoriesFor: #removeFieldNamed:ifAbsent:!fields!private! !
!DatabaseTable categoriesFor: #replacementSubSelect:!As yet unclassified!private! !
!DatabaseTable categoriesFor: #schema!accessing!private! !
!DatabaseTable categoriesFor: #schema:!accessing!private! !
!DatabaseTable categoriesFor: #sequences!accessing!private! !
!DatabaseTable categoriesFor: #shouldBeWritten!accessing!private! !
!DatabaseTable categoriesFor: #sqlString!printing!private! !
!DatabaseTable categoriesFor: #sqlTableName!printing!private! !
!DatabaseTable categoriesFor: #uniqueConstraints!accessing!private! !
!DatabaseTable categoriesFor: #validateIn:!private!validation! !

!DatabaseTable class methodsFor!

named: aString

	^self new name: aString.!

new

	^super new initialize.! !
!DatabaseTable class categoriesFor: #named:!instance creation!private! !
!DatabaseTable class categoriesFor: #new!instance creation!private! !

DatabaseTableConstraint guid: (GUID fromString: '{C6302EF3-9A51-4261-93EE-977EB863CB09}')!
DatabaseTableConstraint comment: 'Abstract superclass reifying a constraint on a value in a column or between values in columns.
'!
!DatabaseTableConstraint categoriesForClass!Unclassified! !
!DatabaseTableConstraint methodsFor!

creationString

	^(self creationStringFor: self table platform) first.!

creationStringFor: aPlatform

	^self subclassResponsibility!

dropString

	^self dropStringFor:  self table platform!

dropStringFor: aPlatform

	^aPlatform createConstraintDropStatement: self!

fields
	^fields!

fields: anObject
	fields := anObject!

forTable: aTable

	table := aTable!

generateName

	^self subclassResponsibility!

initialize

	shouldCreateInDatabase := true.!

isForeignKeyConstraint

	^false!

isPrimaryKeyConstraint

	^false!

isUniqueConstraint

	^false!

name
	name isNil ifTrue: [name := self generateName].
	^name.!

name: aString
	name := aString.!

printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self name;
		nextPut: $).!

shouldCreateInDatabase
	^shouldCreateInDatabase ifNil: [true]!

shouldCreateInDatabase: aBoolean
	"If false, then this is a constraint that we use for in-memory ordering, but it should not be created in the database, possibly for performance, or because it might cause a cycle"
	shouldCreateInDatabase := aBoolean!

table
	^table!

table: anObject
	table := anObject! !
!DatabaseTableConstraint categoriesFor: #creationString!printing!private! !
!DatabaseTableConstraint categoriesFor: #creationStringFor:!printing!private! !
!DatabaseTableConstraint categoriesFor: #dropString!printing!private! !
!DatabaseTableConstraint categoriesFor: #dropStringFor:!printing!private! !
!DatabaseTableConstraint categoriesFor: #fields!accessing!private! !
!DatabaseTableConstraint categoriesFor: #fields:!accessing!private! !
!DatabaseTableConstraint categoriesFor: #forTable:!initializing!private! !
!DatabaseTableConstraint categoriesFor: #generateName!accessing!private! !
!DatabaseTableConstraint categoriesFor: #initialize!initializing!private! !
!DatabaseTableConstraint categoriesFor: #isForeignKeyConstraint!public!testing! !
!DatabaseTableConstraint categoriesFor: #isPrimaryKeyConstraint!public!testing! !
!DatabaseTableConstraint categoriesFor: #isUniqueConstraint!public!testing! !
!DatabaseTableConstraint categoriesFor: #name!accessing!private! !
!DatabaseTableConstraint categoriesFor: #name:!accessing!private! !
!DatabaseTableConstraint categoriesFor: #printOn:!printing!private! !
!DatabaseTableConstraint categoriesFor: #shouldCreateInDatabase!accessing!private! !
!DatabaseTableConstraint categoriesFor: #shouldCreateInDatabase:!accessing!private! !
!DatabaseTableConstraint categoriesFor: #table!accessing!private! !
!DatabaseTableConstraint categoriesFor: #table:!accessing!private! !

!DatabaseTableConstraint class methodsFor!

forTable: aDatabaseTable
	^self new
		forTable: aDatabaseTable;
		yourself!

new
	^super new initialize.! !
!DatabaseTableConstraint class categoriesFor: #forTable:!instance creation!private! !
!DatabaseTableConstraint class categoriesFor: #new!instance creation!private! !

GlorpDatabasePackage guid: (GUID fromString: '{C7BC9B39-2CDE-402C-B677-B899D431A997}')!
GlorpDatabasePackage comment: 'LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you''ll need to release the
source of any modifications.

If you require any further clarification, we''d be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'!
!GlorpDatabasePackage categoriesForClass!Unclassified! !
!GlorpDatabasePackage class methodsFor!

LICENSE
"LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you'll need to release the
source of any modifications.

If you require any further clarification, we'd be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"! !
!GlorpDatabasePackage class categoriesFor: #LICENSE!instance creation!public! !

GlorpDatabaseValueGenerator guid: (GUID fromString: '{AD18B1B4-381A-4D43-88B2-A2B80E899DE0}')!
GlorpDatabaseValueGenerator comment: 'This is an abstract superclass for thing that generate values when objects are written to the database. The most common usage is the various types of sequences, identity columns and other primary key generating mechanism. But it can also be used for optimistic locking via version numbers or timestamps. It doesn''t actually have any code at the moment, so it might just be deletable, or maybe there''s some refactoring to be done.'!
!GlorpDatabaseValueGenerator categoriesForClass!Unclassified! !
GlorpPreparedStatement guid: (GUID fromString: '{892094C5-5C12-4BA3-B1EC-3FCD489E41A4}')!
GlorpPreparedStatement comment: 'This represents a prepared statement in the database. 

Instance Variables:
	busy	<Boolean>	Are we currently in use - that is to say, are we executing and/or holding an active cursor.
	signature	<String>	A string that we can use to match up to commands and see if we are appropriate for the command to use. The signature of a command is its SQL String plus a string representing the types of its parameters.
	statement	<Object>	The actual statement object. Because we don''t really know what type this is, and it varies by dialect, we allow the accessor to extract it from us and do the real work.

'!
!GlorpPreparedStatement categoriesForClass!Unclassified! !
!GlorpPreparedStatement methodsFor!

busy
	^busy.!

busy: aBoolean
	busy := aBoolean.!

glorpNoticeOfExpiryIn: aSession
	self release.!

initialize
	busy := false.!

release
	super release.
	statement isNil ifFalse: [
		| stmt |
		stmt := statement.
		statement := nil.
		stmt disconnect].!

signature
	^signature!

signature: aString
	signature := aString!

statement
	^statement!

statement: aStatementHandle
	statement := aStatementHandle! !
!GlorpPreparedStatement categoriesFor: #busy!accessing!private! !
!GlorpPreparedStatement categoriesFor: #busy:!accessing!private! !
!GlorpPreparedStatement categoriesFor: #glorpNoticeOfExpiryIn:!As yet unclassified!private! !
!GlorpPreparedStatement categoriesFor: #initialize!initialize-release!private! !
!GlorpPreparedStatement categoriesFor: #release!As yet unclassified!private! !
!GlorpPreparedStatement categoriesFor: #signature!accessing!private! !
!GlorpPreparedStatement categoriesFor: #signature:!accessing!private! !
!GlorpPreparedStatement categoriesFor: #statement!accessing!private! !
!GlorpPreparedStatement categoriesFor: #statement:!accessing!private! !

!GlorpPreparedStatement class methodsFor!

new
	"Answer a newly created and initialized instance."

	^super new initialize! !
!GlorpPreparedStatement class categoriesFor: #new!instance creation!private! !

Login guid: (GUID fromString: '{6E5D7A79-77F9-4ECD-9C91-3AE8BE51DD42}')!
Login comment: 'Enough information to log in to a particular database.

Instance Variables:
	connectString	<String>	The string that will be passed to the driver level to tell us what to connect to. This might be e.g. an ODBC data source name, or it might be a host name plus socket/database name for Postgresql.
	database	<DatabasePlatform>	The platform describing what type of database this is. Used here mostly to indicate how we need to talk to the underlying connection layer, which may be different by database.
	name	<String>	A name we can use to describe this. No significance to the database.
	password	<String>	The connect password.
	schema	<String>	What should our default schema be. Used mostly internally, to set the schema of the tables we are using.
	username	<String>	The connect username.

'!
!Login categoriesForClass!Unclassified! !
!Login methodsFor!

= aLogin 
	^self class == aLogin class and: 
			[self name = aLogin name and: 
					[self database class = aLogin database class and: 
							[self username = aLogin username and: 
									[self connectString = aLogin connectString]]]]!

connectString
	^connectString!

connectString: aString 
	connectString := aString!

database
	^database!

database: aDatabasePlatform 
	database := aDatabasePlatform!

hash
	^self name hash + self database class hash + self username hash + self connectString hash!

initialize
	secure := true.!

name
	name isNil ifTrue: [^self connectString] ifFalse: [^name].!

name: aString
	name := aString.!

password
	"Return the password for this login. If we are in a secure mode, then erase the password as soon as it is accessed."
	| returnValue |
	returnValue := password.
	secure ifTrue: [password := nil].
	^returnValue.!

password: aString 
	password := aString!

platform
	^self database.!

printOn: aStream

	aStream nextPutAll: 'a Login('.
	database printOn: aStream.
	aStream nextPutAll: ', '.
	username printOn: aStream.
	aStream nextPutAll: ', '.
	connectString printOn: aStream.
	aStream nextPutAll: ', '.
	schema printOn: aStream.
	aStream nextPutAll: ')'.!

schema
	^schema!

schema: aString
	schema := aString.!

secure
	^secure!

secure: aBoolean
	secure := aBoolean!

sessionForDescriptorSystem: aDescriptorSystemClass
	"Create a session for this login, using aDescriptorSystemClass for its metadata"
	^aDescriptorSystemClass sessionForLogin: self.!

username
	^username!

username: aString 
	username := aString! !
!Login categoriesFor: #=!accessing!public! !
!Login categoriesFor: #connectString!accessing!private! !
!Login categoriesFor: #connectString:!accessing!private! !
!Login categoriesFor: #database!accessing!private! !
!Login categoriesFor: #database:!accessing!public! !
!Login categoriesFor: #hash!accessing!public! !
!Login categoriesFor: #initialize!initialize-release!public! !
!Login categoriesFor: #name!accessing!private! !
!Login categoriesFor: #name:!accessing!private! !
!Login categoriesFor: #password!accessing!public! !
!Login categoriesFor: #password:!accessing!private! !
!Login categoriesFor: #platform!accessing!private! !
!Login categoriesFor: #printOn:!printing!public! !
!Login categoriesFor: #schema!accessing!private! !
!Login categoriesFor: #schema:!accessing!private! !
!Login categoriesFor: #secure!accessing!public! !
!Login categoriesFor: #secure:!accessing!public! !
!Login categoriesFor: #sessionForDescriptorSystem:!private!session creation! !
!Login categoriesFor: #username!accessing!private! !
!Login categoriesFor: #username:!accessing!private! !

!Login class methodsFor!

new
	^super new initialize.! !
!Login class categoriesFor: #new!instance creation!public! !

DolphinDatabaseAccessor guid: (GUID fromString: '{545D15D7-9586-442F-A91B-DBA365B10B4A}')!
DolphinDatabaseAccessor comment: 'The database accessor subclass for Dolphin Smalltalk.'!
!DolphinDatabaseAccessor categoriesForClass!Unclassified! !
!DolphinDatabaseAccessor methodsFor!

basicExecuteSQLString: aString
	| resultSet array |
	self log: aString.
	self logOnly ifTrue: [^self].
	resultSet := connection query: aString.	"Write statements don't like asArray if there's not result. DDL statements don't even like the isEmpty test"
	array := [resultSet isEmpty ifTrue: [#()] ifFalse: [resultSet asArray]] on: Dialect error do: [:ex | ex return: #()].
	resultSet
		close;
		free.
	^self cursorFor: (ReadStream on: array)!

basicExecuteSQLString: aString returnResult: aBoolean binding: aBindingArray doing: aBlock
	| statement array results |
	statement := (aBindingArray isNil or: [aBindingArray isEmpty])
		ifTrue: [connection exec: aString]
		ifFalse: [connection prepare: aString].
	[(aBindingArray notNil and: [aBindingArray notEmpty])
		ifTrue:
			[statement
				values: aBindingArray;
				exec].
	currentStatement := statement.
	aBlock ensure: [currentStatement := nil].
	aBoolean
		ifTrue:
			[results := statement results.
			array := [results asArray]
				ensure:
					[results
						close;
						free].
			^self cursorFor: (ReadStream on: array)]]
		ensure:
			[statement
				close;
				free]!

beginTransaction

	connection beginRWTxn.!

commitTransaction
	self log: 'Commit Transaction'.
	self logOnly ifTrue: [^self].
	connection commitTxn!

externalDatabaseErrorSignal
	^DBError!

isInTransaction

	^connection isInTransaction!

isLoggedIn

	^connection notNil.!

loginIfError: aBlock
	self
		execute:
			[connection := (DBConnection new)
				dsn: currentLogin connectString;
				uid: currentLogin username;
				pwd: currentLogin password;
				connect]
		ifError: aBlock!

logout

	connection notNil
		ifTrue:
			[[connection close.] ensure: [connection := nil]].!

releaseCursor: aCursor in: aPreparedStatement
	"Release the cursor. We're trapping exceptions, so return true or false to indicate success/failure"

	aCursor isNil ifTrue: [^self].
	aPreparedStatement isNil ifTrue: [^aCursor class ~~ ReadStream ifTrue: [self halt] ifFalse: [self]].
	^[aCursor close.
	true] on: Error do: [:ex | ex return: false]!

rollbackTransaction
	self log: 'Rollback Transaction'.
	self logOnly ifTrue: [^self].
	connection rollbackTxn!

rowCount
	^currentStatement numRows!

showDialog: aString
	MessageBox warning: aString! !
!DolphinDatabaseAccessor categoriesFor: #basicExecuteSQLString:!executing!public! !
!DolphinDatabaseAccessor categoriesFor: #basicExecuteSQLString:returnResult:binding:doing:!As yet unclassified!private! !
!DolphinDatabaseAccessor categoriesFor: #beginTransaction!private!transactions! !
!DolphinDatabaseAccessor categoriesFor: #commitTransaction!public!transactions! !
!DolphinDatabaseAccessor categoriesFor: #externalDatabaseErrorSignal!executing!private! !
!DolphinDatabaseAccessor categoriesFor: #isInTransaction!private!transactions! !
!DolphinDatabaseAccessor categoriesFor: #isLoggedIn!accessing!private! !
!DolphinDatabaseAccessor categoriesFor: #loginIfError:!login!public! !
!DolphinDatabaseAccessor categoriesFor: #logout!login!private! !
!DolphinDatabaseAccessor categoriesFor: #releaseCursor:in:!As yet unclassified!private! !
!DolphinDatabaseAccessor categoriesFor: #rollbackTransaction!public!transactions! !
!DolphinDatabaseAccessor categoriesFor: #rowCount!As yet unclassified!private! !
!DolphinDatabaseAccessor categoriesFor: #showDialog:!login!private! !

RowBasedCommand guid: (GUID fromString: '{E0F86C60-D760-4EBC-831F-C392D2FCC0DD}')!
RowBasedCommand comment: 'This is a command based on one or more in-memory rows. The purposes is to write, or delete that row or rows. This knows enough to be able to use features like array binding to do the row operations more efficiently.

Subclasses must implement the following messages:
	bound values
		arrayBoundFields
		singleRowBoundFields

Instance Variables:
	allRows	<(Collection of: (DatabaseRow))> A collection of rows. May be nil. If we have multiples, then we will be array bound or otherwise bulk-written under certain circumstances.
	row	<DatabaseRow>	The primary row, either the single row we''re writing, or the one we''re writing right now out of the allRows collection.
	rowCount	<Integer>	The row count from the last statement we executed.
	succeeded	<Boolean>	Has the statement overall succeded or failed.

'!
!RowBasedCommand categoriesForClass!Unclassified! !
!RowBasedCommand methodsFor!

allRows
	^allRows.!

allRows: aCollectionOfDatabaseRows
	allRows := aCollectionOfDatabaseRows.!

arrayBindings
	"Return bindings in a form suitable for Oracle array binding, i.e. a list of arguments, but each argument is itself a list of multiple values"
	| allBindings fields |
	fields := self arrayBoundFields.
	allBindings := (1 to: fields size)
		collect: [:each | Array new: allRows size].
	1
		to: allRows size
		do:
			[:rowNumber | 
			| inputRow |
			inputRow := allRows at: rowNumber.
			1
				to: fields size
				do:
					[:columnNumber | 
					(allBindings at: columnNumber)
						at: rowNumber
						put: (inputRow at: (fields at: columnNumber) ifAbsent: [nil])]].
	^allBindings asArray.!

arrayBoundFields
	"Return the list of fields to be bound in the SQL string we will use for array binding. For array binding, we need to include all possible fields, rather than those that just have values in a particular row"
	self subclassResponsibility.!

batchStatementBindings
	"Return our bindings, which we assume are for multiple rows, as one big array, one statement after the other"

	| bound fields |
	fields := self arrayBoundFields.
	bound := OrderedCollection new: allRows size * fields size.
	allRows do: [:eachRow | 
		fields do: [:each | 
			| fieldValue |
			fieldValue := eachRow at: each ifAbsent: [nil].
			(self canBind: fieldValue to: each type) ifTrue: [bound add: fieldValue]]].
	^bound asArray.!

bindings
	^(self shouldUseGroupWriting)
		ifTrue: [self platform bindingsForGroupWritingFor: self]
		ifFalse: [self singleRowBindings].!

bindingTypes
	| all basic size |
	^(self shouldUseGroupWriting)
		ifTrue: [
			basic := self row nonGeneratedFields collect: [:each | each type].
			size := basic size.
			all := Array new: (allRows size * size).
			0 to: allRows size - 1 do: [:i |
				| index |
				index := i * size.
				all replaceElementsFrom: index + 1 to: index + size withArray: basic startingAt: 1].
			all]
		ifFalse: [self singleRowBoundFields collect: [:each | each type]].!

executeBoundIn: anAccessor
	| result |
	(self hasMultipleRows and: [self shouldUseGroupWriting not])
		ifTrue:
			[self allRows
				do:
					[:each | 
					self row: each.
					result := anAccessor executeCommandBound: self.
					result release.
					failed := failed or: [self needsRowCount and: [rowCount ~= 1]].
					each postWriteAssignSequencesUsing: anAccessor]]
		ifFalse:
			[result := anAccessor executeCommandBound: self.
			result release.
			failed := self needsRowCount and: [rowCount ~= self expectedNumberOfModifiedRows].
			row postWriteAssignSequencesUsing: anAccessor].
	"Results aren't interesting"
	^ anAccessor cursorFor: #() readStream!

executeUnboundIn: anAccessor
	anAccessor returnRowCount: self needsRowCount.
	(self hasMultipleRows and: [self shouldUseGroupWriting not])
		ifTrue:
			[self allRows
				do:
					[:each | 
					self row: each.
					anAccessor
						executeSQLStringNoResult: self sqlString
						doing:
							[failed := failed or: [self needsRowCount and: [anAccessor rowCount ~= 1]]]
						forCommand: self.
					each postWriteAssignSequencesUsing: anAccessor]]
		ifFalse:
			[self tooBigForSingleWrite
				ifTrue:
					[self subCommands do: [:each | each executeUnboundIn: anAccessor]]
				ifFalse:
					[anAccessor
						executeSQLStringNoResult: self sqlString
						doing:
							[failed := self needsRowCount
								and: [anAccessor rowCount ~= self expectedNumberOfModifiedRows]]
						forCommand: self].
			row postWriteAssignSequencesUsing: anAccessor].
	^#() readStream.!

expectedNumberOfModifiedRows
	^self hasMultipleRows ifTrue: [allRows size] ifFalse: [1].!

hasMultipleRows
	^allRows notNil and: [allRows size > 1].!

initialize
	super initialize.
	failed := false.!

maximumSingleWriteSize
	"How many values will write at once. This is only really applicable when not doing binding and concatenating many statements together for one trip to the DB"
	^1000.!

parameterTypeSignature
	"Return a string indicating the parameter types. Complicated by the possibility of multiple rows, and of null values"
	| result |
	result := WriteStream on: String new.
	self hasMultipleRows
		ifTrue:
			[row
				keysAndValuesDo:
					[:eachKey :eachValInFirstRow| 
					| nextValue |
					eachValInFirstRow notNil
						ifTrue: [result nextPutAll: eachValInFirstRow class name]
						ifFalse:
							[allRows
								do:
									[:eachRow | 
									(nextValue := eachRow at: eachKey ifAbsent: [nil]) notNil
										ifTrue: [result nextPutAll: nextValue class name]]]]]
		ifFalse:
			[row
				keysAndValuesDo:
					[:eachKey :eachValue | result nextPutAll: eachValue class name]].
	^result contents.!

row
	^row.!

row: anObject
	row := anObject.
	sqlString := nil.!

rowAtATimeBindings
	"Return multiple sets of bindings in a form suitable for iterating over"
	| allBindings fields rowTemplate |
	fields := self arrayBoundFields.
	rowTemplate := Array new: fields size.
	allBindings := Array new: allRows size.
	1 to: allBindings size do: [:i | | currentRow input |
		currentRow := rowTemplate copy.
		input := allRows at: i.
		allBindings at: i put: currentRow.
		1 to: fields size do: [:j |
			currentRow at: j put: (input at: (fields at: j) ifAbsent: [nil])]].
	^allBindings asArray.!

rowCount
	^rowCount.!

rowCount: anInteger
	rowCount := anInteger.!

singleRowBindings
	| bound |
	bound := OrderedCollection new.
	self singleRowBoundFields
		do:
			[:each | 
			| fieldValue |
			fieldValue := row at: each ifAbsent: [nil].
			(self canBind: fieldValue to: each type) ifTrue: [bound add: fieldValue]].
	^bound asArray.!

singleRowBoundFields
	"Return the list of fields to be bound in the SQL string we will use for a single operation. We only need to include those fields that actually have values in the particular row we're operating on"
	self subclassResponsibility.!

subCommands
	| startPosition commands endPosition subRows newCommand |
	startPosition := 1.
	commands := OrderedCollection new.
	[startPosition > allRows size] whileFalse: [
		endPosition := (startPosition + self maximumSingleWriteSize - 1) min: allRows size.
		subRows := allRows copyFrom: startPosition to: endPosition.
		newCommand := self class forRows: subRows useBinding: self useBinding session: self session.
		commands add: newCommand.
		startPosition := endPosition + 1].
	^commands.!

succeeded
	^failed not.!

supportsGroupWriting
	"Return true if array binding or a similar mechanism can be used to execute us with multiple commands at once. I think in general this can only ever make sense for inserts (and possibly selects under some circumstances). Updates may want to update only part of the fields."
	^false.!

tables
	^Array with: self row table.!

tooBigForSingleWrite
	^self hasMultipleRows and: [allRows size > self maximumSingleWriteSize].! !
!RowBasedCommand categoriesFor: #allRows!accessing!private! !
!RowBasedCommand categoriesFor: #allRows:!accessing!private! !
!RowBasedCommand categoriesFor: #arrayBindings!bound values!private! !
!RowBasedCommand categoriesFor: #arrayBoundFields!bound values!private! !
!RowBasedCommand categoriesFor: #batchStatementBindings!bound values!private! !
!RowBasedCommand categoriesFor: #bindings!bound values!private! !
!RowBasedCommand categoriesFor: #bindingTypes!bound values!private! !
!RowBasedCommand categoriesFor: #executeBoundIn:!executing!private! !
!RowBasedCommand categoriesFor: #executeUnboundIn:!bound values!private! !
!RowBasedCommand categoriesFor: #expectedNumberOfModifiedRows!executing!private! !
!RowBasedCommand categoriesFor: #hasMultipleRows!private!testing! !
!RowBasedCommand categoriesFor: #initialize!initializing!private! !
!RowBasedCommand categoriesFor: #maximumSingleWriteSize!private!testing! !
!RowBasedCommand categoriesFor: #parameterTypeSignature!accessing!private! !
!RowBasedCommand categoriesFor: #row!accessing!private! !
!RowBasedCommand categoriesFor: #row:!accessing!private! !
!RowBasedCommand categoriesFor: #rowAtATimeBindings!bound values!private! !
!RowBasedCommand categoriesFor: #rowCount!accessing!private! !
!RowBasedCommand categoriesFor: #rowCount:!accessing!private! !
!RowBasedCommand categoriesFor: #singleRowBindings!bound values!private! !
!RowBasedCommand categoriesFor: #singleRowBoundFields!bound values!private! !
!RowBasedCommand categoriesFor: #subCommands!executing!public! !
!RowBasedCommand categoriesFor: #succeeded!private!testing! !
!RowBasedCommand categoriesFor: #supportsGroupWriting!private!testing! !
!RowBasedCommand categoriesFor: #tables!private!reporting! !
!RowBasedCommand categoriesFor: #tooBigForSingleWrite!private!testing! !

!RowBasedCommand class methodsFor!

forRow: aDatabaseRow useBinding: aBoolean session: aSession
	^(self new)
		row: aDatabaseRow;
		useBinding: aBoolean;
		session: aSession;
		yourself.!

forRows: aCollectionOfDatabaseRows useBinding: aBoolean session: aSession
	^(self new)
		allRows: aCollectionOfDatabaseRows;
		row: aCollectionOfDatabaseRows first;
		useBinding: aBoolean;
		session: aSession;
		yourself.! !
!RowBasedCommand class categoriesFor: #forRow:useBinding:session:!instance creation!public! !
!RowBasedCommand class categoriesFor: #forRows:useBinding:session:!instance creation!public! !

SelectCommand guid: (GUID fromString: '{C3CB2D99-8D12-4A1B-94AB-8266B414EA8D}')!
SelectCommand comment: 'This represents a SELECT query to the database. This can either be based on a GLORP query, or on a hard-coded SQL string.

Subclasses must implement the following messages:
	accessing
		printSQL

Instance Variables:
	blockFactor	<SmallInteger>	The blocking factor to use for this query. This is most likely to make a difference for these kind of operations, and so is directly settable here.
	parameters	<Dictionary from: DatabaseField to: Object>	The parameters to the query.

'!
!SelectCommand categoriesForClass!Unclassified! !
!SelectCommand methodsFor!

blockFactor: anInteger
	blockFactor := anInteger.!

canBind: aValue to: aType
	aValue isNil ifTrue: [^false].
	^super canBind: aValue to: aType.!

isReadCommand
	^true.!

parameters
	^parameters!

parameters: aDictionary
	parameters := aDictionary!

parameterTypeSignature
	| result |
	result := WriteStream on: String new.
	self bindings do: [:each | result nextPutAll: each class name].
	^result contents.!

printSQL
	self subclassResponsibility.!

succeeded

	^true.! !
!SelectCommand categoriesFor: #blockFactor:!accessing!private! !
!SelectCommand categoriesFor: #canBind:to:!private!testing! !
!SelectCommand categoriesFor: #isReadCommand!private!testing! !
!SelectCommand categoriesFor: #parameters!accessing!private! !
!SelectCommand categoriesFor: #parameters:!accessing!private! !
!SelectCommand categoriesFor: #parameterTypeSignature!accessing!private! !
!SelectCommand categoriesFor: #printSQL!accessing!private! !
!SelectCommand categoriesFor: #succeeded!private!testing! !

!SelectCommand class methodsFor!

forQuery: aQuery parameters: aDictionary
	^(self new)
		query: aQuery;
		parameters: aDictionary;
		yourself.!

forQuery: aQuery parameters: aDictionary useBinding: aBoolean session: aSession
	^(self new)
		query: aQuery;
		parameters: aDictionary;
		useBinding: aBoolean;
		session: aSession;
		yourself.! !
!SelectCommand class categoriesFor: #forQuery:parameters:!instance creation!private! !
!SelectCommand class categoriesFor: #forQuery:parameters:useBinding:session:!instance creation!public! !

SQLStringCommand guid: (GUID fromString: '{4D34B6EA-7475-4C96-BEE6-EFBED6808B33}')!
SQLStringCommand comment: 'This represents a command that executes arbitrary SQL and is not a select. For selects use SQLStringSelectCommand.'!
!SQLStringCommand categoriesForClass!Unclassified! !
!SQLStringCommand methodsFor!

arrayBoundFields
	"Don't have enough information to do this, I don't think"
	^#()!

bindings
	^parameters.!

bindingTypes
	^#()!

initialize
	super initialize.
	maximumBlobSizeToRetrieveDirectly := super maximumLobSizeToRetrieveDirectly.!

maximumLobSizeToRetrieveDirectly
	^maximumBlobSizeToRetrieveDirectly!

maximumLobSizeToRetrieveDirectly: anInteger
	maximumBlobSizeToRetrieveDirectly := anInteger!

parameters
	^parameters.!

parameters: anArray
	parameters := anArray!

parameterTypeSignature
	^''!

printSQL
	" *** This method was defined by Glorp.DatabaseCommand as a subclass responsibility.
	Replace its body with a proper implementation. *** "

	self error: 'Subclass responsibility stub not reimplemented'!

setSQLString: aString
	sqlString := aString.!

succeeded
	" *** This method was defined by Glorp.DatabaseCommand as a subclass responsibility.
	Replace its body with a proper implementation. *** "

	self error: 'Subclass responsibility stub not reimplemented'! !
!SQLStringCommand categoriesFor: #arrayBoundFields!accessing!private! !
!SQLStringCommand categoriesFor: #bindings!accessing!private! !
!SQLStringCommand categoriesFor: #bindingTypes!accessing!private! !
!SQLStringCommand categoriesFor: #initialize!initialize-release!private! !
!SQLStringCommand categoriesFor: #maximumLobSizeToRetrieveDirectly!accessing!private! !
!SQLStringCommand categoriesFor: #maximumLobSizeToRetrieveDirectly:!accessing!private! !
!SQLStringCommand categoriesFor: #parameters!accessing!private! !
!SQLStringCommand categoriesFor: #parameters:!accessing!private! !
!SQLStringCommand categoriesFor: #parameterTypeSignature!accessing!private! !
!SQLStringCommand categoriesFor: #printSQL!executing!private! !
!SQLStringCommand categoriesFor: #setSQLString:!accessing!private! !
!SQLStringCommand categoriesFor: #succeeded!private!testing! !

!SQLStringCommand class methodsFor!

for: aString
	^self new setSQLString: aString.! !
!SQLStringCommand class categoriesFor: #for:!instance creation!private! !

DeleteCommand guid: (GUID fromString: '{1D737490-3A28-4E71-AC0D-B0224971EE04}')!
DeleteCommand comment: 'A command to delete our set of rows.'!
!DeleteCommand categoriesForClass!Unclassified! !
!DeleteCommand methodsFor!

arrayBoundFields
	"For a delete, it makes no difference, we only ever include the primary key fields"
	^self singleRowBoundFields.!

blockFactor
	^1.!

printSQL
	self nextPutAll: 'DELETE FROM '.
	row table printSQLOn: self withParameters: #().
	self nextPutAll: ' WHERE '.
	row printPrimaryKeyTemplateOn: self.!

singleRowBoundFields
	| fields |
	fields := row table primaryKeyFields.
	fields isEmpty ifTrue: [fields := row fields].
	^fields asArray! !
!DeleteCommand categoriesFor: #arrayBoundFields!accessing!private! !
!DeleteCommand categoriesFor: #blockFactor!accessing!private! !
!DeleteCommand categoriesFor: #printSQL!accessing!private! !
!DeleteCommand categoriesFor: #singleRowBoundFields!accessing!private! !

InsertCommand guid: (GUID fromString: '{5A243EEA-6A95-453E-88FD-7B9ECB3D54FF}')!
InsertCommand comment: 'A command to insert our set of rows.'!
!InsertCommand categoriesForClass!Unclassified! !
!InsertCommand methodsFor!

arrayBoundFields
	"For array binding, include all the fields, except if we're inserting into a database with identity columns, where we must omit those columns"
	^self platform usesIdentityColumns
		ifTrue: [row table fields reject: [:each | each type isIdentityColumn]]
		ifFalse: [row table fields].!

basicPrintSQLFor: aRow
	| fields |
	self nextPutAll: 'INSERT INTO '.
	aRow table printSQLOn: self withParameters: #().
	fields := self shouldUseGroupWriting
		ifTrue: [self arrayBoundFields]
		ifFalse: [aRow nonGeneratedFieldsWithValues].
	self nextPutAll: ' ('.
	GlorpHelper
		do: [:each | self nextPutAll: (self platform nameForColumn: each name)]
		for: fields
		separatedBy: [self nextPutAll: ','].
	self nextPutAll: ') '.
	self nextPutAll: ' VALUES ('.
	GlorpHelper
		do:
			[:each | 
			(self canBind: (aRow at: each ifAbsent: [nil]) to: each type)
				ifTrue: [self nextPut: $?]
				ifFalse: [aRow printValueOfField: each on: self]]
		for: fields
		separatedBy: [self nextPutAll: ','].
	self nextPutAll: ')'.!

printSQL
	| iterations |
	iterations := self shouldUseGroupWriting
		ifTrue:
			[self platform usesArrayBindingRatherThanGrouping
				ifTrue: [1]
				ifFalse: [allRows size]]
		ifFalse: [1].
	GlorpHelper
		do:
			[:i | self basicPrintSQLFor: (i = 1 ifTrue: [row] ifFalse: [allRows at: i])]
		for: (1 to: iterations)
		separatedBy: [self nextPutAll: '; '].!

singleRowBoundFields
	"Include only the fields that have values in our particular row"
	^row nonGeneratedFieldsWithValues!

supportsGroupWriting
	^true.! !
!InsertCommand categoriesFor: #arrayBoundFields!accessing!public! !
!InsertCommand categoriesFor: #basicPrintSQLFor:!accessing!public! !
!InsertCommand categoriesFor: #printSQL!accessing!private! !
!InsertCommand categoriesFor: #singleRowBoundFields!accessing!private! !
!InsertCommand categoriesFor: #supportsGroupWriting!private!testing! !

UpdateCommand guid: (GUID fromString: '{1DDD485D-4624-47AC-9189-CFA460924DAC}')!
UpdateCommand comment: 'A command to update our set of rows.'!
!UpdateCommand categoriesForClass!Unclassified! !
!UpdateCommand methodsFor!

arrayBindings
	"I don't think we can array bind updates. There may be values in the row that we haven't read, and if we attempt to write all the fields we'll be overwriting them with nulls. It's possible that there's a set of rows with all the same fields and we could update them all the same way, but that isn't done yet"
	self error: 'cannot array bind an update'.!

arrayBoundFields
	^row table nonPrimaryKeyFields asArray , row table primaryKeyFields asArray.!

bindingTypes
	^super bindingTypes, (row lockKeyFields collect: [:each | each type]).!

blockFactor
	^1.!

fieldsNeedingUpdate
	^self shouldUpdateOnlyChangedFields 
		ifTrue: [row fieldsNeedingUpdate]
		ifFalse: [row nonPrimaryKeyFields]!

needsRowCount
	^row lockKeyFields isEmpty not.!

printSQL
	| bodyFields |
	self nextPutAll: 'UPDATE '.
	row table printSQLOn: self withParameters: #().
	self nextPutAll: ' SET '.
	bodyFields := self fieldsNeedingUpdate.
	GlorpHelper 
		do: [:field | row printEqualityTemplateForField: field on: self]
		for: bodyFields
		separatedBy: [self nextPut: $,].
	self nextPutAll: ' WHERE '.
	row printPrimaryKeyTemplateOn: self.
	row lockKeyFields notEmpty
		ifTrue: 
			[self nextPutAll: ' AND '.
			GlorpHelper
				do: [:field | row printLockEqualityForField: field on: self]
				for: row lockKeyFields
				separatedBy: [self nextPutAll: ' AND ']]!

shouldUpdateOnlyChangedFields
	"Return true if we should write only changed fields out in an update statement, or if we should write all of them. The tradeoff is that writing only the changed ones will produce less SQL, but make more variations of the statement, and require us to keep more prepared statements around if we're using binding. By default, use this option only if we're not using binding by default. It would be nice if we could delegate this decision to somewhere the user has control over, e.g. the descriptor system, but we don't actually know about it"
	^self system shouldUpdateOnlyChangedFieldsFor: self.!

singleRowBindings
	| basic lockFields lockValues |
	basic := super singleRowBindings.
	lockFields := row lockKeyFields.
	lockFields isEmpty ifTrue: [^basic].
	lockValues := OrderedCollection new: 2.
	row lockKeyFields do: [:each | 	
			| fieldValue |
			fieldValue := row oldVersionAt: each ifAbsent: [nil].
			(self canBind: fieldValue to: each type) ifTrue: [lockValues add: fieldValue]].
	^basic, lockValues.!

singleRowBoundFields
	"Note: We will actually have to bind one more thing than this, the optimistic lock fields (if present). That's handled in the caller"
	| basicFields |
	basicFields := self fieldsNeedingUpdate.
	^basicFields , row table primaryKeyFields asArray! !
!UpdateCommand categoriesFor: #arrayBindings!accessing!private! !
!UpdateCommand categoriesFor: #arrayBoundFields!accessing!private! !
!UpdateCommand categoriesFor: #bindingTypes!bound values!private! !
!UpdateCommand categoriesFor: #blockFactor!accessing!private! !
!UpdateCommand categoriesFor: #fieldsNeedingUpdate!accessing!private! !
!UpdateCommand categoriesFor: #needsRowCount!accessing!private! !
!UpdateCommand categoriesFor: #printSQL!printing!private! !
!UpdateCommand categoriesFor: #shouldUpdateOnlyChangedFields!public!testing! !
!UpdateCommand categoriesFor: #singleRowBindings!bound values!private! !
!UpdateCommand categoriesFor: #singleRowBoundFields!bound values!private! !

QuerySelectCommand guid: (GUID fromString: '{8C01ABE3-C0F7-4EDE-AAC0-05E9EB280C09}')!
QuerySelectCommand comment: 'This is the normal class to use for select queries. It generates SQL for a select operation returning a set of rows.

Instance Variables:
	boundExpressions	<(Collection of: GlorpExpression)>	The expressions for which we need bind parameters. We expect that these are things that resolve to fields, or more generally, to something the database will accept as a valid thing to bind ofr.
	query	<Query>	The query in which we''re executing.

'!
!QuerySelectCommand categoriesForClass!Unclassified! !
!QuerySelectCommand methodsFor!

bindings
	self useBinding ifFalse: [^#()].
	boundExpressions isNil ifTrue: [self findBoundExpressions].
	^boundExpressions collect: [:each | each bindingIn: self].!

bindingTypes
	self useBinding ifFalse: [^#()].
	boundExpressions isNil ifTrue: [self findBoundExpressions].
	^boundExpressions collect: [:each | each bindingType].!

blockFactor
	blockFactor isNil ifTrue: [blockFactor := query expectedRows].
	^blockFactor.!

boundExpressions
	^boundExpressions!

boundExpressions: anObject
	boundExpressions := anObject!

findBoundExpressions
	self useBinding ifFalse: [^nil].
	boundExpressions := OrderedCollection new.
	query addBoundExpressionsIn: self to: boundExpressions.!

maximumLobSizeToRetrieveDirectly
	^query isNil
		ifTrue: [super maximumLobSizeToRetrieveDirectly]
		ifFalse: [query maximumLobSizeToRetrieveDirectly].!

printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	query printOn: aStream.
	aStream nextPut: $).!

printSQL
	stream nextPutAll: 'SELECT '.
	query printSelectFieldsOn: self.
	self findBoundExpressions.
	query printTablesOn: self.
	query printWhereClauseOn: self.
	query printJoinsOn: self.
	query printGroupByOn: self.
	query printOrderingOn: self.
	query printPostLimitOn: self.
	query printOffsetOn: self.!

query
	^query.!

query: aQuery
	query := aQuery.!

tables
	^(self query tablesToPrint collect: [:each | each parent]) asSet.! !
!QuerySelectCommand categoriesFor: #bindings!accessing!private! !
!QuerySelectCommand categoriesFor: #bindingTypes!accessing!private! !
!QuerySelectCommand categoriesFor: #blockFactor!accessing!private! !
!QuerySelectCommand categoriesFor: #boundExpressions!accessing!private! !
!QuerySelectCommand categoriesFor: #boundExpressions:!accessing!private! !
!QuerySelectCommand categoriesFor: #findBoundExpressions!accessing!private! !
!QuerySelectCommand categoriesFor: #maximumLobSizeToRetrieveDirectly!accessing!private! !
!QuerySelectCommand categoriesFor: #printOn:!printing!private! !
!QuerySelectCommand categoriesFor: #printSQL!accessing!public! !
!QuerySelectCommand categoriesFor: #query!accessing!private! !
!QuerySelectCommand categoriesFor: #query:!accessing!private! !
!QuerySelectCommand categoriesFor: #tables!private!reporting! !

SQLStringSelectCommand guid: (GUID fromString: '{696BD2E4-204D-4432-8B38-EDE6EA3FBA67}')!
SQLStringSelectCommand comment: 'This is a select command whose SQL is given, rather than being generated from a query.

'!
!SQLStringSelectCommand categoriesForClass!Unclassified! !
!SQLStringSelectCommand methodsFor!

bindings
	"We're not actually sure if we're part of a query with a where clause, or being used in a more ad hoc way. So if we have a query, use either our pre-set parameters, or the query's bindings, whichever produces more. In a tie, pick the query's bindings."
	| queryBindings |
	query isNil ifTrue: [^parameters].
	queryBindings := super bindings.
	^queryBindings size >= parameters size 
		ifTrue: [queryBindings]
		ifFalse: [parameters].!

bindingTypes
	^query isNil ifTrue: [#()] ifFalse: [super bindingTypes].!

blockFactor
	blockFactor isNil ifTrue: [
		blockFactor := query isNil ifTrue: [5] ifFalse: [super blockFactor]].
	^blockFactor.!

printSQL
	"We assume the SQL is provided."
	^self.!

setSQLString: aString
	sqlString := aString.!

tables
	^query isNil ifTrue: [Array with: #Unknown] ifFalse: [super tables].! !
!SQLStringSelectCommand categoriesFor: #bindings!accessing!private! !
!SQLStringSelectCommand categoriesFor: #bindingTypes!accessing!private! !
!SQLStringSelectCommand categoriesFor: #blockFactor!accessing!private! !
!SQLStringSelectCommand categoriesFor: #printSQL!private!sql! !
!SQLStringSelectCommand categoriesFor: #setSQLString:!accessing!private! !
!SQLStringSelectCommand categoriesFor: #tables!private!reporting! !

!SQLStringSelectCommand class methodsFor!

sqlString: aString parameters: aDictionaryOrArray useBinding: aBoolean session: aSession
	^(self new)
		setSQLString: aString;
		parameters: aDictionaryOrArray;
		useBinding: aBoolean;
		session: aSession.! !
!SQLStringSelectCommand class categoriesFor: #sqlString:parameters:useBinding:session:!As yet unclassified!public! !

SQLite3PragmaCommand guid: (GUID fromString: '{D1E79FFD-FC78-4730-BA89-478FBE5C89A4}')!
SQLite3PragmaCommand comment: ''!
!SQLite3PragmaCommand categoriesForClass!Unclassified! !
!SQLite3PragmaCommand methodsFor!

printSQL
	stream nextPutAll: 'PRAGMA '.
	query resultClass = DatabaseField ifTrue: [stream nextPutAll: 'table_info ('].
	query resultClass name = #SQLite3ForeignKeyData ifTrue: [stream nextPutAll: 'foreign_key_list('].
	query resultClass name = #SQLite3IndexData ifTrue: [stream nextPutAll: 'index_list('].
	query resultClass name = #SQLite3IndexColumnData ifTrue: [stream nextPutAll: 'index_info('].	"query printSelectFieldsOn: self."
	self findBoundExpressions.	"query printTablesOn: self."
	query printWhereClauseOn: self.
	stream nextPutAll: ')'	"query printJoinsOn: self.
	query printOrderingOn: self.
	query printGroupByOn: self.
	query printPostLimitOn: self.
	query printOffsetOn: self."! !
!SQLite3PragmaCommand categoriesFor: #printSQL!public!sql! !

DelegatingDatabaseConverter guid: (GUID fromString: '{5C3A43D7-F175-40C9-95EE-7EE3B8DFE1C5}')!
DelegatingDatabaseConverter comment: 'This is a converter which delegates the conversion to methods on some other object, typically the database platform.

Instance Variables:
	dbToStSelector	<Symbol>	The symbol we perform when converter from database to object representation. Takes two arguments, the value, and the database type.
	host	<Object>	The object that holds the methods specified by the selectors.
	stToDbSelector	<Symbol>	The symbol we perform when converter from object to database representation. Takes two arguments, the value, and the database type.

'!
!DelegatingDatabaseConverter categoriesForClass!Unclassified! !
!DelegatingDatabaseConverter methodsFor!

convert: anObject fromDatabaseRepresentationAs: aDatabaseType 
	^host perform: dbToStSelector with: anObject with: aDatabaseType!

convert: anObject toDatabaseRepresentationAs: aDatabaseType 
	^host perform: stToDbSelector with: anObject with: aDatabaseType.!

hostedBy: anObject fromStToDb: stDbSelector fromDbToSt: dbStSelector

	host := anObject.
	stToDbSelector := stDbSelector.
	dbToStSelector := dbStSelector!

named: aSymbol hostedBy: anObject fromStToDb: stDbSelector fromDbToSt: dbStSelector

	name := aSymbol.
	host := anObject.
	stToDbSelector := stDbSelector.
	dbToStSelector := dbStSelector! !
!DelegatingDatabaseConverter categoriesFor: #convert:fromDatabaseRepresentationAs:!converting!private! !
!DelegatingDatabaseConverter categoriesFor: #convert:toDatabaseRepresentationAs:!converting!private! !
!DelegatingDatabaseConverter categoriesFor: #hostedBy:fromStToDb:fromDbToSt:!initialize-release!private! !
!DelegatingDatabaseConverter categoriesFor: #named:hostedBy:fromStToDb:fromDbToSt:!initialize-release!private! !

!DelegatingDatabaseConverter class methodsFor!

hostedBy: anObject fromStToDb: stDbSelector fromDbToSt: dbStSelector

	^super new
		hostedBy: anObject fromStToDb: stDbSelector fromDbToSt: dbStSelector!

named: aSymbol hostedBy: anObject fromStToDb: stDbSelector fromDbToSt: dbStSelector
	^super new
		named: aSymbol
		hostedBy: anObject
		fromStToDb: stDbSelector
		fromDbToSt: dbStSelector.! !
!DelegatingDatabaseConverter class categoriesFor: #hostedBy:fromStToDb:fromDbToSt:!instance creation!private! !
!DelegatingDatabaseConverter class categoriesFor: #named:hostedBy:fromStToDb:fromDbToSt:!instance creation!private! !

GlorpNullConverter guid: (GUID fromString: '{0A5F26CD-A4FB-4BD9-8B3A-84AE85DDA947}')!
GlorpNullConverter comment: 'This is a no-op converter. When converting in either direction, it just returns the value.'!
!GlorpNullConverter categoriesForClass!Unclassified! !
!GlorpNullConverter methodsFor!

convert: anObject fromDatabaseRepresentationAs: aDatabaseType
	^anObject.!

convert: anObject toDatabaseRepresentationAs: aDatabaseType
	^anObject.! !
!GlorpNullConverter categoriesFor: #convert:fromDatabaseRepresentationAs:!converting!private! !
!GlorpNullConverter categoriesFor: #convert:toDatabaseRepresentationAs:!converting!private! !

PluggableDatabaseConverter guid: (GUID fromString: '{05C308C1-10D2-43DB-8575-527E2954A128}')!
PluggableDatabaseConverter comment: 'This is a converter where the conversion is specified by two blocks.

Instance Variables:
	dbToSt	<BlockClosur>	The block to run to convert from the database representation to Smalltalk representation. Takes one argument, the value to be converted.
	stToDb	<BlockClosure>	The block to run to convert from Smalltalk representation to the database representation. Takes one argument, the value to be converted.

'!
!PluggableDatabaseConverter categoriesForClass!Unclassified! !
!PluggableDatabaseConverter methodsFor!

convert: anObject fromDatabaseRepresentationAs: aDatabaseType 
	^dbToSt isNil ifTrue: [anObject] ifFalse: [dbToSt value: anObject]!

convert: anObject toDatabaseRepresentationAs: aDatabaseType 
	^stToDb isNil ifTrue: [anObject] ifFalse: [stToDb value: anObject]!

dbToStConverter: aBlock

	dbToSt := aBlock.!

stToDbConverter: aBlock

	stToDb := aBlock.! !
!PluggableDatabaseConverter categoriesFor: #convert:fromDatabaseRepresentationAs:!converting!private! !
!PluggableDatabaseConverter categoriesFor: #convert:toDatabaseRepresentationAs:!converting!private! !
!PluggableDatabaseConverter categoriesFor: #dbToStConverter:!accessing!private! !
!PluggableDatabaseConverter categoriesFor: #stToDbConverter:!accessing!private! !

AdabasLikePlatform guid: (GUID fromString: '{2103C834-842A-45E2-A196-C5C111CC5598}')!
AdabasLikePlatform comment: 'This is a platform for Adabas and related databases.'!
!AdabasLikePlatform categoriesForClass!Unclassified! !
!AdabasLikePlatform methodsFor!

supportsConstraints
	"^<Boolean> This method returns true, if the database support a contraint concept"

	^false! !
!AdabasLikePlatform categoriesFor: #supportsConstraints!private!SQL! !

DB2Platform guid: (GUID fromString: '{F7622E1D-834D-48F9-86BB-8120C2FF6AD2}')!
DB2Platform comment: 'This is a Glorp platform for DB2 databases.
'!
!DB2Platform categoriesForClass!Unclassified! !
!DB2Platform methodsFor!

areSequencesExplicitlyCreated

	^true!

bindingsForGroupWritingFor: aCommand
	"Return the bindings array for a group write. This can be in different formats, depending on the database and perhaps the mechanism in place."
	^aCommand batchStatementBindings.!

blob
	^self typeNamed: #blob ifAbsentPut: [GlorpBlobType new typeString: 'BLOB'].!

booleanToBooleanConverter
	^DelegatingDatabaseConverter
		named: #booleanToBoolean
		hostedBy: self
		fromStToDb: #convertBooleanToInteger:for:
		fromDbToSt: #convertDBBooleanToBoolean:for:.!

canUseInWhereClause: aValue type: aType
	"This is mostly for test support purposes. If we know we can't use a particular kind of value in the where clause on some database, don't try."

	(super canUseInWhereClause: aValue type: aType) ifFalse: [^false].
	"In DB2 we don't seem to be able to compare lobs against anything.
	But if you convert the lob itself, that works sometimes. Eg.,
		... WHERE CHAR(clb) = 'abcdefghijklmnop'.
		... WHERE SUBSTR(clb,1, 16) LIKE 'abcdefghijklmnop'.
		... WHERE CHAR(SUBSTR(clb,1, 16)) = 'abcdefghijklmnop'.
	Note that LIKE works with SUBSTR(), but = works with CHAR()."
	((Set with: (self blob) with: (self clob) with: (self nclob)) includes: aType) ifTrue: [^false].
	^true.!

char
	^self typeNamed: #char ifAbsentPut: [GlorpCharType new typeString: 'CHARACTER'].!

charactersThatNeedEscaping
	"There seem to be all kind of contradictory bits of information about what sql server does/requires for escaped characters, all of which differ from standard sql. Empirically the only thing that requires escaping appears to be single quote"
	^#($' ).!

clob

	^self typeNamed: #clob ifAbsentPut: [GlorpClobType new typeString: 'CLOB'].!

compoundOperationFor: aSymbol
	"Return the platform specific version of a compound statement symbol"
	aSymbol == #INTERSECT ifTrue: [^'WHERE EXISTS'].
	aSymbol == #MINUS ifTrue: [^'WHERE NOT EXISTS'].
	^aSymbol.!

createAlterColumnStatement: aField newType: aType usingExpression: expression

	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
	self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' ALTER COLUMN';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll:  ' SET DATA TYPE ';
		nextPutAll: aType typeString.
	expression ifNotNil: [ sqlStatementStream nextPutAll: ' USING ', expression ].

	^sqlStatementStream contents!

createConstraintAddStatement:  aConstraint
	| sqlStatementStream |
	aConstraint shouldCreateInDatabase ifFalse: [^''].
	self supportsConstraints
			ifTrue: [sqlStatementStream := WriteStream on: String new.
					sqlStatementStream
						nextPutAll:  'ALTER TABLE';
						space.
					self printDDLTableNameFor: aConstraint table on: sqlStatementStream.
					(aConstraint creationStringFor: self)
						do: [ :each | sqlStatementStream nextPutAll: ' ADD ', each.]
						separatedBy: [sqlStatementStream nextPut: Character space].
					^sqlStatementStream contents].
	^String new!

databaseIdentitySequenceClass
	"Uses the IDENTITY column mechanism."

	^DB2IdentitySequence!

databaseSequenceClass
	"Use the SEQUENCE generator instead of the IDENTITY column mechanism."

	^DB2Sequence!

date
	"Return the type we use to represent dates."
	^self typeNamed: #date ifAbsentPut: [GlorpDateType new].!

decimal
	^self numeric.!

double
	^self typeNamed: #double ifAbsentPut: [GlorpDoubleType new typeString: 'DOUBLE' ].!

escapeFor: aCharacter
	^String with: $' with: aCharacter.
"	^'\', (aCharacter asInteger printStringRadix: 16)."!

float
	^self typeNamed: #float ifAbsentPut: [GlorpFloatType new typeString: 'REAL' ].!

float4
	^self float.!

float8
	^self double.!

initializeFunctions
	"DB2 doesn't allow CAST(date AS timestamp), but it does have a TIMESTAMP function."

	| baseExists dbSpecificExists selectList baseNotExists dbSpecificNotExists |
	super initializeFunctions.
	functions
		at: #, put: (InfixFunction named: '+');
		at: #timestamp: put: ((PrefixFunction named: 'TIMESTAMP') type: (self timestamp)); "TIMESTAMP(date, time)"
		at: #time0 put: ((StandaloneFunction named: 'TIME(''00:00:00'')') type: (self time));
		at: #castTo: put: ((PrefixFunction named: 'CAST') separator: ' AS '). "CAST(aThing AS aType)"
		"at: #like: put: (InfixFunction named: 'LIKE').
		at: #copyFrom:to: put: (SubstringFunction named: 'SUBSTRING');"

	"Like Oracle, SQL Server doesn't allow functions like EXISTS in a field list,
	but will allow a select there as an argument to a function, such as CASE.
	So print it that way if we're in the select list."

	baseExists := functions at: #exists:.
	dbSpecificExists := DualRoleFunction new.
	dbSpecificExists function: '#exists:'.
	dbSpecificExists whereClauseVersion: baseExists.
	selectList := Case new argumentTypes: (Array with: (self varchar: 1) with: self integer with: self integer).
	selectList argumentModifierBlock: [:function :args | 
		function base: args first.
		Array with: 'x' asGlorpExpression with: 1 asGlorpExpression with: 0 asGlorpExpression].
	dbSpecificExists selectListVersion: selectList.
	dbSpecificExists type: self integer.
	functions at: #exists: put: dbSpecificExists.

	"And similarly NOT EXISTS"
	baseNotExists := functions at: #notExists:.
	dbSpecificNotExists := DualRoleFunction new.
	dbSpecificNotExists function: '#notExists:'.
	dbSpecificNotExists whereClauseVersion: baseNotExists.
	selectList := Case new argumentTypes: (Array with: (self varchar: 1) with: self integer with: self integer).
	selectList argumentModifierBlock: [:function :args | 
		function base: args first.
		Array with: 'x' asGlorpExpression with: 0 asGlorpExpression with: 1 asGlorpExpression].
	dbSpecificNotExists selectListVersion: selectList.
	dbSpecificNotExists type: self integer.
	functions at: #notExists: put: dbSpecificNotExists.!

int
	^self integer.!

int2
	^self smallint.!

int4

	^self integer.!

int8
	^self typeNamed: #int8 ifAbsentPut: [GlorpIntegerType new typeString: 'BIGINT'].!

isDB2Platform

	^true!

maximumLengthOfColumnName
	"^<Integer> I return the max. length of a column name"

	^128!

maximumLengthOfTableName
	"^<Integer> I return the max. length of a table name"

	^128!

maximumQueryValueSize
	"If there's a limit on how big a bound input parameter can be for a query, return it here.
	For literal strings and lobs, at least, the DB2 limit is 32K. Not sure about bound objects.
	If those can be larger, then we can indicate that fact by introducing a companion method
	later. Note that we write literal LOBs as 2-byte hex chars per byte. So, the limit is
	actually 16384-9-1, eg., blob(X'0A41') makes 9 extras."
	^16374!

maximumSizeToGroupWriteFor: aCollectionOfDatabaseRows
	"If we are going to group write, how many rows of this collection should we do it for at once"
	^aCollectionOfDatabaseRows size min: 250.!

nclob

	^self typeNamed: #nclob ifAbsentPut: [GlorpClobType new typeString: 'dbclob'].!

numeric
	"DB2 NUMERIC columns default to precision=5, scale=0."
	
	^self typeNamed: #numeric ifAbsentPut: [GlorpNumericType new typeString: 'DECIMAL']!

nvarchar
	^self typeNamed: #nvarchar ifAbsentPut: [GlorpVarCharType new typeString: 'nvarchar'].!

printBlob: aByteArray on: aStream for: aType
	aByteArray isNil ifTrue: [^aStream nextPutAll: 'NULL'].
	aStream nextPutAll: 'BLOB(X'''.
	aByteArray
		do:
			[:each | 
			each < 16 ifTrue: [aStream nextPut: $0].
			each
				printOn: aStream
				base: 16
				showRadix: false].
	aStream nextPutAll: ''') '	"	aStream nextPutAll: 'CAST(X'''.
	aByteArray do: [:each |
		each printOn: aStream paddedWith: $0 to: 2 base: 16].
	aStream nextPutAll: ''' AS BLOB) '."!

printBoolean: aBoolean for: aType
	"If the platform is MS SQL Server, a boolean object in a SELECT needs to be converted to  BIT type, since T, Y, or true make SQLServer complain. "
	
	| string |
	aBoolean isNil ifTrue: [^'NULL'].
	"We may be getting passed something that's already 1 or 0, or something else instead of a boolean. That probably shouldn't be happening, but let's handle it gracefully. We define gracefully to mean 1 is true, and everything else is false"
	string := (aBoolean == 1 or: [aBoolean = true]) 
		ifTrue: ['1'] 
		ifFalse: ['0'].
	^'CONVERT( BIT, ' , string , ' )'.!

printDate: aDate for: aType
	"Print a date (or timestamp) as yyyy-mm-dd"
	| stream |
	aDate isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPut: $'.
	self
		printDate: aDate
		isoFormatOn: stream.
	stream nextPut: $'.
	^stream contents.!

printPostLimit: anInteger on: aCommand
	aCommand nextPutAll: ' FETCH FIRST  '.
	aCommand nextPutAll: anInteger printString.
	aCommand nextPutAll: ' ROWS ONLY '!

printSqlStatementToListTablesInSchema: schemaString on: aStream
	"Not standardized"

	| lowerSchema |
	lowerSchema := schemaString isNil
		ifTrue: [schemaString]
		ifFalse: [schemaString asLowercase].
	aStream nextPutAll: 'SELECT table_name FROM sysibm.tables '.
	(lowerSchema notNil and: [lowerSchema notEmpty]) ifTrue:
		[aStream
			nextPutAll: 'WHERE lower(table_schema) = ';
			nextPut: $';
			nextPutAll: lowerSchema;
			nextPut: $']!

printSqlStatementToTestExistenceOfTable: tableName inSchema: schemaString on: aStream
	" This query will test only for tables that the logged on user has access to... "

	| lowerTableName lowerSchema |
	lowerTableName := tableName asLowercase.
	lowerSchema := schemaString isNil
		ifTrue: [schemaString]
		ifFalse: [schemaString asLowercase].
	aStream
		nextPutAll: 'SELECT count(*) FROM sysibm.tables ';
		nextPutAll: 'WHERE lower(table_name) = ';
		nextPut: $';
		nextPutAll: lowerTableName;
		nextPut: $'.
	(lowerSchema notNil and: [lowerSchema notEmpty]) ifTrue:
		[aStream
			nextPutAll: ' AND lower(table_schema) = ';
			nextPut: $';
			nextPutAll: lowerSchema;
			nextPut: $']!

printTime: aTime for: aType
	"Print a time (or timestamp) as hh:mm:ss.fff"
	| stream |
	aTime isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPutAll: 'TIME( '''.
	self
		printTime: aTime
		isoFormatOn: stream
		milliseconds: self supportsMillisecondsInTimes.
	stream nextPutAll: ''' )'.
	^stream contents.!

printTimestamp: aTimestamp on: stream for: aType
	aTimestamp isNil ifTrue: [stream nextPutAll: 'NULL'. ^self].
	stream nextPut: $'.
	self
		printDate: aTimestamp
		isoFormatOn: stream.
	stream nextPutAll: ' '.
	self
		printTime: aTimestamp
		isoFormatOn: stream.
	stream nextPut: $'.!

queryWithUnsupportedOperationsEliminatedFrom: aQuery do: aBlock
	"If aQuery has operations that we don't support, rewrite it to do them in terms of lower level operations. In particular, rewrite INTERSECT/EXCEPT operations into EXISTS clauses in a single query. Pass the new query to aBlock."
	| newQuery |
	newQuery := aQuery rewriteIntersect.
	newQuery := newQuery rewriteExcept.
	newQuery == aQuery ifFalse: [aBlock value: newQuery].!

real
	^self float.!

requiresCastsForNullsInUnions
	"If we have explicit nulls in a query which is unioned, do we need to cast them to the appropriate type in order to keep the database happy."
	^true.!

sequence
	^self serial.!

serial
	"This is the sequence based type used by Glorp."

	^self typeNamed: #serial ifAbsentPut: [GlorpSerialType new typeString: (self integer typeString)].!

serialIdentity
	"This is the identity column type used by DB2, but not supported by Glorp. It is here just
	so that we can recognize identity columns when they are encountered."

	^self typeNamed: #serialIdentity ifAbsentPut: [DB2IdentityType new typeString: 'integer generated always as identity '].!

sqlTextForNULLAttributeConstraint
	"^<String>"

	^''!

supportsANSIJoins
	"Do we support the JOIN <tableName> USING <criteria> syntax. Currently hard-coded, but may also vary by database version"
	^true.!

supportsBinding
	"Binding works only with VW EXDI so far"
	^Dialect isVisualWorks.!

supportsConstraints

	^true!

supportsDecimalsOnAllNumerics
	"Return true if a general 'numeric' type will allow numbers after the decimal place.
	NUMERIC(5) will not store 321.12 (it'll keep 321.)."
	^false.!

supportsGroupWritingFor: aCommand
	"^aCommand supportsGroupWriting."
	"INSERT ...; INSERT ...; INSERT... fails,
	but this works: INSERT INTO table (id, c) VALUES (1,'1'), (2,'2'), (3,'3').
	For now, just do everything separately."
	^false!

supportsLimit
	"Do we support anything analogous to the postgresql LIMIT, returning only the first N rows"
	^true.!

supportsMillisecondsInTimes
	"Use TIMESTAMP for up to microseconds."
	^false!

supportsTableOwners
	"Return true if this platform supports table owners, i.e. expects table names of the form Bern.TW_* rather than just TW_* in its SQL."
"Access, Firebird and PostGreSQL do not, Oracle does, others I know not."

	^true!

text
	^super text queryType: self varchar.!

time
	^self typeNamed: #time ifAbsentPut: [GlorpTimeType new typeString: 'TIME'].!

timestamp
	^self typeNamed: #timestamp ifAbsentPut: [GlorpTimeStampType new typeString: 'TIMESTAMP'].!

timeStampTypeString
	^'datetime'.!

tinyint
	^self typeNamed: #tinyInt ifAbsentPut: [GlorpIntegerType new typeString: 'SMALLINT'].!

typesWithValues
	"Return our types list, but makes sure it has at least basic types in it"

	self date.
	self double.
	self int8.
	self decimal.
	self real.
	^super typesWithValues!

typeWithDatabaseName: aString characterWidth: aNumber extraInformation: extraValue
	"Return our type which matches the database name for the type. Assumes all variable width fields are strings right now."
	extraValue = 'IDENTITY' ifTrue: [^self serialIdentity].
	^super typeWithDatabaseName: aString characterWidth: aNumber.!

useMicrosoftOuterJoins
	"Return true if we use the Microsoft x *= y syntax for outer joins"
	^false.!

usesArrayBindingRatherThanGrouping
	"Return true if we use array binding for grouped writes rather than printing the sql multiple times. Only applies if we support grouped writes"
	^false.!

usesIdentityColumns
	^false!

varbinary
	^self typeNamed: #varbinary ifAbsentPut: [GlorpVarBinaryType new].!

varchar
	^self typeNamed: #varchar ifAbsentPut: [GlorpVarCharType new typeString: 'VARCHAR'].!

vwEXDIConnectionClass
	^'DB2Connection' asQualifiedReference valueIfUndefinedLoadFrom: 'DB2EXDI'! !
!DB2Platform categoriesFor: #areSequencesExplicitlyCreated!constants!public! !
!DB2Platform categoriesFor: #bindingsForGroupWritingFor:!binding!public! !
!DB2Platform categoriesFor: #blob!public!types! !
!DB2Platform categoriesFor: #booleanToBooleanConverter!conversion-boolean!public! !
!DB2Platform categoriesFor: #canUseInWhereClause:type:!public!testing! !
!DB2Platform categoriesFor: #char!public!types! !
!DB2Platform categoriesFor: #charactersThatNeedEscaping!conversion-strings!public! !
!DB2Platform categoriesFor: #clob!public!types! !
!DB2Platform categoriesFor: #compoundOperationFor:!database-specific!public! !
!DB2Platform categoriesFor: #createAlterColumnStatement:newType:usingExpression:!public!services columns! !
!DB2Platform categoriesFor: #createConstraintAddStatement:!public!services constraints! !
!DB2Platform categoriesFor: #databaseIdentitySequenceClass!constants!public! !
!DB2Platform categoriesFor: #databaseSequenceClass!public!sequences! !
!DB2Platform categoriesFor: #date!public!types! !
!DB2Platform categoriesFor: #decimal!public!types! !
!DB2Platform categoriesFor: #double!public!types! !
!DB2Platform categoriesFor: #escapeFor:!conversion-strings!public! !
!DB2Platform categoriesFor: #float!public!types! !
!DB2Platform categoriesFor: #float4!public!types! !
!DB2Platform categoriesFor: #float8!public!types! !
!DB2Platform categoriesFor: #initializeFunctions!functions!public! !
!DB2Platform categoriesFor: #int!public!types! !
!DB2Platform categoriesFor: #int2!public!types! !
!DB2Platform categoriesFor: #int4!public!types! !
!DB2Platform categoriesFor: #int8!public!types! !
!DB2Platform categoriesFor: #isDB2Platform!public!testing! !
!DB2Platform categoriesFor: #maximumLengthOfColumnName!constants!public! !
!DB2Platform categoriesFor: #maximumLengthOfTableName!constants!public! !
!DB2Platform categoriesFor: #maximumQueryValueSize!public!testing! !
!DB2Platform categoriesFor: #maximumSizeToGroupWriteFor:!binding!public! !
!DB2Platform categoriesFor: #nclob!public!types! !
!DB2Platform categoriesFor: #numeric!public!types! !
!DB2Platform categoriesFor: #nvarchar!public!types! !
!DB2Platform categoriesFor: #printBlob:on:for:!conversion-strings!public! !
!DB2Platform categoriesFor: #printBoolean:for:!conversion-boolean!public! !
!DB2Platform categoriesFor: #printDate:for:!conversion-times!public! !
!DB2Platform categoriesFor: #printPostLimit:on:!database-specific!public! !
!DB2Platform categoriesFor: #printSqlStatementToListTablesInSchema:on:!public!services tables! !
!DB2Platform categoriesFor: #printSqlStatementToTestExistenceOfTable:inSchema:on:!public!services tables! !
!DB2Platform categoriesFor: #printTime:for:!conversion-times!public! !
!DB2Platform categoriesFor: #printTimestamp:on:for:!conversion-times!public! !
!DB2Platform categoriesFor: #queryWithUnsupportedOperationsEliminatedFrom:do:!database-specific!public! !
!DB2Platform categoriesFor: #real!public!types! !
!DB2Platform categoriesFor: #requiresCastsForNullsInUnions!public!testing! !
!DB2Platform categoriesFor: #sequence!public!types! !
!DB2Platform categoriesFor: #serial!public!types! !
!DB2Platform categoriesFor: #serialIdentity!public!types! !
!DB2Platform categoriesFor: #sqlTextForNULLAttributeConstraint!constants!public! !
!DB2Platform categoriesFor: #supportsANSIJoins!public!testing! !
!DB2Platform categoriesFor: #supportsBinding!public!testing! !
!DB2Platform categoriesFor: #supportsConstraints!constants!public! !
!DB2Platform categoriesFor: #supportsDecimalsOnAllNumerics!public!testing! !
!DB2Platform categoriesFor: #supportsGroupWritingFor:!public!testing! !
!DB2Platform categoriesFor: #supportsLimit!public!testing! !
!DB2Platform categoriesFor: #supportsMillisecondsInTimes!constants!public! !
!DB2Platform categoriesFor: #supportsTableOwners!public!testing! !
!DB2Platform categoriesFor: #text!public!types! !
!DB2Platform categoriesFor: #time!public!types! !
!DB2Platform categoriesFor: #timestamp!public!types! !
!DB2Platform categoriesFor: #timeStampTypeString!public!types! !
!DB2Platform categoriesFor: #tinyint!public!types! !
!DB2Platform categoriesFor: #typesWithValues!public!type helpers! !
!DB2Platform categoriesFor: #typeWithDatabaseName:characterWidth:extraInformation:!public!testing! !
!DB2Platform categoriesFor: #useMicrosoftOuterJoins!public!testing! !
!DB2Platform categoriesFor: #usesArrayBindingRatherThanGrouping!public!testing! !
!DB2Platform categoriesFor: #usesIdentityColumns!public!testing! !
!DB2Platform categoriesFor: #varbinary!public!types! !
!DB2Platform categoriesFor: #varchar!public!types! !
!DB2Platform categoriesFor: #vwEXDIConnectionClass!accessing!public! !

!DB2Platform class methodsFor!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'! !
!DB2Platform class categoriesFor: #LICENSE!LICENSE!public! !

MySQLPlatform guid: (GUID fromString: '{FDBE1327-D31B-40A3-AB2A-EFE515585021}')!
MySQLPlatform comment: 'This is a definition for MySQL databases. It''s not done yet, just here as a placeholder.'!
!MySQLPlatform categoriesForClass!Unclassified! !
!MySQLPlatform methodsFor!

areSequencesExplicitlyCreated
	^false.!

bindingsForGroupWritingFor: aCommand
	"Return the bindings array for a group write. This can be in different formats, depending on the database and perhaps the mechanism in place."
	^aCommand batchStatementBindings.!

blob
	"Return the MySQL type for long blob."
	^self typeNamed: #blob ifAbsentPut: [GlorpBlobType new typeString: 'LONGBLOB'].!

canUseInWhereClause: aValue type: aType
	"This is mostly for test support purposes. If we know we can't use a particular kind of value in the where clause on some database, don't try."

	(super canUseInWhereClause: aValue type: aType) ifFalse: [^false].
	"On MySQL we don't seem to be able to compare constants against blobs."
	aType = self blob ifTrue: [^false].
	^true.!

charactersThatNeedEscaping
	^#($' $\)!

clob
	"Return the MySQL type for long text."
	^self typeNamed: #clob ifAbsentPut: [GlorpMySQLClobType new typeString: 'LONGTEXT'].!

databaseSequenceClass
	^MySQLSequence.!

date
	"Return the type we use to represent dates."
	^self typeNamed: #date ifAbsentPut: [GlorpDateType new].!

double
	^self typeNamed: #double ifAbsentPut: [GlorpDoubleType new typeString: 'double' ].!

endColumnQuote
	"Return the character used to start quoting the name of a column so that we can avoid conflicts with reserved names. The default MySQL quoting is to use backquotes rather than the ANSI style."
	^'`'.!

float
	^self typeNamed: #float ifAbsentPut: [GlorpFloatType new typeString: 'float' ].!

float4
	^self float.!

float8
	^self double.!

initializeReservedWords
	reservedWords := Set new.
	reservedWords 
		add: 'key'.!

int2
	^self smallint!

int4
	^self typeNamed: #int4 ifAbsentPut: [GlorpIntegerType new typeString: 'int'].!

int8
	^self typeNamed: #int8 ifAbsentPut: [GlorpIntegerType new typeString: 'bigint'].!

integer

	^self typeNamed: #integer ifAbsentPut: [GlorpIntegerType new typeString: 'int'].!

isMySQLPlatform
	^true.!

maximumLengthOfColumnName
	"^<Integer> I return the maximum length of a column name.  (I am not sent at present;  I am here so that if I and #maximumLengthOfTableName (which is sent) should ever differ on some database platform, I can be used in revised truncation logic.)
	Don't know the answer for this platform;  make something large up."

	^64!

maximumLengthOfTableName
	"^<Integer> I return the max. length of a table name"
	"Don't know, make something large up."
	^64!

nameForPrimaryKeyConstraintsWhenDropping
	"Return the name we use to describe a constraint when dropping them. This is required because MySQL wants you to use 'DROP PRIMARY KEY' rather than the ANSI 'DROP CONSTRAINT'."
	^'PRIMARY KEY'.!

number

	^self typeNamed: #number ifAbsentPut: [GlorpNumericType new typeString: 'decimal'].!

printDate: aDate for: aType
	| stream |
	aDate isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPut: $".
	self
		printDate: aDate
		isoFormatOn: stream.
	stream nextPut: $".
	^stream contents.!

printTime: aTime for: aType
	| stream |
	aTime isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPut: $".
	self
		printTime: aTime
		isoFormatOn: stream
		milliseconds: false.
	stream nextPut: $".
	^stream contents.!

printTimestamp: aTimestamp on: stream for: aType
	aTimestamp isNil ifTrue: [stream nextPutAll: 'NULL'. ^self].
	stream nextPut: $".
	self
		printDate: aTimestamp
		isoFormatOn: stream.
	stream nextPutAll: ' '.
	self
		printTime: aTimestamp
		isoFormatOn: stream
		milliseconds: false.
	stream nextPut: $".!

queryWithUnsupportedOperationsEliminatedFrom: aQuery do: aBlock
	"If aQuery has operations that we don't support, rewrite it to do them in terms of lower level operations. In particular, rewrite INTERSECT/EXCEPT operations into EXISTS clauses in a single query. Pass the new query to aBlock."
	| newQuery |
	newQuery := aQuery rewriteIntersect.
	newQuery := newQuery rewriteExcept.
	newQuery == aQuery ifFalse: [aBlock value: newQuery].!

sequence
	^self serial.!

serial

	^self typeNamed: #serial ifAbsentPut: [GlorpSerialType new typeString: 'int AUTO_INCREMENT'].!

smallint

	^self typeNamed: #smallint ifAbsentPut: [GlorpSmallIntType new typeString: 'SMALLINT'].!

startColumnQuote
	"Return the character used to start quoting the name of a column so that we can avoid conflicts with reserved names. The default MySQL quoting is to use backquotes rather than the ANSI style."
	^'`'.!

stringToByteArrayOneWayConverter
	"This is a special converter for MySQL that is a null conversion on the way out, but converts the byte arrays that we get back from blobs into strings."
	^DelegatingDatabaseConverter
		named: #stringToByteArrayOneWay
		hostedBy: self
		fromStToDb: #nullConversion:for:
		fromDbToSt: #byteArrayToString:for:.!

supportsANSIJoins
	"Do we support the JOIN <tableName> USING <criteria> syntax. Currently hard-coded, but may also vary by database version"
	^true.!

supportsBinding
	"MySQL supports binding, although early versions didn't."
	^true.!

supportsDecimalsOnAllNumerics
	"Return true if a general 'numeric' type will allow numbers after the decimal place"
	^false.!

supportsIntersect
	"Do we support operations like UNION ALL And INTERSECT."
	^false.!

supportsMillisecondsInTimes
	^false.!

time
	^self typeNamed: #time ifAbsentPut: [GlorpTimeType new typeString: 'time'].!

timestamp
	"Although MySQL has a type called 'timestamp', 'datetime' is more suitable."
	^self typeNamed: #timestamp ifAbsentPut: [GlorpTimeStampType new typeString: 'DATETIME'].!

typeWithDatabaseName: aString characterWidth: aNumber extraInformation: extraValue
	"Return our type which matches the database name for the type. Assumes all variable width fields are strings right now."
	extraValue = 'auto_increment' ifTrue: [^self serial].
	^super typeWithDatabaseName: aString characterWidth: aNumber.!

usesArrayBindingRatherThanGrouping
	^false.!

usesIdentityColumns
	^true.!

usesInformationSchema

	^true!

usesPrimaryKeyUniqueConstraintNames
	"When dropping/creating primary key constraints, do they have distinguished names."
	^false.!

varchar
	^self typeNamed: #varchar ifAbsentPut: [GlorpVarCharType new].!

vwEXDIConnectionClass
	^'MySQLConnection' asQualifiedReference valueIfUndefinedLoadFrom: 'MySQLEXDI'!

writeConstraintToDrop: aConstraint onStream: aStream
	"Return the term we use to describe a constraint when dropping them.  This is required because MySQL wants you to use
		'DROP PRIMARY KEY' for primary key constraints (and then says don't name the constraint name in the drop statement!!)
		'DROP FOREIGN KEY'  for foreign key constraints
		'DROP KEY' for unique constraints (it also supports DROP INDEX for this)
	rather than the ANSI 'DROP CONSTRAINT'."

	aConstraint isForeignKeyConstraint ifTrue: [aStream nextPutAll: 'FOREIGN KEY'].
	aConstraint isPrimaryKeyConstraint ifTrue: [aStream nextPutAll: 'PRIMARY KEY'].
	aConstraint isUniqueConstraint ifTrue: [aStream nextPutAll: 'KEY'].
	aConstraint isPrimaryKeyConstraint ifFalse:
		[aStream
			nextPut: $ ;
			nextPutAll: aConstraint name].! !
!MySQLPlatform categoriesFor: #areSequencesExplicitlyCreated!constants!private! !
!MySQLPlatform categoriesFor: #bindingsForGroupWritingFor:!binding!private! !
!MySQLPlatform categoriesFor: #blob!private!types! !
!MySQLPlatform categoriesFor: #canUseInWhereClause:type:!private!testing! !
!MySQLPlatform categoriesFor: #charactersThatNeedEscaping!constants!private! !
!MySQLPlatform categoriesFor: #clob!private!types! !
!MySQLPlatform categoriesFor: #databaseSequenceClass!private!sequences! !
!MySQLPlatform categoriesFor: #date!private!types! !
!MySQLPlatform categoriesFor: #double!private!types! !
!MySQLPlatform categoriesFor: #endColumnQuote!private!services tables! !
!MySQLPlatform categoriesFor: #float!private!types! !
!MySQLPlatform categoriesFor: #float4!private!types! !
!MySQLPlatform categoriesFor: #float8!private!types! !
!MySQLPlatform categoriesFor: #initializeReservedWords!constants!private! !
!MySQLPlatform categoriesFor: #int2!private!types! !
!MySQLPlatform categoriesFor: #int4!private!types! !
!MySQLPlatform categoriesFor: #int8!private!types! !
!MySQLPlatform categoriesFor: #integer!private!types! !
!MySQLPlatform categoriesFor: #isMySQLPlatform!private!testing! !
!MySQLPlatform categoriesFor: #maximumLengthOfColumnName!constants!public! !
!MySQLPlatform categoriesFor: #maximumLengthOfTableName!constants!private! !
!MySQLPlatform categoriesFor: #nameForPrimaryKeyConstraintsWhenDropping!constants!private! !
!MySQLPlatform categoriesFor: #number!private!types! !
!MySQLPlatform categoriesFor: #printDate:for:!conversion-times!private! !
!MySQLPlatform categoriesFor: #printTime:for:!conversion-times!private! !
!MySQLPlatform categoriesFor: #printTimestamp:on:for:!conversion-times!private! !
!MySQLPlatform categoriesFor: #queryWithUnsupportedOperationsEliminatedFrom:do:!database-specific!private! !
!MySQLPlatform categoriesFor: #sequence!private!types! !
!MySQLPlatform categoriesFor: #serial!private!types! !
!MySQLPlatform categoriesFor: #smallint!private!types! !
!MySQLPlatform categoriesFor: #startColumnQuote!private!services tables! !
!MySQLPlatform categoriesFor: #stringToByteArrayOneWayConverter!conversion-strings!private! !
!MySQLPlatform categoriesFor: #supportsANSIJoins!private!testing! !
!MySQLPlatform categoriesFor: #supportsBinding!public!testing! !
!MySQLPlatform categoriesFor: #supportsDecimalsOnAllNumerics!private!testing! !
!MySQLPlatform categoriesFor: #supportsIntersect!private!testing! !
!MySQLPlatform categoriesFor: #supportsMillisecondsInTimes!constants!private! !
!MySQLPlatform categoriesFor: #time!private!types! !
!MySQLPlatform categoriesFor: #timestamp!private!types! !
!MySQLPlatform categoriesFor: #typeWithDatabaseName:characterWidth:extraInformation:!private!types! !
!MySQLPlatform categoriesFor: #usesArrayBindingRatherThanGrouping!private!testing! !
!MySQLPlatform categoriesFor: #usesIdentityColumns!private!testing! !
!MySQLPlatform categoriesFor: #usesInformationSchema!public!testing! !
!MySQLPlatform categoriesFor: #usesPrimaryKeyUniqueConstraintNames!private!testing! !
!MySQLPlatform categoriesFor: #varchar!private!types! !
!MySQLPlatform categoriesFor: #vwEXDIConnectionClass!accessing!private! !
!MySQLPlatform categoriesFor: #writeConstraintToDrop:onStream:!public!services constraints! !

!MySQLPlatform class methodsFor!

createLoginFromConnectionDictionary: aDict
	" *** This method was defined by Glorp.DatabasePlatform class as a subclass responsibility.
	Replace its body with a proper implementation. *** "

	self error: 'Subclass responsibility stub not reimplemented'! !
!MySQLPlatform class categoriesFor: #createLoginFromConnectionDictionary:!*eoglorp!private! !

OcelotPlatform guid: (GUID fromString: '{30C20F7F-8C5C-42FD-BD7F-62FAE6B2C4CA}')!
OcelotPlatform comment: 'This is a platform for the open-source Ocelot database.'!
!OcelotPlatform categoriesForClass!Unclassified! !
!OcelotPlatform methodsFor!

areSequencesExplicitlyCreated

^false!

boolean
	^self typeNamed: #boolean ifAbsentPut: [GlorpBooleanType new typeString: 'boolean'].!

clob

	^self typeNamed: #clob ifAbsentPut: [GlorpClobType new typeString: 'clob'].!

databaseSequenceClass
	^InMemorySequence.!

date
	^self typeNamed: #date ifAbsentPut: [GlorpDateType new].!

double
	^self float8!

float
	^self float4.!

float4
	^self typeNamed: #float4 ifAbsentPut: [GlorpFloatType new typeString: 'float'].!

float8
	^self typeNamed: #float8 ifAbsentPut: [GlorpDoubleType new typeString: 'double precision'].!

int2
	^self smallint.!

int4

	^self integer.!

int8

	^self integer!

isOcelotPlatform

	^true!

isODBCPlatform

	^true!

numeric
	^self typeNamed: #numeric ifAbsentPut: [GlorpNumericType new].!

paddingCharacter
	^32 asCharacter.!

padString: aString for: aType 
	| padding |
	aString isNil ifTrue: [^nil].
	(self usesNullForEmptyStrings and: [aString isEmpty]) ifTrue: [^nil].
	aString size > aType width 
				ifTrue: [^aString copyFrom: 1 to: aType width].
				
	aType isVariableWidth ifTrue: [^aString].
	padding := String new: aType width - aString size.
	padding atAllPut: 32 asCharacter.
	^aString , padding!

primaryKeysAreAutomaticallyUnique

	^true.!

printDate: aDate for: aType
	| stream |
	aDate isNil ifTrue: [^nil].
	stream := WriteStream on: String new.
	stream nextPutAll: 'date '''.
	self
		printDate: aDate
		isoFormatOn: stream.
	stream nextPutAll: ''''.
	^stream contents.!

printTime: aTime for: aType
	| stream |
	aTime isNil ifTrue: [^nil].
	stream := WriteStream on: String new.
	stream nextPutAll: 'time '''.
	self
		printTime: aTime
		isoFormatOn: stream.
	stream nextPutAll: ''''.
	^stream contents.!

printTimestamp: aTimestamp on: stream for: aType

	aTimestamp isNil ifTrue: [stream nextPutAll: 'NULL'. ^self].
	stream nextPutAll: 'timestamp '''.
	self
		printDate: aTimestamp
		isoFormatOn: stream.
	stream nextPut: $ .
	self
		printTime: aTimestamp
		isoFormatOn: stream.
	"	milliseconds: true."
	stream nextPutAll: ''''.!

sequence

	^self inMemorySequence!

serial

	^self sequence!

sqlTextForNULLAttributeConstraint
	"^<String>"

	^''!

supportsANSIJoins
	"Do we support the JOIN <tableName> USING <criteria> syntax. Currently hard-coded, but may also vary by database version"

	^true!

supportsMillisecondsInTimes
	"I'm guessing here"
	^true.!

text
	^self clob.!

time
	^self typeNamed: #time ifAbsentPut: [GlorpTimeType new typeString: 'time'].!

timestamp
	^self typeNamed: #timestamp ifAbsentPut: [GlorpTimeStampType new typeString: 'timestamp'].!

unpadString: aString for: aType

	aString isNil ifTrue: [^nil].
	^aType isVariableWidth 
		ifTrue: [aString]
		ifFalse: [(ReadStream on: aString) upTo: 32 asCharacter].!

usesArrayBindingRatherThanGrouping
	^false.!

usesNullForEmptyStrings
	"Return true if this database is likely to use nil as an empty string value"
	^false.!

varchar
	^self typeNamed: #varchar ifAbsentPut: [GlorpVarCharType new].! !
!OcelotPlatform categoriesFor: #areSequencesExplicitlyCreated!constants!private! !
!OcelotPlatform categoriesFor: #boolean!private!types! !
!OcelotPlatform categoriesFor: #clob!private!types! !
!OcelotPlatform categoriesFor: #databaseSequenceClass!constants!private! !
!OcelotPlatform categoriesFor: #date!private!types! !
!OcelotPlatform categoriesFor: #double!private!types! !
!OcelotPlatform categoriesFor: #float!private!types! !
!OcelotPlatform categoriesFor: #float4!private!types! !
!OcelotPlatform categoriesFor: #float8!private!types! !
!OcelotPlatform categoriesFor: #int2!private!types! !
!OcelotPlatform categoriesFor: #int4!private!types! !
!OcelotPlatform categoriesFor: #int8!private!types! !
!OcelotPlatform categoriesFor: #isOcelotPlatform!private!testing! !
!OcelotPlatform categoriesFor: #isODBCPlatform!private!testing! !
!OcelotPlatform categoriesFor: #numeric!private!types! !
!OcelotPlatform categoriesFor: #paddingCharacter!conversion-strings!private! !
!OcelotPlatform categoriesFor: #padString:for:!conversion-strings!private! !
!OcelotPlatform categoriesFor: #primaryKeysAreAutomaticallyUnique!constants!private! !
!OcelotPlatform categoriesFor: #printDate:for:!conversion times!private! !
!OcelotPlatform categoriesFor: #printTime:for:!conversion times!private! !
!OcelotPlatform categoriesFor: #printTimestamp:on:for:!conversion times!private! !
!OcelotPlatform categoriesFor: #sequence!private!types! !
!OcelotPlatform categoriesFor: #serial!private!types! !
!OcelotPlatform categoriesFor: #sqlTextForNULLAttributeConstraint!constants!private! !
!OcelotPlatform categoriesFor: #supportsANSIJoins!private!testing! !
!OcelotPlatform categoriesFor: #supportsMillisecondsInTimes!constants!private! !
!OcelotPlatform categoriesFor: #text!private!types! !
!OcelotPlatform categoriesFor: #time!private!types! !
!OcelotPlatform categoriesFor: #timestamp!private!types! !
!OcelotPlatform categoriesFor: #unpadString:for:!conversion-strings!private! !
!OcelotPlatform categoriesFor: #usesArrayBindingRatherThanGrouping!private!testing! !
!OcelotPlatform categoriesFor: #usesNullForEmptyStrings!constants!private! !
!OcelotPlatform categoriesFor: #varchar!private!types! !

OraclePlatform guid: (GUID fromString: '{811C4906-95D6-4245-8CF6-646E37DE254F}')!
OraclePlatform comment: 'This is a platform for Oracle databases. Currently there''s only one, but it might be better for some purposes to separate out by versions, as some older versions don''t support all features.'!
!OraclePlatform categoriesForClass!Unclassified! !
!OraclePlatform methodsFor!

areSequencesExplicitlyCreated
	^true.!

bindingsForGroupWritingFor: aCommand
	"Return the bindings array for a group write. This can be in different formats, depending on the database and perhaps the mechanism in place."
	^aCommand arrayBindings.!

boolean
	^self int2.!

canUseInWhereClause: aValue type: aType
	"This is mostly for test support purposes. If we know we can't use a particular kind of value in the where clause on some database, don't try."

	(super canUseInWhereClause: aValue type: aType) ifFalse: [^false].
	"On Oracle we effectively can't compare constants against blobs."
	aType = self blob ifFalse: [^true].
	aValue isNil ifTrue: [^true].
	^false.!

char
	^self typeNamed: #char ifAbsentPut: [GlorpCharType new].!

clob

	^self typeNamed: #clob ifAbsentPut: [GlorpClobType new typeString: 'clob'].!

compoundOperationFor: aSymbol
	"Return the platform specific version of a compound statement symbol"
	aSymbol == #EXCEPT ifTrue: [^'MINUS'].
	^aSymbol.!

createAlterColumnStatement: aField newType: aType usingExpression: expression

	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
	self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' MODIFY (';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll: aType typeString, ')'.

	^sqlStatementStream contents!

createAlterColumnStatement: aField setDefault: anObject
	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
	self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' MODIFY (';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll:  'DEFAULT';
		space.
	anObject printOn: sqlStatementStream.
	sqlStatementStream
		nextPutAll:  ')'.
	
	^sqlStatementStream contents!

createAlterColumnStatementDropDefault: aField
	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
		self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' MODIFY (';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll:  'DEFAULT NULL)'.

	^sqlStatementStream contents!

createAlterColumnStatementDropNotNull: aField
	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
		self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' MODIFY (';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll:  'NULL)'.

	
	^sqlStatementStream contents!

createAlterColumnStatementSetNotNull: aField
	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
		self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' MODIFY (';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll:  self sqlTextForNOTNULLAttributeConstraint;
		nextPutAll: ')'.

	
	^sqlStatementStream contents!

databaseSequenceClass
	^OracleSequence.!

date
	^self typeNamed: #date ifAbsentPut: [GlorpTimeStampType new typeString: 'date'].!

double
	^self float8!

endColumnAddString
	"The string we use as a suffix when adding a colum"
	^' ) '.!

float
	^self float4.!

float4
	^self typeNamed: #float4 ifAbsentPut: [GlorpFloatType new typeString: 'float'].!

float8
	^self typeNamed: #float8 ifAbsentPut: [GlorpDoubleType new typeString: 'double precision'].!

initializeFunctions
	| baseExists dbSpecificExists selectList baseNotExists dbSpecificNotExists |
	super initializeFunctions.

	"Oracle doesn't allow functions like EXISTS in a field list, but will allow a select there as an argument to a function, such as DECODE. So print it that way if we're in the select list."
	baseExists := functions at: #exists:.
	dbSpecificExists := DualRoleFunction new.
	dbSpecificExists function: '#exists:'.
	dbSpecificExists whereClauseVersion: baseExists.
	selectList := PrefixFunction named: 'DECODE'.
	selectList argumentModifierBlock: [:function :args | 
		function base: args first.
		Array with: 'x' asGlorpExpression with: 1 asGlorpExpression with: 0 asGlorpExpression].
	selectList type: baseExists type.
	dbSpecificExists selectListVersion: selectList.
	dbSpecificExists type: baseExists type.
	functions at: #exists: put: dbSpecificExists.

	"And similarly NOT EXISTS"
	baseNotExists := functions at: #notExists:.
	dbSpecificNotExists := DualRoleFunction new.
	dbSpecificNotExists function: '#notExists:'.
	dbSpecificNotExists whereClauseVersion: baseNotExists.
	selectList := PrefixFunction named: 'DECODE'.
	selectList argumentModifierBlock: [:function :args | 
		function base: args first.
		Array with: 'x' asGlorpExpression with: 0 asGlorpExpression with: 1 asGlorpExpression].
	selectList type: baseNotExists type.
	dbSpecificNotExists selectListVersion: selectList.
	dbSpecificNotExists type: baseExists type.
	functions at: #notExists: put: dbSpecificNotExists.!

initializeReservedWords
	super initializeReservedWords.
	reservedWords add: 'number'.!

int

	^self integer!

int2

	^self integer!

int4

	^self integer!

int8

	^self integer!

isOraclePlatform

	^true!

maximumLengthOfColumnName
	"^<Integer> I return the maximum length of a column name.  (I am not sent at present;  I am here so that if I and #maximumLengthOfTableName (which is sent) should ever differ on some database platform, I can be used in revised truncation logic.)"

	^30!

maximumLengthOfTableName
	"^<Integer> I return the max. length of a table name"

	^30!

maximumQueryValueSize
	"If there's a limit on how big a bound input parameter can be for a query, return it here. By default, no maximum. "
	^4001!

measuresStringFieldWidthsInBytes
	"Return true if we measure the width of a string in bytes rather than characters. So, a 255-character field can hold 255 bytes worth of string, not 255 characters. This is important with encodings."

	^true!

number

	^self typeNamed: #number ifAbsentPut: [GlorpNumericType new typeString: 'number'].!

primaryKeysAreAutomaticallyUnique
	"Return false if, in addition to specifying something as a primary key, we must separately specify it as unique"

	^true.!

printBlob: aByteArray on: aStream for: aType
	aByteArray isNil ifTrue: [^aStream nextPutAll: 'NULL'].
	aStream nextPutAll: 'HEXTORAW('''.
	aByteArray
		do:
			[:each | 
			each < 16 ifTrue: [aStream nextPut: $0].
			each
				printOn: aStream
				base: 16
				showRadix: false].
	aStream nextPutAll: ''')'!

printDate: aDate for: aType
	| stream |
	aDate isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPutAll: 'to_date('''.
	self
		printDate: aDate
		isoFormatOn: stream.
	stream nextPutAll: ''','''.
	stream nextPutAll: 'YYYY-MM-DD'.
	stream nextPutAll: ''')'.
	^stream contents.!

printNumericFieldOfSize: anInteger on: aStream

	aStream 
		nextPutAll: 'number('.
	anInteger printOn: aStream.
	aStream nextPutAll: ')'.!

printSqlStatementToListTablesInSchema: schemaString on: aStream
	"Not standardized"

	| lowerSchema |
	lowerSchema := (schemaString isNil or: [schemaString isEmpty])
		ifTrue: [schemaString]
		ifFalse: [schemaString asLowercase].
	aStream
		nextPutAll: 'SELECT table_name FROM all_tables '.
	(lowerSchema notNil and: [lowerSchema notEmpty]) ifTrue:
		[aStream 
			nextPutAll: 'WHERE lower(owner) = ';
			nextPut: $';
			nextPutAll: lowerSchema;
			nextPut: $']!

printSqlStatementToTestExistenceOfTable: tableName inSchema: schemaString on: aStream
	" This query will test only for tables that the logged on user has access to... "

	| lowerTableName lowerSchema |
	lowerTableName := tableName asLowercase.
	lowerSchema := schemaString isNil
		ifTrue: [schemaString]
		ifFalse: [schemaString asLowercase].
	aStream
		nextPutAll: 'SELECT count(*) FROM all_tables ';
		nextPutAll: 'WHERE lower(table_name) = ';
		nextPut: $';
		nextPutAll: lowerTableName;
		nextPut: $';
		nextPutAll: ' AND lower(owner) = ';
		nextPut: $';
		nextPutAll: lowerSchema;
		nextPut: $'!

printStringFieldOfSize: anInteger on: aStream

	aStream 
		nextPutAll: 'varchar2('.
	anInteger printOn: aStream.
	aStream nextPutAll: ')'.!

printTime: aTime for: aType
	| stream |
	aTime isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPutAll: 'to_date('''.
	self
		printTime: aTime
		isoFormatOn: stream.
	stream nextPutAll: ''','''.
	stream nextPutAll: 'HH24:MI:SS'.
	stream nextPutAll: ''')'.
	^stream contents.!

printTimestamp: aTimestamp on: stream for: aType

	aTimestamp isNil ifTrue: [stream nextPutAll: 'NULL'. ^self].
	stream nextPutAll: 'to_date('''.
	self
		printDate: aTimestamp
		isoFormatOn: stream.
	stream nextPut: $ .
	self
		printTime: aTimestamp
		isoFormatOn: stream
		milliseconds: false.
	stream nextPutAll: ''','''.
	stream nextPutAll: 'YYYY-MM-DD HH24:MI:SS'.
	stream nextPutAll: ''')'.!

raw
	^self typeNamed: #raw ifAbsentPut: [GlorpBlobType new typeString: 'raw'].!

requiresTransactionForTableOperations
	"Return true if we need a transaction in order to do table creation, deletion, and so forth. Some databases require it. Others don't like it"
	^false.!

sequence

	^self serial.!

serial

	^self typeNamed: #serial ifAbsentPut: [GlorpSerialType new typeString: (self integer typeString)].!

sqlTextCurrentServerUTCTimestamp

	^'SELECT SYS_EXTRACT_UTC(CURRENT_TIMESTAMP) FROM DUAL'!

startColumnAddString
	"The string we use as a prefix when adding a colum"
	^' ( '.!

supportsANSIJoins
	"Do we support the JOIN <tableName> USING <criteria> syntax. Currently hard-coded, but may also vary by database version"
	^false.!

supportsBinding
	"Binding works only with VW EXDI so far"
	^Dialect isVisualWorks.!

supportsGroupWritingFor: aCommand
	^self supportsBinding and: [self useBinding and: [aCommand supportsGroupWriting]].!

supportsMillisecondsInTimes
	^false.!

supportsTableOwners
	^true!

text
	^self clob.!

time
	^self timestamp.!

timestamp
	^self typeNamed: #timestamp ifAbsentPut: [GlorpTimeStampType new typeString: 'date'].!

timestampConverter
	^DelegatingDatabaseConverter
		named: #timestamp
		hostedBy: self
		fromStToDb: #asTimestamp:for:
		fromDbToSt: #readTimestamp:for:.!

useOracleOuterJoins
	"Return true if we use the old Oracle x = y (+) syntax for outer joins"
	^true.!

usesArrayBindingRatherThanGrouping
	"Return true if we use array binding for grouped writes rather than printing the sql multiple times. Only applies if we support grouped writes"
	^true.!

usesNullForEmptyStrings
	"Return true if this database is likely to use nil as an empty string value"
	^true.!

varchar
	^self typeNamed: #varchar ifAbsentPut: [GlorpVarCharType new typeName: 'varchar2'].!

vwEXDIConnectionClass
	"Prefer Threaded over not if nothing is loaded, but use whatever is loaded if there is anything"

	'OracleThreadedConnection' asQualifiedReference ifDefinedDo:
		[:class | ^class].
	'OracleConnection' asQualifiedReference ifDefinedDo:
		[:class | ^class].		
	^'OracleThreadedConnection' asQualifiedReference valueIfUndefinedLoadFrom: 'OracleThapiEXDI'! !
!OraclePlatform categoriesFor: #areSequencesExplicitlyCreated!constants!private! !
!OraclePlatform categoriesFor: #bindingsForGroupWritingFor:!binding!private! !
!OraclePlatform categoriesFor: #boolean!private!types! !
!OraclePlatform categoriesFor: #canUseInWhereClause:type:!private!testing! !
!OraclePlatform categoriesFor: #char!private!types! !
!OraclePlatform categoriesFor: #clob!private!types! !
!OraclePlatform categoriesFor: #compoundOperationFor:!database-specific!private! !
!OraclePlatform categoriesFor: #createAlterColumnStatement:newType:usingExpression:!private!services columns! !
!OraclePlatform categoriesFor: #createAlterColumnStatement:setDefault:!private!services columns! !
!OraclePlatform categoriesFor: #createAlterColumnStatementDropDefault:!private!services columns! !
!OraclePlatform categoriesFor: #createAlterColumnStatementDropNotNull:!private!services columns! !
!OraclePlatform categoriesFor: #createAlterColumnStatementSetNotNull:!private!services columns! !
!OraclePlatform categoriesFor: #databaseSequenceClass!private!sequences! !
!OraclePlatform categoriesFor: #date!private!types! !
!OraclePlatform categoriesFor: #double!private!types! !
!OraclePlatform categoriesFor: #endColumnAddString!constants!private! !
!OraclePlatform categoriesFor: #float!private!types! !
!OraclePlatform categoriesFor: #float4!private!types! !
!OraclePlatform categoriesFor: #float8!private!types! !
!OraclePlatform categoriesFor: #initializeFunctions!functions!private! !
!OraclePlatform categoriesFor: #initializeReservedWords!constants!private! !
!OraclePlatform categoriesFor: #int!private!types! !
!OraclePlatform categoriesFor: #int2!private!types! !
!OraclePlatform categoriesFor: #int4!private!types! !
!OraclePlatform categoriesFor: #int8!private!types! !
!OraclePlatform categoriesFor: #isOraclePlatform!private!SQL! !
!OraclePlatform categoriesFor: #maximumLengthOfColumnName!constants!public! !
!OraclePlatform categoriesFor: #maximumLengthOfTableName!constants!private! !
!OraclePlatform categoriesFor: #maximumQueryValueSize!private!testing! !
!OraclePlatform categoriesFor: #measuresStringFieldWidthsInBytes!public!testing! !
!OraclePlatform categoriesFor: #number!private!types! !
!OraclePlatform categoriesFor: #primaryKeysAreAutomaticallyUnique!constants!private! !
!OraclePlatform categoriesFor: #printBlob:on:for:!conversion-strings!public! !
!OraclePlatform categoriesFor: #printDate:for:!conversion-times!private! !
!OraclePlatform categoriesFor: #printNumericFieldOfSize:on:!private!table creation! !
!OraclePlatform categoriesFor: #printSqlStatementToListTablesInSchema:on:!public!services tables! !
!OraclePlatform categoriesFor: #printSqlStatementToTestExistenceOfTable:inSchema:on:!public!services tables! !
!OraclePlatform categoriesFor: #printStringFieldOfSize:on:!private!table creation! !
!OraclePlatform categoriesFor: #printTime:for:!conversion-times!private! !
!OraclePlatform categoriesFor: #printTimestamp:on:for:!conversion-times!private! !
!OraclePlatform categoriesFor: #raw!private!types! !
!OraclePlatform categoriesFor: #requiresTransactionForTableOperations!constants!private! !
!OraclePlatform categoriesFor: #sequence!private!types! !
!OraclePlatform categoriesFor: #serial!private!types! !
!OraclePlatform categoriesFor: #sqlTextCurrentServerUTCTimestamp!constants!public! !
!OraclePlatform categoriesFor: #startColumnAddString!constants!private! !
!OraclePlatform categoriesFor: #supportsANSIJoins!private!testing! !
!OraclePlatform categoriesFor: #supportsBinding!private!testing! !
!OraclePlatform categoriesFor: #supportsGroupWritingFor:!private!testing! !
!OraclePlatform categoriesFor: #supportsMillisecondsInTimes!constants!private! !
!OraclePlatform categoriesFor: #supportsTableOwners!private!testing! !
!OraclePlatform categoriesFor: #text!private!types! !
!OraclePlatform categoriesFor: #time!private!types! !
!OraclePlatform categoriesFor: #timestamp!private!types! !
!OraclePlatform categoriesFor: #timestampConverter!conversion-times!private! !
!OraclePlatform categoriesFor: #useOracleOuterJoins!private!testing! !
!OraclePlatform categoriesFor: #usesArrayBindingRatherThanGrouping!private!testing! !
!OraclePlatform categoriesFor: #usesNullForEmptyStrings!constants!private! !
!OraclePlatform categoriesFor: #varchar!private!types! !
!OraclePlatform categoriesFor: #vwEXDIConnectionClass!accessing!public! !

PostgreSQLPlatform guid: (GUID fromString: '{E0EE2551-E522-48D1-989F-8EBD13F0BEF1}')!
PostgreSQLPlatform comment: 'This is a platform for the open-source PostgreSQL database.
'!
!PostgreSQLPlatform categoriesForClass!Unclassified! !
!PostgreSQLPlatform methodsFor!

areSequencesExplicitlyCreated
	^true.!

bigint
	^self typeNamed: #bigint ifAbsentPut: [GlorpIntegerType new typeString: 'bigint'].!

bindingsForGroupWritingFor: aCommand
	"Return the bindings array for a group write. This can be in different formats, depending on the database and perhaps the mechanism in place."
	^aCommand batchStatementBindings.!

blob

	^self typeNamed: #blob ifAbsentPut: [GlorpBlobType new typeString: 'bytea'].!

boolean
	^self typeNamed: #boolean ifAbsentPut: [GlorpBooleanType new typeString: 'boolean'].!

byteArrayToSQLStringConverter

	^DelegatingDatabaseConverter
		named: #byteArrayToSQLString
		hostedBy: self
		fromStToDb: #convertByteArrayToSQLString:for:
		fromDbToSt: #convertSQLStringToByteArray:for:.!

canBind: aValue to: aType
	"The Postgres EXDI seems to have trouble binding date formats"
	aType = self date ifTrue: [^false].
	aType = self time ifTrue: [^false].
	aType = self timestamp ifTrue: [^false].
	aType = self timetz ifTrue: [^false].
	(Dialect isVisualWorks and: [aValue class == Dialect doublePrecisionFloatClass]) ifTrue: [^false].
	(Dialect isVisualWorks and: [aValue class == Dialect fixedPointClass]) ifTrue: [^false].
	^super canBind: aValue to: aType.!

char
	^self typeNamed: #char ifAbsentPut: [GlorpCharType new typeName: 'character'].!

charactersThatNeedEscaping
	^#($' $\)!

clob

	^self typeNamed: #clob ifAbsentPut: [GlorpClobType new typeString: 'text'].!

convertByteArrayToString: aByteArray for: aType
	| aStream | 
	aByteArray isNil ifTrue: [^nil].
	aStream := (ByteArray new: aByteArray size) writeStream.
	Base64Codec decodeFrom: aByteArray readStream onto: aStream.
	^aStream contents asString!

convertDBBooleanToBoolean: aBoolean for: aType
	aBoolean = '1' ifTrue: [^true].
	aBoolean = '0' ifTrue: [^false].
	^aBoolean.!

convertSQLStringToByteArray: aString for: aType
	| aStream | 
	aString isNil ifTrue: [^nil].
	aStream := (ByteArray new: aString size) writeStream.
	Base64Codec decodeFrom: aString readStream onto: aStream.
	^aStream contents!

databaseSequenceClass
	^PGSequence.!

date
	^self typeNamed: #date ifAbsentPut: [GlorpDateType new typeString: 'date'].!

decimal
	^self numeric.!

double
	^self float8!

float
	^self float4.!

float4
	^self typeNamed: #float4 ifAbsentPut: [GlorpFloatType new].!

float8
	^self typeNamed: #float8 ifAbsentPut: [GlorpDoubleType new].!

int
	^self int4!

int2
	^self typeNamed: #int2 ifAbsentPut: [GlorpIntegerType new typeString: 'int2'].!

int4
	^self typeNamed: #int4 ifAbsentPut: [GlorpIntegerType new typeString: 'int4'].!

int8
	^self typeNamed: #int8 ifAbsentPut: [GlorpIntegerType new typeString: 'int8'].!

isPostgreSQLPlatform
	^true.!

maximumLengthOfColumnName
	"^<Integer> I return the maximum length of a column name.  (I am not sent at present;  I am here so that if I and #maximumLengthOfTableName (which is sent) should ever differ on some database platform, I can be used in revised truncation logic.)"

	^31!

maximumLengthOfTableName
	"^<Integer> I return the max. length of a table name"

	^31!

measuresStringFieldWidthsInBytes
	"Return true if we measure the width of a string in bytes rather than characters. So, a 255-character field can hold 255 bytes worth of string, not 255 characters. This is important with encodings"
	^true.!

numeric
	^self typeNamed: #numeric ifAbsentPut: [GlorpNumericType new].!

printBlob: aByteArray on: aStream for: aType
	| inStream outStream |
	aByteArray isNil ifTrue: [^aStream nextPutAll: 'NULL'].
	aStream nextPut: $'.
	inStream := aByteArray readStream.
	outStream := WriteStream on: String new.
	Base64Codec encodeFrom: inStream onto: outStream.
	aStream nextPutAll: (outStream contents copyWithoutAll: String lineDelimiter).
	aStream nextPut: $'!

printDate: aDate for: aType
	"Print a date (or timestamp) as yyyy-mm-dd"
	| stream |
	aDate isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPut: $'.
	self
		printDate: aDate
		isoFormatOn: stream.
	stream nextPut: $'.
	^stream contents.!

printOffset: anInteger on: aCommand
	aCommand nextPutAll: ' OFFSET '.
	aCommand nextPutAll: anInteger printString!

printPostLimit: anInteger on: aCommand
	aCommand nextPutAll: ' LIMIT '.
	aCommand nextPutAll: anInteger printString!

printTime: aTime for: aType
	"Print a time (or timestamp) as hh:mm:ss.fff"
	| stream |
	aTime isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPut: $'.
	self
		printTime: aTime
		isoFormatOn: stream.
	stream nextPut: $'.
	^stream contents.!

printTimestamp: aTimestamp on: stream for: aType
	aTimestamp isNil ifTrue: [stream nextPutAll: 'NULL'. ^self].
	stream nextPut: $'.
	self
		printDate: aTimestamp
		isoFormatOn: stream.
	stream nextPutAll: ' '.
	self
		printTime: aTimestamp
		isoFormatOn: stream.
	stream nextPut: $'.!

real
	^self float4!

requiresCastsForNullsInUnions
	"If we have explicit nulls in a query which is unioned, do we need to cast them to the appropriate type in order to keep the database happy."
	^true.!

sequence

	^self serial!

serial
	"For postgresql, we use sequences, and explicitly get the values ourselves, so just tell the database that they're integers."
	^self typeNamed: #serial ifAbsentPut: [GlorpSerialType new typeString: 'integer'].!

smallint
	^self int2!

sqlTextCurrentServerUTCTimestamp

	^'SELECT CURRENT_TIMESTAMP AT TIME ZONE ''UTC'' '!

squeakPrintBlob: aByteArray on: aStream for: aType

	aStream nextPut: $'.
	aByteArray do: [:each |
		aStream nextPutAll: '\\'.
		each asInteger printOn: aStream paddedWith: $0 to: 3 base: 8].
	aStream nextPutAll: '''::bytea'.!

supportsANSIJoins
	"Do we support the JOIN <tableName> USING <criteria> syntax."
	^true.!

supportsBinding
	"Binding works only with VW EXDI so far"
	^Dialect isVisualWorks.!

supportsCaseInsensitiveLike
	^true.!

supportsGroupWritingFor: aCommand
	^aCommand supportsGroupWriting.!

supportsLimit
	"Do we support anything analogous to the postgresql LIMIT, returning only the first N rows"
	^true.!

supportsMillisecondsInTimes
	"I think this depends on settings, but it's easier right now to just turn it off"
	^false.!

supportsOffset
	"Do we support anything analogous to the postgresql OFFSET, skipping the first N rows"
	^true.!

supportsSchemaForLogin

	^false!

tableNameFor: aStringRepresentingATableName
	"Return the table name, as modified to be used in a database statement."
	
	^(aStringRepresentingATableName
		detect: [:each | each isUppercase]
		ifNone: [nil]) isNil
	ifTrue: [aStringRepresentingATableName]
	ifFalse: ['"' , aStringRepresentingATableName , '"'].!

time
	^self typeNamed: #time ifAbsentPut: [GlorpTimeType new typeString: 'time'].!

timestamp
	^self typeNamed: #timestamp ifAbsentPut: [GlorpTimeStampType new typeString: 'timestamp without time zone'].!

timeStampTypeString
	^'timestamp'.!

timetz
	^self typeNamed: #timetz ifAbsentPut: [GlorpPGTimeWithTimeZoneType new].!

typesWithValues
	super typesWithValues.
	self boolean.
	self bigint.
	self date.
	self decimal.
	self double.
	self float.
	self float4.
	self float8.
	self int2.
	self int.
	self integer.
	self numeric.
	self real.
	self sequence.
	self serial.
	self timetz.
	^self types.!

usesArrayBindingRatherThanGrouping
	"Return true if we use array binding for grouped writes rather than printing the sql multiple times. Only applies if we support grouped writes"
	^false.!

usesInformationSchema

	^true!

varchar
	^self typeNamed: #varchar ifAbsentPut: [GlorpVarCharType new typeName: 'character varying'].!

vwEXDIConnectionClass
	^'PostgreSQLEXDIConnection' asQualifiedReference valueIfUndefinedLoadFrom: 'PostgreSQLEXDI'! !
!PostgreSQLPlatform categoriesFor: #areSequencesExplicitlyCreated!constants!private! !
!PostgreSQLPlatform categoriesFor: #bigint!private!types! !
!PostgreSQLPlatform categoriesFor: #bindingsForGroupWritingFor:!binding!private! !
!PostgreSQLPlatform categoriesFor: #blob!private!types! !
!PostgreSQLPlatform categoriesFor: #boolean!private!types! !
!PostgreSQLPlatform categoriesFor: #byteArrayToSQLStringConverter!conversion-strings!private! !
!PostgreSQLPlatform categoriesFor: #canBind:to:!private!testing! !
!PostgreSQLPlatform categoriesFor: #char!private!types! !
!PostgreSQLPlatform categoriesFor: #charactersThatNeedEscaping!constants!private! !
!PostgreSQLPlatform categoriesFor: #clob!private!types! !
!PostgreSQLPlatform categoriesFor: #convertByteArrayToString:for:!As yet unclassified!private! !
!PostgreSQLPlatform categoriesFor: #convertDBBooleanToBoolean:for:!As yet unclassified!private! !
!PostgreSQLPlatform categoriesFor: #convertSQLStringToByteArray:for:!conversion-strings!private! !
!PostgreSQLPlatform categoriesFor: #databaseSequenceClass!private!sequences! !
!PostgreSQLPlatform categoriesFor: #date!private!types! !
!PostgreSQLPlatform categoriesFor: #decimal!private!types! !
!PostgreSQLPlatform categoriesFor: #double!private!types! !
!PostgreSQLPlatform categoriesFor: #float!private!types! !
!PostgreSQLPlatform categoriesFor: #float4!private!types! !
!PostgreSQLPlatform categoriesFor: #float8!private!types! !
!PostgreSQLPlatform categoriesFor: #int!private!types! !
!PostgreSQLPlatform categoriesFor: #int2!private!types! !
!PostgreSQLPlatform categoriesFor: #int4!private!types! !
!PostgreSQLPlatform categoriesFor: #int8!private!types! !
!PostgreSQLPlatform categoriesFor: #isPostgreSQLPlatform!private!SQL! !
!PostgreSQLPlatform categoriesFor: #maximumLengthOfColumnName!constants!public! !
!PostgreSQLPlatform categoriesFor: #maximumLengthOfTableName!constants!private! !
!PostgreSQLPlatform categoriesFor: #measuresStringFieldWidthsInBytes!private!testing! !
!PostgreSQLPlatform categoriesFor: #numeric!private!types! !
!PostgreSQLPlatform categoriesFor: #printBlob:on:for:!conversion-strings!private! !
!PostgreSQLPlatform categoriesFor: #printDate:for:!conversion-times!private! !
!PostgreSQLPlatform categoriesFor: #printOffset:on:!database-specific!private! !
!PostgreSQLPlatform categoriesFor: #printPostLimit:on:!database-specific!private! !
!PostgreSQLPlatform categoriesFor: #printTime:for:!conversion-times!private! !
!PostgreSQLPlatform categoriesFor: #printTimestamp:on:for:!conversion-times!private! !
!PostgreSQLPlatform categoriesFor: #real!private!types! !
!PostgreSQLPlatform categoriesFor: #requiresCastsForNullsInUnions!private!testing! !
!PostgreSQLPlatform categoriesFor: #sequence!private!types! !
!PostgreSQLPlatform categoriesFor: #serial!private!types! !
!PostgreSQLPlatform categoriesFor: #smallint!private!types! !
!PostgreSQLPlatform categoriesFor: #sqlTextCurrentServerUTCTimestamp!constants!public! !
!PostgreSQLPlatform categoriesFor: #squeakPrintBlob:on:for:!conversion-strings!private! !
!PostgreSQLPlatform categoriesFor: #supportsANSIJoins!private!testing! !
!PostgreSQLPlatform categoriesFor: #supportsBinding!private!testing! !
!PostgreSQLPlatform categoriesFor: #supportsCaseInsensitiveLike!private!testing! !
!PostgreSQLPlatform categoriesFor: #supportsGroupWritingFor:!private!testing! !
!PostgreSQLPlatform categoriesFor: #supportsLimit!private!testing! !
!PostgreSQLPlatform categoriesFor: #supportsMillisecondsInTimes!constants!private! !
!PostgreSQLPlatform categoriesFor: #supportsOffset!private!testing! !
!PostgreSQLPlatform categoriesFor: #supportsSchemaForLogin!public!testing! !
!PostgreSQLPlatform categoriesFor: #tableNameFor:!private!services tables! !
!PostgreSQLPlatform categoriesFor: #time!private!types! !
!PostgreSQLPlatform categoriesFor: #timestamp!private!types! !
!PostgreSQLPlatform categoriesFor: #timeStampTypeString!private!types! !
!PostgreSQLPlatform categoriesFor: #timetz!private!types! !
!PostgreSQLPlatform categoriesFor: #typesWithValues!private!type helpers! !
!PostgreSQLPlatform categoriesFor: #usesArrayBindingRatherThanGrouping!private!testing! !
!PostgreSQLPlatform categoriesFor: #usesInformationSchema!public!testing! !
!PostgreSQLPlatform categoriesFor: #varchar!private!types! !
!PostgreSQLPlatform categoriesFor: #vwEXDIConnectionClass!accessing!private! !

!PostgreSQLPlatform class methodsFor!

useBindingByDefault
	^false! !
!PostgreSQLPlatform class categoriesFor: #useBindingByDefault!public!testing! !

SQLite3Platform guid: (GUID fromString: '{4A7E8C6F-3D0F-484F-AC24-48E1090B66BE}')!
SQLite3Platform comment: 'I support Glorp''s use of the SQLite database;  see http://www.sqlite.org.  Some notes on the implementation:

SQLite provides no specific support for time-related data other than column-default-value UTC functions:
	CURRENT_DATE format YYYY-MM-DD
	CURRENT_TIME format HH:MM:SS
	CURRENT_TIMESTAMP format YYYY-MM-DD HH:MM:SS
I therefore map Date, Time and Timestamp to and from strings via #to... and (inherited) #read... methods whose formats agree with these functions.  (I also return false to #supportsMillisecondsInTimes, again in agreement with the above functions which do not include millliseconds.)

My #maximumLengthOfTableName value is limited purely for convenience;  it can probably be much larger.

In VW, this version uses the non-threaded SQLite3Connection EXDI class.  Make my #vwEXDIConnectionClass return SQLite3ThreadedConnection to make Glorp use the threaded interface and, if using this with Store, also change its string reference in #asGlorpDatabase, #databaseFor:.  Alternatively, create an SQLlite3ThreadedPlatform subclass of me and add an extra line, instead of changing a line, to the Store methods.  (Note that in order to be thread-safe, your SQLite installation must be compiled with the SQLITE_THREADSAFE preprocessor macro set to 1.  In SQLite3, both the Windows and Linux precompiled binaries in the distribution are compiled this way.  If you are unsure if the SQLite library you are linking against is compiled to be threadsafe you can call the sqlite3_threadsafe()  interface to find out.)

SQLite parses FOREIGN KEY constraints but does not enforce them.

SQLite''s approach to data typing is unusual for SQL.  Its data values are:
	NULL
	INTEGER. The value is a signed integer, stored in 1, 2, 3, 4, 6, or 8 bytes depending on the magnitude of the value.
	REAL. The value is a floating point value, stored as an 8-byte IEEE floating point number.
	TEXT. The value is a text string, stored using the database encoding (UTF-8, UTF-16BE or UTF-16-LE).
	BLOB. The value is a blob of data, stored exactly as it was input
but its typing is dynamic, so any column can in principle store any value, and its column affinities are
	TEXT - can store NULL, TEXT or BLOB
	NUMERIC - can store all five; text values that converts to reals or integers without loss of information are so stored
	INTEGER - as NUMERIC plus reals that convert to integer without loss of information are so stored
	REAL - as NUMERIC but integer values are coerced to reals
	NONE - no coercion is performed.
My int* and float*/double type methods that define Glorp types for SQLite conform to the expressibility of the above data values.

'!
!SQLite3Platform categoriesForClass!Unclassified! !
!SQLite3Platform methodsFor!

areSequencesExplicitlyCreated
	^false!

bindingsForGroupWritingFor: aCommand
	"Return the bindings array for a group write. This can be in different formats, depending on the database and perhaps the mechanism in place."

	^aCommand batchStatementBindings!

canBind: aValue to: aType
	"See #supportsVariableSizedNumerics comment."

	(Dialect isVisualWorks and: [aValue class == Dialect fixedPointClass]) ifTrue: [^false].
	^super canBind: aValue to: aType!

convertIntegerToBoolean: anInteger for: aType
	"This method accomodates an error in earlier SQLite 3 dlls. For the table_info() PRAGMA, 99 means True."

	anInteger isNil ifTrue: [^anInteger].
	anInteger >= 1 ifTrue: [^true].
	anInteger = 0 ifTrue: [^false].
	self error: 'invalid boolean conversion'!

createTableStatementStringFor: aGLORPDatabaseTable 
	"^<String> This method returns a string which can be used to create a database table ..."

	| sqlStatementStream tmpString fkeys |
	tmpString := 'create table'.
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll: (self capitalWritingOfSQLCommands 
					ifTrue: [tmpString asUppercase]
					ifFalse: [tmpString]);
		space.
	self printDDLTableNameFor: aGLORPDatabaseTable on: sqlStatementStream.

	"Now print the columns specification for each field in the table ..."
	self printColumnsSpecificationFor: aGLORPDatabaseTable
		on: sqlStatementStream.
	"Print the primary key constraint, unless using serial pkeys, since it would be a redundant declaration."
	(aGLORPDatabaseTable hasPrimaryKeyConstraints and: [aGLORPDatabaseTable primaryKeyFields first type isSerial not])
			ifTrue: [sqlStatementStream nextPutAll: ', '.
					(aGLORPDatabaseTable primaryKeyConstraints creationStringFor: self)
						do: [ :each | sqlStatementStream nextPutAll: each.]
						separatedBy: [sqlStatementStream nextPut: $,]].

	fkeys := aGLORPDatabaseTable foreignKeyConstraintsForCreation.
	fkeys notEmpty
			ifTrue: [sqlStatementStream nextPutAll: ', '.
					(fkeys collect: [:each| self createForeignKeyConstraintAddStatement: each])
						do: [ :each | sqlStatementStream nextPutAll: each first]
						separatedBy: [sqlStatementStream nextPut: $,]].

	sqlStatementStream
		nextPut: $).
	^sqlStatementStream contents!

databaseSequenceClass
	^SQLite3Sequence!

date
	"SQLite does not support any time-related types;  use text as the type and convert in readDate.../toDate..."

	^self typeNamed: #date ifAbsentPut: [GlorpDateType new typeString: 'date']!

double
	^self typeNamed: #double ifAbsentPut: [GlorpDoubleType new typeString: 'double' ].!

float4
	"The only kind of REAL that SQLite supports is an 8-byte IEEE floating point number, so we just map float4 to float8."

	^self double!

float8
	"The only kind of REAL that SQLite supports is an 8-byte IEEE floating point number."

	^self double!

int2
	^self smallint.!

int4
	^self typeNamed: #int4 ifAbsentPut: [GlorpIntegerType new typeString: 'int'].!

int8
	^self int4.!

isSQLite3Platform
	^true!

maximumLengthOfTableName
	"It is the boast of SQLite that their limits are only for stability purposes:  if it fits in 32 bits then they will handle it.  Since the maximum length of an SQL statement is limited to SQLITE_MAX_LENGTH (defaults to 1000000, cannot be higher than 1073741824), table names had better be shorter and in normal use should be much shorter.  I chose the number below at random;  change it if it causes you any inconvenience."

	^128!

nvarchar
	^self text!

parenthesizeCommandsInCompoundQuery
	^false!

printDate: aDateString for: aType
	"SQLite does not suppport any time-related types.  We convert them all to strings via the to... methods, so parameters to this method should be Strings of the form '1959-09-30'."

	aDateString isNil ifTrue: [^'NULL'].
	"The next line is checking code and can be dropped when SQLite is sufficiently tested."
	aDateString class = Date ifTrue: [self error: 'SQLite does not support date types;  this should have been a String'].
	^aDateString printString!

printSqlStatementToListTablesInSchema: schemaString on: aStream
	"SQLite3 uses a readonly system table "

	aStream
		nextPutAll: 'SELECT name as table_name FROM sqlite_master ';
		nextPutAll: 'WHERE type = ';
		nextPut: $';
		nextPutAll: 'table';
		nextPut: $';
		nextPut: $;!

printSqlStatementToTestExistenceOfTable: tableName inSchema: schemaString on: aStream
	"SQLite3 uses a readonly system table and doesn't support schemas, as far as I know"

	| lowerTableName |
	lowerTableName := tableName asLowercase.
	aStream
		nextPutAll: 'SELECT count(*) FROM sqlite_master ';
		nextPutAll: 'WHERE lower(name) = ';
		nextPut: $';
		nextPutAll: lowerTableName;
		nextPut: $';
		nextPutAll: ' AND type = ';
		nextPut: $';
		nextPutAll: 'table';
		nextPut: $';
		nextPut: $;!

printTime: aTimeString for: aType
	"SQLite does not suppport any time-related types.  We convert them all to strings via the to... methods, so parameters to this method should be Strings of the form 10:20:23."

	aTimeString isNil ifTrue: [^'NULL'].
	"The next line is checking code and can be dropped when SQLite is sufficiently tested."
	aTimeString class = Time ifTrue: [self error: 'SQLite does not support time types;  this should have been a String'].
	^aTimeString printString!

printTimestamp: aTimestampString on: stream for: aType
	"SQLite does not suppport any time-related types.  We convert them all to strings via the to... methods, so parameters to this method should be Strings of the form '1959-03-27 10:20:23'."

	aTimestampString isNil ifTrue: [stream nextPutAll: 'NULL'. ^self].
	"The next line is checking code and can be dropped when SQLite is sufficiently tested."
	aTimestampString class = TimeStamp ifTrue: [self error: 'SQLite does not support timestamp types;  this should have been a String'].
	stream nextPut: $'.
	stream nextPutAll: aTimestampString.
	stream nextPut: $'.!

requiresTransactionForTableOperations
	^false!

sequence
	^self serial!

serial
	"In SQLLite, the column declared as INTEGER PRIMARY KEY will default to ROWID, unless the inserted value is not null. But in Sqlite3 a declaration like: INTEGER PRIMARY KEY AUTOINCREMENT will cause ROWID to be fetched from the sqlite_sequence table, which provides always increasing values."

	^self typeNamed: #serial ifAbsentPut: [GlorpSerialType new typeString: 'INTEGER PRIMARY KEY AUTOINCREMENT'].!

supportsAlterTableForConstraints

	^false!

supportsANSIJoins
	"Do we support the JOIN <tableName> USING <criteria> syntax."
	^true.!

supportsAuthentication
	"SQLite does not support authentication:  anyone can login with any username and password.  The only security is that provided by the OS for the file being used as the database."

	^false!

supportsBinding
	"Return true if this platform supports binding parameters rather than printing them as strings into the SQL statement"
	^true.!

supportsConstraints
	"Return false, because this platform does not support constraint add or drop statements.
	However, foreign key constraints can be created as part of the create table statement.
	Also, a foreign key constraint can be created in the statement that adds that column to
	the table. (Eg., ALTER TABLE ADD COLUMN ...)."

	^false!

supportsMillisecondsInTimes
	"The only time-type-related functions in SQLite are CURRENT_TIME format HH:MM:SS and CURRENT_TIMESTAMP format YYYY-MM-DD HH:MM:SS, neither of which support milliseconds.  As against that, since we map to and from strings, we can easily include milliseconds in these strings.  For now, I go with the SQLite function choice and say no, but I have verified that tests work fine when I set this to true."

	^false!

supportsSchemaForLogin

	^false!

supportsSchemas
	^false!

supportsStringQueryOnBlobs
	"This is something of a guess but I see no evidence that SQLite does and the relevant test certainly fails."

	^false!

supportsVariableSizedNumerics
	"SQLite supports a single 8-byte IEEE floating point type:  e.g. give it 321.12s and it will return 321.12d.  Perhaps clever Glorp converters could arrange for Glorp to know which fields are really FixedPoint numbers, as we do for Date, Time and Timestamp (which SQLite also does not support) but I'll leave that until there's demand for it (ee also #canBind:to:)."

	^false!

text
	^self typeNamed: #nvarchar ifAbsentPut: [GlorpTextType new typeString: 'text']!

time
	"SQLite does not support any time-related types;  use text as the type and convert in readTime.../toTime..."

	^self typeNamed: #time ifAbsentPut: [GlorpTimeType new typeString: 'time']!

timestamp
	"SQLite does not support any time-related types;  use text as the type and convert in readTimestamp.../toTimestamp..."

	^self typeNamed: #timestamp ifAbsentPut: [GlorpTimeStampType new typeString: 'timestamp']!

toDate: anObject for: aType
	"SQLite does not handle timestamps;  all such must be treated as strings and the to... and read... formats must agree.  SQLite does use column-default-value UTC functions:
	CURRENT_DATE format YYYY-MM-DD
	CURRENT_TIME format HH:MM:SS
	CURRENT_TIMESTAMP format YYYY-MM-DD HH:MM:SS
so our string conversions must agree with these formats."

	| stream |
	anObject isNil ifTrue: [^nil].
	stream := String new writeStream.
	self
		printDate: (super toDate: anObject for: aType)
		isoFormatOn: stream.
	^stream contents!

toTime: anObject for: aType
	"SQLite does not handle timestamps;  all such must be treated as strings and the to... and read... formats must agree.  SQLite does use column-default-value UTC functions:
	CURRENT_DATE format YYYY-MM-DD
	CURRENT_TIME format HH:MM:SS
	CURRENT_TIMESTAMP format YYYY-MM-DD HH:MM:SS
so our string conversions must agree with these formats."

	| stream |
	anObject isNil ifTrue: [^nil].
	stream := String new writeStream.
	self
		printTime: (super toTime: anObject for: aType)
		isoFormatOn: stream.
	^stream contents!

toTimestamp: anObject for: aType
	"SQLite does not handle timestamps;  all such must be treated as strings and the to... and read... formats must agree.  SQLite does use column-default-value UTC functions:
	CURRENT_DATE format YYYY-MM-DD
	CURRENT_TIME format HH:MM:SS
	CURRENT_TIMESTAMP format YYYY-MM-DD HH:MM:SS
so our string conversions must agree with these formats."

	| stream timestampObject |
	anObject isNil ifTrue: [^nil].
	stream := String new writeStream.
	timestampObject := super toTimestamp: anObject for: aType.
	self
		printDate: timestampObject
		isoFormatOn: stream.
	stream space.
	self
		printTime: timestampObject
		isoFormatOn: stream.
	^stream contents!

typeWithDatabaseName: aString characterWidth: aNumber
	"Since this is SQLite, let the type names be more flexible."

	| aTypeName total block |
	block := [:sum :name | sum + (aString asLowercase findString: name startingAt: 1)].
	aTypeName := aString.
	total := #('char' 'clob' 'text') inject: 0 into: block.
	total > 0
		ifTrue: [aTypeName := 'text']
		ifFalse: 
			[total := #('num' 'double' 'float') inject: 0 into: block.
			total > 0
				ifTrue: [aTypeName := 'numeric']
				ifFalse: 
					[total := #('int') inject: 0 into: block.
					total > 0
						ifTrue: [aTypeName := 'integer']
						ifFalse: 
							[total := #('blob') inject: 0 into: block.
							total > 0 ifTrue: [aTypeName := 'blob']]]].
	aString isEmpty ifTrue: [aTypeName := 'blob'].
	^super typeWithDatabaseName: aTypeName characterWidth: aNumber!

usesArrayBindingRatherThanGrouping
	^false.!

usesIdentityColumns
	^true.!

varbinary
	" *** This method was defined by Glorp.DatabasePlatform as a subclass responsibility.
	Replace its body with a proper implementation. *** "

	self error: 'Subclass responsibility stub not reimplemented'!

varchar
	^self text.!

vwEXDIConnectionClass
	^'SQLite3Connection' asQualifiedReference valueIfUndefinedLoadFrom: 'SQLite3EXDI'! !
!SQLite3Platform categoriesFor: #areSequencesExplicitlyCreated!constants!private! !
!SQLite3Platform categoriesFor: #bindingsForGroupWritingFor:!binding!private! !
!SQLite3Platform categoriesFor: #canBind:to:!constants!private! !
!SQLite3Platform categoriesFor: #convertIntegerToBoolean:for:!conversion-boolean!public! !
!SQLite3Platform categoriesFor: #createTableStatementStringFor:!public!services tables! !
!SQLite3Platform categoriesFor: #databaseSequenceClass!private!sequences! !
!SQLite3Platform categoriesFor: #date!private!types! !
!SQLite3Platform categoriesFor: #double!private!types! !
!SQLite3Platform categoriesFor: #float4!private!types! !
!SQLite3Platform categoriesFor: #float8!private!types! !
!SQLite3Platform categoriesFor: #int2!private!types! !
!SQLite3Platform categoriesFor: #int4!private!types! !
!SQLite3Platform categoriesFor: #int8!private!types! !
!SQLite3Platform categoriesFor: #isSQLite3Platform!public!testing! !
!SQLite3Platform categoriesFor: #maximumLengthOfTableName!constants!private! !
!SQLite3Platform categoriesFor: #nvarchar!private!types! !
!SQLite3Platform categoriesFor: #parenthesizeCommandsInCompoundQuery!constants!private! !
!SQLite3Platform categoriesFor: #printDate:for:!conversion-times!private! !
!SQLite3Platform categoriesFor: #printSqlStatementToListTablesInSchema:on:!public!services tables! !
!SQLite3Platform categoriesFor: #printSqlStatementToTestExistenceOfTable:inSchema:on:!public!services tables! !
!SQLite3Platform categoriesFor: #printTime:for:!conversion-times!private! !
!SQLite3Platform categoriesFor: #printTimestamp:on:for:!conversion-times!private! !
!SQLite3Platform categoriesFor: #requiresTransactionForTableOperations!private!testing! !
!SQLite3Platform categoriesFor: #sequence!private!types! !
!SQLite3Platform categoriesFor: #serial!public!types! !
!SQLite3Platform categoriesFor: #supportsAlterTableForConstraints!public!testing! !
!SQLite3Platform categoriesFor: #supportsANSIJoins!private!testing! !
!SQLite3Platform categoriesFor: #supportsAuthentication!constants!private! !
!SQLite3Platform categoriesFor: #supportsBinding!private!testing! !
!SQLite3Platform categoriesFor: #supportsConstraints!constants!public! !
!SQLite3Platform categoriesFor: #supportsMillisecondsInTimes!constants!private! !
!SQLite3Platform categoriesFor: #supportsSchemaForLogin!public!testing! !
!SQLite3Platform categoriesFor: #supportsSchemas!public!testing! !
!SQLite3Platform categoriesFor: #supportsStringQueryOnBlobs!private!testing! !
!SQLite3Platform categoriesFor: #supportsVariableSizedNumerics!constants!private! !
!SQLite3Platform categoriesFor: #text!private!types! !
!SQLite3Platform categoriesFor: #time!private!types! !
!SQLite3Platform categoriesFor: #timestamp!private!types! !
!SQLite3Platform categoriesFor: #toDate:for:!conversion-times!private! !
!SQLite3Platform categoriesFor: #toTime:for:!conversion-times!private! !
!SQLite3Platform categoriesFor: #toTimestamp:for:!conversion-times!private! !
!SQLite3Platform categoriesFor: #typeWithDatabaseName:characterWidth:!public!types! !
!SQLite3Platform categoriesFor: #usesArrayBindingRatherThanGrouping!private!testing! !
!SQLite3Platform categoriesFor: #usesIdentityColumns!private!testing! !
!SQLite3Platform categoriesFor: #varbinary!private!types! !
!SQLite3Platform categoriesFor: #varchar!private!types! !
!SQLite3Platform categoriesFor: #vwEXDIConnectionClass!accessing!private! !

SQLServerPlatform guid: (GUID fromString: '{6B68FE51-7C39-4C62-9673-F7CCF2EBD729}')!
SQLServerPlatform comment: 'This is a platform for the Microsoft SQL Server database.'!
!SQLServerPlatform categoriesForClass!Unclassified! !
!SQLServerPlatform methodsFor!

areSequencesExplicitlyCreated

^false!

bindingsForGroupWritingFor: aCommand
	"Return the bindings array for a group write. This can be in different formats, depending on the database and perhaps the mechanism in place."
	^aCommand batchStatementBindings.!

bit
	^self typeNamed: #bit ifAbsentPut: [GlorpBooleanType new typeString: 'bit'].!

blob

	^self typeNamed: #blob ifAbsentPut: [GlorpBlobType new typeString: 'varbinary(max)'; queryType: (self varbinary)].!

boolean
	^self bit.!

booleanToBooleanConverter
	^DelegatingDatabaseConverter
		named: #booleanToBoolean
		hostedBy: self
		fromStToDb: #convertBooleanToInteger:for:
		fromDbToSt: #convertDBBooleanToBoolean:for:.!

capitalWritingOfTableName
	"^<Boolean>"

	^false!

char
	^self typeNamed: #char ifAbsentPut: [GlorpCharType new].!

charactersThatNeedEscaping
	"There seem to be all kind of contradictory bits of information about what sql server does/requires for escaped characters, all of which differ from standard sql. Empirically the only thing that requires escaping appears to be single quote"
	^#($' ).!

clob

	^self typeNamed: #clob ifAbsentPut: [GlorpClobType new typeString: 'varchar(max)'].!

compoundOperationFor: aSymbol
	"Return the platform specific version of a compound statement symbol"
	aSymbol == #INTERSECT ifTrue: [^'WHERE EXISTS'].
	aSymbol == #MINUS ifTrue: [^'WHERE NOT EXISTS'].
	^aSymbol.!

createAlterColumnStatement: aField newType: aType usingExpression: expression

	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
	self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' ALTER COLUMN';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll: aType typeString.
	expression ifNotNil: [ sqlStatementStream nextPutAll: ' USING ', expression ].

	^sqlStatementStream contents!

createAlterColumnStatement: aField setDefault: anObject
	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
	self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' ALTER COLUMN';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll:  'DEFAULT ';
		space.
	anObject printOn: sqlStatementStream.
	sqlStatementStream
		nextPutAll:  ' WITH VALUES'.
	
	^sqlStatementStream contents!

createAlterColumnStatementDropDefault: aField
	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
		self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' ALTER COLUMN';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll:  'DEFAULT NULL'.

	^sqlStatementStream contents!

createAlterColumnStatementDropNotNull: aField
	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
		self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' ALTER COLUMN';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll: aField type typeString;
		space;
		nextPutAll:  'NULL'.

	
	^sqlStatementStream contents!

createAlterColumnStatementSetNotNull: aField
	| sqlStatementStream |
	sqlStatementStream := WriteStream on: String new.
	sqlStatementStream
		nextPutAll:  'ALTER TABLE';
		space.
		self printDDLTableNameFor: aField table on: sqlStatementStream.

	sqlStatementStream
		nextPutAll:  ' ALTER COLUMN';
		space;
		nextPutAll: (self nameForColumn: aField name);
		space;
		nextPutAll: aField type typeString;
		space;
		nextPutAll:  self sqlTextForNOTNULLAttributeConstraint.

	
	^sqlStatementStream contents!

databaseSequenceClass
	"My AccessPlatform subclass has a hardcoded check in StoreDescriptorSystem>>sequenceFor: to ignore this method in favour of calling #justSelectTheMaximumCurrentValueAndAddOneSequenceFor: for AccessPlatform.  Can we retire that?"

	^SQLServerSequence!

date
	"SQL Server does not compare a timestamp that's cast to a date as being equal to a date, which is the behaviour we expect. Its dates are really timestamps internally, so just treat date and timestamp as the same."
	^self timestamp.!

dateConverter
	"SQL server doesn't have plain dates, and doesn't accept them"
	^DelegatingDatabaseConverter
		named: #date
		hostedBy: self
		fromStToDb: #dateToTimestampConversion:for:
		fromDbToSt: #readDate:for:.	"#printDate:for:"!

dateToTimestampConversion: aDate for: aType
	aDate isNil ifTrue: [^aDate].
	^aDate asTimestamp.!

decimal
	^self numeric.!

double
	^self float.!

escapeFor: aCharacter
	^String with: $' with: aCharacter.
"	^'\', (aCharacter asInteger printStringRadix: 16)."!

exdiTypeForDates
	^#Timestamp.!

float

	^self typeNamed: #float ifAbsentPut: [GlorpMSSQLFloatType new].!

float4
	^self float.!

float8
	^self float.!

initializeFunctions
	| baseExists dbSpecificExists selectList baseNotExists dbSpecificNotExists |
	super initializeFunctions.
	functions
		at: #, put: (InfixFunction named: '+');
		at: #copyFrom:to: put: (SubstringFunction named: 'SUBSTRING');
		at: #objectId put: (PrefixFunction named: 'OBJECT_ID');
		at: #columnPropertySchema:property: put: ((PrefixFunction named: 'COLUMNPROPERTY') type: self integer);
		at: #convertTo: put: ((PrefixFunction named: 'CONVERT')). "CONVERT(toType, aThing)"

	"Like Oracle, SQL Server doesn't allow functions like EXISTS in a field list,
	but will allow a select there as an argument to a function, such as CASE.
	So print it that way if we're in the select list."

	baseExists := functions at: #exists:.
	dbSpecificExists := DualRoleFunction new.
	dbSpecificExists function: '#exists:'.
	dbSpecificExists whereClauseVersion: baseExists.
	selectList := Case new argumentTypes: (Array with: (self varchar: 1) with: self integer with: self integer).
	selectList argumentModifierBlock: [:function :args | 
		function base: args first.
		Array with: 'x' asGlorpExpression with: 1 asGlorpExpression with: 0 asGlorpExpression].
	dbSpecificExists selectListVersion: selectList.
	dbSpecificExists type: self integer.
	functions at: #exists: put: dbSpecificExists.

	"And similarly NOT EXISTS"
	baseNotExists := functions at: #notExists:.
	dbSpecificNotExists := DualRoleFunction new.
	dbSpecificNotExists function: '#notExists:'.
	dbSpecificNotExists whereClauseVersion: baseNotExists.
	selectList := Case new argumentTypes: (Array with: (self varchar: 1) with: self integer with: self integer).
	selectList argumentModifierBlock: [:function :args | 
		function base: args first.
		Array with: 'x' asGlorpExpression with: 0 asGlorpExpression with: 1 asGlorpExpression].
	dbSpecificNotExists selectListVersion: selectList.
	dbSpecificNotExists type: self integer.
	functions at: #notExists: put: dbSpecificNotExists.!

initializeReservedWords
	super initializeReservedWords.
	reservedWords add: 'key'!

int
	^self integer.!

int2
	^self smallint.!

int4

	^self integer.!

int8

	^self numeric.!

integer

	^self typeNamed: #integer ifAbsentPut: [GlorpIntegerType new typeString: 'int'].!

isODBCPlatform

	^true!

isSQLServerPlatform

	^true!

maximumLengthOfColumnName
	"^<Integer> I return the maximum length of a column name.  (I am not sent at present;  I am here so that if I and #maximumLengthOfTableName (which is sent) should ever differ on some database platform, I can be used in revised truncation logic.)"

	^128!

maximumLengthOfTableName
	"^<Integer> I return the max. length of a table name"

	^128!

maximumSizeToGroupWriteFor: aCollectionOfDatabaseRows
	"If we are going to group write, how many rows of this collection should we do it for at once"
	^aCollectionOfDatabaseRows size min: 250.!

nclob

	^self typeNamed: #nclob ifAbsentPut: [GlorpClobType new typeString: 'nvarchar(max)'].!

numeric
	^self typeNamed: #numeric ifAbsentPut: [GlorpNumericType new].!

nvarchar
	^self typeNamed: #nvarchar ifAbsentPut: [GlorpVarCharType new typeString: 'nvarchar'].!

printBlob: aByteArray on: aStream for: aType
	aByteArray isNil ifTrue: [^aStream nextPutAll: 'NULL'].
	aStream nextPutAll: '0x'.
	aByteArray
		do:
			[:each | 
			each < 16 ifTrue: [aStream nextPut: $0].
			each
				printOn: aStream
				base: 16
				showRadix: false]!

printBoolean: aBoolean for: aType
	"If the platform is MS SQL Server, a boolean object in a SELECT needs to be converted to  BIT type, since T, Y, or true make SQLServer complain. "
	
	| string |
	aBoolean isNil ifTrue: [^'NULL'].
	"We may be getting passed something that's already 1 or 0, or something else instead of a boolean. That probably shouldn't be happening, but let's handle it gracefully. We define gracefully to mean 1 is true, and everything else is false"
	string := (aBoolean == 1 or: [aBoolean = true]) 
		ifTrue: ['1'] 
		ifFalse: ['0'].
	^'CONVERT( BIT, ' , string , ' )'.!

printDate: aDate for: aType
	"Print a date (or timestamp) as yyyy-mm-dd"
	| stream |
	aDate isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPutAll: '{ d '''.
	self
		printDate: aDate
		isoFormatOn: stream.
	stream nextPutAll: ''' }'.
	^stream contents.!

printPreLimit: anInteger on: aCommand
	aCommand nextPutAll: ' TOP '.
	aCommand nextPutAll: anInteger printString.
	aCommand nextPutAll: ' '!

printSqlStatementToListTablesInSchema: schemaString on: aStream
	"Not standardized"

	| lowerSchema |
	self usesInformationSchema
		ifFalse: [^self subclassResponsibility].

	lowerSchema := (schemaString isNil or: [schemaString isEmpty])
		ifTrue: ['']
		ifFalse: [schemaString asLowercase , '.'].
	aStream
		nextPutAll: 'SELECT table_name FROM ' ;
		nextPutAll: lowerSchema;
		nextPutAll: 'INFORMATION_SCHEMA.TABLES ';
		nextPut: $;!

printSqlStatementToTestExistenceOfTable: tableName inSchema: schemaString on: aStream
	" This query will test only for tables that the logged on user has access to... "

	| lowerTableName lowerSchema |
	lowerTableName := tableName asLowercase.
	lowerSchema := schemaString isNil
		ifTrue: ['']
		ifFalse: [schemaString asLowercase, '.'].
	aStream
		nextPutAll: 'SELECT count(*) FROM ' ;
		nextPutAll: lowerSchema;
		nextPutAll: 'INFORMATION_SCHEMA.TABLES ';
		nextPutAll: 'WHERE lower(table_name) = ';
		nextPut: $';
		nextPutAll: lowerTableName;
		nextPut: $';
		nextPut: $;!

printTime: aTime for: aType
	"Print a time (or timestamp) as hh:mm:ss.fff"
	| stream |
	aTime isNil ifTrue: [^'NULL'].
	stream := WriteStream on: String new.
	stream nextPutAll: '{ t '''.
	self
		printTime: aTime
		isoFormatOn: stream
		milliseconds: self supportsMillisecondsInTimes.
	stream nextPutAll: ''' }'.
	^stream contents.!

printTimestamp: aTimestamp on: stream for: aType
	aTimestamp isNil ifTrue: [stream nextPutAll: 'NULL'. ^self].
	stream nextPutAll: '{ ts '''.
	self
		printDate: aTimestamp
		isoFormatOn: stream.
	stream nextPutAll: ' '.
	self
		printTime: aTimestamp
		isoFormatOn: stream.
	stream nextPutAll: ''' }'.!

queryWithUnsupportedOperationsEliminatedFrom: aQuery do: aBlock
	"If aQuery has operations that we don't support, rewrite it to do them in terms of lower level operations. In particular, rewrite INTERSECT/EXCEPT operations into EXISTS clauses in a single query. Pass the new query to aBlock."
	| newQuery |
	newQuery := aQuery rewriteIntersect.
	newQuery := newQuery rewriteExcept.
	newQuery == aQuery ifFalse: [aBlock value: newQuery].!

real
	^self float.!

removeNestedParensFromString: aString
	"Return a Smalltalk instance of the type indicated by aGlorpType.  SQL-Server wraps aString in parentheses, so uwrap it first.  If aString is nil, ensure we return nil."

	| debracket |
	aString size < 2 ifTrue: [^aString].	"relies on nil size = 0"
	debracket := aString copy.
	[debracket first = $( and: [debracket last = $)]]
		whileTrue: [debracket := debracket copyFrom: 2 to: debracket size - 1].
	^debracket!

reusePreparedStatements
	^false!

sequence
	^self serial.!

serial

	^self typeNamed: #serial ifAbsentPut: [GlorpSerialType new typeString: 'int  IDENTITY (1, 1) '].!

sqlTextCurrentServerUTCTimestamp

	^'SELECT GETUTCDATE()'!

startColumnAddString
	"The string we use as a prefix when adding a colum.
	Override the superclass default because MS doesn't use COLUMN."
	^' '.!

supportsANSIJoins
	"Do we support the JOIN <tableName> USING <criteria> syntax. Currently hard-coded, but may also vary by database version"
	^true.!

supportsBinding
	"Binding works only with VW EXDI so far.  Reuse of prepared statements is set to false by default (see my class-side)."

	^Dialect isVisualWorks.!

supportsDecimalsOnAllNumerics
	"Return true if a general 'numeric' type will allow numbers after the decimal place"
	^false.!

supportsGroupWritingFor: aCommand
	^aCommand supportsGroupWriting.!

supportsLimit
	"Do we support anything analogous to the postgresql LIMIT, returning only the first N rows"
	^true.!

supportsMillisecondsInTimes
	"I'm guessing here"
	^true.!

supportsMultipleOpenCursors
	"Can this database support multiple open cursors at once"
	^false.!

supportsTableOwners
	"Return true if this platform supports table owners, i.e. expects table names of the form Bern.TW_* rather than just TW_* in its SQL."
"Access, Firebird and PostGreSQL do not, Oracle does, others I know not."

	^true!

text
	^super text queryType: self varchar.!

time
	^self typeNamed: #time ifAbsentPut: [GlorpTimeType new typeString: 'datetime'].!

timestamp
	^self typeNamed: #timestamp ifAbsentPut: [GlorpTimeStampType new typeString: 'datetime'].!

timeStampTypeString
	^'datetime'.!

tinyint
	^self typeNamed: #tinyInt ifAbsentPut: [GlorpIntegerType new typeString: 'tinyint'].!

toGlorpType: aGlorpType fromString: aString
	"Since SQL-Server wraps the string in parentheses, uwrap it first."

	^super
		toGlorpType: aGlorpType
		fromString: (self removeNestedParensFromString: aString)!

typeWithDatabaseName: aString characterWidth: aNumber extraInformation: extraValue
	"Return our type which matches the database name for the type. Assumes all variable width fields are strings right now."
	extraValue = 'IDENTITY' ifTrue: [^self serial].
	^super typeWithDatabaseName: aString characterWidth: aNumber.!

useMicrosoftOuterJoins
	"Return true if we use the Microsoft x *= y syntax for outer joins"
	^false.!

usesArrayBindingRatherThanGrouping
	"Return true if we use array binding for grouped writes rather than printing the sql multiple times. Only applies if we support grouped writes"
	^false.!

usesIdentityColumns
	^true.!

usesInformationSchema

	^true!

varbinary
	^self typeNamed: #varbinary ifAbsentPut: [GlorpVarBinaryType new].!

varchar
	^self typeNamed: #varchar ifAbsentPut: [GlorpVarCharType new].! !
!SQLServerPlatform categoriesFor: #areSequencesExplicitlyCreated!constants!private! !
!SQLServerPlatform categoriesFor: #bindingsForGroupWritingFor:!binding!private! !
!SQLServerPlatform categoriesFor: #bit!private!types! !
!SQLServerPlatform categoriesFor: #blob!private!types! !
!SQLServerPlatform categoriesFor: #boolean!private!types! !
!SQLServerPlatform categoriesFor: #booleanToBooleanConverter!conversion-boolean!private! !
!SQLServerPlatform categoriesFor: #capitalWritingOfTableName!constants!private! !
!SQLServerPlatform categoriesFor: #char!private!types! !
!SQLServerPlatform categoriesFor: #charactersThatNeedEscaping!conversion-strings!private! !
!SQLServerPlatform categoriesFor: #clob!private!types! !
!SQLServerPlatform categoriesFor: #compoundOperationFor:!database-specific!private! !
!SQLServerPlatform categoriesFor: #createAlterColumnStatement:newType:usingExpression:!private!services columns! !
!SQLServerPlatform categoriesFor: #createAlterColumnStatement:setDefault:!private!services columns! !
!SQLServerPlatform categoriesFor: #createAlterColumnStatementDropDefault:!private!services columns! !
!SQLServerPlatform categoriesFor: #createAlterColumnStatementDropNotNull:!private!services columns! !
!SQLServerPlatform categoriesFor: #createAlterColumnStatementSetNotNull:!private!services columns! !
!SQLServerPlatform categoriesFor: #databaseSequenceClass!constants!private! !
!SQLServerPlatform categoriesFor: #date!private!types! !
!SQLServerPlatform categoriesFor: #dateConverter!conversion-times!private! !
!SQLServerPlatform categoriesFor: #dateToTimestampConversion:for:!conversion-times!private! !
!SQLServerPlatform categoriesFor: #decimal!private!types! !
!SQLServerPlatform categoriesFor: #double!private!types! !
!SQLServerPlatform categoriesFor: #escapeFor:!conversion-strings!private! !
!SQLServerPlatform categoriesFor: #exdiTypeForDates!exdi specific!private! !
!SQLServerPlatform categoriesFor: #float!private!types! !
!SQLServerPlatform categoriesFor: #float4!private!types! !
!SQLServerPlatform categoriesFor: #float8!private!types! !
!SQLServerPlatform categoriesFor: #initializeFunctions!functions!private! !
!SQLServerPlatform categoriesFor: #initializeReservedWords!constants!private! !
!SQLServerPlatform categoriesFor: #int!private!types! !
!SQLServerPlatform categoriesFor: #int2!private!types! !
!SQLServerPlatform categoriesFor: #int4!private!types! !
!SQLServerPlatform categoriesFor: #int8!private!types! !
!SQLServerPlatform categoriesFor: #integer!private!types! !
!SQLServerPlatform categoriesFor: #isODBCPlatform!private!testing! !
!SQLServerPlatform categoriesFor: #isSQLServerPlatform!private!testing! !
!SQLServerPlatform categoriesFor: #maximumLengthOfColumnName!constants!public! !
!SQLServerPlatform categoriesFor: #maximumLengthOfTableName!constants!private! !
!SQLServerPlatform categoriesFor: #maximumSizeToGroupWriteFor:!binding!private! !
!SQLServerPlatform categoriesFor: #nclob!private!types! !
!SQLServerPlatform categoriesFor: #numeric!private!types! !
!SQLServerPlatform categoriesFor: #nvarchar!private!types! !
!SQLServerPlatform categoriesFor: #printBlob:on:for:!conversion-strings!public! !
!SQLServerPlatform categoriesFor: #printBoolean:for:!conversion-boolean!private! !
!SQLServerPlatform categoriesFor: #printDate:for:!conversion-times!private! !
!SQLServerPlatform categoriesFor: #printPreLimit:on:!database-specific!private! !
!SQLServerPlatform categoriesFor: #printSqlStatementToListTablesInSchema:on:!public!services tables! !
!SQLServerPlatform categoriesFor: #printSqlStatementToTestExistenceOfTable:inSchema:on:!public!services tables! !
!SQLServerPlatform categoriesFor: #printTime:for:!conversion-times!private! !
!SQLServerPlatform categoriesFor: #printTimestamp:on:for:!conversion-times!private! !
!SQLServerPlatform categoriesFor: #queryWithUnsupportedOperationsEliminatedFrom:do:!database-specific!private! !
!SQLServerPlatform categoriesFor: #real!private!types! !
!SQLServerPlatform categoriesFor: #removeNestedParensFromString:!conversion-strings!private! !
!SQLServerPlatform categoriesFor: #reusePreparedStatements!public!testing! !
!SQLServerPlatform categoriesFor: #sequence!private!types! !
!SQLServerPlatform categoriesFor: #serial!private!types! !
!SQLServerPlatform categoriesFor: #sqlTextCurrentServerUTCTimestamp!constants!public! !
!SQLServerPlatform categoriesFor: #startColumnAddString!constants!private! !
!SQLServerPlatform categoriesFor: #supportsANSIJoins!private!testing! !
!SQLServerPlatform categoriesFor: #supportsBinding!public!testing! !
!SQLServerPlatform categoriesFor: #supportsDecimalsOnAllNumerics!private!testing! !
!SQLServerPlatform categoriesFor: #supportsGroupWritingFor:!private!testing! !
!SQLServerPlatform categoriesFor: #supportsLimit!private!testing! !
!SQLServerPlatform categoriesFor: #supportsMillisecondsInTimes!constants!private! !
!SQLServerPlatform categoriesFor: #supportsMultipleOpenCursors!private!testing! !
!SQLServerPlatform categoriesFor: #supportsTableOwners!private!testing! !
!SQLServerPlatform categoriesFor: #text!private!types! !
!SQLServerPlatform categoriesFor: #time!private!types! !
!SQLServerPlatform categoriesFor: #timestamp!private!types! !
!SQLServerPlatform categoriesFor: #timeStampTypeString!private!types! !
!SQLServerPlatform categoriesFor: #tinyint!private!types! !
!SQLServerPlatform categoriesFor: #toGlorpType:fromString:!conversion-strings!private! !
!SQLServerPlatform categoriesFor: #typeWithDatabaseName:characterWidth:extraInformation:!private!types! !
!SQLServerPlatform categoriesFor: #useMicrosoftOuterJoins!private!testing! !
!SQLServerPlatform categoriesFor: #usesArrayBindingRatherThanGrouping!private!testing! !
!SQLServerPlatform categoriesFor: #usesIdentityColumns!private!testing! !
!SQLServerPlatform categoriesFor: #usesInformationSchema!public!testing! !
!SQLServerPlatform categoriesFor: #varbinary!private!types! !
!SQLServerPlatform categoriesFor: #varchar!private!types! !

AdabasDPlatform guid: (GUID fromString: '{880A5594-25AF-44FF-9589-4A0D1CB158FC}')!
AdabasDPlatform comment: 'This is a platform for the Adabas database.'!
!AdabasDPlatform categoriesForClass!Unclassified! !
SAPDBPlatform guid: (GUID fromString: '{E5FEAD81-5C03-45A7-8056-B99B16CEE713}')!
SAPDBPlatform comment: 'Ths is a platform for SAPDB.'!
!SAPDBPlatform categoriesForClass!Unclassified! !
MySQLODBCPlatform guid: (GUID fromString: '{CB6D2CD4-DF28-4572-A9F4-E7332ABE9A08}')!
MySQLODBCPlatform comment: ''!
!MySQLODBCPlatform categoriesForClass!Unclassified! !
!MySQLODBCPlatform methodsFor!

isODBCPlatform

	^true.! !
!MySQLODBCPlatform categoriesFor: #isODBCPlatform!private!testing! !

FirebirdPlatform guid: (GUID fromString: '{934C8873-E751-4F7A-BBA2-16C46B9AA13A}')!
FirebirdPlatform comment: 'This is a Glorp platform for the InterBase and Firebird databases.  It is currently subclassed from Ocelot because both claim to be highly standard-compliant.  It would probably be better if both inherited from some sort of abstract SQL92Platform.

Firebird/InterBase supports binding in all statements (indeed its EXDI appears to require it for blobs) and can also reuse SELECT statements, but reuse of RowBasedCommands (INSERT, DELETE, UPDATE) for blobs is unreliable although binding of such statements is essential.  Set
	reusePreparedStatements: false
on Logins that have an instance of this class as their platform.
'!
!FirebirdPlatform categoriesForClass!Unclassified! !
!FirebirdPlatform methodsFor!

areSequencesExplicitlyCreated
	^true!

bindingsForGroupWritingFor: aCommand
	"Return the bindings array for a group write. This can be in different formats, depending on the database and perhaps the mechanism in place."
	^aCommand batchStatementBindings.!

blobDataToStreamConverter
	^DelegatingDatabaseConverter
		named: #blobDataToStream
		hostedBy: self
		fromStToDb: #convertBlobData:for:
		fromDbToSt: #nullConversion:for:.!

clob

	^self typeNamed: #clob ifAbsentPut: [GlorpClobType new typeString: 'blob sub_type text'].!

convertBlobData: aData for: aType
	"From standard Store getBlobData called in writeSession;  Firebird seems to need it.  Glorp needs to think that two such streams on the same data are equal."

	^EqualOnContentReadStream on: aData!

databaseSequenceClass
	^FirebirdSequence.!

initializeReservedWords
	reservedWords := Set new.
	reservedWords 
		add: 'type';
		add: 'value';
		add: 'password';
		add: 'unique';
		add: 'date';
		add: 'current';
		add: 'order';
		add: 'active';
		add: 'action';
		add: 'database';
		add: 'table';
		add: 'timestamp';
		add: 'position'.!

isODBCPlatform
	"IBEXDI defines an EXDI connection class, not an ODBC one."
 
	^false!

maximumLengthOfColumnName
	"^<Integer> I return the maximum length of a column name.  (I am not sent at present;  I am here so that if I and #maximumLengthOfTableName (which is sent) should ever differ on some database platform, I can be used in revised truncation logic.)"

	^31!

maximumLengthOfTableName
	"^<Integer> I return the max. length of a table name"

	^31!

printSqlStatementToListTablesInSchema: schemaString on: aStream
	"Firebird uses the same table for tables, views and possibly other stuff.  It does not support schemas within a database "

	aStream
		nextPutAll: 'SELECT rdb$relation_name as table_name FROM rdb$relations ';
		nextPutAll: 'WHERE rdb$view_blr is null and (rdb$system_flag is null or rdb$system_flag = 0);'!

printSqlStatementToTestExistenceOfTable: tableName inSchema: schemaString on: aStream
	"Firebird uses the same table for tables, views and possibly other stuff, does NOT support schemas. 
	As currently written, this query will not test for existence of views OR system tables."

	aStream
		nextPutAll: 'SELECT count(*) FROM rdb$relations ';
		nextPutAll: 'WHERE rdb$relation_name = ';
		nextPut: $';
		nextPutAll: tableName;
		nextPut: $';
		nextPutAll: ' AND rdb$view_blr is null AND (rdb$system_flag is null OR rdb$system_flag = 0);'!

requiresTransactionForTableOperations
	"Return true if we need a transaction in order to do table creation, deletion, and so forth. Some databases require it. Others don't like it"
	^false.!

reusePreparedStatements
	"Firebird/InterBase supports binding of parameters (i.e. instead of printing them as strings within the SQL statement);  indeed, when handling Blobs, the current IBEXDI implementation demands it.  However if RowBasedCommands (INSERT, DELETE, UPDATE) attempt to reuse bound statements, there are failures;  every 79th or 358th or whatever reuse of such a statement returns bad data.  (SELECT statements are fine.)  This was hidden when failure to release the GlorpCursor in RowBasedCommands left statements busy, so unreusable;  the fix (first made in Glorp 0.3.181) revealed it.  We return false here so that reusePreparedStatements is initialized to false for any accessor whose login uses this platform."

	^false

"To study further:  is it that prepared-statement-signature-matching needs to distingush the types of the contents of an EqualOnContentsReadStream, or does the database need a bug fix, or does the IBEXDI need to handle Blobs differently?  If the problems with RowBaseCommands cannot be resolved, can we / should we distinguish between reusing them and reusing SELECT statements?"!

sequence

	^self typeNamed: #sequence ifAbsentPut: [GlorpSerialType new typeString: (self integer typeString)].!

supportsBinding
	"Firebird/InterBase supports binding in all statements (indeed its EXDI appears to require it for blobs) and can also reuse SELECT statements, but reuse of RowBasedCommands (INSERT, DELETE, UPDATE) for Blobs is unreliable although binding of such statements is essential (see my #reusePreparedStatementsIfSupported method comment)."

	^true!

supportsMillisecondsInTimes
	^false.!

vwEXDIConnectionClass
	^'InterBaseConnection' asQualifiedReference valueIfUndefinedLoadFrom: 'IBEXDI'! !
!FirebirdPlatform categoriesFor: #areSequencesExplicitlyCreated!constants!private! !
!FirebirdPlatform categoriesFor: #bindingsForGroupWritingFor:!binding!private! !
!FirebirdPlatform categoriesFor: #blobDataToStreamConverter!conversion!private! !
!FirebirdPlatform categoriesFor: #clob!private!types! !
!FirebirdPlatform categoriesFor: #convertBlobData:for:!conversion!private! !
!FirebirdPlatform categoriesFor: #databaseSequenceClass!private!sequences! !
!FirebirdPlatform categoriesFor: #initializeReservedWords!constants!private! !
!FirebirdPlatform categoriesFor: #isODBCPlatform!private!testing! !
!FirebirdPlatform categoriesFor: #maximumLengthOfColumnName!constants!public! !
!FirebirdPlatform categoriesFor: #maximumLengthOfTableName!constants!private! !
!FirebirdPlatform categoriesFor: #printSqlStatementToListTablesInSchema:on:!public!services tables! !
!FirebirdPlatform categoriesFor: #printSqlStatementToTestExistenceOfTable:inSchema:on:!public!services tables! !
!FirebirdPlatform categoriesFor: #requiresTransactionForTableOperations!constants!private! !
!FirebirdPlatform categoriesFor: #reusePreparedStatements!public!testing! !
!FirebirdPlatform categoriesFor: #sequence!private!types! !
!FirebirdPlatform categoriesFor: #supportsBinding!public!testing! !
!FirebirdPlatform categoriesFor: #supportsMillisecondsInTimes!constants!private! !
!FirebirdPlatform categoriesFor: #vwEXDIConnectionClass!accessing!private! !

OracleODBCPlatform guid: (GUID fromString: '{7EC82F37-68B5-4FA8-8160-D57459593F14}')!
OracleODBCPlatform comment: 'This is a platform for Oracle, as accessed through ODBC. The database doesn''t change, but at least in VW we determine which connection type to use based on the platform object, so we need to use an ODBC connection rather than an OCI connection.'!
!OracleODBCPlatform categoriesForClass!Unclassified! !
!OracleODBCPlatform methodsFor!

isODBCPlatform

	^true.! !
!OracleODBCPlatform categoriesFor: #isODBCPlatform!private!testing! !

AccessPlatform guid: (GUID fromString: '{F943CBF3-A437-489B-8DFB-11A196731217}')!
AccessPlatform comment: 'This is a platform for the Microsoft Access database. It subclasses from SQL Server, because the two have a number of characteristics in common.'!
!AccessPlatform categoriesForClass!Unclassified! !
!AccessPlatform methodsFor!

areSequencesExplicitlyCreated

	^false.!

blob

	^self typeNamed: #blob ifAbsentPut: [GlorpBlobType new typeString: 'longbinary'].!

booleanToBooleanConverter
	^DelegatingDatabaseConverter
		named: #booleanToBoolean
		hostedBy: self
		fromStToDb: #convertBooleanToDBBoolean:for:
		fromDbToSt: #convertDBBooleanToBoolean:for:.!

canBind: aValue to: aType
	(Dialect isVisualWorks and: [aValue class == Dialect fixedPointClass]) ifTrue: [^false].
	^super canBind: aValue to: aType.!

clob

	^self typeNamed: #clob ifAbsentPut: [GlorpClobType new typeString: 'memo'].!

endColumnQuote
	^']'.!

initializeFunctions
	super initializeFunctions.
	functions
		at: #copyFrom:to: put: (SubstringFunction named: 'MID')!

initializeReservedWords
	super initializeReservedWords.
	reservedWords add: 'timestamp'.!

isAccessPlatform

	^true!

longText

	^self typeNamed: #longText ifAbsentPut: [GlorpClobType new typeString: 'memo'].!

maximumLengthOfColumnName
	"^<Integer> I return the maximum length of a column name.  (I am not sent at present;  I am here so that if I and #maximumLengthOfTableName (which is sent) should ever differ on some database platform, I can be used in revised truncation logic.)"

	^64!

maximumLengthOfTableName
	"^<Integer> I return the max. length of a table name"

	^64!

memo

	^self typeNamed: #memo ifAbsentPut: [GlorpClobType new typeString: 'memo'].!

requiresCastsForNullsInUnions
	"If we have explicit nulls in a query which is unioned, do we need to cast them to the appropriate type in order to keep the database happy."
	^true.!

serial

	^self typeNamed: #serial ifAbsentPut: [GlorpSerialType new typeString: 'autoincrement'].!

startColumnQuote
	^'['.!

supportsAliasInCompoundQuery
	"Return true if, when we have a subselect, we can (and/or must) alias the result with a table name. Access doesn't like the table name"
	^false.!

supportsAuthentication
	^false!

supportsDecimalsOnAllNumerics
	"Return true if a general 'numeric' type will allow numbers after the decimal place"
	^true.!

supportsGroupWritingFor: aCommand
	^false.!

supportsInformationSchema
	^false.!

supportsMillisecondsInTimes
	^false.!

supportsSchemas
	^false!

supportsStringQueryOnBlobs
	"Return true if we can do string queries on parts of blobs."
	^false.!

supportsTableOwners
	"Return true if this platform supports table owners, i.e. expects table names of the form Bern.TW_* rather than just TW_* in its SQL."
"Access, Firebird and PostGreSQL do not, Oracle does, others I know not."

	^false!

supportsVariableSizedNumerics
	"Return true if this platform can support numbers with a varying size and number of decimal places. Access, notably, doesn't seem to be able to"
	^false.!

text

	^self typeNamed: #text ifAbsentPut: [GlorpClobType new typeString: 'longtext'].!

useMicrosoftOuterJoins
	"Return true if we use the Microsoft x *= y syntax for outer joins. Oddly enough, false for access 2k"
	^false.!

usesNullForFalse
	"Return true if this database is likely to use nil as an empty string value"
	^true.! !
!AccessPlatform categoriesFor: #areSequencesExplicitlyCreated!constants!private! !
!AccessPlatform categoriesFor: #blob!private!types! !
!AccessPlatform categoriesFor: #booleanToBooleanConverter!conversion-boolean!private! !
!AccessPlatform categoriesFor: #canBind:to:!public!testing! !
!AccessPlatform categoriesFor: #clob!private!types! !
!AccessPlatform categoriesFor: #endColumnQuote!private!services tables! !
!AccessPlatform categoriesFor: #initializeFunctions!functions!public! !
!AccessPlatform categoriesFor: #initializeReservedWords!constants!public! !
!AccessPlatform categoriesFor: #isAccessPlatform!private!testing! !
!AccessPlatform categoriesFor: #longText!private!types! !
!AccessPlatform categoriesFor: #maximumLengthOfColumnName!constants!public! !
!AccessPlatform categoriesFor: #maximumLengthOfTableName!constants!private! !
!AccessPlatform categoriesFor: #memo!private!types! !
!AccessPlatform categoriesFor: #requiresCastsForNullsInUnions!public!testing! !
!AccessPlatform categoriesFor: #serial!private!types! !
!AccessPlatform categoriesFor: #startColumnQuote!private!services tables! !
!AccessPlatform categoriesFor: #supportsAliasInCompoundQuery!public!testing! !
!AccessPlatform categoriesFor: #supportsAuthentication!constants!private! !
!AccessPlatform categoriesFor: #supportsDecimalsOnAllNumerics!private!testing! !
!AccessPlatform categoriesFor: #supportsGroupWritingFor:!private!testing! !
!AccessPlatform categoriesFor: #supportsInformationSchema!private!testing! !
!AccessPlatform categoriesFor: #supportsMillisecondsInTimes!constants!private! !
!AccessPlatform categoriesFor: #supportsSchemas!public!testing! !
!AccessPlatform categoriesFor: #supportsStringQueryOnBlobs!private!testing! !
!AccessPlatform categoriesFor: #supportsTableOwners!private!testing! !
!AccessPlatform categoriesFor: #supportsVariableSizedNumerics!private!SQL! !
!AccessPlatform categoriesFor: #text!private!types! !
!AccessPlatform categoriesFor: #useMicrosoftOuterJoins!private!testing! !
!AccessPlatform categoriesFor: #usesNullForFalse!constants!private! !

ForeignKeyConstraint guid: (GUID fromString: '{A6B22328-04E0-4184-913E-96ED3AB857C8}')!
ForeignKeyConstraint comment: 'This represents a foreign key constraint in the database. It''s used during table creation, but more important it''s used when calculating the order in which we need to write rows. Also, if joins aren''t specified on a relationship, we will try to infer them from the foreign key constraints.

Instance Variables:
	name	<Object>	The name of the constraint.
	shouldCreateInDatabase	<Boolean>	We may have table definitions that we don''t want to create in the database, either because we know they''re already there, or because they are for "imaginary" tables.
	sourceFields	<(SequenceableCollection of: DatabaseField)>	The fields we constrain from. The order is fairly arbitrary, but at least we use it consistently.
	suffixExpression	<String>	A bit of a hack to allow us to append some extra text to the constraint. In particular, added to suppor the ability to append "ON DELETE CASCADE".
	targetFields	<(SequenceableCollection of: DatabaseField)>	The fields we constraint to.

'!
!ForeignKeyConstraint categoriesForClass!Unclassified! !
!ForeignKeyConstraint methodsFor!

addSourceField: aField
	(self sourceFields allSatisfy: [:each | aField table = each table]) ifFalse: [self error: 'Fields in foreign key constraints must all have the same source table'].
	self sourceFields add: aField.!

addTargetField: aField
	(self targetFields allSatisfy: [:each | aField table = each table]) ifFalse: [self error: 'Fields in foreign key constraints must all have the same target table'].
	self targetFields add: aField.!

asJoin
	^Join fromAll: self sourceFields toAll: self targetFields.!

asReverseJoin
	^Join fromAll: self targetFields toAll: self sourceFields.!

creationStringFor: aPlatform

	^aPlatform createForeignKeyConstraintAddStatement: self!

generateName
	| stream maxLength tableLength roomToBeVerbose shortTableLength longTableLength |
	self sourceFields isEmpty ifTrue: [^'INVALID CONSTRAINT'].
	"Figure out if we have room for verbose separators or if we should maximize the space for the table/field names"
	maxLength := self sourceFields first table platform maximumLengthOfTableName.
	shortTableLength := ((maxLength - 8) // 2) min: 10.
	longTableLength := (maxLength // 2) - 1.
	roomToBeVerbose := shortTableLength >= 10.
	tableLength := roomToBeVerbose ifTrue: [shortTableLength] ifFalse: [longTableLength].

	stream := WriteStream on: (String new: 100).
	self printForConstraintName: self sourceFields first on: stream maxLength: tableLength.
	roomToBeVerbose ifTrue: [stream nextPutAll: '_TO_'].
	self printForConstraintName: self targetFields first on: stream maxLength: tableLength.
	stream nextPutAll: (roomToBeVerbose ifTrue: ['_REF'] ifFalse: ['R']).
	^stream contents.!

initialize

	self sourceFields: (OrderedCollection new: 2).
	targetFields := OrderedCollection new: 2.!

isForeignKeyConstraint

	^true!

isImaginary
	^self targetTable isImaginary or: [self sourceTable isImaginary].!

isSimilarForeignKeyConstraint: aConstraint

	self sourceFields size ~= aConstraint sourceFields size ifTrue: [^false].
	self sourceFields asSortedCollection with: aConstraint sourceFields asSortedCollection do: [:field1 :field2 | 
		(field1 matchesField: field2) ifFalse: [^false]].

	self targetFields size ~= aConstraint targetFields size ifTrue: [^false].
	self targetFields asSortedCollection with: aConstraint targetFields asSortedCollection do: [:field1 :field2 | 
		(field1 matchesField: field2) ifFalse: [^false]].
	^true!

isTheSameAs: aForeignKeyConstraint
	"Validate that two constraints are exactly the same. Most useful for testing purposes, rather than normal usage, which is why we don't just use the #= method."
	self name asUppercase = aForeignKeyConstraint name asUppercase ifFalse: [^false].
	self suffixExpression = aForeignKeyConstraint suffixExpression ifFalse: [^false].
	self sourceFields size ~= aForeignKeyConstraint sourceFields size ifTrue: [^false].
	self sourceFields asSortedCollection with: aForeignKeyConstraint sourceFields asSortedCollection do: [:a :b | (a isTheSameAs: b) ifFalse: [^false]].	
	self targetFields size ~= aForeignKeyConstraint targetFields size ifTrue: [^false].
	self targetFields asSortedCollection with: aForeignKeyConstraint targetFields asSortedCollection do: [:a :b | (a isTheSameAs: b) ifFalse: [^false]].	
	self shouldCreateInDatabase = aForeignKeyConstraint shouldCreateInDatabase ifFalse: [^false].
	^true.!

matchesForeignKeyConstraint: aConstraint
	name asUppercase = aConstraint name asUppercase ifFalse: [^false].
	self sourceFields size ~= aConstraint sourceFields size ifTrue: [^false].
	self sourceFields with: aConstraint sourceFields do: [:field1 :field2 | 
		(field1 matchesField: field2) ifFalse: [^false]].

	self targetFields size ~= aConstraint targetFields size ifTrue: [^false].
	self targetFields with: aConstraint targetFields do: [:field1 :field2 | 
		(field1 matchesField: field2) ifFalse: [^false]].
	^true!

printForConstraintName: aField on: aStream maxLength: maxLength 
	| constraintName |
	constraintName := aField table name , '_' , aField name.
	constraintName size > maxLength 
		ifTrue: [constraintName := constraintName copyFrom: 1 to: maxLength].
	aStream nextPutAll: constraintName!

refersToTable: aTable
	^self targetTable yourSelf == aTable.!

rowsRelatedTo: aRow
	| myWrapper relatedRows sourceField targetField allWrappers |
	"Note that we don't care which field, as presumably all of them must point the same place for a composite key, so grab the first pair"
	sourceField := self sourceFields first.
	targetField := self targetFields first.
	myWrapper := aRow wrapperAt: sourceField ifAbsent: [nil].
	myWrapper isNil ifTrue: [^#()].
	relatedRows := Set new: 5.
	allWrappers := OrderedCollection with: myWrapper.
	allWrappers addAll: myWrapper allInnerWrappers.
	allWrappers
		do: [:eachWrapper |
			(eachWrapper containedBy glorpAtIdentity: targetField ifAbsent: [#()])
				do: [:eachRow | eachRow == aRow ifFalse: [relatedRows add: eachRow]]].
	^relatedRows.!

someSourceField
	"Return any one of our source fields"
	^self sourceFields first.!

sourceField: aField
	self sourceFields isEmpty ifFalse: [self error: 'Adding a field to a non-empty constraint'].
	self sourceFields add: aField.!

sourceField: aDatabaseField targetField: anotherDatabaseField
	self
		sourceField: aDatabaseField
		targetField: anotherDatabaseField
		suffixExpression: nil.!

sourceField: aDatabaseField targetField: anotherDatabaseField suffixExpression: suffixExpressionString
	self
		sourceField: aDatabaseField;
		targetField: anotherDatabaseField;
		suffixExpression: suffixExpressionString.!

sourceFields
	^sourceFields!

sourceFields: aCollection
	
	sourceFields := aCollection.!

sourceFields: sourceFieldCollection targetFields: targetFieldCollection suffixExpression: suffixExpressionString
	self
		sourceFields: sourceFieldCollection;
		targetFields: targetFieldCollection;
		suffixExpression: suffixExpressionString.!

sourceTable
	"All the source fields have to come from the same table, so just pick one"
	^self sourceFields first table.!

suffixExpression
	^suffixExpression!

suffixExpression: anObject
	suffixExpression := anObject.!

table
	^self sourceTable.!

targetField: aField

	self targetFields isEmpty ifFalse: [self error: 'Adding a field to a non-empty constraint'].
	self targetFields add: aField.!

targetFields
	^targetFields!

targetFields: aCollection
	targetFields := aCollection.!

targetTable
	"All the target fields have to come from the same table, so just pick one"
	^self targetFields isEmpty ifTrue: [nil] ifFalse: [self targetFields first table].! !
!ForeignKeyConstraint categoriesFor: #addSourceField:!accessing!private! !
!ForeignKeyConstraint categoriesFor: #addTargetField:!accessing!private! !
!ForeignKeyConstraint categoriesFor: #asJoin!converting!private! !
!ForeignKeyConstraint categoriesFor: #asReverseJoin!converting!private! !
!ForeignKeyConstraint categoriesFor: #creationStringFor:!printing!private! !
!ForeignKeyConstraint categoriesFor: #generateName!printing!private! !
!ForeignKeyConstraint categoriesFor: #initialize!initializing!public! !
!ForeignKeyConstraint categoriesFor: #isForeignKeyConstraint!public!testing! !
!ForeignKeyConstraint categoriesFor: #isImaginary!private!testing! !
!ForeignKeyConstraint categoriesFor: #isSimilarForeignKeyConstraint:!public!testing! !
!ForeignKeyConstraint categoriesFor: #isTheSameAs:!comparing!public! !
!ForeignKeyConstraint categoriesFor: #matchesForeignKeyConstraint:!public!testing! !
!ForeignKeyConstraint categoriesFor: #printForConstraintName:on:maxLength:!printing!private! !
!ForeignKeyConstraint categoriesFor: #refersToTable:!public!testing! !
!ForeignKeyConstraint categoriesFor: #rowsRelatedTo:!private!relationships! !
!ForeignKeyConstraint categoriesFor: #someSourceField!accessing!private! !
!ForeignKeyConstraint categoriesFor: #sourceField:!accessing!private! !
!ForeignKeyConstraint categoriesFor: #sourceField:targetField:!initializing!private! !
!ForeignKeyConstraint categoriesFor: #sourceField:targetField:suffixExpression:!initializing!private! !
!ForeignKeyConstraint categoriesFor: #sourceFields!accessing!private! !
!ForeignKeyConstraint categoriesFor: #sourceFields:!accessing!private! !
!ForeignKeyConstraint categoriesFor: #sourceFields:targetFields:suffixExpression:!initializing!private! !
!ForeignKeyConstraint categoriesFor: #sourceTable!accessing!private! !
!ForeignKeyConstraint categoriesFor: #suffixExpression!accessing!private! !
!ForeignKeyConstraint categoriesFor: #suffixExpression:!accessing!private! !
!ForeignKeyConstraint categoriesFor: #table!accessing!private! !
!ForeignKeyConstraint categoriesFor: #targetField:!accessing!private! !
!ForeignKeyConstraint categoriesFor: #targetFields!accessing!private! !
!ForeignKeyConstraint categoriesFor: #targetFields:!accessing!private! !
!ForeignKeyConstraint categoriesFor: #targetTable!accessing!private! !

!ForeignKeyConstraint class methodsFor!

sourceField: aDatabaseField targetField: anotherDatabaseField
	^self
		sourceField: aDatabaseField
		targetField: anotherDatabaseField
		suffixExpression: nil.!

sourceField: aDatabaseField targetField: anotherDatabaseField suffixExpression: suffixExpression
	^(self new)
		sourceField: aDatabaseField targetField: anotherDatabaseField suffixExpression: suffixExpression;
		yourself.!

sourceFields: sourceFields targetFields: targetFields
	^self
		sourceFields: sourceFields
		targetFields: targetFields
		suffixExpression: nil.!

sourceFields: sourceFields targetFields: targetFields suffixExpression: suffixExpression
	^(self new)
		sourceFields: sourceFields targetFields: targetFields suffixExpression: suffixExpression;
		yourself.! !
!ForeignKeyConstraint class categoriesFor: #sourceField:targetField:!instance creation!private! !
!ForeignKeyConstraint class categoriesFor: #sourceField:targetField:suffixExpression:!instance creation!private! !
!ForeignKeyConstraint class categoriesFor: #sourceFields:targetFields:!instance creation!private! !
!ForeignKeyConstraint class categoriesFor: #sourceFields:targetFields:suffixExpression:!instance creation!private! !

PrimaryKeyConstraint guid: (GUID fromString: '{C41953A3-D2FC-48A6-ACB5-A970984D12F3}')!
PrimaryKeyConstraint comment: 'This is a class representing non-foreign key database constraints. It''s not actually used for very much, but is mostly so we can model the metadata in the database''s information_schema to get information we need about primary keys and other information that may be represented as constraints rather than field attributes.'!
!PrimaryKeyConstraint categoriesForClass!Unclassified! !
!PrimaryKeyConstraint methodsFor!

creationStringFor: aPlatform

	^aPlatform createPrimaryKeyConstraintAddStatement: self!

forTable: aTable

	super forTable: aTable.
	name := self primaryKeyConstraintName.
	self primaryKeyFields:  table primaryKeyFields!

forTable: aDatabaseTable fields: collDatabaseFields

	self forTable: aDatabaseTable.
	self primaryKeyFields: collDatabaseFields!

generateName

	^self table name, '_PK'.!

isPrimaryKeyConstraint

	^true!

isTheSameAs: aPrimaryKeyConstraint
	"Validate that two constraints are exactly the same. Most useful for testing purposes, rather than normal usage, which is why we don't just use the #= method."
	self name asUppercase = aPrimaryKeyConstraint name asUppercase ifFalse: [^false].
	self primaryKeyFields size = aPrimaryKeyConstraint primaryKeyFields size ifFalse: [^false].
	self primaryKeyFields asSortedCollection with: aPrimaryKeyConstraint primaryKeyFields asSortedCollection do: [:a :b | (a isTheSameAs: b) ifFalse: [^false]].	
	self shouldCreateInDatabase = aPrimaryKeyConstraint shouldCreateInDatabase ifFalse: [^false].
	^true.!

matchesPrimaryKeyConstraint: aConstraint
	name = aConstraint name ifFalse: [^false].
	self primaryKeyFields size ~= aConstraint primaryKeyFields size ifTrue: [^false].
	self primaryKeyFields with: aConstraint primaryKeyFields do: [ :field1 :field2 | 
		(field1 matchesField: field2) ifFalse: [^false]].
	^true!

primaryKeyConstraintName

	^self table name, '_PK'.!

primaryKeyFields

	^fields!

primaryKeyFields: aCollection

	fields := aCollection! !
!PrimaryKeyConstraint categoriesFor: #creationStringFor:!printing!private! !
!PrimaryKeyConstraint categoriesFor: #forTable:!initializing!private! !
!PrimaryKeyConstraint categoriesFor: #forTable:fields:!initializing!private! !
!PrimaryKeyConstraint categoriesFor: #generateName!printing!private! !
!PrimaryKeyConstraint categoriesFor: #isPrimaryKeyConstraint!public!testing! !
!PrimaryKeyConstraint categoriesFor: #isTheSameAs:!comparing!public! !
!PrimaryKeyConstraint categoriesFor: #matchesPrimaryKeyConstraint:!private!testing! !
!PrimaryKeyConstraint categoriesFor: #primaryKeyConstraintName!accessing!private! !
!PrimaryKeyConstraint categoriesFor: #primaryKeyFields!accessing!private! !
!PrimaryKeyConstraint categoriesFor: #primaryKeyFields:!accessing!private! !

!PrimaryKeyConstraint class methodsFor!

forTable: aDatabaseTable fields: collDatabaseFields
	^self new
		forTable: aDatabaseTable  fields: collDatabaseFields;
		yourself! !
!PrimaryKeyConstraint class categoriesFor: #forTable:fields:!instance creation!private! !

UniqueConstraint guid: (GUID fromString: '{0EB804C8-1DB5-4AAA-81EB-F3D5649F18C5}')!
UniqueConstraint comment: 'UniqueConstraint implementation has not been finished

Instance Variables:
	isForPrimaryKey	<Boolean>	description of isForPrimaryKey

'!
!UniqueConstraint categoriesForClass!Unclassified! !
!UniqueConstraint methodsFor!

creationStringFor: aPlatform

	^aPlatform createUniqueConstraintAddStatement: self!

forTable: aTable fields: aCollectionOfFields

	self table: aTable.
	self fields: aCollectionOfFields!

generateName
	| stream maxLength tableLength roomToBeVerbose shortTableLength longTableLength |
	(self fields isNil or: [self fields isEmpty]) ifTrue: [^'INVALID CONSTRAINT'].
	"Figure out if we have room for verbose separators or if we should maximize the space for the table/field names"
	maxLength := self fields first table platform maximumLengthOfTableName.
	shortTableLength := ((maxLength - 7) // 2) min: 10.
	longTableLength := (maxLength - 7).
	roomToBeVerbose := shortTableLength >= 10.
	tableLength := roomToBeVerbose ifTrue: [longTableLength] ifFalse: [shortTableLength].

	stream := WriteStream on: (String new: 100).
	self printForConstraintNames: self fields on: stream maxLength: tableLength.
	stream nextPutAll: (roomToBeVerbose ifTrue: ['_UNIQ'] ifFalse: ['U']).
	^stream contents.!

isUniqueConstraint

	^true!

matchesUniqueConstraint: aConstraint
	
	name asUppercase = aConstraint name asUppercase ifFalse: [^false].
	self fields size ~= aConstraint fields size ifTrue: [^false].
	self fields with: aConstraint fields do: [:each :other | (each matchesField: other) ifFalse: [^false]].
	^true!

printForConstraintNames: aCollection on: aStream maxLength: maxLength 
	| constraintName |
	constraintName := self table name.
	aCollection do:
		[:each | constraintName := constraintName, each name].
	constraintName size > maxLength 
		ifTrue: [constraintName := constraintName copyFrom: 1 to: maxLength].
	aStream nextPutAll: constraintName! !
!UniqueConstraint categoriesFor: #creationStringFor:!printing!public! !
!UniqueConstraint categoriesFor: #forTable:fields:!instance creation!public! !
!UniqueConstraint categoriesFor: #generateName!accessing!public! !
!UniqueConstraint categoriesFor: #isUniqueConstraint!public!testing! !
!UniqueConstraint categoriesFor: #matchesUniqueConstraint:!public!testing! !
!UniqueConstraint categoriesFor: #printForConstraintNames:on:maxLength:!printing!public! !

!UniqueConstraint class methodsFor!

forTable: aTable fields: aCollectionOfFields

	| instance |
	instance := self new.
	instance forTable: aTable fields: aCollectionOfFields.
	^instance! !
!UniqueConstraint class categoriesFor: #forTable:fields:!instance creation!public! !

VersionType guid: (GUID fromString: '{29849093-0025-4749-8656-E1F326C20E41}')!
VersionType comment: 'This represents the type of something used for a version field.

Instance Variables:
	underlyingType	<DatabaseType>	The underlying type of our field.
	sequence <> The generator used to generate the next value.

'!
!VersionType categoriesForClass!Unclassified! !
!VersionType methodsFor!

converterForStType: aClass

	^underlyingType converterForStType: aClass.!

exdiType
	"Return a symbol that the VisualWorks EXDI uses to indicate the database type -- this is a symbol which is often the name of a Smalltalk class, but with variations for multiple db types that correspond to the same Smalltalk symbol. It also varies by Database. This is equivalent to the values returned by methods like #odbcConversion or #oracleConversion"
	^self underlyingType exdiType.!

generateOverExistingValues
	generatesOverExistingValues := true.!

generatesOverExistingValues
	"Do we generate our value even if there's alread one present in the field"
	^generatesOverExistingValues.!

generatesOverExistingValues: aBoolean
	"Do we generate our value even if there's alread one present in the field"
	generatesOverExistingValues := aBoolean.!

generator
	^generator!

generator: aDatabaseValueGenerator
	generator := aDatabaseValueGenerator!

generatorFor: aDatabaseType
	aDatabaseType impliedSmalltalkType == Integer ifTrue: [^VersionNumberVersionGenerator new].
	aDatabaseType impliedSmalltalkType == Dialect timestampClass ifTrue: [^TimestampVersionGenerator new].
	self error: 'unknown version type'.!

initialize
	super initialize.
	generatesOverExistingValues := false.!

isGenerated
	^true.!

postWriteAssignSequenceValueFor: aDatabaseField in: aDatabaseRow using: anAccessor

	^self.!

preWriteAssignSequenceValueFor: aDatabaseField in: aDatabaseRow using: anAccessor

	^generator preWriteAssignSequenceValueFor: aDatabaseField in: aDatabaseRow using: anAccessor.!

print: aValue on: aStream
	self underlyingType print: aValue on: aStream.!

typeString
	^underlyingType typeString.!

underlyingType
	^underlyingType!

underlyingType: aDatabaseType
	underlyingType := aDatabaseType.
	platform := aDatabaseType platform.
	generator := self generatorFor: aDatabaseType.!

underlyingType: aDatabaseType generator: aGenerator
	underlyingType := aDatabaseType.
	platform := aDatabaseType platform.
	generator := aGenerator.!

width
	^self underlyingType width.! !
!VersionType categoriesFor: #converterForStType:!converting!private! !
!VersionType categoriesFor: #exdiType!exdi specific!private! !
!VersionType categoriesFor: #generateOverExistingValues!private!testing! !
!VersionType categoriesFor: #generatesOverExistingValues!private!testing! !
!VersionType categoriesFor: #generatesOverExistingValues:!private!testing! !
!VersionType categoriesFor: #generator!accessing!private! !
!VersionType categoriesFor: #generator:!accessing!private! !
!VersionType categoriesFor: #generatorFor:!initializing!private! !
!VersionType categoriesFor: #initialize!initializing!private! !
!VersionType categoriesFor: #isGenerated!private!testing! !
!VersionType categoriesFor: #postWriteAssignSequenceValueFor:in:using:!private!sequencing! !
!VersionType categoriesFor: #preWriteAssignSequenceValueFor:in:using:!private!sequencing! !
!VersionType categoriesFor: #print:on:!private!SQL! !
!VersionType categoriesFor: #typeString!private!SQL! !
!VersionType categoriesFor: #underlyingType!accessing!private! !
!VersionType categoriesFor: #underlyingType:!accessing!private! !
!VersionType categoriesFor: #underlyingType:generator:!accessing!private! !
!VersionType categoriesFor: #width!accessing!private! !

!VersionType class methodsFor!

underlyingType: aDatabaseType
	^self new underlyingType: aDatabaseType.!

underlyingType: aDatabaseType generator: aGenerator
	^self new underlyingType: aDatabaseType generator: aGenerator.! !
!VersionType class categoriesFor: #underlyingType:!instance creation!private! !
!VersionType class categoriesFor: #underlyingType:generator:!instance creation!private! !

DatabaseSequence guid: (GUID fromString: '{9B2E60F1-5E46-47ED-B20C-D1D63C7693B4}')!
DatabaseSequence comment: 'This is an abstract superclass for identifier generation mechanisms in the database. For example, Oracle sequences, SQL Server identity columns, and so on.

Subclasses must implement the following messages:
	sequencing
		getSequenceValueFromDatabaseFor:in:using:
		postWriteAssignSequenceValueFor:in:using:

Instance Variables:
	reservedNumbers	<OrderedCollection>	Numbers that we have already reserved and have available. This is only useful if the sequence can generate more than one identifier in a single request, and we have configured it (and ourselves) to do so and use those numbers. This is useful for efficiency when inserting lots of data.

'!
!DatabaseSequence categoriesForClass!Unclassified! !
!DatabaseSequence methodsFor!

creationString

	^'Create sequence ', self qualifiedName.!

dropString

	^'Drop string unspecified for this type of sequence'.!

getSequenceValueFromDatabaseFor: aField in: aRow using: aSession 
	self subclassResponsibility.!

hasReservedNumbers
	^reservedNumbers notNil and: [reservedNumbers isEmpty not].!

initialize!

isIdentityColumn
	^false.!

nextReservedNumber
	^reservedNumbers removeFirst.!

postWriteAssignSequenceValueFor: aField in: aRow using: anAccessor
	self subclassResponsibility.!

preWriteAssignSequenceValueFor: aField in: aRow using: anAccessor
	self hasReservedNumbers
		ifTrue: [aRow at: aField put: self nextReservedNumber]
		ifFalse:
			[self
				getSequenceValueFromDatabaseFor: aField
				in: aRow
				using: anAccessor].!

reservedNumbers
	reservedNumbers isNil ifTrue: [reservedNumbers := OrderedCollection new: 50].
	^reservedNumbers.! !
!DatabaseSequence categoriesFor: #creationString!private!sequencing! !
!DatabaseSequence categoriesFor: #dropString!private!sequencing! !
!DatabaseSequence categoriesFor: #getSequenceValueFromDatabaseFor:in:using:!private!sequencing! !
!DatabaseSequence categoriesFor: #hasReservedNumbers!private!sequencing! !
!DatabaseSequence categoriesFor: #initialize!initialize/release!private! !
!DatabaseSequence categoriesFor: #isIdentityColumn!private!testing! !
!DatabaseSequence categoriesFor: #nextReservedNumber!private!sequencing! !
!DatabaseSequence categoriesFor: #postWriteAssignSequenceValueFor:in:using:!public!sequencing! !
!DatabaseSequence categoriesFor: #preWriteAssignSequenceValueFor:in:using:!private!sequencing! !
!DatabaseSequence categoriesFor: #reservedNumbers!accessing!private! !

!DatabaseSequence class methodsFor!

named: aString
	^self new name: aString.!

new

	^super new initialize.! !
!DatabaseSequence class categoriesFor: #named:!instance creation!private! !
!DatabaseSequence class categoriesFor: #new!instance creation!private! !

VersionGenerator guid: (GUID fromString: '{005644BA-B417-4AE5-BE66-47D24CDB3F78}')!
VersionGenerator comment: 'An abstract superclass for generators that generate version numbers (rather than sequence numbers).

Subclasses must implement the following messages:
	sequencing
		nextValueFor:in:

'!
!VersionGenerator categoriesForClass!Unclassified! !
!VersionGenerator methodsFor!

nextValueFor: aField in: aRow
	self subclassResponsibility.!

preWriteAssignSequenceValueFor: aField in: aRow using: anAccessor
	"Overwrite a pre-existing value in the field if necessary"
	aRow at: aField forcePut: (self nextValueFor: aField in: aRow).! !
!VersionGenerator categoriesFor: #nextValueFor:in:!private!sequencing! !
!VersionGenerator categoriesFor: #preWriteAssignSequenceValueFor:in:using:!private!sequencing! !

InMemorySequence guid: (GUID fromString: '{5FBA70D9-2F79-47A2-AC04-46153E61BE98}')!
InMemorySequence comment: 'This is a sequence generator that just generates numbers in memory. Mostly useful for testing, because the numbers aren''t guaranteed unique in the database unless we are the only user, and our initial value was appropriately set.

Instance Variables:
	count	<Integer>	The last used identifier.
'!
!InMemorySequence categoriesForClass!Unclassified! !
!InMemorySequence methodsFor!

getSequenceValueFromDatabaseFor: aField in: aRow using: aSession

	aRow at: aField put: (count := count + 1).!

initialize

	super initialize.
	count := 0.!

postWriteAssignSequenceValueFor: aField in: aRow!

postWriteAssignSequenceValueFor: aField in: aRow using: anAccessor!

preWriteAssignSequenceValueFor: aField in: aRow

	aRow at: aField put: (count := count + 1).! !
!InMemorySequence categoriesFor: #getSequenceValueFromDatabaseFor:in:using:!public! !
!InMemorySequence categoriesFor: #initialize!initialize/release!private! !
!InMemorySequence categoriesFor: #postWriteAssignSequenceValueFor:in:!private!sequencing! !
!InMemorySequence categoriesFor: #postWriteAssignSequenceValueFor:in:using:!public!sequencing! !
!InMemorySequence categoriesFor: #preWriteAssignSequenceValueFor:in:!private!sequencing! !

!InMemorySequence class methodsFor!

default

	^self new.! !
!InMemorySequence class categoriesFor: #default!defaults!private! !

NamedSequence guid: (GUID fromString: '{9E8237E6-1F4E-4C01-B7A5-21F6129E1A32}')!
NamedSequence comment: 'This is a database sequence that is identified by name.

Subclasses must implement the following messages:
	SQL
		tableSelectCommandFor:in:reserving:

Instance Variables:
	name	<String>	The name of the sequence
	schema	<String>	The schema that contains the sequence. Subclasses should set this up appropriately.
	tableSelectCommand	<SQLStringSelectCommand>	The command to execute to get the next value. 

'!
!NamedSequence categoriesForClass!Unclassified! !
!NamedSequence methodsFor!

availableReservedNumbers
	^self reservedNumbers size.!

dropString

	^'Drop sequence ', self qualifiedName.!

maximumToReserveAtOnce
		^1000.!

name
	^name.!

name: aString
	name := aString.!

printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: (name isNil ifTrue: ['unnamed'] ifFalse: [name]);
		nextPut: $).!

qualifiedName
	^(schema isNil or: [schema isEmpty]) ifTrue: [name] ifFalse: [schema, '.', name].!

reserveSequenceNumbers: anInteger in: aSession for: aTable
	| numberNeeded numberToReserve |
	numberNeeded := anInteger - self availableReservedNumbers.
	numberNeeded <= 0 ifTrue: [^self].
	self
		reserveViaTableSelect: numberNeeded
		in: aSession
		for: aTable.
	[anInteger - self availableReservedNumbers >0 ] whileTrue: [
		numberToReserve := (anInteger - self availableReservedNumbers) min: (self maximumToReserveAtOnce).
		self reserveViaBackupMechanism: numberToReserve in: aSession].!

reserveViaTableSelect: anInteger in: aSession for: aTable
	| rows command |
	command := self
		tableSelectCommandFor: (self tableToSelectFromIn: aSession for: aTable)
		in: aSession
		reserving: anInteger.
	command parameters: (Array with: anInteger).
	rows := aSession accessor executeCommand: command.
	rows do: [:each | self reservedNumbers add: each first].!

schema
	^schema!

schema: aString
	schema := aString!

tableSelectCommandFor: aTable in: aSession reserving: anObject
	self subclassResponsibility.!

tableToSelectFromFor: aTable
	"Sometimes it's inefficient to select sequence values against our own table and we can use a database-specific system table instead to be much faster. Subclasses can override where this is the case"
	^aTable.! !
!NamedSequence categoriesFor: #availableReservedNumbers!private!SQL! !
!NamedSequence categoriesFor: #dropString!private!sequencing! !
!NamedSequence categoriesFor: #maximumToReserveAtOnce!private!SQL! !
!NamedSequence categoriesFor: #name!accessing!private! !
!NamedSequence categoriesFor: #name:!accessing!private! !
!NamedSequence categoriesFor: #printOn:!printing!private! !
!NamedSequence categoriesFor: #qualifiedName!accessing!private! !
!NamedSequence categoriesFor: #reserveSequenceNumbers:in:for:!private!SQL! !
!NamedSequence categoriesFor: #reserveViaTableSelect:in:for:!public!SQL! !
!NamedSequence categoriesFor: #schema!accessing!private! !
!NamedSequence categoriesFor: #schema:!accessing!private! !
!NamedSequence categoriesFor: #tableSelectCommandFor:in:reserving:!private!SQL! !
!NamedSequence categoriesFor: #tableToSelectFromFor:!private!SQL! !

NullSequence guid: (GUID fromString: '{A4061874-B215-428F-8363-A3920F06D8C6}')!
NullSequence comment: 'This is a sequence that doesn''t do anything. It''s really just a placeholder.'!
!NullSequence categoriesForClass!Unclassified! !
!NullSequence methodsFor!

getSequenceValueFromDatabaseFor: aField in: aRow using: aSession!

postWriteAssignSequenceValueFor: aField in: aRow!

postWriteAssignSequenceValueFor: aField in: aRow using: anAccessor!

preWriteAssignSequenceValueFor: aField in: aRow! !
!NullSequence categoriesFor: #getSequenceValueFromDatabaseFor:in:using:!public! !
!NullSequence categoriesFor: #postWriteAssignSequenceValueFor:in:!private!sequencing! !
!NullSequence categoriesFor: #postWriteAssignSequenceValueFor:in:using:!public!sequencing! !
!NullSequence categoriesFor: #preWriteAssignSequenceValueFor:in:!private!sequencing! !

!NullSequence class methodsFor!

default

	^self new.!

new

	Singleton isNil ifTrue: [Singleton := self basicNew].
	^Singleton.! !
!NullSequence class categoriesFor: #default!defaults!private! !
!NullSequence class categoriesFor: #new!instance creation!private! !

SQLite3Sequence guid: (GUID fromString: '{CCA4FF9A-E6B6-4F3A-80FE-B333D78A7981}')!
SQLite3Sequence comment: 'Sequences for SQLite.'!
!SQLite3Sequence categoriesForClass!Unclassified! !
!SQLite3Sequence methodsFor!

getSequenceValueFromDatabaseFor: aField in: aRow using: aSession!

isIdentityColumn
	^true.!

name: aString
	"We ignore names"!

postWriteAssignSequenceValueFor: aDatabaseField in: aDatabaseRow using: anAccessor
	
	| value |
	value := anAccessor connection isNil ifTrue: [nil] ifFalse: [anAccessor connection lastAutoIncrementValue asInteger].
	aDatabaseRow at: aDatabaseField put: value!

reserveSequenceNumbers: anInteger in: aSession for: aTable
	"No real sequences here, just identity columns, which we can't pre-allocate"!

schema: aString
	"This is SQLite - we don't have schemas"! !
!SQLite3Sequence categoriesFor: #getSequenceValueFromDatabaseFor:in:using:!private!sequencing! !
!SQLite3Sequence categoriesFor: #isIdentityColumn!private!testing! !
!SQLite3Sequence categoriesFor: #name:!private!sequencing! !
!SQLite3Sequence categoriesFor: #postWriteAssignSequenceValueFor:in:using:!private!sequencing! !
!SQLite3Sequence categoriesFor: #reserveSequenceNumbers:in:for:!private!sequencing! !
!SQLite3Sequence categoriesFor: #schema:!private!sequencing! !

TableBasedSequence guid: (GUID fromString: '{9B86B9C6-1FB7-429A-BDE8-B1C0FA7E5584}')!
TableBasedSequence comment: 'This is a sequence that gets its values from a sequence table. It doesn''t look like it''s fully implemented, or used.

Instance Variables:
	sequenceTableName	<String>	The name of the sequence table to use.

'!
!TableBasedSequence categoriesForClass!Unclassified! !
!TableBasedSequence methodsFor!

isIdentityColumn
	^true.!

sequenceTableName
	"Private - Answer the value of the receiver's ''sequenceTableName'' instance variable."

	^sequenceTableName!

sequenceTableName: aString

	sequenceTableName := aString! !
!TableBasedSequence categoriesFor: #isIdentityColumn!private!testing! !
!TableBasedSequence categoriesFor: #sequenceTableName!accessing!private! !
!TableBasedSequence categoriesFor: #sequenceTableName:!accessing!private! !

!TableBasedSequence class methodsFor!

default

	^self new sequenceTableName: 'SEQUENCE'.! !
!TableBasedSequence class categoriesFor: #default!defaults!private! !

DB2IdentitySequence guid: (GUID fromString: '{6DD91981-ABD3-4F85-935B-1181DB1272FB}')!
DB2IdentitySequence comment: 'DB2IdentitySequence is the sequence class for DB2 identity columns. Glorp prefers sequence objects over identity columns, and although DB2 has both, Glorp defaults to using sequences. Therefore there is no code here to create an identity column. Primary keys will use sequences instead. DB2IdentitySequence primarily helps in the analysis of tables that were created outside Glorp.

One can obtain the most recently installed number using a query like this.
	VALUES IDENTITY_VAL_LOCAL().
'!
!DB2IdentitySequence categoriesForClass!Unclassified! !
!DB2IdentitySequence methodsFor!

getSequenceValueFromDatabaseFor: aDatabaseField in: aDatabaseRow using: anAccessor
	"I do nothing, but I am here for subclassResponsibility reasons"!

isIdentityColumn
	^true.!

postWriteAssignSequenceValueFor: aDatabaseField in: aDatabaseRow using: anAccessor 
	aDatabaseRow at: aDatabaseField
		put: (((anAccessor 
				executeSQLString: 'VALUES IDENTITY_VAL_LOCAL()') first atIndex: 1) asInteger).!

reserveSequenceNumbers: anInteger in: aSession for: aTable
	"No real sequences here, just identity columns, which we can't pre-allocate"! !
!DB2IdentitySequence categoriesFor: #getSequenceValueFromDatabaseFor:in:using:!public!sequencing! !
!DB2IdentitySequence categoriesFor: #isIdentityColumn!public!testing! !
!DB2IdentitySequence categoriesFor: #postWriteAssignSequenceValueFor:in:using:!public!sequencing! !
!DB2IdentitySequence categoriesFor: #reserveSequenceNumbers:in:for:!public!SQL! !

FirebirdSequence guid: (GUID fromString: '{1209542B-DE7C-4150-84C5-E28F09B877BA}')!
FirebirdSequence comment: 'Implements firebird sequence generation by calling out to gen_id()

'!
!FirebirdSequence categoriesForClass!Unclassified! !
!FirebirdSequence methodsFor!

creationString

	^'create generator ' , self qualifiedName.!

dropString
	^'DELETE FROM RDB$GENERATORS
        WHERE RDB$GENERATOR_NAME=''', self qualifiedName, ''''.!

getSequenceValueFromDatabaseFor: aDatabaseField in: aDatabaseRow using: anAccessor 
	| statement rows pk |
	statement := 'select gen_id(' , self qualifiedName 
				, ',1) from rdb$database'.
	rows := anAccessor executeSQLString: statement.
	pk := rows first first.
	aDatabaseRow at: aDatabaseField put: pk!

postWriteAssignSequenceValueFor: aField in: aRow using: anAccessor!

reserveViaBackupMechanism: anInteger in: aSession
	"For interbase, the generator mechanism should always work"
	self error: 'This should never happen'.!

reserveViaTableSelect: anInteger in: aSession for: aTable
	"Interbase returns the top of the generated range in a single row. Do the iteration ourselves"
	| rows command max |
	command := self
		tableSelectCommandFor: aTable
		in: aSession
		reserving: anInteger.
	command parameters: (Array with: anInteger).
	command useBinding: false.
	rows := aSession accessor executeCommand: command.
	max := rows first first.
	max - anInteger + 1 to: max do: [:i | self reservedNumbers add: i].! !
!FirebirdSequence categoriesFor: #creationString!public! !
!FirebirdSequence categoriesFor: #dropString!private!sequencing! !
!FirebirdSequence categoriesFor: #getSequenceValueFromDatabaseFor:in:using:!private!sequencing! !
!FirebirdSequence categoriesFor: #postWriteAssignSequenceValueFor:in:using:!public!sequencing! !
!FirebirdSequence categoriesFor: #reserveViaBackupMechanism:in:!As yet unclassified!private! !
!FirebirdSequence categoriesFor: #reserveViaTableSelect:in:for:!private!sequencing! !

JustSelectTheMaximumSequenceValueAndAddOne guid: (GUID fromString: '{88BC9707-DA30-41DC-99CC-33041DF6E81C}')!
JustSelectTheMaximumSequenceValueAndAddOne comment: 'This is a sequence that just does a select max(primaryKeyFieldName) for the table in question and adds one to it. This is, um, less-than-perfectly efficient, and I''m not at all clear that it''ll work for a multi-user system. But it''s what Store used to do on SQL Server, so we''d like to be able to mimic it. And it''s actually easier to preallocate for than identity columns, which is nice. This isn''t really named in the same sense as the others, but it''s useful to inherit the named preallocation behaviour, and we store the field to select against in the name.

Instance Variables:
	tableName	<DatabaseTable>	the table we sequence.
	field <DatabaseField> the field we sequence.'!
!JustSelectTheMaximumSequenceValueAndAddOne categoriesForClass!Unclassified! !
!JustSelectTheMaximumSequenceValueAndAddOne methodsFor!

field
	field isNil ifTrue: [field := table primaryKeyFields first].
	^field.!

field: aDatabaseField
	field := aDatabaseField.!

getSequenceValueFromDatabaseFor: aField in: aRow using: aSession

	| stream rows value |
	"I repeat: ick"
	stream := WriteStream on: (String new: 50).
	stream nextPutAll: 'SELECT MAX('.
	aField printSQLOn: stream withParameters: #().
	stream nextPutAll: ') FROM '.
	aRow table printSQLOn: stream withParameters: #().
	rows := aSession accessor executeSQLString: stream contents.
	value := rows first first isNil ifTrue: [1] ifFalse: [rows first first + 1].
	aRow at: aField put: value.
	^value.!

reserveSequenceNumbers: anInteger in: anAccessor for: aTable
	| numberNeeded firstNumber fakeRow |
	numberNeeded := anInteger - self availableReservedNumbers.
	numberNeeded <= 0 ifTrue: [^self].
	fakeRow := DatabaseRow newForTable: self field table.
	firstNumber := self
		getSequenceValueFromDatabaseFor: self field
		in: fakeRow
		using: anAccessor.
	self reservedNumbers addAll: (firstNumber to: firstNumber + numberNeeded - 1).!

table: aTable
	table := aTable.! !
!JustSelectTheMaximumSequenceValueAndAddOne categoriesFor: #field!accessing!private! !
!JustSelectTheMaximumSequenceValueAndAddOne categoriesFor: #field:!accessing!private! !
!JustSelectTheMaximumSequenceValueAndAddOne categoriesFor: #getSequenceValueFromDatabaseFor:in:using:!other!private! !
!JustSelectTheMaximumSequenceValueAndAddOne categoriesFor: #reserveSequenceNumbers:in:for:!other!private! !
!JustSelectTheMaximumSequenceValueAndAddOne categoriesFor: #table:!accessing!private! !

MySQLSequence guid: (GUID fromString: '{D288ED14-D987-46FC-B2B6-96D97FB209C3}')!
MySQLSequence comment: 'This is a sequence type for MySQL AutoIncrement fields.'!
!MySQLSequence categoriesForClass!Unclassified! !
!MySQLSequence methodsFor!

getSequenceValueFromDatabaseFor: aField in: aRow using: aSession!

isIdentityColumn
	^true.!

postWriteAssignSequenceValueFor: aDatabaseField in: aDatabaseRow using: anAccessor 
	aDatabaseRow at: aDatabaseField
		put: ((anAccessor 
				executeSQLString: 'SELECT LAST_INSERT_ID()') first atIndex: 1).!

reserveSequenceNumbers: anInteger in: aSession for: aTable
	"No real sequences here, just identity columns, which we can't pre-allocate"! !
!MySQLSequence categoriesFor: #getSequenceValueFromDatabaseFor:in:using:!private!sequencing! !
!MySQLSequence categoriesFor: #isIdentityColumn!private!testing! !
!MySQLSequence categoriesFor: #postWriteAssignSequenceValueFor:in:using:!private!sequencing! !
!MySQLSequence categoriesFor: #reserveSequenceNumbers:in:for:!private!sequencing! !

OracleSequence guid: (GUID fromString: '{E2D88006-E124-4DE7-AE5C-FE90D9871683}')!
OracleSequence comment: 'This supports Oracle sequence using select nextval from sys.allObjects. It also supports getting multiple sequence numbers at once. Going against AllObjects seemed to work much better than going against any real table.

Instance Variables:
	accessProtect	<RecursionLock | Semaphore>	Mutual exclusion on our sequence allocation, since we may need to protect our cache of already-allocated numbers, and prevent two different processes from going to the database simultaneously. This might want to be pushed up to a superclass so it can be shared.
	count	<Number>	How many identifiers have we gotten in a row without going to the database.
	sequenceIncrement	<SmallInteger>	By how much does the sequence in the database increment on each nextVal call.

'!
!OracleSequence categoriesForClass!Unclassified! !
!OracleSequence methodsFor!

alwaysGetFromDb
	^sequenceIncrement = 1!

creationString

	^'create sequence ', self qualifiedName.!

getSequenceValueFromDatabaseFor: aField in: aRow using: anAccessor 
	aRow at: aField put: (self nextSequenceNumber: anAccessor)!

initialize
	super initialize.
	count := 0.
	sequenceIncrement := 1.
	"This seems to hang on the semaphore in ObjectStudio. Fake around it"
	accessProtect := Dialect isObjectStudio ifTrue: [nil] ifFalse: [Semaphore forMutualExclusion].!

name: aString
	name := aString copyFrom: 1 to: (aString size min: OraclePlatform new maximumLengthOfTableName).!

nextSequenceNumber: anAccessor 
	accessProtect critical: 
			[count := count + 1.
			^(self alwaysGetFromDb or: [count \\ sequenceIncrement = 1]) 
				ifTrue: 
					[count := (anAccessor 
								executeSQLString: self nextSequenceNumberSelectString) first 
								first]
				ifFalse: [count]]!

nextSequenceNumberSelectString
	^'SELECT ' , self qualifiedName , '.NEXTVAL FROM DUAL'!

postWriteAssignSequenceValueFor: aField in: aRow using: anAccessor!

reserveViaBackupMechanism: anInteger in: aSession
	"The regular mechanism didn't work, (probably because there weren't enough rows in the table), try a different technique. Oracle doesn't repeatedly evaluate nextval if we do it multiple times on a line, so instead guess at a table we know should have lots of rows in Oracle and use it"

	| rows command |
	command := self backupReserveCommand: anInteger in: aSession.
	command blockFactor: anInteger.
	rows := aSession accessor executeCommand: command.
	rows do: [:each | self reservedNumbers add: each first].!

reserveViaTableSelect: anInteger in: aSession for: aTable
	"In Oracle, there can be performance issues selecting against our tables, which we don't seem to get if we just use SYS.ALLOBJECTS. I'm suspicious the problem is if we have fragmentation of the file containing the table, but regardless, ALLOBJECTS seems consistently fast, so always use it"
	self reserveViaBackupMechanism: anInteger in: aSession.

"	| rows command |
	command := self
		tableSelectCommandFor: aTable
		in: aSession
		reserving: anInteger.
	command parameters: (Array with: anInteger).
	rows := aSession accessor executeCommand: command.
	rows do: [:each | self reservedNumbers add: each first]."!

sequenceIncrement
	^sequenceIncrement!

sequenceIncrement: aNumber 
	sequenceIncrement := aNumber! !
!OracleSequence categoriesFor: #alwaysGetFromDb!private!testing! !
!OracleSequence categoriesFor: #creationString!public! !
!OracleSequence categoriesFor: #getSequenceValueFromDatabaseFor:in:using:!public! !
!OracleSequence categoriesFor: #initialize!initalize-release!private! !
!OracleSequence categoriesFor: #name:!accessing!private! !
!OracleSequence categoriesFor: #nextSequenceNumber:!private! !
!OracleSequence categoriesFor: #nextSequenceNumberSelectString!private! !
!OracleSequence categoriesFor: #postWriteAssignSequenceValueFor:in:using:!public! !
!OracleSequence categoriesFor: #reserveViaBackupMechanism:in:!private! !
!OracleSequence categoriesFor: #reserveViaTableSelect:in:for:!private! !
!OracleSequence categoriesFor: #sequenceIncrement!accessing!private! !
!OracleSequence categoriesFor: #sequenceIncrement:!accessing!private! !

PGSequence guid: (GUID fromString: '{3E4397CE-723A-4761-B23C-DCB708B48495}')!
PGSequence comment: 'This implements postgresql sequences using nextVal. The mechanism is similar to Oracle sequences, but we don''t have to give it a table to select against when all we want is a bunch of numbers, so the implementation is simpler.'!
!PGSequence categoriesForClass!Unclassified! !
!PGSequence methodsFor!

getSequenceValueFromDatabaseFor: aDatabaseField in: aDatabaseRow using: anAccessor

	| statement rows pk |

	"this could be made smarter by aggregating keys to fetch into a single sql query - maybe later -tb"
	statement := 'select nextval(''',  self qualifiedName, ''')'.

	rows := anAccessor executeSQLString: statement.
	pk := rows first first.

	aDatabaseRow at: aDatabaseField put: pk.!

postWriteAssignSequenceValueFor: aDatabaseField in: aDatabaseRow using: anAccessor

"No-op;  for user information, we show, commented, the code that was provided here circa Glorp 0.2.*.
	aDatabaseRow at: aDatabaseField
		put: (aSession accessor 
				executeSQLString: 'SELECT CURRVAL(' , self qualifiedName 	, ')')."!

reserveViaBackupMechanism: anInteger in: aSession
	"The regular mechanism didn't work, (probably because there weren't enough rows in the table), try a different technique."

	| command row |
	command := self backupReserveCommand: anInteger in: aSession.
	row := (aSession accessor executeCommand: command) first.
	self reservedNumbers addAll: row.!

tableToSelectFromIn: session for: aTable
	"Sometimes it's inefficient to select sequence values against our own table and we can use a database-specific system table instead to be much faster. Subclasses can override where this is the case"
	^(session databaseTableClass named: 'pg_attribute').! !
!PGSequence categoriesFor: #getSequenceValueFromDatabaseFor:in:using:!private!SQL! !
!PGSequence categoriesFor: #postWriteAssignSequenceValueFor:in:using:!public!SQL! !
!PGSequence categoriesFor: #reserveViaBackupMechanism:in:!private!SQL! !
!PGSequence categoriesFor: #tableToSelectFromIn:for:!public!SQL! !

SQLServerSequence guid: (GUID fromString: '{D7D643A3-795D-4E05-8303-02CCE8CB8F0A}')!
SQLServerSequence comment: 'This implements SQL Server sequences, which aren''t really sequences at all, but identity columns. That means we can''t do more than one at a time, and we have to do an extra select to get the value back.'!
!SQLServerSequence categoriesForClass!Unclassified! !
!SQLServerSequence methodsFor!

getSequenceValueFromDatabaseFor: aField in: aRow using: aSession!

isIdentityColumn
	^true.!

postWriteAssignSequenceValueFor: aDatabaseField in: aDatabaseRow using: anAccessor 
	aDatabaseRow at: aDatabaseField
		put: ((anAccessor 
				executeSQLString: 'SELECT @@IDENTITY') first atIndex: 1).!

reserveSequenceNumbers: anInteger in: aSession for: aTable
	"No real sequences here, just identity columns, which we can't pre-allocate"! !
!SQLServerSequence categoriesFor: #getSequenceValueFromDatabaseFor:in:using:!private!sequencing! !
!SQLServerSequence categoriesFor: #isIdentityColumn!private!testing! !
!SQLServerSequence categoriesFor: #postWriteAssignSequenceValueFor:in:using:!private!sequencing! !
!SQLServerSequence categoriesFor: #reserveSequenceNumbers:in:for:!private!sequencing! !

DB2Sequence guid: (GUID fromString: '{649C87EE-5F1C-46D3-9C8A-D957D029B3C1}')!
DB2Sequence comment: 'DB2Sequence implements DB2 sequences using nextVal. The mechanism is similar to that of Postgres.
Get a sequence number using a query like this. 
	SELECT NEXT VALUE FOR my_sequence.
Note that DB2 sequences are only supported in single partition databases.

Instance Variables
	accessProtect	<RecursionLock | Semaphore>	Mutual exclusion on our sequence allocation, since we may need to protect our cache of already-allocated numbers, and prevent two different processes from going to the database simultaneously. This might want to be pushed up to a superclass so it can be shared.
	count	<Number>	How many identifiers have we gotten in a row without going to the database.
	sequenceIncrement	<SmallInteger>	By how much does the sequence in the database increment on each nextVal call.

'!
!DB2Sequence categoriesForClass!Unclassified! !
!DB2Sequence methodsFor!

creationString

	^'create sequence ', self qualifiedName.!

name: aString
	name := aString copyFrom: 1 to: (aString size min: DB2Platform new maximumLengthOfTableName).!

nextSequenceNumberSelectString
	"Using SYSIBM is not ideal because it's supposed to be internal, but it seems to be accepted."

	^'SELECT NEXT VALUE FOR ' , self qualifiedName , ' FROM SYSIBM.SYSDUMMY1'! !
!DB2Sequence categoriesFor: #creationString!public! !
!DB2Sequence categoriesFor: #name:!accessing!public! !
!DB2Sequence categoriesFor: #nextSequenceNumberSelectString!private! !

AdHocVersionGenerator guid: (GUID fromString: '{AFFA3EDD-33E3-4944-B3D0-51847228AAF5}')!
AdHocVersionGenerator comment: 'This is a version generator that takes an arbitrary block and runs it.

Instance Variables:
	block	<BlockClosure>	The block we evaluate to get the next version.
	numberOfArguments	<Integer>	Generated automatically. How many arguments does block take. If zero, we invoke it without our arguments. If not, we give it the arguments to use.

'!
!AdHocVersionGenerator categoriesForClass!Unclassified! !
!AdHocVersionGenerator methodsFor!

block
	^block!

block: aBlock
	block := aBlock.
	numberOfArguments := Dialect argumentCountFor: block.!

nextValueFor: aField in: aRow
	"Get the next value by evaluating the block. If it doesn't care about the field or row, don't pass them as arguments. If it does, then give it both."
	numberOfArguments = 0 ifTrue: [^block value] ifFalse: [^block value: aField value: aRow].! !
!AdHocVersionGenerator categoriesFor: #block!accessing!private! !
!AdHocVersionGenerator categoriesFor: #block:!accessing!private! !
!AdHocVersionGenerator categoriesFor: #nextValueFor:in:!private!sequencing! !

!AdHocVersionGenerator class methodsFor!

block: aBlock
	^self new block: aBlock.! !
!AdHocVersionGenerator class categoriesFor: #block:!instance creation!private! !

TimestampVersionGenerator guid: (GUID fromString: '{4A9E54FA-0DD8-4EB2-8D61-FEBFC68686A8}')!
TimestampVersionGenerator comment: 'A generator that puts Timestamp now into fields when it is invoked.'!
!TimestampVersionGenerator categoriesForClass!Unclassified! !
!TimestampVersionGenerator methodsFor!

nextValueFor: aField in: aRow
	"If a platform maps the timestamp class to something else (e.g. SQLite maps timestamps to strings as it has no support for time-related types), then it must be allowed to do so here as well.  For most platforms, the wrapping #toTimestamp:for: call will be a noop."

	^aField type platform
		toTimestamp: Dialect timestampNow
		for: aField type! !
!TimestampVersionGenerator categoriesFor: #nextValueFor:in:!private!sequencing! !

VersionNumberVersionGenerator guid: (GUID fromString: '{648C7850-1201-4905-BB3E-6F609E97A964}')!
VersionNumberVersionGenerator comment: 'This generates version numbers by adding one to the previous value of the designated field.'!
!VersionNumberVersionGenerator categoriesForClass!Unclassified! !
!VersionNumberVersionGenerator methodsFor!

nextValueFor: aField in: aRow
	| oldValue |
	oldValue := aRow at: aField ifAbsent: [0].
	^(oldValue isNil ifTrue: [0] ifFalse: [oldValue]) + 1! !
!VersionNumberVersionGenerator categoriesFor: #nextValueFor:in:!private!sequencing! !

"Binary Globals"!

