| package |
package := Package name: 'GlorpMappings'.
package paxVersion: 1;
	basicComment: 'This package defines the Descriptor and the various different Mappings and AttributeModels it can contain.

This is part of the GLORP system, an open-source library (LGPL(S)) for persisting Smalltalk objects in relational databases. For more information see the #notice property, the accompanying files and/or the web sites http://www.glorp.org or http://www.sourceforge.net/projects/glorp 
'.

package basicPackageVersion: 'image'.


package classNames
	add: #AdHocMapping;
	add: #BasicDictionaryMapping;
	add: #BasicTypeResolver;
	add: #ConditionalMapping;
	add: #ConditionalToManyMapping;
	add: #ConstantMapping;
	add: #Descriptor;
	add: #DescriptorSystem;
	add: #DictionaryMapping;
	add: #DirectMapping;
	add: #DirectToManyMapping;
	add: #DynamicDescriptorSystem;
	add: #EmbeddedValueOneToOneMapping;
	add: #FilteredTypeMapping;
	add: #FilteredTypeResolver;
	add: #GlorpAttributeModel;
	add: #GlorpClassModel;
	add: #GlorpMappingsPackage;
	add: #HorizontalTypeMapping;
	add: #HorizontalTypeResolver;
	add: #IdentityTypeMapping;
	add: #IdentityTypeResolver;
	add: #ManyToManyMapping;
	add: #Mapping;
	add: #OneToManyMapping;
	add: #OneToOneMapping;
	add: #RelationshipMapping;
	add: #SQLite3PragmaMapping;
	add: #ToManyMapping;
	add: #TypeMapping;
	add: #TypeResolver;
	yourself.

package methodNames
	add: #GlorpSession -> #databaseTableClass;
	add: #GlorpSession -> #embeddedObjectsFor:;
	add: #SimpleQuery -> #asGlorpExpressionOn:;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: 'GlorpCore';
	add: 'GlorpDatabase';
	add: 'GlorpExpressions';
	add: 'GlorpMisc';
	add: 'GlorpQueries';
	add: 'GlorpUnitOfWork';
	yourself).

package!

"Class Definitions"!

Object subclass: #Descriptor
	instanceVariableNames: 'tables mappings system mappedFields cachePolicy typeResolver mapsPrimaryKeys keyFields classModel multipleTableJoin imaginaryTableReplacements mappingsForFields'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #DescriptorSystem
	instanceVariableNames: 'session platform descriptors tables sequences typeResolvers cachePolicy allClasses useDirectAccessForMapping classModels useJoinsForAnySatisfy'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #GlorpAttributeModel
	instanceVariableNames: 'name type collectionType keyType classModel attributeIndex useDirectAccess setSelector canRead canWrite'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #GlorpClassModel
	instanceVariableNames: 'system describedClass subclasses attributes'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #GlorpMappingsPackage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #Mapping
	instanceVariableNames: 'descriptor attribute debugRead debugWrite'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #TypeResolver
	instanceVariableNames: 'members system'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DescriptorSystem subclass: #DynamicDescriptorSystem
	instanceVariableNames: 'attributeNameToFieldNameBlock'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Mapping subclass: #AdHocMapping
	instanceVariableNames: 'fromDbMappingBlock toDbMappingBlock mappedFields shouldProxy'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Mapping subclass: #ConditionalMapping
	instanceVariableNames: 'conditionalField conditionalMethod cases otherwiseCase conditionalFieldMapping'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Mapping subclass: #ConstantMapping
	instanceVariableNames: 'constantValue valueIsSession'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Mapping subclass: #DirectMapping
	instanceVariableNames: 'field converter shouldProxy query'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Mapping subclass: #RelationshipMapping
	instanceVariableNames: 'query join usesLinkTable shouldProxy linkTableJoin targetTableJoin reverseJoin shouldUseFilteredRead rowMapKeyConstructorBlock controlsTables isExclusive linkFields proxyAction'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Mapping subclass: #TypeMapping
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
RelationshipMapping subclass: #OneToOneMapping
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
RelationshipMapping subclass: #ToManyMapping
	instanceVariableNames: 'orderBy shouldWriteTheOrderField'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
OneToOneMapping subclass: #EmbeddedValueOneToOneMapping
	instanceVariableNames: 'fieldTranslation'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ToManyMapping subclass: #ConditionalToManyMapping
	instanceVariableNames: 'conditional'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ToManyMapping subclass: #DictionaryMapping
	instanceVariableNames: 'keyMapping valueMapping associationDescriptor'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ToManyMapping subclass: #DirectToManyMapping
	instanceVariableNames: 'resultField'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ToManyMapping subclass: #ManyToManyMapping
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ToManyMapping subclass: #OneToManyMapping
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ToManyMapping subclass: #SQLite3PragmaMapping
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
DictionaryMapping subclass: #BasicDictionaryMapping
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TypeMapping subclass: #FilteredTypeMapping
	instanceVariableNames: 'field key keyDictionary'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TypeMapping subclass: #HorizontalTypeMapping
	instanceVariableNames: 'mappedClass isAbstract'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TypeMapping subclass: #IdentityTypeMapping
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TypeResolver subclass: #BasicTypeResolver
	instanceVariableNames: 'concreteMembers subclassDescriptorsBuilt rootDescriptor rootClass'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TypeResolver subclass: #IdentityTypeResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
BasicTypeResolver subclass: #FilteredTypeResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
BasicTypeResolver subclass: #HorizontalTypeResolver
	instanceVariableNames: 'shouldUseSingleQuery'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!GlorpSession methodsFor!

databaseTableClass

	^system databaseTableClass!

embeddedObjectsFor: anObject
	"Return all the objects embedded into this one"
	| embedded descriptor |
	descriptor := self descriptorFor: anObject.
	descriptor isNil ifTrue: [^#()].
	embedded := OrderedCollection new: 3.
	descriptor mappings do: [:each |
		(each class == EmbeddedValueOneToOneMapping) ifTrue: [
			embedded add: (each getValueFrom: anObject)]].
	^embedded.! !
!GlorpSession categoriesFor: #databaseTableClass!accessing!public! !
!GlorpSession categoriesFor: #embeddedObjectsFor:!private! !

!SimpleQuery methodsFor!

asGlorpExpressionOn: anExpression
	"If we're being converted into an expression, that means we're a subselect inside an expression block. We don't actually have any relationship to anExpression, so create our own base and make a pseudo-mapping with no join to describe the relationship. Don't even give it a string-name, just use ourselves as the key"
	
	| expression localBase phantomMapping phantomExpression |
	phantomMapping := OneToOneMapping new
		referenceClass: self resultClass;
		attributeName: #temporaryAttributeFromSubselect;
		join: Join new;
		descriptor: (anExpression system descriptorFor: self resultClass).
	phantomExpression := anExpression getMapping: phantomMapping named: self.
	expression := GeneralSubSelectExpression 
		basedOn: phantomExpression.
	expression query: self.
	localBase := SubSelectBaseExpression on: phantomExpression.
	self baseExpression: localBase.
	self tablePrefix: ('s', anExpression ultimateBaseExpression availableSubSelectNumber printString, 't').
	^expression.! !
!SimpleQuery categoriesFor: #asGlorpExpressionOn:!converting!private! !

"End of package definition"!

"Source Globals"!

"Classes"!

Descriptor guid: (GUID fromString: '{F6E663B7-3DED-40E4-B0E0-6BC85E26073E}')!
Descriptor comment: 'For each persistent class, there is a descriptor. This is what tells GLORP how to read and write objects of this class. The descriptor is not actually a very complicated object, because most of the information is in the mappings described below. But the descriptor does keep hold of a few per-class pieces of information, notably

Instance Variables:
	cachePolicy	<CachePolicy>	The cache policy to use. If left blank, default to what the system specifies.
	describedClass	<Class>	The persistent class that we map.
	mappedFields	<Collection of: DatabaseField>	The database fields we have mappings for (cached computed value)
	mappings	<(Collection of: Mapping)>	Our mappings.
	mapsPrimaryKeys	<Object>	Do we map the primary keys (cached computed value)
	multipleTableCriteria	<(Collection of: Join)>	description of multipleTableCriteria
	system	<DescriptorSystem>	The descriptor system that we belong to.
	tables	<(SequenceableCollection of: (DatabaseTable))>	The tables that we map
	typeResolver	<TypeResolver>	description of typeResolver

'!
!Descriptor categoriesForClass!Unclassified! !
!Descriptor methodsFor!

addMapping: aMapping 
	self basicAddMapping: aMapping.
	aMapping hasAttribute ifTrue: [
		(aMapping classModel attributeNamed: aMapping attribute name) == aMapping attribute ifFalse: [self error: 'If you have a classModel you must use newMapping: rather than addMapping:']].!

addMultipleTableJoin: anExpression 
	self multipleTableJoin add: anExpression!

addTable: aDatabaseTable 
	tables add: aDatabaseTable.!

allDescribedConcreteClasses
	^ self typeMapping allDescribedConcreteClasses!

allMappingsForField: aField 
	"Return all of the mappings that use this field"
	^mappings select: [:each | each mappedFields includes: aField]!

allTables
	"Return all tables used by this descriptor or its subclasses. For a non-inheritance descriptor, this is the same as tables."
	^self typeResolver allTablesFor: self.!

basicAddMapping: aMapping 
	mappings := mappings, (Array with: aMapping).
	aMapping descriptor: self.
	self resetMappedFields.!

buildReplacementTableSelect
	"We have an imaginary table which is defined by selecting from our attributes instead of from the table. Build the query for that."

	| sourceMappings targetMappings mainQuery subQuery |
	mainQuery := nil.
	sourceMappings := imaginaryTableReplacements
		collect: [:each | self mappingForAttributeNamed: each first].
	targetMappings := imaginaryTableReplacements
		collect: [:each | self mappingForAttributeNamed: each last].
	sourceMappings
		with: targetMappings
		do:
			[:eachSourceMapping :eachTargetMapping | 
			subQuery := self
				buildReplacementTableSelectFrom: eachSourceMapping
				to: eachTargetMapping.
			mainQuery := mainQuery isNil
				ifTrue: [subQuery]
				ifFalse: [mainQuery unionAll: subQuery]].
	mainQuery session: self session.
	self primaryTable replacementSubSelect: mainQuery.!

buildReplacementTableSelectFrom: sourceMapping to: targetMapping
	"Build a select statement that combines the two mappings. We expect that both are relationships from a single object whose table is imaginary, so we need to knit together the information from the mappings to join the real tables directly together."

	| fullJoin targetJoin sourceJoin subQuery join |
	subQuery := Query returningManyOf: sourceMapping referenceClass.
	subQuery session: self session.	
	subQuery setUpExpressions.
	"Invert because these will both start with the imaginary one on the left and we want to merge them together"
	sourceJoin := (sourceMapping join
		replacing: self primaryTable
		withEquivalenceFrom: targetMapping join) inverseJoin.
	targetJoin := targetMapping join
		replacing: self primaryTable
		withEquivalenceFrom: sourceMapping join.
	fullJoin := sourceJoin = targetJoin
		ifTrue: [sourceJoin]
		ifFalse: [sourceJoin , targetJoin].
	join := fullJoin asGeneralGlorpExpression asExpressionJoiningSource: subQuery baseExpression toTarget: subQuery baseExpression.
	subQuery addJoin: join.
	sourceMapping usesLinkTable ifTrue: [subQuery addJoin: (sourceMapping  expressionFromLinkToReferenceTableWithBase: subQuery baseExpression)].
	targetMapping usesLinkTable ifTrue: [subQuery addJoin: (targetMapping  expressionFromLinkToReferenceTableWithBase: subQuery baseExpression)].
	self
		setRetrievalExpressionsInReplacementTableForQuery: subQuery
		fromMapping: sourceMapping
		toMapping: targetMapping.
	^subQuery.!

cachePolicy
	cachePolicy isNil ifTrue: [^system cachePolicy].
	^cachePolicy!

cachePolicy: aCachePolicy
	cachePolicy:= aCachePolicy!

classesRequiringIndependentQueries

	^ self typeResolver classesRequiringIndependentQueriesFor: self describedClass!

classModel
	^classModel!

classModel: aGlorpClassModel
	classModel := aGlorpClassModel!

computeMapsPrimaryKeys
	| primaryKeyFields |
	self tables isEmpty ifTrue: [^false].
	primaryKeyFields := self primaryTable primaryKeyFields.
	primaryKeyFields isEmpty ifTrue: [^false].
	primaryKeyFields
		do: [:each | (self mappedFields anySatisfy: [ :eachMappedField |
			eachMappedField = each]) ifFalse: [^false]].
	^true.!

createDeleteRowFor: anObject table: aTable in: aRowMap
	"Create records for rows that require deletion"

	(self primaryKeyMappingsFor: aTable)
		do: [:each | 
			each first mapFromObject: anObject intoRowsIn: aRowMap].!

createDeleteRowsFor: anObject in: aRowMap
	"Create records for rows that require deletion"

	|  |
	anObject class == self describedClass ifFalse: [self error: 'wrong descriptor for this object'].
	"If we don't map primary keys, then we're something that gets embedded, and a delete isn't necessary"
	self mapsPrimaryKeys ifFalse: [^self]. 

	self tables do: [:eachTable |
		self createDeleteRowFor: anObject table: eachTable in: aRowMap.
		aRowMap rowForTable: eachTable withKey: anObject ifPresent: [:eachRow | eachRow forDeletion: true]].
	"It's possible that we might not have any direct mapping for a secondary table's primary keys, so 
allow the multiple table criteria to specify them if that's the only one. If they're not, then they don't do any harm"
	self multipleTableJoin do: [:each |
		each mapFromSource: anObject andTarget: anObject intoRowsIn: aRowMap.
		each allTables do: [:eachTable |
			aRowMap rowForTable: eachTable withKey: anObject ifPresent: [:eachRow | eachRow forDeletion: true]]].

	self mappings do: [:each | each  createDeleteRowFor: anObject in: aRowMap].!

createRowsFor: anObject in: aRowMap
	| row |
	anObject class == self describedClass
		ifFalse: [self error: 'wrong descriptor for this object'].
	mappings do: [:each | 
			each stopIfDebuggingWrite.
			each mapFromObject: anObject intoRowsIn: aRowMap].

	"If we haven't written out our primary keys, then make sure we do so, but if absolutely none of our mappings have written anything, then just ignore it."
	row := aRowMap rowForTable: self primaryTable withKey: anObject ifAbsent: [^self].
	self primaryTable primaryKeyFields do: [:each | 
		self setFieldValue: each 	in: row from: anObject].

	"This is a bit of a hack/heuristic. If the multiple table join is an outer join, and the row doesn't exist yet, then assume we won't need it and don't do the join. It means none of our mappings wrote to it. Hope that in that case no one else's will. It seems reasonable that we wouldn't map an object to multiple tables unless we planned to map some field within it, but it's still a bit of a risk."
	self multipleTableJoin do: [:each | 
		(each isOuterJoin not or: [each allTables anySatisfy: [:eachTable | 
			aRowMap includesRowForTable: eachTable withKey: anObject]]) 
				ifTrue: [
					each
						mapFromSource: anObject
						andTarget: anObject
						intoRowsIn: aRowMap]].!

defaultTracing
	^Tracing new.!

describedClass
	^classModel describedClass.!

describedClass: aClass
	classModel := self system classModelFor: aClass.!

describedConcreteClassFor: row withBuilder: builder
	"Lookup the class that is represented by the row when there is a possibility
		of this row representing any class within a hierarchy. "
	^self typeResolver describedConcreteClassFor: row withBuilder: builder descriptor: self!

directMapping
	^self newMapping: DirectMapping.!

directMappingFor: attributeName
	"Create a direct mapping from the attribute of this name to the field in the table with a matching name. For convenience when there's a good correspondence between the field and instance variable names."
	| field |
	self tables do: [:each |
		field := each fieldNamed: attributeName ifAbsent: [nil] caseSensitive: false].
	field isNil ifTrue: [self error: 'Cannot find field named ', attributeName printString].
	^self directMapping from: attributeName to: field.!

directMappingForField: aField 
	"Return a single, direct mapping for this field. There may conceivably be more than one, but they all have to agree, so it shouldn't matter as far as the value. There may also be none."

	^(self mappingsForFields at: aField ifAbsent: [^nil]) 
		detect: [:each | each isRelationship not]
		ifNone: [nil]!

directMappingsForField: aField 
	"Return all the relationship mappings that use this field"

	^(self mappingsForFields at: aField ifAbsent: [^#()]) 
		reject: [:each | each isRelationship].!

fieldFor: aFieldOrString
	"We allow either a field or a string naming the field, and return the appropriate field object."
	aFieldOrString isString ifFalse: [^aFieldOrString].
	self tables do: [:each |
		| field | 
		field := each fieldNamed: aFieldOrString.
		field isNil ifFalse: [^field]].
	self error: 'Cannot find field ', aFieldOrString, '. Please pass in the field object'.!

fieldsForSelectStatement
	| myFields inheritedFields |
	myFields := self mappedFields.
	inheritedFields := self typeResolver fieldsForSelectStatement.
	^inheritedFields isEmpty ifTrue: [myFields] ifFalse: [myFields, inheritedFields].!

imaginaryTableIsDefinedByRelationFrom: sourceAttributeName to: targetAttributeName
	self imaginaryTableReplacements add: (Array with: sourceAttributeName with: targetAttributeName).!

imaginaryTableReplacements
	imaginaryTableReplacements isNil ifTrue: [imaginaryTableReplacements := OrderedCollection new: 4].
	^imaginaryTableReplacements.!

initialize

	mappings := Array new.
	tables := OrderedCollection new: 1.!

isForAssociation
	^self describedClass == Association.!

isTypeMappingRoot
	^self typeResolver isTypeMappingRoot: self!

keyFields
	"Return the fields we use as primary keys for caching purposes. Typically, these are just going to be the primary keys of our primary table. Objects intended to be embedded won't map the primary keys at all. Weird objects may supply their own keys (e.g. functions rather than fields directly)"

	^keyFields == nil
		ifTrue:
			[self mapsPrimaryKeys
				ifTrue: [self primaryTable primaryKeyFields]
				ifFalse: [nil]]
		ifFalse: [keyFields].!

keyFields: aFieldCollection
	"Set the fields we use as primary keys for caching purposes. Typically, these are just going to be the primary keys of our primary table"
	keyFields := aFieldCollection.!

mappedFields
	"Return all the fields that are mapped, in the order that they occur in the table. This specifically omits fields that are not in the selected set of tables (so e.g. even if a mapping uses a link table field, if it's not in the list of tables to select from, we don't want it). We also include mapped, non-constant expressions, which we would normally expect to be functions"
	mappedFields isNil ifTrue: [
		| fieldSet |
		fieldSet := IdentitySet new: mappings size.
		mappings do: [:each | fieldSet addAll: each fieldsForDescriptorSelectStatement].
		mappedFields := OrderedCollection new.
		tables do: [:each |
			each fields do: [:eachField | (fieldSet includes: eachField) ifTrue: [mappedFields add: eachField]]].
		fieldSet do: [:each | 
 			(each isGlorpExpression and: [each isConstantExpression not]) ifTrue: [mappedFields add: each]]].
	^mappedFields.!

mappingForAttributeNamed: aSymbol

	^mappings detect: [:each | each attributeName == aSymbol] ifNone: [
		self isForAssociation ifTrue: [self valueDescriptor mappingForAttributeNamed: aSymbol]]!

mappingForField: aField

	^(self mappingsForFields at: aField ifAbsent: [^nil]) first.!

mappings
	^ReadStream on: mappings!

mappingsForFields
	"Return a dictionary from fields to the mappings that contain them. This specifically omits fields that are not in the selected set of tables (so e.g. even if a mapping uses a link table field, if it's not in the list of tables to select from, we don't want it). We also include mapped, non-constant expressions, which we would normally expect to be functions"
	mappingsForFields isNil ifTrue: [
		mappingsForFields := IdentityDictionary new: mappings size.
		mappings do: [:eachMapping | 
			eachMapping mappedFields do: [:eachField | 
				(mappingsForFields at: eachField ifAbsentPut: [OrderedCollection new]) add: eachMapping]]].
	^mappingsForFields.!

mapsPrimaryKeys
	mapsPrimaryKeys isNil ifTrue: [mapsPrimaryKeys := self computeMapsPrimaryKeys].
	^mapsPrimaryKeys!

multipleTableJoin
	multipleTableJoin isNil
		ifTrue: [multipleTableJoin := OrderedCollection new: 1].
	^multipleTableJoin.!

newMapping: aClass
	| mapping |
	mapping := aClass new.
	self basicAddMapping: mapping.
	^mapping.!

oneToOneMapping
	^self newMapping: OneToOneMapping.!

populateObject: anObject inBuilder: anElementBuilder
	"Answer an object using the values for the specified fields."

	mappings
		do: [:each | 
			each stopIfDebuggingRead.
			each mapObject: anObject inElementBuilder: anElementBuilder].!

primaryKeyCollectionFor: anObject
	"Find the primary key for anObject. Always return it as a collection."
	| fields |
	anObject yourSelf class == self describedClass
		ifFalse: [self error: 'Wrong descriptor for this object'].
	fields := self primaryTable primaryKeyFields.
	^fields collect: [:each | self valueOfField: each fromObject: anObject].!

primaryKeyExpressionFor: anObject basedOn: anExpression relation: aSymbol
	"anObject can be either a real domain object, or a constant expression containing it, or an expression representing the object"

	| expression value pkMappings |
	value := anObject isGlorpExpression
		ifTrue:
			[anObject isConstantExpression ifTrue: [anObject value] ifFalse: [anObject]]
		ifFalse:
			[anObject class == self describedClass
				ifFalse: [self error: 'Wrong descriptor for this object'].
			anObject].
	self isForAssociation ifTrue: [
		"We just rewrite as self value = x"
		^((anExpression get: #value) get: aSymbol withArguments: (Array with:  value)) rewriteEquality].
	expression := nil.
	pkMappings := self primaryKeyMappings.
	(pkMappings isEmpty ifTrue: [self mappings] ifFalse: [pkMappings])
		do:
			[:each | 
			| clause |
			clause := each
				expressionFor: value
				basedOn: anExpression
				relation: aSymbol.
			expression := aSymbol = #<> ifTrue: [clause OR: expression] ifFalse: [clause AND: expression]].
	^expression.!

primaryKeyFor: anObject
	"Find the primary key for anObject. If it's a single field, just return a single value. Otherwise, return a collection. Caller is expected to know what to expect."
	| fields |
	anObject class == self describedClass
		ifFalse: [self error: 'Wrong descriptor for this object'].
	fields := self primaryTable primaryKeyFields.
	^fields size = 1
		ifTrue: [self valueOfField: (fields at: 1) fromObject: anObject]
		ifFalse:
			[fields collect: [:each | self valueOfField: each fromObject: anObject]].!

primaryKeyMappings
	"Return things that map to our primary key fields. Prefer direct mappings"
	| fields pkMappings |
	fields := self primaryTable primaryKeyFields.
	pkMappings := OrderedCollection new: fields size.
	fields do: [:each |  | mapping |
		mapping := self directMappingForField: each.
		mapping isNil ifTrue: [mapping := self mappingForField: each].
		mapping isNil ifFalse: [pkMappings add: mapping]].
	^pkMappings.!

primaryKeyMappingsFor: aTable
	"Return our primary key mappings as a collection of mappings, one collection per primary key"

	| allMappings |
	allMappings := Dictionary new: aTable primaryKeyFields size.
	aTable primaryKeyFields
		do: [:each | 
			| mappingsForField |
			mappingsForField := self mappingsForFields at: each ifAbsent: [nil].
			mappingsForField isNil ifFalse: [allMappings at: each put: mappingsForField]].
	^allMappings.!

primaryTable
	tables isEmpty ifTrue: [^nil].
	^tables first.!

printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: '('.
	self describedClass printOn: aStream.
	aStream nextPutAll: ')'.!

readBackNewRowInformationFor: anObject!

readBackNewRowInformationFor: anObject in: aRowMap 
	anObject yourSelf class == self describedClass ifFalse: [self error: 'wrong descriptor for this object'].
	mappings do: [:each | each readBackNewRowInformationFor: anObject fromRowsIn: aRowMap]!

referencedIndependentObjectsFrom: anObject do: aBlock

	mappings do: [:each |
		(each referencedIndependentObjectsFrom: anObject) do: [:eachReferencedObject |
			aBlock value: eachReferencedObject]].!

referencedIndependentObjectsWithMappingsFrom: anObject do: aBlock

	mappings do: [:each |
		(each referencedIndependentObjectsFrom: anObject) do: [:eachReferencedObject |
			aBlock value: eachReferencedObject value: each]].!

registerTypeResolver: aResolver
	self registerTypeResolver: aResolver abstract: false!

registerTypeResolver: aResolver abstract: shouldBeAbstract
	shouldBeAbstract ifFalse: [ self beAbstract ].
	aResolver register: self.!

relationshipMappingForField: aField 
	"Return a single, direct mapping for this field. There may conceivably be more than one, but they all have to agree, so it shouldn't matter as far as the value. There may also be none."

	^(self mappingsForFields at: aField ifAbsent: [^nil]) 
		detect: [:each | each isRelationship]
		ifNone: [nil]!

relationshipMappingsForField: aField 
	"Return all the relationship mappings that use this field"

	^(self mappingsForFields at: aField ifAbsent: [^#()]) 
		select: [:each | each isRelationship].!

removeMapping:  aMapping

	mappings := mappings copyWithout: aMapping!

resetMappedFields
	mappedFields := nil.!

session

	^system session.!

setFieldValue: each in: row from: anObject
	"Try to set the value of the field, which is normally a primary key field, in the row, based on data in the object. Useful for the case where we have only a relationship mapping to the object that holds our primary key, and it is uninstantiated"
	| value |
	(row wrapperAt: each ifAbsent: [FieldValueWrapper new]) hasValue
		ifFalse:
			[value := self valueOfField: each fromObject: anObject ifNone: [^self].
			"We're using this to get primary key fields. Nil is not a useful answer."
			value isNil ifTrue: [^self].
			row at: each put: value].!

setRetrievalExpressionsInReplacementTableForQuery: subQuery fromMapping: sourceMapping toMapping: targetMapping
	self primaryTable fields
		do:
			[:each | 
			| sourceReplacement replacement expression |
			sourceReplacement := sourceMapping join targetForSource: each.
			replacement := sourceReplacement isNil
				ifTrue: [targetMapping join targetForSource: each]
				ifFalse: [sourceReplacement].
			expression := replacement isGlorpExpression
				ifTrue: [replacement] 
				ifFalse: [(BaseExpression new getTable: replacement table) getField: replacement].
			expression alias: each name.
			subQuery retrieve: expression].!

setupTracing: aTracing
	"Find all the other objects that need to be read when this one is read"

	self trace: aTracing context: aTracing base.!

supportsOrdering

	typeResolver isNil ifTrue: [^true].
	^typeResolver supportsOrderingFor: self.!

system
	^system!

system: anObject
	system := anObject!

table
	tables isEmpty ifTrue: [^nil].
	^tables first.!

table: aDatabaseTable
	
	| table |
	table := aDatabaseTable isString
		ifTrue: [table := self system tableNamed: aDatabaseTable]
		ifFalse: [aDatabaseTable].
	tables add: table.!

tables
	^tables!

toManyMapping
	^self newMapping: ToManyMapping.!

trace: aTracing context: anExpression
	"For each mapping, check if the relationship is involved in the set of things
to be read"
	mappings do: [:each |
		each trace: aTracing context: anExpression].!

typeMapping
	^mappings detect: [ :each | each isTypeMapping ] ifNone: [
		| mapping |
		mapping := IdentityTypeMapping new.
		self addMapping: mapping.
		mapping]!

typeMapping: aMapping
	self addMapping: aMapping!

typeMappingCriteriaIn: base
	| r l |
	r := ConstantExpression for: self typeMapping keys.
	l := FieldExpression forField: self typeMapping field basedOn: base.
	^l in: r!

typeMappingRoot
	^self typeResolver typeMappingRoot!

typeMappingRootDescriptor
	^self typeResolver typeMappingRootDescriptor!

typeResolver
	typeResolver isNil
		ifTrue: [  IdentityTypeResolver new register: self  ].
	^typeResolver!

typeResolver: anObject
	typeResolver := anObject!

validate
	"Do any tests we can to make sure that this definition makes sense in this context"
	| readableMappings attributeNames |
	self system validateDescriptor: self.
	self mappings do: [:eachMapping |
		eachMapping validate].
	"Validate that we have at most one readable mapping for an instance variable"
	readableMappings := self mappings select: [:each | each canRead and: [each isTypeMapping not and: [each attributeName isEmpty not]]].
	attributeNames := readableMappings collect: [:each | each attributeName].
	(attributeNames asSet size = readableMappings size) 
		ifFalse: [MultipleMappingsForInstanceVariable new
					model: self describedClass;
					readableMappings: readableMappings;
					signal].

	self typeResolver validate.!

valueDescriptor
	| valueMapping |
	valueMapping := self mappingForAttributeNamed: #value.
	valueMapping isNil ifTrue: [^nil].
	^valueMapping referenceDescriptor.!

valueOfField: aField fromObject: anObject
	^self valueOfField: aField fromObject: anObject ifNone: [nil].!

valueOfField: aField fromObject: anObject ifNone: aBlock
	| directMapping notFound |
	directMapping := self directMappingForField: aField.
	directMapping isNil
		ifFalse: [^directMapping valueOfField: aField fromObject: anObject].
	notFound := Object new.
	(self relationshipMappingsForField: aField)
		do:
			[:each | 
			| candidate |
			(each hasValueThatCanProvideField: aField from: anObject)
				ifTrue: [
					candidate := each valueOfField: aField fromObject: anObject ifNone: [notFound].
					candidate == notFound ifFalse: [^candidate]]].
	^aBlock value.! !
!Descriptor categoriesFor: #addMapping:!accessing!private! !
!Descriptor categoriesFor: #addMultipleTableJoin:!accessing!private! !
!Descriptor categoriesFor: #addTable:!accessing!private! !
!Descriptor categoriesFor: #allDescribedConcreteClasses!private!type mapping! !
!Descriptor categoriesFor: #allMappingsForField:!accessing!private! !
!Descriptor categoriesFor: #allTables!accessing!private! !
!Descriptor categoriesFor: #basicAddMapping:!accessing!private! !
!Descriptor categoriesFor: #buildReplacementTableSelect!internal!private! !
!Descriptor categoriesFor: #buildReplacementTableSelectFrom:to:!internal!private! !
!Descriptor categoriesFor: #cachePolicy!accessing!private! !
!Descriptor categoriesFor: #cachePolicy:!accessing!private! !
!Descriptor categoriesFor: #classesRequiringIndependentQueries!private!type resolution! !
!Descriptor categoriesFor: #classModel!accessing!private! !
!Descriptor categoriesFor: #classModel:!accessing!private! !
!Descriptor categoriesFor: #computeMapsPrimaryKeys!private!testing! !
!Descriptor categoriesFor: #createDeleteRowFor:table:in:!deleting!private! !
!Descriptor categoriesFor: #createDeleteRowsFor:in:!deleting!private! !
!Descriptor categoriesFor: #createRowsFor:in:!mapping!private! !
!Descriptor categoriesFor: #defaultTracing!private!tracing! !
!Descriptor categoriesFor: #describedClass!accessing!private! !
!Descriptor categoriesFor: #describedClass:!accessing!private! !
!Descriptor categoriesFor: #describedConcreteClassFor:withBuilder:!internal!private! !
!Descriptor categoriesFor: #directMapping!creating mappings!private! !
!Descriptor categoriesFor: #directMappingFor:!creating mappings!private! !
!Descriptor categoriesFor: #directMappingForField:!accessing!private! !
!Descriptor categoriesFor: #directMappingsForField:!accessing!private! !
!Descriptor categoriesFor: #fieldFor:!creating mappings!private! !
!Descriptor categoriesFor: #fieldsForSelectStatement!accessing!private! !
!Descriptor categoriesFor: #imaginaryTableIsDefinedByRelationFrom:to:!accessing!private! !
!Descriptor categoriesFor: #imaginaryTableReplacements!As yet unclassified!private! !
!Descriptor categoriesFor: #initialize!accessing!private! !
!Descriptor categoriesFor: #isForAssociation!private!testing! !
!Descriptor categoriesFor: #isTypeMappingRoot!private!testing! !
!Descriptor categoriesFor: #keyFields!accessing!private! !
!Descriptor categoriesFor: #keyFields:!accessing!private! !
!Descriptor categoriesFor: #mappedFields!accessing!private! !
!Descriptor categoriesFor: #mappingForAttributeNamed:!accessing!private! !
!Descriptor categoriesFor: #mappingForField:!accessing!private! !
!Descriptor categoriesFor: #mappings!mapping!private! !
!Descriptor categoriesFor: #mappingsForFields!accessing!private! !
!Descriptor categoriesFor: #mapsPrimaryKeys!private!testing! !
!Descriptor categoriesFor: #multipleTableJoin!accessing!private! !
!Descriptor categoriesFor: #newMapping:!creating mappings!private! !
!Descriptor categoriesFor: #oneToOneMapping!creating mappings!private! !
!Descriptor categoriesFor: #populateObject:inBuilder:!mapping!private! !
!Descriptor categoriesFor: #primaryKeyCollectionFor:!mapping!private! !
!Descriptor categoriesFor: #primaryKeyExpressionFor:basedOn:relation:!mapping!private! !
!Descriptor categoriesFor: #primaryKeyFor:!mapping!private! !
!Descriptor categoriesFor: #primaryKeyMappings!mapping!private! !
!Descriptor categoriesFor: #primaryKeyMappingsFor:!deleting!private! !
!Descriptor categoriesFor: #primaryTable!accessing!private! !
!Descriptor categoriesFor: #printOn:!printing!private! !
!Descriptor categoriesFor: #readBackNewRowInformationFor:!internal!private! !
!Descriptor categoriesFor: #readBackNewRowInformationFor:in:!mapping!private! !
!Descriptor categoriesFor: #referencedIndependentObjectsFrom:do:!internal!private! !
!Descriptor categoriesFor: #referencedIndependentObjectsWithMappingsFrom:do:!internal!private! !
!Descriptor categoriesFor: #registerTypeResolver:!private!type mapping! !
!Descriptor categoriesFor: #registerTypeResolver:abstract:!private!type mapping! !
!Descriptor categoriesFor: #relationshipMappingForField:!accessing!private! !
!Descriptor categoriesFor: #relationshipMappingsForField:!accessing!private! !
!Descriptor categoriesFor: #removeMapping:!accessing!private! !
!Descriptor categoriesFor: #resetMappedFields!accessing!private! !
!Descriptor categoriesFor: #session!accessing!private! !
!Descriptor categoriesFor: #setFieldValue:in:from:!mapping!private! !
!Descriptor categoriesFor: #setRetrievalExpressionsInReplacementTableForQuery:fromMapping:toMapping:!internal!private! !
!Descriptor categoriesFor: #setupTracing:!private!tracing! !
!Descriptor categoriesFor: #supportsOrdering!private!testing! !
!Descriptor categoriesFor: #system!accessing!private! !
!Descriptor categoriesFor: #system:!accessing!private! !
!Descriptor categoriesFor: #table!accessing!private! !
!Descriptor categoriesFor: #table:!accessing!private! !
!Descriptor categoriesFor: #tables!accessing!private! !
!Descriptor categoriesFor: #toManyMapping!creating mappings!private! !
!Descriptor categoriesFor: #trace:context:!private!tracing! !
!Descriptor categoriesFor: #typeMapping!accessing!private! !
!Descriptor categoriesFor: #typeMapping:!accessing!private! !
!Descriptor categoriesFor: #typeMappingCriteriaIn:!private!type mapping! !
!Descriptor categoriesFor: #typeMappingRoot!private!type resolution! !
!Descriptor categoriesFor: #typeMappingRootDescriptor!private!type resolution! !
!Descriptor categoriesFor: #typeResolver!accessing!private! !
!Descriptor categoriesFor: #typeResolver:!accessing!private! !
!Descriptor categoriesFor: #validate!private!validation! !
!Descriptor categoriesFor: #valueDescriptor!accessing!private! !
!Descriptor categoriesFor: #valueOfField:fromObject:!mapping!private! !
!Descriptor categoriesFor: #valueOfField:fromObject:ifNone:!mapping!private! !

!Descriptor class methodsFor!

new

	^super new initialize.! !
!Descriptor class categoriesFor: #new!instance creation!private! !

DescriptorSystem guid: (GUID fromString: '{01840814-67C5-435F-9ECD-C0D7BB3F75CD}')!
DescriptorSystem comment: 'This holds onto the Glorp metadata. Each session will have a DescriptorSystem associated with it, which describes the tables, descriptors, and class models that are to be made persistent.

Subclasses must implement the following messages:
	accessing
		allTableNames
		constructAllClasses

Instance Variables:
	allClasses	<Collection of: Behavior>	The classes we have metadata for.
	cachePolicy	<CachePolicy>	The default cache policy to use. Descriptors can override this.
	classModels	<Dictionary from: Behavior to: GlorpClassModel>	Our class models, indexed by class.
	descriptors	<Dictionary from: Behavior to: Descriptor>	Our descriptors, indexed by class.
	platform	<DatabasePlatform>	The platform for which we will be used. This is required because the table definitions will refer to the platform in order to get the appropriate types for fields.
	sequences	<Dictionary from: String to: DatabaseSequence>	The sequences that we have available, indexed by name asUppercase.
	session	<GlorpSession>	The session we provide metadata to.
	tables	<Dictionary from: String to: DatabaseTable>	Our database tables, indexed by name asUppercase.
	typeResolvers	<Dictionary from: Behavior to: TypeResolver>	The type resolvers that we will use for inheritance, indexed by class.
	useDirectAccessForMapping	<Boolean>	By default, should we use direct instance variable access (instVarAt:) or send get/set messages. Individual attributes can override this.
	useJoinsForAnySatisfy	<Boolean>	By default, should we use joins for anySatisfy and similar queries, or should we use EXISTS subselects.

'!
!DescriptorSystem categoriesForClass!Unclassified! !
!DescriptorSystem methodsFor!

addClassesFromDescriptorMethodsFor: aClass
	| match |
	match := 'descriptorFor'.
	Dialect selectorsFor: aClass do: 
			[:each | 
			(each copyFrom: 1 to: (match size min: each size)) = match
				ifTrue:
					[| className actualClass |
					className := each copyFrom: match size + 1 to: each size - 1.
					actualClass := Dialect
						lookupClassNamed: className
						inContextOf: self class
						ifAbsent: [nil].
					actualClass isNil ifFalse: [allClasses add: actualClass]]].!

addClassModelFor: aClass
	^classModels
		at: aClass
		ifAbsentPut: [self newClassModelFor: aClass]!

additionalRelatedRowsFor: aRow in: aRowMap
	"This method serves as a hook. It is called when sorting rows before writing to the database, and can be used to return a list of additional related rows that aren't found by the normal foreign key constraints. These rows will act as an ordering constraint, and will have to be written before aRow."
	^nil.!

addTablesFromInitializeMethodsFor: aClass
	| match |
	match := 'tableFor'. 
	Dialect selectorsFor: aClass do:
			[:each | 
			(each copyFrom: 1 to: (match size min: each size)) = match
				ifTrue:
					[| tableName |
					tableName := each copyFrom: match size + 1 to: each size - 1.
					self tableNamed: tableName]].!

allClasses
	allClasses isEmpty ifTrue: [allClasses := self constructAllClasses].
	^allClasses!

allClassModels
	^self allClasses collect: [:each | 
		self classModelFor: each].!

allDescriptors

	^self allClasses collect: [:each | 
		self descriptorFor: each].!

allSequences
	sequences isEmpty ifFalse: [^sequences].
	sequences := self allSequencesFor: self allTables.
	^sequences.!

allSequencesFor: aCollection
	"Find all the sequences associated with aCollectionOfTables"
	| result |
	result := Dictionary new.
	aCollection do: [:each |
		each allSequences keysAndValuesDo: [:eachName :eachSequence |
				result at: eachName put: eachSequence]].
	^result.!

allTableNames
	"Return a list of all the table names that this system uses."
	^#().!

allTables
	tables isEmpty ifTrue: [self constructAllTables].
	^tables.!

cachePolicy
	"Return the default cache policy that will be used for descriptors that don't specify their own policy"
	cachePolicy isNil ifTrue: [cachePolicy := CachePolicy default].
	^cachePolicy.!

cachePolicy: aCachePolicy
	cachePolicy := aCachePolicy.!

canUseGroupedWrites
	"We can use grouped writes as long as there are no identity columns in use"
	self platform usesIdentityColumns ifFalse: [^true].
	^(self allTables anySatisfy: [:each |
		each primaryKeyFields anySatisfy: [:eachField |
			eachField type isSerial and: [eachField type sequence isIdentityColumn]]]) not.!

canUseGroupedWritesFor: aTable
	"We can use grouped writes as long as there are no identity columns in use"
	self platform usesIdentityColumns ifFalse: [^true].
	^(aTable primaryKeyFields anySatisfy: [:eachField |
			eachField type isSerial and: [eachField type sequence isIdentityColumn]]) not.!

classModelFor: aClass
	(self allClasses includes: aClass) ifFalse: [^nil].
	^classModels
		at: aClass
		ifAbsentPut: [self newClassModelFor: aClass]!

constructAllClasses
	"Subclasses can override this method to add classes. But by default we also look for all descriptor methods in our system, and add them to the class list, so if the methods are written there, then we don't need to do anything further. If the descriptor methods are defined on the classes themselves, then we'll still have to construct the class list manually."

	self constructClassesFromDescriptorMethods.
	^allClasses.!

constructAllTables
	"Subclasses can override this method to add tables to the list. But by default we also look for all table methods in our system, and add them to the class list, so if the methods are written there, then we don't need to do anything further. If the table methods are defined on the classes themselves, then we'll still have to construct the list manually."
	self allTableNames do: [:each | 
		self tableNamed: each asString].
	self constructTablesFromInitializeMethods.
	^tables.!

constructClassesFromDescriptorMethods
	self withAllSuperclassesDo: [:aClass | self addClassesFromDescriptorMethodsFor: aClass].

	self class includeAllClassesWithDescriptorMethods
		ifTrue:
			[Object withAllSubclasses
				do:
					[:each | 
					(each class
						compiledMethodAt: #glorpSetupDescriptor:forSystem:
						ifAbsent: [nil]) isNil ifFalse: [allClasses add: each]]].!

constructTablesFromInitializeMethods
	self withAllSuperclassesDo: [:aClass | self addTablesFromInitializeMethodsFor: aClass].!

converterBetweenStType: aClass andField: aField 
	| selector |
	selector := ('converterBetween' , aClass name , 'AndField:') asSymbol.
	(self respondsTo: selector) ifTrue: [^self perform: selector with: aField].
	^aField converterForStType: aClass!

createMappingIn: aDescriptor fromClassModelForAttributeNamed: aSymbol 
	| attribute referenceModel |
	attribute := aDescriptor classModel attributeNamed: aSymbol.
	referenceModel := self classModelFor: attribute type.
	referenceModel isNil 
		ifTrue: 
			[^(aDescriptor newMapping: DirectMapping) from: aSymbol
				to: (aDescriptor table 
						fieldNamed: (self fieldNameForAttributeName: aSymbol))]
		ifFalse: 
			[attribute isCollectionAttribute 
				ifTrue: [^(aDescriptor newMapping: ToManyMapping) attributeName: aSymbol]
				ifFalse: [^(aDescriptor newMapping: OneToOneMapping) attributeName: aSymbol]]!

createMappingsFromClassModelIn: aDescriptor 
	aDescriptor classModel attributesDo: 
			[:each | 
			(aDescriptor mappingForAttributeNamed: each name) isNil 
				ifTrue: 
					[self createMappingIn: aDescriptor
						fromClassModelForAttributeNamed: each name]]!

databaseIdentitySequenceNamed: aString

	^self databaseSequenceNamed: aString!

databaseSchema

	^ self session schema!

databaseSequenceNamed: aString

	^sequences at: aString asUppercase ifAbsentPut: [platform newDatabaseSequenceNamed: aString].!

databaseSequenceNamed: aString ifAbsentPut: aBlock

	^sequences at: aString asUppercase ifAbsentPut: aBlock.!

databaseTableClass

	^self class databaseTableClass!

defaultTracing

	^Tracing new!

descriptorFor: aClassOrObject 
	| theClass newDescriptor |
	aClassOrObject == Proxy ifTrue: [self error: 'Cannot find descriptor for the class Proxy. Pass in the instance'].
	theClass := aClassOrObject glorpCanHaveDescriptorDirectly 
				ifTrue: [aClassOrObject]
				ifFalse: [aClassOrObject isGlorpProxy 
							ifTrue: [aClassOrObject getValue class]
							ifFalse: [aClassOrObject class]].
	^descriptors
		at: theClass
		ifAbsent: [
			(self allClasses includes: theClass) ifTrue: [
				newDescriptor := self newDescriptorFor: theClass.
				descriptors at: theClass put: newDescriptor.
				self initializeDescriptor: newDescriptor.
				newDescriptor.
			]].!

descriptorForTable: aTable
	"Return a descriptor that uses aTable. If there is more than one, the choice is random."
	^descriptors detect: [:each | each tables includes: aTable] ifNone: [nil].!

descriptorsForTable: aTable
	"Return all descriptors that use aTable."
	^descriptors select: [:each | each tables includes: aTable].!

existingTableNamed: aString

	^tables at: aString asUppercase ifAbsent: [self error: 'missing table'].!

fieldNameForAttributeName: aSymbol 
	^aSymbol asString asUppercase!

flushAllClasses
	allClasses := OrderedCollection new.!

hasDescriptorFor: aClassOrObject 

	^(self descriptorFor: aClassOrObject) notNil.!

initialize
	descriptors := Dictionary new.
	tables := Dictionary new.
	classModels := Dictionary new.
	typeResolvers := Dictionary new.
	sequences := Dictionary new.
	useDirectAccessForMapping := true.
	allClasses := OrderedCollection new.!

initializeClassModel: aClassModel
	| selector |
	selector := ('classModelFor' , aClassModel describedClass name , ':')
		asSymbol.
	(self respondsTo: selector)
		ifTrue: [^self perform: selector with: aClassModel].
	(aClassModel describedClass respondsTo: #glorpSetupClassModel:forSystem:)
		ifTrue:
			[^aClassModel describedClass
				glorpSetupClassModel: aClassModel
				forSystem: self].
	"Otherwise, just initialize it dynamically, so that old code with only descriptors will still work"
	^aClassModel.!

initializeDescriptor: aDescriptor 
	| selector |
	selector := ('descriptorFor' , aDescriptor describedClass name , ':') asSymbol.
	(self respondsTo: selector) 
		ifTrue: [self perform: selector with: aDescriptor]
		ifFalse: [
			(aDescriptor describedClass respondsTo: #glorpSetupDescriptor:forSystem:)
				ifTrue: [aDescriptor describedClass
								glorpSetupDescriptor: aDescriptor
								forSystem: self]
				ifFalse: [self noDescriptorInitializerFound: aDescriptor]].!

initializeTable: newTable
	| initializerName |
	initializerName := ('tableFor' , (newTable name asUppercase) , ':') asSymbol.
	(self respondsTo: initializerName)
		ifTrue: [self perform: initializerName with: newTable]
		ifFalse: [self noTableInitializerFound: newTable].
	newTable postInitializeIn: self.!

joinFor: aMapping toTables: toTables fromConstraints: fromConstraints toConstraints: toConstraints
	"Return the join to be used for aMapping. The set of tables we expect this to be related to, and the constraints from this table, and to this table are provided as possibly useful input."
	| numberOfConstraints |
	numberOfConstraints := (Set new addAll: toConstraints; addAll: fromConstraints; yourself) size.
	numberOfConstraints = 0 ifTrue: [
		CannotAutomaticallyDetermineJoin new 
			tables: toTables;
			constraints: (toConstraints, fromConstraints);
			signal: 'Cannot automatically determine join, no constraints found.'].
	numberOfConstraints > 1 ifTrue: [
		CannotAutomaticallyDetermineJoin new
			tables: toTables;
			constraints: (toConstraints, fromConstraints);
			signal: 'Cannot automatically determine join, too many constraints found.'].
	^toConstraints isEmpty ifTrue: [fromConstraints first asReverseJoin] ifFalse: [toConstraints first asJoin].!

maximumSizeToGroupWriteFor: aCollectionOfDatabaseRows
	"If we are going to group write, how many rows of this collection should we do it for at once. Normally we delegate this to the platform, but we allow the system to intervene if desired"
	^platform maximumSizeToGroupWriteFor: aCollectionOfDatabaseRows.!

newClassModelFor: aClass

	| newClassModel |
	newClassModel := GlorpClassModel new.
	newClassModel system: self.
	newClassModel describedClass: aClass.
	self initializeClassModel: newClassModel.
	^newClassModel.!

newDescriptorFor: aClass

	| newDescriptor |
	(self allClasses includes: aClass) ifFalse: [^nil].
	newDescriptor := Descriptor new.
	newDescriptor system: self.
	newDescriptor classModel: (self classModelFor: aClass).
	^newDescriptor.!

newTypeResolverFor: aClass 
	| selector |
	(self allClasses includes: aClass) ifFalse: [^nil].
	selector := ('typeResolverFor' , aClass name) asSymbol.
	^(self respondsTo: selector) 
		ifTrue: [self perform: selector]
		ifFalse: [aClass glorpTypeResolver]!

noDescriptorInitializerFound: aDescriptor
	self error: 'No descriptor initializer found for ', aDescriptor printString.!

noTableInitializerFound: aDatabaseTable
	self error: 'No table initializer found for ', aDatabaseTable printString.!

platform

	^platform!

platform: dbPlatform

	platform := dbPlatform!

privateAndBasicAddDescriptorFor: anObject as: aDescriptor
	"Just add the descriptor and don't try to validate. anObject may not actually even be a class. Assume the descriptor is already properly initialized"
	descriptors at: anObject put: aDescriptor.
	aDescriptor system: self.!

sequenceNameForField: aDatabaseField
	"Return the name of the sequence we will use for a field. Subclasses can override to use different naming conventions."
	^(aDatabaseField table sqlString), '_', (aDatabaseField name), '_seq'!

sequenceTypeNamed: aString
	"Return a sequence type whose sequence has the given name."
	^self platform sequence sequence: (self databaseSequenceNamed: aString).!

session
	^session!

session: anObject
	session := anObject!

sessionForLogin: aGlorpLogin
	"Create a session associated with the given login, and using us as a descriptor system. This is the easiest way to get a full blown session going."
	| newSession |
	platform isNil ifTrue: [self platform: aGlorpLogin database].
	aGlorpLogin database class = self platform class ifFalse: [self error: 'Login is incompatible with the database specified in the descriptor system'].
	newSession := GlorpSession new.
	newSession accessor: (DatabaseAccessor forLogin: aGlorpLogin).
	newSession system: self.
	^newSession.!

setUpDefaults
	"For systems that are configurable, set them up for testing configuration"!

shouldForeignKeyValueBeTreatedAsNull: aValue
	"Return true if this foreign key value should be treated as null, meaning there's no relationship. Normally this means just nil, but descriptor systems can override if they have special values which they use (e.g. 0) that indicate a missing relationship. This can improve performance by avoiding unnecessary queries to the database for things we know will never return a result"
	^aValue isNil.!

shouldResetExpiryTimeForObjectOfClass: aClass readFrom: aQuery
	"Return true if, when an object is read from a query, even if it's not refreshing. we 'touch' it, resetting its expiry time. By default, we do this if the cache policy for that type of object will remove from the cache. The rationale is that this means we're using the object. Removing an object in use is a very bad thing, and can cause loss of identity and other bad things. However, if expiry only indicates a need to refresh, we don't need to touch the object. Subclasses can override to provide different behaviour."

	aQuery shouldRefresh ifTrue: [^true].
	aQuery returnProxies ifTrue: [^false].
	^(self session cacheForClass: aClass)  willRemoveItems!

shouldUpdateOnlyChangedFieldsFor: aRow
	"Return true if we should write only changed fields out in an update statement, or if we should write all of them. The tradeoff is that writing only the changed ones will produce less SQL, but make more variations of the statement, and require us to keep more prepared statements around if we're using binding. By default, use this option only if we're not using binding by default"
	^aRow useBinding not.!

tableNamed: aString

	^self tableNamed: aString ifNone: [NoTableFound new tag: aString; signal].!

tableNamed: aString ifNone: aBlock

	^tables at: aString asString asUppercase
		ifAbsent: 
			[| newTable |
			newTable := self databaseTableClass new.
			newTable name: aString.
			tables at: aString asUppercase put: newTable.
			self initializeTable: newTable.
			newTable isEmpty
				ifTrue: 
					[tables removeKey: aString asUppercase.
					^aBlock value].
			newTable]!

tablesInCommitOrder

	^(TableSorter for: self allTables) sort.!

typeResolverFor: aClassOrObject 

	| theClass |
	aClassOrObject == Proxy ifTrue: [self error: 'Cannot find type resolver for the class Proxy. Pass in the instance'].
	theClass := aClassOrObject glorpCanHaveDescriptorDirectly 
				ifTrue: [aClassOrObject]
				ifFalse: [aClassOrObject isGlorpProxy ifTrue: [aClassOrObject getValue class] ifFalse: [aClassOrObject class]].
	^typeResolvers at: theClass ifAbsentPut: [self newTypeResolverFor: theClass]!

unmappedInstanceVariables
	"Return a list, for all of our classes, of instance variables with no mappings. This can be used by inspection to see if we're missing mappings we think we ought to have."
	"self unmappedInstanceVariables"

	| unmapped |
	unmapped := Dictionary new.
	self allClasses do: [:eachClass |
		eachClass allInstVarNames do: [:eachInstVarName |
			((self descriptorFor: eachClass) mappingForAttributeNamed: eachInstVarName asSymbol)
				isNil ifTrue: [
					(unmapped at: eachClass ifAbsentPut: [OrderedCollection new])
						add: eachInstVarName]]].
	^unmapped.!

useDirectAccessForMapping
	^useDirectAccessForMapping!

useDirectAccessForMapping: anObject
	useDirectAccessForMapping := anObject!

useJoinsForAnySatisfy
	useJoinsForAnySatisfy isNil ifTrue: [useJoinsForAnySatisfy := false].
	^useJoinsForAnySatisfy.!

useJoinsForAnySatisfy: aBoolean
	useJoinsForAnySatisfy := aBoolean.!

validate
	session isNil ifFalse: [session system == self ifFalse: [InvalidSessionSetup  signal:  'Invalid session setup']].
	self allTables do: [:each | each validateIn: self].
	self allClassModels do: [:each | each validateIn: self].
	self allDescriptors do: [:each | each validate].!

validateDescriptor: aDescriptor
	"Validate aDescriptor in the context of this system, doing any specific validations that need the full environment information or rely on subclass-specific information here."!

validateRows: aUnitOfWork
	"This is a hook to allow the system to do application-specific validation of the rows to be written. I suppose it's also possible to modify the rows here. Subclasses can override if they have useful work to do here. This will be called after the rows are generated, pre-write notification has been sent, rows have been sorted for writing, and and just before sequence numbers (if pre-write) are assigned, and writing starts"!

validateRowsPostWrite: aUnitOfWork
	"Validate after writing. This is a bit late, but some things may not be catchable ahead of time"!

withAllSuperclassesDo: aBlock
	| currentClass |
	currentClass := self class.
	[currentClass == Object]
		whileFalse:
			[aBlock value: currentClass.
			currentClass := currentClass superclass].! !
!DescriptorSystem categoriesFor: #addClassesFromDescriptorMethodsFor:!initialization!private! !
!DescriptorSystem categoriesFor: #addClassModelFor:!api!private! !
!DescriptorSystem categoriesFor: #additionalRelatedRowsFor:in:!callbacks!private! !
!DescriptorSystem categoriesFor: #addTablesFromInitializeMethodsFor:!initialization!private! !
!DescriptorSystem categoriesFor: #allClasses!accessing!private! !
!DescriptorSystem categoriesFor: #allClassModels!accessing!private! !
!DescriptorSystem categoriesFor: #allDescriptors!accessing!private! !
!DescriptorSystem categoriesFor: #allSequences!accessing!private! !
!DescriptorSystem categoriesFor: #allSequencesFor:!accessing!private! !
!DescriptorSystem categoriesFor: #allTableNames!accessing!private! !
!DescriptorSystem categoriesFor: #allTables!accessing!private! !
!DescriptorSystem categoriesFor: #cachePolicy!accessing!private! !
!DescriptorSystem categoriesFor: #cachePolicy:!accessing!private! !
!DescriptorSystem categoriesFor: #canUseGroupedWrites!private!testing! !
!DescriptorSystem categoriesFor: #canUseGroupedWritesFor:!private!testing! !
!DescriptorSystem categoriesFor: #classModelFor:!api!private! !
!DescriptorSystem categoriesFor: #constructAllClasses!initialization!private! !
!DescriptorSystem categoriesFor: #constructAllTables!initialization!private! !
!DescriptorSystem categoriesFor: #constructClassesFromDescriptorMethods!initialization!private! !
!DescriptorSystem categoriesFor: #constructTablesFromInitializeMethods!initialization!private! !
!DescriptorSystem categoriesFor: #converterBetweenStType:andField:!private! !
!DescriptorSystem categoriesFor: #createMappingIn:fromClassModelForAttributeNamed:!private! !
!DescriptorSystem categoriesFor: #createMappingsFromClassModelIn:!api!private! !
!DescriptorSystem categoriesFor: #databaseIdentitySequenceNamed:!api!public! !
!DescriptorSystem categoriesFor: #databaseSchema!accessing!private! !
!DescriptorSystem categoriesFor: #databaseSequenceNamed:!api!private! !
!DescriptorSystem categoriesFor: #databaseSequenceNamed:ifAbsentPut:!api!private! !
!DescriptorSystem categoriesFor: #databaseTableClass!api!public! !
!DescriptorSystem categoriesFor: #defaultTracing!accessing!private! !
!DescriptorSystem categoriesFor: #descriptorFor:!api!private! !
!DescriptorSystem categoriesFor: #descriptorForTable:!api!private! !
!DescriptorSystem categoriesFor: #descriptorsForTable:!api!private! !
!DescriptorSystem categoriesFor: #existingTableNamed:!api!private! !
!DescriptorSystem categoriesFor: #fieldNameForAttributeName:!callbacks!private! !
!DescriptorSystem categoriesFor: #flushAllClasses!api!private! !
!DescriptorSystem categoriesFor: #hasDescriptorFor:!api!private! !
!DescriptorSystem categoriesFor: #initialize!initialization!private! !
!DescriptorSystem categoriesFor: #initializeClassModel:!private! !
!DescriptorSystem categoriesFor: #initializeDescriptor:!private! !
!DescriptorSystem categoriesFor: #initializeTable:!private! !
!DescriptorSystem categoriesFor: #joinFor:toTables:fromConstraints:toConstraints:!callbacks!private! !
!DescriptorSystem categoriesFor: #maximumSizeToGroupWriteFor:!optimization parameters!private! !
!DescriptorSystem categoriesFor: #newClassModelFor:!private! !
!DescriptorSystem categoriesFor: #newDescriptorFor:!private! !
!DescriptorSystem categoriesFor: #newTypeResolverFor:!private! !
!DescriptorSystem categoriesFor: #noDescriptorInitializerFound:!private!validation! !
!DescriptorSystem categoriesFor: #noTableInitializerFound:!private!validation! !
!DescriptorSystem categoriesFor: #platform!accessing!private! !
!DescriptorSystem categoriesFor: #platform:!accessing!private! !
!DescriptorSystem categoriesFor: #privateAndBasicAddDescriptorFor:as:!private! !
!DescriptorSystem categoriesFor: #sequenceNameForField:!private!sequences! !
!DescriptorSystem categoriesFor: #sequenceTypeNamed:!api!private! !
!DescriptorSystem categoriesFor: #session!accessing!private! !
!DescriptorSystem categoriesFor: #session:!accessing!private! !
!DescriptorSystem categoriesFor: #sessionForLogin:!private!session creation! !
!DescriptorSystem categoriesFor: #setUpDefaults!initialization!private! !
!DescriptorSystem categoriesFor: #shouldForeignKeyValueBeTreatedAsNull:!private!settings! !
!DescriptorSystem categoriesFor: #shouldResetExpiryTimeForObjectOfClass:readFrom:!private!settings! !
!DescriptorSystem categoriesFor: #shouldUpdateOnlyChangedFieldsFor:!optimization parameters!public! !
!DescriptorSystem categoriesFor: #tableNamed:!api!private! !
!DescriptorSystem categoriesFor: #tableNamed:ifNone:!api!public! !
!DescriptorSystem categoriesFor: #tablesInCommitOrder!internal/writing!private! !
!DescriptorSystem categoriesFor: #typeResolverFor:!api!private! !
!DescriptorSystem categoriesFor: #unmappedInstanceVariables!private!validation! !
!DescriptorSystem categoriesFor: #useDirectAccessForMapping!accessing!private! !
!DescriptorSystem categoriesFor: #useDirectAccessForMapping:!accessing!private! !
!DescriptorSystem categoriesFor: #useJoinsForAnySatisfy!private!settings! !
!DescriptorSystem categoriesFor: #useJoinsForAnySatisfy:!private!settings! !
!DescriptorSystem categoriesFor: #validate!private!validation! !
!DescriptorSystem categoriesFor: #validateDescriptor:!private!validation! !
!DescriptorSystem categoriesFor: #validateRows:!private!validation! !
!DescriptorSystem categoriesFor: #validateRowsPostWrite:!private!validation! !
!DescriptorSystem categoriesFor: #withAllSuperclassesDo:!initialization!private! !

!DescriptorSystem class methodsFor!

createTablesFor: aLogin
	"Create the tables for this descriptor system in the given database. This will fail if the tables already exist. Use recreateTables for that."

	| session |
	session := self sessionForLogin: aLogin.
	session login.
	[session createTables] ensure: 
		[session logout].!

databaseTableClass

	^DatabaseTable!

dropTablesFor: aLogin
	"Drop the tables for this descriptor system in the given database."

	| session |
	session := self sessionForLogin: aLogin.
	session login.
	[session dropTables] ensure: [session logout].!

forPlatform: dbPlatform

	^super new initialize;
		platform: dbPlatform!

includeAllClassesWithDescriptorMethods
	"If this is true, the descriptor system will automatically include all classes that define the #glorpSetupDescriptor:forSystem: method on the class side. If false, it will only include classes which have descriptor methods defined in this class, or those explicitly added in the #constructAllClasses method."
	^false.!

new

	^super new initialize.!

recreateTablesFor: aLogin
	"Drop and recreate the tables for this descriptor system in the given database."

	| session |
	session := self sessionForLogin: aLogin.
	session login.
	[session recreateTables] ensure: [session logout].!

sessionForLogin: aGlorpLogin
	"Create a session associated with the given login, and using us as a descriptor system. This is the easiest way to get a full blown session going."
	| system session |
	system := self forPlatform: aGlorpLogin database.
	session := GlorpSession new.
	session accessor: (DatabaseAccessor forLogin: aGlorpLogin).
	session system: system.
	^session.! !
!DescriptorSystem class categoriesFor: #createTablesFor:!private!utility! !
!DescriptorSystem class categoriesFor: #databaseTableClass!class defaults!public! !
!DescriptorSystem class categoriesFor: #dropTablesFor:!private!utility! !
!DescriptorSystem class categoriesFor: #forPlatform:!instance creation!private! !
!DescriptorSystem class categoriesFor: #includeAllClassesWithDescriptorMethods!configuration!private! !
!DescriptorSystem class categoriesFor: #new!instance creation!private! !
!DescriptorSystem class categoriesFor: #recreateTablesFor:!private!utility! !
!DescriptorSystem class categoriesFor: #sessionForLogin:!instance creation!private! !

GlorpAttributeModel guid: (GUID fromString: '{BD403FBA-05A8-4D90-92B3-E557FC278BB6}')!
GlorpAttributeModel comment: 'This is the metadata for an individual instance variable. It will be held in a GlorpClassModel, and used for accessing the variable when building or writing out objects.

Instance Variables:
	attributeIndex	<Integer>	What is the index in the class of the instance variable. Used if we are using instVarAt: to get values.
	classModel	<GlorpClassModel>	The class model that contains us.
	collectionType	<Class>	If we are an attribute that holds a collection, what is the class of the collection.
	keyType	<Class>	If we represent a dictionary attribute, what is the type of our key (type will be used for the type of the value).
	name	<Symbol>	The name of the attribute/instance variable.
	setSelector	<Symbol>	If we are using message access to get/set values, what is the selector to use for setting. Defaults to name, '':''.
	type	<Class>	The class containing this instance variable
	useDirectAccess	<Boolean>	If true (the default) we use instVarAt:/instVarAt:put: to access the attribute values.

'!
!GlorpAttributeModel categoriesForClass!Unclassified! !
!GlorpAttributeModel methodsFor!

add: anObject to: aCollection in: ourObject
	"If this is an array we can't just add to it, we must concatenate and re-set the value"
	| newCollection |
	aCollection class == Array 
		ifTrue: [	
			newCollection := aCollection, (Array with: anObject).
			self setValueIn: ourObject to: newCollection.
			^self].
	(anObject class == Association and: [anObject key == nil])
		ifFalse: [aCollection add: anObject].!

beForPseudoVariable
	"Make this be a pseudo-variable - one which is neither read nor written, and doesn't have a corresponding instance variable, but is only used for queries."
	canRead := false.
	canWrite := false.!

canRead
	^canRead!

canRead: aBoolean
	canRead := aBoolean!

canWrite
	^canWrite!

canWrite: aBoolean
	canWrite := aBoolean!

checkTargetType: anObject
	"Check that anObject's type corresponds to something we know how to handle for this attribute. We expect this to be a real object, not a collection."
	self checkTargetType: anObject against: self type.!

checkTargetType: anObject against: aClass
	"Check that anObject's type corresponds to aClass, which we expect as our values. We expect this to be a real object, not a collection."
	| targetModel |
	anObject yourSelf isNil ifTrue: [^self].
	aClass isNil ifTrue: [self error: 'No type to check against in class model'].
	anObject glorpIsCollection ifTrue: [(GlorpInvalidTypeError new object: anObject; expectedType: self collectionType) signal].
	targetModel := self system classModelFor: aClass.
	targetModel notNil ifTrue: [
		(targetModel matches: anObject)
			ifFalse: [(GlorpInvalidTypeError new object: anObject; expectedType: aClass) signal]].!

checkTargetTypeForKey: aKey value: anObject
	"Check that anObject's type corresponds to something we know how to handle for this attribute. We expect this to be a real object, not a collection. This is the version used for checking a dictionary, where we check both key and value, but only if they correspond to mapped classes."
	(self system classModelFor: self keyType) isNil 
		ifFalse: [self checkTargetType: aKey against: self keyType].
	(self system classModelFor: self type) isNil
		ifFalse: [self checkTargetType: anObject against: type].!

checkType: anObject
	(self classModel matches: anObject)
		ifFalse: [self error: 'Wrong type of object'].!

classModel
	^classModel!

classModel: aGlorpClassModel
	classModel := aGlorpClassModel!

collectionType
	^collectionType!

collectionType: aClass
	collectionType := aClass!

copyFrom: anAttributeModel
	"Copy over the attributes if it defines them and we don't"
	anAttributeModel isNil ifTrue: [^self].
	self type isNil ifTrue: [self type: anAttributeModel type].
	self collectionType isNil
		ifTrue: [self collectionType: anAttributeModel collectionType].
	self privateUseDirectAccess isNil
		ifTrue: [useDirectAccess := anAttributeModel privateUseDirectAccess].
	canRead := anAttributeModel canRead.
	canWrite := anAttributeModel canWrite.!

directGetValueFrom: anObject 
	| index |
	index := self instVarIndexIn: anObject.
	index = 0 ifTrue: [self raiseInvalidAttributeError].
	^anObject instVarAt: index!

directSetValueIn: anObject to: aValue 
	| index |
	index := self instVarIndexIn: anObject.
	index = 0 ifTrue: [self raiseInvalidAttributeError].
	^anObject instVarAt: index put: aValue!

getValueFrom: anObject
	| real |
	real := anObject yourSelf.
	self checkType: real.
	self useDirectAccess ifTrue: [^self directGetValueFrom: real].
	^real perform: self name.!

initialize
	canRead := true.
	canWrite := true.!

instVarIndex
	| soughtName |
	attributeIndex == nil ifFalse: [^attributeIndex].
	soughtName := Dialect instVarNameFor: name.
	^attributeIndex := self classModel describedClass allInstVarNames 
				indexOf: soughtName!

instVarIndexIn: anObject 
	^self instVarIndex!

isCollectionAttribute
	^collectionType notNil.!

isDictionaryAttribute
	^collectionType isBehavior and: [collectionType includesBehavior: Dictionary].!

isForPseudoVariable
	"Return true if this is a pseudo-variable - one which is neither read nor written, and doesn't have a corresponding instance variable, but is only used for queries."
	^(canRead not & canWrite not) or: [self isValid not].!

isReadable
	self canRead ifFalse: [^false].
	^self useDirectAccess 
		ifTrue: [self instVarIndex > 0]
		ifFalse: [self classModel describedClass canUnderstand: self name]!

isValid
	^name notNil and: [classModel notNil].!

isWritable
	self canWrite ifFalse: [^false].
	^self useDirectAccess 
		ifTrue: [self instVarIndex > 0]
		ifFalse: [self classModel describedClass canUnderstand: self setSelector]!

keyType
	^keyType!

keyType: aClass
	keyType := aClass!

knit: ourObject to: anotherObject 
	"Set up the relationship from our object to another one, indicated by our mapping."

	self isCollectionAttribute 
		ifTrue: 
			[| collection |
			collection := self getValueFrom: ourObject.
			(collection isGlorpProxy and: [collection isInstantiated not]) 
				ifTrue: 
					[collection := self newCollection.
					self setValueIn: ourObject to: collection.
					self 
						add: anotherObject
						to: collection
						in: ourObject.
					^self].
			(collection includes: anotherObject) 
				ifFalse: 
					[self 
						add: anotherObject
						to: collection
						in: ourObject]]
		ifFalse: [self setValueIn: ourObject to: anotherObject]!

loopOver: mightBeACollection doing: aBlock
	"It is possible that the value for a dictionary attribute is nil. If so, do not map it back"
	| index |
	self isCollectionAttribute ifFalse: [self checkTargetType: mightBeACollection. ^aBlock value: 1 value: mightBeACollection].
	mightBeACollection isNil ifTrue: [^nil].
	self isDictionaryAttribute ifTrue: [^mightBeACollection keysAndValuesDo: [:key :value |
		self checkTargetTypeForKey: key value: value. 
		value ifNotNil: [aBlock value: key value: value]]].
	index := 1.
	mightBeACollection glorpIsCollection ifFalse: [
		(GlorpInvalidTypeError new object: mightBeACollection; expectedType: self collectionType) signal].
	mightBeACollection
		do:
			[:each |
			| object | 
			object := each glorpRealObject. 
			self checkTargetType: object.
			aBlock value: index value: object.
			index := index + 1].!

name
	^name isNil ifTrue: [''] ifFalse: [name]!

name: aSymbol
	| symbol |
	symbol := aSymbol isNil ifTrue: [aSymbol] ifFalse: [aSymbol asSymbol].
	name := symbol.!

newCollection
	^collectionType isBehavior ifTrue: [collectionType new] ifFalse: [collectionType copyEmpty].!

printOn: aStream
	aStream nextPutAll: self classModel printString, '>>', self name.!

privateUseDirectAccess
	^useDirectAccess.!

raiseInvalidAttributeError
	self error: 'Invalid attribute: ', classModel printString, '>>', name printString.!

setSelector
	setSelector  == nil ifTrue: [setSelector := (name, ':') asSymbol].
	^setSelector.!

setValueIn: anObject to: aValue 
	| real |
	self isValid ifFalse: [^self].
	real := anObject yourSelf.
	self checkType: real.
	self useDirectAccess ifTrue: [^self directSetValueIn: real to: aValue].
	^real perform: self setSelector with: aValue!

system
	classModel isNil ifTrue: [^nil].
	^classModel system.!

type
	^type!

type: aClass
	type := aClass!

useDirectAccess
	useDirectAccess isNil ifFalse: [^useDirectAccess].
	self system isNil ifTrue: [^true].  "A global default, although only applicable if we're not in a valid setup"
	^self system useDirectAccessForMapping.!

useDirectAccess: aBoolean
	useDirectAccess := aBoolean.!

validate
	name isEmpty ifTrue: [self error: 'Invalid attribute name']!

validateCanRead
	self isReadable ifFalse: [self raiseInvalidAttributeError]!

validateCanWrite
	self isWritable ifFalse: [self raiseInvalidAttributeError]! !
!GlorpAttributeModel categoriesFor: #add:to:in:!knitting!private! !
!GlorpAttributeModel categoriesFor: #beForPseudoVariable!accessing!private! !
!GlorpAttributeModel categoriesFor: #canRead!accessing!private! !
!GlorpAttributeModel categoriesFor: #canRead:!accessing!private! !
!GlorpAttributeModel categoriesFor: #canWrite!accessing!private! !
!GlorpAttributeModel categoriesFor: #canWrite:!accessing!private! !
!GlorpAttributeModel categoriesFor: #checkTargetType:!get/set!private! !
!GlorpAttributeModel categoriesFor: #checkTargetType:against:!get/set!private! !
!GlorpAttributeModel categoriesFor: #checkTargetTypeForKey:value:!get/set!private! !
!GlorpAttributeModel categoriesFor: #checkType:!get/set!private! !
!GlorpAttributeModel categoriesFor: #classModel!accessing!private! !
!GlorpAttributeModel categoriesFor: #classModel:!accessing!private! !
!GlorpAttributeModel categoriesFor: #collectionType!accessing!private! !
!GlorpAttributeModel categoriesFor: #collectionType:!accessing!private! !
!GlorpAttributeModel categoriesFor: #copyFrom:!copying!private! !
!GlorpAttributeModel categoriesFor: #directGetValueFrom:!private! !
!GlorpAttributeModel categoriesFor: #directSetValueIn:to:!private! !
!GlorpAttributeModel categoriesFor: #getValueFrom:!get/set!private! !
!GlorpAttributeModel categoriesFor: #initialize!initialization!private! !
!GlorpAttributeModel categoriesFor: #instVarIndex!private! !
!GlorpAttributeModel categoriesFor: #instVarIndexIn:!private! !
!GlorpAttributeModel categoriesFor: #isCollectionAttribute!private!validation! !
!GlorpAttributeModel categoriesFor: #isDictionaryAttribute!mapping!private! !
!GlorpAttributeModel categoriesFor: #isForPseudoVariable!private!testing! !
!GlorpAttributeModel categoriesFor: #isReadable!private!validation! !
!GlorpAttributeModel categoriesFor: #isValid!private! !
!GlorpAttributeModel categoriesFor: #isWritable!private!validation! !
!GlorpAttributeModel categoriesFor: #keyType!accessing!private! !
!GlorpAttributeModel categoriesFor: #keyType:!accessing!private! !
!GlorpAttributeModel categoriesFor: #knit:to:!knitting!private! !
!GlorpAttributeModel categoriesFor: #loopOver:doing:!mapping!public! !
!GlorpAttributeModel categoriesFor: #name!accessing!private! !
!GlorpAttributeModel categoriesFor: #name:!accessing!private! !
!GlorpAttributeModel categoriesFor: #newCollection!private! !
!GlorpAttributeModel categoriesFor: #printOn:!printing!private! !
!GlorpAttributeModel categoriesFor: #privateUseDirectAccess!private! !
!GlorpAttributeModel categoriesFor: #raiseInvalidAttributeError!private! !
!GlorpAttributeModel categoriesFor: #setSelector!private! !
!GlorpAttributeModel categoriesFor: #setValueIn:to:!get/set!private! !
!GlorpAttributeModel categoriesFor: #system!accessing!private! !
!GlorpAttributeModel categoriesFor: #type!accessing!private! !
!GlorpAttributeModel categoriesFor: #type:!accessing!private! !
!GlorpAttributeModel categoriesFor: #useDirectAccess!private!testing! !
!GlorpAttributeModel categoriesFor: #useDirectAccess:!accessing!private! !
!GlorpAttributeModel categoriesFor: #validate!private!validation! !
!GlorpAttributeModel categoriesFor: #validateCanRead!private!validation! !
!GlorpAttributeModel categoriesFor: #validateCanWrite!private!validation! !

!GlorpAttributeModel class methodsFor!

named: aSymbol
	^self new name: aSymbol.!

named: aSymbol collection: aCollectionClass of: aClass
	^(self new)
		name: aSymbol;
		type: aClass;
		collectionType: aCollectionClass.!

named: aSymbol collectionOf: aClass
	^self named: aSymbol collection: OrderedCollection of: aClass.!

named: aSymbol dictionary: dictionaryClass from: keyClass to: valueClass
	^(self new)
		name: aSymbol;
		type: valueClass;
		keyType: keyClass;
		collectionType: dictionaryClass.!

named: aSymbol dictionaryFrom: keyClass to: valueClass
	^self named: aSymbol dictionary: Dictionary from: keyClass to: valueClass.!

named: aSymbol type: aClass
	^self new name: aSymbol; type: aClass.!

new
	^super new initialize.! !
!GlorpAttributeModel class categoriesFor: #named:!instance creation!private! !
!GlorpAttributeModel class categoriesFor: #named:collection:of:!instance creation!private! !
!GlorpAttributeModel class categoriesFor: #named:collectionOf:!instance creation!private! !
!GlorpAttributeModel class categoriesFor: #named:dictionary:from:to:!instance creation!private! !
!GlorpAttributeModel class categoriesFor: #named:dictionaryFrom:to:!instance creation!private! !
!GlorpAttributeModel class categoriesFor: #named:type:!instance creation!private! !
!GlorpAttributeModel class categoriesFor: #new!instance creation!private! !

GlorpClassModel guid: (GUID fromString: '{D9A3AF8F-E88C-4037-B903-75186BF0D7B2}')!
GlorpClassModel comment: 'This holds the metadata describing a class. Since Smalltalk classes are fairly self-describing, this mostly consists of the names and types of the instance variables that we want to be persistent. If those variables are collections, we also want to know the types of the items in the collections.

Instance Variables:
	attributes	<(Collection of: GlorpAttributeModel)>	Our attributes, one for each instance variable.
	describedClass	<Class>	The class we hold metadata for.
	subclasses	<Array of: Class>	The collection of subclasses of our describedClass. Computed automatically, and used as a performance optimization for inheritance testing.
	system	<DescriptorSystem>	The system that contains us.

'!
!GlorpClassModel categoriesForClass!Unclassified! !
!GlorpClassModel methodsFor!

addAttribute: anAttributeDescription
	attributes add: anAttributeDescription.
	anAttributeDescription classModel: self.
	^anAttributeDescription.!

attributeNamed: aSymbol
	^attributes detect: [:each | each name = aSymbol] ifNone: [self newAttributeNamed: aSymbol].!

attributeNamed: aSymbol type: aClass
	^attributes
		detect:
			[:each | each name = aSymbol and: [each type isNil or: [each type = aClass]]]
		ifNone: [self newAttributeNamed: aSymbol type: aClass].!

attributeNamed: aSymbol type: aClass collectionType: collectionClass
	^attributes
		detect:
			[:each | 
			| collectionTypeMatches typeMatches nameMatches |
			nameMatches := each name = aSymbol.
			typeMatches := each type isNil or: [each type = aClass].
			collectionTypeMatches := each collectionType isNil
				or: [each collectionType = collectionClass].
			nameMatches & typeMatches & collectionTypeMatches]
		ifNone: [self newAttributeNamed: aSymbol type: aClass].!

attributes
	^attributes.!

attributesDo: aBlock
	attributes do: aBlock!

describedClass
	^describedClass!

describedClass: aClass
	describedClass := aClass.
	subclasses := aClass allSubclasses isEmpty ifTrue: [nil] ifFalse: [aClass allSubclasses asArray].!

hasAttributeNamed: aSymbol

	attributes do: [:attribute | attribute name = aSymbol ifTrue: [^true]].
	^false!

initialize
	attributes := OrderedCollection new.!

matches: anObject
	"Try to optimize this check as much as possible. We cache the list of subclasses, and inline the check for isKindOf:"
	| theClass |
	theClass := anObject glorpRealObject class.
	describedClass == theClass ifTrue: [^true].
	nil == subclasses ifTrue: [^false].
	1 to: subclasses size do: [:i | theClass == (subclasses at: i)
		ifTrue: [^true]].
	^false.!

newAttributeNamed: aSymbol
	^self addAttribute: (GlorpAttributeModel named: aSymbol).!

newAttributeNamed: aSymbol collection: collectionClass of: aClass
	^self addAttribute: (GlorpAttributeModel named: aSymbol collection: collectionClass of: aClass).!

newAttributeNamed: aSymbol collectionOf: aClass
	^self addAttribute: (GlorpAttributeModel named: aSymbol collectionOf: aClass).!

newAttributeNamed: aSymbol dictionaryFrom: keyClass to: valueClass
	^self addAttribute: (GlorpAttributeModel named: aSymbol dictionaryFrom: keyClass to: valueClass).!

newAttributeNamed: aSymbol type: aClass
	^self addAttribute: (GlorpAttributeModel named: aSymbol type: aClass).!

printOn: aStream
	aStream nextPutAll: '~~'.
	self describedClass isNil ifFalse: [aStream nextPutAll: self describedClass name].
	aStream nextPutAll: '~~'.!

system
	^system!

system: aDescriptorSystem
	system := aDescriptorSystem!

validateIn: aDescriptorSystem 
	attributes do: [:each | each validate]! !
!GlorpClassModel categoriesFor: #addAttribute:!accessing!private! !
!GlorpClassModel categoriesFor: #attributeNamed:!accessing!private! !
!GlorpClassModel categoriesFor: #attributeNamed:type:!accessing!private! !
!GlorpClassModel categoriesFor: #attributeNamed:type:collectionType:!accessing!private! !
!GlorpClassModel categoriesFor: #attributes!accessing!private! !
!GlorpClassModel categoriesFor: #attributesDo:!attributes!private! !
!GlorpClassModel categoriesFor: #describedClass!accessing!private! !
!GlorpClassModel categoriesFor: #describedClass:!accessing!private! !
!GlorpClassModel categoriesFor: #hasAttributeNamed:!accessing!private! !
!GlorpClassModel categoriesFor: #initialize!initializing!private! !
!GlorpClassModel categoriesFor: #matches:!private!validation! !
!GlorpClassModel categoriesFor: #newAttributeNamed:!attributes!private! !
!GlorpClassModel categoriesFor: #newAttributeNamed:collection:of:!attributes!private! !
!GlorpClassModel categoriesFor: #newAttributeNamed:collectionOf:!attributes!private! !
!GlorpClassModel categoriesFor: #newAttributeNamed:dictionaryFrom:to:!attributes!private! !
!GlorpClassModel categoriesFor: #newAttributeNamed:type:!attributes!private! !
!GlorpClassModel categoriesFor: #printOn:!printing!private! !
!GlorpClassModel categoriesFor: #system!accessing!private! !
!GlorpClassModel categoriesFor: #system:!accessing!private! !
!GlorpClassModel categoriesFor: #validateIn:!private!validation! !

!GlorpClassModel class methodsFor!

for: aClass
	^self new describedClass: aClass.!

new
	^super new initialize.! !
!GlorpClassModel class categoriesFor: #for:!instance creation!private! !
!GlorpClassModel class categoriesFor: #new!instance creation!private! !

GlorpMappingsPackage guid: (GUID fromString: '{9323DFCB-9542-4A4C-B1BA-9DE0B770DD83}')!
GlorpMappingsPackage comment: 'This class represents package information for the GlorpMappings package, for systems that don''t have things like package properties to store things like copyright and license information.

LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you''ll need to release the
source of any modifications.

If you require any further clarification, we''d be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'!
!GlorpMappingsPackage categoriesForClass!Unclassified! !
!GlorpMappingsPackage class methodsFor!

LICENSE
"LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you'll need to release the
source of any modifications.

If you require any further clarification, we'd be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"! !
!GlorpMappingsPackage class categoriesFor: #LICENSE!instance creation!public! !

Mapping guid: (GUID fromString: '{9FB71C40-9A69-4482-9CBC-AB20F1056107}')!
Mapping comment: 'This is the abstract superclass for all Glorp mappings. A mapping defines how one particular instance variable will be mapped into and out of the database. There is considerable variation among the subclasses.

Subclasses must implement the following messages:
	accessing
		allTables
	mapping
		mapFromObject:intoRowsIn:
		mapObject:inElementBuilder:
		referencedIndependentObjectsFrom:
		trace:context:
	testing
		controlsTables
		isRelationship
		isStoredInSameTable
		mappedFields

Instance Variables:
	attribute	<GlorpAttributeModel>	The description of the instance variable we map.
	canRead	<Boolean>	Is this mapping allowed to read.
	canWrite	<Boolean>	Is this mapping allowed to write.
	debugRead	<Boolean>	For debugging purposes. If true, we will halt when this mapping is about to read..
	debugWrite	<Boolean>	For debugging purposes. If true, we will halt when this mapping is about to write.
	descriptor	<Descriptor>	The descriptor that contains this mapping.

'!
!Mapping categoriesForClass!Unclassified! !
!Mapping methodsFor!

allRelationsFor: rootExpression do: aBlock andBetweenDo: anotherBlock
	"Normal mappings just operate on a single expression"
	aBlock value: rootExpression leftChild value: rootExpression rightChild.!

allTables

	self subclassResponsibility.!

applicableMappingForObject: anObject 
	"For polymorphism with conditional mappings"
	^self!

attribute
	attribute isNil ifTrue: [attribute := GlorpAttributeModel new].
	^attribute.!

attributeAccessor
	^self attribute.!

attributeName
	^self attribute name.!

attributeName: aSymbol
	| oldAttribute symbolName |
	symbolName := aSymbol asSymbol.
	oldAttribute := attribute.
	attribute := self classModel attributeNamed: symbolName.
	oldAttribute isNil
		ifTrue: [self initializeAttribute]
		ifFalse: [attribute copyFrom: oldAttribute].!

beExclusive
	self error: 'Only relationship mappings can be exclusive'.!

beForPseudoVariable
	self attribute beForPseudoVariable.!

beReadOnly
	self canWrite: false.!

canBeUsedForRetrieve
	"Return true if this is a valid argument for a retrieve: clause"
	self isRelationship ifFalse: [^true].
	^self isToManyRelationship not.!

canRead
	^self attribute canRead.!

canWrite
	^self attribute canWrite.!

canWrite: aBoolean
	self attribute canWrite: aBoolean.!

classModel
	"If we don't have one, just make a temporary one. We'll fix it later"
	descriptor isNil ifTrue: [^GlorpClassModel new].
	^self descriptor classModel.!

controlsTables
	"Return true if this type of mapping 'owns' the tables it's associated with, and expression nodes using this mapping should alias those tables where necessary"

	self subclassResponsibility!

convertedDbValueOf: stValue 
	"Mappings in general don't know how to do type conversion. Subclasses that deal with database primitive types should override."
	^stValue!

copyAttributeAttributes
	"We have an attributemodel that was created before we actually had a descriptor. It's possible that it had properties set. Make sure we copy them. If necessary, initialize things that require it."
	| oldAttribute |
	oldAttribute := attribute.
	attribute := self classModel attributeNamed: oldAttribute name type: oldAttribute type collectionType: oldAttribute collectionType.
	attribute copyFrom: oldAttribute.
	self initializeAttribute.!

createDeleteRowFor: anObject in: aRowMap
	"Only applies to mappings that write into link tables"!

debugRead
	^debugRead!

debugRead: aBoolean
	debugRead := aBoolean!

debugWrite
	^debugWrite!

debugWrite: aBoolean
	debugWrite := aBoolean!

descriptor
	^descriptor!

descriptor: aDescriptor
	descriptor := aDescriptor.
	attribute isNil ifFalse: [self copyAttributeAttributes].!

expressionFor: anObject
	"Return an expression representing the value of the object. This can be nil, an object value or values, an expression, or a collection of expressions (for a composite key, if we're passed an expression)"

	anObject isNil ifTrue: [^#(nil)].
	anObject isGlorpExpression ifFalse: [
		^self mappedFields collect: [:each | 	
			self valueOfField: each fromObject: anObject]].
	^self mappedFields
		collect: [:each | (anObject getField: each)].!

expressionFor: anObject basedOn: anExpression relation: aSymbol
	"Return our expression using the object's values. e.g. if this was a direct mapping from id->ID and the object had id: 3, then return TABLE.ID=3. Used when rewriting object=object into field=field"

	| myValue result |
	myValue := self expressionFor: anObject.
	result := nil.
	myValue with: self join allTargetFields do: [:eachValue :eachField |
		| source |
		source := anExpression get: self attribute name.
		source hasDescriptor ifTrue: [source := source getField: eachField].
		result := (source get: aSymbol withArguments: (Array with: eachValue)) AND: result].
	^result.!

fieldsForDescriptorSelectStatement
	"Return a collection of fields that this mapping will read from a row when the containing object is read, as opposed to when we're specifically told to use this field. Normally it amounts to the same thing."
	^self mappedFields.!

fieldsForSelectStatement
	"Return a collection of fields that this mapping will read from a row"
	self canRead ifFalse: [^#()].
	^self mappedFields!

getValueFrom: anObject

	^self attribute getValueFrom: anObject.!

hasAttribute
	"Return true if this maps an attribute, which is normally true"
	^true!

hasImpliedClauses
	"Return true if this implies multiple sql clauses"
	^false.!

hasValueThatCanProvideField: aField from: anObject
	"Return true if we have enough information in ourselves to populate our fields. The only time this won't be true is if we're either holding an uninstantiated proxy or have a to-many relationship mapping with an empty collection."
	^true.!

includesSubFieldsInSelectStatement
	^false!

initialize
	debugRead := false.
	debugWrite := false.!

initializeAttribute
	"This is where we can initialize aspects of the attribute, if required. The motivating example is initializing the collection type of the attribute if it wasn't set. We need it to have some type, but we don't want to initialize it while we're doing basic initialization and aren't yet properly attached to a descriptor or class model"!

isExclusive
	^false.!

isExclusive: aBoolean
	aBoolean ifTrue: [self error: 'Only relationship mappings can be exclusive'].!

isForPseudoVariable
	^self attribute isForPseudoVariable or: [self attributeAccessor isValid not].!

isRelationship
	"True when the mapping associates different persistent classes."

	^self subclassResponsibility!

isStoredInSameTable
	"True when the mapping is between two objects that occupy the same table, e.g. an embedded mapping."

	^self subclassResponsibility!

isToManyRelationship
	^false.!

isTypeMapping
	^false!

joinExpressionFor: anExpression

	^nil.!

loopOverAttributeOf: anObject doing: aTwoArgumentBlock
	"Loop over the target, which we extract from anObject, and iterate over it evaluating the block. For things that aren't collections, iterating means just run once. Note that the argument is expected to be an instance of our descriptor, as opposed to the attribute method, which expects the target object"
	| target |
	target := self getValueFrom: anObject.
	self attribute loopOver: target doing: aTwoArgumentBlock.!

mapFromObject: anObject intoRowsIn: aRowMap

	self subclassResponsibility.!

mapObject: anObject inElementBuilder: anObject1
	self subclassResponsibility.!

mappedFields
	self subclassResponsibility!

mappedTables
	^(self mappedFields collect: [:each | each table]) asSet.!

mapsPrimaryKeys
	^(self mappedFields detect: [:each | each isPrimaryKey] ifNone: [nil]) notNil.!

mapsSimpleObject
	"Return true if we know that we map something without a descriptor. For some mappings, like ad hoc or conditional, we don't really know, so we default to false"
	^false.!

multipleTableExpressionsFor: anExpression

	^#().!

newProxyIn: anObjectBuilder 
	| proxy |
	proxy := Proxy new.
	proxy query: (self queryFor: anObjectBuilder).
	proxy glorpProxyAction: self proxyAction.
	^proxy!

printOn: aStream

	super printOn: aStream.
	aStream 
		nextPutAll: '(';
		nextPutAll: (attribute isNil ifTrue: [''] ifFalse: [self attributeName]) ;
		nextPutAll: ')'.!

proxyAction
	^nil.!

readBackNewRowInformationFor: anObject fromRowsIn: aRowMap 
	"
	self subclassResponsibility. ?"!

readOnly
	^self canWrite not.!

readOnly: aBoolean
	self canWrite: aBoolean not.!

referenceClass
	^self attribute type.!

referencedIndependentObjectsFrom: anObject

	self subclassResponsibility.!

session
	
	^self descriptor session.!

setValueIn: anObject to: aValue
	self canRead ifFalse: [^self].
	self attribute setValueIn: anObject to: aValue.!

stopIfDebuggingRead
	debugRead ifTrue: [self halt].!

stopIfDebuggingWrite
	debugWrite ifTrue: [self halt].!

system
	
	^self descriptor system.!

trace: aTracing context: anExpression

	self subclassResponsibility.!

translateFields: anOrderedCollection 
	"Normal mappings don't translate"
	^anOrderedCollection.!

translateTable: aTable
	"If we have a field translation, return the translated (i.e. real) table to use. This ideally allows us to use either the real table in which something is embedded, or the imaginary table"
	^aTable.!

validate
	self validateAttribute!

validateAttribute
	self canWrite ifTrue: [self attribute validateCanWrite].
	self canRead ifTrue: [self attribute validateCanRead]!

valueOfField: aField fromObject: anObject
	self error: 'Can''t get value of field'.! !
!Mapping categoriesFor: #allRelationsFor:do:andBetweenDo:!printing SQL!private! !
!Mapping categoriesFor: #allTables!accessing!private! !
!Mapping categoriesFor: #applicableMappingForObject:!mapping!private! !
!Mapping categoriesFor: #attribute!accessing!private! !
!Mapping categoriesFor: #attributeAccessor!accessing!private! !
!Mapping categoriesFor: #attributeName!accessing!private! !
!Mapping categoriesFor: #attributeName:!accessing!private! !
!Mapping categoriesFor: #beExclusive!api!private! !
!Mapping categoriesFor: #beForPseudoVariable!accessing!private! !
!Mapping categoriesFor: #beReadOnly!accessing!private! !
!Mapping categoriesFor: #canBeUsedForRetrieve!private!testing! !
!Mapping categoriesFor: #canRead!accessing!private! !
!Mapping categoriesFor: #canWrite!accessing!private! !
!Mapping categoriesFor: #canWrite:!accessing!private! !
!Mapping categoriesFor: #classModel!accessing!private! !
!Mapping categoriesFor: #controlsTables!private!testing! !
!Mapping categoriesFor: #convertedDbValueOf:!private! !
!Mapping categoriesFor: #copyAttributeAttributes!initialize/release!private! !
!Mapping categoriesFor: #createDeleteRowFor:in:!mapping!private! !
!Mapping categoriesFor: #debugRead!debugging!private! !
!Mapping categoriesFor: #debugRead:!debugging!private! !
!Mapping categoriesFor: #debugWrite!debugging!private! !
!Mapping categoriesFor: #debugWrite:!debugging!private! !
!Mapping categoriesFor: #descriptor!accessing!private! !
!Mapping categoriesFor: #descriptor:!accessing!private! !
!Mapping categoriesFor: #expressionFor:!mapping!private! !
!Mapping categoriesFor: #expressionFor:basedOn:relation:!mapping!private! !
!Mapping categoriesFor: #fieldsForDescriptorSelectStatement!accessing!private! !
!Mapping categoriesFor: #fieldsForSelectStatement!accessing!private! !
!Mapping categoriesFor: #getValueFrom:!public! !
!Mapping categoriesFor: #hasAttribute!private!testing! !
!Mapping categoriesFor: #hasImpliedClauses!private!testing! !
!Mapping categoriesFor: #hasValueThatCanProvideField:from:!private!testing! !
!Mapping categoriesFor: #includesSubFieldsInSelectStatement!private!testing! !
!Mapping categoriesFor: #initialize!initialize/release!private! !
!Mapping categoriesFor: #initializeAttribute!initialize/release!private! !
!Mapping categoriesFor: #isExclusive!api!private! !
!Mapping categoriesFor: #isExclusive:!api!private! !
!Mapping categoriesFor: #isForPseudoVariable!accessing!private! !
!Mapping categoriesFor: #isRelationship!private!testing! !
!Mapping categoriesFor: #isStoredInSameTable!private!testing! !
!Mapping categoriesFor: #isToManyRelationship!private!testing! !
!Mapping categoriesFor: #isTypeMapping!private!testing! !
!Mapping categoriesFor: #joinExpressionFor:!preparing!private! !
!Mapping categoriesFor: #loopOverAttributeOf:doing:!iterating!private! !
!Mapping categoriesFor: #mapFromObject:intoRowsIn:!mapping!private! !
!Mapping categoriesFor: #mapObject:inElementBuilder:!mapping!private! !
!Mapping categoriesFor: #mappedFields!private!testing! !
!Mapping categoriesFor: #mappedTables!mapping!private! !
!Mapping categoriesFor: #mapsPrimaryKeys!private!testing! !
!Mapping categoriesFor: #mapsSimpleObject!private!testing! !
!Mapping categoriesFor: #multipleTableExpressionsFor:!preparing!private! !
!Mapping categoriesFor: #newProxyIn:!private!proxies! !
!Mapping categoriesFor: #printOn:!public! !
!Mapping categoriesFor: #proxyAction!private!proxies! !
!Mapping categoriesFor: #readBackNewRowInformationFor:fromRowsIn:!mapping!private! !
!Mapping categoriesFor: #readOnly!accessing!private! !
!Mapping categoriesFor: #readOnly:!accessing!private! !
!Mapping categoriesFor: #referenceClass!accessing!private! !
!Mapping categoriesFor: #referencedIndependentObjectsFrom:!mapping!private! !
!Mapping categoriesFor: #session!accessing!private! !
!Mapping categoriesFor: #setValueIn:to:!public! !
!Mapping categoriesFor: #stopIfDebuggingRead!debugging!private! !
!Mapping categoriesFor: #stopIfDebuggingWrite!debugging!private! !
!Mapping categoriesFor: #system!accessing!private! !
!Mapping categoriesFor: #trace:context:!mapping!private! !
!Mapping categoriesFor: #translateFields:!mapping!private! !
!Mapping categoriesFor: #translateTable:!mapping!private! !
!Mapping categoriesFor: #validate!private!validation! !
!Mapping categoriesFor: #validateAttribute!private!validation! !
!Mapping categoriesFor: #valueOfField:fromObject:!mapping!private! !

!Mapping class methodsFor!

new

	^super new initialize.! !
!Mapping class categoriesFor: #new!instance creation!private! !

TypeResolver guid: (GUID fromString: '{0A86B384-4002-41C0-B5CB-3B9B23FC0E50}')!
TypeResolver comment: 'TypeResolvers are used to implement inheritance. Each descriptor has a type resolver, which can be asked for the types associated with that descriptor. By default we get an IdentityTypeResolver, which just gives us a single class. With inheritance we can get different types.

Subclasses must implement the following messages:
	accessing
		classesRequiringIndependentQueriesFor:
		describedConcreteClassFor:withBuilder:descriptor:
	type resolution
		typeMappingRootDescriptor

Instance Variables:
	members	<(SequenceableCollection of: (Descriptor))>	The descriptors of the classes that this resolver applies to.
	system	<DescriptorSystem>	The system in which the members live.

'!
!TypeResolver categoriesForClass!Unclassified! !
!TypeResolver methodsFor!

addMember: aDescriptor
	members isNil ifTrue: [ members := OrderedCollection new].
	members add: aDescriptor!

allTablesFor: aDescriptor
	"Return all tables used by this descriptor or its subclasses. For a non-inheritance descriptor, this is the same as tables."
	self subclassResponsibility.!

classesRequiringIndependentQueriesFor: aClass
	self subclassResponsibility!

describedClasses
	^members collect: [:each | each describedClass ].!

describedConcreteClassFor: row withBuilder: builder descriptor: aDescriptor
	self subclassResponsibility!

fieldsForSelectStatement
	"Return fields that are needed in a select statement - i.e. return all inherited fields that are part of the tables we are already selecting for this object"
	^#().!

register: aDescriptor
	^self register: aDescriptor abstract: false!

register: aDescriptor abstract: abstract

	self system: aDescriptor system.
	self addMember: aDescriptor.
	aDescriptor typeResolver: self!

setUpExpressionsForQuery: aQuery
	"A hook to let us do any modifications to the query that we need."
	^self.!

shouldUseSingleQuery
	"Return true if we should attempt to combine queries for multiple subclasses in disjoint tables into one."
	^false.!

supportsOrderingFor: aDescriptor
	^true.!

system
	^system!

system: anObject
	system := anObject!

typeMappingRootDescriptor
	self subclassResponsibility!

usesInheritance
	^false.!

validate
	^self.! !
!TypeResolver categoriesFor: #addMember:!accessing!private! !
!TypeResolver categoriesFor: #allTablesFor:!accessing!private! !
!TypeResolver categoriesFor: #classesRequiringIndependentQueriesFor:!accessing!private! !
!TypeResolver categoriesFor: #describedClasses!other!private! !
!TypeResolver categoriesFor: #describedConcreteClassFor:withBuilder:descriptor:!accessing!private! !
!TypeResolver categoriesFor: #fieldsForSelectStatement!private!type resolution! !
!TypeResolver categoriesFor: #register:!private!registering! !
!TypeResolver categoriesFor: #register:abstract:!private!registering! !
!TypeResolver categoriesFor: #setUpExpressionsForQuery:!private!query setup! !
!TypeResolver categoriesFor: #shouldUseSingleQuery!accessing!private! !
!TypeResolver categoriesFor: #supportsOrderingFor:!private!testing! !
!TypeResolver categoriesFor: #system!accessing!private! !
!TypeResolver categoriesFor: #system:!accessing!private! !
!TypeResolver categoriesFor: #typeMappingRootDescriptor!private!type resolution! !
!TypeResolver categoriesFor: #usesInheritance!private!testing! !
!TypeResolver categoriesFor: #validate!private!validation! !

DynamicDescriptorSystem guid: (GUID fromString: '{DA66A314-D478-4B92-9262-352B8359B964}')!
DynamicDescriptorSystem comment: 'This is a descriptor system whose descriptors and tables are created dynamically rather than out of generated code. Note that identity is extremely important, so care is required to set these up properly.
'!
!DynamicDescriptorSystem categoriesForClass!Unclassified! !
!DynamicDescriptorSystem methodsFor!

addEmptyDescriptorFor: aClass 
	| descriptor |
	descriptor := Descriptor new.
	descriptor system: self.
	self privateDescriptorAt: aClass put: descriptor.
	descriptor classModel: (self classModelFor: aClass).
	^descriptor!

addEmptyTableNamed: aString 
	| table |
	table := self databaseTableClass named: aString.
	self privateTableAt: aString put: table.
	^table!

allClasses
	^descriptors keys!

allTableNames
	^tables keys!

attributeNameToFieldNameBlock: aBlock
	attributeNameToFieldNameBlock := aBlock!

fieldNameForAttributeName: aSymbol 
	attributeNameToFieldNameBlock isNil  
		ifTrue: [^super fieldNameForAttributeName: aSymbol].
	^attributeNameToFieldNameBlock value: aSymbol!

initializeClassModel: aClassModel
	"Assume we'll initialize it either before or after this"!

privateDescriptorAt: aClass put: aDescriptor
	"Normally you don't want to be setting tables explicitly, as it may defeat the identity management but it's here if needed"
	descriptors at: aClass put: aDescriptor.!

privateTableAt: aString put: aTable
	"Normally you don't want to be setting tables explicitly, as it may defeat the identity management but it's here if needed"
	tables at: aString asUppercase put: aTable.! !
!DynamicDescriptorSystem categoriesFor: #addEmptyDescriptorFor:!accessing!private! !
!DynamicDescriptorSystem categoriesFor: #addEmptyTableNamed:!accessing!public! !
!DynamicDescriptorSystem categoriesFor: #allClasses!accessing!private! !
!DynamicDescriptorSystem categoriesFor: #allTableNames!accessing!private! !
!DynamicDescriptorSystem categoriesFor: #attributeNameToFieldNameBlock:!accessing!private! !
!DynamicDescriptorSystem categoriesFor: #fieldNameForAttributeName:!hooks!private! !
!DynamicDescriptorSystem categoriesFor: #initializeClassModel:!accessing!private! !
!DynamicDescriptorSystem categoriesFor: #privateDescriptorAt:put:!private! !
!DynamicDescriptorSystem categoriesFor: #privateTableAt:put:!private! !

AdHocMapping guid: (GUID fromString: '{4E82390F-A0D8-44A8-ACD1-97E9160E92C0}')!
AdHocMapping comment: 'AdHocMapping is a configurable sort of mapping, done via two blocks. It''s not easy to use, but gives you very direct access to the underlying mechanisms, so it should be possible to write mappings that are otherwise very complicated or impossible.

Instance Variables:
	fromDbMappingBlock	<BlockClosure>	The block we invoke when reading from the database. It''s given arguments of the "row" we got from the database (note that this doesn''t necessarily correspond to a single row, but is all the fields we selected in the query) , the elementBuilder associated with our containing object, and the "context", the expression from the query for what we''re reading from the database. Typical usage is to use the context to translate a field into the appropriate aliased field from the query, and use the element builder to get the value of that field. e.g. elementBuilder valueOfField: (context translateField: someField) in: row
	mappedFields	<SequenceableCollection of: DatabaseField>	The list of fields we work with. We need to specify this so that Glorp knows that it needs to e.g. includes these fields in the list of those selected.
	shouldProxy	<Boolean>	Should we create a proxy for our target objects when we read an object containing this mapping.
	toDbMappingBlock	<BlockClosure>	The block we invoke when writing to the database. It takes as arguments the rowmap being created, the target object we are writing, and a dictionary of the rows we already have associated with this object. The block is expected to update the rowmap with the new information.

'!
!AdHocMapping categoriesForClass!Unclassified! !
!AdHocMapping methodsFor!

allRelationsFor: rootExpression
	"We may have multiple relationships."
	| tables rows result rightChild rightHandSide |
	tables := self mappedTables.
	rows := Dictionary new.
	tables do: [:each | rows at: each put: (DatabaseRow newForTable: each)].
	rightChild := rootExpression rightChild.
	rightHandSide := rightChild isConstantExpression ifTrue: [rightChild value] ifFalse: [rightChild].
	toDbMappingBlock value: rows value: rightHandSide value: Dictionary new. 
	result := nil.
	rows do: [:eachRow |
		| table |
		table := rootExpression leftChild base getTable: eachRow table.
		eachRow fieldsAndValidValuesDo: [:eachField :eachValue |  | newExp |
			newExp := (table getField: eachField) get: rootExpression relation withArguments: (Array with: eachValue).
			result := newExp AND: result.]].
	^result.!

controlsTables
	^false!

convertedDbValueOf: anObject
	| tables rows |
	tables := self mappedTables.
	rows := Dictionary new.
	tables do: [:each | rows at: each put: (DatabaseRow newForTable: each)].
	toDbMappingBlock value: rows value: anObject.
	rows keysAndValuesDo: [:eachTable :eachRow |
		^eachRow at: mappedFields first].!

field
	| fields |
	fields := self mappedFields.
	fields size = 1 ifFalse: [self error: 'This mapping does not correspond to exactly one field'].
	^fields first.!

forAttribute: aSymbol fromDb: fromBlock toDb: toBlock mappingFields: aFieldCollection
	self
		setAttribute: aSymbol
		fromDb: fromBlock
		toDb: toBlock
		mappingFields: aFieldCollection.!

hasImpliedClauses
	"We may imply more than one clause, or a clause which is different from the one directly implied by the relationship"
	^true!

initialize

	super initialize.
	self shouldProxy: false.!

isRelationship
	"True when the mapping associates different persistent classes."

	^false!

isStoredInSameTable
	^true.!

mapFromObject: anObject intoRowsIn: aRowMap

	| target sourceRows targetRows valueDescriptor |
	self canWrite ifFalse: [^self].
	target := self getValueFrom: anObject.
	target := self session realObjectFor: target.
	target := aRowMap mementoFor: target.
	sourceRows := Dictionary new: 3.
	targetRows := Dictionary new: 3.
	descriptor tables do: [:each |
		sourceRows at: each put: (aRowMap findOrAddRowForTable: each withKey: anObject)].
	valueDescriptor := self system descriptorFor: target.
	valueDescriptor isNil ifFalse: [
		valueDescriptor tables do: [:each |
			targetRows at: each put: (aRowMap findOrAddRowForTable: each withKey: target)]].
	toDbMappingBlock value: sourceRows value: target value: targetRows.!

mapObject: anObject inElementBuilder: anElementBuilder

	| readBlock row |
	row := anElementBuilder row.
	readBlock := [fromDbMappingBlock value: row value: anElementBuilder value: BaseExpression new].
	self shouldProxy 
		ifTrue: [self setValueIn: anObject to: (AdHocProxy readBlock: readBlock)]
		ifFalse: [self setValueIn: anObject to: readBlock value]!

mappedFields

	^mappedFields.!

referencedIndependentObjectsFrom: anObject
	"Assumes that the only object this might affect is our primary attribute. That's probably valid. I think."
	| object otherDescriptor |
	object := self getValueFrom: anObject.
	(object isGlorpProxy and: [object isInstantiated not]) ifTrue: [^#()].
	otherDescriptor := self system descriptorFor: object.
	^otherDescriptor isNil ifTrue: [#()] ifFalse: [Array with: object].!

setAttribute: aSymbol fromDb: fromBlock toDb: toBlock mappingFields: aFieldCollection
 
	self attributeName: aSymbol.
	fromDbMappingBlock := fromBlock.
	toDbMappingBlock := toBlock.
	mappedFields := aFieldCollection.!

shouldProxy

	^shouldProxy.!

shouldProxy: aBoolean

	shouldProxy := aBoolean.!

trace: aTracing context: anExpression
	^self.!

valueIn: anElementBuilder as: anExpression
	^fromDbMappingBlock
		value: anElementBuilder row
		value: anElementBuilder
		value: anExpression.! !
!AdHocMapping categoriesFor: #allRelationsFor:!printing SQL!private! !
!AdHocMapping categoriesFor: #controlsTables!private!testing! !
!AdHocMapping categoriesFor: #convertedDbValueOf:!printing SQL!private! !
!AdHocMapping categoriesFor: #field!mapping!private! !
!AdHocMapping categoriesFor: #forAttribute:fromDb:toDb:mappingFields:!initialize/release!private! !
!AdHocMapping categoriesFor: #hasImpliedClauses!private!testing! !
!AdHocMapping categoriesFor: #initialize!initialize/release!private! !
!AdHocMapping categoriesFor: #isRelationship!private!testing! !
!AdHocMapping categoriesFor: #isStoredInSameTable!private!testing! !
!AdHocMapping categoriesFor: #mapFromObject:intoRowsIn:!public! !
!AdHocMapping categoriesFor: #mapObject:inElementBuilder:!public! !
!AdHocMapping categoriesFor: #mappedFields!mapping!private! !
!AdHocMapping categoriesFor: #referencedIndependentObjectsFrom:!mapping!private! !
!AdHocMapping categoriesFor: #setAttribute:fromDb:toDb:mappingFields:!initialize/release!private! !
!AdHocMapping categoriesFor: #shouldProxy!accessing!private! !
!AdHocMapping categoriesFor: #shouldProxy:!accessing!private! !
!AdHocMapping categoriesFor: #trace:context:!mapping!private! !
!AdHocMapping categoriesFor: #valueIn:as:!mapping!private! !

!AdHocMapping class methodsFor!

forAttribute: aSymbol fromDb: fromBlock toDb: toBlock mappingFields: aFieldCollection 
	^super new 
		setAttribute: aSymbol
		fromDb: fromBlock
		toDb: toBlock
		mappingFields: aFieldCollection! !
!AdHocMapping class categoriesFor: #forAttribute:fromDb:toDb:mappingFields:!instance creation!private! !

ConditionalMapping guid: (GUID fromString: '{8009C7E5-D64E-44AF-9119-665CC7C487F1}')!
ConditionalMapping comment: 'This is a mapping that does different things depending on some testable condition. Depending on whether we''re reading or writing, we are likely to test different things. This is useful if a field can be interpreted in more than one way. The API is rather messy.

Instance Variables:
	cases	<SequenceableCollection of: (Association)>	Used on read - a list of associations representing the cases we test. The key is either a one-argument block to be evaluated or a one-argument selector to be performed by the descriptorSystem, returning a boolean, and the value is the mapping to use if it returns true. 
	conditionalField	<DatabaseField>	The field whose value will be used as the argument to the blocks/selectors in cases.
	otherwiseCase	<Mapping>	If none of the other cases evaluate to true on read, use this mapping.

	conditionalMethod	<Block | Symbol>	Used on write - a zero-argument selector or one-argument block. The selector will be sent to the object being written, the block will be evaluated with that object as the argument. This is expected to return the object that will be written via the appropriate mapping.
	conditionalFieldMapping	<Mapping>	Used on write - the mapping used to write out the value in the conditional field.

'!
!ConditionalMapping categoriesForClass!Unclassified! !
!ConditionalMapping methodsFor!

applicableMappingForConditionalValue: rowValue
	cases
		do:
			[:each | 
			((Dialect isBlock: each key)
				ifTrue: [each key value: rowValue]
				ifFalse: [self descriptor system perform: each key with: rowValue])
				ifTrue: [^each value]].
	^otherwiseCase.!

applicableMappingForObject: anObject
	| conditionalValue |
	conditionalValue := self conditionalValueFor: anObject.
	^self applicableMappingForConditionalValue: conditionalValue.!

applicableMappingForRow: anArray in: anElementBuilder
	"If a conditional field has been specified, then pass in the extracted value. Otherwise pass in the whole row"
	| rowValue |
	rowValue := conditionalField isNil
		ifFalse: [anElementBuilder valueOfField: conditionalField in: anArray]
		ifTrue: [anArray].
	^self applicableMappingForConditionalValue: rowValue.!

attributeName
	^self representativeMapping attributeName.!

cases
	^cases.!

casesDo: aBlock
	"Iterate over all of our cases"
	cases do: [:each | aBlock value: each value].
	aBlock value: otherwiseCase.!

conditionalField: aField

	(conditionalField notNil and: [conditionalField ~~ aField]) ifTrue: [self error: 'You can only test one field in a conditional mapping'].
	conditionalField := aField.!

conditionalFieldMapping
	^conditionalFieldMapping!

conditionalFieldMapping: aMapping
	"This is a write-only mapping for the conditional field value, which writes out the result of performing the conditional method"
	conditionalFieldMapping := aMapping.
	descriptor isNil ifFalse: [aMapping descriptor: descriptor].
	(descriptor mappings includes: aMapping) ifTrue: [self error: 'Mapping is part of a conditional and part of the main descriptor. Be sure to send newMapping: to the conditional, not to the descriptor'].!

conditionalMethod: aSymbol
	(conditionalMethod notNil and: [conditionalMethod ~~ aSymbol]) ifTrue: [self error: 'You can only test one attribute in a conditional mapping'].
	conditionalMethod := aSymbol.!

conditionalValueFor: anObject
	^(Dialect isBlock: conditionalMethod)
		ifTrue: [conditionalMethod value: anObject]
		ifFalse: [anObject perform: conditionalMethod].!

controlsTables
	^self cases first value controlsTables.!

descriptor: aDescriptor

	super descriptor: aDescriptor.
	cases do: [:each | each value descriptor: aDescriptor].
	otherwiseCase isNil ifFalse: [otherwiseCase descriptor: aDescriptor].
	conditionalFieldMapping isNil ifFalse: [conditionalFieldMapping descriptor: aDescriptor].!

field
	^self mappedFields asArray first.!

forField: aField attribute: attributeBlock if: testBlock useMapping: aMapping
	self conditionalField: aField.
	self conditionalMethod: attributeBlock.
	self if: testBlock then: aMapping.!

hasField
	^self mappedFields size = 1.!

if: conditionSelector then: aMapping

	cases add: (Association key: conditionSelector value: aMapping).
	descriptor isNil ifFalse: [aMapping descriptor: descriptor].
	(descriptor mappings includes: aMapping) ifTrue: [self error: 'Mapping is part of a conditional and part of the main descriptor. Be sure to send newMapping: to the conditional, not to the descriptor'].!

initialize
	
	super initialize.
	cases := OrderedCollection new.!

isRelationship

	^false!

joinExpressionFor: anExpression

	^self representativeMapping joinExpressionFor: anExpression.!

mapFromObject: anObject intoRowsIn: aRowMap 
	self canWrite ifFalse: [^self].
	(self applicableMappingForObject: anObject) 
		mapFromObject: anObject
		intoRowsIn: aRowMap.
	conditionalFieldMapping isNil ifTrue: [^self].
	conditionalFieldMapping 
		mapFromObject: (self conditionalValueFor: anObject)
		intoRowsIn: aRowMap!

mapObject: anObject inElementBuilder: anElementBuilder
	(self applicableMappingForRow: anElementBuilder row in: anElementBuilder)
		mapObject: anObject
		inElementBuilder: anElementBuilder.!

mappedFields

	| all |
	all := Set new.
	conditionalFieldMapping isNil 
		ifTrue: [all add: conditionalField]
		ifFalse: [all addAll: conditionalFieldMapping mappedFields].
	cases do: [:each |
		all addAll: each value mappedFields].
	^all.!

newMapping: aMappingClass
	"Implement this so that we can create our sub-mappings without having them added to the parent descriptor as independent mappings"
	| mapping |
	mapping := aMappingClass new.
	mapping descriptor: self descriptor.
	descriptor resetMappedFields.
	^mapping.!

otherwise
	^otherwiseCase.!

otherwise: aMapping

	otherwiseCase := aMapping.
	descriptor isNil ifFalse: [otherwiseCase descriptor: descriptor].
	(descriptor mappings includes: aMapping) ifTrue: [self error: 'Mapping is part of a conditional and part of the main descriptor. Be sure to send newMapping: to the conditional, not to the descriptor'].!

referenceClass
	^self representativeMapping referenceClass.!

referencedIndependentObjectsFrom: anObject

	| allReferencedObjects |
	allReferencedObjects := OrderedCollection new.
	cases do: [:each | 
		allReferencedObjects addAll: (each value referencedIndependentObjectsFrom: anObject)].
	^allReferencedObjects.!

representativeMapping
	"At some points we may need to have this mapping behave some way without knowing yet which of its cases is satisfied. We assume that there is some basic level of similarity so that we can use one mapping as a representative to discover, e.g. what the reference class is. This works all right for cases where we have a mapping or a constant. It wouldn't work very well if we have a one to one mapping to one class in case (a) but a many-many to an entirely different class in the other"
	^cases first value.!

trace: aTracing context: anExpression
	"To make a join, we need to look at all of our possible cases"

	conditionalFieldMapping isNil ifFalse: [
		conditionalFieldMapping trace: aTracing context: anExpression].
	cases do: [:each |
		each value trace: aTracing context: anExpression].!

validateAttribute
	cases do: [:each | each value validateAttribute].
	otherwiseCase validateAttribute.
	conditionalFieldMapping isNil ifFalse: [conditionalFieldMapping validateAttribute].! !
!ConditionalMapping categoriesFor: #applicableMappingForConditionalValue:!mapping!private! !
!ConditionalMapping categoriesFor: #applicableMappingForObject:!mapping!private! !
!ConditionalMapping categoriesFor: #applicableMappingForRow:in:!mapping!private! !
!ConditionalMapping categoriesFor: #attributeName!accessing!private! !
!ConditionalMapping categoriesFor: #cases!accessing!private! !
!ConditionalMapping categoriesFor: #casesDo:!accessing!private! !
!ConditionalMapping categoriesFor: #conditionalField:!accessing!private! !
!ConditionalMapping categoriesFor: #conditionalFieldMapping!accessing!private! !
!ConditionalMapping categoriesFor: #conditionalFieldMapping:!accessing!private! !
!ConditionalMapping categoriesFor: #conditionalMethod:!accessing!private! !
!ConditionalMapping categoriesFor: #conditionalValueFor:!mapping!private! !
!ConditionalMapping categoriesFor: #controlsTables!private!testing! !
!ConditionalMapping categoriesFor: #descriptor:!accessing!private! !
!ConditionalMapping categoriesFor: #field!accessing!private! !
!ConditionalMapping categoriesFor: #forField:attribute:if:useMapping:!conditions!private! !
!ConditionalMapping categoriesFor: #hasField!private!testing! !
!ConditionalMapping categoriesFor: #if:then:!conditions!private! !
!ConditionalMapping categoriesFor: #initialize!initialize/release!private! !
!ConditionalMapping categoriesFor: #isRelationship!private!testing! !
!ConditionalMapping categoriesFor: #joinExpressionFor:!preparing!private! !
!ConditionalMapping categoriesFor: #mapFromObject:intoRowsIn:!mapping!private! !
!ConditionalMapping categoriesFor: #mapObject:inElementBuilder:!mapping!private! !
!ConditionalMapping categoriesFor: #mappedFields!accessing!private! !
!ConditionalMapping categoriesFor: #newMapping:!mapping!private! !
!ConditionalMapping categoriesFor: #otherwise!accessing!private! !
!ConditionalMapping categoriesFor: #otherwise:!conditions!private! !
!ConditionalMapping categoriesFor: #referenceClass!accessing!private! !
!ConditionalMapping categoriesFor: #referencedIndependentObjectsFrom:!mapping!private! !
!ConditionalMapping categoriesFor: #representativeMapping!accessing!private! !
!ConditionalMapping categoriesFor: #trace:context:!conditions!private! !
!ConditionalMapping categoriesFor: #validateAttribute!private!validation! !

ConstantMapping guid: (GUID fromString: '{8B21B1C6-4AF2-41D2-8190-79D2B0395F35}')!
ConstantMapping comment: 'Sometimes you just want a constant value to be set, either in the row, the object or both. And sometimes you just want a non-mapping (e.g. with a ConditionalMapping where one
of the conditions means "this isn''t mapped"). This mapping represents these situations.
It also handles the special case where it''s useful to have access to the session inside a
domain object, by allowing you to map it to an instance variable.

So far only the case of mapping to an inst var is implemented.

Instance Variables:
'!
!ConstantMapping categoriesForClass!Unclassified! !
!ConstantMapping methodsFor!

constantValue
	^constantValue.!

constantValue: anObject

	constantValue := anObject.!

constantValueIn: aSession

	^valueIsSession
		ifTrue: [aSession]
		ifFalse: [constantValue].!

constantValueIsSession

	valueIsSession := true.!

controlsTables
	"Return true if this type of method 'owns' the tables it's associated with, and expression nodes using this mapping should alias those tables where necessary"

	^false!

getValueFrom: anObject

	^constantValue.!

initialize

	super initialize.
	valueIsSession := false.!

isRelationship
	^false!

mapFromObject: anObject intoRowsIn: aRowMap!

mapObject: anObject inElementBuilder: anElementBuilder
	| value |
	value := anElementBuilder isNil
		ifTrue: [constantValue]
		ifFalse: [self constantValueIn: anElementBuilder session].
	self setValueIn: anObject to: value.!

mappedFields
	"Return a collection of fields that this mapping will write into any of the containing object's rows"

	^#().!

referencedIndependentObjectsFrom: anObject

	^#().!

trace: aTracing context: anExpression

	^self.!

validateAttribute
	^self.! !
!ConstantMapping categoriesFor: #constantValue!accessing!private! !
!ConstantMapping categoriesFor: #constantValue:!accessing!private! !
!ConstantMapping categoriesFor: #constantValueIn:!accessing!private! !
!ConstantMapping categoriesFor: #constantValueIsSession!accessing!private! !
!ConstantMapping categoriesFor: #controlsTables!private!testing! !
!ConstantMapping categoriesFor: #getValueFrom:!api!private! !
!ConstantMapping categoriesFor: #initialize!initialize/release!private! !
!ConstantMapping categoriesFor: #isRelationship!private!testing! !
!ConstantMapping categoriesFor: #mapFromObject:intoRowsIn:!mapping!private! !
!ConstantMapping categoriesFor: #mapObject:inElementBuilder:!mapping!private! !
!ConstantMapping categoriesFor: #mappedFields!accessing!private! !
!ConstantMapping categoriesFor: #referencedIndependentObjectsFrom:!mapping!private! !
!ConstantMapping categoriesFor: #trace:context:!mapping!private! !
!ConstantMapping categoriesFor: #validateAttribute!private!validation! !

DirectMapping guid: (GUID fromString: '{E7ADEF26-5633-4402-BF2D-027A5397ECB2}')!
DirectMapping comment: 'This is the simplest type of mapping, it maps directly between an instance variable that holds some "primitive" type, and a field in a database table.

Instance Variables:
	converter	<DatabaseConverter>	The converter used to transform the data into database form and back again. This will be automatically computed based on the types defined, if possible, but for more complex conversions (e.g. booleans into strings in the database) a converter can be explicitly specified.
	field	<DatabaseField | FunctionExpression>	The field to write to. One complication is that (for reading purposes) the field may be a function. So we might have a mapping that reads from the TO_UPPER of some field, or even an aggregate function like MAX.

'!
!DirectMapping categoriesForClass!Unclassified! !
!DirectMapping methodsFor!

buildQuery
	"We just want to read our main object back again. So we create a join from our primary key fields to themselves. One side will remain field references, the other side will turn into parameters when it's converted to a where clause."
	| whereClause |
	whereClause := Join new.
	descriptor primaryTable primaryKeyFields do: [:each |
		whereClause addSource: each target: each].
	query := Query returningOneOf: descriptor describedClass where: whereClause.
	query retrieve: self attribute name.
	query session: self session.
	^query.!

controlsTables
	"Return true if this type of method 'owns' the tables it's associated with, and expression nodes using this mapping should alias those tables where necessary"

	^false!

convertedDbValueOf: stValue 
	^converter isNil
		ifTrue: [stValue]
		ifFalse: [converter convert: stValue toDatabaseRepresentationAs: self field type]!

convertedStValueOf: dbValue 
	^(converter isNil)
		ifTrue: [dbValue]
		ifFalse: [converter convert: dbValue fromDatabaseRepresentationAs: self field type]!

converter
	^converter.!

converter: aDatabaseConverter
	converter := aDatabaseConverter.!

convertValueToDatabaseForm: aValue
	
	converter isNil ifTrue: [^aValue].
	^converter convertedDbValueOf: aValue.!

expressionFor: anObject basedOn: anExpression relation: aSymbol
	"Return our expression using the object's values. e.g. if this was a direct mapping from id->ID and the object had id: 3, then return TABLE.ID=3"

	| value |
	value := anObject isNil
		ifTrue: [nil]
		ifFalse:
			[anObject isGlorpExpression
				ifTrue: [anObject getMapping: self named: self attributeName]
				ifFalse: [anObject glorpIsCollection
					ifTrue: [anObject collect: [:each | attribute getValueFrom: each]]
					ifFalse: [attribute getValueFrom: anObject]]].
	^(anExpression get: self attribute name) get: aSymbol withArguments: (Array with: value).!

field
	^field!

field: aDatabaseFieldOrExpression
	field := (Dialect isBlock: aDatabaseFieldOrExpression)
		ifTrue: [
			self beReadOnly.
			aDatabaseFieldOrExpression asGlorpExpressionOn: (BaseExpression new descriptor: descriptor)]
		ifFalse: [aDatabaseFieldOrExpression].!

fieldsForDescriptorSelectStatement
	"Return a collection of fields that this mapping will read from a row when the containing object is read, as opposed to when we're specifically told to use this field. Normally it amounts to the same thing, but if a direct mapping is proxied, we don't want its field when the parent is read"
	^self shouldProxy ifTrue: [self descriptor primaryTable primaryKeyFields] ifFalse: [super fieldsForDescriptorSelectStatement].!

from: anAttributeName to: aField

	| type |
	self attributeName: anAttributeName.
	self field: (descriptor fieldFor: aField).
	type := self attribute type isNil ifTrue: [field impliedSmalltalkType] ifFalse: [self attribute type].
	self type: type.!

from: anAttributeName type: aClass to: aField

	self
		attributeName: anAttributeName;
		field: aField;
		type: aClass.!

fromPseudoVariable: anAttributeName to: aField

	^self fromPseudoVariable: anAttributeName
		type: aField impliedSmalltalkType
		to: aField!

fromPseudoVariable: anAttributeName type: aClass to: aField

	self from: anAttributeName type: aClass to: aField.
	self beForPseudoVariable.!

initialize
	super initialize.
	shouldProxy := false.!

isRelationship
	"True when the mapping associates different persistent classes."

	^false!

isStoredInSameTable

	^true!

mapFromObject: anObject intoRowsIn: aRowMap

	^self mapFromObject: anObject toTarget: anObject puttingRowsIn: aRowMap.!

mapFromObject: anObject toTarget: target puttingRowsIn: aRowMap

	| dbValue value row |
	self canWrite ifFalse: [^self].
	value := self getValueFrom: anObject.
	value := self session realObjectFor: value ifNone: [^self].
	dbValue := self convertedDbValueOf: value.
	row := aRowMap findOrAddRowForTable: self field table withKey: target.
	row at: field put: dbValue.!

mapObject: anObject inElementBuilder: anElementBuilder
	| value |
	self shouldProxy ifTrue: [
		^self setValueIn: anObject to: ((self newProxyIn: anElementBuilder)
						session: descriptor session;
						parameters: (self parametersIn: anElementBuilder);
						glorpOwner: anObject)].
	value := self valueInBuilder: anElementBuilder.
	self setValueIn: anObject to: value.!

mappedFields
	"Return a collection of fields that this mapping will write into any of the containing object's rows"

	^Array with: self field!

mapsSimpleObject
	^true.!

parametersIn: anElementBuilder
	| parameters |
	parameters := IdentityDictionary new.
	descriptor primaryTable primaryKeyFields do: [:each | parameters at: each put: (anElementBuilder valueOfField: each)].
	^parameters.!

printSQLOn: aStream withParameters: aDictionary
	self field printSQLOn: aStream withParameters:aDictionary.!

query

	query isNil ifTrue: [self buildQuery].
	^query.!

queryFor: anObjectBuilder
	"If we have a specific query we should use in this case, rather than our general one, construct it. Right now that only applies if we're using filtered reads"
	^self query.!

readBackNewRowInformationFor: anObject fromRowsIn: aRowMap 
	| value row |
	field isGenerated ifFalse: [^self].
	self canWrite ifFalse: [^false].
	row := aRowMap findOrAddRowForTable: self field table withKey: anObject.
	value := self convertedStValueOf: (row at: field ifAbsent: [^self]).
	attribute setValueIn: anObject to: value!

referencedIndependentObjectsFrom: anObject

	^#().!

setConverterBetween: aClass and: aDbField 
	aClass isNil ifTrue: [^self].
	converter := descriptor ifNotNil: 
					[descriptor system converterBetweenStType: aClass andField: aDbField]
				ifNil: [aDbField converterForStType: aClass]!

shouldProxy

	^shouldProxy.!

shouldProxy: aBoolean

	shouldProxy := aBoolean.!

trace: aTracing context: anExpression
	^self.!

type: aClass
	self attribute type: aClass.
	field isNil ifFalse: [
		self setConverterBetween: aClass and: field].!

validate
	super validate.
	(field isGlorpExpression and: [self readOnly not]) ifTrue: [self error: 'Mappings to functions cannot be written'].!

valueIn: anElementBuilder as: anExpression
	| dbValue valueToGet |
	valueToGet := anExpression canTranslateFields 
		ifTrue: [anExpression translateField: field] 
		ifFalse: [anExpression].
	dbValue := anElementBuilder
		valueOfField: valueToGet.
	^self convertedStValueOf: dbValue.!

valueInBuilder: anElementBuilder
	^self valueInBuilder: anElementBuilder as: field.!

valueInBuilder: anElementBuilder as: anExpression
	| dbValue |
	dbValue := anElementBuilder valueOfField: anExpression.
	^self convertedStValueOf: dbValue.!

valueOfField: aField fromObject: anObject
	field = aField ifFalse: [self error: 'Mapping doesn''t describe field'].
	^self convertedDbValueOf: (self getValueFrom: anObject).! !
!DirectMapping categoriesFor: #buildQuery!mapping!private! !
!DirectMapping categoriesFor: #controlsTables!private!testing! !
!DirectMapping categoriesFor: #convertedDbValueOf:!private! !
!DirectMapping categoriesFor: #convertedStValueOf:!private! !
!DirectMapping categoriesFor: #converter!accessing!private! !
!DirectMapping categoriesFor: #converter:!accessing!private! !
!DirectMapping categoriesFor: #convertValueToDatabaseForm:!mapping!private! !
!DirectMapping categoriesFor: #expressionFor:basedOn:relation:!mapping!private! !
!DirectMapping categoriesFor: #field!accessing!private! !
!DirectMapping categoriesFor: #field:!accessing!private! !
!DirectMapping categoriesFor: #fieldsForDescriptorSelectStatement!accessing!private! !
!DirectMapping categoriesFor: #from:to:!initialize-release!private! !
!DirectMapping categoriesFor: #from:type:to:!initialize-release!private! !
!DirectMapping categoriesFor: #fromPseudoVariable:to:!initialize-release!private! !
!DirectMapping categoriesFor: #fromPseudoVariable:type:to:!initialize-release!private! !
!DirectMapping categoriesFor: #initialize!initialize-release!private! !
!DirectMapping categoriesFor: #isRelationship!private!testing! !
!DirectMapping categoriesFor: #isStoredInSameTable!private!testing! !
!DirectMapping categoriesFor: #mapFromObject:intoRowsIn:!mapping!private! !
!DirectMapping categoriesFor: #mapFromObject:toTarget:puttingRowsIn:!mapping!private! !
!DirectMapping categoriesFor: #mapObject:inElementBuilder:!mapping!private! !
!DirectMapping categoriesFor: #mappedFields!private!testing! !
!DirectMapping categoriesFor: #mapsSimpleObject!private!testing! !
!DirectMapping categoriesFor: #parametersIn:!mapping!private! !
!DirectMapping categoriesFor: #printSQLOn:withParameters:!printing SQL!private! !
!DirectMapping categoriesFor: #query!mapping!private! !
!DirectMapping categoriesFor: #queryFor:!mapping!private! !
!DirectMapping categoriesFor: #readBackNewRowInformationFor:fromRowsIn:!mapping!private! !
!DirectMapping categoriesFor: #referencedIndependentObjectsFrom:!enumerating!private! !
!DirectMapping categoriesFor: #setConverterBetween:and:!initialize-release!private! !
!DirectMapping categoriesFor: #shouldProxy!accessing!private! !
!DirectMapping categoriesFor: #shouldProxy:!accessing!private! !
!DirectMapping categoriesFor: #trace:context:!mapping!private! !
!DirectMapping categoriesFor: #type:!accessing!private! !
!DirectMapping categoriesFor: #validate!private!validation! !
!DirectMapping categoriesFor: #valueIn:as:!mapping!private! !
!DirectMapping categoriesFor: #valueInBuilder:!mapping!private! !
!DirectMapping categoriesFor: #valueInBuilder:as:!mapping!private! !
!DirectMapping categoriesFor: #valueOfField:fromObject:!mapping!private! !

!DirectMapping class methodsFor!

from: attributeName to: field

	^self from: attributeName
		type: field impliedSmalltalkType
		to: field!

from: attributeName type: aClass to: field

	^self new
		attributeName: attributeName;
		field: field;
		type: aClass.!

fromPseudoVariable: attributeName to: field

	^self fromPseudoVariable: attributeName
		type: field impliedSmalltalkType
		to: field!

fromPseudoVariable: attributeName type: aClass to: field

	^self new
		attributeName: attributeName;
		field: field;
		beForPseudoVariable.! !
!DirectMapping class categoriesFor: #from:to:!instance creation!private! !
!DirectMapping class categoriesFor: #from:type:to:!instance creation!private! !
!DirectMapping class categoriesFor: #fromPseudoVariable:to:!instance creation!private! !
!DirectMapping class categoriesFor: #fromPseudoVariable:type:to:!instance creation!private! !

RelationshipMapping guid: (GUID fromString: '{A59D4E5D-3624-4B11-8736-3BDC058E865E}')!
RelationshipMapping comment: 'This is an abstract superclass for relationships between one mapped object and another (or a collection of others).

Instance Variables:
	join	<Join>	The join between our table and the target. Note that if we use a link table, this will be a join between our table and the link table.
	targetTableJoin	<Join>	If we''re using a link table, it may be desirable to also join to attributes of the main table. This optional extra join allows that.

	query	<SimpleQuery>	The query that will return the other side of the relationship, given parameters describing our side.
	linkFields	<Collection of: DatabaseField>	When there''s a link table, we need to know the other side of the relationship, from link table to target. Usually we can figure it out based on the target table and the foreign key constraints, but sometimes (e.g. if there are multiple relationships to the same table) we need to be told which fields are the ones that connect to this table. Again that''s FROM the LINK table TO the TARGET table, not the entry in the link table that we''d connect  to.
	reverseJoin	<Join>	caches the other end of the relationship, as above.
	rowMapKeyConstructorBlock	<BlockClosure>	Usually row map keys are constructed automatically, but it''s possible that in weird cases we  might need to construct complicated ones (e.g. if there are 3 things that need to participate instead of just two, as in Store''s use of the same link table and the same row to indicate the relationship from package to class and package to metaclass).
	shouldProxy	<Boolean>	Should we proxy or directly construct the referenced object
	shouldUseFilteredRead	<Boolean>	should we use a filtered read.
	usesLinkTable	<Boolean>	Do we use a link table, or is it a direct join to the target.
	isExclusive <Boolean> Is this an exclusive relationship - so that when the parent object is deleted, the child or children will also be deleted.

'!
!RelationshipMapping categoriesForClass!Unclassified! !
!RelationshipMapping methodsFor!

addPhantomMappingExpressionOn: base 
	| phantomMapping phantomMappingName phantomExpression |
	phantomMapping := self reverseMapping.
	phantomMappingName := 'glorp_internal_' , self attributeName , '_to_' 
				, self descriptor describedClass name.
	phantomExpression := base getMapping: phantomMapping
				named: phantomMappingName.
	phantomExpression bePhantom.
	^phantomExpression!

allTables

	(self join) isNil ifTrue: [^#()].
	^(self join) allTables.!

basicReferenceClass
	^self attribute type.!

basicReferenceDescriptor
	^self system descriptorFor: self basicReferenceClass.!

beExclusive
	"Make this an exclusive relationship - so that when the parent object is deleted, the child or children will also be deleted"
	isExclusive := true.!

buildGeneralQueryIn: anObjectBuilder
	"For filtered queries. We need to build this in terms of the parent query"

	| base parentCriteria fullCriteria phantomExpression newQuery |
	base := BaseExpression new.
	base descriptor: self referenceDescriptor.
	phantomExpression := self addPhantomMappingExpressionOn: base.
	parentCriteria := anObjectBuilder query whereClause.
	fullCriteria := parentCriteria
		rebuildOn: phantomExpression.
	newQuery := Query returningManyOf: self referenceClass.
	"newQuery collectionType: self collectionType."
	fullCriteria isEmptyExpression ifFalse: [newQuery where: fullCriteria].
	self
		setGeneralQueryRetrievalIn: newQuery
		basedOn: base
		phantom: phantomExpression.
	^newQuery.!

buildQuery
	| toMany |
	toMany := self attribute isCollectionAttribute.
	query := toMany
		ifTrue: [Query read: self referenceClass where: self join]
		ifFalse: [Query readOneOf: self referenceClass where: self join].
	query session: self session.
	query baseExpression descriptor: (self session descriptorFor: self referenceClass).
	query collectionType: self collectionType.
	self usesLinkTable ifTrue: [
		query AND: (self expressionFromLinkToReferenceTableWithBase: query baseExpression)].
	^query.!

buildRowMapKeyForSource: source target: target index: index
	^(RowMapKey new)
		key1: target yourSelf;
		key2: source yourSelf.!

collectionType
	^nil.!

constraints
	"Return a list of the constraints from our link table. If the linkField is set, filter the constraints to those that include that field. "
	| referenceKeys linkTable referenceTables allConstraints relevantConstraints |
	referenceKeys := (self join) targetKeys asOrderedCollection.
	linkTable := referenceKeys first table.
	"If we haven't been told the relevant link table fields, assume we can find them by looking at all the ones that aren't the ones from our source to the link, and all the rest will be from the link to the target"
	allConstraints := linkTable foreignKeyConstraints.
	relevantConstraints := linkFields isNil 
		ifTrue: [allConstraints reject: [:each |  each sourceFields allSatisfy: [:eachSourceField |
			(referenceKeys includes: eachSourceField)]]]
		ifFalse: [allConstraints select: [:each | each sourceFields anySatisfy: [:eachSourceField |
			linkFields includes: eachSourceField]]].

	"Validate that we can handle this case"
	referenceTables := (relevantConstraints collect: [:each | each targetTable]) asSet.
	referenceTables size > 1 ifTrue: [self error: 'Cannot handle this general a case. Try hinting using relevantLinkTableFields:'].
	referenceTables size = 0 ifTrue: [self error: 'No tables found. Did you set up foreign key references in the table definitions?'].

	^relevantConstraints.!

constraintsFrom: oneTableSet to: anotherTableSet
	^(self oneWayConstraintsFrom: oneTableSet to: anotherTableSet), 
		(self oneWayConstraintsFrom: anotherTableSet to: oneTableSet)!

constructRowMapKeyAs: aBlock
	"Give us the opportunity to construct a custom row map key. This is useful if you need to force two relationships to share a link table entry"
	rowMapKeyConstructorBlock := aBlock.!

controlsTables
	"Return true if this type of method 'owns' the tables it's associated with, and expression nodes using this mapping should alias those tables where necessary"

	^controlsTables!

controlsTables: aBoolean
	"We can indicate that this relationship does not actually control its tables. This is going to be true if the parent controls the same tables and coexists with us. Think StoreVersionlessPundle and Pundle"
	controlsTables := aBoolean.!

createDeleteRowFor: anObject in: aRowMap
	"This is like firing the mapping, but instead of creating the data, we just need to check if there are link table entries, and if so, make sure they are deleted"
	| target mementoizedTarget|
	self canWrite ifFalse: [^self].
	self usesLinkTable ifFalse: [^self].
	target := self getValueFrom: anObject.
	target := self session realObjectFor: target ifNone: [^self].
	(self isValidTarget: target) ifTrue: [
		mementoizedTarget := aRowMap collectionMementoFor: target.
		self createDeleteRowForLinkFrom: anObject toTarget: mementoizedTarget in: aRowMap].!

createDeleteRowForLinkFrom: anObject toTarget: target in: aRowMap
	| linkTable |
	self usesLinkTable ifFalse: [^false].
	linkTable := self linkTableJoin allTables asArray first.
	self attribute
		loopOver: target
		doing:
			[:index :eachTarget | 
			| intermediate |
			intermediate := self
				rowMapKeyForSource: anObject
				target: eachTarget
				index: index.
			aRowMap
				rowForTable: linkTable
				withKey: intermediate
				ifPresent:
					[:linkRow | linkRow isEmpty ifFalse: [linkRow forDeletion: true]]].!

expressionFromLinkToReferenceTableWithBase: base

	| reverse |
	reverse := self reverseJoin asGlorpExpressionOn: base.
	^reverse asGeneralGlorpExpressionWithFields.!

extendedJoin
	"In order to do a many-to-many read we need more information than just the write, we need to know 
	the relationship to the other table. Construct that based on the table information"

	| generalMappingCriteria base |
	self usesLinkTable ifFalse: [^self join].
	generalMappingCriteria := self join asGeneralGlorpExpressionWithFields.
	base := generalMappingCriteria ultimateBaseExpression.
	^(self expressionFromLinkToReferenceTableWithBase: base) AND: generalMappingCriteria.!

extendedJoinFromSource: sourceExpression toTarget: targetExpression
	"In order to do a many-to-many read we need more information than just the write, we need to know 
	the relationship to the other table. Construct that based on the table information"

	| generalMappingCriteria |
	self usesLinkTable 
		ifFalse: [^self join asExpressionJoiningSource: sourceExpression toTarget: targetExpression].

	generalMappingCriteria := self join asExpressionJoiningSource: sourceExpression toTarget: targetExpression.
	^(self expressionFromLinkToReferenceTableWithBase: targetExpression) AND: generalMappingCriteria.!

hasValueThatCanProvideField: aField from: anObject
	"Return true if we have enough information in ourselves to populate our fields. The only time this won't be true is if we're either holding an uninstantiated proxy or have a to-many relationship mapping with an empty collection. Assumes that aField is one that we map."
	| value |
	value := self getValueFrom: anObject.
	value isGlorpProxy ifFalse: [^value notNil].
	value isInstantiated ifTrue: [^true].
	^value parameters includesKey: aField.!

initialize

	super initialize.
	shouldProxy := true.
	shouldUseFilteredRead := false.
	usesLinkTable := false.
	controlsTables := true.
	isExclusive := false.!

initializeJoin
	"We haven't been given a join. There might be a compound one, or else see if we can make an educated guess at one"
	linkTableJoin notNil ifTrue: [join := linkTableJoin, targetTableJoin. ^self].
	self initializeJoinTo: self relatedTables.!

initializeJoinTo: otherTables
	"We haven't been giving a join. See if we can make an educated guess at one by looking for tables with constraints to us. If linkFields is set, it tells us the source field for the relationship between the link table and the target, so in looking for our own relationship to the link table, we exclude that."
	| toConstraints fromConstraints |
	self descriptor tables isEmpty ifTrue: [^self join: Join new].
	(self referenceDescriptor tables isEmpty and: [self referenceDescriptor typeResolver usesInheritance]) ifTrue: [^self join: Join new].
	toConstraints := self oneWayConstraintsFrom: self descriptor tables to: otherTables.
	linkFields isNil ifFalse: [toConstraints := toConstraints reject: [:eachConstraint | eachConstraint targetFields anySatisfy: [:eachField | linkFields includes: eachField]]].
	fromConstraints :=  self oneWayConstraintsFrom: otherTables to: self descriptor tables.
	linkFields isNil ifFalse: [fromConstraints := fromConstraints reject: [:eachConstraint | eachConstraint sourceFields anySatisfy: [:eachField | linkFields includes: eachField]]].

	self join: (self system joinFor: self toTables: otherTables fromConstraints: fromConstraints toConstraints: toConstraints).!

isExclusive
	"Return whether this an exclusive relationship - so that when the parent object is deleted, the child or children will also be deleted"
	^isExclusive.!

isExclusive: aBoolean
	"Set whether this an exclusive relationship - so that when the parent object is deleted, the child or children will also be deleted"
	isExclusive := aBoolean.!

isRelationship
	"True when the mapping associates different persistent classes."

	^true!

isStoredInSameTable
	"Normally, a relationship mapping isn't stored in the same table, but if we're part of a dictionary mapping and the table is mapped to an association, then we want to be treated that way. The test is a horrible hack"
	^self classModel describedClass == Association.!

isToManyRelationship
	^self attribute isCollectionAttribute.!

isValidTarget: anObject

	^anObject isGlorpProxy
		ifTrue: [anObject isInstantiated]
		ifFalse: [true].!

join
	join isNil ifTrue: [self initializeJoin].
	^join.!

join: aJoin
	"We allow the join to be specified either as a Join (the normal case) or as an expression. We expect an expression specifying the relationship in terms of other relationships. So, e.g. to specify the join from InsurancePlan to PolicyHolder we might specify
[:eachPolicyHolder | eachPolicyHolder employer availablePlans].  Note that the relationship is described from the point of view of the target, and is just a list of mappings. Then we take this internally and transform it into an equality expression we can use to define the relationship."
	| base basicExpression |
	(Dialect isBlock: aJoin) 
		ifTrue: [
			self beReadOnly.
			base := JoinBaseExpression new.
			base descriptor: (self system descriptorFor: self referenceClass).
			basicExpression := aJoin asGlorpExpressionOn: base.
			"It doesn't matter what name we give the parameter, because it will be rewritten into a primary key comparison anyway, that will use the fields as 'names'. Also, it's important that we assign into join here, because otherwise the rewriteEquality operation will try and figure out a join for us, since it's still nil."
			join := basicExpression get: #= withArguments: (
				Array with: (ParameterExpression forField: 'sourceObject' basedOn: basicExpression join)).
			join := join rewriteEquality]
		ifFalse: [join := aJoin].
	[join owningMapping: self] on: MessageNotUnderstood do: [:ex | ex | ex return].!

joinExpressionFor: targetExpression
	"We're looking for the object represented by this mapping, and we know the object represented by its source. Use our mapping criteria to construct a join that traverses that instance of this relationship"

	| sourceExpression |
	(self controlsTables not and: [self classModel describedClass == Association]) ifTrue: [^nil].
	sourceExpression := targetExpression base.
	^self extendedJoinFromSource: sourceExpression toTarget: targetExpression.!

joinFromLinkToReferenceTable
	"Generate the inverse mapping expression, i.e. the one connecting the link table to the reference table, using the foreign key constraints. We assume that if there's a general part, it applies only to the original join, and not to the reverse join that we're generating."

	| expression generalPart |
	expression := Join new.
	self constraints do: [:eachConstraint | 
		eachConstraint sourceFields with: eachConstraint targetFields do: 
			[:eachSource :eachTarget |
				expression addSource: eachSource target: eachTarget]].
	generalPart := self join generalExpressionPart.
	^generalPart isNil ifTrue: [expression] ifFalse: [generalPart AND: expression].!

knitFromBuilder: sourceBuilder toBuilder: targetBuilder 
	self canRead ifFalse: [^self].
	self attribute knit: sourceBuilder instance to: targetBuilder instance!

linkField: aDatabaseField
	linkFields := Array with: aDatabaseField.!

linkFields
	^linkFields.!

linkFields: aCollection
	"Set the fields that are foreign keys *from* the link table (this is only used if we have one) to the *target* table. This is used as a hint in the case where the link table is ambiguous, and is easier than specifying the full join."
	linkFields := aCollection.!

linkTableJoin
	linkTableJoin isNil ifTrue: [^self join].
	^linkTableJoin!

linkTableJoin: aJoin
	"Set the join from the link table to the target table. Normally this is calculated automatically (see reverseJoin), but it can be set manually if that's not possible (for example, because you want to include a constant value in the join, that won't be included in the foreign key constraints). See also relevantLinkTableFields if you need to give a hint."
	linkTableJoin := aJoin.
	[aJoin owningMapping: self] on: MessageNotUnderstood do: [:ex | ex | ex return].!

mapFromObject: anObject intoRowsIn: aRowMap 
	"Our target is a collection. The tricky bit is that if we're building rows into a RowMapForMementos, then the collection we contain isn't the one we want to use. We want the old version. Ask the row map to give it to us. If it's a normal row map, we'll just get the same thing back" 
	| target mementoizedTarget|
	self canWrite ifFalse: [^self].
	target := self getValueFrom: anObject.
	target := self session realObjectFor: target ifNone: [^self].
	(self isValidTarget: target) ifTrue: [
		mementoizedTarget := aRowMap collectionMementoFor: target.
		self mapFromObject: anObject toTarget: mementoizedTarget puttingRowsIn: aRowMap].!

mapFromObject: anObject toTarget: target puttingRowsIn: aRowMap
	self attribute
		loopOver: target
		doing:
			[:index :eachTarget | 
			| intermediate |
			intermediate := self
				rowMapKeyForSource: anObject
				target: eachTarget
				index: index.
			self
				mapOneFromObject: anObject
				toTarget: eachTarget
				intermediate: intermediate
				atPosition: index
				puttingRowsIn: aRowMap].!

mapObject: anObject inElementBuilder: anElementBuilder
	| value session parameters |
	session := descriptor session.
	parameters := self parametersIn: anElementBuilder.
	value := self shouldProxy
		ifTrue:
			[| proxy |
			proxy := (self newProxyIn: anElementBuilder)
				session: session;
				parameters: parameters;
				glorpOwner: anObject.
			"If we know in advance the proxy would return nil, or an empty collection, just put that in rather than the proxy"
			(proxy query shortCircuitEmptyReturn: parameters)
				ifTrue: [proxy getValue]
				ifFalse: [proxy]]
		ifFalse:
			[(self queryFor: anElementBuilder)
				executeWithParameters: parameters
				in: session].
	self setValueIn: anObject to: value.!

mapOneFromObject: sourceObject toTarget: targetObject intermediate: aRowMapKey atPosition: index puttingRowsIn: aRowMap
	self linkTableJoin
		mapFromSource: sourceObject
		andTarget: aRowMapKey
		intoRowsIn: aRowMap.
	self targetTableJoin isNil
		ifFalse:
			[self targetTableJoin
				mapFromSource: sourceObject
				andTarget: (self mapsSimpleObject ifTrue: [aRowMapKey] ifFalse: [targetObject])
				intoRowsIn: aRowMap].
	self usesLinkTable
		ifTrue:
			[self reverseJoin
				mapFromSource: aRowMapKey
				andTarget: (self mapsSimpleObject ifTrue: [aRowMapKey] ifFalse: [targetObject])
				intoRowsIn: aRowMap].
	self
		writeExtraInformationInto: aRowMap
		atKey: aRowMapKey
		forSourceObject: sourceObject
		targetObject: targetObject
		atIndex: index.!

mappedFields
	"Return a collection of fields that this mapping will write into any of the containing object's rows"

	^self join allSourceFields select: [:each | each isMappable and: [(each isGlorpExpression not or: [each isConstantExpression not])]].!

mappingCriteria
	^self join.!

mappingCriteria: aJoin
	self join: aJoin.!

multipleTableExpressionsFor: anExpression
	^self referenceDescriptor multipleTableJoin
		collect: [:each | each asExpressionJoiningSource: anExpression toTarget: anExpression]!

needsSpecialRowMapKey
	"Return true if we have to use a special key object for the row map, rather than just our target"
	^self usesLinkTable.!

oneWayConstraintsFrom: oneTableSet to: anotherTableSet
	| matches |
	matches := OrderedCollection new: 3.
	oneTableSet do: [:eachFromTable |
		 anotherTableSet do: [:eachToTable |
			matches addAll:  (eachFromTable foreignKeyConstraints
					select: [:eachConstraint | eachConstraint refersToTable: eachToTable])]].
	^matches.!

parametersIn: anElementBuilder
	| parameters |
	self shouldUseFilteredRead ifTrue: [^anElementBuilder command parameters].
	parameters := IdentityDictionary new.
	self join
		mappableSourceFieldsDo:
			[:eachSource | parameters at: eachSource put: (anElementBuilder valueOfField: eachSource)].
	^parameters.!

proxyAction
	"Return an action each proxy should take when it's instantiated. This is a two-argument block
		[:owner :target | "
	^proxyAction.!

proxyAction: aBlock
	"Set an action each proxy should take when it's instantiated. This is a two-argument block
		[:owner :target | "
	proxyAction := aBlock.!

query

	query isNil ifTrue: [self buildQuery].
	^query.!

queryFor: anObjectBuilder 
	"If we have a specific query we should use in this case, rather than our general one, construct it. Right now that only applies if we're using filtered reads"
	| basicQuery collection primaryKey newQuery cachedQuery filterBlock |
	self shouldUseFilteredRead 
		ifFalse: [^self query].
	cachedQuery := anObjectBuilder cachedQueryFor: self.
	cachedQuery isNil 
		ifTrue: 
			[basicQuery := self buildGeneralQueryIn: anObjectBuilder.
			self query tracing alsoFetchExpressions 
				do: [:each | basicQuery alsoFetch: each].
			collection := GlorpVirtualCollection new setQuery: basicQuery]
		ifFalse: [collection := cachedQuery collection].
	primaryKey := anObjectBuilder primaryKey.
	filterBlock := self descriptor keyFields size = 1 
				ifTrue: [[:objects :parameters | (objects at: 1) = primaryKey]]
				ifFalse: 
					[
					[:objects :parameters | 
					(1 to: primaryKey size) inject: true
						into: [:sum :i | sum and: [(primaryKey at: i) = (objects at: i)]]]].
	newQuery := FilteringQuery on: collection filter: filterBlock.
	newQuery collectBlock: [:each | each last].
	newQuery readsOneObject: self attribute isCollectionAttribute not.
	newQuery collectionType: self collectionType.
	anObjectBuilder cachedQueryFor: self is: newQuery.
	^newQuery!

referenceClass: aClass
	self attribute type: aClass.!

referenceDescriptor

	^self system descriptorFor: self referenceClass.!

referencedIndependentObjectsFrom: anObject

	| value |
	self canWrite ifFalse: [^#()].
	value := self getValueFrom: anObject.
	value == nil ifTrue: [^#()].
	^Array with: value.!

referenceTableFieldFor: aField
	^self usesLinkTable
		ifTrue: [nil]
		ifFalse: [self join targetForSource: aField].!

relatedTables
	| otherTables possibleLinkTables |
	otherTables := self basicReferenceDescriptor tables.
	usesLinkTable ifFalse: [^otherTables].

	linkFields isNil ifFalse: [^(linkFields collect: [:each | each table]) asSet asArray].
	possibleLinkTables := self system allTables
		select:
			[:each | 
			(each hasConstraintsToAnyOf: self descriptor tables)
				and: [each hasConstraintsToAnyOf: otherTables]].
	possibleLinkTables size ~= 1
		ifTrue: [CannotAutomaticallyDetermineJoin new
			tables: possibleLinkTables;
			signal: 'Cannot automatically determine link table..'].
	^possibleLinkTables.!

relevantLinkTableFields
	^linkFields.!

relevantLinkTableFields: aCollection
	linkFields := aCollection.!

reverseJoin

	reverseJoin isNil ifTrue: [reverseJoin := self joinFromLinkToReferenceTable].
	^reverseJoin.!

reverseJoin: aJoin
	"Set the reverse join - the join from the link table to the target table. Normally this will be calculated based on the foreign key constraints, but in rare cases (e.g. a foreign key that may be to more than one table) you may want to set it explicitly"

	reverseJoin := aJoin.!

reverseMapping
	"Construct the backwards mapping that corresponds to us. If it's using a link table, then we are at least potentially a toMany mapping. Otherwise, we can point backwards only to a single object."
	| mapping |
	^self usesLinkTable
		ifTrue: 
			[mapping := (ManyToManyMapping new)
				attributeName: #temporaryAttributeFromSubselect;
				referenceClass: self descriptor describedClass;
				join: self reverseJoin inverseJoin;
				descriptor: self descriptor.
			mapping reverseJoin: self join inverseJoin onlyJoinParts]
		ifFalse: 
			[(OneToOneMapping new)
				attributeName: #temporaryAttributeFromSubselect;
				referenceClass: self descriptor describedClass;
				join: self join inverseJoin;
				descriptor: self descriptor]!

rowMapKeyForSource: source target: target index: index
	"Create the row map key. Make sure we're holding onto the real objects, so proxies don't time out on us"

	self needsSpecialRowMapKey ifFalse: [^target].
	(target yourSelf isNil or: [source yourSelf isNil]) ifTrue: [^nil].
	^rowMapKeyConstructorBlock isNil
		ifTrue:
			[self
				buildRowMapKeyForSource: source
				target: target
				index: index]
		ifFalse:
			[rowMapKeyConstructorBlock value: source yourSelf value: target yourSelf].!

setGeneralQueryRetrievalIn: newQuery basedOn: base phantom: phantomExpression
	self descriptor keyFields
		do: [:eachField | newQuery retrieve: (phantomExpression getField: eachField)].
	newQuery retrieve: base.!

shouldProxy

	^shouldProxy.!

shouldProxy: aBoolean

	shouldProxy := aBoolean.!

shouldUseFilteredRead
	^shouldUseFilteredRead.!

shouldUseFilteredRead: aBoolean
	shouldUseFilteredRead := aBoolean.!

singleJoinFromLinkToReferenceTableForConstraints: constraints
	"Generate the inverse mapping expression, i.e. the one connecting the link table to the reference table, using the foreign key constraints. We assume that if there's a general part, it applies only to the original join, and not to the reverse join that we're generating."

	| expression generalPart |
	expression := Join new.
	constraints do: [:eachConstraint | 
		eachConstraint sourceFields with: eachConstraint targetFields do: 
			[:eachSource :eachTarget |
				expression addSource: eachSource target: eachTarget]].
	generalPart := self join generalExpressionPart.
	^generalPart isNil ifTrue: [expression] ifFalse: [generalPart AND: expression].!

targetTableJoin
	^targetTableJoin!

targetTableJoin: aJoin
	"Set the join from the source table to the target table. Note that is is only required if you are using a link table, and in addition to the link table join need to connect from the source table directly to the target table as well.  This is *NOT* supposed to be a join from the link table to the target table. That's reverseJoin, and is calculated automatically. See relevantLinkTableFields if you need to give a hint, or linkTableJoin: if you want to set it explicitly."
	targetTableJoin := aJoin.
	aJoin owningMapping: self.!

targetTables
	^((self usesLinkTable ifTrue: [self reverseJoin] ifFalse: [self join]) allTargetFields collect: [:each | each table]) asSet.!

trace: aTracing context: anExpression
	"Currently we don't trace relationships across tables, so all we do here
is accumulate the list of embedded mappings"
	| newContext |
	(aTracing tracesThrough: self) ifFalse: [^self].
	newContext := anExpression get: self attributeName.
	aTracing addExpression: newContext.
	self referenceDescriptor trace: aTracing context: newContext.!

useFilteredRead
	shouldUseFilteredRead := true.!

useLinkTable
	usesLinkTable := true.!

usesLinkTable
	^usesLinkTable!

usesLinkTable: anObject
	usesLinkTable := anObject!

validate
	| targetJoinSourceFields targetJoinTargetTables |
	super validate.
	self join isNil ifTrue: [^self].
	targetJoinSourceFields := targetTableJoin isNil ifTrue: [#()] ifFalse: [targetTableJoin allSourceFields].
	targetJoinTargetTables := targetTableJoin isNil
		ifTrue: [#()]
		ifFalse: [targetTableJoin allTargetFields collect: [:each | each table]].
	self join allSourceFields asArray , targetJoinSourceFields
		do:
			[:each | 
			(each isDatabaseField or: [each isKindOf: FunctionExpression])
				ifTrue:
					[(self descriptor tables includes: each table)
						ifFalse: [self error: 'Joins should go from the table(s) associated with this descriptor to the other table(s)']]].
	self targetTables asArray , targetJoinTargetTables
		do:
			[:each | 
			(self referenceDescriptor allTables includes: each)
				ifFalse: [self error: 'Joins should go to the table(s) associated with the reference class of the mapping']].

	join allSourceFields asSet size = join allSourceFields size
		ifFalse:
			[self error: 'The same field occurs twice in the join. This might be legal, but is probably a cut and paste error.'].
	join allTargetFields asSet size = join allTargetFields size
		ifFalse:
			[self error: 'The same field occurs twice in the join. This might be legal, but is probably a cut and paste error.'].

	(self isExclusive and: [self canWrite not])
		ifTrue: [self error: 'It is not meaningful to have an exclusive relationship that is read-only.']!

valueOfField: aField fromObject: anObject
	^self valueOfField: aField fromObject: anObject ifNone: [self error: 'Cannot find value'].!

valueOfField: aField fromObject: anObject ifNone: aBlock
	"Return the value of a given field in our target object. Used in extracting primary keys when they are mapped via relationships"
	| targetField target |
	targetField := self referenceTableFieldFor: aField.
	targetField isNil ifTrue: [^aBlock value].
	target := self getValueFrom: anObject.
	"If it's a proxy and uninstantiated, we have to get the value from the proxy"
	(target isGlorpProxy and: [target isInstantiated not])
		ifTrue: [^target parameters at: aField].
	"If the object is instantiated, then we prefer to get the value from the object, because if it's non-primary key then the proxy may not know it, or it may vary from what the proxy has. Well, that seems unlikely, but try this anyway"
	self attribute 	
		loopOver: target 
		doing: [:index :eachTarget | | value |
			value := self referenceDescriptor valueOfField: targetField fromObject: eachTarget ifNone: [self].
			value == self ifFalse: [^value]].
	"We couldn't get it from the object, probably because it's unmapped. See if the proxy has it after all"
	target isGlorpProxy ifTrue: [^target parameters at: aField].
	"We couldn't get the value at all. This may mean that the object is new, so it has no proxies holding the value. Return nil and hope it's new and that it will all get figured out"
	^aBlock value.!

writeExtraInformationInto: aRowMap atKey: rowMapKey forSourceObject: sourceObject targetObject: targetObject  atIndex: index! !
!RelationshipMapping categoriesFor: #addPhantomMappingExpressionOn:!mapping!private! !
!RelationshipMapping categoriesFor: #allTables!accessing!private! !
!RelationshipMapping categoriesFor: #basicReferenceClass!accessing!private! !
!RelationshipMapping categoriesFor: #basicReferenceDescriptor!accessing!private! !
!RelationshipMapping categoriesFor: #beExclusive!api!private! !
!RelationshipMapping categoriesFor: #buildGeneralQueryIn:!mapping!private! !
!RelationshipMapping categoriesFor: #buildQuery!mapping!private! !
!RelationshipMapping categoriesFor: #buildRowMapKeyForSource:target:index:!mapping!private! !
!RelationshipMapping categoriesFor: #collectionType!accessing!private! !
!RelationshipMapping categoriesFor: #constraints!private!private/expressions! !
!RelationshipMapping categoriesFor: #constraintsFrom:to:!initializing!private! !
!RelationshipMapping categoriesFor: #constructRowMapKeyAs:!api!private! !
!RelationshipMapping categoriesFor: #controlsTables!private!testing! !
!RelationshipMapping categoriesFor: #controlsTables:!accessing!private! !
!RelationshipMapping categoriesFor: #createDeleteRowFor:in:!mapping!private! !
!RelationshipMapping categoriesFor: #createDeleteRowForLinkFrom:toTarget:in:!mapping!private! !
!RelationshipMapping categoriesFor: #expressionFromLinkToReferenceTableWithBase:!private!private/expressions! !
!RelationshipMapping categoriesFor: #extendedJoin!mapping!private! !
!RelationshipMapping categoriesFor: #extendedJoinFromSource:toTarget:!mapping!private! !
!RelationshipMapping categoriesFor: #hasValueThatCanProvideField:from:!private!testing! !
!RelationshipMapping categoriesFor: #initialize!initializing!private! !
!RelationshipMapping categoriesFor: #initializeJoin!initializing!private! !
!RelationshipMapping categoriesFor: #initializeJoinTo:!initializing!private! !
!RelationshipMapping categoriesFor: #isExclusive!api!private! !
!RelationshipMapping categoriesFor: #isExclusive:!api!private! !
!RelationshipMapping categoriesFor: #isRelationship!private!testing! !
!RelationshipMapping categoriesFor: #isStoredInSameTable!private!testing! !
!RelationshipMapping categoriesFor: #isToManyRelationship!private!testing! !
!RelationshipMapping categoriesFor: #isValidTarget:!mapping!private! !
!RelationshipMapping categoriesFor: #join!accessing!private! !
!RelationshipMapping categoriesFor: #join:!accessing!private! !
!RelationshipMapping categoriesFor: #joinExpressionFor:!preparing!private! !
!RelationshipMapping categoriesFor: #joinFromLinkToReferenceTable!private!private/expressions! !
!RelationshipMapping categoriesFor: #knitFromBuilder:toBuilder:!mapping!private! !
!RelationshipMapping categoriesFor: #linkField:!api!private! !
!RelationshipMapping categoriesFor: #linkFields!api!private! !
!RelationshipMapping categoriesFor: #linkFields:!api!private! !
!RelationshipMapping categoriesFor: #linkTableJoin!accessing!private! !
!RelationshipMapping categoriesFor: #linkTableJoin:!accessing!private! !
!RelationshipMapping categoriesFor: #mapFromObject:intoRowsIn:!mapping!private! !
!RelationshipMapping categoriesFor: #mapFromObject:toTarget:puttingRowsIn:!mapping!private! !
!RelationshipMapping categoriesFor: #mapObject:inElementBuilder:!mapping!private! !
!RelationshipMapping categoriesFor: #mapOneFromObject:toTarget:intermediate:atPosition:puttingRowsIn:!mapping!private! !
!RelationshipMapping categoriesFor: #mappedFields!private!testing! !
!RelationshipMapping categoriesFor: #mappingCriteria!accessing!private! !
!RelationshipMapping categoriesFor: #mappingCriteria:!accessing!private! !
!RelationshipMapping categoriesFor: #multipleTableExpressionsFor:!preparing!private! !
!RelationshipMapping categoriesFor: #needsSpecialRowMapKey!private!testing! !
!RelationshipMapping categoriesFor: #oneWayConstraintsFrom:to:!initializing!private! !
!RelationshipMapping categoriesFor: #parametersIn:!mapping!private! !
!RelationshipMapping categoriesFor: #proxyAction!accessing!private! !
!RelationshipMapping categoriesFor: #proxyAction:!accessing!private! !
!RelationshipMapping categoriesFor: #query!mapping!private! !
!RelationshipMapping categoriesFor: #queryFor:!private!proxies! !
!RelationshipMapping categoriesFor: #referenceClass:!accessing!private! !
!RelationshipMapping categoriesFor: #referenceDescriptor!accessing!private! !
!RelationshipMapping categoriesFor: #referencedIndependentObjectsFrom:!api!private! !
!RelationshipMapping categoriesFor: #referenceTableFieldFor:!mapping!private! !
!RelationshipMapping categoriesFor: #relatedTables!initializing!private! !
!RelationshipMapping categoriesFor: #relevantLinkTableFields!api!private! !
!RelationshipMapping categoriesFor: #relevantLinkTableFields:!api!private! !
!RelationshipMapping categoriesFor: #reverseJoin!mapping!private! !
!RelationshipMapping categoriesFor: #reverseJoin:!accessing!private! !
!RelationshipMapping categoriesFor: #reverseMapping!mapping!private! !
!RelationshipMapping categoriesFor: #rowMapKeyForSource:target:index:!mapping!private! !
!RelationshipMapping categoriesFor: #setGeneralQueryRetrievalIn:basedOn:phantom:!mapping!private! !
!RelationshipMapping categoriesFor: #shouldProxy!accessing!private! !
!RelationshipMapping categoriesFor: #shouldProxy:!accessing!private! !
!RelationshipMapping categoriesFor: #shouldUseFilteredRead!api!private! !
!RelationshipMapping categoriesFor: #shouldUseFilteredRead:!api!private! !
!RelationshipMapping categoriesFor: #singleJoinFromLinkToReferenceTableForConstraints:!private!private/expressions! !
!RelationshipMapping categoriesFor: #targetTableJoin!accessing!private! !
!RelationshipMapping categoriesFor: #targetTableJoin:!accessing!private! !
!RelationshipMapping categoriesFor: #targetTables!private!validation! !
!RelationshipMapping categoriesFor: #trace:context:!private!processing! !
!RelationshipMapping categoriesFor: #useFilteredRead!api!private! !
!RelationshipMapping categoriesFor: #useLinkTable!accessing!private! !
!RelationshipMapping categoriesFor: #usesLinkTable!accessing!private! !
!RelationshipMapping categoriesFor: #usesLinkTable:!accessing!private! !
!RelationshipMapping categoriesFor: #validate!public!validation! !
!RelationshipMapping categoriesFor: #valueOfField:fromObject:!mapping!private! !
!RelationshipMapping categoriesFor: #valueOfField:fromObject:ifNone:!mapping!private! !
!RelationshipMapping categoriesFor: #writeExtraInformationInto:atKey:forSourceObject:targetObject:atIndex:!mapping!private! !

TypeMapping guid: (GUID fromString: '{86037E62-72C4-44A1-AA23-56E97C03855E}')!
TypeMapping comment: 'TypeMappings are used to implement inheritance. They do not map to particular instance variables, but instead define things about the way the class is read. The default type mapping is IdentityTypeMapping, which does nothing. To use inheritance, substitute either a FilteredTypeMapping or a HorizontalTypeMapping. TypeMappings become part of a TypeResolver, which keeps the information for a class and its subclasses, each of which will have a typeMapping specific to that class.

Subclasses must implement the following messages:
	testing
		isAbstract

'!
!TypeMapping categoriesForClass!Unclassified! !
!TypeMapping methodsFor!

addTypeMappingCriteriaTo: collection in: expression
	^self!

allDescribedConcreteClasses

	^Array with: (self describedClass)!

canBeTypeMappingParent
	^true!

describedClass
	^self descriptor describedClass!

describedConcreteClassFor: aRow withBuilder: builder

	^self mappedClass!

descriptorForSuperclass
	^self system descriptorFor: self mappedClass superclass!

hasAttribute
	"Return true if this maps an attribute, which is normally true"
	^false.!

isAbstract
	self subclassResponsibility!

isRelationship

	^false.!

isTypeMapping
	^true!

isTypeMappingRoot
	| superClassDescriptor |
	superClassDescriptor := self descriptorForSuperclass.
	^superClassDescriptor isNil or: [ superClassDescriptor typeMapping canBeTypeMappingParent not ]!

mapFromObject: anObject intoRowsIn: aRowMap
	"do  nothing"!

mapObject: anObject inElementBuilder: anElementBuilder
	"do  nothing"!

mappedClass
	^self descriptor describedClass!

referencedIndependentObjectsFrom: anObject
	^#().!

typeMappingRoot
	^self isTypeMappingRoot 
		ifTrue: [self mappedClass]
		ifFalse: [self descriptorForSuperclass typeMapping typeMappingRoot]!

validateAttribute
	" TypeMapping doesn't use the attribute, validation would therefore fail. Probably should introduce AttributeMapping as sibling to TypeMapping "! !
!TypeMapping categoriesFor: #addTypeMappingCriteriaTo:in:!mapping!private! !
!TypeMapping categoriesFor: #allDescribedConcreteClasses!mapping!private! !
!TypeMapping categoriesFor: #canBeTypeMappingParent!private!testing! !
!TypeMapping categoriesFor: #describedClass!mapping!private! !
!TypeMapping categoriesFor: #describedConcreteClassFor:withBuilder:!mapping!private! !
!TypeMapping categoriesFor: #descriptorForSuperclass!mapping!private! !
!TypeMapping categoriesFor: #hasAttribute!private!testing! !
!TypeMapping categoriesFor: #isAbstract!private!testing! !
!TypeMapping categoriesFor: #isRelationship!private!testing! !
!TypeMapping categoriesFor: #isTypeMapping!private!testing! !
!TypeMapping categoriesFor: #isTypeMappingRoot!private!testing! !
!TypeMapping categoriesFor: #mapFromObject:intoRowsIn:!mapping!private! !
!TypeMapping categoriesFor: #mapObject:inElementBuilder:!mapping!private! !
!TypeMapping categoriesFor: #mappedClass!accessing!private! !
!TypeMapping categoriesFor: #referencedIndependentObjectsFrom:!mapping!private! !
!TypeMapping categoriesFor: #typeMappingRoot!mapping!private! !
!TypeMapping categoriesFor: #validateAttribute!private!validation! !

OneToOneMapping guid: (GUID fromString: '{A51098B0-1EF0-4DE0-8986-0F7CD4DD6EEE}')!
OneToOneMapping comment: 'This represents a simple one to one relationship, e.g. from Person to Address. Assuming each person has only one address. Note that this has no methods, and only really exists for backward compatibility purposes, since a to-one relationship is the default for RelationshipMapping.'!
!OneToOneMapping categoriesForClass!Unclassified! !
ToManyMapping guid: (GUID fromString: '{BF52511E-845B-49D3-919B-4B230F245AB4}')!
ToManyMapping comment: 'This is a superclass for mappings which represent a relationship between one object and a collection of others. It isn''t an abstract superclass, you can use this class directly, and you will get the same behaviour as if you were using OneToManyMapping. Or if you set useLinkTable to true, you''ll get the same behaviour as a ManyToManyMapping.

Instance Variables:
	orderBy	<(SequenceableCollection of: (GlorpExpression | BlockClosure))>	By what criteria should we order the collection of items when we are reading.
	shouldWriteTheOrderField	<Boolean>	On write, when we write an item in the collection, should we also write the index at which it occurs in the collection into the database. This lets us preserve ordering if we just used a SequenceableCollection in memory. If this is true, we assume that the first item in orderBy will give us the field to write the index into.

'!
!ToManyMapping categoriesForClass!Unclassified! !
!ToManyMapping methodsFor!

buildGeneralQueryIn: anObjectBuilder
	| newQuery |
	newQuery := super buildGeneralQueryIn: anObjectBuilder.
	orderBy isNil ifFalse: [orderBy do: [:each | newQuery orderBy: each]].
	^newQuery.!

buildQuery
	query := super buildQuery.
	orderBy isNil ifFalse: [orderBy do: [:each | query orderBy: each]].
	^query.!

collectionType
	^self attribute collectionType.!

collectionType: aClass
	self attribute collectionType: aClass.!

defaultCollectionType
	^OrderedCollection.!

hasValueThatCanProvideField: aField from: anObject
	"Return true if we have enough information in ourselves to populate our fields. The only time this won't be true is if we're either holding an uninstantiated proxy or have a to-many relationship mapping with an empty collection. Inline the superclass implementation to avoid fetching the value twice."
	| value |
	(super hasValueThatCanProvideField: aField from: anObject) ifFalse: [^false].
	value := self getValueFrom: anObject.
	^value isEmpty not.!

initialize
	super initialize.
	shouldWriteTheOrderField := false.!

initializeAttribute
	"This is where we can initialize aspects of the attribute, if required. The motivating example is initializing the collection type of the attribute if it wasn't set. We need it to have some type, but we don't want to initialize it while we're doing basic initialization and aren't yet properly attached to a descriptor or class model"
	super initializeAttribute.
	self attribute collectionType isNil
		ifTrue: [self attribute collectionType: self defaultCollectionType].!

keyForOrderingFrom: anObject to: target
	self usesLinkTable ifFalse: [^target].
	^self orderField table == self linkTable ifTrue: [target] ifFalse: [anObject].!

linkTable
	| referenceKeys |
	self usesLinkTable ifFalse: [^nil].
	referenceKeys := (self join) targetKeys asOrderedCollection.
	^referenceKeys first table.!

orderBy
	^orderBy.!

orderBy: aBlockOrSelector
	orderBy isNil ifTrue: [orderBy := OrderedCollection new].
	orderBy add: aBlockOrSelector.!

orderField
	| exp |
	exp := Query new expressionBlockFor: orderBy first.
	^(exp asGlorpExpressionOn: (BaseExpression new descriptor: self referenceDescriptor)) field.!

shouldWriteTheOrderField
	^shouldWriteTheOrderField!

shouldWriteTheOrderField: aBoolean
	shouldWriteTheOrderField := aBoolean.!

validate

	| targetClassModel |
	super validate.
	(self collectionType = GlorpCursoredStream and: [self shouldUseFilteredRead]) ifTrue: [self error: 'Filtered mappings can cause duplicates and so cannot return cursors'].

	(shouldWriteTheOrderField and: [orderBy isNil]) ifTrue: [self error: 'Mapping tries to write the collection order to a field, but has no order by field specified'].
	
	orderBy isNil ifTrue: [^self].
	targetClassModel := self system classModelFor: attribute type.

	"Note that this will only validate when the orderBy is a symbol - not a block, expression, or field"
	orderBy do: [:attributeName |
		attributeName isSymbol ifTrue: [
			(targetClassModel hasAttributeNamed: attributeName) ifFalse: [
				self error: attribute printString, ' is attempting to order by #', attributeName, ' which is not a mapped attribute of ', targetClassModel describedClass name]]].!

writeExtraInformationInto: aRowMap atKey: rowMapKey forSourceObject: sourceObject targetObject: targetObject atIndex: index
	shouldWriteTheOrderField
		ifTrue:
			[| keyForOrdering |
			keyForOrdering := self keyForOrderingFrom: sourceObject to: rowMapKey.
			aRowMap
				at: self orderField
				key: keyForOrdering
				put: index].!

writeTheOrderField
	shouldWriteTheOrderField := true.! !
!ToManyMapping categoriesFor: #buildGeneralQueryIn:!mapping!private! !
!ToManyMapping categoriesFor: #buildQuery!mapping!private! !
!ToManyMapping categoriesFor: #collectionType!api!private! !
!ToManyMapping categoriesFor: #collectionType:!api!private! !
!ToManyMapping categoriesFor: #defaultCollectionType!initialize/release!private! !
!ToManyMapping categoriesFor: #hasValueThatCanProvideField:from:!private!testing! !
!ToManyMapping categoriesFor: #initialize!initialize/release!private! !
!ToManyMapping categoriesFor: #initializeAttribute!initialize/release!private! !
!ToManyMapping categoriesFor: #keyForOrderingFrom:to:!mapping!private! !
!ToManyMapping categoriesFor: #linkTable!private!private/expressions! !
!ToManyMapping categoriesFor: #orderBy!api!private! !
!ToManyMapping categoriesFor: #orderBy:!api!private! !
!ToManyMapping categoriesFor: #orderField!private!private/expressions! !
!ToManyMapping categoriesFor: #shouldWriteTheOrderField!api!private! !
!ToManyMapping categoriesFor: #shouldWriteTheOrderField:!api!private! !
!ToManyMapping categoriesFor: #validate!private!validation! !
!ToManyMapping categoriesFor: #writeExtraInformationInto:atKey:forSourceObject:targetObject:atIndex:!mapping!private! !
!ToManyMapping categoriesFor: #writeTheOrderField!api!private! !

EmbeddedValueOneToOneMapping guid: (GUID fromString: '{072D1A0D-2C75-434A-B20D-E83F4BCA7D82}')!
EmbeddedValueOneToOneMapping comment: 'This represents a one-to-one mapping in which the referenced object is stored as part of the same table as the containing object.'!
!EmbeddedValueOneToOneMapping categoriesForClass!Unclassified! !
!EmbeddedValueOneToOneMapping methodsFor!

controlsTables
	"Return true if this type of method 'owns' the tables it's associated with, and expression nodes using this mapping should alias those tables where necessary"

	^false!

defaultTransformationExpressionFor: aDescriptor 
	"If there's no transformation, get all the mapped fields from the other descriptor and construct a transformation of each onto itself. This lets us unify the fields in my row with the fields in its row"

	| fields transform |
	fields := IdentitySet new.
	aDescriptor mappings do: [:each | fields addAll: each mappedFields].
	transform := Join new.
	fields do: [:each | transform addSource: each target: each].
	^transform!

fieldsForSelectStatement
	"Return a collection of fields that this mapping will read from a row"
	"Return nothing, because our sub-objects will take care of adding their own fields, translated correctly through us."
	^#().!

fieldTranslation
	^fieldTranslation!

fieldTranslation: aPrimaryKeyExpression 
	fieldTranslation := aPrimaryKeyExpression!

hasFieldTranslation

	^fieldTranslation notNil.!

hasTransformation

	^false.!

isStoredInSameTable
	^true!

join
	^nil.!

join: anExpression
	self error: 'Embedded mappings do not use joins'.!

joinExpressionFor: targetExpression
	"We're looking for the object represented by this mapping, and we know the object represented by its source. Use our mapping criteria to construct a join that traverses that instance of this relationship.
	Embedded values never induce a join."

	^nil.!

knitFromBuilder: sourceBuilder toBuilder: targetBuilder 
	sourceBuilder requiresPopulating ifFalse: [^self].
	super knitFromBuilder: sourceBuilder toBuilder: targetBuilder!

mapFromObject: anObject toTarget: target puttingRowsIn: aRowMap 

	self transformationExpression 
		mapFromSource: anObject
		andTarget: target
		intoRowsIn: aRowMap.

	self transformationExpression allTables do: [:eachTable |	
		aRowMap rowForTable: eachTable withKey: target ifPresent: [  :each |
			each shouldBeWritten: false]].!

mapObject: anObject inElementBuilder: anElementBuilder
	| myTraceNode myBuilder |
	"If the object already has a value in my slot, then this it got a cache hit, the embedded value was carried along for the ride, and we don't need to assign anything"
	(self getValueFrom: anObject) isNil ifFalse: [^self].	"Otherwise, we need to look up the trace node that corresponds to this mapping, and get its instance"
	myTraceNode := anElementBuilder expression get: self attributeName.
	myBuilder := anElementBuilder query elementBuilderFor: myTraceNode.
	self setValueIn: anObject to: myBuilder instance.!

mappedFields
	"Return a collection of fields that this mapping will write into any of the containing object's rows"

	fieldTranslation isNil ifFalse: [^fieldTranslation allSourceFields].
	^self referenceDescriptor mappedFields.!

reverseMapping
	^self class new
		attributeName: #temporaryReverseMappingForEmbeddedMapping;
		referenceClass: self descriptor describedClass;
		descriptor: self descriptor.!

shouldProxy

	^false.!

transformationExpression

	^self hasFieldTranslation
		ifTrue: [fieldTranslation]
		ifFalse: [fieldTranslation := self defaultTransformationExpressionFor: (self referenceDescriptor)].!

translateFields: anOrderedCollection 
	fieldTranslation isNil ifTrue: [^anOrderedCollection].
	^anOrderedCollection collect: [:each |
		fieldTranslation sourceForTarget: each].!

translateTable: aTable
	"If we have a field translation, return the translated (i.e. real) table to use. This ideally allows us to use either the real table in which something is embedded, or the imaginary table"

	fieldTranslation isNil ifTrue: [^aTable].
	1 to: fieldTranslation allTargetFields size
		do:
			[:i | 
			(fieldTranslation allTargetFields at: i) table == aTable
				ifTrue: [^(fieldTranslation allSourceFields at: i) table]].
	^aTable.!

valueOfField: aField fromObject: anObject ifNone: aBlock
	| targetField target |
	targetField := self transformationExpression targetForSource: aField.
	targetField isNil ifTrue: [^aBlock value].
	target := self getValueFrom: anObject.
	^self referenceDescriptor valueOfField: targetField fromObject: target.! !
!EmbeddedValueOneToOneMapping categoriesFor: #controlsTables!private!testing! !
!EmbeddedValueOneToOneMapping categoriesFor: #defaultTransformationExpressionFor:!private!transformations! !
!EmbeddedValueOneToOneMapping categoriesFor: #fieldsForSelectStatement!internal!private! !
!EmbeddedValueOneToOneMapping categoriesFor: #fieldTranslation!accessing!private! !
!EmbeddedValueOneToOneMapping categoriesFor: #fieldTranslation:!accessing!private! !
!EmbeddedValueOneToOneMapping categoriesFor: #hasFieldTranslation!accessing!private! !
!EmbeddedValueOneToOneMapping categoriesFor: #hasTransformation!private!transformations! !
!EmbeddedValueOneToOneMapping categoriesFor: #isStoredInSameTable!private!testing! !
!EmbeddedValueOneToOneMapping categoriesFor: #join!accessing!private! !
!EmbeddedValueOneToOneMapping categoriesFor: #join:!accessing!private! !
!EmbeddedValueOneToOneMapping categoriesFor: #joinExpressionFor:!preparing!private! !
!EmbeddedValueOneToOneMapping categoriesFor: #knitFromBuilder:toBuilder:!mapping!private! !
!EmbeddedValueOneToOneMapping categoriesFor: #mapFromObject:toTarget:puttingRowsIn:!mapping!private! !
!EmbeddedValueOneToOneMapping categoriesFor: #mapObject:inElementBuilder:!mapping!private! !
!EmbeddedValueOneToOneMapping categoriesFor: #mappedFields!internal!private! !
!EmbeddedValueOneToOneMapping categoriesFor: #reverseMapping!mapping!public! !
!EmbeddedValueOneToOneMapping categoriesFor: #shouldProxy!private!testing! !
!EmbeddedValueOneToOneMapping categoriesFor: #transformationExpression!private!transformations! !
!EmbeddedValueOneToOneMapping categoriesFor: #translateFields:!mapping!private! !
!EmbeddedValueOneToOneMapping categoriesFor: #translateTable:!mapping!private! !
!EmbeddedValueOneToOneMapping categoriesFor: #valueOfField:fromObject:ifNone:!mapping!private! !

ConditionalToManyMapping guid: (GUID fromString: '{6DB3ED70-A9C6-4D4E-8B41-05BC78AB83FC}')!
ConditionalToManyMapping comment: 'This is a conditional mapping, but one where the condition is applied to a relationship with a collection, and applies to individual elements of the collection. So, rather than making a single choice overall of which mapping to apply, we loop over the collection and make the choice separately for each element.

This is experimental, and probably suffers from lots of limitations.'!
!ConditionalToManyMapping categoriesForClass!Unclassified! !
!ConditionalToManyMapping methodsFor!

applicableMappingForObject: anObject
	"We can actually be called here with our parent object, but the contexts in which that happens we basically just want to do nothing, so return our conditional mapping."
	[self attribute checkTargetType: anObject] on: GlorpInvalidTypeError do: [:ex | ^self conditional].
	^self conditional applicableMappingForObject: anObject.!

applicableMappingForRow: anArray in: anElementBuilder
	^self conditional applicableMappingForRow: anArray in: anElementBuilder.!

buildQuery
	query := CompositeQuery new.
	self conditional casesDo: [:each |
		query addQuery: each query].
	^query!

conditional
	conditional isNil ifTrue: [conditional := self newMapping: ConditionalMapping].
	^conditional.!

conditional: anObject
	conditional := anObject!

conditionalField: aField
	self conditional conditionalField: aField.!

conditionalMethod: aSymbol
	self conditional conditionalMethod: aSymbol.!

forField: aField attribute: attributeBlock if: testBlock useMapping: aMapping
	self conditionalField: aField.
	self conditionalMethod: attributeBlock.
	self if: testBlock then: aMapping.!

if: conditionSelector then: aMapping
	self conditional if: conditionSelector then: aMapping.!

mapOneFromObject: sourceObject toTarget: targetObject intermediate: aRowMapKey atPosition: index puttingRowsIn: aRowMap
	
	(self applicableMappingForObject: targetObject) mapOneFromObject:
			sourceObject toTarget: targetObject intermediate: aRowMapKey atPosition:
			index puttingRowsIn: aRowMap.!

mappedFields
	^self conditional mappedFields!

newMapping: aMappingClass
	"Implement this so that we can create our sub-mappings without having them added to the parent descriptor as independent mappings"
	| mapping |
	mapping := aMappingClass new.
	mapping descriptor: self descriptor.
	descriptor resetMappedFields.
	^mapping.!

otherwise: aMapping
	self conditional otherwise: aMapping.!

parametersIn: anElementBuilder
	| parameters |
	self shouldUseFilteredRead ifTrue: [^anElementBuilder command parameters].
	parameters := IdentityDictionary new.
	self conditional casesDo: [:each |
		each join mappableSourceFieldsDo:
			[:eachSource | parameters at: eachSource put: (anElementBuilder valueOfField: eachSource)]].
	^parameters.!

rowMapKeyForSource: source target: target index: index
	^(self applicableMappingForObject: target) rowMapKeyForSource: source target: target index: index.! !
!ConditionalToManyMapping categoriesFor: #applicableMappingForObject:!mapping!private! !
!ConditionalToManyMapping categoriesFor: #applicableMappingForRow:in:!mapping!private! !
!ConditionalToManyMapping categoriesFor: #buildQuery!mapping!private! !
!ConditionalToManyMapping categoriesFor: #conditional!accessing!private! !
!ConditionalToManyMapping categoriesFor: #conditional:!accessing!private! !
!ConditionalToManyMapping categoriesFor: #conditionalField:!conditions!private! !
!ConditionalToManyMapping categoriesFor: #conditionalMethod:!conditions!private! !
!ConditionalToManyMapping categoriesFor: #forField:attribute:if:useMapping:!conditions!private! !
!ConditionalToManyMapping categoriesFor: #if:then:!conditions!private! !
!ConditionalToManyMapping categoriesFor: #mapOneFromObject:toTarget:intermediate:atPosition:puttingRowsIn:!mapping!private! !
!ConditionalToManyMapping categoriesFor: #mappedFields!accessing!private! !
!ConditionalToManyMapping categoriesFor: #newMapping:!accessing!private! !
!ConditionalToManyMapping categoriesFor: #otherwise:!conditions!private! !
!ConditionalToManyMapping categoriesFor: #parametersIn:!mapping!private! !
!ConditionalToManyMapping categoriesFor: #rowMapKeyForSource:target:index:!mapping!private! !

DictionaryMapping guid: (GUID fromString: '{7C9E72C5-32A3-4916-A952-9CC9BCCB348A}')!
DictionaryMapping comment: 'This allows us to map a dictionary into tables. It should, I hope, support arbitrarily either mapping to single fields, or to objects, for either the key or the value. At the moment, it at least passes the tests for the value->object and value->value cases.

Instance Variables:
	keyMapping	<DatabaseMapping>	The mapping to use to read/write the key. It should be for an attribute named #key.
	valueMapping	<DatabaseMapping>	The mapping to use to read/write the value. It should be for an attribute named #value. Note that this is optional, and isn''t a full blown mapping, because if we have a relationship, we will normally just use this mapping to describe it directly. If we have a direct mapping for the value, then we have this odd situation where we use the Join from this mapping, but the direct mapping to control the read/write of the variable.
	associationDescriptor	<Descriptor>	We manipulate the key/value pairs as associations in some places, and have a mock descriptor for association, specific to this mapping, which is used to get the key/value.


'!
!DictionaryMapping categoriesForClass!Unclassified! !
!DictionaryMapping methodsFor!

associationClassModel
	| model |
	model := GlorpClassModel for: Association.
	model newAttributeNamed: #key.
	model newAttributeNamed: #value.
	^model.!

associationDescriptor
	associationDescriptor isNil ifTrue: [
		associationDescriptor := Descriptor new.
		associationDescriptor system: self system.
		associationDescriptor classModel: self associationClassModel].
	^associationDescriptor.!

basicReferenceDescriptor
	^self system descriptorFor: self basicReferenceClass.!

buildQuery
	| keyExpression valueExpression |
	query := super buildQuery.
	keyExpression := self keyExpression.
	valueExpression := self valueExpression.
	query retrieveDictionaryWithKey: keyExpression value: valueExpression.
	^query.!

buildRowMapKeyForSource: source target: target index: index
	^self mapsSimpleObject 
		ifTrue: [
			MultipleRowMapKey
				with: target yourSelf
				with: source yourSelf
				with: index]
		ifFalse: [super buildRowMapKeyForSource: source target: target index: index].!

chooseBaseForMapping: aMapping from: base and: phantomExpression
	aMapping isNil ifTrue: [^base].
	^(aMapping mappedFields
		allSatisfy: [:each | base tables includes: each table])
		ifTrue: [base]
		ifFalse: [phantomExpression].!

defaultCollectionType
	^Dictionary.!

field
	^self valueField.!

glorpCanHaveDescriptorDirectly
	"We allow instances of ourselves to have a descriptor, which normally only classes can do. This is so that we can have multiple different descriptors in use for e.g. String, depending on the particular usage."
"	^self mapsSimpleObject."
	^true.!

initialize
	super initialize.
	usesLinkTable := true.!

initializeValueMapping
	| des |
	self descriptor system privateAndBasicAddDescriptorFor: self as: self associationDescriptor.
	(self associationDescriptor classModel attributeNamed: #key) type: self attribute keyType.
	(self associationDescriptor classModel attributeNamed: #value) type: self attribute type.
	des := self referenceDescriptor.
	(des isNil or: [des tables isEmpty]) ifTrue: [self initializeValueMappingJoin].!

initializeValueMappingJoin
	| tables |
	"We don't know which of these will come first in the descriptor, so call the other initialization if it looks like we have enough information, and it hasn't been done"
	self referenceDescriptor isNil ifTrue: [self initializeValueMapping].
	"Have we already done this?"
	self referenceDescriptor tables isEmpty ifFalse: [^self].

	tables := self targetTables asArray.
	tables do: [:each | self referenceDescriptor addTable: each].
	(self referenceDescriptor newMapping: DirectMapping) 
		from: #key to: self keyField.
	self mapsSimpleObject 
		ifTrue: [
			(self referenceDescriptor newMapping: DirectMapping) 
			from: #value to: self valueField]
		ifFalse: [
			(self referenceDescriptor newMapping: OneToOneMapping)
				attributeName: #value;	
				linkFields: self linkFields;
				usesLinkTable: self usesLinkTable;
				join: self join;
				controlsTables: false].!

keyExpression
	"Return the expression to use for our key. The key can be built using data from either the target object, or from the link table, so build the expression on one or the other, as appropriate"
	 | base intermediate |
	base := BaseExpression new.
	intermediate := (self descriptor tables includes: keyMapping mappedFields first table)
		ifTrue: [base] 
		ifFalse: [base getTable: keyMapping mappedFields first table].
	^intermediate getMapping: keyMapping named: #key.!

keyField
	^keyMapping field.!

keyField: aDatabaseField
	keyMapping := self newMapping: DirectMapping.
	keyMapping attributeName: #key.
	keyMapping from: #key to: aDatabaseField.!

keyMapping
	^keyMapping!

keyMapping: aMapping
	keyMapping := aMapping!

mappedFields
	"Return a collection of fields that this mapping will write into any of the containing object's rows"

	| fields |
	fields := super mappedFields copy.
	fields addAll: keyMapping mappedFields.
	valueMapping isNil ifFalse: [
		fields addAll: valueMapping mappedFields].
	^fields.!

mapsSimpleObject
	^valueMapping notNil and: [valueMapping mapsSimpleObject].!

name
	"This can get called when we're masquering as a class, for purposes of indexing descriptors, so just print something useful"
	^self printString.!

needsSpecialRowMapKey
	^super needsSpecialRowMapKey or: [
		self mapsSimpleObject].!

newMapping: aMappingClass
	"Implement this so that we can create our sub-mappings without having them added to the parent descriptor as independent mappings"
	| mapping |
	mapping := aMappingClass new.
	mapping descriptor: self associationDescriptor.
	descriptor resetMappedFields.
	^mapping.!

pseudoDescriptor
	"Return a pretend descriptor for our reference class. Since that class is a primitive, it won't have a real descriptor, and it may vary by usage anyway"
	| pseudoDescriptor |
	pseudoDescriptor := Descriptor new.
	pseudoDescriptor system: descriptor system.
	^pseudoDescriptor.!

referenceClass
	"Return ourselves as the reference class, as a sort of a proxy for what we really want, which is a primitive, and varies according to the relationship anyway. Note that we have stuffed in a descriptor in the system, with ourselves (this instance) as the class."
"	^self mapsSimpleObject ifTrue: [self] ifFalse: [super referenceClass]."
	^self.!

referenceDescriptor
	| ref |
	ref := super referenceDescriptor.
	^ref isNil 
		ifTrue: [
			"This means we haven't yet added our fake descriptor, and are in initialization, so just return the descriptor for the 'real' reference class"
			self system descriptorFor: self basicReferenceClass]
		ifFalse: [ref].!

setGeneralQueryRetrievalIn: newQuery basedOn: base phantom: phantomExpression
	| keyExpression valueExpression collType keyExpressionBase valueExpressionBase |
	self descriptor primaryKeyMappings
		do:
			[:eachMapping | newQuery retrieve: (phantomExpression get: eachMapping attributeName)].
	keyExpressionBase := self
		chooseBaseForMapping: keyMapping
		from: base
		and: phantomExpression.
	keyExpression := self keyExpression asGlorpExpressionOn: keyExpressionBase.
	valueExpressionBase := self
		chooseBaseForMapping: valueMapping
		from: base
		and: phantomExpression.
	valueExpression := self valueExpression
		asGlorpExpressionOn: valueExpressionBase.
	collType := (GlorpDictionaryDescription new)
		keyExpression: keyExpression;
		valueExpression: valueExpression.
	newQuery collectionType: collType.
	newQuery retrieve: base.!

validate
	self initializeValueMapping.
	super validate.
	self attribute isDictionaryAttribute
		ifFalse: [self error: 'Dictionary Mapping that does not have a dictionary type in the attribute'].

	"The reason you can't do this, is that we end up throwing away the primary key of the target table when we read in the objects, and we have no way to properly do deletes/updates without that information"
	(self usesLinkTable and: [self mapsSimpleObject]) ifTrue: [self error: 'Dictionary mappings to simple objects cannot use a link table'].!

valueExpression
	 | base intermediate |
	valueMapping isNil ifTrue: [ ^[:each | each value] ].
	base := BaseExpression new.
	intermediate := (self descriptor tables includes: valueMapping mappedFields first table)
		ifTrue: [base] 
		ifFalse: [base getTable: valueMapping mappedFields first table].
	^intermediate getMapping: valueMapping named: #value.!

valueField
	^valueMapping field.!

valueField: aDatabaseField
	| mapping |
	mapping := self newMapping: DirectMapping.
	mapping attributeName: #value.
	mapping from: #value to: aDatabaseField.
	self valueMapping: mapping.!

valueMapping
	^valueMapping.!

valueMapping: aMapping
	valueMapping := aMapping.!

writeExtraInformationInto: aRowMap atKey: rowMapKey forSourceObject: sourceObject targetObject: targetObject atIndex: eachKey
	"Write out any additional information that goes in the link table or else the target table. That means our key, if necessary. If we have a specific value mapping, rather than just using the normal attributes of this mapping, then write that as well. That normally happens when our value is simple"
	
	"We need to know if the key mapping is trying to write to the link table, or to the target. If it's to the link table, then we always use the row map key. If it's to the target table, then we need to use whatever is appropriate for the target table, which is a row map key if we have a simple target, and otherwise it's just the target object"
	| target |
	target := (keyMapping mappedTables includes: self linkTable)
		ifTrue: [rowMapKey]
		ifFalse: [self mapsSimpleObject ifTrue: [rowMapKey] ifFalse: [targetObject]].
	keyMapping
		mapFromObject: eachKey -> nil
		toTarget: target
		puttingRowsIn: aRowMap.

	valueMapping isNil ifTrue: [^self].
	"I'm not sure this will ever be relevant. If the value mapping goes to the link table, then does that mean the key is really the value? And if so, why not just consider the other one to be the link?"
	target := (valueMapping mappedTables includes: self linkTable)
		ifTrue: [rowMapKey]
		ifFalse: [self mapsSimpleObject ifTrue: [rowMapKey] ifFalse: [targetObject]].
	valueMapping
		mapFromObject: nil -> targetObject
		toTarget: target
		puttingRowsIn: aRowMap.! !
!DictionaryMapping categoriesFor: #associationClassModel!mapping!private! !
!DictionaryMapping categoriesFor: #associationDescriptor!mapping!private! !
!DictionaryMapping categoriesFor: #basicReferenceDescriptor!accessing!private! !
!DictionaryMapping categoriesFor: #buildQuery!mapping!private! !
!DictionaryMapping categoriesFor: #buildRowMapKeyForSource:target:index:!mapping!private! !
!DictionaryMapping categoriesFor: #chooseBaseForMapping:from:and:!mapping!private! !
!DictionaryMapping categoriesFor: #defaultCollectionType!initializing!private! !
!DictionaryMapping categoriesFor: #field!accessing!private! !
!DictionaryMapping categoriesFor: #glorpCanHaveDescriptorDirectly!private!testing! !
!DictionaryMapping categoriesFor: #initialize!initializing!private! !
!DictionaryMapping categoriesFor: #initializeValueMapping!initializing!private! !
!DictionaryMapping categoriesFor: #initializeValueMappingJoin!initializing!private! !
!DictionaryMapping categoriesFor: #keyExpression!mapping!private! !
!DictionaryMapping categoriesFor: #keyField!accessing!private! !
!DictionaryMapping categoriesFor: #keyField:!accessing!private! !
!DictionaryMapping categoriesFor: #keyMapping!accessing!private! !
!DictionaryMapping categoriesFor: #keyMapping:!accessing!private! !
!DictionaryMapping categoriesFor: #mappedFields!accessing!private! !
!DictionaryMapping categoriesFor: #mapsSimpleObject!private!testing! !
!DictionaryMapping categoriesFor: #name!accessing!private! !
!DictionaryMapping categoriesFor: #needsSpecialRowMapKey!private!testing! !
!DictionaryMapping categoriesFor: #newMapping:!mapping!private! !
!DictionaryMapping categoriesFor: #pseudoDescriptor!mapping!private! !
!DictionaryMapping categoriesFor: #referenceClass!accessing!private! !
!DictionaryMapping categoriesFor: #referenceDescriptor!accessing!private! !
!DictionaryMapping categoriesFor: #setGeneralQueryRetrievalIn:basedOn:phantom:!mapping!private! !
!DictionaryMapping categoriesFor: #validate!private!validation! !
!DictionaryMapping categoriesFor: #valueExpression!mapping!private! !
!DictionaryMapping categoriesFor: #valueField!accessing!private! !
!DictionaryMapping categoriesFor: #valueField:!accessing!private! !
!DictionaryMapping categoriesFor: #valueMapping!accessing!private! !
!DictionaryMapping categoriesFor: #valueMapping:!accessing!private! !
!DictionaryMapping categoriesFor: #writeExtraInformationInto:atKey:forSourceObject:targetObject:atIndex:!mapping!private! !

DirectToManyMapping guid: (GUID fromString: '{7512FC36-884D-4AFF-9154-29A5CCF5DC3F}')!
DirectToManyMapping comment: 'This is a to-many mapping where the class in the collection is a simple type, not a compound object. This requires fiddling with the descriptors, to allow each such mapping to define a different descriptor for the target class. This should be regarded as something of a prototype. It seems to work, at least for basic operations, but it may well have problems.

Instance Variables:
	resultField	<DatabaseField>	The field in the target table in which we have our results.

'!
!DirectToManyMapping categoriesForClass!Unclassified! !
!DirectToManyMapping methodsFor!

attributeName: anAttributeName
	^self attributeName: anAttributeName referenceClass: nil.!

attributeName: anAttributeName referenceClass: aClass
	"In addition to setting the attribute, use that information to further configure our faked-up descriptor. We can pass in a reference class here, although this is only really useful if we don't already have an explicit classModel."
	| model |
	super attributeName: anAttributeName.
	aClass isNil ifFalse: [attribute type: aClass].
	model := GlorpClassModel for: super referenceClass.
	"We add a mapping for our value. We don't normally use this, because we expect people to refer directly to the attribute, but it means, for example, that the field shows up in our mapped fields without us having to fake it up. And you could write a query referring to anAttribute myValue if you wanted"
	model newAttributeNamed: #myValue type: self attribute type.
	self referenceDescriptor classModel: model.!

buildQuery
	query := super buildQuery.
	query retrieve: [:x | x getField: resultField].
	^query.!

buildRowMapKeyForSource: source target: target index: index
	"What should this do??? We want to distinguish the entries in e.g. # (1 2 3 3 4 3 5). Or do we? They aren't distinguishable in the DB. Does that mean we always want to write the order field? Or what? What if this is a set in memory. Then we really can't have two entries"
	^MultipleRowMapKey
		with: target yourSelf
		with: source yourSelf
		with: index.!

convertedDbValueOf: anObject
	"It may be that we need to introduce a converter here, the same as with a regular direct mapping"
	^anObject.!

descriptor: aDescriptor
	super descriptor: aDescriptor.
	aDescriptor system privateAndBasicAddDescriptorFor: self as: self pseudoDescriptor.!

field
	^resultField.!

glorpCanHaveDescriptorDirectly
	"We allow instances of ourselves to have a descriptor, which normally only classes can do. This is so that we can have multiple different descriptors in use for e.g. String, depending on the particular usage."
	^true.!

join: aJoin
	"In addition to setting up the join, use that information to configure our faked up descriptor"
	| tables |
	super join: aJoin.
	tables := self targetTables asArray.
	tables size > 1
		ifTrue: [self error: 'Cannot have multiple tables for a direct toMany'].
	self referenceDescriptor table: tables first.
	"We add a mapping for our value. We don't normally use this, because we expect people to refer directly to the attribute, but it means, for example, that the field shows up in our mapped fields without us having to fake it up. And you could write a query referring to anAttribute myValue if you wanted"
	(self referenceDescriptor newMapping: DirectMapping) 
		from: #myValue to: resultField.!

mapsSimpleObject
	^true.!

needsSpecialRowMapKey
	"Return true if we have to use a special key object for the row map, rather than just our target. We always do, because our target may not have identity."

	^true.!

pseudoDescriptor
	"Return a pretend descriptor for our reference class. Since that class is a primitive, it won't have a real descriptor, and it may vary by usage anyway"
	| pseudoDescriptor |
	pseudoDescriptor := Descriptor new.
	pseudoDescriptor system: descriptor system.
	^pseudoDescriptor.!

referenceClass
	"Return ourselves as the reference class, as a sort of a proxy for what we really want, which is a primitive, and varies according to the relationship anyway. Note that we have stuffed in a descriptor in the system, with ourselves (this instance) as the class."
	^self.!

resultField
	^resultField.!

resultField: aField
	resultField := aField.!

validate
	super validate.
	"We have to write the order field. Otherwise, we get confused e.g. on deletion, because we don't have a way of telling the rows in the rowmap apart when the indexes have changed"
	(self readOnly not and: [shouldWriteTheOrderField not]) ifTrue: [self error: 'To-many mappings to simple types must write an order field so that we have some way to tell the rows apart'].

	"If we're for a pseudo-variable, we're neither reading nor writing, so we don''t need the order."
	self isForPseudoVariable ifTrue: [^self].
	orderBy isNil ifTrue: [self error: 'To-many mappings to simple types must specify and write an order field so that we have some way to tell the rows apart'].!

writeExtraInformationInto: aRowMap atKey: rowMapKey forSourceObject: sourceObject targetObject: targetObject atIndex: index
	super
		writeExtraInformationInto: aRowMap
		atKey: rowMapKey
		forSourceObject: sourceObject
		targetObject: targetObject
		atIndex: index.
	aRowMap
		at: self resultField
		key: rowMapKey
		put: targetObject.! !
!DirectToManyMapping categoriesFor: #attributeName:!accessing!private! !
!DirectToManyMapping categoriesFor: #attributeName:referenceClass:!accessing!private! !
!DirectToManyMapping categoriesFor: #buildQuery!accessing!private! !
!DirectToManyMapping categoriesFor: #buildRowMapKeyForSource:target:index:!mapping!private! !
!DirectToManyMapping categoriesFor: #convertedDbValueOf:!converting!private! !
!DirectToManyMapping categoriesFor: #descriptor:!accessing!private! !
!DirectToManyMapping categoriesFor: #field!accessing!private! !
!DirectToManyMapping categoriesFor: #glorpCanHaveDescriptorDirectly!private!testing! !
!DirectToManyMapping categoriesFor: #join:!accessing!private! !
!DirectToManyMapping categoriesFor: #mapsSimpleObject!private!testing! !
!DirectToManyMapping categoriesFor: #needsSpecialRowMapKey!private!testing! !
!DirectToManyMapping categoriesFor: #pseudoDescriptor!mapping!private! !
!DirectToManyMapping categoriesFor: #referenceClass!accessing!private! !
!DirectToManyMapping categoriesFor: #resultField!accessing!private! !
!DirectToManyMapping categoriesFor: #resultField:!accessing!private! !
!DirectToManyMapping categoriesFor: #validate!private!validation! !
!DirectToManyMapping categoriesFor: #writeExtraInformationInto:atKey:forSourceObject:targetObject:atIndex:!accessing!private! !

ManyToManyMapping guid: (GUID fromString: '{8574AF45-3753-439B-AF41-D33B65AB4659}')!
ManyToManyMapping comment: 'This is a to-many relationship in which we use a link table to the collection, rather than a foreign key in the items of the collection pointing back to the "owner". Using this class is exactly equivalent to just using a ToManyMapping and setting the useLinkTable field to true.'!
!ManyToManyMapping categoriesForClass!Unclassified! !
!ManyToManyMapping methodsFor!

initialize
	super initialize.
	usesLinkTable := true.! !
!ManyToManyMapping categoriesFor: #initialize!initializing!private! !

OneToManyMapping guid: (GUID fromString: '{3763232F-3D67-4080-BDDF-D3DC4F14C2D1}')!
OneToManyMapping comment: 'This represents a mapping from one object to a collection of others. This class is really only here for backward compatibility. Using ToManyMapping directly will give exactly the same behaviour.

'!
!OneToManyMapping categoriesForClass!Unclassified! !
SQLite3PragmaMapping guid: (GUID fromString: '{883D37D8-775F-4BEB-AE84-F7516A16124A}')!
SQLite3PragmaMapping comment: ''!
!SQLite3PragmaMapping categoriesForClass!Unclassified! !
!SQLite3PragmaMapping methodsFor!

buildQuery
	| toMany |
	toMany := self attribute isCollectionAttribute.
	query := toMany
		ifTrue: [SQLite3PragmaQuery read: self referenceClass where: self join]
		ifFalse: [SQLite3PragmaQuery readOneOf: self referenceClass where: self join].
	query session: self session.
	query baseExpression descriptor: (self session descriptorFor: self referenceClass).
	query collectionType: self collectionType.
	self usesLinkTable ifTrue: [
		query AND: (self expressionFromLinkToReferenceTableWithBase: query baseExpression)].
	orderBy isNil ifFalse: [orderBy do: [:each | query orderBy: each]].
	^query.!

initialize
	"Initialize a newly created instance. This method must answer the receiver."

	super initialize.
	" *** Replace this comment with the appropriate initialization code *** "
	^self!

setValueIn: anObject to: aValue
	"This method serves as a hook to do special work for the collection objects, #fields and #foreignKeysData.
	It connects the objects to their owning tables (a SQLite3DatabaseTable object), and also resets the field
	type for a primary key field whose owning table has 'AUTOINCREMENT' declared for its pkey field. Such
	a field has type #serial. It would be nicer if that info were available elsewhere, but right now, this is where
	the table and field first meet (and where that info can now be shared)."

	self canRead ifFalse: [^self].
	self attribute setValueIn: anObject to: aValue.
	self attribute name = #fields
		ifTrue: 
			[aValue do: 
					[:fld |
					fld table: anObject.
					fld isPrimaryKey
						ifTrue: [anObject hasAutoIncrement ifTrue: [fld type: self descriptor system platform serial]]]].
	self attribute name = #foreignKeysData
		ifTrue: [aValue do: [:fld | fld sourceTableName: anObject name]]! !
!SQLite3PragmaMapping categoriesFor: #buildQuery!mapping!public! !
!SQLite3PragmaMapping categoriesFor: #initialize!initialize-release!public! !
!SQLite3PragmaMapping categoriesFor: #setValueIn:to:!public! !

BasicDictionaryMapping guid: (GUID fromString: '{2A658804-E9F0-4BAF-88F1-8E35A3197564}')!
BasicDictionaryMapping comment: 'This class is obsolete, but exists only so that people using the old name will not have to change their code.'!
!BasicDictionaryMapping categoriesForClass!Unclassified! !
FilteredTypeMapping guid: (GUID fromString: '{43CA977A-7FC3-47D7-85AE-AE4DDCC5E7DC}')!
FilteredTypeMapping comment: 'FilteredTypeMapping assumes that all objects are stored in a single table, and there''s a field in that table that identifies which subclass it is.

Instance Variables:
	field	<DatabaseField>	The field which identifies the subclass.
	key	<Object>	The value we expect to find in that field for this particular subclass
	keyDictionary	<Dictionary>	A dictionary for us and all of our subclasses from key values to subclass.

'!
!FilteredTypeMapping categoriesForClass!Unclassified! !
!FilteredTypeMapping methodsFor!

addTypeMappingCriteriaTo: collection in: base
	| singleRightValue r l |
	singleRightValue := self keys size = 1.
	r := ConstantExpression for: (singleRightValue
											ifTrue: [self keys asArray first]
											ifFalse: [self keys]).
	l := FieldExpression forField: self field basedOn: base.
	collection add: (singleRightValue ifTrue: [l equals: r] ifFalse: [l in: r]).!

buildKeyDictionary
	| subclassDescriptor |
	keyDictionary := Dictionary new.
	key isNil ifFalse: [keyDictionary at: key put: descriptor describedClass].
	descriptor describedClass glorpAllSubclasses do: 
			[:each | 
			subclassDescriptor := descriptor system descriptorFor: each.
			(subclassDescriptor isNil 
				or: [subclassDescriptor typeMapping keyedBy isNil]) 
					ifFalse: 
						[keyDictionary at: subclassDescriptor typeMapping keyedBy
							put: subclassDescriptor describedClass]]!

describedConcreteClassFor: aRow withBuilder: builder

	^self keyDictionary at: (aRow atIndex: (builder translateFieldPosition: field))!

field
	"If we don't have this, look it up from one of our sub-mappings"
	| concrete |
	field isNil ifTrue: [
		concrete := self descriptor typeResolver concreteMembers detect: [:each | each typeMapping field notNil].
		field := concrete typeMapping field].
	^field.!

field: aField keyedBy: aKey
	field := aField.
	key := aKey.!

keyDictionary
	keyDictionary isNil ifTrue: [ self buildKeyDictionary ].
	^keyDictionary!

keyedBy
	^key!

keyedBy: aKey
	key := aKey!

keys
	^self keyDictionary keys!

mapFromObject: anObject intoRowsIn: aRowMap

	| row |
	self canWrite ifFalse: [^self].
	row := aRowMap findOrAddRowForTable: self field table withKey: anObject.
	row at: field put: key.!

mappedFields
	"Return a collection of fields that this mapping will write into any of the containing object's rows"

	^Array with: self field.!

trace: aTracing context: anExpression
	^self.! !
!FilteredTypeMapping categoriesFor: #addTypeMappingCriteriaTo:in:!mapping!private! !
!FilteredTypeMapping categoriesFor: #buildKeyDictionary!initialize-release!private! !
!FilteredTypeMapping categoriesFor: #describedConcreteClassFor:withBuilder:!private!types! !
!FilteredTypeMapping categoriesFor: #field!accessing!private! !
!FilteredTypeMapping categoriesFor: #field:keyedBy:!initialize-release!private! !
!FilteredTypeMapping categoriesFor: #keyDictionary!accessing!private! !
!FilteredTypeMapping categoriesFor: #keyedBy!accessing!private! !
!FilteredTypeMapping categoriesFor: #keyedBy:!accessing!private! !
!FilteredTypeMapping categoriesFor: #keys!private!types! !
!FilteredTypeMapping categoriesFor: #mapFromObject:intoRowsIn:!mapping!private! !
!FilteredTypeMapping categoriesFor: #mappedFields!accessing!private! !
!FilteredTypeMapping categoriesFor: #trace:context:!mapping!private! !

!FilteredTypeMapping class methodsFor!

to: field keyedBy: key
	^self new
		field: field
		keyedBy: key! !
!FilteredTypeMapping class categoriesFor: #to:keyedBy:!instance creation!private! !

HorizontalTypeMapping guid: (GUID fromString: '{0FDB013B-4F2A-4634-A640-61B6D790BF1D}')!
HorizontalTypeMapping comment: 'HorizontalTypeMapping assumes that all objects are stored in different tables. Only leaf classes in the inheritance hierarchy can be mapped. The descriptors for the classes themselves have the necessary information, our only job is to be involved in building up the query. Queries can be done either by splitting up into a series of separate queries, or as one horribly complex query with union all clauses.

Instance Variables:
	isAbstract	<Boolean>	Is this an abstract superclass.
	mappedClass	<Class>	The class that we are mapping.

'!
!HorizontalTypeMapping categoriesForClass!Unclassified! !
!HorizontalTypeMapping methodsFor!

allDescribedConcreteClasses
	| col |
	col := OrderedCollection new
		add: self describedClass;
		addAll: self describedClass glorpAllSubclasses;
		yourself.
	self needsWork: 'This belongs in someone else''s responsibility'.
	^col select: [:each |
		(self system descriptorFor: each) typeMapping isAbstract not ].!

beAbstract
	isAbstract := true!

isAbstract
	^isAbstract isNil ifTrue: [ isAbstract := false ] ifFalse: [ isAbstract ]!

mapFromObject: anObject intoRowsIn: aRowMap

	"do nothing"!

mappedClass: aClass
	mappedClass := aClass!

mappedFields
	^#()!

trace: aTracing context: anExpression

	^self! !
!HorizontalTypeMapping categoriesFor: #allDescribedConcreteClasses!mapping!private! !
!HorizontalTypeMapping categoriesFor: #beAbstract!accessing!private! !
!HorizontalTypeMapping categoriesFor: #isAbstract!accessing!private! !
!HorizontalTypeMapping categoriesFor: #mapFromObject:intoRowsIn:!mapping!private! !
!HorizontalTypeMapping categoriesFor: #mappedClass:!initializing!private! !
!HorizontalTypeMapping categoriesFor: #mappedFields!mapping!private! !
!HorizontalTypeMapping categoriesFor: #trace:context:!mapping!private! !

!HorizontalTypeMapping class methodsFor!

forClass: aClass
	^self new
		mappedClass: aClass;
		yourself! !
!HorizontalTypeMapping class categoriesFor: #forClass:!instance creation!private! !

IdentityTypeMapping guid: (GUID fromString: '{2757007F-6054-41DE-BE45-406E736F4C1A}')!
IdentityTypeMapping comment: 'This is the default type mapping for classes that aren''t using inheritance. It implements all of the protocol as no-ops.'!
!IdentityTypeMapping categoriesForClass!Unclassified! !
!IdentityTypeMapping methodsFor!

canBeTypeMappingParent
	^false!

isAbstract
	^false!

isTypeMappingRoot
	^true!

mappedFields
	^#()!

trace: aTracing context: anExpression
	"do nothing"! !
!IdentityTypeMapping categoriesFor: #canBeTypeMappingParent!private!testing! !
!IdentityTypeMapping categoriesFor: #isAbstract!private!testing! !
!IdentityTypeMapping categoriesFor: #isTypeMappingRoot!private!testing! !
!IdentityTypeMapping categoriesFor: #mappedFields!mapping!private! !
!IdentityTypeMapping categoriesFor: #trace:context:!mapping!private! !

BasicTypeResolver guid: (GUID fromString: '{8E73D3CA-CEC6-4572-BE66-0C903360B782}')!
BasicTypeResolver comment: 'This is an abstract superclass for TypeResolvers that do inheritance in some form.

Instance Variables:
	concreteMembers	<(Collection of: (Descriptor))>	Which of our members can actually be instantiated, as opposed to abstract superclasses.
	rootClass	<Behavior>	Which is the root class of the inheritance hierarchy we represent.
	rootDescriptor	<Descriptor>	The descriptor for the root class
	subclassDescriptorsBuilt	<Boolean>	Have we asked the system for the descriptors for all of the root''s subclasses yet?

'!
!BasicTypeResolver categoriesForClass!Unclassified! !
!BasicTypeResolver methodsFor!

allDescribedConcreteClasses
	self subclassDescriptorsBuilt ifFalse: [self forceSubclassDescriptorLoads].
	^self concreteMembers collect: [ :each | each describedClass ]!

concreteMembers
	^concreteMembers isNil 
		ifTrue: [ concreteMembers := OrderedCollection new ]
		ifFalse: [ concreteMembers]!

forceSubclassDescriptorLoads
	self rootClass allSubclassesDo: [ :each | self system descriptorFor: each ].
	subclassDescriptorsBuilt := true!

register: aDescriptor abstract: abstract

	super register: aDescriptor abstract: abstract.
	abstract ifFalse: [ self concreteMembers add: aDescriptor ]!

rootClass
	^rootClass!

rootClass: anObject
	rootClass := anObject!

rootDescriptor
	^rootDescriptor isNil
		ifTrue: [ rootDescriptor := self system descriptorFor: self rootClass ]
		ifFalse: [ rootDescriptor ]!

rootDescriptor: anObject
	rootDescriptor := anObject!

subclassDescriptorsBuilt
	^subclassDescriptorsBuilt isNil
		ifTrue: [ subclassDescriptorsBuilt := false ]
		ifFalse: [ subclassDescriptorsBuilt ]!

typeMappingRootDescriptor
	^self rootDescriptor!

usesInheritance
	^true.!

validate
	| tables sequences |
	super validate.
	tables := self concreteMembers collect: [:each | each primaryTable].
	sequences := tables inject: #() into: [:sum :each | sum, ((each primaryKeyFields) collect: [:eachField | eachField type sequence])].
	sequences asSet size > 1 ifTrue: [self error: 'When using inheritance, all tables must use the same sequence. Otherwise, the primary key references will be ambiguous.'].! !
!BasicTypeResolver categoriesFor: #allDescribedConcreteClasses!private! !
!BasicTypeResolver categoriesFor: #concreteMembers!accessing!private! !
!BasicTypeResolver categoriesFor: #forceSubclassDescriptorLoads!private! !
!BasicTypeResolver categoriesFor: #register:abstract:!private!registering! !
!BasicTypeResolver categoriesFor: #rootClass!accessing!private! !
!BasicTypeResolver categoriesFor: #rootClass:!accessing!private! !
!BasicTypeResolver categoriesFor: #rootDescriptor!accessing!private! !
!BasicTypeResolver categoriesFor: #rootDescriptor:!other!private! !
!BasicTypeResolver categoriesFor: #subclassDescriptorsBuilt!accessing!private! !
!BasicTypeResolver categoriesFor: #typeMappingRootDescriptor!private!type resolution! !
!BasicTypeResolver categoriesFor: #usesInheritance!private!testing! !
!BasicTypeResolver categoriesFor: #validate!private!validation! !

IdentityTypeResolver guid: (GUID fromString: '{37451137-325C-4F0D-B21B-64900DCF686C}')!
IdentityTypeResolver comment: 'This is a trivial type resolver (and the default one) that just uses a single class, with no inheritance.'!
!IdentityTypeResolver categoriesForClass!Unclassified! !
!IdentityTypeResolver methodsFor!

allTablesFor: aDescriptor
	"Return all tables used by this descriptor or its subclasses. For a non-inheritance descriptor, this is the same as tables."
	^aDescriptor tables.!

classesRequiringIndependentQueriesFor: aClass
	^Array with: aClass!

describedConcreteClassFor: aRow withBuilder: builder descriptor: aDescriptor
	^aDescriptor describedClass!

descriptorsRequiringIndependentQueriesFor: aClass
	^members.!

typeMappingRootDescriptor
	^members first! !
!IdentityTypeResolver categoriesFor: #allTablesFor:!accessing!private! !
!IdentityTypeResolver categoriesFor: #classesRequiringIndependentQueriesFor:!private!type resolution! !
!IdentityTypeResolver categoriesFor: #describedConcreteClassFor:withBuilder:descriptor:!accessing!private! !
!IdentityTypeResolver categoriesFor: #descriptorsRequiringIndependentQueriesFor:!As yet unclassified!private! !
!IdentityTypeResolver categoriesFor: #typeMappingRootDescriptor!accessing!private! !

FilteredTypeResolver guid: (GUID fromString: '{C4D2680F-CE44-412A-B542-81221E01CB60}')!
FilteredTypeResolver comment: 'This is a type resolver for inheritance where all of the classes are represented in a single table, with some sort of identifying field for which subclass they are. The table would have the union of all possible fields for all classes. It''s called filtering, because for any particular class, we filter out those instances from all those represented in the table.'!
!FilteredTypeResolver categoriesForClass!Unclassified! !
!FilteredTypeResolver methodsFor!

allTablesFor: aDescriptor
	"Return all tables used by this descriptor or its subclasses. Since all our subclasses use a single table, this is the same as for a normal descriptor."
	^aDescriptor tables.!

askConcreteMembersForConcreteClassFor: row withBuilder: builder
	concreteMembers do: [:each |
		each describedConcreteClassFor: row withBuilder: builder].!

classesRequiringIndependentQueriesFor: aClass
	^Array with: aClass!

describedConcreteClassFor: row withBuilder: builder descriptor: aDescriptor
	^aDescriptor typeMapping describedConcreteClassFor: row withBuilder: builder!

fieldsForSelectStatement
	"Return fields that are needed in a select statement - i.e. return all inherited fields that are part of the tables we are already selecting for this object, but not in the main descriptor"
	| fields rootFields |
	fields := OrderedCollection new.
	rootFields := self rootDescriptor mappedFields asSet.
	self concreteMembers do: [:each |
		each == self rootDescriptor ifFalse: [
			each mappedFields do: [:eachSubField |
				(rootFields includes: eachSubField) ifFalse: [
					fields add: eachSubField]]]].
	^fields.!

isAbstract: aDescriptor
	^aDescriptor == self rootDescriptor!

register: aDescriptor abstract: aBoolean
	super register: aDescriptor abstract: aBoolean.
	aBoolean
		ifTrue:
			[| fakeTypeMapping |
			fakeTypeMapping := FilteredTypeMapping to: nil keyedBy: nil.
			fakeTypeMapping attribute name: #filteredTypeResolverFilterMapping.
			fakeTypeMapping readOnly: true.
			aDescriptor typeMapping: fakeTypeMapping].!

register: aDescriptor keyedBy: aKey field: aField
	self register: aDescriptor abstract: false.
	aDescriptor typeMapping: (FilteredTypeMapping to: aField keyedBy: aKey).! !
!FilteredTypeResolver categoriesFor: #allTablesFor:!accessing!private! !
!FilteredTypeResolver categoriesFor: #askConcreteMembersForConcreteClassFor:withBuilder:!private!type resolving! !
!FilteredTypeResolver categoriesFor: #classesRequiringIndependentQueriesFor:!accessing!private! !
!FilteredTypeResolver categoriesFor: #describedConcreteClassFor:withBuilder:descriptor:!private!type resolving! !
!FilteredTypeResolver categoriesFor: #fieldsForSelectStatement!private!type resolving! !
!FilteredTypeResolver categoriesFor: #isAbstract:!private!type resolving! !
!FilteredTypeResolver categoriesFor: #register:abstract:!private!registering! !
!FilteredTypeResolver categoriesFor: #register:keyedBy:field:!private!registering! !

!FilteredTypeResolver class methodsFor!

forRootClass: aClass
	^self new
		rootClass: aClass;
		yourself.! !
!FilteredTypeResolver class categoriesFor: #forRootClass:!instance creation!private! !

HorizontalTypeResolver guid: (GUID fromString: '{CAC39DAA-03AB-46DB-A18A-167CF64BB802}')!
HorizontalTypeResolver comment: 'This implements inheritance where each concrete class is represented in its own table. Each table will duplicate the fields that are in common between the concrete classes.'!
!HorizontalTypeResolver categoriesForClass!Unclassified! !
!HorizontalTypeResolver methodsFor!

allTablesFor: aDescriptor
	"Return all tables used by this descriptor or its subclasses."
	^(self descriptorsRequiringIndependentQueriesFor: aDescriptor describedClass) inject: OrderedCollection new into: [:sum :eachDescriptor |
		sum addAll: ((eachDescriptor == aDescriptor) ifTrue: [eachDescriptor tables] ifFalse: [eachDescriptor allTables]).
		sum].!

classesRequiringIndependentQueriesFor: aClass
	^(self descriptorsRequiringIndependentQueriesFor: aClass) collect: [:each |
		each describedClass].!

describedConcreteClassFor: row withBuilder: builder descriptor: aDescriptor
	^aDescriptor describedClass!

descriptorsRequiringIndependentQueriesFor: aClass
	^self concreteMembers select: [ :each | each describedClass includesBehavior: aClass ].!

isTypeMappingRoot: aDescriptor
	^aDescriptor == rootDescriptor!

mappingForSubclass: aDescriptor connectTo: existingSubclassMapping
	"Create a pseudo-mapping for the subclass."
	| attributeName mapping newJoin |
	attributeName := self pseudoAttributeNameForClass: aDescriptor describedClass.
	mapping := self rootDescriptor mappingForAttributeNamed: attributeName.
	mapping isNil ifTrue: [
		rootDescriptor classModel newAttributeNamed: attributeName type: aDescriptor describedClass.
		mapping := (rootDescriptor newMapping: OneToOneMapping)
			attributeName: attributeName.
		mapping beForPseudoVariable.
		existingSubclassMapping isNil ifFalse: [
			newJoin := Join new.
			mapping join: newJoin]].
	^mapping.!

pseudoAttributeNameForClass: aClass
	^('subclassPseudoVariable', aClass name) asSymbol.!

setUpExpressionsForQuery: aQuery
	| mapping firstMapping allDescriptors |
	(self concreteMembers includes: aQuery resultClass) ifTrue: [^self].
	self shouldUseSingleQuery ifFalse: [^self].
	firstMapping := nil.
	allDescriptors := self
		descriptorsRequiringIndependentQueriesFor: aQuery resultClass.
	allDescriptors size = 1 ifTrue: [^self].
	allDescriptors
		do:
			[:eachSubclassDescriptor | 
			mapping := self
				mappingForSubclass: eachSubclassDescriptor
				connectTo: firstMapping.
			firstMapping isNil ifTrue: [firstMapping := mapping].
			aQuery
				alsoFetch:
					[:object | (object getMapping: mapping named: mapping attribute name) bePhantom asOuterJoin]].!

shouldUseSingleQuery
	"Return true if we should attempt to combine queries for multiple subclasses in disjoint tables into one."

	shouldUseSingleQuery isNil ifTrue: [shouldUseSingleQuery := false].
	^shouldUseSingleQuery.!

supportsOrderingFor: aDescriptor
	concreteMembers detect: [:each | each == aDescriptor] ifNone: [^false].
	^true.!

useSingleQuery: aBoolean
	shouldUseSingleQuery := aBoolean! !
!HorizontalTypeResolver categoriesFor: #allTablesFor:!accessing!private! !
!HorizontalTypeResolver categoriesFor: #classesRequiringIndependentQueriesFor:!accessing!private! !
!HorizontalTypeResolver categoriesFor: #describedConcreteClassFor:withBuilder:descriptor:!accessing!private! !
!HorizontalTypeResolver categoriesFor: #descriptorsRequiringIndependentQueriesFor:!accessing!private! !
!HorizontalTypeResolver categoriesFor: #isTypeMappingRoot:!private!testing! !
!HorizontalTypeResolver categoriesFor: #mappingForSubclass:connectTo:!private!query setup! !
!HorizontalTypeResolver categoriesFor: #pseudoAttributeNameForClass:!private!query setup! !
!HorizontalTypeResolver categoriesFor: #setUpExpressionsForQuery:!private!query setup! !
!HorizontalTypeResolver categoriesFor: #shouldUseSingleQuery!accessing!private! !
!HorizontalTypeResolver categoriesFor: #supportsOrderingFor:!private!testing! !
!HorizontalTypeResolver categoriesFor: #useSingleQuery:!accessing!private! !

!HorizontalTypeResolver class methodsFor!

forRootClass: aClass
	^self new
		rootClass: aClass;
		yourself.! !
!HorizontalTypeResolver class categoriesFor: #forRootClass:!instance creation!private! !

"Binary Globals"!

